'use strict';function ea(a){var b=0;return function(){return b<a.length?{done:!1,value:a[b++]}:{done:!0}}}var fa="function"==typeof Object.defineProperties?Object.defineProperty:function(a,b,c){if(a==Array.prototype||a==Object.prototype)return a;a[b]=c.value;return a};
function ia(a){a=["object"==typeof globalThis&&globalThis,a,"object"==typeof window&&window,"object"==typeof self&&self,"object"==typeof global&&global];for(var b=0;b<a.length;++b){var c=a[b];if(c&&c.Math==Math)return c}throw Error("Cannot find global object");}var ka=ia(this);function qa(a,b){if(b)a:{var c=ka;a=a.split(".");for(var d=0;d<a.length-1;d++){var e=a[d];if(!(e in c))break a;c=c[e]}a=a[a.length-1];d=c[a];b=b(d);b!=d&&null!=b&&fa(c,a,{configurable:!0,writable:!0,value:b})}}
qa("Symbol.asyncIterator",function(a){return a?a:Symbol("Symbol.asyncIterator")});function ra(a){function b(d){return a.next(d)}function c(d){return a.throw(d)}return new Promise(function(d,e){function f(g){g.done?d(g.value):Promise.resolve(g.value).then(b,c).then(f,e)}f(a.next())})}function ta(a){return ra(a())}
function ua(a){this[Symbol.asyncIterator]=function(){return this};this[Symbol.iterator]=function(){return a};this.next=function(b){return Promise.resolve(a.next(b))};void 0!==a["throw"]&&(this["throw"]=function(b){return Promise.resolve(a["throw"](b))});void 0!==a["return"]&&(this["return"]=function(b){return Promise.resolve(a["return"](b))})}qa("Object.values",function(a){return a?a:function(b){var c=[],d;for(d in b)Object.prototype.hasOwnProperty.call(b,d)&&c.push(b[d]);return c}});
qa("Array.prototype.includes",function(a){return a?a:function(b,c){var d=this;d instanceof String&&(d=String(d));var e=d.length;c=c||0;for(0>c&&(c=Math.max(c+e,0));c<e;c++){var f=d[c];if(f===b||Object.is(f,b))return!0}return!1}});function va(a,b){a instanceof String&&(a+="");var c=0,d=!1,e={next:function(){if(!d&&c<a.length){var f=c++;return{value:b(f,a[f]),done:!1}}d=!0;return{done:!0,value:void 0}}};e[Symbol.iterator]=function(){return e};return e}
qa("Array.prototype.values",function(a){return a?a:function(){return va(this,function(b,c){return c})}});qa("globalThis",function(a){return a||ka});qa("Array.prototype.flat",function(a){return a?a:function(b){b=void 0===b?1:b;var c=[];Array.prototype.forEach.call(this,function(d){Array.isArray(d)&&0<b?(d=Array.prototype.flat.call(d,b-1),c.push.apply(c,d)):c.push(d)});return c}});
qa("Object.entries",function(a){return a?a:function(b){var c=[],d;for(d in b)Object.prototype.hasOwnProperty.call(b,d)&&c.push([d,b[d]]);return c}});function wa(a){a=Math.trunc(a)||0;0>a&&(a+=this.length);if(!(0>a||a>=this.length))return this[a]}qa("Array.prototype.at",function(a){return a?a:wa});function xa(a){return a?a:wa}qa("Int8Array.prototype.at",xa);qa("Uint8Array.prototype.at",xa);qa("Uint8ClampedArray.prototype.at",xa);qa("Int16Array.prototype.at",xa);qa("Uint16Array.prototype.at",xa);
qa("Int32Array.prototype.at",xa);qa("Uint32Array.prototype.at",xa);qa("Float32Array.prototype.at",xa);qa("Float64Array.prototype.at",xa);qa("String.prototype.at",function(a){return a?a:wa});qa("String.prototype.padStart",function(a){return a?a:function(b,c){if(null==this)throw new TypeError("The 'this' value for String.prototype.padStart must not be null or undefined");b-=this.length;c=void 0!==c?String(c):" ";return(0<b&&c?c.repeat(Math.ceil(b/c.length)).substring(0,b):"")+this}});
qa("Object.fromEntries",function(a){return a?a:function(b){var c={};if(!(Symbol.iterator in b))throw new TypeError(""+b+" is not iterable");b=b[Symbol.iterator].call(b);for(var d=b.next();!d.done;d=b.next()){d=d.value;if(Object(d)!==d)throw new TypeError("iterable for fromEntries should yield objects");c[d[0]]=d[1]}return c}});/*

 Copyright The Closure Library Authors.
 SPDX-License-Identifier: Apache-2.0
*/
var ya=this||self;function Ba(a){var b=typeof a;b="object"!=b?b:a?Array.isArray(a)?"array":b:"null";return"array"==b||"object"==b&&"number"==typeof a.length}function Ca(a){var b=typeof a;return"object"==b&&null!=a||"function"==b}function Da(a){return a};var Ja,Ka;a:{for(var Ma=["CLOSURE_FLAGS"],Na=ya,Qa=0;Qa<Ma.length;Qa++)if(Na=Na[Ma[Qa]],null==Na){Ka=null;break a}Ka=Na}var Ta=Ka&&Ka[610401301];Ja=null!=Ta?Ta:!1;var Ua;const Xa=ya.navigator;Ua=Xa?Xa.userAgentData||null:null;function Ya(a){return Ja?Ua?Ua.brands.some(({brand:b})=>b&&-1!=b.indexOf(a)):!1:!1}function $a(a){var b;a:{if(b=ya.navigator)if(b=b.userAgent)break a;b=""}return-1!=b.indexOf(a)};function ab(){return Ja?!!Ua&&0<Ua.brands.length:!1}function eb(){return ab()?Ya("Chromium"):($a("Chrome")||$a("CriOS"))&&!(ab()?0:$a("Edge"))||$a("Silk")};const fb=Array.prototype.map?function(a,b,c){return Array.prototype.map.call(a,b,c)}:function(a,b,c){const d=a.length,e=Array(d),f="string"===typeof a?a.split(""):a;for(let g=0;g<d;g++)g in f&&(e[g]=b.call(c,f[g],g,a));return e};!$a("Android")||eb();var gb=eb();$a("Safari")&&(eb()||(ab()?0:$a("Coast"))||(ab()?0:$a("Opera"))||(ab()?0:$a("Edge"))||(ab()?Ya("Microsoft Edge"):$a("Edg/"))||ab()&&Ya("Opera"));class hb{}class ib{}Object.freeze(new hb);Object.freeze(new ib);Object.freeze(new ib);/*

 Copyright Google LLC All Rights Reserved.

 Use of this source code is governed by an MIT-style license that can be
 found in the LICENSE file at https://angular.io/license
 @param {!tsickle_zone_impl_1.ZonePrivate} api
 @param {?} target
 @param {string} targetName
 @param {string} method
 @param {!Array<string>} callbacks
 @return {void}
*/
function lb(a,b,c,d,e){const f=Zone.__symbol__(d);if(!b[f]){var g=b[f]=b[d];b[d]=function(h,m,p){m&&m.prototype&&e.forEach(function(u){const w=`${c}.${d}::`+u,B=m.prototype;try{if(B.hasOwnProperty(u)){const G=a.ObjectGetOwnPropertyDescriptor(B,u);G&&G.value?(G.value=a.wrapWithCurrentZone(G.value,w),a._redefineProperty(m.prototype,u,G)):B[u]&&(B[u]=a.wrapWithCurrentZone(B[u],w))}else B[u]&&(B[u]=a.wrapWithCurrentZone(B[u],w))}catch(G){}});return g.call(b,h,m,p)};a.attachOriginToPatched(b[d],g)}};/*

 Copyright Google LLC All Rights Reserved.

 Use of this source code is governed by an MIT-style license that can be
 found in the LICENSE file at https://angular.io/license
*/
const mb=globalThis;function nb(a){return(mb.__Zone_symbol_prefix||"__zone_symbol__")+a}
function ob(){function a(F){g&&g.mark&&g.mark(F)}function b(F,I){g&&g.measure&&g.measure(F,I)}function c(F){ba||mb[B]&&(ba=mb[B].resolve(0));if(ba){let I=ba[G];I||(I=ba.then);I.call(ba,F)}else mb[w](F,0)}function d(F){0===ja&&0===L.length&&c(e);F&&L.push(F)}function e(){if(!O){for(O=!0;L.length;){const F=L;L=[];for(let I=0;I<F.length;I++){const aa=F[I];try{aa.zone.runTask(aa,null,null)}catch(Aa){K.onUnhandledError(Aa)}}}K.microtaskDrainDone();O=!1}}function f(){}const g=mb.performance;a("Zone");class h{static assertZonePatched(){if(mb.Promise!==
na.ZoneAwarePromise)throw Error("Zone.js has detected that ZoneAwarePromise `(window|global).Promise` has been overwritten.\nMost likely cause is that a Promise polyfill has been loaded after Zone.js (Polyfilling Promise api is not necessary when zone.js is loaded. If you must load one, do so before loading zone.js.)");}static get root(){let F=h.current;for(;F.parent;)F=F.parent;return F}static get current(){return W.zone}static get currentTask(){return S}static __load_patch(F,I,aa=!1){if(na.hasOwnProperty(F)){if(I=
!0===mb[nb("forceDuplicateZoneCheck")],!aa&&I)throw Error("Already loaded patch: "+F);}else mb["__Zone_disable_"+F]||(aa="Zone:"+F,a(aa),na[F]=I(mb,h,K),b(aa,aa))}get parent(){return this.Sa}get name(){return this.th}constructor(F,I){this.Sa=F;this.th=I?I.name||"unnamed":"<root>";this.D=I&&I.properties||{};this.C=new p(this,this.Sa&&this.Sa.C,I)}get(F){const I=this.getZoneWith(F);if(I)return I.D[F]}getZoneWith(F){let I=this;for(;I;){if(I.D.hasOwnProperty(F))return I;I=I.Sa}return null}fork(F){if(!F)throw Error("ZoneSpec required!");
return this.C.fork(this,F)}wrap(F,I){if("function"!==typeof F)throw Error("Expecting function got: "+F);const aa=this.C.intercept(this,F,I),Aa=this;return function(){return Aa.runGuarded(aa,this,arguments,I)}}run(F,I,aa,Aa){W={parent:W,zone:this};try{return this.C.invoke(this,F,I,aa,Aa)}finally{W=W.parent}}runGuarded(F,I=null,aa,Aa){W={parent:W,zone:this};try{try{return this.C.invoke(this,F,I,aa,Aa)}catch(Za){if(this.C.handleError(this,Za))throw Za;}}finally{W=W.parent}}runTask(F,I,aa){if(F.zone!=
this)throw Error("A task can only be run in the zone of creation! (Creation: "+(F.zone||ca).name+"; Execution: "+this.name+")");if("notScheduled"!==F.state||"eventTask"!==F.type&&"macroTask"!==F.type){var Aa="running"!=F.state;Aa&&F.wd("running","scheduled");F.runCount++;var Za=S;S=F;W={parent:W,zone:this};try{"macroTask"==F.type&&F.data&&!F.data.isPeriodic&&(F.cancelFn=void 0);try{return this.C.invokeTask(this,F,I,aa)}catch(vb){if(this.C.handleError(this,vb))throw vb;}}finally{"notScheduled"!==F.state&&
"unknown"!==F.state&&("eventTask"==F.type||F.data&&F.data.isPeriodic?Aa&&F.wd("scheduled","running"):(F.runCount=0,this.Jk(F,-1),Aa&&F.wd("notScheduled","running","notScheduled"))),W=W.parent,S=Za}}}scheduleTask(F){if(F.zone&&F.zone!==this)for(var I=this;I;){if(I===F.zone)throw Error(`can not reschedule task to ${this.name} which is descendants of the original zone ${F.zone.name}`);I=I.parent}F.wd("scheduling","notScheduled");I=[];F.Vi=I;F.zb=this;try{F=this.C.scheduleTask(this,F)}catch(aa){throw F.wd("unknown",
"scheduling","notScheduled"),this.C.handleError(this,aa),aa;}F.Vi===I&&this.Jk(F,1);"scheduling"==F.state&&F.wd("scheduled","scheduling");return F}scheduleMicroTask(F,I,aa,Aa){return this.scheduleTask(new u("microTask",F,I,aa,Aa,void 0))}scheduleMacroTask(F,I,aa,Aa,Za){return this.scheduleTask(new u("macroTask",F,I,aa,Aa,Za))}scheduleEventTask(F,I,aa,Aa,Za){return this.scheduleTask(new u("eventTask",F,I,aa,Aa,Za))}cancelTask(F){if(F.zone!=this)throw Error("A task can only be cancelled in the zone of creation! (Creation: "+
(F.zone||ca).name+"; Execution: "+this.name+")");if("scheduled"===F.state||"running"===F.state){F.wd("canceling","scheduled","running");try{this.C.cancelTask(this,F)}catch(I){throw F.wd("unknown","canceling"),this.C.handleError(this,I),I;}this.Jk(F,-1);F.wd("notScheduled","canceling");F.runCount=0;return F}}Jk(F,I){const aa=F.Vi;-1==I&&(F.Vi=null);for(let Aa=0;Aa<aa.length;Aa++)aa[Aa].Jk(F.type,I)}}h.__symbol__=nb;const m={name:"",onHasTask:(F,I,aa,Aa)=>F.hasTask(aa,Aa),onScheduleTask:(F,I,aa,Aa)=>
F.scheduleTask(aa,Aa),onInvokeTask:(F,I,aa,Aa,Za,vb)=>F.invokeTask(aa,Aa,Za,vb),onCancelTask:(F,I,aa,Aa)=>F.cancelTask(aa,Aa)};class p{get zone(){return this.zb}constructor(F,I,aa){this.aB={microTask:0,macroTask:0,eventTask:0};this.zb=F;this.IF=I;this.yp=aa&&(aa&&aa.onFork?aa:I.yp);this.Pt=aa&&(aa.onFork?I:I.Pt);this.eA=aa&&(aa.onFork?this.zb:I.eA);this.Fp=aa&&(aa.onIntercept?aa:I.Fp);this.iu=aa&&(aa.onIntercept?I:I.iu);this.hu=aa&&(aa.onIntercept?this.zb:I.hu);this.Ip=aa&&(aa.onInvoke?aa:I.Ip);this.lu=
aa&&(aa.onInvoke?I:I.lu);this.ju=aa&&(aa.onInvoke?this.zb:I.ju);this.Ap=aa&&(aa.onHandleError?aa:I.Ap);this.Wt=aa&&(aa.onHandleError?I:I.Wt);this.Vt=aa&&(aa.onHandleError?this.zb:I.Vt);this.Qm=aa&&(aa.onScheduleTask?aa:I.Qm);this.Rp=aa&&(aa.onScheduleTask?I:I.Rp);this.Qp=aa&&(aa.onScheduleTask?this.zb:I.Qp);this.Gm=aa&&(aa.onInvokeTask?aa:I.Gm);this.Hp=aa&&(aa.onInvokeTask?I:I.Hp);this.Gp=aa&&(aa.onInvokeTask?this.zb:I.Gp);this.ym=aa&&(aa.onCancelTask?aa:I.ym);this.rp=aa&&(aa.onCancelTask?I:I.rp);
this.qp=aa&&(aa.onCancelTask?this.zb:I.qp);this.Yt=this.au=this.Zt=this.vk=null;F=aa&&aa.onHasTask;const Aa=I&&I.vk;if(F||Aa)this.vk=F?aa:m,this.Zt=I,this.au=this,this.Yt=this.zb,aa.onScheduleTask||(this.Qm=m,this.Rp=I,this.Qp=this.zb),aa.onInvokeTask||(this.Gm=m,this.Hp=I,this.Gp=this.zb),aa.onCancelTask||(this.ym=m,this.rp=I,this.qp=this.zb)}fork(F,I){return this.yp?this.yp.onFork(this.Pt,this.zone,F,I):new h(F,I)}intercept(F,I,aa){return this.Fp?this.Fp.onIntercept(this.iu,this.hu,F,I,aa):I}invoke(F,
I,aa,Aa,Za){return this.Ip?this.Ip.onInvoke(this.lu,this.ju,F,I,aa,Aa,Za):I.apply(aa,Aa)}handleError(F,I){return this.Ap?this.Ap.onHandleError(this.Wt,this.Vt,F,I):!0}scheduleTask(F,I){let aa=I;if(this.Qm)this.vk&&aa.Vi.push(this.au),(aa=this.Qm.onScheduleTask(this.Rp,this.Qp,F,I))||(aa=I);else if(I.scheduleFn)I.scheduleFn(I);else if("microTask"==I.type)d(I);else throw Error("Task is missing scheduleFn.");return aa}invokeTask(F,I,aa,Aa){return this.Gm?this.Gm.onInvokeTask(this.Hp,this.Gp,F,I,aa,Aa):
I.callback.apply(aa,Aa)}cancelTask(F,I){if(this.ym)F=this.ym.onCancelTask(this.rp,this.qp,F,I);else{if(!I.cancelFn)throw Error("Task is not cancelable");F=I.cancelFn(I)}return F}hasTask(F,I){try{this.vk&&this.vk.onHasTask(this.Zt,this.Yt,F,I)}catch(aa){this.handleError(F,aa)}}Jk(F,I){const aa=this.aB,Aa=aa[F];I=aa[F]=Aa+I;if(0>I)throw Error("More tasks executed then were scheduled.");0!=Aa&&0!=I||this.hasTask(this.zb,{cD:0<aa.microTask,YC:0<aa.macroTask,ZF:0<aa.eventTask,xb:F})}}class u{constructor(F,
I,aa,Aa,Za,vb){this.zb=null;this.runCount=0;this.Vi=null;this.qc="notScheduled";this.type=F;this.source=I;this.data=Aa;this.scheduleFn=Za;this.cancelFn=vb;if(!aa)throw Error("callback is not defined");this.callback=aa;const kb=this;this.invoke="eventTask"===F&&Aa&&Aa.Us?u.invokeTask:function(){return u.invokeTask.call(mb,kb,this,arguments)}}static invokeTask(F,I,aa){F||(F=this);ja++;try{return F.runCount++,F.zone.runTask(F,I,aa)}finally{1==ja&&e(),ja--}}get zone(){return this.zb}get state(){return this.qc}cancelScheduleRequest(){this.wd("notScheduled",
"scheduling")}wd(F,I,aa){if(this.qc===I||this.qc===aa)this.qc=F,"notScheduled"==F&&(this.Vi=null);else throw Error(`${this.type} '${this.source}': can not transition to '${F}', expecting state '${I}'${aa?" or '"+aa+"'":""}, was '${this.qc}'.`);}toString(){return this.data&&"undefined"!==typeof this.data.handleId?this.data.handleId.toString():Object.prototype.toString.call(this)}toJSON(){return{type:this.type,state:this.state,source:this.source,zone:this.zone.name,runCount:this.runCount}}}const w=
nb("setTimeout"),B=nb("Promise"),G=nb("then");let L=[],O=!1,ba;const ca={name:"NO ZONE"},na={},K={symbol:nb,currentZoneFrame:()=>W,onUnhandledError:f,microtaskDrainDone:f,scheduleMicroTask:d,showUncaughtError:()=>!h[nb("ignoreConsoleErrorUncaughtError")],patchEventTarget:()=>[],patchOnProperties:f,patchMethod:()=>f,bindArguments:()=>[],patchThen:()=>f,patchMacroTask:()=>f,patchEventPrototype:()=>f,isIEOrEdge:()=>!1,getGlobalObjects:()=>{},ObjectDefineProperty:()=>f,ObjectGetOwnPropertyDescriptor:()=>
{},ObjectCreate:()=>{},ArraySlice:()=>[],patchClass:()=>f,wrapWithCurrentZone:()=>f,filterProperties:()=>[],attachOriginToPatched:()=>f,_redefineProperty:()=>f,patchCallbacks:()=>f,nativeScheduleMicroTask:c};let W={parent:null,zone:new h(null,null)},S=null,ja=0;b("Zone","Zone");return h};var pb=Object.getOwnPropertyDescriptor,rb=Object.defineProperty,sb=Object.getPrototypeOf,zb=Object.create,Ab=Array.prototype.slice,Bb=nb("addEventListener"),Cb=nb("removeEventListener"),Db=nb("");function Eb(a,b){return Zone.current.wrap(a,b)}const Fb="undefined"!==typeof window,Hb=Fb?window:void 0,Jb=Fb&&Hb||globalThis;function Kb(a,b){for(let c=a.length-1;0<=c;c--)"function"===typeof a[c]&&(a[c]=Eb(a[c],b+"_"+c));return a}
function Lb(a){var b=["getCurrentPosition","watchPosition"];const c=a.constructor.name;for(let d=0;d<b.length;d++){const e=b[d],f=a[e];if(f){const g=pb(a,e);Mb(g)&&(a[e]=(h=>{function m(){return h.apply(this,Kb(arguments,c+"."+e))}Nb(m,h);return m})(f))}}}function Mb(a){return a?!1===a.writable?!1:!("function"===typeof a.get&&"undefined"===typeof a.set):!0}
var Ob="undefined"!==typeof WorkerGlobalScope&&self instanceof WorkerGlobalScope,Pb=!("nw"in Jb)&&"undefined"!==typeof Jb.process&&"[object process]"===Jb.process.toString(),Qb=!Pb&&!Ob&&!(!Fb||!Hb.HTMLElement),Rb="undefined"!==typeof Jb.process&&"[object process]"===Jb.process.toString()&&!Ob&&!(!Fb||!Hb.HTMLElement);const Sb={};
function Vb(a){if(a=a||Jb.event){var b=Sb[a.type];b||(b=Sb[a.type]=nb("ON_PROPERTY"+a.type));var c=this||a.target||Jb;b=c[b];Qb&&c===Hb&&"error"===a.type?(c=a,c=b&&b.call(this,c.message,c.filename,c.lineno,c.colno,c.error),!0===c&&a.preventDefault()):(c=b&&b.apply(this,arguments),void 0==c||c||a.preventDefault());return c}}
function Wb(a,b,c){let d=pb(a,b);!d&&c&&pb(c,b)&&(d={enumerable:!0,configurable:!0});if(d&&d.configurable&&(c=nb("on"+b+"patched"),!a.hasOwnProperty(c)||!a[c])){delete d.writable;delete d.value;var e=d.get,f=d.set,g=b.slice(2),h=Sb[g];h||(h=Sb[g]=nb("ON_PROPERTY"+g));d.set=function(m){let p=this;p||a!==Jb||(p=Jb);p&&("function"===typeof p[h]&&p.removeEventListener(g,Vb),f&&f.call(p,null),p[h]=m,"function"===typeof m&&p.addEventListener(g,Vb,!1))};d.get=function(){let m=this;m||a!==Jb||(m=Jb);if(!m)return null;
var p=m[h];return p?p:e&&(p=e.call(this))?(d.set.call(this,p),"function"===typeof m.removeAttribute&&m.removeAttribute(b),p):null};rb(a,b,d);a[c]=!0}}function Xb(a,b,c){if(b)for(var d=0;d<b.length;d++)Wb(a,"on"+b[d],c);else{b=[];for(d in a)"on"==d.slice(0,2)&&b.push(d);for(d=0;d<b.length;d++)Wb(a,b[d],c)}}const $b=nb("originalInstance");
function ac(a){const b=Jb[a];if(b){Jb[nb(a)]=b;Jb[a]=function(){const e=Kb(arguments,a);switch(e.length){case 0:this[$b]=new b;break;case 1:this[$b]=new b(e[0]);break;case 2:this[$b]=new b(e[0],e[1]);break;case 3:this[$b]=new b(e[0],e[1],e[2]);break;case 4:this[$b]=new b(e[0],e[1],e[2],e[3]);break;default:throw Error("Arg list too long.");}};Nb(Jb[a],b);var c=new b(function(){}),d;for(d in c)"XMLHttpRequest"===a&&"responseBlob"===d||function(e){"function"===typeof c[e]?Jb[a].prototype[e]=function(){return this[$b][e].apply(this[$b],
arguments)}:rb(Jb[a].prototype,e,{set:function(f){"function"===typeof f?(this[$b][e]=Eb(f,a+"."+e),Nb(this[$b][e],f)):this[$b][e]=f},get:function(){return this[$b][e]}})}(d);for(d in b)"prototype"!==d&&b.hasOwnProperty(d)&&(Jb[a][d]=b[d])}}
function cc(a,b,c){let d=a;for(;d&&!d.hasOwnProperty(b);)d=sb(d);!d&&a[b]&&(d=a);a=nb(b);let e=null;if(d&&(!(e=d[a])||!d.hasOwnProperty(a))){e=d[a]=d[b];const f=d&&pb(d,b);if(Mb(f)){const g=c(e,a,b);d[b]=function(){return g(this,arguments)};Nb(d[b],e)}}return e}
function dc(a,b,c){function d(f){const g=f.data;g.bf[g.en]=function(){f.invoke.apply(this,arguments)};e.apply(g.target,g.bf);return f}let e=null;e=cc(a,b,f=>function(g,h){const m=c(g,h);return 0<=m.en&&"function"===typeof h[m.en]?Zone.current.scheduleMacroTask(m.name,h[m.en],m,d,void 0):f.apply(g,h)})}function Nb(a,b){a[nb("OriginalDelegate")]=b}let ec=!1,fc=!1;
function gc(){if(ec)return fc;ec=!0;try{const a=Hb.navigator.userAgent;if(-1!==a.indexOf("MSIE ")||-1!==a.indexOf("Trident/")||-1!==a.indexOf("Edge/"))fc=!0}catch(a){}return fc};function hc(a,b,c){if(!c||0===c.length)return b;c=c.filter(e=>e.target===a);if(!c||0===c.length)return b;const d=c[0].AC;return b.filter(e=>-1===d.indexOf(e))}function ic(a){return Object.getOwnPropertyNames(a).filter(b=>b.startsWith("on")&&2<b.length).map(b=>b.substring(2))};let jc=!1;if("undefined"!==typeof window)try{const a=Object.defineProperty({},"passive",{get:function(){jc=!0}});window.addEventListener("test",a,a);window.removeEventListener("test",a,a)}catch(a){jc=!1}const mc={Us:!0};var nc={},oc={};const qc=new RegExp("^"+Db+"(\\w+)(true|false)$"),rc=nb("propagationStopped");function sc(a,b){var c=(b?b(a):a)+"false";b=(b?b(a):a)+"true";c=Db+c;b=Db+b;nc[a]={};nc[a]["false"]=c;nc[a]["true"]=b}
function tc(a,b,c,d){function e(ba){return h(this,ba,!0)}function f(ba){return h(this,ba,!1)}function g(ba,ca,na){if(!ba.sr){var K=ba.callback;"object"===typeof K&&K.handleEvent&&(ba.callback=ja=>K.handleEvent(ja),ba.lf=K);try{ba.invoke(ba,ca,[na])}catch(ja){var W=ja}var S=ba.options;S&&"object"===typeof S&&S.once&&ca[u].call(ca,na.type,ba.lf?ba.lf:ba.callback,S);return W}}function h(ba,ca,na){if(ca=ca||a.event){ba=ba||ca.target||a;var K=ba[nc[ca.type][na?"true":"false"]];if(K){na=[];if(1===K.length)(ca=
g(K[0],ba,ca))&&na.push(ca);else{K=K.slice();for(let W=0;W<K.length&&(!ca||!0!==ca[rc]);W++){const S=g(K[W],ba,ca);S&&na.push(S)}}if(1===na.length)throw na[0];for(ca=0;ca<na.length;ca++){const W=na[ca];b.nativeScheduleMicroTask(()=>{throw W;})}}}}function m(ba,ca){function na(sa,Ha,Wa,cb,Zb=!1,Mc=!1){return function(){const pc=this||a;let bc=arguments[0];ca&&ca.Zg&&(bc=ca.Zg(bc));let Nc=arguments[1];if(!Nc||Pb&&"uncaughtException"===bc)return sa.apply(this,arguments);let ve=!1;if("function"!==typeof Nc){if(!Nc.handleEvent)return sa.apply(this,
arguments);ve=!0}if(!Za||Za(sa,Nc,pc,arguments)){var Ed=jc&&!!wb&&-1!==wb.indexOf(bc),Dc=aa(I(arguments[2],Ed)),Fd=null==Dc?void 0:Dc.signal;if(null==Fd||!Fd.aborted){if(tb)for(var Mf=0;Mf<tb.length;Mf++)if(bc===tb[Mf])return Ed?sa.call(pc,bc,Nc,Dc):sa.apply(this,arguments);Ed=Dc?"boolean"===typeof Dc?!0:Dc.capture:!1;Mf=Dc&&"object"===typeof Dc?Dc.once:!1;var Nf=Zone.current,Yc=nc[bc];Yc||(sc(bc,bb),Yc=nc[bc]);var Ec=Yc[Ed?"true":"false"];Yc=pc[Ec];var Oc=!1;if(Yc){if(Oc=!0,vb)for(Ec=0;Ec<Yc.length;Ec++)if(Oa(Yc[Ec],
Nc))return}else Yc=pc[Ec]=[];var Kg;Ec=pc.constructor.name;var Uk=oc[Ec];Uk&&(Kg=Uk[bc]);Kg||(Kg=Ec+Ha+(bb?bb(bc):bc));ma.options=Dc;Mf&&(ma.options.once=!1);ma.target=pc;ma.capture=Ed;ma.Kh=bc;ma.LC=Oc;if(Oc=Aa?mc:void 0)Oc.HE=ma;Fd&&(ma.options.signal=void 0);var jd=Nf.scheduleEventTask(Kg,Nc,Oc,Wa,cb);if(Fd){ma.options.signal=Fd;const Of=()=>jd.zone.cancelTask(jd);sa.call(Fd,"abort",Of,{once:!0});Oc&&(Oc.js=()=>Fd.removeEventListener("abort",Of))}ma.target=null;Oc&&(Oc.HE=null);Mf&&(ma.options.once=
!0);if(jc||"boolean"!==typeof jd.options)jd.options=Dc;jd.target=pc;jd.capture=Ed;jd.Kh=bc;ve&&(jd.lf=Nc);Mc?Yc.unshift(jd):Yc.push(jd);if(Zb)return pc}}}}function K(sa,Ha){const Wa=typeof Ha;return"function"===Wa&&sa.callback===Ha||"object"===Wa&&sa.lf===Ha}function W(sa){return da.call(sa.target,sa.Kh,sa.invoke,sa.options)}function S(sa){return Ia.call(ma.target,ma.Kh,sa.invoke,ma.options)}function ja(sa){return T.call(ma.target,ma.Kh,sa.invoke,ma.options)}function F(sa){if(!sa.sr){var Ha=nc[sa.Kh];
let Wa;Ha&&(Wa=Ha[sa.capture?"true":"false"]);if(Ha=Wa&&sa.target[Wa])for(let cb=0;cb<Ha.length;cb++)if(Ha[cb]===sa){Ha.splice(cb,1);sa.sr=!0;0===Ha.length&&(sa.Qu=!0,sa.target[Wa]=null);break}}if(sa.Qu)return da.call(sa.target,sa.Kh,sa.capture?e:f,sa.options)}function I(sa,Ha){return!jc&&"object"===typeof sa&&sa?!!sa.capture:jc&&Ha?"boolean"===typeof sa?{capture:sa,passive:!0}:sa?"object"===typeof sa&&!1!==sa.passive?Object.assign({},sa,{passive:!0}):sa:{passive:!0}:sa}function aa(sa){if("object"===
typeof sa&&null!==sa){const Ha=Object.assign({},sa);sa.signal&&(Ha.signal=sa.signal);return Ha}return sa}if(!ba)return!1;let Aa=!0;ca&&void 0!==ca.Us&&(Aa=ca.Us);const Za=ca&&ca.fF;let vb=!0;ca&&void 0!==ca.uB&&(vb=ca.uB);let kb=!1;ca&&void 0!==ca.XD&&(kb=ca.XD);let jb=ba;for(;jb&&!jb.hasOwnProperty(p);)jb=sb(jb);!jb&&ba[p]&&(jb=ba);if(!jb||jb[G])return!1;const bb=ca&&ca.YF,ma={},T=jb[G]=jb[p],da=jb[nb(u)]=jb[u];ba=jb[nb(w)]=jb[w];const oa=jb[nb(B)]=jb[B];let Ia;ca&&ca.prepend&&(Ia=jb[nb(ca.prepend)]=
jb[ca.prepend]);const db=Aa?F:W,Oa=ca&&ca.sn?ca.sn:K,tb=Zone[nb("UNPATCHED_EVENTS")],wb=a[nb("PASSIVE_EVENTS")];jb[p]=na(T,L,Aa?function(){if(!ma.LC)return T.call(ma.target,ma.Kh,ma.capture?e:f,ma.options)}:ja,db,kb);Ia&&(jb.prependListener=na(Ia,".prependListener:",S,db,kb,!0));jb[u]=function(){const sa=this||a;var Ha=arguments[0];ca&&ca.Zg&&(Ha=ca.Zg(Ha));var Wa=arguments[2];Wa=Wa?"boolean"===typeof Wa?!0:Wa.capture:!1;const cb=arguments[1];if(!cb)return da.apply(this,arguments);if(!Za||Za(da,cb,
sa,arguments)){var Zb=nc[Ha],Mc;Zb&&(Mc=Zb[Wa?"true":"false"]);var pc=Mc&&sa[Mc];if(pc)for(let bc=0;bc<pc.length;bc++)if(Zb=pc[bc],Oa(Zb,cb)){pc.splice(bc,1);Zb.sr=!0;0===pc.length&&(Zb.Qu=!0,sa[Mc]=null,Wa||"string"!==typeof Ha||(sa[Db+"ON_PROPERTY"+Ha]=null));Ha=Zb.data;let Nc;if(null==(Nc=Ha)?0:Nc.js)Ha.js(),Ha.js=null;Zb.zone.cancelTask(Zb);if(kb)return sa;return}return da.apply(this,arguments)}};jb[w]=function(sa){var Ha=this||a,Wa=sa;ca&&ca.Zg&&(Wa=ca.Zg(Wa));sa=[];Ha=uc(Ha,bb?bb(Wa):Wa);for(Wa=
0;Wa<Ha.length;Wa++){const cb=Ha[Wa];sa.push(cb.lf?cb.lf:cb.callback)}return sa};jb[B]=function(sa){var Ha=this||a;if(sa){ca&&ca.Zg&&(sa=ca.Zg(sa));var Wa=nc[sa];if(Wa){var cb=Ha[Wa["false"]];Ha=Ha[Wa["true"]];if(cb)for(cb=cb.slice(),Wa=0;Wa<cb.length;Wa++){const Zb=cb[Wa];this[u].call(this,sa,Zb.lf?Zb.lf:Zb.callback,Zb.options)}if(Ha)for(Ha=Ha.slice(),cb=0;cb<Ha.length;cb++)Wa=Ha[cb],this[u].call(this,sa,Wa.lf?Wa.lf:Wa.callback,Wa.options)}}else{sa=Object.keys(Ha);for(Ha=0;Ha<sa.length;Ha++)(cb=
(cb=qc.exec(sa[Ha]))&&cb[1])&&"removeListener"!==cb&&this[B].call(this,cb);this[B].call(this,"removeListener")}if(kb)return this};Nb(jb[p],T);Nb(jb[u],da);oa&&Nb(jb[B],oa);ba&&Nb(jb[w],ba);return!0}const p=d&&d.add||"addEventListener",u=d&&d.EG||"removeEventListener",w=d&&d.jG||"eventListeners",B=d&&d.FG||"removeAllListeners",G=nb(p),L="."+p+":";let O=[];for(let ba=0;ba<c.length;ba++)O[ba]=m(c[ba],d);return O}
function uc(a,b){if(!b){var c=[];for(let e in a){var d=qc.exec(e);if((d=d&&d[1])&&(!b||d===b)&&(d=a[e]))for(let f=0;f<d.length;f++)c.push(d[f])}return c}c=nc[b];c||(sc(b),c=nc[b]);b=a[c["false"]];a=a[c["true"]];return b?a?b.concat(a):b.slice():a?a.slice():[]}function wc(a,b){(a=a.Event)&&a.prototype&&b.patchMethod(a.prototype,"stopImmediatePropagation",c=>function(d,e){d[rc]=!0;c&&c.apply(d,e)})};function xc(){yc.__load_patch("util",(a,b,c)=>{const d=ic(a);c.patchOnProperties=Xb;c.patchMethod=cc;c.bindArguments=Kb;c.patchMacroTask=dc;const e=b.__symbol__("BLACK_LISTED_EVENTS"),f=b.__symbol__("UNPATCHED_EVENTS");a[f]&&(a[e]=a[f]);a[e]&&(b[e]=b[f]=a[e]);c.patchEventPrototype=wc;c.patchEventTarget=tc;c.isIEOrEdge=gc;c.ObjectDefineProperty=rb;c.ObjectGetOwnPropertyDescriptor=pb;c.ObjectCreate=zb;c.ArraySlice=Ab;c.patchClass=ac;c.wrapWithCurrentZone=Eb;c.filterProperties=hc;c.attachOriginToPatched=
Nb;c._redefineProperty=Object.defineProperty;c.patchCallbacks=lb;c.getGlobalObjects=()=>({globalSources:oc,zoneSymbolEventNames:nc,eventNames:d,isBrowser:Qb,isMix:Rb,isNode:Pb,TRUE_STR:"true",FALSE_STR:"false",ZONE_SYMBOL_PREFIX:Db,ADD_EVENT_LISTENER_STR:"addEventListener",REMOVE_EVENT_LISTENER_STR:"removeEventListener"})})};let zc,Ac,Bc,Cc,Fc;
function Gc(){zc=Zone.__symbol__;Ac=Object[zc("defineProperty")]=Object.defineProperty;Bc=Object[zc("getOwnPropertyDescriptor")]=Object.getOwnPropertyDescriptor;Cc=Object.create;Fc=zc("unconfigurables");Object.defineProperty=function(a,b,c){if(a&&a[Fc]&&a[Fc][b])throw new TypeError("Cannot assign to read only property '"+b+"' of "+a);const d=c.configurable;"prototype"!==b&&(c=Hc(a,b,c));return Ic(a,b,c,d)};Object.defineProperties=function(a,b){Object.keys(b).forEach(function(c){Object.defineProperty(a,c,
b[c])});for(const c of Object.getOwnPropertySymbols(b)){let d;(null==(d=Object.getOwnPropertyDescriptor(b,c))?0:d.enumerable)&&Object.defineProperty(a,c,b[c])}return a};Object.create=function(a,b){"object"!==typeof b||Object.isFrozen(b)||Object.keys(b).forEach(function(c){b[c]=Hc(a,c,b[c])});return Cc(a,b)};Object.getOwnPropertyDescriptor=function(a,b){const c=Bc(a,b);c&&a&&a[Fc]&&a[Fc][b]&&(c.configurable=!1);return c}}function Jc(a,b,c){const d=c.configurable;c=Hc(a,b,c);return Ic(a,b,c,d)}
function Hc(a,b,c){Object.isFrozen(c)||(c.configurable=!0);c.configurable||(a[Fc]||Object.isFrozen(a)||Ac(a,Fc,{writable:!0,value:{}}),a[Fc]&&(a[Fc][b]=!0));return c}
function Ic(a,b,c,d){try{return Ac(a,b,c)}catch(e){if(c.configurable){"undefined"==typeof d?delete c.configurable:c.configurable=d;try{return Ac(a,b,c)}catch(f){d=!1;if("createdCallback"===b||"attachedCallback"===b||"detachedCallback"===b||"attributeChangedCallback"===b)d=!0;if(!d)throw f;d=null;try{d=JSON.stringify(c)}catch(g){d=c.toString()}console.log(`Attempting to configure '${b}' with descriptor '${d}' on object '${a}' and got error, giving up: ${f}`)}}else throw e;}};function Pc(a,b){const {eventNames:c,globalSources:d,zoneSymbolEventNames:e,TRUE_STR:f,FALSE_STR:g,ZONE_SYMBOL_PREFIX:h}=b.getGlobalObjects();var m="ApplicationCache EventSource FileReader InputMethodContext MediaController MessagePort Node Performance SVGElementInstance SharedWorker TextTrack TextTrackCue TextTrackList WebKitNamedFlow Window Worker WorkerGlobalScope XMLHttpRequest XMLHttpRequestEventTarget XMLHttpRequestUpload IDBRequest IDBOpenDBRequest IDBDatabase IDBTransaction IDBCursor DBIndex WebSocket".split(" ");
let p=[];var u="Anchor Area Audio BR Base BaseFont Body Button Canvas Content DList Directory Div Embed FieldSet Font Form Frame FrameSet HR Head Heading Html IFrame Image Input Keygen LI Label Legend Link Map Marquee Media Menu Meta Meter Mod OList Object OptGroup Option Output Paragraph Pre Progress Quote Script Select Source Span Style TableCaption TableCell TableCol Table TableRow TableSection TextArea Title Track UList Unknown Video".split(" ");a.wtf?p=u.map(na=>"HTML"+na+"Element").concat(m):
a.EventTarget?p.push("EventTarget"):p=m;const w=a.__Zone_disable_IE_check||!1,B=a.__Zone_enable_cross_context_check||!1,G=b.isIEOrEdge(),L={MSPointerCancel:"pointercancel",MSPointerDown:"pointerdown",MSPointerEnter:"pointerenter",MSPointerHover:"pointerhover",MSPointerLeave:"pointerleave",MSPointerMove:"pointermove",MSPointerOut:"pointerout",MSPointerOver:"pointerover",MSPointerUp:"pointerup"};for(m=0;m<c.length;m++){var O=c[m],ba=h+(O+g),ca=h+(O+f);e[O]={};e[O][g]=ba;e[O][f]=ca}for(m=0;m<u.length;m++)for(O=
u[m],ba=d[O]={},ca=0;ca<c.length;ca++){const na=c[ca];ba[na]=O+".addEventListener:"+na}u=[];for(m=0;m<p.length;m++)O=a[p[m]],u.push(O&&O.prototype);b.patchEventTarget(a,b,u,{fF:function(na,K,W,S){if(!w&&G)if(B)try{const ja=K.toString();if("[object FunctionWrapper]"===ja||"function __BROWSERTOOLS_CONSOLE_SAFEFUNC() { [native code] }"==ja)return na.apply(W,S),!1}catch(ja){return na.apply(W,S),!1}else{if(K=K.toString(),"[object FunctionWrapper]"===K||"function __BROWSERTOOLS_CONSOLE_SAFEFUNC() { [native code] }"==
K)return na.apply(W,S),!1}else if(B)try{K.toString()}catch(ja){return na.apply(W,S),!1}return!0},Zg:na=>L[na]||na});Zone[b.symbol("patchEventTarget")]=!!a.EventTarget};function Qc(a,b){const {ADD_EVENT_LISTENER_STR:c,REMOVE_EVENT_LISTENER_STR:d}=a.getGlobalObjects(),e=b.WebSocket;b.EventTarget||a.patchEventTarget(b,a,[e.prototype]);b.WebSocket=function(g,h){const m=1<arguments.length?new e(g,h):new e(g);let p,u;const w=a.ObjectGetOwnPropertyDescriptor(m,"onmessage");w&&!1===w.configurable?(p=a.ObjectCreate(m),u=m,[c,d,"send","close"].forEach(function(B){p[B]=function(){const G=a.ArraySlice.call(arguments);if(B===c||B===d){var L=0<G.length?G[0]:void 0;L&&(L=Zone.__symbol__("ON_PROPERTY"+
L),m[L]=p[L])}return m[B].apply(m,G)}})):p=m;a.patchOnProperties(p,["close","error","message","open"],u);return p};const f=b.WebSocket;for(const g in e)f[g]=e[g]};function Rc(a,b){const {isBrowser:c,isMix:d}=a.getGlobalObjects();if((c||d)&&!a.ObjectGetOwnPropertyDescriptor(HTMLElement.prototype,"onclick")&&"undefined"!==typeof Element){var e=a.ObjectGetOwnPropertyDescriptor(Element.prototype,"onclick");if(e&&!e.configurable)return!1;if(e){a.ObjectDefineProperty(Element.prototype,"onclick",{enumerable:!0,configurable:!0,get:function(){return!0}});var f=!!document.createElement("div").onclick;a.ObjectDefineProperty(Element.prototype,"onclick",e);return f}}var g=
b.XMLHttpRequest;if(!g)return!1;e=g.prototype;if(f=a.ObjectGetOwnPropertyDescriptor(e,"onreadystatechange"))return a.ObjectDefineProperty(e,"onreadystatechange",{enumerable:!0,configurable:!0,get:function(){return!0}}),g=!!(new g).onreadystatechange,a.ObjectDefineProperty(e,"onreadystatechange",f||{}),g;const h=a.symbol("fake");a.ObjectDefineProperty(e,"onreadystatechange",{enumerable:!0,configurable:!0,get:function(){return this[h]},set:function(m){this[h]=m}});a=new g;e=()=>{};a.onreadystatechange=
e;e=a[h]===e;a.onreadystatechange=null;return e}const Sc="abort animationcancel animationend animationiteration auxclick beforeinput blur cancel canplay canplaythrough change compositionstart compositionupdate compositionend cuechange click close contextmenu curechange dblclick drag dragend dragenter dragexit dragleave dragover drop durationchange emptied ended error focus focusin focusout gotpointercapture input invalid keydown keypress keyup load loadstart loadeddata loadedmetadata lostpointercapture mousedown mouseenter mouseleave mousemove mouseout mouseover mouseup mousewheel orientationchange pause play playing pointercancel pointerdown pointerenter pointerleave pointerlockchange mozpointerlockchange webkitpointerlockerchange pointerlockerror mozpointerlockerror webkitpointerlockerror pointermove pointout pointerover pointerup progress ratechange reset resize scroll seeked seeking select selectionchange selectstart show sort stalled submit suspend timeupdate volumechange touchcancel touchmove touchstart touchend transitioncancel transitionend waiting wheel webglcontextrestored webglcontextlost webglcontextcreationerror autocomplete autocompleteerror toggle afterscriptexecute beforescriptexecute DOMContentLoaded freeze fullscreenchange mozfullscreenchange webkitfullscreenchange msfullscreenchange fullscreenerror mozfullscreenerror webkitfullscreenerror msfullscreenerror readystatechange visibilitychange resume absolutedeviceorientation afterinput afterprint appinstalled beforeinstallprompt beforeprint beforeunload devicelight devicemotion deviceorientation deviceorientationabsolute deviceproximity hashchange languagechange message mozbeforepaint offline online paint pageshow pagehide popstate rejectionhandled storage unhandledrejection unload userproximity vrdisplayconnected vrdisplaydisconnected vrdisplaypresentchange beforecopy beforecut beforepaste copy cut paste dragstart loadend animationstart search transitionrun transitionstart webkitanimationend webkitanimationiteration webkitanimationstart webkittransitionend activate afterupdate ariarequest beforeactivate beforedeactivate beforeeditfocus beforeupdate cellchange controlselect dataavailable datasetchanged datasetcomplete errorupdate filterchange layoutcomplete losecapture move moveend movestart propertychange resizeend resizestart rowenter rowexit rowsdelete rowsinserted command compassneedscalibration deactivate help mscontentzoom msmanipulationstatechanged msgesturechange msgesturedoubletap msgestureend msgesturehold msgesturestart msgesturetap msgotpointercapture msinertiastart mslostpointercapture mspointercancel mspointerdown mspointerenter mspointerhover mspointerleave mspointermove mspointerout mspointerover mspointerup pointerout mssitemodejumplistitemremoved msthumbnailclick stop storagecommit".split(" ");
function Tc(a){const b=a.symbol("unbound");for(let c=0;c<Sc.length;c++){const d=Sc[c],e="on"+d;self.addEventListener(d,function(f){f=f.target;let g,h;for(h=f?f.constructor.name+"."+e:"unknown."+e;f;)f[e]&&!f[e][b]&&(g=a.wrapWithCurrentZone(f[e],h),g[b]=f[e],f[e]=g),f=f.parentElement},!0)}};function Uc(){const a="undefined"!==typeof window?window:"undefined"!==typeof global?global:"undefined"!==typeof self?self:{};a[(a.__Zone_symbol_prefix||"__zone_symbol__")+"legacyPatch"]=function(){const b=a.Zone;b.__load_patch("defineProperty",(c,d,e)=>{e._redefineProperty=Jc;Gc()});b.__load_patch("registerElement",(c,d,e)=>{const {isBrowser:f,isMix:g}=e.getGlobalObjects();(f||g)&&"registerElement"in c.document&&e.patchCallbacks(e,document,"Document","registerElement",["createdCallback","attachedCallback",
"detachedCallback","attributeChangedCallback"])});b.__load_patch("EventTargetLegacy",(c,d,e)=>{Pc(c,e);const {isNode:f,isMix:g}=e.getGlobalObjects();f&&!g||Rc(e,c)||(d="undefined"!==typeof WebSocket,Tc(e),e.patchClass("XMLHttpRequest"),d&&Qc(e,c),Zone[e.symbol("patchEvents")]=!0)})}};function Vc(a,b){b.patchMethod(a,"queueMicrotask",()=>function(c,d){Zone.current.scheduleMicroTask("queueMicrotask",d[0])})};const Zc=nb("zoneTask");
function $c(a,b,c,d){function e(p){const u=p.data;u.bf[0]=function(){return p.invoke.apply(this,arguments)};u.handleId=g.apply(a,u.bf);return p}function f(p){return h.call(a,p.data.handleId)}let g=null,h=null;b+=d;c+=d;const m={};g=cc(a,b,p=>function(u,w){if("function"===typeof w[0]){const B={isPeriodic:"Interval"===d,delay:"Timeout"===d||"Interval"===d?w[1]||0:void 0,bf:w},G=w[0];w[0]=function(){try{return G.apply(this,arguments)}finally{B.isPeriodic||("number"===typeof B.handleId?delete m[B.handleId]:
B.handleId&&(B.handleId[Zc]=null))}};u=Zone.current.scheduleMacroTask(b,w[0],B,e,f);if(!u)return u;w=u.data.handleId;"number"===typeof w?m[w]=u:w&&(w[Zc]=u);w&&w.Mj&&w.Ss&&"function"===typeof w.Mj&&"function"===typeof w.Ss&&(u.Mj=w.Mj.bind(w),u.Ss=w.Ss.bind(w));return"number"===typeof w||w?w:u}return p.apply(a,w)});h=cc(a,c,p=>function(u,w){u=w[0];let B;"number"===typeof u?B=m[u]:(B=u&&u[Zc])||(B=u);B&&"string"===typeof B.type?"notScheduled"!==B.state&&(B.cancelFn&&B.data.isPeriodic||0===B.runCount)&&
("number"===typeof u?delete m[u]:u&&(u[Zc]=null),B.zone.cancelTask(B)):p.apply(a,w)})};function ad(){var a=yc;a.__load_patch("legacy",b=>{(b=b[a.__symbol__("legacyPatch")])&&b()});a.__load_patch("timers",b=>{$c(b,"set","clear","Timeout");$c(b,"set","clear","Interval");$c(b,"set","clear","Immediate")});a.__load_patch("requestAnimationFrame",b=>{$c(b,"request","cancel","AnimationFrame");$c(b,"mozRequest","mozCancel","AnimationFrame");$c(b,"webkitRequest","webkitCancel","AnimationFrame")});a.__load_patch("blocking",(b,c)=>{const d=["alert","prompt","confirm"];for(let e=0;e<d.length;e++)cc(b,
d[e],(f,g,h)=>function(m,p){return c.current.run(f,b,p,h)})});a.__load_patch("EventTarget",(b,c,d)=>{d.patchEventPrototype(b,d);if(!Zone[d.symbol("patchEventTarget")]){var {eventNames:e,zoneSymbolEventNames:f,TRUE_STR:g,FALSE_STR:h,ZONE_SYMBOL_PREFIX:m}=d.getGlobalObjects();for(c=0;c<e.length;c++){const p=e[c],u=m+(p+h),w=m+(p+g);f[p]={};f[p][h]=u;f[p][g]=w}(c=b.EventTarget)&&c.prototype&&d.patchEventTarget(b,d,[c&&c.prototype])}(c=b.XMLHttpRequestEventTarget)&&c.prototype&&d.patchEventTarget(b,d,
[c.prototype])});a.__load_patch("MutationObserver",()=>{ac("MutationObserver");ac("WebKitMutationObserver")});a.__load_patch("IntersectionObserver",()=>{ac("IntersectionObserver")});a.__load_patch("FileReader",()=>{ac("FileReader")});a.__load_patch("on_property",(b,c,d)=>{if((!Pb||Rb)&&!Zone[d.symbol("patchEvents")]){c=b.__Zone_ignore_on_properties;d=[];if(Qb){var e=window;d=d.concat("Document SVGElement Element HTMLElement HTMLBodyElement HTMLMediaElement HTMLFrameSetElement HTMLFrameElement HTMLIFrameElement HTMLMarqueeElement Worker".split(" "));
a:{try{var f=Hb.navigator.userAgent;if(-1!==f.indexOf("MSIE ")||-1!==f.indexOf("Trident/")){var g=!0;break a}}catch(m){}g=!1}var h=g?[{target:e,AC:["error"]}]:[];g=e;f=ic(e);h=c?c.concat(h):c;e=sb(e);g&&(f=hc(g,f,h),Xb(g,f,e))}d=d.concat("XMLHttpRequest XMLHttpRequestEventTarget IDBIndex IDBRequest IDBOpenDBRequest IDBDatabase IDBTransaction IDBCursor WebSocket".split(" "));for(e=0;e<d.length;e++)(f=b[d[e]])&&f.prototype&&(g=f.prototype,f=ic(f.prototype),g&&(f=hc(g,f,c),Xb(g,f,void 0)))}});a.__load_patch("customElements",
(b,c,d)=>{const {isBrowser:e,isMix:f}=d.getGlobalObjects();(e||f)&&b.customElements&&"customElements"in b&&d.patchCallbacks(d,b.customElements,"customElements","define","connectedCallback disconnectedCallback adoptedCallback attributeChangedCallback formAssociatedCallback formDisabledCallback formResetCallback formStateRestoreCallback".split(" "))});a.__load_patch("XHR",(b,c)=>{(function(p){function u(S){const ja=S.data,F=ja.target;F[g]=!1;F[m]=!1;var I=F[f];L||(L=F[Bb],O=F[Cb]);I&&O.call(F,"readystatechange",
I);I=F[f]=()=>{if(F.readyState===F.DONE)if(!ja.aborted&&F[g]&&"scheduled"===S.state){const aa=F[c.__symbol__("loadfalse")];if(0!==F.status&&aa&&0<aa.length){const Aa=S.invoke;S.invoke=function(){const Za=F[c.__symbol__("loadfalse")];for(let vb=0;vb<Za.length;vb++)Za[vb]===S&&Za.splice(vb,1);ja.aborted||"scheduled"!==S.state||Aa.call(S)};aa.push(S)}else S.invoke()}else ja.aborted||!1!==F[g]||(F[m]=!0)};L.call(F,"readystatechange",I);F[d]||(F[d]=S);K.apply(F,ja.bf);F[g]=!0;return S}function w(){}function B(S){S=
S.data;S.aborted=!0;return W.apply(S.target,S.bf)}var G=p.XMLHttpRequest;if(G){G=G.prototype;var L=G[Bb],O=G[Cb];!L&&(p=p.XMLHttpRequestEventTarget)&&(p=p.prototype,L=p[Bb],O=p[Cb]);var ba=cc(G,"open",()=>function(S,ja){S[e]=0==ja[2];S[h]=ja[1];return ba.apply(S,ja)}),ca=nb("fetchTaskAborting"),na=nb("fetchTaskScheduling"),K=cc(G,"send",()=>function(S,ja){if(!0===c.current[na]||S[e])return K.apply(S,ja);ja={target:S,url:S[h],isPeriodic:!1,bf:ja,aborted:!1};const F=Zone.current.scheduleMacroTask("XMLHttpRequest.send",
w,ja,u,B);S&&!0===S[m]&&!ja.aborted&&"scheduled"===F.state&&F.invoke()}),W=cc(G,"abort",()=>function(S,ja){const F=S[d];if(F&&"string"==typeof F.type)null==F.cancelFn||F.data&&F.data.aborted||F.zone.cancelTask(F);else if(!0===c.current[ca])return W.apply(S,ja)})}})(b);const d=nb("xhrTask"),e=nb("xhrSync"),f=nb("xhrListener"),g=nb("xhrScheduled"),h=nb("xhrURL"),m=nb("xhrErrorBeforeScheduled")});a.__load_patch("geolocation",b=>{b.navigator&&b.navigator.geolocation&&Lb(b.navigator.geolocation)});a.__load_patch("PromiseRejectionEvent",
(b,c)=>{function d(e){return function(f){uc(b,e).forEach(g=>{var h=b.PromiseRejectionEvent;h&&(h=new h(e,{promise:f.promise,reason:f.rejection}),g.invoke(h))})}}b.PromiseRejectionEvent&&(c[nb("unhandledPromiseRejectionHandler")]=d("unhandledrejection"),c[nb("rejectionHandledHandler")]=d("rejectionhandled"))});a.__load_patch("queueMicrotask",(b,c,d)=>{Vc(b,d)})};function bd(){yc.__load_patch("canvas",(a,b,c)=>{a=a.HTMLCanvasElement;"undefined"!==typeof a&&a.prototype&&a.prototype.toBlob&&c.patchMacroTask(a.prototype,"toBlob",(d,e)=>({name:"HTMLCanvasElement.toBlob",target:d,en:0,bf:e}))})};function cd(a,b,c){const d={};for(const e in a)d[e]=b.call(c,a[e],e,a);return d};var dd;var fd=class{constructor(a){this.C=a}toString(){return this.C+""}};function gd(a){return a instanceof fd&&a.constructor===fd?a.C:"type_error:TrustedResourceUrl"}var kd={};function ld(a){if(void 0===dd){var b=null;var c=ya.trustedTypes;if(c&&c.createPolicy){try{b=c.createPolicy("goog#html",{createHTML:Da,createScript:Da,createScriptURL:Da})}catch(d){ya.console&&ya.console.error(d.message)}dd=b}else dd=b}a=(b=dd)?b.createScriptURL(a):a;return new fd(a,kd)};/*

 SPDX-License-Identifier: Apache-2.0
*/
var md=class{constructor(a){this.C=a}toString(){return this.C}};function nd(a){if(a instanceof md)return a.C;throw Error("");};class od{constructor(a){this.tr=a}}function rd(a){return new od(b=>b.substr(0,a.length+1).toLowerCase()===a+":")}const sd=[rd("data"),rd("http"),rd("https"),rd("mailto"),rd("ftp"),new od(a=>/^[^:]*([/?#]|$)/.test(a))];function td(a){if(a instanceof md)return a;for(let b=0;b<sd.length;++b){const c=sd[b];if(c instanceof od&&c.tr(a))return new md(a)}}var ud=/^\s*(?!javascript:)(?:[\w+.-]+:|[^:/?#]*(?:[/?#]|$))/i;function vd(a){a instanceof md?a=nd(a):a=ud.test(a)?a:void 0;return a};function wd(a,b){b=vd(b);void 0!==b&&(a.href=b)};function xd(a,b){a.src=gd(b);var c;let d;(c=(b=null==(d=(c=(a.ownerDocument&&a.ownerDocument.defaultView||window).document).querySelector)?void 0:d.call(c,"script[nonce]"))?b.nonce||b.getAttribute("nonce")||"":"")&&a.setAttribute("nonce",c)};var yd=String.prototype.repeat?function(a,b){return a.repeat(b)}:function(a,b){return Array(b+1).join(a)};function zd(a){throw Error("Data "+a.D+" not defined.");}function Bd(){var a=Cd;if(null==a.C)return null;if("boolean"===typeof a.C)return a.C;if("string"===typeof a.C){const b=a.C.toLowerCase();if("true"===b||"1"===b)return!0;if("false"===b||"0"===b)return!1}throw new TypeError(`Invalid boolean data <${a.D}>: ${a.C} (typeof ${typeof a.C})`);}function Dd(a,b){return fb(b,function(c,d){return new Gd(this.D+"["+d+"]",c)},a)}function Hd(a){a=a.F();return""==a.trim()?[]:a.split(",").map(b=>b.trim())}
class Gd{constructor(a,b){this.D=a;this.C=b}F(a){if(null==this.C)return 0==arguments.length&&zd(this),a;if("string"===typeof this.C)return this.C;throw new TypeError("Invalid string data <"+this.D+">: "+this.C+"-"+typeof this.C);}number(a){if(null==this.C)return 0==arguments.length&&zd(this),a;if("number"===typeof this.C)return this.C;if("string"===typeof this.C){const b=Number(this.C);if(!isNaN(b)&&!/^[\s\xa0]*$/.test(this.C))return b}throw new TypeError("Invalid numeric data <"+this.D+">: "+this.C+
"-"+typeof this.C);}toString(){if(null==this.C)var a=null;else if("string"===typeof this.C)a=this.C;else throw new TypeError(`Invalid string data <${this.D}>: ${this.C} (typeof ${typeof this.C})`);null===a&&zd(this);return a}array(a){if(null==this.C){if(0==arguments.length)throw Error("Data "+this.D+" not defined.");return a}return Dd(this,Ba(this.C)?this.C:"string"!==typeof this.C?[this.C]:Hd(this))}object(a){if(null==this.C){if(0==arguments.length)throw Error("Data "+this.D+" not defined.");return a}if(!Ba(this.C)&&
Ca(this.C))return cd(this.C,function(b,c){return new Gd(this.D+"."+c,b)},this);throw new TypeError("Invalid object data <"+this.D+">: "+this.C+"-"+typeof this.C);}};function Id(a=window){return a.WIZ_global_data}function Jd(a=window){return(a=Id(a))&&"mXaIFf"in a?a.mXaIFf:null};function Kd(){yc.__load_patch("Error",(a,b,c)=>{function d(K){let W={fz:K.zone.name},S=W;for(;K.parent;){K=K.parent;const ja={fz:K.zone.name};W=W.parent=ja}return S}function e(K,W,S=!0){K=K.split("\n");let ja=0;for(;K[ja]!==u&&K[ja]!==w&&K[ja]!==B&&K[ja]!==G&&K[ja]!==L&&ja<K.length;)ja++;for(;ja<K.length&&W;ja++){let F=K[ja];if(F.trim())switch(p[F]){case g.dt:K.splice(ja,1);ja--;break;case g.transition:W.parent?W=W.parent:W=null;K.splice(ja,1);ja--;break;default:K[ja]+=S?` [${W.zone.name}]`:` [${W.fz}]`}}return K.join("\n")}
function f(){let K=m.apply(this,arguments);const W=K.originalStack=K.stack;if(f.stackRewrite&&W){let S=c.currentZoneFrame();if("lazy"===O)K[c.symbol("zoneFrameNames")]=d(S);else if("default"===O)try{K.stack=K.zoneAwareStack=e(W,S)}catch(ja){}}return this instanceof m&&this.constructor!=m?(Object.keys(K).concat("stack","message").forEach(S=>{const ja=K[S];if(void 0!==ja)try{this[S]=ja}catch(F){}}),this):K}const g={dt:0,transition:1},h=c.symbol("zoneJsInternalStackFrames"),m=a[c.symbol("Error")]=a.Error,
p={};let u,w,B,G,L;a.Error=f;const O=a.__Zone_Error_BlacklistedStackFrames_policy||a.__Zone_Error_ZoneJsInternalStackFrames_policy||"default";f.prototype=m.prototype;f[h]=p;f.stackRewrite=!1;const ba=c.symbol("zoneAwareStack");"lazy"===O&&Object.defineProperty(f.prototype,"zoneAwareStack",{configurable:!0,enumerable:!0,get:function(){this[ba]||(this[ba]=e(this.originalStack,this[c.symbol("zoneFrameNames")],!1));return this[ba]},set:function(K){this.originalStack=K;this[ba]=e(this.originalStack,this[c.symbol("zoneFrameNames")],
!1)}});const ca=["stackTraceLimit","captureStackTrace","prepareStackTrace"];(a=Object.keys(m))&&a.forEach(K=>{0===ca.filter(W=>W===K).length&&Object.defineProperty(f,K,{get:function(){return m[K]},set:function(W){m[K]=W}})});m.hasOwnProperty("stackTraceLimit")&&(m.stackTraceLimit=Math.max(m.stackTraceLimit,15),Object.defineProperty(f,"stackTraceLimit",{get:function(){return m.stackTraceLimit},set:function(K){return m.stackTraceLimit=K}}));m.hasOwnProperty("captureStackTrace")&&Object.defineProperty(f,
"captureStackTrace",{value:function(K,W){m.captureStackTrace(K,W)}});Object.defineProperty(f,"prepareStackTrace",{get:function(){return m.vx},set:function(K){return K&&"function"===typeof K?m.vx=function(W,S){if(S)for(let ja=0;ja<S.length;ja++)if("zoneCaptureStackTrace"===S[ja].getFunctionName()){S.splice(ja,1);break}return K.call(this,W,S)}:m.vx=K}});if("disable"!==O){var na=b.current.fork({name:"detect",onHandleError:function(K,W,S,ja){if(ja.originalStack&&Error===f)for(K=ja.originalStack.split(/\n/),
ja=S=W=!1;K.length;){let F=K.shift();if(/:\d+:\d+/.test(F)||"ZoneAwareError"===F){let I=F.split("(")[0].split("@")[0],aa=g.transition;-1!==I.indexOf("ZoneAwareError")&&(-1!==I.indexOf("new ZoneAwareError")?(u=F,w=F.replace("new ZoneAwareError","new Error.ZoneAwareError")):(B=F,G=F.replace("Error.",""),-1===F.indexOf("Error.ZoneAwareError")&&(L=F.replace("ZoneAwareError","Error.ZoneAwareError"))),p[w]=g.dt);-1!==I.indexOf("runGuarded")?S=!0:-1!==I.indexOf("runTask")?ja=!0:-1!==I.indexOf("run")?W=!0:
aa=g.dt;p[F]=aa;if(W&&S&&ja){f.stackRewrite=!0;break}}}return!1}}).fork({name:"child",onScheduleTask:function(K,W,S,ja){return K.scheduleTask(S,ja)},onInvokeTask:function(K,W,S,ja,F,I){return K.invokeTask(S,ja,F,I)},onCancelTask:function(K,W,S,ja){return K.cancelTask(S,ja)},onInvoke:function(K,W,S,ja,F,I,aa){return K.invoke(S,ja,F,I,aa)}});b=Error.stackTraceLimit;Error.stackTraceLimit=100;na.run(()=>{na.runGuarded(()=>{const K=()=>{};na.scheduleEventTask(h,()=>{na.scheduleMacroTask(h,()=>{na.scheduleMicroTask(h,
()=>{throw Error();},void 0,W=>{W.wd=K;W.invoke()});na.scheduleMicroTask(h,()=>{throw Error();},void 0,W=>{W.wd=K;W.invoke()})},void 0,W=>{W.wd=K;W.invoke()},()=>{})},void 0,W=>{W.wd=K;W.invoke()},()=>{})})});Error.stackTraceLimit=b}})};function Ld(){yc.__load_patch("ZoneAwarePromise",(a,b,c)=>{function d(){}function e(){let T=!1;return function(da){return function(){T||(T=!0,da.apply(null,arguments))}}}function f(T){if(T&&T.toString===Object.prototype.toString){const da=T.constructor&&T.constructor.name;return(da?da:"")+": "+JSON.stringify(T)}return T?T.toString():Object.prototype.toString.call(T)}function g(T){c.onUnhandledError(T);try{const da=b[ja];"function"===typeof da&&da.call(this,T)}catch(da){}}function h(T){return T}function m(T){return bb.reject(T)}
function p(T,da){return oa=>{try{u(T,da,oa)}catch(Ia){u(T,!1,Ia)}}}function u(T,da,oa){var Ia=e();if(T===oa)throw new TypeError("Promise resolved with itself");if(null===T[F]){var db=null;try{if("object"===typeof oa||"function"===typeof oa)db=oa&&oa.then}catch(Oa){return Ia(()=>{u(T,!1,Oa)})(),T}if(!1!==da&&oa instanceof bb&&oa.hasOwnProperty(F)&&oa.hasOwnProperty(I)&&null!==oa[F])w(oa),u(T,oa[F],oa[I]);else if(!1!==da&&"function"===typeof db)try{db.call(oa,Ia(p(T,da)),Ia(p(T,!1)))}catch(Oa){Ia(()=>
{u(T,!1,Oa)})()}else{T[F]=da;Ia=T[I];T[I]=oa;T[aa]===aa&&!0===da&&(T[F]=T[Za],T[I]=T[Aa]);!1===da&&oa instanceof Error&&(db=b.currentTask&&b.currentTask.data&&b.currentTask.data.__creationTrace__)&&ba(oa,vb,{configurable:!0,enumerable:!1,writable:!0,value:db});for(db=0;db<Ia.length;)B(T,Ia[db++],Ia[db++],Ia[db++],Ia[db++]);if(0==Ia.length&&0==da){T[F]=0;da=oa;try{throw Error("Uncaught (in promise): "+f(oa)+(oa&&oa.stack?"\n"+oa.stack:""));}catch(Oa){da=Oa}K&&(da.throwOriginal=!0);da.rejection=oa;
da.promise=T;da.zone=b.current;da.task=b.currentTask;na.push(da);c.scheduleMicroTask()}}}return T}function w(T){if(0===T[F]){try{var da=b[kb];da&&"function"===typeof da&&da.call(this,{rejection:T[I],promise:T})}catch(oa){}T[F]=!1;for(da=0;da<na.length;da++)T===na[da].promise&&na.splice(da,1)}}function B(T,da,oa,Ia,db){w(T);const Oa=T[F],tb=Oa?"function"===typeof Ia?Ia:h:"function"===typeof db?db:m;da.scheduleMicroTask("Promise.then",()=>{try{const wb=T[I],sa=!!oa&&aa===oa[aa];sa&&(oa[Aa]=wb,oa[Za]=
Oa);const Ha=da.run(tb,void 0,sa&&tb!==m&&tb!==h?[]:[wb]);u(oa,!0,Ha)}catch(wb){u(oa,!1,wb)}},oa)}function G(T){const da=T.prototype,oa=O(da,"then");if(!oa||!1!==oa.writable&&oa.configurable){var Ia=da.then;da[S]=Ia;T.prototype.then=function(db,Oa){return(new bb((tb,wb)=>{Ia.call(this,tb,wb)})).then(db,Oa)};T[ma]=!0}}function L(T){return function(da,oa){da=T.apply(da,oa);if(da instanceof bb)return da;oa=da.constructor;oa[ma]||G(oa);return da}}const O=Object.getOwnPropertyDescriptor,ba=Object.defineProperty,
ca=c.symbol,na=[],K=!1!==a[ca("DISABLE_WRAPPING_UNCAUGHT_PROMISE_REJECTION")];var W=ca("Promise");const S=ca("then");c.onUnhandledError=T=>{if(c.showUncaughtError()){const da=T&&T.rejection;da?console.error("Unhandled Promise rejection:",da instanceof Error?da.message:da,"; Zone:",T.zone.name,"; Task:",T.task&&T.task.source,"; Value:",da,da instanceof Error?da.stack:void 0):console.error(T)}};c.microtaskDrainDone=()=>{for(;na.length;){const T=na.shift();try{T.zone.runGuarded(()=>{if(T.throwOriginal)throw T.rejection;
throw T;})}catch(da){g(da)}}};const ja=ca("unhandledPromiseRejectionHandler"),F=ca("state"),I=ca("value"),aa=ca("finally"),Aa=ca("parentPromiseValue"),Za=ca("parentPromiseState"),vb=ca("currentTaskTrace"),kb=ca("rejectionHandledHandler"),jb=a.AggregateError;class bb{static toString(){return"function ZoneAwarePromise() { [native code] }"}static resolve(T){return T instanceof bb?T:u(new this(null),!0,T)}static reject(T){return u(new this(null),!1,T)}static withResolvers(){const T={};T.promise=new bb((da,
oa)=>{T.resolve=da;T.reject=oa});return T}static any(T){if(!T||"function"!==typeof T[Symbol.iterator])return Promise.reject(new jb([],"All promises were rejected"));const da=[];let oa=0;try{for(let Oa of T)oa++,da.push(bb.resolve(Oa))}catch(Oa){return Promise.reject(new jb([],"All promises were rejected"))}if(0===oa)return Promise.reject(new jb([],"All promises were rejected"));let Ia=!1;const db=[];return new bb((Oa,tb)=>{for(let wb=0;wb<da.length;wb++)da[wb].then(sa=>{Ia||(Ia=!0,Oa(sa))},sa=>{db.push(sa);
oa--;0===oa&&(Ia=!0,tb(new jb(db,"All promises were rejected")))})})}static race(T){function da(tb){Ia(tb)}function oa(tb){db(tb)}let Ia,db,Oa=new this((tb,wb)=>{Ia=tb;db=wb});for(let tb of T)tb&&tb.then||(tb=this.resolve(tb)),tb.then(da,oa);return Oa}static all(T){return bb.Tu(T)}static allSettled(T){return(this&&this.prototype instanceof bb?this:bb).Tu(T,{IE:da=>({status:"fulfilled",value:da}),ZB:da=>({status:"rejected",reason:da})})}static Tu(T,da){let oa,Ia,db=new this((sa,Ha)=>{oa=sa;Ia=Ha}),
Oa=2,tb=0;const wb=[];for(let sa of T){sa&&sa.then||(sa=this.resolve(sa));const Ha=tb;try{sa.then(Wa=>{wb[Ha]=da?da.IE(Wa):Wa;Oa--;0===Oa&&oa(wb)},Wa=>{da?(wb[Ha]=da.ZB(Wa),Oa--,0===Oa&&oa(wb)):Ia(Wa)})}catch(Wa){Ia(Wa)}Oa++;tb++}Oa-=2;0===Oa&&oa(wb);return db}constructor(T){if(!(this instanceof bb))throw Error("Must be an instanceof Promise.");this[F]=null;this[I]=[];try{const da=e();T&&T(da(p(this,!0)),da(p(this,!1)))}catch(da){u(this,!1,da)}}get [Symbol.toStringTag](){return"Promise"}get [Symbol.species](){return bb}then(T,
da){var oa,Ia=null==(oa=this.constructor)?void 0:oa[Symbol.species];Ia&&"function"===typeof Ia||(Ia=this.constructor||bb);oa=new Ia(d);Ia=b.current;null==this[F]?this[I].push(Ia,oa,T,da):B(this,Ia,oa,T,da);return oa}catch(T){return this.then(null,T)}finally(T){var da,oa=null==(da=this.constructor)?void 0:da[Symbol.species];oa&&"function"===typeof oa||(oa=bb);da=new oa(d);da[aa]=aa;oa=b.current;null==this[F]?this[I].push(oa,da,T,T):B(this,oa,da,T,T);return da}}bb.resolve=bb.resolve;bb.reject=bb.reject;
bb.race=bb.race;bb.all=bb.all;W=a[W]=a.Promise;a.Promise=bb;const ma=ca("thenPatched");c.patchThen=G;W&&(G(W),cc(a,"fetch",T=>L(T)));Promise[b.__symbol__("uncaughtPromiseErrors")]=na;return bb})};function Md(){yc.__load_patch("toString",a=>{function b(){if("function"===typeof this){var h=this[d];if(h)return"function"===typeof h?c.call(h):Object.prototype.toString.call(h);if(this===Promise&&(h=a[e])||this===Error&&(h=a[f]))return c.call(h)}return c.call(this)}const c=Function.prototype.toString,d=nb("OriginalDelegate"),e=nb("Promise"),f=nb("Error");b[d]=c;Function.prototype.toString=b;const g=Object.prototype.toString;Object.prototype.toString=function(){return"function"===typeof Promise&&
this instanceof Promise?"[object Promise]":g.call(this)}})};var Nd;if(!(Nd="undefined"===typeof window)){var Cd=new Gd("mXaIFf",Jd(window));const a=Bd();Nd=null===a?!0:a}if(Nd){var yc;const a=globalThis,b=!0===a[nb("forceDuplicateZoneCheck")];if(a.Zone&&(b||"function"!==typeof a.Zone.__symbol__))throw Error("Zone already loaded.");null!=a.Zone||(a.Zone=ob());yc=a.Zone;Kd();Ld();Md();xc();Uc();ad();bd()};function Od(a){for(let b in a)if(a[b]===Od)return b;throw Error("Could not find renamed property on target object.");};function Pd(a){return{ma:a.ma,fa:a.fa||null,ba:a.ba,value:void 0}}function Qd(a){return{Wa:a.Wa||[],imports:a.imports||[]}}function Rd(a){var b;(b=a.hasOwnProperty(Sd)?a[Sd]:null)||(b=a.hasOwnProperty(Wd)?a[Wd]:null);return b}function Xd(a){return(a=a&&(a[Sd]||a[Wd]))?a:null}function Yd(a){return a&&(a.hasOwnProperty(Zd)||a.hasOwnProperty($d))?a[Zd]:null}var Sd=Od({na:Od}),Zd=Od({Xa:Od}),Wd=Od({mG:Od}),$d=Od({nG:Od});var ae=class extends Error{constructor(a,b){super(`${`NG0${Math.abs(a)}`}${b?": "+b:""}`);this.code=a}};var be=Od({Ca:Od}),ce=Od({Fa:Od}),de=Od({WG:Od}),ee=Od({Ya:Od}),fe=Od({K:Od}),ge=Od({xf:Od}),he=Od({Lz:Od});function ie(a){return"string"===typeof a?a:null==a?"":String(a)};function je(a){if("string"===typeof a)return a;if(Array.isArray(a))return"["+a.map(je).join(", ")+"]";if(null==a)return""+a;if(a.DD)return`${a.DD}`;if(a.name)return`${a.name}`;a=a.toString();if(null==a)return""+a;const b=a.indexOf("\n");return-1===b?a:a.substring(0,b)}function le(a,b){return null==a||""===a?null===b?"":b:null==b||""===b?a:a+" "+b};/*

 The MIT License

 Copyright (c) 2014-2016 Google, Inc. http://angular.io

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
*/
function me(){throw new ae(-201,!1);};function ne(a,b){return Object.is(a,b)};let oe=null,pe=!1,qe=1;var re=Symbol("SIGNAL");function se(a){const b=oe;oe=a;return b}var te={version:0,Mw:0,sc:!1,oc:void 0,ei:void 0,qe:void 0,ko:0,gd:void 0,Eg:void 0,pq:!1,qq:!1,yx:()=>!1,zx:()=>{},rq:()=>{},yv:()=>{}};function ue(a){if(pe)throw Error("");if(null!==oe){oe.yv(a);var b=oe.ko++;we(oe);b<oe.oc.length&&oe.oc[b]!==a&&xe(oe)&&ye(oe.oc[b],oe.qe[b]);oe.oc[b]!==a&&(oe.oc[b]=a,oe.qe[b]=xe(oe)?ze(a,oe,b):0);oe.ei[b]=a.version}}
function Ae(a){if(!xe(a)||a.sc)if(a.sc||a.Mw!==qe)(a.yx(a)||Be(a))&&a.zx(a),a.sc=!1,a.Mw=qe}function Ce(a){if(void 0!==a.gd){var b=pe;pe=!0;try{for(const c of a.gd)c.sc||De(c)}finally{pe=b}}}function Ee(){let a;return!1!==(null==(a=oe)?void 0:a.pq)}function De(a){a.sc=!0;Ce(a);let b;null==(b=a.rq)||b.call(a,a)}function Fe(a){a&&(a.ko=0);return se(a)}
function Ge(a,b){se(b);if(a&&void 0!==a.oc&&void 0!==a.qe&&void 0!==a.ei){if(xe(a))for(b=a.ko;b<a.oc.length;b++)ye(a.oc[b],a.qe[b]);for(;a.oc.length>a.ko;)a.oc.pop(),a.ei.pop(),a.qe.pop()}}function Be(a){we(a);for(let b=0;b<a.oc.length;b++){const c=a.oc[b],d=a.ei[b];if(d!==c.version)return!0;Ae(c);if(d!==c.version)return!0}return!1}function He(a){we(a);if(xe(a))for(let b=0;b<a.oc.length;b++)ye(a.oc[b],a.qe[b]);a.oc.length=a.ei.length=a.qe.length=0;a.gd&&(a.gd.length=a.Eg.length=0)}
function ze(a,b,c){Ie(a);we(a);if(0===a.gd.length)for(let d=0;d<a.oc.length;d++)a.qe[d]=ze(a.oc[d],a,d);a.Eg.push(c);return a.gd.push(b)-1}function ye(a,b){Ie(a);we(a);if(1===a.gd.length)for(var c=0;c<a.oc.length;c++)ye(a.oc[c],a.qe[c]);c=a.gd.length-1;a.gd[b]=a.gd[c];a.Eg[b]=a.Eg[c];a.gd.length--;a.Eg.length--;b<a.gd.length&&(c=a.Eg[b],a=a.gd[b],we(a),a.qe[c]=b)}function xe(a){let b,c;return a.qq||0<(null!=(c=null==a?void 0:null==(b=a.gd)?void 0:b.length)?c:0)}
function we(a){null!=a.oc||(a.oc=[]);null!=a.qe||(a.qe=[]);null!=a.ei||(a.ei=[])}function Ie(a){null!=a.gd||(a.gd=[]);null!=a.Eg||(a.Eg=[])};function Je(a){const b=Object.create(Ke);b.AB=a;a=()=>{Ae(b);ue(b);if(b.value===Le)throw b.error;return b.value};a[re]=b;return a}
const Me=Symbol("UNSET"),Ne=Symbol("COMPUTING"),Le=Symbol("ERRORED"),Ke=Object.assign({},te,{value:Me,sc:!0,error:null,Lq:ne,yx(a){return a.value===Me||a.value===Ne},zx(a){if(a.value===Ne)throw Error("Detected cycle in computations.");const b=a.value;a.value=Ne;const c=Fe(a);let d;try{d=a.AB()}catch(e){d=Le,a.error=e}finally{Ge(a,c)}b!==Me&&b!==Le&&d!==Le&&a.Lq(b,d)?a.value=b:(a.value=d,a.version++)}});function Oe(){throw Error();};function Pe(a){const b=Object.create(Qe);b.value=a;a=()=>{ue(b);return b.value};a[re]=b;return a}function Re(a,b){Ee()||Oe();a.Lq(a.value,b)||(a.value=b,a.version++,qe++,Ce(a))}var Qe=Object.assign({},te,{Lq:ne,value:void 0});function Se(a,b,c){const d=Object.create(Ye);c&&(d.pq=!0);d.Dn=a;d.Oc=b;const e=f=>{d.ej=f};d.Mj={notify:()=>De(d),run:()=>{if(null!==d.Dn){if(pe)throw Error("Schedulers cannot synchronously execute watches while scheduling.");d.sc=!1;if(!d.qw||Be(d)){d.qw=!0;var f=Fe(d);try{d.ej(),d.ej=Ze,d.Dn(e)}finally{Ge(d,f)}}}},fe:()=>d.ej(),destroy:()=>{if(null!==d.Dn||null!==d.Oc)He(d),d.ej(),d.Dn=null,d.Oc=null,d.ej=Ze},[re]:d};return d.Mj}
const Ze=()=>{},Ye=Object.assign({},te,{qq:!0,pq:!1,rq:a=>{null!==a.Oc&&a.Oc(a.Mj)},qw:!1,ej:Ze});let $e;function af(a){const b=$e;$e=a;return b}function bf(a,b,c){if((a=Rd(a))&&"root"==a.fa)return void 0===a.value?a.value=a.ba():a.value;if(c&8)return null;if(void 0!==b)return b;me()};const cf=Od({ut:Od});function df(a){a.ut=df;a.toString=function(){return je(this())};return a}function ef(a){return ff(a)?a():a}function ff(a){return"function"===typeof a&&a.hasOwnProperty(cf)&&a.ut===df};var gf=class{constructor(a,b){this.C=a;this.na=void 0;"number"==typeof b?this.xf=b:void 0!==b&&(this.na=Pd({ma:this,fa:b.fa||"root",ba:b.ba}))}get Eb(){return this}toString(){return`InjectionToken ${this.C}`}};var hf=globalThis;var jf={};const kf=/\n/gm;let lf=void 0;function mf(a){const b=lf;lf=a;return b}function nf(a,b=0){if(void 0===lf)throw new ae(-203,!1);return null===lf?bf(a,void 0,b):lf.get(a,b&8?null:void 0,b)}function r(a,b=0){return($e||nf)(ef(a),b)}function of(a,b=0){return r(a,pf(b))}function pf(a){return"undefined"===typeof a||"number"===typeof a?a:0|(a.optional&&8)|(a.host&&1)|(a.self&&2)|(a.JG&&4)}
function qf(a){const b=[];for(let c=0;c<a.length;c++){const d=ef(a[c]);if(Array.isArray(d)){if(0===d.length)throw new ae(900,!1);let e=void 0,f=0;for(let g=0;g<d.length;g++){const h=d[g],m=h.__NG_DI_FLAG__;"number"===typeof m?-1===m?e=h.ma:f|=m:e=h}b.push(r(e,f))}else b.push(r(d))}return b}function rf(a,b){a.__NG_DI_FLAG__=b;a.prototype.__NG_DI_FLAG__=b;return a}
function sf(a,b,c=null){a=a&&"\n"===a.charAt(0)&&"\u0275"==a.charAt(1)?a.slice(2):a;var d=je(b);if(Array.isArray(b))d=b.map(je).join(" -> ");else if("object"===typeof b){d=[];for(let e in b)if(b.hasOwnProperty(e)){let f=b[e];d.push(e+":"+("string"===typeof f?JSON.stringify(f):je(f)))}d=`{${d.join(", ")}}`}return`${"R3InjectorError"}${c?"("+c+")":""}[${d}]: ${a.replace(kf,"\n  ")}`};var tf=new gf("");var uf=new gf("",-1);var vf=new gf("");var wf=class{get(a,b=jf){if(b===jf)throw a=Error(`NullInjectorError: No provider for ${je(a)}!`),a.name="NullInjectorError",a;return b}};function xf(a,b,c){let d=0;for(;d<c.length;){var e=c[d];if("number"===typeof e){if(0!==e)break;d++;e=c[d++];var f=c[d++];const g=c[d++];a.setAttribute(b,f,g,e)}else f=c[++d],64===e.charCodeAt(0)?a.setProperty(b,e,f):a.setAttribute(b,e,f),d++}}function yf(a,b){if(null!==b&&0!==b.length)if(null===a||0===a.length)a=b.slice();else{let c=-1;for(let d=0;d<b.length;d++){const e=b[d];"number"===typeof e?c=e:0!==c&&(-1===c||2===c?zf(a,c,e,b[++d]):zf(a,c,e,null))}}return a}
function zf(a,b,c,d){let e=0,f=a.length;if(-1===b)f=-1;else for(;e<a.length;){var g=a[e++];if("number"===typeof g)if(g===b){f=-1;break}else if(g>b){f=e-1;break}}for(;e<a.length;){g=a[e];if("number"===typeof g)break;else if(g===c){null!==d&&(a[e+1]=d);return}e++;null!==d&&e++}-1!==f&&(a.splice(f,0,b),e=f+1);a.splice(e++,0,c);null!==d&&a.splice(e++,0,d)};function Af(a){return 4===a.type&&"ng-template"!==a.value}
function Bf(a,b,c){let d=4;const e=a.attrs;if(null!==e)a:{for(var f=0;f<e.length;f++){var g=e[f];if(3===g||4===g||6===g){g=f;break a}}g=e.length}else g=0;f=!1;for(let G=0;G<b.length;G++){var h=b[G];if("number"===typeof h){if(!f&&0!==(d&1)&&0!==(h&1))return!1;f&&0===(h&1)||(f=!1,d=h|d&1)}else if(!f)if(d&4){if(d=2|d&1,""!==h&&h!==(4!==a.type||c?a.value:"ng-template")||""===h&&1===b.length){if(0===(d&1))return!1;f=!0}}else if(d&8){var m;if(!(m=null===e)){a:{m=void 0;var p=a,u=0;if(c)for(;u<e.length&&
"string"===typeof e[u];u+=2){if(p="class"===e[u]){b:{p=e[u+1].toLowerCase();for(var w=0,B=p.length;;){w=p.indexOf(h,w);if(-1===w){p=w;break b}if(0===w||32>=p.charCodeAt(w-1)){const L=h.length;if(w+L===B||32>=p.charCodeAt(w+L)){p=w;break b}}w+=1}p=void 0}p=-1!==p}if(p){h=!0;break a}}else if(Af(p)){h=!1;break a}u=e.indexOf(1,u);if(-1<u)for(;++u<e.length&&"string"===typeof(m=e[u]);)if(m.toLowerCase()===h){h=!0;break a}h=!1}m=!h}if(m){if(0===(d&1))return!1;f=!0}}else{m=b[++G];a:if(null===e)h=-1;else if(u=
0,c||!Af(a)){for(p=!1;u<e.length;){B=e[u];if(B===h){h=u;break a}if(3===B||6===B)p=!0;else if(1===B||2===B){for(B=e[++u];"string"===typeof B;)B=e[++u];continue}else if(4===B)break;else if(0===B){u+=4;continue}u+=p?1:2}h=-1}else b:{u=e.indexOf(4);if(-1<u)for(u++;u<e.length;){p=e[u];if("number"===typeof p)break;if(p===h){h=u;break b}u++}h=-1}if(-1===h){if(0===(d&1))return!1;f=!0}else if(""!==m&&(h=h>g?"":e[h+1].toLowerCase(),d&2&&m!==h)){if(0===(d&1))return!1;f=!0}}}return 0===(d&1)||f}
function Cf(a,b,c=!1){for(let d=0;d<b.length;d++)if(Bf(a,b[d],c))return!0;return!1}function Df(a,b){return a?":not("+b.trim()+")":b}function Ef(a){let b=a[0],c=1,d=2,e="",f=!1;for(;c<a.length;){let g=a[c];if("string"===typeof g)if(d&2){const h=a[++c];e+="["+g+(0<h.length?'="'+h+'"':"")+"]"}else d&8?e+="."+g:d&4&&(e+=" "+g);else""!==e&&0!==(g&1)&&(b+=Df(f,e),e=""),d=g,f=f||0!==(d&1);c++}""!==e&&(b+=Df(f,e));return b};function Ff(a){return{toString:a}.toString()};var Hf={},If=[];function Jf(a){return Ff(()=>{var b=Kf(a),c;b=Object.assign({},b,{ya:a.ya,za:a.za,sa:a.sa,Ba:a.Ba||null,bc:a.bc,Kr:0===a.Pa,Sv:null,qx:null,Ha:b.ka&&a.Ha||null,Rq:null,li:null!=(c=a.li)?c:!1,data:a.data||{},pb:a.pb||0,styles:a.styles||If,Gb:null,Uj:a.Uj||null,ub:null,id:""});Lf(b);c=a.Ha;b.Sv=Pf(c,!1);b.qx=Pf(c,!0);c=0;const d=[b.ha,b.bc,b.Za,b.Ka,b.Ba,b.za,b.ya,b.pb,b.ka,b.li,b.ib,JSON.stringify(b.inputs),JSON.stringify(b.outputs),Object.getOwnPropertyNames(b.type.prototype),!!b.hc,!!b.eb].join("|");
for(const e of d)c=Math.imul(31,c)+e.charCodeAt(0)<<0;b.id="c"+(c+2147483648);return b})}function Qf(a){return a[be]||a[ce]||null}function Rf(a){return null!==a}function Sf(a){return Ff(()=>({type:a.type,Pk:a.Pk||If,LB:a.LB||If,imports:a.imports||If,exports:a.exports||If,PG:null,Uj:a.Uj||null,id:a.id||null}))}
function Tf(a,b){if(null==a)return Hf;const c={};for(const e in a)if(a.hasOwnProperty(e)){var d=a[e];let f,g=0;if(Array.isArray(d)){g=d[0];f=d[1];let h;d=null!=(h=d[2])?h:f}else f=d;b?(c[f]=0!==g?[e,g]:e,b[f]=d):c[f]=e}return c}function Uf(a){return Ff(()=>{const b=Kf(a);Lf(b);return b})}function Vf(a){return a[de]||null}
function Kf(a){const b={};return{type:a.type,fs:null,ba:null,Ia:a.Ia||null,Za:a.Za||0,Ka:a.Ka||null,hc:a.hc||null,jj:b,He:null,GC:a.inputs||Hf,ib:a.ib||null,ka:!0===a.ka,li:!0===a.li,ha:a.ha||If,eb:a.eb||null,features:a.features||null,Ho:null,sj:null,tl:null,inputs:Tf(a.inputs,b),outputs:Tf(a.outputs),OF:null}}function Lf(a){let b;null==(b=a.features)||b.forEach(c=>c(a))}function Pf(a,b){if(!a)return null;const c=b?Vf:Qf;return()=>("function"===typeof a?a():a).map(d=>c(d)).filter(Rf)};function Wf(a){return a.hasOwnProperty(fe)?a[fe]:null};function Xf(a,b){a.forEach(c=>Array.isArray(c)?Xf(c,b):b(c))}function Yf(a,b){return b>=a.length-1?a.pop():a.splice(b,1)[0]}function Zf(a,b,c){let d=$f(a,b);if(0<=d)a[d|1]=c;else{var e=d=~d;let f=a.length;if(f==e)a.push(b,c);else if(1===f)a.push(c,a[0]),a[0]=b;else{f--;for(a.push(a[f-1],a[f]);f>e;)a[f]=a[f-2],f--;a[e]=b;a[e+1]=c}}return d}function ag(a,b){b=$f(a,b);if(0<=b)return a[b|1]}
function $f(a,b){let c=0,d=a.length>>1;for(;d!==c;){const e=c+(d-c>>1),f=a[e<<1];if(b===f)return e<<1;f>b?d=e:c=e+1}return~(d<<1)};function bg(...a){return{fk:cg(a),VG:!0}}function cg(...a){const b=[],c=new Set;let d;const e=f=>{b.push(f)};Xf(a,f=>{dg(f,e,[],c)&&(d||(d=[]),d.push(f))});void 0!==d&&eg(d,e);return b}function eg(a,b){for(let c=0;c<a.length;c++){const {Dj:d,Wa:e}=a[c];fg(e,f=>{b(f,d)})}}
function dg(a,b,c,d){a=ef(a);if(!a)return!1;let e=null;var f=Yd(a);const g=!f&&(a[be]||null);if(f||g){if(g&&!g.ka)return!1;e=a}else{var h=a.Dj;if(f=Yd(h))e=h;else return!1}h=d.has(e);if(g){if(h)return!1;d.add(e);if(g.Ha){f="function"===typeof g.Ha?g.Ha():g.Ha;for(var m of f)dg(m,b,c,d)}}else if(f){if(null!=f.imports&&!h){d.add(e);let p;Xf(f.imports,u=>{dg(u,b,c,d)&&(p||(p=[]),p.push(u))});void 0!==p&&eg(p,b)}h||(m=Wf(e)||(()=>new e),b({pa:e,Gc:m,Bc:If},e),b({pa:vf,nb:e,Eb:!0},e),b({pa:tf,nb:()=>r(e),
Eb:!0},e));f=f.Wa;if(null!=f&&!h){const p=a;fg(f,u=>{b(u,p)})}}else return!1;return e!==a&&void 0!==a.Wa}function fg(a,b){for(let c of a)c&&c.fk&&(c=c.fk),Array.isArray(c)?fg(c,b):b(c)}var gg=Od({pa:String,nb:Od});function hg(a){return null!==a&&"object"==typeof a&&gg in a}function ig(a){return"function"===typeof a};var jg=new gf("");const kg={},lg={};let mg=void 0;function ng(){void 0===mg&&(mg=new wf);return mg}var og=class{};function pg(a,b){b=ef(b);let c=ig(b)?b:ef(b&&b.pa);const d=qg(b);if(!ig(b)&&!0===b.Eb){let e=a.D.get(c);e||(e=rg(void 0,kg,!0),e.ba=()=>qf(e.Eb),a.D.set(c,e));c=b;e.Eb.push(b)}a.D.set(c,d)}function sg(a){if(a.Ga)throw new ae(205,!1);}function tg(a){const b=se(null),c=mf(a),d=af();try{const e=a.get(tf,If,2);for(const f of e)f()}finally{mf(c),af(d),se(b)}}
var wg=class extends og{get C(){return this.Ga}constructor(a,b,c,d){super();this.parent=b;this.source=c;this.N=d;this.D=new Map;this.G=new Set;this.F=[];this.Ga=!1;ug(a,e=>pg(this,e));this.D.set(uf,rg(void 0,this));d.has("environment")&&this.D.set(og,rg(void 0,this));a=this.D.get(jg);null!=a&&"string"===typeof a.value&&this.N.add(a.value);this.J=new Set(this.get(vf,If,2))}destroy(){sg(this);this.Ga=!0;const a=se(null);try{for(const c of this.G)c.va();const b=this.F;this.F=[];for(const c of b)c()}finally{this.D.clear(),
this.G.clear(),this.J.clear(),se(a)}}ob(a){sg(this);this.F.push(a);return()=>{const b=this.F.indexOf(a);-1!==b&&this.F.splice(b,1)}}get(a,b=jf,c=0){sg(this);if(a.hasOwnProperty(he))return a[he](this);c=pf(c);const d=mf(this),e=af();try{if(!(c&4)){let B=this.D.get(a);if(void 0===B){const G=("function"===typeof a||"object"===typeof a&&a instanceof gf)&&Rd(a);var f;if(f=G)if(G.fa){var g=ef(G.fa);f="string"===typeof g?"any"===g||this.N.has(g):this.J.has(g)}else f=!1;f?B=rg(vg(a),kg):B=null;this.D.set(a,
B)}if(null!=B){a:{b=B;const G=se(null);try{b.value===kg&&(b.value=lg,b.value=b.ba());var h;if(h="object"===typeof b.value&&b.value){var m=b.value;h=null!==m&&"object"===typeof m&&"function"===typeof m.va}h&&this.G.add(b.value);var p=b.value;break a}finally{se(G)}p=void 0}return p}}const w=c&2?ng():this.parent;b=c&8&&b===jf?null:b;return w.get(a,b)}catch(w){if("NullInjectorError"===w.name){(w.ngTempTokenPath=w.ngTempTokenPath||[]).unshift(je(a));if(d)throw w;p=w;var u=this.source;const B=p.ngTempTokenPath;
a.__source&&B.unshift(a.__source);p.message=sf("\n"+p.message,B,u);p.ngTokenPath=B;p.ngTempTokenPath=null;throw p;}throw w;}finally{af(e),mf(d)}}toString(){const a=[],b=this.D;for(const c of b.keys())a.push(je(c));return`R3Injector[${a.join(", ")}]`}};function vg(a){var b=Rd(a);b=null!==b?b.ba:Wf(a);if(null!==b)return b;if(a instanceof gf)throw new ae(204,!1);if(a instanceof Function)return xg(a);throw new ae(204,!1);}
function xg(a){if(0<a.length)throw new ae(204,!1);const b=Xd(a);return null!==b?()=>b.ba(a):()=>new a}function qg(a){if(hg(a))return rg(void 0,a.nb);a=Bg(a);return rg(a,kg)}function Bg(a){var b=void 0;if(ig(a))return b=ef(a),Wf(b)||vg(b);if(hg(a))b=()=>ef(a.nb);else if(a&&a.Gc)b=()=>a.Gc(...qf(a.Bc||[]));else if(a&&a.Ab)b=()=>r(ef(a.Ab));else{const c=ef(a&&(a.Fc||a.pa));if(a.Bc)b=()=>new c(...qf(a.Bc));else return Wf(c)||vg(c)}return b}function rg(a,b,c=!1){return{ba:a,value:b,Eb:c?[]:void 0}}
function ug(a,b){for(const c of a)Array.isArray(c)?ug(c,b):c&&c&&c.fk?ug(c.fk,b):b(c)};function Cg(a,b){a instanceof wg&&sg(a);a=mf(a);const c=af();try{return b()}finally{mf(a),af(c)}}function Dg(){if(void 0===$e&&null==lf)throw new ae(-203,!1);};function Eg(a){return Array.isArray(a)&&"object"===typeof a[1]}function Fg(a){return Array.isArray(a)&&!0===a[1]};var Gg=class{constructor(a,b,c){this.xx=a;this.xq=b;this.Bn=c}};function Hg(){return Ig}function Ig(a){a.type.prototype.uc&&(a.Ho=Jg);return Lg}Hg.ax=!0;function Lg(){const a=this.__ngSimpleChanges__||null,b=null==a?void 0:a.current;if(b){const c=a.bs;if(c===Hf)a.bs=b;else for(let d in b)c[d]=b[d];a.current=null;this.uc(b)}}function Jg(a,b,c,d,e){d=this.jj[d];var f;(f=a.__ngSimpleChanges__||null)||(f=a.__ngSimpleChanges__={bs:Hf,current:null});var g=f;f=g.current||(g.current={});g=g.bs;const h=g[d];f[d]=new Gg(h&&h.xq,c,g===Hf);null!==b?b.Wu(b,c):a[e]=c};function Mg(a){for(;Array.isArray(a);)a=a[0];return a}function Ng(a,b){return Mg(b[a.index])}function Og(a,b){a=b[a];return Eg(a)?a:a[0]}function Pg(a,b){return null===b||void 0===b?null:a[b]}function Qg(a){let b;return!!(a[2]&9216||(null==(b=a[23])?0:b.sc))}function Rg(a){let b;null==(b=a[10].cj)||b.notify(7);a[2]&64&&(a[2]|=1024);Qg(a)&&Sg(a)}function Sg(a){let b;null==(b=a[10].cj)||b.notify(0);for(a=Tg(a);null!==a&&!(a[2]&8192);){a[2]|=8192;if(128!==(a[2]&128))break;a=Tg(a)}}
function Ug(a,b){if(256===(a[2]&256))throw new ae(911,!1);null===a[21]&&(a[21]=[]);a[21].push(b)}function Tg(a){a=a[3];return Fg(a)?a[3]:a};var Wg=Vg(null),Xg=null;let Yg=!1;function Zg(){return Wg.Tf}function t(a){Wg.Dh=a;return a[8]}function x(a){Wg.Dh=null;return a}function $g(){let a=Wg.Gh;for(;null!==a&&64===a.type;)a=a.parent;return a}function ah(a,b){const c=Wg;c.Gh=a;c.Sf=b}function bh(){const a=Wg;let b=a.bn;-1===b&&(b=a.bn=a.ub.jq);return b}function ch(a){const b=Wg,c=b.cd;b.cd+=a;return c}function dh(a){const b=Wg.ij;return-1===b?null:a[b]}
function eh(a,b,c){if(c&4){for(;!(b=b.parent,null!==b||c&1);){b=a[1];b=2===b.type?b.Ov:1===b.type?a[5]:null;if(null===b)break;a=a[14];if(b.type&10)break}if(null===b)return!1}c=Wg=fh();c.Gh=b;c.Tf=a;return!0}function gh(a){const b=fh(),c=a[1];Wg=b;b.Gh=c.firstChild;b.Tf=a;b.ub=c;b.Dh=a;b.cd=c.jq;b.tw=!1}function fh(){const a=Wg,b=null===a?null:a.If;return null===b?Vg(a):b}
function Vg(a){const b={Gh:null,Sf:!0,Tf:null,ub:null,selectedIndex:-1,Dh:null,vn:0,al:null,ij:-1,bn:-1,cd:-1,ge:0,parent:a,If:null,tw:!1};null!==a&&(a.If=b);return b}function hh(){const a=Wg;Wg=a.parent;a.Gh=null;a.Tf=null;return a}function ih(){const a=hh();a.Sf=!0;a.ub=null;a.selectedIndex=-1;a.Dh=null;a.vn=0;a.ij=-1;a.al=null;a.bn=-1;a.cd=-1;a.ge=0}function jh(){const a=Wg;return a.ub.data[a.selectedIndex]}function kh(){Wg.al="svg"}let lh=!0;function mh(a,b){for(let d=b.nd,e=b.Ce;d<e;d++){var c=a.data[d].type.prototype;b=c.Bj;const f=c.Yw,g=c.pd,h=c.Zw;c=c.va;if(b){let m,p;(null!=(p=(m=a).Xk)?p:m.Xk=[]).push(-d,b)}if(f){let m,p;(null!=(p=(m=a).Xk)?p:m.Xk=[]).push(d,f);let u,w;(null!=(w=(u=a).sq)?w:u.sq=[]).push(d,f)}if(g){let m,p;(null!=(p=(m=a).pm)?p:m.pm=[]).push(-d,g)}if(h){let m,p;(null!=(p=(m=a).pm)?p:m.pm=[]).push(d,h);let u,w;(null!=(w=(u=a).Ws)?w:u.Ws=[]).push(d,h)}if(null!=c){let m,p;(null!=(p=(m=a).bl)?p:m.bl=[]).push(d,c)}}}
function nh(a,b,c,d){(a[2]&3)===c&&oh(a,b,c,d)}function ph(a,b){let c=a[2];(c&3)===b&&(a[2]=(c&16383)+1)}function oh(a,b,c,d){var e=void 0!==d?a[17]&65535:0;const f=null!=d?d:-1,g=b.length-1;let h=0;for(;e<g;e++)if("number"===typeof b[e+1]){if(h=b[e],null!=d&&h>=d)break}else{0>b[e]&&(a[17]+=65536);if(h<f||-1==f){const m=0>b[e],p=b[e+1],u=a[m?-b[e]:b[e]];m?a[2]>>14<a[17]>>16&&(a[2]&3)===c&&(a[2]+=16384,qh(u,p)):qh(u,p);a[17]=(a[17]&4294901760)+e+2}e++}}
function qh(a,b){const c=se(null);try{b.call(a)}finally{se(c)}};var rh=class{constructor(a,b){var c=y;this.ba=a;this.F=!1;this.J=b;this.G=c}};function sh(a,b){for(a>>=16;0<a;)b=b[14],a--;return b};let th=!0;function uh(a){const b=th;th=a;return b}let vh=0;const wh={};function xh(a,b,c){let d;"string"===typeof c?d=c.charCodeAt(0)||0:c.hasOwnProperty(ge)&&(d=c[ge]);null==d&&(d=c[ge]=vh++);c=d&255;b.data[a+(c>>5)]|=1<<c}function yh(a,b){var c=zh(a,b);if(-1!==c)return c;c=b[1];c.Cc&&(a.Ad=b.length,Ah(c.data,a),Ah(b,null),Ah(c.sg,null));c=Bh(a,b);a=a.Ad;if(-1!==c){const d=c&32767,e=sh(c,b),f=e[1].data;for(let g=0;8>g;g++)b[a+g]=e[d+g]|f[d+g]}b[a+8]=c;return a}
function Ah(a,b){a.push(0,0,0,0,0,0,0,0,b)}function zh(a,b){return-1===a.Ad||a.parent&&a.parent.Ad===a.Ad||null===b[a.Ad+8]?-1:a.Ad}function Bh(a,b){if(a.parent&&-1!==a.parent.Ad)return a.parent.Ad;a=0;let c=b;for(;null!==c;){b=Ch(c);if(null===b)break;a++;c=c[14];if(-1!==b.Ad)return b.Ad|a<<16}return-1}
function Dh(a){var b=$g();if("class"===a)return b.dj;if("style"===a)return b.styles;if(b=b.attrs){const c=b.length;let d=0;for(;d<c;){const e=b[d];if(3===e||4===e||6===e)break;if(0===e)d+=2;else if("number"===typeof e)for(d++;d<c&&"string"===typeof b[d];)d++;else{if(e===a)return b[d+1];d+=2}}}return null}function Eh(a,b,c,d){c&8&&void 0===d&&(d=null);if(0===(c&3)){a=a[9];const e=af();try{return a?a.get(b,d,c&8):bf(b,d,c&8)}finally{af(e)}}c&8||void 0!==d?b=d:(me(),b=void 0);return b}
function Fh(a,b,c,d=0,e){if(null!==a){if(b[2]&2048&&!(d&2)){a:{var f=wh;var g=a;let h=b;for(;null!==g&&null!==h&&h[2]&2048&&!(h[2]&512);){const m=Gh(g,h,c,d|2);if(m!==wh){f=m;break a}g=g.parent;if(!g){if(g=h[20])if(g=g.get(c,wh,d),g!==wh){f=g;break a}g=Ch(h);h=h[14]}}}if(f!==wh)return f}a=Gh(a,b,c,d);if(a!==wh)return a}return Eh(b,c,d,e)}
function Gh(a,b,c,d){var e=wh;if("string"===typeof c)var f=c.charCodeAt(0)||0;else f=c.hasOwnProperty(ge)?c[ge]:void 0,f="number"===typeof f?0<=f?f&255:Hh:f;if("function"===typeof f){if(!eh(b,a,d))return d&1?d&8||void 0!==e?b=e:(me(),b=void 0):b=Eh(b,c,d,e),b;try{var g=f(d);if(null!=g||d&8)return g;me()}finally{hh()}}else if("number"===typeof f){g=null;let m=zh(a,b);let p=d&1?b[15][5]:null;if(-1===m||d&4){var h=-1===m?Bh(a,b):b[m+8];-1===h||d&2?m=-1:(g=b[1],m=h&32767,b=sh(h,b))}for(;-1!==m;){a=b[1];
if(a.data[m+(f>>5)]&1<<f){h=b[1];const u=h.data[m+8],w=Ih(u,h,c,null==g?-1<u.Kf&&th:g!=h&&0!==(u.type&3),d&1&&p===u);h=null!==w?Jh(b,h,w,u):wh;if(h!==wh)return h}h=b[m+8];-1!==h&&!(d&2||d&1&&b[1].data[m+8]===p)&&b[m+(f>>5)]&1<<f?(g=a,m=h&32767,b=sh(h,b)):m=-1}}return e}function Ih(a,b,c,d,e){var f=a.Lg;const g=b.data;var h=f&1048575;b=a.nd;a=a.Ce;f>>=20;d=d?h:h+f;for(h=e?h+f:a;d<h;d++)if(f=g[d],d<b&&c===f||d>=b&&f.type===c)return d;return e&&(e=g[b])&&e.sa&&e.type===c?b:null}
function Jh(a,b,c,d){let e=a[c];const f=b.data;if(e instanceof rh){const h=e;if(h.F)throw a=f[c],new ae(-200,"function"===typeof a?a.name||a.toString():"object"===typeof a&&null!=a&&"function"===typeof a.type?a.type.name||a.type.toString():ie(a));const m=uh(h.J);h.F=!0;const p=h.G?af(h.G):null;eh(a,d,0);try{if(e=a[c]=h.ba(void 0,f,a,d),b.Cc&&c>=d.nd){var g=f[c];const u=g.type.prototype.Fb,w=g.type.prototype.Cj;if(g.type.prototype.uc){const B=Ig(g);let G;(null!=(G=b.nf)?G:b.nf=[]).push(c,B);let L;
(null!=(L=b.Kg)?L:b.Kg=[]).push(c,B)}if(u){let B;(null!=(B=b.nf)?B:b.nf=[]).push(-c,u)}if(w){let B;(null!=(B=b.nf)?B:b.nf=[]).push(c,w);let G;(null!=(G=b.Kg)?G:b.Kg=[]).push(c,w)}}}finally{null!==p&&af(p),uh(m),h.F=!1,hh()}}return e}var Kh=class{constructor(a,b){this.F=a;this.D=b}get(a,b,c){return Fh(this.F,this.D,a,pf(c),b)}};function Hh(){return new Kh($g(),Zg())}
function Lh(a){return Ff(()=>{var b=a.prototype.constructor;b=b[fe]||Sh(b);const c=Object.prototype;let d=Object.getPrototypeOf(a.prototype).constructor;for(;d&&d!==c;){const e=d[fe]||Sh(d);if(e&&e!==b)return e;d=Object.getPrototypeOf(d)}return e=>new e})}function Sh(a){return ff(a)?()=>{const b=Sh(ef(a));return b&&b()}:Wf(a)}function Ch(a){const b=a[1],c=b.type;return 2===c?b.Ov:1===c?a[5]:null};(new gf("")).xf=a=>{const b=$g();if(null===b)throw new ae(204,!1);if(b.type&2)return b.value;if(a&8)return null;throw new ae(204,!1);};function Th(a){return function(...b){if(a){b=a(...b);for(const c in b)this[c]=b[c]}}}function Uh(a){return Ff(()=>{function b(...d){if(this instanceof b)return c.apply(this,d),this;const e=new b(...d);return function(f,g,h){for(g=f.hasOwnProperty("__parameters__")?f.__parameters__:Object.defineProperty(f,"__parameters__",{value:[]}).__parameters__;g.length<=h;)g.push(null);(g[h]=g[h]||[]).push(e);return f}}const c=Th(a);return b})};var Vh=rf(Uh(a=>({ma:a})),-1),Wh=rf(Uh(),8);rf(Uh(),2);var Xh=rf(Uh(),4);rf(Uh(),1);Od({pa:String,nb:Od});function Yh(a,b=null,c=null,d){a=Zh(a,b,c,d);tg(a);return a}function Zh(a,b=null,c=null,d,e=new Set){c=[c||If,bg(a)];d=d||("object"===typeof a?void 0:je(a));return new wg(c,b||ng(),d||null,e)};function $h(a){if(Array.isArray(a))return Yh({name:""},void 0,a,"");let b;const c=null!=(b=a.name)?b:"";return Yh({name:c},a.parent,a.Wa,c)}var ai=class{},bi=new wf;ai.na=Pd({ma:ai,fa:"any",ba:()=>r(uf)});ai.xf=-1;var ci=class{};var di=class{},ei=class{};var fi=class{},gi=new gf("",{fa:"root",ba:()=>!1});new gf("",{fa:"root",ba:()=>!1});var hi=new gf("");let ii=void 0;const ji=new Map;let ki=0;function li(a,b){Eg(b)?(a.__ngContext__=b[19],ji.set(b[19],b)):a.__ngContext__=b};function mi(a){for(;null!==a&&!Fg(a);)a=a[4];return a};var ni=new gf("",{fa:"root",ba:()=>"ng"}),oi=new gf(""),pi=new gf("",{fa:"platform",ba:()=>"unknown"});new gf("");var qi=new gf(""),ri=new gf("",{fa:"root",ba:()=>{let a,b;if(void 0!==ii)var c=ii;else if("undefined"!==typeof document)c=document;else throw new ae(210,!1);return(null==(a=c.body)?void 0:null==(b=a.querySelector("[ngCspNonce]"))?void 0:b.getAttribute("ngCspNonce"))||null}}),si={bv:[16,32,48,64,96,128,256,384,640,750,828,1080,1200,1920,2048,3840],xG:30,RF:!1,QF:!1};
new gf("",{fa:"root",ba:()=>si});var ti=class{},ui=class{};function vi(a,b){return new wi(Ng(a,b))}var wi=class{constructor(a){this.R=a}};wi.xf=function(){return vi($g(),Zg())};function xi(a){return a instanceof wi?a.R:a};var yi=class{},zi=class{constructor(){this.cl=null}};zi.xf=()=>{const a=Zg(),b=Og($g().index,a);return(Eg(b)?b:a)[11]};var Ai=class{constructor(){this.Gt=console}handleError(a){const b=this.dA(a);this.Gt.error("ERROR",a);b&&this.Gt.error("ORIGINAL ERROR",b)}dA(a){for(a=a&&a.ngOriginalError;a&&a.ngOriginalError;)a=a.ngOriginalError;return a||null}},Bi=new gf("",{fa:"root",ba:()=>of(Ai).handleError.bind(this)});var Ci=class{};Ci.xf=Di;Ci.Lz=a=>a;class Ei extends Ci{constructor(){var a=Zg();super();this.D=a}ob(a){Ug(this.D,a);return()=>{var b=this.D;if(null!==b[21]){var c=b[21].indexOf(a);-1!==c&&b[21].splice(c,1)}}}}function Di(){return new Ei};function Fi(a){return a.ownerDocument.defaultView}function Gi(a){return a.ownerDocument};const Hi=new Set;function Ii(a){if(!Hi.has(a)){Hi.add(a);var b,c;null==(b=performance)||null==(c=b.mark)||c.call(b,"mark_feature_usage",{detail:{bG:a}})}};/*


 Copyright (c) 2015-2018 Google, Inc., Netflix, Inc., Microsoft Corp. and contributors
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at
     http://www.apache.org/licenses/LICENSE-2.0
 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
let Ji=!1,Ki=!1;
var Li={set dk(a){this.QD||(a?console.warn("DEPRECATED! RxJS was set to use deprecated synchronous error handling behavior by code at: \n"+Error().stack):Ji&&console.log("RxJS: Back to a better error behavior. Thank you. <3"));Ji=a},get dk(){return Ji},set My(a){this.QD||(a?console.warn("DEPRECATED! RxJS was set to use deprecated next context. This will result in deoptimizations when creating any new subscription. \n"+Error().stack):Ki&&console.log("RxJS: back to more optimized subscription creation. Thank you. <3"));Ki=
a},get My(){return Ki}};function Mi(){};function Ni(a){setTimeout(()=>{throw a;})};var Oi={closed:!0,next:Mi,error(a){if(Li.dk)throw a;Ni(a)},complete:Mi};function Pi(a){a=a(b=>{b.name=b.constructor.name;b.stack=Error().stack});a.prototype=Object.create(Error.prototype);return a.prototype.constructor=a};var Qi=Pi(a=>function(b){a(this);this.message=b?`${b.length} errors occurred during unsubscription:
${b.map((c,d)=>`${d+1}) ${c.toString()}`).join("\n  ")}`:"";this.name="UnsubscriptionError";this.errors=b});function Ri(a){return"function"===typeof a};var Si=class{constructor(){this.FC=void 0;this.closed=!1;this.Ti=this.ec=this.ng=null}unsubscribe(){let a;if(!this.closed){this.closed=!0;const c=this.ng;let d;if(c)this.ng=null,c.remove(this);else if(d=this.ec){this.ec=null;for(var b of d)b.remove(this)}b=this.FC;if(Ri(b))try{b()}catch(f){a=f instanceof Qi?f.errors:[f]}const e=this.Ti;this.Ti=null;if(e)for(const f of e)try{"function"===typeof f?f():f.unsubscribe()}catch(g){let h;a=null!=(h=a)?h:[];g instanceof Qi?a=[...a,...g.errors]:a.push(g)}if(a)throw new Qi(a);
}}add(a){if(a&&a!==this)if(this.closed)"function"===typeof a?a():a.unsubscribe();else{if(a instanceof Si){if(a.closed||a.sA(this))return;a.Nz(this)}let b;this.Ti=null!=(b=this.Ti)?b:[];this.Ti.push(a)}}sA(a){let b;return this.ng===a||(null==(b=this.ec)?void 0:b.includes(a))||!1}Nz(a){const b=this.ng;let c;b?(this.ec=[b,a],this.ng=null):(c=this.ec)?c.push(a):this.ng=a}QA(a){const b=this.ng;let c;if(b)b===a&&(this.ng=null);else if(c=this.ec)a=c.indexOf(a),0<=a&&c.splice(a,1)}remove(a){const b=this.Ti;
if(b){const c=b.indexOf(a);0<=c&&b.splice(c,1)}a instanceof Si&&a.QA(this)}},Ti=new Si;Ti.closed=!0;Si.EMPTY=Ti;function Ui(a){return a instanceof Si||a&&"closed"in a&&"function"===typeof a.remove&&"function"===typeof a.add&&"function"===typeof a.unsubscribe};var Vi=class extends Si{static create(a,b,c){a=new Vi(a,b,c);a.cg=!1;return a}constructor(a,b,c){super();this.Ns=null;this.Sb=this.cg=this.Ms=!1;switch(arguments.length){case 0:this.destination=Oi;break;case 1:if(!a){this.destination=Oi;break}if("object"===typeof a){a instanceof Vi?(this.cg=a.cg,this.destination=a,a.add(this)):(this.cg=!0,this.destination=new Wi(this,a));break}default:this.cg=!0,this.destination=new Wi(this,a,b,c)}}next(a){this.Sb||this.Oa(a)}error(a){this.Sb||(this.Sb=!0,this.ve(a))}complete(){this.Sb||
(this.Sb=!0,this.pc())}unsubscribe(){this.closed||(this.Sb=!0,super.unsubscribe())}Oa(a){this.destination.next(a)}ve(a){this.destination.error(a);this.unsubscribe()}pc(){this.destination.complete();this.unsubscribe()}},Wi=class extends Vi{constructor(a,b,c,d){super();this.D=a;let e;if(Ri(b))e=b;else if(b&&(e=b.next,c=b.error,d=b.complete,b!==Oi)){let f;Li.My?(f=Object.create(b),f.unsubscribe=this.unsubscribe.bind(this)):f=b;e=e&&e.bind(f);c=c&&c.bind(f);d=d&&d.bind(f);Ui(b)&&b.add(this.unsubscribe.bind(this))}this.Oa=
e;this.ve=c;this.pc=d}next(a){if(!this.Sb&&this.Oa)try{this.Oa(a)}catch(b){this.C(b)}}error(a){if(!this.Sb)if(this.ve){try{this.ve(a)}catch(b){this.C(b);return}this.unsubscribe()}else this.C(a)}C(a){this.unsubscribe();if(Li.dk){const b=this.D;if(null==b?0:b.cg)b.Ns=a,b.Ms=!0;else throw a;}else Ni(a)}complete(){if(!this.Sb){if(this.pc)try{this.pc()}catch(a){this.C(a);return}this.unsubscribe()}}unsubscribe(){if(!this.closed){const a=this.D;this.D=null;a.unsubscribe();super.unsubscribe()}}};var Xi="function"===typeof Symbol&&Symbol.observable||"@@observable";function Yi(a){return a};function Zi(a){return 0===a.length?Yi:1===a.length?a[0]:function(b){return a.reduce((c,d)=>d(c),b)}};function $i(a){return a&&"function"===typeof a.next&&"function"===typeof a.error&&"function"===typeof a.complete}class aj extends Vi{constructor(a){super();this.destination=a}};function C(a,...b){return 0===b.length?a:Zi(b)(a)}
var bj=class{constructor(a){a&&(this.ye=a)}bo(a){const b=new bj;b.source=this;b.operator=a;return b}subscribe(a,b,c){const d=this.operator;a:{if(a){if(a instanceof Vi||$i(a)&&Ui(a))break a;if($i(a)){a=new aj(a);break a}}a=a||b||c?new Vi(a,b,c):new Vi(Oi)}d?a.add(d.call(a,this.source)):a.add(this.source||Li.dk&&!a.cg?this.ye(a):this.D(a));if(Li.dk&&a.cg&&(a.cg=!1,a.Ms))throw a.Ns;return a}D(a){try{return this.ye(a)}catch(c){if(Li.dk)a.Ms=!0,a.Ns=c;else{var b;a:{for(b=a;b;){const d=b.destination,e=
b.Sb;if(b.closed||e){b=!1;break a}d&&d instanceof Vi?b=d:b=null}b=!0}b?a.error(c):Ni(c)}}}forEach(a,b){b=cj(b);return new b((c,d)=>{let e;e=this.subscribe(f=>{try{a(f)}catch(g){d(g),e&&e.unsubscribe()}},d,c)})}ye(a){const b=this.source;return b&&b.subscribe(a)}[Xi](){return this}};bj.create=a=>new bj(a);function cj(a){a||(a=Promise);if(!a)throw Error("no Promise impl found");return a};var dj=class extends Si{constructor(a,b){super();this.Vg=a;this.Ks=b;this.closed=!1}unsubscribe(){if(!this.closed){this.closed=!0;var a=this.Vg,b=a.nc;this.Vg=null;!b||0===b.length||a.Sb||a.closed||(a=b.indexOf(this.Ks),-1!==a&&b.splice(a,1))}}};var ej=Pi(a=>function(){a(this);this.message="object unsubscribed"});function fj(a){const b=new bj;b.source=a;return b}
var hj=class extends bj{constructor(){super();this.nc=[];this.Fe=this.Sb=this.closed=!1;this.C=null}bo(a){const b=new gj(this,this);b.operator=a;return b}next(a){if(this.closed)throw new ej;if(!this.Sb){var b=this.nc;const c=b.length;b=b.slice();for(let d=0;d<c;d++)b[d].next(a)}}error(a){if(this.closed)throw new ej;this.Fe=!0;this.C=a;this.Sb=!0;var b=this.nc;const c=b.length;b=b.slice();for(let d=0;d<c;d++)b[d].error(a);this.nc.length=0}complete(){if(this.closed)throw new ej;this.Sb=!0;var a=this.nc;
const b=a.length;a=a.slice();for(let c=0;c<b;c++)a[c].complete();this.nc.length=0}unsubscribe(){this.closed=this.Sb=!0;this.nc=null}D(a){if(this.closed)throw new ej;return super.D(a)}ye(a){if(this.closed)throw new ej;if(this.Fe)return a.error(this.C),Si.EMPTY;if(this.Sb)return a.complete(),Si.EMPTY;this.nc.push(a);return new dj(this,a)}};hj.create=(a,b)=>new gj(a,b);
var gj=class extends hj{constructor(a,b){super();this.destination=a;this.source=b}next(a){const b=this.destination;b&&b.next&&b.next(a)}error(a){this.destination&&this.destination.error(a)}complete(){this.destination&&this.destination.complete()}ye(a){return this.source?this.source.subscribe(a):Si.EMPTY}};function ij(a){if(a.Fe)throw a.C;if(a.closed)throw new ej;return a.Lc}var jj=class extends hj{constructor(a){super();this.Lc=a}get value(){return ij(this)}ye(a){const b=super.ye(a);b&&!b.closed&&a.next(this.Lc);return b}next(a){super.next(this.Lc=a)}};var kj=new bj(a=>a.complete());function lj(a,b){return new bj(c=>{const d=new Si;let e=0;d.add(b.Oc(function(){e===a.length?c.complete():(c.next(a[e++]),c.closed||d.add(this.Oc()))}));return d})};var mj=a=>b=>{for(let c=0,d=a.length;c<d&&!b.closed;c++)b.next(a[c]);b.complete()};function nj(a){return a&&"function"===typeof a.Oc};function oj(...a){let b=a[a.length-1];return nj(b)?(a.pop(),lj(a,b)):new bj(mj(a))};function pj(a){return new bj(b=>b.error("function"===typeof a?a():a))};var qj={now(){return(qj.rb||Date).now()},rb:void 0};var rj=class extends hj{constructor(a=Infinity,b=Infinity,c=qj){super();this.aa=c;this.Bb=[];this.J=!1;this.F=1>a?1:a;this.O=1>b?1:b;Infinity===b?(this.J=!0,this.next=this.P):this.next=this.T}P(a){if(!this.Sb){const b=this.Bb;b.push(a);b.length>this.F&&b.shift()}super.next(a)}T(a){this.Sb||(this.Bb.push({time:this.G(),value:a}),this.N());super.next(a)}ye(a){const b=this.J,c=b?this.Bb:this.N(),d=c.length;let e;if(this.closed)throw new ej;this.Sb||this.Fe?e=Si.EMPTY:(this.nc.push(a),e=new dj(this,a));
if(b)for(var f=0;f<d&&!a.closed;f++)a.next(c[f]);else for(f=0;f<d&&!a.closed;f++)a.next(c[f].value);this.Fe?a.error(this.C):this.Sb&&a.complete();return e}G(){const a=this.aa;return a?a.now():qj.now()}N(){const a=this.G(),b=this.F,c=this.O,d=this.Bb,e=d.length;let f=0;for(;f<e&&!(a-d[f].time<c);)f++;e>b&&(f=Math.max(f,e-b));0<f&&d.splice(0,f);return d}};var tj=class{constructor(a,b=sj){this.D=a;this.now=b}Oc(a,b=0,c){return(new this.D(this,a)).Oc(c,b)}},sj=qj.now;var uj=Pi(a=>function(){a(this);this.message="no elements in sequence"});var vj=class extends Vi{constructor(a,b,c,d){super(a);this.C=d;b&&(this.Oa=function(e){try{null==b||b(e)}catch(f){this.error(f)}});c&&(this.pc=function(){try{c()}catch(e){this.destination.error(e)}this.unsubscribe()})}unsubscribe(){let a;this.closed||null!=(a=this.C)&&a.call(this);super.unsubscribe()}};function wj(a,b){if(a&&"function"===typeof a.bo)return a.bo(b);throw new TypeError("Unable to lift unknown Observable type");};function xj(){return a=>wj(a,function(b){const c=this;let d=null;b.Ek++;const e=new vj(c,void 0,void 0,()=>{if(!b||0>=b.Ek||0<--b.Ek)d=null;else{var g=b.Hi,h=d;d=null;!g||h&&g!==h||g.unsubscribe();c.unsubscribe()}}),f=b.subscribe(e);e.closed||(d=b.connect());return f})};var zj=class extends bj{constructor(a,b){super();this.source=a;this.vy=b;this.Ek=0;this.Hm=!1}ye(a){return this.Jn().subscribe(a)}Jn(){const a=this.yh;if(!a||a.Sb)this.yh=this.vy();return this.yh}connect(){let a=this.Hi;a||(this.Hm=!1,a=this.Hi=new Si,a.add(this.source.subscribe(new yj(this.Jn(),this))),a.closed&&(this.Hi=null,a=Si.EMPTY));return a}Wl(){return xj()(this)}},Aj;const Bj=zj.prototype;
Aj={operator:{value:null},Ek:{value:0,writable:!0},yh:{value:null,writable:!0},Hi:{value:null,writable:!0},ye:{value:Bj.ye},Hm:{value:Bj.Hm,writable:!0},Jn:{value:Bj.Jn},connect:{value:Bj.connect},Wl:{value:Bj.Wl}};
class yj extends Vi{constructor(a,b){super();this.destination=a;this.D=b}ve(a){this.C();super.ve(a)}pc(){this.D.Hm=!0;this.C();super.pc()}C(){const a=this.D;if(a){this.D=null;const b=a.Hi;a.Ek=0;a.yh=null;a.Hi=null;b&&b.unsubscribe()}}unsubscribe(){this.closed||(this.C(),super.unsubscribe())}};function Cj(a){return function(b){if("function"!==typeof a)throw new TypeError("argument is not a function. Are you looking for `mapTo()`?");return wj(b,new Dj(a))}}var Dj=class{constructor(a){this.project=a}call(a,b){return b.subscribe(new Ej(a,this.project))}};class Ej extends Vi{constructor(a,b){super(a);this.project=b;this.count=0;this.C=this}Oa(a){let b;try{b=this.project.call(this.C,a,this.count++)}catch(c){this.destination.error(c);return}this.destination.next(b)}};var Fj=Array.isArray||(a=>a&&"number"===typeof a.length);var Gj="function"===typeof Symbol&&Symbol.iterator?Symbol.iterator:"@@iterator";function Hj(a){return null!==a&&"object"===typeof a};function Ij(a){return b=>{Jj(a,b).catch(c=>b.error(c))}}
function Jj(a,b){return ta(function*(){var c;try{var d,e=a[Symbol.asyncIterator];if(void 0!==e)var f=e.call(a);else{var g="undefined"!=typeof Symbol&&Symbol.iterator&&a[Symbol.iterator];if(g)var h=g.call(a);else if("number"==typeof a.length)h={next:ea(a)};else throw Error(String(a)+" is not an iterable or ArrayLike");f=new ua(h)}for(d=f;;){var m=yield d.next();if(m.done)break;b.next(m.value)}}catch(u){var p={error:u}}finally{try{m&&!m.done&&(c=d.return)&&(yield c.call(d))}finally{if(p)throw p.error;
}}b.complete()})};var Kj=a=>b=>{const c=a[Gj]();do{let d;try{d=c.next()}catch(e){b.error(e);return}if(d.done){b.complete();break}b.next(d.value);if(b.closed)break}while(1);"function"===typeof c.return&&b.add(()=>{c.return&&c.return()});return b};var Lj=a=>b=>{const c=a[Xi]();if("function"!==typeof c.subscribe)throw new TypeError("Provided object does not correctly implement Symbol.observable");return c.subscribe(b)};var Mj=a=>b=>{a.then(c=>{b.closed||(b.next(c),b.complete())},c=>b.error(c)).then(null,Ni);return b};var Nj=a=>{if(a&&"function"===typeof a[Xi])return Lj(a);if(a&&"number"===typeof a.length&&"function"!==typeof a)return mj(a);if(a&&"function"!==typeof a.subscribe&&"function"===typeof a.then)return Mj(a);if(a&&"function"===typeof a[Gj])return Kj(a);if(Symbol&&Symbol.asyncIterator&&a&&"function"===typeof a[Symbol.asyncIterator])return Ij(a);throw new TypeError(`You provided ${Hj(a)?"an invalid object":`'${a}'`} where a stream was expected.`+" You can provide an Observable, Promise, Array, or Iterable.");
};var Oj=class extends Vi{constructor(a){super();this.parent=a}Oa(a){this.parent.Gg(a)}ve(a){this.parent.Jr(a);this.unsubscribe()}pc(){this.parent.Fg();this.unsubscribe()}},Pj=class extends Vi{constructor(a,b){super();this.parent=a;this.C=b}Oa(a){this.parent.Gg(null,a,this.C)}ve(a){this.parent.Jr(a);this.unsubscribe()}pc(){this.parent.Fg();this.unsubscribe()}},Qj=class extends Vi{Gg(a){this.destination.next(a)}Jr(a){this.destination.error(a)}Fg(){this.destination.complete()}},Rj=class extends Vi{Gg(a,
b){this.destination.next(b)}Jr(a){this.destination.error(a)}Fg(){this.destination.complete()}};function Sj(a,b){if(!b.closed)return a instanceof bj?a.subscribe(b):Nj(a)(b)};const Tj={};function Uj(...a){let b=void 0,c=void 0,d=void 0;nj(a[a.length-1])&&(c=a.pop());"function"===typeof a[a.length-1]&&(b=a.pop());if(1===a.length){const e=a[0];Fj(e)&&(a=e);Hj(e)&&Object.getPrototypeOf(e)===Object.prototype&&(d=Object.keys(e),a=d.map(f=>e[f]))}return wj(c?lj(a,c):new bj(mj(a)),new Vj(b,d))}
var Vj=class{constructor(a,b){this.C=a;this.keys=b}call(a,b){return b.subscribe(new Wj(a,this.C,this.keys))}},Wj=class extends Rj{constructor(a,b,c){super(a);this.D=b;this.keys=c;this.active=0;this.values=[];this.F=[]}Oa(a){this.values.push(Tj);this.F.push(a)}pc(){const a=this.F,b=a.length;if(0===b)this.destination.complete();else{this.C=this.active=b;for(let c=0;c<b;c++)this.add(Sj(a[c],new Pj(this,c)))}}Fg(){0===--this.active&&this.destination.complete()}Gg(a,b,c){const d=this.values;var e=d[c];
e=this.C?e===Tj?--this.C:this.C:0;d[c]=b;0===e&&(this.D?this.G(d):this.destination.next(this.keys?this.keys.reduce((f,g,h)=>(f[g]=d[h],f),{}):d.slice()))}G(a){let b;try{b=this.D.apply(this,a)}catch(c){this.destination.error(c);return}this.destination.next(b)}};function Xj(a){return a instanceof bj?a:new bj(Nj(a))};function Yj(a,b){var c=Infinity;if("function"===typeof b)return d=>C(d,Yj((e,f)=>C(Xj(a(e,f)),Cj((g,h)=>b(e,g,f,h))),c));"number"===typeof b&&(c=b);return d=>wj(d,new Zj(a,c))}
var Zj=class{constructor(a,b=Infinity){this.project=a;this.C=b}call(a,b){return b.subscribe(new ak(a,this.project,this.C))}},ak=class extends Qj{constructor(a,b,c=Infinity){super(a);this.destination=a;this.project=b;this.F=c;this.D=!1;this.C=[];this.index=this.active=0}Oa(a){if(this.active<this.F){let b;const c=this.index++;try{b=this.project(a,c)}catch(d){this.destination.error(d);return}this.active++;a=new Oj(this);this.destination.add(a);Sj(b,a)}else this.C.push(a)}pc(){this.D=!0;0===this.active&&
0===this.C.length&&this.destination.complete();this.unsubscribe()}Gg(a){this.destination.next(a)}Fg(){const a=this.C;this.active--;0<a.length?this.Oa(a.shift()):0===this.active&&this.D&&this.destination.complete()}};function bk(a=Infinity){return Yj(Yi,a)};function mk(...a){return bk(1)(oj(...a))};function nk(a){return new bj(b=>{let c;try{c=a()}catch(d){b.error(d);return}return Xj(c).subscribe(b)})};function ok(...a){if(1===a.length){const b=a[0];if(Fj(b))return pk(b,null);if(Hj(b)&&Object.getPrototypeOf(b)===Object.prototype)return a=Object.keys(b),pk(a.map(c=>b[c]),a)}if("function"===typeof a[a.length-1]){const b=a.pop();a=1===a.length&&Fj(a[0])?a[0]:a;return C(pk(a,null),Cj(c=>b(...c)))}return pk(a,null)}
function pk(a,b){return new bj(c=>{const d=a.length;if(0===d)c.complete();else{var e=Array(d),f=0,g=0;for(let h=0;h<d;h++){const m=Xj(a[h]);let p=!1;c.add(m.subscribe({next:u=>{p||(p=!0,g++);e[h]=u},error:u=>c.error(u),complete:()=>{f++;f!==d&&p||(g===d&&c.next(b?b.reduce((u,w,B)=>(u[w]=e[B],u),{}):e),c.complete())}}))}}})};function qk(a,b,c){if(Ri(c)){var d=c;c=void 0}return d?C(qk(a,b,c),Cj(e=>Fj(e)?d(...e):d(e))):new bj(e=>{rk(a,b,function(f){1<arguments.length?e.next(Array.prototype.slice.call(arguments)):e.next(f)},e,c)})}
function rk(a,b,c,d,e){let f;if(a&&"function"===typeof a.addEventListener&&"function"===typeof a.removeEventListener)a.addEventListener(b,c,e),f=()=>a.removeEventListener(b,c,e);else if(a&&"function"===typeof a.on&&"function"===typeof a.off)a.on(b,c),f=()=>a.off(b,c);else if(a&&"function"===typeof a.addListener&&"function"===typeof a.removeListener)a.addListener(b,c),f=()=>a.removeListener(b,c);else if(a&&a.length)for(let g=0,h=a.length;g<h;g++)rk(a[g],b,c,d,e);else throw new TypeError("Invalid event target");
d.add(f)};var sk=class extends Si{Oc(){return this}};var tk=class extends sk{constructor(a,b){super();this.scheduler=a;this.F=b;this.pending=!1}Oc(a,b=0){if(this.closed)return this;this.state=a;a=this.id;const c=this.scheduler;null!=a&&(this.id=this.C(c,a,b));this.pending=!0;this.delay=b;this.id=this.id||this.D(c,this.id,b);return this}D(a,b,c=0){a=a.flush.bind(a,this);return setInterval(a,c)}C(a,b,c=0){if(null!=c&&this.delay===c&&!1===this.pending)return b;clearInterval(b)}qj(a,b){if(this.closed)return Error("executing a cancelled action");this.pending=
!1;if(a=this.G(a,b))return a;!1===this.pending&&null!=this.id&&(this.id=this.C(this.scheduler,this.id,null))}G(a){let b=!1,c=void 0;try{this.F(a)}catch(d){b=!0,c=!!d&&d||Error(d)}if(b)return this.unsubscribe(),c}unsubscribe(){if(!this.closed){const a=this.id,b=this.scheduler,c=b.actions,d=c.indexOf(this);this.state=this.F=null;this.pending=!1;this.scheduler=null;-1!==d&&c.splice(d,1);null!=a&&(this.id=this.C(b,a,null));this.delay=null;super.unsubscribe()}}};var uk=class extends tj{constructor(a,b=sj){super(a,b);this.actions=[];this.active=!1;this.C=void 0}flush(a){const b=this.actions;if(this.active)b.push(a);else{var c;this.active=!0;do if(c=a.qj(a.state,a.delay))break;while(a=b.shift());this.active=!1;if(c){for(;a=b.shift();)a.unsubscribe();throw c;}}}};var vk=new uk(tk);function wk(...a){let b=Infinity,c=void 0,d=a[a.length-1];nj(d)?(c=a.pop(),1<a.length&&"number"===typeof a[a.length-1]&&(b=a.pop())):"number"===typeof d&&(b=a.pop());return!c&&1===a.length&&a[0]instanceof bj?a[0]:bk(b)(c?lj(a,c):new bj(mj(a)))};function xk(a){return function(b){return wj(b,new yk(a))}}class yk{constructor(a){this.pf=a}call(a,b){return b.subscribe(new zk(a,this.pf))}}class zk extends Vi{constructor(a,b){super(a);this.pf=b;this.count=0}Oa(a){let b;try{b=this.pf.call(void 0,a,this.count++)}catch(c){this.destination.error(c);return}b&&this.destination.next(a)}};function Ak(a=0){var b;let c=-1;null!=vk&&(nj(vk)?b=vk:c=vk);nj(b)||(b=vk);return new bj(d=>{const e=Math.max(0,a instanceof Date&&!isNaN(a)?+a-b.now():a);return b.Oc(Bk,e,{counter:0,FD:c,Ks:d})})}function Bk(a){const b=a.FD,c=a.Ks,d=a.counter++;c.next(d);if(!c.closed){if(0>b)return c.complete();this.Oc(a,b)}};Pi(a=>function(b=null){a(this);this.message="Timeout has occurred";this.name="TimeoutError";this.info=b});let Ck=1,Dk;const Ek={};function Fk(a){return a in Ek?(delete Ek[a],!0):!1}function Gk(a){const b=Ck++;Ek[b]=!0;Dk||(Dk=Promise.resolve());Dk.then(()=>Fk(b)&&a());return b};var Hk=new class extends uk{flush(a){this.active=!0;this.C=void 0;const b=this.actions;let c,d=-1;a=a||b.shift();let e=b.length;do if(c=a.qj(a.state,a.delay))break;while(++d<e&&(a=b.shift()));this.active=!1;if(c){for(;++d<e&&(a=b.shift());)a.unsubscribe();throw c;}}}(class extends tk{constructor(a,b){super(a,b);this.scheduler=a;this.F=b}D(a,b,c=0){if(null!==c&&0<c)return super.D(a,b,c);a.actions.push(this);return a.C||(a.C=Gk(a.flush.bind(a,void 0)))}C(a,b,c=0){if(null!=c&&0<c||null==c&&0<this.delay)return super.C(a,
b,c);0===a.actions.length&&(Fk(b),a.C=void 0)}});Pi(a=>function(){a(this);this.message="argument out of range"});Pi(a=>function(b){a(this);this.message=b});Pi(a=>function(b){a(this);this.message=b});function Ik(a){return!!a&&(a instanceof bj||"function"===typeof a.bo&&"function"===typeof a.subscribe)};class Jk extends hj{constructor(a=!1){super();this.G=void 0;this.F=a;if(void 0!==$e||null!=lf){let b;this.G=null!=(b=of(Ci,{optional:!0}))?b:void 0}}emit(a){const b=se(null);try{super.next(a)}finally{se(b)}}subscribe(a,b,c){var d=a;b=b||(()=>null);if(a&&"object"===typeof a){let e;d=null==(e=a.next)?void 0:e.bind(a);let f;b=null==(f=a.error)?void 0:f.bind(a);let g;c=null==(g=a.complete)?void 0:g.bind(a)}this.F&&(b=Kk(b),d&&(d=Kk(d)),c&&(c=Kk(c)));d=super.subscribe({next:d,error:b,complete:c});a instanceof
Si&&a.add(d);return d}}function Kk(a){return b=>{setTimeout(a,void 0,b)}};function Lk(a){let b=!0;setTimeout(()=>{b&&(b=!1,a())});"function"===typeof hf.requestAnimationFrame&&hf.requestAnimationFrame(()=>{b&&(b=!1,a())});return()=>{b=!1}}function Mk(a){let b=!0;queueMicrotask(()=>{b&&a()});return()=>{b=!1}};function Nk(){};function Ok(){return"undefined"!==typeof Zone&&!0===Zone.current.get("isAngularZone")}function Pk(){if(Ok())throw new ae(909,!1);}function Qk(a,b){return a.JA.run(b)}
var Tk=class{constructor({XB:a=!1,Io:b=!1,cm:c=!1}){this.pl=this.Nn=!1;this.isStable=!0;this.ro=new Jk(!1);this.ex=new Jk(!1);this.qo=new Jk(!1);this.onError=new Jk(!1);if("undefined"==typeof Zone)throw new ae(908,!1);Zone.assertZonePatched();this.Oi=0;this.JA=this.Cf=Zone.current;Zone.TaskTrackingZoneSpec&&(this.Cf=this.Cf.fork(new Zone.TaskTrackingZoneSpec));a&&Zone.longStackTraceZoneSpec&&(this.Cf=this.Cf.fork(Zone.longStackTraceZoneSpec));this.Io=!c&&b;this.cm=c;this.Qk=!1;Rk(this)}run(a,b,c){return this.Cf.run(a,
b,c)}runTask(a,b,c,d){const e=this.Cf;a=e.scheduleEventTask("NgZoneEvent: "+d,a,Sk,Nk,Nk);try{return e.runTask(a,b,c)}finally{e.cancelTask(a)}}runGuarded(a,b,c){return this.Cf.runGuarded(a,b,c)}};const Sk={};function Vk(a){if(0==a.Oi&&!a.pl&&!a.isStable)try{a.Oi++,a.ex.emit(null)}finally{if(a.Oi--,!a.pl)try{Qk(a,()=>a.qo.emit(null))}finally{a.isStable=!0}}}function Wk(a){a.Dw||a.Qk||(a.Qk=!0,Zone.root.run(()=>{Lk(()=>{a.Qk=!1;Xk(a);a.Dw=!0;Vk(a);a.Dw=!1})}),Xk(a))}
function Rk(a){a.Cf=a.Cf.fork({name:"angular",properties:{isAngularZone:!0},onInvokeTask:(b,c,d,e,f,g)=>{if(Yk(g,"__ignore_ng_zone__"))return b.invokeTask(d,e,f,g);try{return Zk(a),b.invokeTask(d,e,f,g)}finally{(a.Io&&"eventTask"===e.type||a.cm)&&Wk(a),a.Oi--,Vk(a)}},onInvoke:(b,c,d,e,f,g,h)=>{try{return Zk(a),b.invoke(d,e,f,g,h)}finally{!a.cm||a.Qk||Yk(g,"__scheduler_tick__")||Wk(a),a.Oi--,Vk(a)}},onHasTask:(b,c,d,e)=>{b.hasTask(d,e);c===d&&("microTask"==e.xb?(a.tA=e.cD,Xk(a),Vk(a)):"macroTask"==
e.xb&&(a.Nn=e.YC))},onHandleError:(b,c,d,e)=>{b.handleError(d,e);Qk(a,()=>a.onError.emit(e));return!1}})}function Xk(a){a.pl=a.tA||(a.Io||a.cm)&&!0===a.Qk?!0:!1}function Zk(a){a.Oi++;a.isStable&&(a.isStable=!1,a.ro.emit(null))}function Yk(a,b){if(!Array.isArray(a)||1!==a.length)return!1;let c,d;return!0===(null==(c=a[0])?void 0:null==(d=c.data)?void 0:d[b])};const $k={destroy(){}};function al(a,b){!b&&Dg();let c;const d=null!=(c=null==b?void 0:b.xa)?c:of(ai);if("browser"!==(null!=d?d:of(ai)).get(pi))return $k;Ii("NgAfterRender");let e,f;const g=null!=(f=(e=d.get(bl)).handler)?f:e.handler=new cl;let h;const m=null!=(h=null==b?void 0:b.ai)?h:2;b=()=>{g.unregister(u);p()};const p=d.get(Ci).ob(b),u=Cg(d,()=>new dl(m,a));g.register(u);return{destroy:b}}
function el(a,b){!b&&Dg();let c;const d=null!=(c=null==b?void 0:b.xa)?c:of(ai);if("browser"!==(null!=d?d:of(ai)).get(pi))return $k;Ii("NgAfterNextRender");let e,f;const g=null!=(f=(e=d.get(bl)).handler)?f:e.handler=new cl;let h;const m=null!=(h=null==b?void 0:b.ai)?h:2,p=()=>{g.unregister(w);u()},u=d.get(Ci).ob(p),w=Cg(d,()=>new dl(m,()=>{p();a()}));g.register(w);return{destroy:p}}
class dl{constructor(a,b){this.ai=a;this.C=b;this.zone=of(Tk);this.yn=of(Ai,{optional:!0});let c;null==(c=of(fi,{optional:!0}))||c.notify(6)}invoke(){try{Qk(this.zone,this.C)}catch(a){let b;null==(b=this.yn)||b.handleError(a)}}}
class cl{constructor(){this.F=!1;this.C={[0]:new Set,[1]:new Set,[2]:new Set,[3]:new Set};this.D=new Set}register(a){(this.F?this.D:this.C[a.ai]).add(a)}unregister(a){this.C[a.ai].delete(a);this.D.delete(a)}qj(){this.F=!0;for(const a of Object.values(this.C))for(const b of a)b.invoke();this.F=!1;for(const a of this.D)this.C[a.ai].add(a);this.D.clear()}destroy(){for(const a of Object.values(this.C))a.clear();this.D.clear()}}function fl(a){const b=[...a.C];a.C.length=0;for(const c of b)c()}
var bl=class{constructor(){this.handler=null;this.C=[]}qj(){fl(this);let a;null==(a=this.handler)||a.qj()}va(){let a;null==(a=this.handler)||a.destroy();this.handler=null;this.C.length=0}};bl.na=Pd({ma:bl,fa:"root",ba:()=>new bl});function gl(a,b){const c=Yg?64:1088;let d;for(null==(d=a[10].cj)||d.notify(b);a;){a[2]|=c;b=Tg(a);if(0!==(a[2]&512)&&!b)break;a=b}};new gf("");var hl=new gf("",{fa:"root",ba:()=>!1});new gf("");new gf("");function D(a=1){il(Wg.ub,Zg(),Wg.selectedIndex+a)}function il(a,b,c){3===(b[2]&3)?(a=a.Kg,null!==a&&oh(b,a,3,c)):(a=a.nf,null!==a&&nh(b,a,0,c));Wg.selectedIndex=c};function y(a,b=0){const c=Zg();return null===c?r(a,b):Fh($g(),c,ef(a),b)};function jl(a,b,c,d,e,f){const g=se(null);try{let h=null;0!==(e&1)&&(h=b[d][re]);null!==h&&void 0!==h.Dy&&(f=h.Dy(f));0!==(e&2)&&(f=a.He[d].call(b,f));null!==a.Ho?a.Ho(b,h,f,c,d):(a=h,null!==a?a.Wu(a,f):b[d]=f)}finally{se(g)}};const kl=/^>|^->|\x3c!--|--\x3e|--!>|<!-$/g,ll=/(<|>)/g;function ml(){return"".replace(kl,a=>a.replace(ll,"\u200b$1\u200b"))};function nl(a,b,c,d,e){if(null!=d){var f=!1;if(Fg(d))var g=d;else Eg(d)&&(f=!0,d=d[0]);d=Mg(d);if(0===a&&null!==c)null==e?b.appendChild(c,d):b.insertBefore(c,d,e||null,!0);else if(1===a&&null!==c)b.insertBefore(c,d,e||null,!0);else if(2===a){const h=b.parentNode(d);h&&b.removeChild(h,d,f)}else 3===a&&b.cl(d);if(null!=g)for(d=g[7],d!==Mg(g)&&nl(a,b,c,d,e),e=10;e<g.length;e++)f=g[e],ol(b,a,f[1].firstChild,f,c,d,!1)}}
function pl(a,b){let c;null==(c=b[10].cj)||c.notify(8);ol(b[11],2,a.firstChild,b,null,null,!1)}function ql(a,b){const c=a[9],d=b[3];Eg(d)?a[2]|=2:b[15]!==d[3][15]&&(a[2]|=2);null===c?a[9]=[b]:c.push(b)}function rl(a,b){a=a[9];a.splice(a.indexOf(b),1)}function sl(a,b){if(!(10>=a.length)){var c=10+b,d=a[c];if(d){const e=d[16];null!==e&&e!==a&&rl(e,d);0<b&&(a[c-1][4]=d[4]);a=Yf(a,10+b);pl(d[1],d);d[0]=null;d[5]=null;b=a[18];null!==b&&b.kj(a[1]);d[3]=null;d[4]=null;d[2]&=-129}return d}}
function tl(a,b){if(!(b[2]&256)){var c=b[11];c.cl&&ol(c,3,a.firstChild,b,null,null,!1);if(a=b[12])for(;a;){c=null;if(Eg(a))c=a[12];else{const d=a[10];d&&(c=d)}if(!c){for(;a&&!a[4]&&a!==b;)Eg(a)&&ul(a[1],a),a=a[3];null===a&&(a=b);Eg(a)&&ul(a[1],a);c=a&&a[4]}a=c}else ul(b[1],b)}}
function ul(a,b){if(!(b[2]&256)){var c=se(null);try{b[2]&=-129;b[2]|=256;b[23]&&He(b[23]);var d;if(null!=a&&null!=(d=a.bl))for(let m=0;m<d.length;m+=2){const p=b[d[m]];if(!(p instanceof rh)){const u=d[m+1];if(Array.isArray(u))for(let w=0;w<u.length;w+=2)u[w+1].call(p[u[w]]);else u.call(p)}}var e=a.fe;const f=b[7];if(null!==e)for(d=0;d<e.length-1;d+=2)if("string"===typeof e[d]){const m=e[d+3];if(0<=m)f[m]();else f[-m].unsubscribe();d+=2}else e[d].call(f[e[d+1]]);null!==f&&(b[7]=null);const g=b[21];
if(null!==g)for(b[21]=null,e=0;e<g.length;e++)(0,g[e])();1===b[1].type&&b[11].destroy();const h=b[16];if(null!==h&&Fg(b[3])){h!==b[3]&&rl(h,b);const m=b[18];null!==m&&m.kj(a)}ji.delete(b[19])}finally{se(c)}}}function vl(a,b,c){for(;null!==b&&b.type&40;)b=b.parent;if(null===b)return c[0];const d=b.Kf;return-1<d&&({pb:a}=a.data[b.nd+d],2===a||0===a)?null:Ng(b,c)}
function wl(a,b,c,d){a=vl(a,d.parent,b);const e=b[11];d=d.parent||b[5];b=d.type&40?Ng(d,b):null;if(null!=a)if(Array.isArray(c))for(let h=0;h<c.length;h++){d=e;var f=a,g=c[h];null!==b?d.insertBefore(f,g,b,!1):d.appendChild(f,g)}else null!==b?e.insertBefore(a,c,b,!1):e.appendChild(a,c)}
function xl(a,b){if(null!==b){var c=b.type;if(c&3)return Ng(b,a);if(c&4)return yl(-1,a[b.index]);if(c&8){c=b.If;if(null!==c)return xl(a,c);a=a[b.index];return Fg(a)?yl(-1,a):Mg(a)}if(c&32)return(void 0)(b,a)()||Mg(a[b.index]);c=null!==b?a[15][5].Me[b.Me]:null;if(null!==c){if(Array.isArray(c))return c[0];a=Tg(a[15]);return xl(a,c)}return xl(a,b.next)}return null}function yl(a,b){a+=11;if(a<b.length){a=b[a];const c=a[1].firstChild;if(null!==c)return xl(a,c)}return b[7]}
function ol(a,b,c,d,e,f,g){for(;null!=c;){const m=d[c.index];var h=c.type;g&&0===b&&(m&&li(Mg(m),d),c.flags|=2);if(32!==(c.flags&32))if(h&8)ol(a,b,c.If,d,e,f,!1),nl(b,a,e,m,f);else if(h&32){h=(void 0)(c,d);let p;for(;p=h();)nl(b,a,e,p,f);nl(b,a,e,m,f)}else h&16?zl(a,b,d,c,e,f):nl(b,a,e,m,f);c=g?c.es:c.next}}function zl(a,b,c,d,e,f){var g=c[15];c=g[5].Me[d.Me];if(Array.isArray(c))for(d=0;d<c.length;d++)nl(b,a,e,c[d],f);else g=g[3],128===(d.flags&128)&&(c.flags|=128),ol(a,b,c,g,e,f,!0)}
function Al(a,b,c){const d=c.Je,e=c.dj;c=c.styles;null!==d&&xf(a,b,d);null!==e&&(""===e?a.removeAttribute(b,"class"):a.setAttribute(b,"class",e));null!==c&&a.setAttribute(b,"style",c)};var Bl={};class Cl{constructor(a){this.C=a}toString(){return`SafeValue must use [property]=binding: ${this.C}`+" (see https://g.co/ng/security#xss)"}}class Dl extends Cl{ot(){return"URL"}}class El extends Cl{ot(){return"ResourceURL"}}function Fl(a){return a instanceof Cl?a.C:a}function Gl(a,b){a=a instanceof Cl&&a.ot()||null;if(null!=a&&a!==b){if("ResourceURL"===a&&"URL"===b)return!0;throw Error(`Required a safe ${b}, got a ${a} (see ${"https://g.co/ng/security#xss"})`);}return a===b};let Hl;function Il(){if(void 0===Hl&&(Hl=null,hf.trustedTypes))try{Hl=hf.trustedTypes.createPolicy("angular",{createHTML:a=>a,createScript:a=>a,createScriptURL:a=>a})}catch(a){}return Hl}function Jl(a){let b;return(null==(b=Il())?void 0:b.createHTML(a))||a};class Kl{constructor(a){this.C=a}In(a){a="<body><remove></remove>"+a;try{const b=(new window.DOMParser).parseFromString(Jl(a),"text/html").body;if(null===b)return this.C.In(a);b.removeChild(b.firstChild);return b}catch(b){return null}}}class Ll{constructor(a){this.C=a;this.D=this.C.implementation.createHTMLDocument("sanitization-inert")}In(a){const b=this.D.createElement("template");b.innerHTML=Jl(a);return b}};const Ml=/^(?!javascript:)(?:[a-z0-9+.-]+:|[^&:\/?#]*(?:[\/?#]|$))/i;function Nl(a){a=String(a);return a.match(Ml)?a:"unsafe:"+a};function Ol(a){const b={};for(const c of a.split(","))b[c]=!0;return b}function Pl(...a){const b={};for(const c of a)for(const d in c)c.hasOwnProperty(d)&&(b[d]=!0);return b}
const Ql=Ol("area,br,col,hr,img,wbr"),Rl=Ol("colgroup,dd,dt,li,p,tbody,td,tfoot,th,thead,tr"),Sl=Ol("rp,rt"),Tl=Pl(Sl,Rl),Ul=Pl(Rl,Ol("address,article,aside,blockquote,caption,center,del,details,dialog,dir,div,dl,figure,figcaption,footer,h1,h2,h3,h4,h5,h6,header,hgroup,hr,ins,main,map,menu,nav,ol,pre,section,summary,table,ul")),Vl=Pl(Sl,Ol("a,abbr,acronym,audio,b,bdi,bdo,big,br,cite,code,del,dfn,em,font,i,img,ins,kbd,label,map,mark,picture,q,ruby,rp,rt,s,samp,small,source,span,strike,strong,sub,sup,time,track,tt,u,var,video"));
var Wl=Pl(Ql,Ul,Vl,Tl),Xl=Ol("background,cite,href,itemtype,longdesc,poster,src,xlink:href");
const Yl=Ol("abbr,accesskey,align,alt,autoplay,axis,bgcolor,border,cellpadding,cellspacing,class,clear,color,cols,colspan,compact,controls,coords,datetime,default,dir,download,face,headers,height,hidden,hreflang,hspace,ismap,itemscope,itemprop,kind,label,lang,language,loop,media,muted,nohref,nowrap,open,preload,rel,rev,role,rows,rowspan,rules,scope,scrolling,shape,size,sizes,span,srclang,srcset,start,summary,tabindex,target,title,translate,type,usemap,valign,value,vspace,width"),Zl=Ol("aria-activedescendant,aria-atomic,aria-autocomplete,aria-busy,aria-checked,aria-colcount,aria-colindex,aria-colspan,aria-controls,aria-current,aria-describedby,aria-details,aria-disabled,aria-dropeffect,aria-errormessage,aria-expanded,aria-flowto,aria-grabbed,aria-haspopup,aria-hidden,aria-invalid,aria-keyshortcuts,aria-label,aria-labelledby,aria-level,aria-live,aria-modal,aria-multiline,aria-multiselectable,aria-orientation,aria-owns,aria-placeholder,aria-posinset,aria-pressed,aria-readonly,aria-relevant,aria-required,aria-roledescription,aria-rowcount,aria-rowindex,aria-rowspan,aria-selected,aria-setsize,aria-sort,aria-valuemax,aria-valuemin,aria-valuenow,aria-valuetext");
var $l=Pl(Xl,Yl,Zl);const am=Ol("script,style,template");class bm{constructor(){this.C=[]}chars(a){this.C.push(cm(a))}}function dm(a){a=a.nodeName;return"string"===typeof a?a:"FORM"}function em(a){return Error(`Failed to sanitize html because the element is clobbered: ${a.outerHTML}`)}const fm=/[\uD800-\uDBFF][\uDC00-\uDFFF]/g,gm=/([^#-~ |!])/g;
function cm(a){return a.replace(/&/g,"&amp;").replace(fm,function(b){return"&#"+(1024*(b.charCodeAt(0)-55296)+(b.charCodeAt(1)-56320)+65536)+";"}).replace(gm,function(b){return"&#"+b.charCodeAt(0)+";"}).replace(/</g,"&lt;").replace(/>/g,"&gt;")}let hm;function im(a){return"content"in a&&a.nodeType===Node.ELEMENT_NODE&&"TEMPLATE"===a.nodeName?a.content:null};var jm={NONE:0,pF:1,xF:2,wF:3,URL:4,uF:5,0:"NONE",1:"HTML",2:"STYLE",3:"SCRIPT",4:"URL",5:"RESOURCE_URL"};function ym(a,b,c,d,e,f,g,h,m,p,u){const w=b.sg.slice();w[0]=e;w[2]=d|204;if(null!==p||a&&a[2]&2048)w[2]|=2048;w[17]=0;w[3]=w[14]=a;w[8]=c;w[10]=g||a&&a[10];w[11]=h||a&&a[11];w[9]=m||a&&a[9]||null;w[5]=f;w[19]=ki++;w[6]=u;w[20]=p;w[15]=2==b.type?a[15]:w;return w}
function zm(a,b,c,d,e){var f=a.data[b];if(null===f){f=Wg.Gh;const m=Wg.Sf;var g=a.data,h=m?f:f&&f.parent;let p=h?h.Ad:-1,u=0;null!==Xg&&(u|=128);b=g[b]={type:c,index:b,iG:null,Ad:p,nd:-1,Ce:-1,tn:-1,Kf:-1,zG:null,flags:u,Lg:0,value:d,attrs:e,Je:null,wr:null,ww:void 0,inputs:null,outputs:null,ub:null,next:null,ND:null,es:null,If:null,parent:h,Me:null,styles:null,Is:null,Fo:void 0,dj:null,jn:null,Eo:void 0,Tk:0,hm:0};null===a.firstChild&&(a.firstChild=b);null!==f&&(m?null==f.If&&null!==b.parent&&(f.If=
b):null===f.next&&(f.next=b,b.ND=f));f=b;Wg.tw&&(f.flags|=32)}else f.type&64&&(f.type=c,f.value=d,f.attrs=e,a=Wg,b=a.Gh,a=a.Sf?b:b.parent,f.Ad=null===a?-1:a.Ad);ah(f,!0);return f}function Am(a,b,c,d){if(0===c)return-1;const e=b.length;for(let f=0;f<c;f++)b.push(d),a.sg.push(d),a.data.push(null);return e}function Bm(a,b,c,d,e){const f=Wg.selectedIndex;try{Wg.selectedIndex=-1,d&2&&25<b.length&&il(a,b,25),c(d,e)}finally{Wg.selectedIndex=f}}
function Cm(a,b,c){if(0!==(b.flags&4)){const d=se(null);try{const e=b.nd,f=b.Ce;for(b=e;b<f;b++){const g=a.data[b];g.hc&&g.hc(1,c[b],b)}}finally{se(d)}}}
function Dm(a,b,c){var d=Ng(c,b);const e=c.nd,f=c.Ce;if(-1<c.Kf){var g=a.data[e+c.Kf],h=Ng(c,b),m=Em(g),p=b[10].ns,u=16;g.li?u=4096:g.Kr&&(u=64);g=Fm(b,ym(b,m,null,u,h,c,null,p.Zk(h,g),null,null,null));b[c.index]=g}a.Cc||yh(c,b);li(d,b);d=c.ww;for(g=e;g<f;g++){h=a.data[g];m=Jh(b,a,g,c);li(m,b);if(null!==d&&(p=d[g-e],null!==p))for(u=0;u<p.length;){const w=p[u++],B=p[u++],G=p[u++],L=p[u++];jl(h,m,w,B,G,L)}h.sa&&(Og(c.index,b)[8]=Jh(b,a,g,c))}64===(c.flags&64)&&Gm(a,b,c)}
function Hm(a,b,c=Ng){const d=b.wr;if(null!==d){let f=b.index+1;for(let g=0;g<d.length;g+=2){var e=d[g+1];e=-1===e?c(b,a):a[e];a[f++]=e}}}function Em(a){const b=a.ub;return null===b||b.uw?a.ub=Im(1,null,a.sa,a.ya,a.za,a.Sv,a.qx,a.eb,a.Uj,a.Ba,a.id):b}
function Im(a,b,c,d,e,f,g,h,m,p,u){d=25+d;e=d+e;const w=[];for(let B=0;B<e;B++)w.push(B<d?null:Bl);p="function"===typeof p?p():p;return w[1]={type:a,sg:w,sa:c,queries:null,eb:h,Ov:b,data:w.slice().fill(null,d),jq:d,dw:e,cr:null,Cc:!0,Cn:!0,sy:!1,qy:!1,nf:null,Kg:null,Xk:null,sq:null,pm:null,Ws:null,bl:null,fe:null,hc:null,Lf:null,Tv:"function"===typeof f?f():f,ID:"function"===typeof g?g():g,firstChild:null,Uj:m,Ba:p,uw:!1,py:u}}
function Jm(a,b,c,d,e){for(let g in b){if(!b.hasOwnProperty(g))continue;var f=b[g];if(void 0===f)continue;null!=d||(d={});let h,m=0;Array.isArray(f)?(h=f[0],m=f[1]):h=f;f=g;if(null!==e){if(!e.hasOwnProperty(g))continue;f=e[g]}0===a?Km(d,c,f,h,m):Km(d,c,f,h)}return d}function Km(a,b,c,d,e){let f;a.hasOwnProperty(c)?(f=a[c]).push(b,d):f=a[c]=[b,d];void 0!==e&&f.push(e)}
function Lm(a,b,c,d,e,f,g,h){const m=Ng(b,c);let p=b.inputs,u;!h&&null!=p&&(u=p[d])?(Mm(a,c,u,d,e),-1<b.Kf&&(a=Og(b.index,c),a[2]&16||(a[2]|=64))):b.type&3&&(d="class"===d?"className":"for"===d?"htmlFor":"formaction"===d?"formAction":"innerHtml"===d?"innerHTML":"readonly"===d?"readOnly":"tabindex"===d?"tabIndex":d,e=null!=g?g(e,b.value||"",d):e,f.setProperty(m,d,e))}
function Nm(a,b,c,d){const e=null===d?null:{"":-1};var f=a.Tv;let g=null,h=null;if(f)for(let u=0;u<f.length;u++){const w=f[u];if(Cf(c,w.ha,!1))if(g||(g=[]),w.sa)if(null!==w.sj){const B=[];h=h||new Map;w.sj(w,B,h);g.unshift(...B,w);Om(a,c,B.length)}else g.unshift(w),Om(a,c,0);else{h=h||new Map;let B,G;null==(G=(B=w).sj)||G.call(B,w,g,h);g.push(w)}}f=null===g?null:[g,h];var m;let p;null===f?m=p=null:[m,p]=f;null!==m&&Pm(a,b,c,m,e,p);if(e&&d)for(a=c.wr=[],b=0;b<d.length;b+=2){m=e[d[b+1]];if(null==m)throw new ae(-301,
!1);a.push(d[b],m)}c.Je=yf(c.Je,c.attrs)}
function Pm(a,b,c,d,e,f){for(var g=0;g<d.length;g++){var h=yh(c,b);xh(h,a,d[g].type)}h=a.data.length;g=d.length;c.flags|=1;c.nd=h;c.Ce=h+g;c.Lg=h;for(h=0;h<d.length;h++)g=d[h],g.fs&&g.fs(g);g=h=!1;var m=Am(a,b,d.length,null);for(var p=0;p<d.length;p++){var u=d[p];c.Je=yf(c.Je,u.Ka);var w=a,B=c,G=b,L=m;w.data[L]=u;var O=u.ba||(u.ba=Wf(u.type));O=new rh(O,!!u.sa);w.sg[L]=O;G[L]=O;O=w;var ba=B;w=Am(w,G,u.Za,Bl);if(G=u.Ia){B=O.cr;null===B&&(B=O.cr=[]);O=~ba.index;a:{for(ba=B.length;0<ba;){const S=B[--ba];
if("number"===typeof S&&0>S){ba=S;break a}}ba=0}ba!=O&&B.push(O);B.push(L,w,G)}L=m;O=u;if(w=e){if(O.ib)for(G=0;G<O.ib.length;G++)w[O.ib[G]]=L;O.sa&&(w[""]=L)}null!==u.hc&&(c.flags|=4);if(null!==u.Ia||null!==u.Ka||0!==u.Za)c.flags|=64;u=u.type.prototype;if(!h&&(u.uc||u.Fb||u.Cj)){var ca=void 0,na=void 0;(null!=(na=(ca=a).nf)?na:ca.nf=[]).push(c.index);h=!0}if(!g&&(u.uc||u.Cj)){var K=void 0,W=void 0;(null!=(W=(K=a).Kg)?W:K.Kg=[]).push(c.index);g=!0}m++}K=c.nd;b=c.Ce;a=a.data;d=c.attrs;e=[];for(na=ca=
null;K<b;K++){W=a[K];g=(h=f?f.get(W):null)?h.outputs:null;ca=Jm(0,W.inputs,K,ca,h?h.inputs:null);na=Jm(1,W.outputs,K,na,g);if(null===ca||null===d||Af(c))W=null;else{W=ca;h=K;g=null;for(m=0;m<d.length;)if(p=d[m],0===p)m+=4;else{if(5!==p){if("number"===typeof p)break;if(W.hasOwnProperty(p))for(null===g&&(g=[]),u=W[p],L=0;L<u.length;L+=3)if(u[L]===h){g.push(p,u[L+1],u[L+2],d[m+1]);break}}m+=2}W=g}e.push(W)}null!==ca&&(ca.hasOwnProperty("class")&&(c.flags|=8),ca.hasOwnProperty("style")&&(c.flags|=16));
c.ww=e;c.inputs=ca;c.outputs=na}function Gm(a,b,c){var d=c.nd;const e=c.Ce,f=c.index;c=Wg.ij;try{for(Wg.selectedIndex=f;d<e;d++){const g=a.data[d],h=b[d];Wg.ij=d;null===g.Ia&&0===g.Za&&null===g.Ka||null===g.Ia||g.Ia(1,h)}}finally{Wg.selectedIndex=-1,Wg.ij=c}}function Om(a,b,c){b.Kf=c;let d;(null!=(d=a.Lf)?d:a.Lf=[]).push(b.index)}
function Qm(a,b){const c=a.hc;if(null!==c){const d=se(null);try{for(let e=0;e<c.length;e+=2){const f=c[e],g=c[e+1];if(-1!==g){const h=a.data[g];Wg.ge=f;h.hc(2,b[g],g)}}}finally{se(d)}}}function Fm(a,b){a[12]?a[13][4]=b:a[12]=b;return a[13]=b}function Rm(a,b,c){Wg.ge=0;const d=se(null);try{b(a,c)}finally{se(d)}}function Sm(a,b,c){if(null===a||a.sa)a:{for(a=c[b.index];Array.isArray(a);){if("object"===typeof a[1]){c=a;break a}a=a[0]}c=null}return c[11]}
function Tm(a,b){(a=(a=a[9])?a.get(Ai,null):null)&&a.handleError(b)}function Mm(a,b,c,d,e){for(let f=0;f<c.length;){const g=c[f++],h=c[f++],m=c[f++];jl(a.data[g],b[g],d,h,m,e)}}function Um(a,b){a[11].setValue(Mg(a[Wg.selectedIndex]),b)};function Vm(a,b,c){gh(b);try{var d=a.eb;null!==d&&Rm(1,d,c);var e=a.sa;null!==e&&Bm(a,b,e,1,c);a.Cc&&(a.Cc=!1);let f;null!=(f=b[18])&&Wm(f,a);a.qy&&Qm(a,b);a.sy&&Rm(2,a.eb,c);const g=a.Lf;if(null!==g)for(e=0;e<g.length;e++){const h=Og(g[e],b),m=h[1];c=m;d=h;for(let p=d.length;p<c.sg.length;p++)d.push(c.sg[p]);null!==h[0]&&null===h[6]&&(h[6]=null);Vm(m,h,h[8])}}catch(f){throw a.Cc&&(a.uw=!0,a.Cc=!1),f;}finally{b[2]&=-5,ih()}};function Xm(a,b,c){let d=c?a.styles:null,e=c?a.dj:null,f=0;if(null!==b)for(let h=0;h<b.length;h++){var g=b[h];if("number"===typeof g)f=g;else if(1==f)e=le(e,g);else if(2==f){const m=b[++h];d=le(d,g+": "+m+";")}}c?a.styles=d:a.Is=d;c?a.dj=e:a.jn=e};function Ym(a,b,c,d,e=!1){for(;null!==c;){var f=b[c.index];null!==f&&d.push(Mg(f));if(Fg(f)){var g=d;for(let h=10;h<f.length;h++){const m=f[h],p=m[1].firstChild;null!==p&&Ym(m[1],m,p,g)}f[7]!==f[0]&&g.push(f[7])}f=c.type;if(f&8)Ym(a,b,c.If,d);else if(f&32)for(f=(void 0)(c,b);g=f();)d.push(g);else f&16&&(f=null!==c?b[15][5].Me[c.Me]:null,Array.isArray(f)?d.push(...f):(g=Tg(b[15]),Ym(g[1],g,f,d,!0)));c=e?c.es:c.next}return d};let Zm=[];const $m=Object.assign({},te,{qq:!0,rq:a=>{Sg(a.Tf)},yv(){this.Tf[23]=this}});function an(a,b=!0,c=0){const d=a[10],e=d.ns;var f;null==(f=e.an)||f.call(e);try{f=Yg;try{for(Yg=!0,bn(a,c),c=0;Qg(a);){if(100===c)throw new ae(103,!1);c++;bn(a,1)}}finally{Yg=f}}catch(g){throw b&&Tm(a,g),g;}finally{let g;null==(g=e.end)||g.call(e);let h;null==(h=d.xw)||h.flush()}}function cn(a,b){for(let c=mi(a[12]);null!==c;c=mi(c[4]))for(let d=10;d<c.length;d++)a=c[d],128===(a[2]&128)&&bn(a,b)}
function bn(a,b){var c=a[1],d=a[2],e=a[23],f=!!(0===b&&d&16);f||(f=!!(d&64&&0===b));f||(f=!!(d&1024));f||(f=!!((null==e?0:e.sc)&&Be(e)));f||(f=!1);e&&(e.sc=!1);a[2]&=-9217;if(f){if(e=c.sa,b=a[8],f=a[2],256!==(f&256)){var g;null!=(g=a[10].xw)&&g.flush();gh(a);d=g=null;if(2!==c.type){if(null!=(g=a[23]))var h=g;else g=null!=(h=Zm.pop())?h:Object.create($m),g.Tf=a,h=g;d=h;g=Fe(d)}try{a[17]=0;Wg.cd=c.jq;null!==e&&Bm(c,a,e,2,b);if(h=3===(f&3)){var m=c.Kg;null!==m&&oh(a,m,3,null)}else{const O=c.nf;null!==
O&&nh(a,O,0,null);ph(a,0)}for(var p=mi(a[12]);null!==p;p=mi(p[4]))if(p[2]&2){var u=p[9];for(m=0;m<u.length;m++){var w=u[m];w[2]&1024||(w[2]|=1024,128===(w[2]&128)&&Sg(w))}}cn(a,0);null!==c.hc&&Qm(c,a);if(h){const O=c.sq;null!==O&&oh(a,O,3)}else{const O=c.Xk;null!==O&&nh(a,O,1);ph(a,1)}const B=c.cr;if(null!==B)try{for(w=0;w<B.length;w++){const O=B[w];if(0>O)Wg.selectedIndex=~O;else{p=O;const ba=B[++w],ca=B[++w];u=Wg;u.cd=u.bn=ba;Wg.ij=p;ca(2,a[p])}}}finally{Wg.selectedIndex=-1}const G=c.Lf;null!==
G&&dn(a,G,0);const L=c.eb;null!==L&&Rm(2,L,b);if(h){const O=c.Ws;null!==O&&oh(a,O,3)}else{const O=c.pm;null!==O&&nh(a,O,2);ph(a,2)}!0===c.Cn&&(c.Cn=!1);if(a[22]){for(const O of a[22])O();a[22]=null}a[2]&=-73}catch(B){throw Sg(a),B;}finally{null!==d&&(Ge(d,g),a=d,a.Tf[23]!==a&&(a.Tf=null,Zm.push(a))),ih()}}}else d&8192&&(cn(a,1),c=c.Lf,null!==c&&dn(a,c,1))}function dn(a,b,c){for(let e=0;e<b.length;e++){var d=c;const f=Og(b[e],a);128===(f[2]&128)&&bn(f,d)}};function en(a){gl(a.N||a.D,4)}function fn(a){a.D[2]|=1024;an(a.D,a.J)}
var gn=class{get rootNodes(){const a=this.D,b=a[1];return Ym(b,a,b.firstChild,[])}constructor(a,b,c=!0){this.D=a;this.N=b;this.J=c;this.F=null;this.G=!1}get context(){return this.D[8]}set context(a){this.D[8]=a}get C(){return 256===(this.D[2]&256)}destroy(){if(this.F)this.F.kj(this);else if(this.G){const a=this.D[3];if(Fg(a)){const b=a[8],c=b?b.indexOf(this):-1;-1<c&&(sl(a,c),Yf(b,c))}this.G=!1}tl(this.D[1],this.D)}ob(a){Ug(this.D,a)}detach(){this.D[2]&=-129}};var hn=class{};hn.na=Pd({ma:hn,fa:"root",ba:()=>null});var jn={};function kn(a,b){return new ln(b[be]||null,a.Dj)}var mn=class extends ci{constructor(a){super();this.Dj=a}};function nn(a){const b=[];for(const c in a){if(!a.hasOwnProperty(c))continue;const d=a[c];void 0!==d&&b.push({Ax:Array.isArray(d)?d[0]:d,LG:c})}return b}
var on=class{constructor(a,b){this.xa=a;this.Gj=b}get(a,b,c){c=pf(c);const d=this.xa.get(a,jn,c);return d!==jn||b===jn?d:this.Gj.get(a,b,c)}},ln=class extends ui{get inputs(){var a=this.C;const b=a.He;a=nn(a.inputs);if(null!==b)for(const c of a)b.hasOwnProperty(c.Ax)&&(c.transform=b[c.Ax]);return a}get outputs(){return nn(this.C.outputs)}constructor(a,b){super();this.C=a;this.Dj=b;this.nq=a.type;this.bm=a.ha.map(Ef).join(",");this.bc=a.bc?a.bc:[];this.IC=!!b}create(a,b,c,d){const e=se(null);try{d=
d||this.Dj;let ja,F=d instanceof og?d:null==(ja=d)?void 0:ja.xa;F&&null!==this.C.Rq&&(F=this.C.Rq(F)||F);var f=F?new on(a,F):a,g=f.get(yi,null);if(null===g)throw new ae(407,!1);const I=f.get(hn,null),aa=f.get(bl,null),Aa=f.get(fi,null);a={ns:g,HG:I,xw:null,KF:aa,cj:Aa};var h=g.Zk(null,this.C);const Za=this.C.ha[0][0]||"div";if(c){g=h;var m=this.C.pb,p=f.get(hl,!1)||3===m;var u=g.xs(c,p)}else{const ma=Za.toLowerCase();u=h.createElement(Za,"svg"===ma?"svg":"math"===ma?"math":null)}m=512;this.C.li?m|=
4096:this.C.Kr||(m|=16);p=null;null!==u&&(p=null);const vb=Im(0,null,null,1,0,null,null,null,null,null,null),kb=ym(null,vb,null,m,null,null,a,h,f,null,p);gh(kb);let jb,bb;try{var w=this.C;f=null;if(w.sj){var B=[];f=new Map;w.sj(w,B,f);B.push(w)}else B=[w];var G=kb[1];kb[25]=u;var L=zm(G,25,2,"#host",null);p=u;G=w;m=B;var O=kb[1];for(var ba of m)L.Je=yf(L.Je,ba.Ka);null!==L.Je&&(Xm(L,L.Je,!0),null!==p&&Al(h,p,L));ba=null;null!==p&&(ba=null);var ca=a.ns.Zk(p,G);p=16;G.li?p=4096:G.Kr&&(p=64);const ma=
ym(kb,Em(G),null,p,kb[L.index],L,a,ca,null,null,ba);O.Cc&&Om(O,L,m.length-1);Fm(kb,ma);var na=kb[L.index]=ma;bb=vb.data[25];if(u)if(c)xf(h,u,["ng-version","0.0.0-PLACEHOLDER"]);else{var K=w.ha[0];c=[];L=[];O=1;for(ca=2;O<K.length;){let tb=K[O];if("string"===typeof tb)2===ca?""!==tb&&c.push(tb,K[++O]):8===ca&&L.push(tb);else{if(0!==(ca&1))break;ca=tb}O++}const {attrs:db,dj:Oa}={attrs:c,dj:L};db&&xf(h,u,db);if(Oa&&0<Oa.length){K=h;h=u;var W=Oa.join(" ");""===W?K.removeAttribute(h,"class"):K.setAttribute(h,
"class",W)}}if(void 0!==b){var S=this.bc;const db=bb.Me=[];for(W=0;W<S.length;W++){const Oa=b[W];db.push(null!=Oa?Array.from(Oa):null)}}b=w;w=f;S=[pn];const T=$g(),da=kb[1],oa=Ng(T,kb);Pm(da,kb,T,B,null,w);for(w=0;w<B.length;w++){const db=Jh(kb,da,T.nd+w,T);li(db,kb)}Gm(da,kb,T);oa&&li(oa,kb);const Ia=Jh(kb,da,T.nd+T.Kf,T);na[8]=kb[8]=Ia;if(null!==S)for(const db of S)db(Ia,b);Cm(da,T,kb);jb=Ia;Vm(vb,kb,null)}finally{ih()}return new qn(this.nq,jb,vi(bb,kb),kb,bb)}finally{se(e)}}},qn=class extends ti{constructor(a,
b,c,d,e){super();this.location=c;this.G=d;this.F=e;this.D=null;this.instance=b;this.C=new gn(d,void 0,!1);this.nq=a}Ho(a,b){var c=this.F.inputs;let d;null!==c&&(d=c[a])&&(null!=this.D||(this.D=new Map),this.D.has(a)&&Object.is(this.D.get(a),b)||(c=this.G,Mm(c[1],c,d,a,b),this.D.set(a,b),a=Og(this.F.index,c),gl(a,1)))}get xa(){return new Kh(this.F,this.G)}destroy(){this.C.destroy()}ob(a){this.C.ob(a)}};function pn(){mh(Zg()[1],$g())};var rn=class extends di{constructor(a,b){super();this.Sa=b;this.np=[];this.F=[];this.C=new mn(this);var c=(a[ee]||null).Pk;this.np=c instanceof Function?c():c;this.D=Zh(a,b,[{pa:di,nb:this},{pa:ci,nb:this.C}],je(a),new Set(["environment"]));tg(this.D);this.instance=this.D.get(a)}get xa(){return this.D}destroy(){const a=this.D;!a.C&&a.destroy();this.F.forEach(b=>b());this.F=null}ob(a){this.F.push(a)}},sn=class extends ei{constructor(a){super();this.C=a}create(a){return new rn(this.C,a)}},tn=class extends di{constructor(a){super();
this.C=new mn(this);this.instance=null;const b=new wg([...a.Wa,{pa:di,nb:this},{pa:ci,nb:this.C}],a.parent||ng(),a.Nv,new Set(["environment"]));this.xa=b;a.Mx&&tg(b)}destroy(){this.xa.destroy()}ob(a){this.xa.ob(a)}};function un(a,b,c=null){return(new tn({Wa:a,parent:b,Nv:c,Mx:!0})).xa};function vn(a){return!!a&&"function"===typeof a.then};var wn=new gf("");function xn(a){if(!a.C){var b=[];for(const d of a.D){const e=d();if(vn(e))b.push(e);else if(e&&"function"===typeof e.subscribe){const f=new Promise((g,h)=>{e.subscribe({complete:g,error:h})});b.push(f)}}var c=()=>{a.done=!0;a.resolve()};Promise.all(b).then(()=>{c()}).catch(d=>{a.reject(d)});0===b.length&&c();a.C=!0}}var yn=class{constructor(){this.done=this.C=!1;this.F=new Promise((b,c)=>{this.resolve=b;this.reject=c});let a;this.D=null!=(a=of(wn,{optional:!0}))?a:[]}};
yn.K=function(a){return new (a||yn)};yn.na=Pd({ma:yn,ba:yn.K,fa:"root"});var zn=class{log(a){console.log(a)}warn(a){console.warn(a)}};zn.K=function(a){return new (a||zn)};zn.na=Pd({ma:zn,ba:zn.K,fa:"platform"});var An=class{constructor(){this.F=0;this.C=new Set;this.ql=new jj(!1)}get D(){return this.ql.value}add(){this.D||this.ql.next(!0);const a=this.F++;this.C.add(a);return a}remove(a){this.C.delete(a);0===this.C.size&&this.D&&this.ql.next(!1)}va(){this.C.clear();this.D&&this.ql.next(!1)}};An.K=function(a){return new (a||An)};An.na=Pd({ma:An,ba:An.K,fa:"root"});function Bn(a){return"function"===typeof a&&void 0!==a[re]};var Cn=new gf(""),Dn=new gf("");function En(a){return a.D?a.D.kG.map(b=>({source:b.source,CB:b.CB,data:b.data})):[]}function Fn(a,b,c,d){let e=-1;c&&0<c&&(e=setTimeout(()=>{a.C=a.C.filter(f=>f.Ps!==e);b()},c));a.C.push({TB:b,Ps:e,Ly:d})}
var In=class{constructor(a,b,c){this.qa=a;this.G=b;this.F=!0;this.C=[];this.D=null;Gn||(Gn=c,Hn(b));this.N();a.run(()=>{this.D="undefined"==typeof Zone?null:Zone.current.get("TaskTrackingZone")})}N(){this.qa.ro.subscribe({next:()=>{this.F=!1}});Qk(this.qa,()=>{this.qa.qo.subscribe({next:()=>{Pk();queueMicrotask(()=>{this.F=!0;this.J()})}})})}isStable(){return this.F&&!this.qa.Nn}J(){if(this.isStable())queueMicrotask(()=>{for(;0!==this.C.length;){let a=this.C.pop();clearTimeout(a.Ps);a.TB()}});else{let a=
En(this);this.C=this.C.filter(b=>b.Ly&&b.Ly(a)?(clearTimeout(b.Ps),!1):!0)}}whenStable(a,b,c){if(c&&!this.D)throw Error('Task tracking zone is required when passing an update callback to whenStable(). Is "zone.js/plugins/task-tracking" loaded?');Fn(this,a,b,c);this.J()}findProviders(){return[]}};In.K=function(a){return new (a||In)(r(Tk),r(Jn),r(Dn))};In.na=Pd({ma:In,ba:In.K});var Jn=class{constructor(){this.C=new Map}kl(a,b=!0){let c,d;return null!=(d=null==(c=Gn)?void 0:c.kl(this,a,b))?d:null}};
Jn.K=function(a){return new (a||Jn)};Jn.na=Pd({ma:Jn,ba:Jn.K,fa:"platform"});let Gn;function Kn(a){return function(b){return wj(b,new Ln(a))}}class Ln{constructor(a){this.C=a}call(a,b){return b.subscribe(new Mn(a,this.C))}}function Nn(a){const b=a.value,c=a.D,d=a.C;d&&(a.remove(d),a.C=null,d.unsubscribe());c&&(a.value=null,a.D=!1,a.destination.next(b))}
class Mn extends Qj{constructor(a,b){super(a);this.F=b;this.value=null;this.D=!1;this.C=null}Oa(a){this.value=a;this.D=!0;if(!this.C){let b;try{const c=this.F;b=c(a)}catch(c){return this.destination.error(c)}a=Sj(b,new Oj(this));!a||a.closed?Nn(this):this.add(this.C=a)}}Gg(){Nn(this)}Fg(){Nn(this)}};function On(a){return Kn(()=>Ak(a))};function Pn(a){return b=>wj(b,function(c){const d=this,e=new Si;let f=null,g=!1,h;f=c.subscribe(new Qn(d,m=>{try{h=Xj(a(m,Pn(a)(c)))}catch(p){d.error(p)}h&&(f?(f.unsubscribe(),f=null,e.add(h.subscribe(d))):g=!0)}));g?(f.unsubscribe(),f=null,e.add(h.subscribe(d))):e.add(f);return e})}class Qn extends Vi{constructor(a,b){super(a);this.onError=b}ve(a){this.onError(a);this.unsubscribe()}};function Rn(a){return b=>wj(b,new Sn(a))}class Sn{constructor(a){this.C=a;this.scheduler=vk}call(a,b){return b.subscribe(new Tn(a,this.C,this.scheduler))}}function Un(a){const b=a.D;null!==b&&(a.remove(b),b.unsubscribe(),a.D=null)}function Vn(a){Un(a);if(a.C){const b=a.F;a.F=null;a.C=!1;a.destination.next(b)}}
class Tn extends Vi{constructor(a,b,c){super(a);this.G=b;this.scheduler=c;this.F=this.D=null;this.C=!1}Oa(a){Un(this);this.F=a;this.C=!0;this.add(this.D=this.scheduler.Oc(Wn,this.G,this))}pc(){Vn(this);this.destination.complete()}}function Wn(a){Vn(a)};function Xn(a=null){return b=>wj(b,new Yn(a))}class Yn{constructor(a){this.defaultValue=a}call(a,b){return b.subscribe(new Zn(a,this.defaultValue))}}class Zn extends Vi{constructor(a,b){super(a);this.defaultValue=b;this.C=!0}Oa(a){this.C=!1;this.destination.next(a)}pc(){this.C&&this.destination.next(this.defaultValue);this.destination.complete()}};function $n(a=vk){return b=>wj(b,new ao(a))}class ao{constructor(a){this.delay=0;this.scheduler=a}call(a,b){return b.subscribe(new bo(a,this.delay,this.scheduler))}}function co(a){var b=a.source;const c=b.C,d=a.scheduler,e=a.destination;for(;0<c.length&&0>=c[0].time-d.now();)e.next(c.shift().value);0<c.length?(b=Math.max(0,c[0].time-d.now()),this.Oc(a,b)):(b.Sb?b.destination.complete():this.unsubscribe(),b.active=!1)}
class bo extends Vi{constructor(a,b,c){super(a);this.destination=a;this.delay=b;this.scheduler=c;this.C=[];this.active=!1}D(a){this.active=!0;const b=this.destination;b.add(a.Oc(co,this.delay,{source:this,destination:b,scheduler:a}))}Oa(a){const b=this.scheduler;a=new eo(b.now()+this.delay,a);this.C.push(a);!1===this.active&&this.D(b)}ve(a){this.C.length=0;this.destination.error(a);this.unsubscribe()}pc(){0===this.C.length&&this.destination.complete();this.unsubscribe()}}
class eo{constructor(a,b){this.time=a;this.value=b}};function fo(){return a=>wj(a,new go)}class go{call(a,b){return b.subscribe(new ho(a))}}class ho extends Vi{constructor(a){super(a);this.C=!1}D(a,b){return a===b}Oa(a){let b;try{b=a}catch(d){return this.destination.error(d)}let c=!1;if(this.C)try{const d=this.D;c=d(this.key,b)}catch(d){return this.destination.error(d)}else this.C=!0;c||(this.key=b,this.destination.next(a))}};function io(){if(isNaN(1))throw new TypeError("'count' is not a number");return a=>wj(a,new jo)}class jo{constructor(){this.count=1}call(a,b){return b.subscribe(new ko(a,this.count))}}class ko extends Vi{constructor(a,b){super(a);this.count=b;this.C=0}Oa(a){const b=this.count,c=++this.C;c<=b&&(this.destination.next(a),c===b&&(this.destination.complete(),this.unsubscribe()))}};function lo(a=mo){return b=>wj(b,new no(a))}class no{constructor(a){this.C=a}call(a,b){return b.subscribe(new oo(a,this.C))}}class oo extends Vi{constructor(a,b){super(a);this.D=b;this.C=!1}Oa(a){this.C=!0;this.destination.next(a)}pc(){if(this.C)return this.destination.complete();let a;try{a=this.D()}catch(b){a=b}this.destination.error(a)}}function mo(){return new uj};function po(a){return b=>wj(b,new qo(a))}class qo{constructor(a){this.callback=a}call(a,b){a=b.subscribe(a);a.add(this.callback);return a}};function ro(a,b){const c=2<=arguments.length;return d=>C(d,a?xk((e,f)=>a(e,f,d)):Yi,io(),c?Xn(b):lo(()=>new uj))};function so(){if(isNaN(1))throw new TypeError("'count' is not a number");return function(a){return wj(a,new to)}}class to{constructor(){this.total=1}call(a,b){return b.subscribe(new uo(a,this.total))}}
class uo extends Vi{constructor(a,b){super(a);this.total=b;this.C=[];this.count=0}Oa(a){const b=this.C,c=this.total,d=this.count++;b.length<c?b.push(a):b[d%c]=a}pc(){const a=this.destination;let b=this.count;if(0<b){const c=this.count>=this.total?this.total:this.count,d=this.C;for(let e=0;e<c;e++){const f=b++%c;a.next(d[f])}}a.complete()}};function vo(a,b){const c=2<=arguments.length;return d=>C(d,a?xk((e,f)=>a(e,f,d)):Yi,so(),c?Xn(b):lo(()=>new uj))};function wo(a){return b=>wj(b,new xo(a))}class xo{constructor(a){this.value=a}call(a,b){return b.subscribe(new yo(a,this.value))}}class yo extends Vi{constructor(a,b){super(a);this.value=b}Oa(){this.destination.next(this.value)}};function zo(a,b){let c=!1;2<=arguments.length&&(c=!0);return function(d){return wj(d,new Ao(a,b,c))}}class Ao{constructor(a,b,c=!1){this.C=a;this.D=b;this.F=c}call(a,b){return b.subscribe(new Bo(a,this.C,this.D,this.F))}}class Bo extends Vi{constructor(a,b,c,d){super(a);this.D=b;this.qc=c;this.C=d;this.index=0}Oa(a){const b=this.destination;if(this.C){const c=this.index++;let d;try{d=this.D(this.qc,a,c)}catch(e){b.error(e);return}this.qc=d;b.next(d)}else this.qc=a,this.C=!0,b.next(a)}};function Co(){var a=Do;return function(b){let c;c="function"===typeof a?a:function(){return a};const d=Object.create(b,Aj);d.source=b;d.vy=c;return d}};function Do(){return new hj}function Eo(){return a=>xj()(Co()(a))};function Fo(){var a={bufferSize:1,Wl:!0};let b;a&&"object"===typeof a?b=a:b={bufferSize:a,iF:void 0,Wl:!1,scheduler:void 0};return c=>wj(c,Go(b))}
function Go({bufferSize:a=Infinity,iF:b=Infinity,Wl:c,scheduler:d}){let e,f=0,g;return function(h){f++;let m;e?m=e.subscribe(this):(e=new rj(a,b,d),m=e.subscribe(this),g=h.subscribe({next(p){e.next(p)},error(p){const u=e;e=g=void 0;u.error(p)},complete(){g=void 0;e.complete()}}),g.closed&&(g=void 0));this.add(()=>{f--;m.unsubscribe();c&&0===f&&g&&(g.unsubscribe(),e=g=void 0)})}};function Ho(){return a=>wj(a,new Io)}class Io{constructor(){this.total=1}call(a,b){return b.subscribe(new Jo(a,this.total))}}class Jo extends Vi{constructor(a,b){super(a);this.total=b;this.count=0}Oa(a){++this.count>this.total&&this.destination.next(a)}};function Ko(...a){const b=a[a.length-1];return nj(b)?(a.pop(),c=>mk(a,c,b)):c=>mk(a,c)};function Lo(a){return b=>wj(b,new Mo(a))}class Mo{constructor(a){this.project=a}call(a,b){return b.subscribe(new No(a,this.project))}}
class No extends Qj{constructor(a,b){super(a);this.destination=a;this.project=b;this.index=0}Oa(a){let b;const c=this.index++;try{b=this.project(a,c)}catch(d){this.destination.error(d);return}(a=this.C)&&a.unsubscribe();a=new Oj(this);this.destination.add(a);this.C=a;Sj(b,a)}pc(){const a=this.C;a&&!a.closed||super.pc();this.C=void 0;this.unsubscribe()}Fg(){this.C=void 0;this.Sb&&super.pc()}Gg(a){this.destination.next(a)}};function Oo(a){return b=>wj(b,function(c){const d=this;Xj(a).subscribe(new vj(d,()=>d.complete(),Mi));!d.closed&&c.subscribe(d)})};function Po(a){return b=>wj(b,new Qo(a))}class Qo{constructor(a){this.pf=a}call(a,b){return b.subscribe(new Ro(a,this.pf))}}class Ro extends Vi{constructor(a,b){super(a);this.pf=b;this.index=0}Oa(a){var b=this.destination;let c;try{c=this.pf(a,this.index++)}catch(d){b.error(d);return}b=this.destination;c?b.next(a):b.complete()}};function So(a){return function(b){return wj(b,new To(a))}}class To{constructor(a){this.C=a;this.complete=this.error=void 0}call(a,b){return b.subscribe(new Uo(a,this.C,this.error,this.complete))}}
class Uo extends Vi{constructor(a,b,c,d){super(a);this.C=this.D=this.F=Mi;this.D=c||Mi;this.C=d||Mi;Ri(b)?(this.Db=this,this.F=b):b&&(this.Db=b,this.F=b.next||Mi,this.D=b.error||Mi,this.C=b.complete||Mi)}Oa(a){try{this.F.call(this.Db,a)}catch(b){this.destination.error(b);return}this.destination.next(a)}ve(a){try{this.D.call(this.Db,a)}catch(b){this.destination.error(b);return}this.destination.error(a)}pc(){try{this.C.call(this.Db)}catch(a){this.destination.error(a);return}return this.destination.complete()}}
;var Vo={ur:!0,Rs:!1};function Wo(a){return b=>wj(b,new Xo(a))}class Xo{constructor(a){var b=!!Vo.ur,c=!!Vo.Rs;this.C=a;this.ur=b;this.Rs=c}call(a,b){return b.subscribe(new Yo(a,this.C,this.ur,this.Rs))}}function Zo(a){const b=a.D;a.C&&(a.destination.next(b),$o(a,b));a.C=!1;a.D=null}function $o(a,b){let c;try{c=a.G(b)}catch(d){a.destination.error(d);return}a.add(a.F=Sj(c,new Oj(a)))}function ap(a){const b=a.F,c=a.N;b&&b.unsubscribe();a.F=null;c&&Zo(a)}
class Yo extends Qj{constructor(a,b,c,d){super(a);this.destination=a;this.G=b;this.J=c;this.N=d;this.D=null;this.C=!1}Oa(a){this.C=!0;this.D=a;this.F||(this.J?Zo(this):$o(this,a))}Gg(){ap(this)}Fg(){ap(this)}};var bp=new gf("");function cp(){Oe=()=>{throw new ae(600,!1);}}function dp(a,b,c){try{const d=c();return vn(d)?d.catch(e=>{Qk(b,()=>a.handleError(e));throw e;}):d}catch(d){throw Qk(b,()=>a.handleError(d)),d;}}
function ep(a,b){let c=null;a.Ra.C||(c=a.Ra.get(yi,null,{optional:!0}));let d=0;const e=a.ea;for(;10>d;){const f=0===d;if(b||!f){a.ja.next(f);for(let {D:g,J:h}of a.D)fp(g,h,f,a.aa)}else{let g,h,m;null==(g=c)||null==(m=(h=g).an)||m.call(h);let p,u,w;null==(p=c)||null==(w=(u=p).end)||w.call(u)}d++;fl(e);if(!a.P.some(({D:g})=>Qg(g))&&(e.qj(),!a.P.some(({D:g})=>Qg(g))))break}}
function gp(a,b){a.D.push(b);if(b.G)throw new ae(902,!1);b.F=a;a=0!==(b.D[2]&512);const c=b.D[16];null===c||a||ql(c,b.D);Rg(b.D)}
var ip=class{constructor(){this.N=[];this.Ga=this.G=!1;this.F=[];this.D=[];this.ta=of(Bi);this.ea=of(bl);this.aa=of(gi);this.oa=new Set;this.ja=new hj;this.O=new hj;this.T=[];this.Lf=[];this.isStable=C(of(An).ql,Cj(a=>!a));this.Ra=of(og)}get P(){return[...this.oa.keys(),...this.D]}get C(){return this.Ga}get xa(){return this.Ra}Pk(a,b){const c=a instanceof ui;if(!this.Ra.get(yn).done)throw new ae(405,!1);let d;c?d=a:d=kn(this.Ra.get(ci),a);this.T.push(d.nq);a=d.IC?void 0:this.Ra.get(di);const e=d.create(bi,
[],b||d.bm,a),f=e.location.R,g=e.xa.get(Cn,null);null!=g&&g.G.C.set(f,g);e.ob(()=>{this.kj(e.C);hp(this.Lf,e);null!=g&&g.G.C.delete(f)});this.ca(e);return e}J(a){if(this.G)throw new ae(101,!1);const b=se(null);try{this.G=!0,ep(this,a)}catch(c){this.ta(c)}finally{this.G=!1,se(b),this.O.next()}}kj(a){hp(this.D,a);a.F=null;const b=0!==(a.D[2]&512),c=a.D[16];null===c||b||rl(c,a.D);pl(a.D[1],a.D)}ca(a){gp(this,a.C);this.J(!0);this.Lf.push(a);const b=this.Ra.get(bp,[]);[...this.N,...b].forEach(c=>c(a))}va(){if(!this.Ga)try{this.F.forEach(a=>
a()),this.D.slice().forEach(a=>a.destroy())}finally{this.Ga=!0,this.D=[],this.N=[],this.F=[]}}ob(a){this.F.push(a);return()=>hp(this.F,a)}destroy(){if(this.Ga)throw new ae(406,!1);const a=this.Ra;a.destroy&&!a.C&&a.destroy()}get ua(){return this.D.length}};ip.K=function(a){return new (a||ip)};ip.na=Pd({ma:ip,ba:ip.K,fa:"root"});function hp(a,b){b=a.indexOf(b);-1<b&&a.splice(b,1)}function fp(a,b,c,d){(c||Qg(a))&&an(a,b,c&&!d?0:1)};function jp(a,b,c,d){const e=se(null);try{const f=b.ub;let g,h,m;const p=ym(a,f,c,a[2]&4096?4096:16,null,b,null,null,null!=(g=null==d?void 0:d.xa)?g:null,null!=(h=null==d?void 0:d.WB)?h:null,null!=(m=null==d?void 0:d.yq)?m:null);p[16]=a[b.index];const u=a[18];null!==u&&(p[18]=u.xd(f));Vm(f,p,c);return p}finally{se(e)}}function kp(a,b){b=10+b;if(b<a.length)return a[b]}function lp(a,b){return!b||null===b.firstChild||128===(a.flags&128)}
function mp(a,b,c,d=!0){const e=b[1];var f=10+c;const g=a.length;0<c&&(a[f-1][4]=b);c<g-10?(b[4]=a[f],f=10+c,f>=a.length?a.push(b):a.splice(f,0,b)):(a.push(b),b[4]=null);b[3]=a;f=b[16];null!==f&&a!==f&&ql(f,b);f=b[18];null!==f&&Wm(f,e);Rg(b);b[2]|=128;d&&(c=yl(c,a),f=b[11],d=f.parentNode(a[7]),null!==d&&(a=a[5],b[0]=d,b[5]=a,ol(f,1,e.firstChild,b,d,c,!1)));b=b[6];null!==b&&null!==b.firstChild&&(b.firstChild=null)};var np=class{};np.xf=op;function op(){return pp($g(),Zg())}function qp(a,b,c,d){const e=b.D;if(Fg(e[3])){var f=a.indexOf(b);-1!==f?a.detach(f):(f=e[3],f=new rp(f,f[5],f[3]),f.detach(f.indexOf(b)))}c=a.ip(c);a=a.lg;mp(a,e,c,d);if(b.F)throw new ae(902,!1);b.G=!0;d=sp(a);c>=d.length?d.push(b):d.splice(c,0,b);return b}
function tp(a,b,c,d,e){var f=b&&"function"!==typeof b;if(!f){var g=c||{};c=g.index;d=g.xa;e=g.Bo;g=g.Pf||g.oG}b=f?b:new ln(b[be]||null);d=d||a.Gj;g||null!=b.Dj||(f=(f?d:a.Gj).get(og,null))&&(g=f);e=b.create(d,e,null,g);qp(a,e.C,c,lp(a.yk,null));return e}
const rp=class extends np{constructor(a,b,c){super();this.lg=a;this.yk=b;this.xk=c}get element(){return vi(this.yk,this.xk)}get xa(){return new Kh(this.yk,this.xk)}get Gj(){const a=Bh(this.yk,this.xk);if(-1!==a){const b=sh(a,this.xk);return new Kh(b[1].data[(a&32767)+8],b)}return new Kh(null,this.xk)}clear(){for(;0<this.length;)this.remove(this.length-1)}get(a){const b=this.lg[8];return null!==b&&b[a]||null}get length(){return this.lg.length-10}xd(a,b,c){let d,e;"number"===typeof c?d=c:null!=c&&(d=
c.index,e=c.xa);a=up(a,b||{},e,null);qp(this,a,d,lp(this.yk,null));return a}kr(a,b){return qp(this,a,b,!0)}move(a,b){return this.kr(a,b)}indexOf(a){const b=this.lg[8];return null!==b?b.indexOf(a):-1}remove(a){a=this.ip(a,-1);const b=sl(this.lg,a);b&&(Yf(sp(this.lg),a),tl(b[1],b))}detach(a){a=this.ip(a,-1);const b=sl(this.lg,a);return b&&null!=Yf(sp(this.lg),a)?new gn(b):null}ip(a,b=0){return null==a?this.length+b:a}};function sp(a){return a[8]||(a[8]=[])}
function pp(a,b){let c;const d=b[a.index];Fg(d)?c=d:(c=[d,!0,0,b,null,a,null,null,null,null],b[a.index]=c,Fm(b,c));vp(c,b,a,d);return new rp(c,a,b)}let vp=wp;function wp(a,b,c,d){if(!a[7]){if(c.type&8)d=Mg(d);else{var e=b[11];d=e.createComment("");const g=Ng(c,b);c=e.parentNode(g);b=e;var f=d;e=e.nextSibling(g);b.insertBefore(c,f,e,!1)}a[7]=d}};function xp(a){return null===a||"function"!==typeof a&&"object"!==typeof a?!1:Array.isArray(a)||!(a instanceof Map)&&Symbol.iterator in a}function yp(a,b){if(Array.isArray(a))for(var c=0;c<a.length;c++)b(a[c]);else for(a=a[Symbol.iterator]();!(c=a.next()).done;)b(c.value)};function zp(a,b,c){if(Object.is(a[b],c))return!1;a[b]=c;return!0}function Ap(a,b,c,d){c=zp(a,b,c);return zp(a,b+1,d)||c};function Bp(a,b,c,d,e,f,g,h,m,p){const u=c+25;if(b.Cc){const w=b.Ba;g=zm(b,u,4,g||null,h||null);Nm(b,a,g,Pg(w,m));mh(b,g);d=g.ub=Im(2,g,d,e,f,b.Tv,b.ID,null,b.Uj,w,null);null!==b.queries&&(b.queries.sa(b,g),d.queries=b.queries.Jq(g));d=g}else d=b.data[u];ah(d,!1);c=Cp(b,a,d,c);lh&&wl(b,a,c,d);li(c,a);c=[c,!0,0,a,null,d,null,c,null,null];a[u]=c;Fm(a,c);1===(d.flags&1)&&Dm(b,a,d);null!=m&&Hm(a,d,p)}function H(a,b,c,d,e,f,g,h){const m=Zg(),p=Wg.ub;f=Pg(p.Ba,f);Bp(m,p,a,b,c,d,e,f,g,h);return H}
let Cp=Dp;function Dp(a,b){lh=!0;return b[11].createComment("")};new gf("DEFER_BLOCK_DEPENDENCY_INTERCEPTOR");new gf("");function Ep(a){var b=Object.getPrototypeOf(a.type.prototype).constructor,c=!0;const d=[a];for(;b;){var e=void 0;if(a.sa)e=b.Ca||b.Fa;else{if(b.Ca)throw new ae(903,!1);e=b.Fa}if(e){if(c){d.push(e);var f=a;f.inputs=Fp(a.inputs);f.He=Fp(a.He);f.jj=Fp(a.jj);f.outputs=Fp(a.outputs);(f=e.Ia)&&Gp(a,f);f=e.eb;var g=e.hc;f&&Hp(a,f);g&&Ip(a,g);f=a;g=e;for(var h in g.inputs)if(g.inputs.hasOwnProperty(h)&&!f.inputs.hasOwnProperty(h)){var m=g.inputs[h];if(void 0!==m&&(f.inputs[h]=m,f.jj[h]=g.jj[h],null!==g.He)){m=
Array.isArray(m)?m[0]:m;if(!g.He.hasOwnProperty(m))continue;let p;null!=(p=f).He||(p.He={});f.He[m]=g.He[m]}}f=a.outputs;g=e.outputs;for(const p in g)g.hasOwnProperty(p)&&!f.hasOwnProperty(p)&&(f[p]=g[p]);e.sa&&e.data.animation&&(f=a.data,f.animation=(f.animation||[]).concat(e.data.animation))}if(e=e.features)for(f=0;f<e.length;f++)(g=e[f])&&g.ax&&g(a),g===Ep&&(c=!1)}b=Object.getPrototypeOf(b)}a=0;b=null;for(c=d.length-1;0<=c;c--)h=d[c],h.Za=a+=h.Za,h.Ka=yf(h.Ka,b=yf(b,h.Ka))}
function Fp(a){return a===Hf?{}:a===If?[]:a}function Hp(a,b){const c=a.eb;a.eb=c?(d,e)=>{b(d,e);c(d,e)}:b}function Ip(a,b){const c=a.hc;a.hc=c?(d,e,f)=>{b(d,e,f);c(d,e,f)}:b}function Gp(a,b){const c=a.Ia;a.Ia=c?(d,e)=>{b(d,e);c(d,e)}:b};function Jp(a,b,c){if(null!==a.tl)for(const e of a.tl){const f=e.zq[ce]||null;a=f.jj;var d=e.inputs;for(const g in d)d.hasOwnProperty(g)&&(a[d[g]]=a[g]);Jp(f,b,c);c.set(f,e);b.push(f)}}function Kp(a){if(void 0===a||0===a.length)return Hf;const b={};for(let c=0;c<a.length;c+=2)b[a[c]]=a[c+1];return b};function Lp(a){const b=a.GC,c={};for(const d in b)if(b.hasOwnProperty(d)){const e=b[d];Array.isArray(e)&&e[3]&&(c[d]=e[3])}a.He=c};function Mp(a,b,c,d){var e=Zg(),f=Wg.cd++;if(zp(e,f,b)){f=jh();var g=Ng(f,e);e=e[11];f=f.value;null==b?e.removeAttribute(g,a,d):(b=null==c?ie(b):c(b,f||"",a),e.setAttribute(g,a,b,d))}return Mp};function Q(a,b,c){const d=Zg();var e=Wg.cd++;if(zp(d,e,b)){e=Wg.ub;const f=jh();Lm(e,f,d,a,b,d[11],c,!1)}return Q}function Np(a,b,c,d,e){e=e?"class":"style";Mm(a,c,b.inputs[e],e,d)};function Op(a,b,c,d){const e=a[c+1],f=null===b;let g=d?e>>17&32767:(e&131068)>>2,h=!1;for(;0!==g&&(!1===h||f);){const p=a[g+1];var m=a[g];if(null===m||null==b||(Array.isArray(m)?m[1]:m)===b||Array.isArray(m)&&"string"===typeof b&&0<=$f(m,b))h=!0,a[g+1]=d?p|1:p|2;g=d?p>>17&32767:(p&131068)>>2}h&&(a[c+1]=d?e|2:e|1)};var Pp=0,Qp=0,Rp=0;function Sp(a,b){const c=Pp;if(c===b)return-1;for(b=Qp=b;b<c&&32<a.charCodeAt(b);)b++;b=Rp=b;return Tp(a,b,c)}function Tp(a,b,c){for(;b<c&&32>=a.charCodeAt(b);)b++;return b};function Up(a,b,c){Vp(a,b,c,!1);return Up}function Wp(a,b){Vp(a,b,null,!0);return Wp}function Vp(a,b,c,d){const e=Zg(),f=Wg.ub,g=ch(2);f.Cn&&Xp(f,a,g,d);if(b!==Bl&&zp(e,g,b)){var h=f.data[Wg.selectedIndex],m=e[11];null!=b&&""!==b&&("string"===typeof c?b+=c:"object"===typeof b&&(b=je(Fl(b))));rq(f,h,e,m,a,e[g+1]=b,d,g)}}
function sq(a,b){const c=Wg.ub;var d=ch(2);c.Cn&&Xp(c,null,d,!0);const e=Zg();if(b!==Bl&&zp(e,d,b)){var f=c.data[Wg.selectedIndex];if(0===(f.flags&8)||d>=c.dw){var g=e[11],h=e[d+1];if(null==b||""===b)var m=If;else if(m=[],b=Fl(b),Array.isArray(b))for(var p=0;p<b.length;p++)a(m,b[p],!0);else if("object"===typeof b)for(p in b)b.hasOwnProperty(p)&&a(m,p,b[p]);else if("string"===typeof b)for(Rp=Qp=0,Pp=b.length,a=Sp(b,Tp(b,0,Pp));0<=a;a=Sp(b,a))Zf(m,b.substring(Qp,Rp),!0);m=e[d+1]=m;h===Bl&&(h=If);a=
b=0;p=0<h.length?h[0]:null;let u=0<m.length?m[0]:null;for(;null!==p||null!==u;){const w=b<h.length?h[b+1]:void 0,B=a<m.length?m[a+1]:void 0;let G=null,L=void 0;p===u?(b+=2,a+=2,w!==B&&(G=u,L=B)):null===u||null!==p&&p<u?(b+=2,G=p):(a+=2,G=u,L=B);null!==G&&rq(c,f,e,g,G,L,!0,d);p=b<h.length?h[b]:null;u=a<m.length?m[a]:null}}else d=f.jn,null!==d&&(b=le(d,b?b:"")),Np(c,f,e,b,!0)}}
function Xp(a,b,c,d){const e=a.data;if(null===e[c+1]){var f=e[Wg.selectedIndex];a=c>=a.dw;0!==(f.flags&(d?8:16))&&null===b&&!a&&(b=!1);var g=f,h=dh(e),m=d?g.Eo:g.Fo;if(null===h)0===(d?g.Tk:g.hm)&&(b=tq(null,e,g,b,d),b=uq(b,g.attrs,d),m=null);else{var p=g.tn;if(-1===p||e[p]!==h)if(b=tq(h,e,g,b,d),null===m)h=d?g.Tk:g.hm,h=0!==(h&131068)>>2?e[h>>17&32767]:void 0,void 0!==h&&Array.isArray(h)&&(h=tq(null,e,g,h[1],d),h=uq(h,g.attrs,d),e[(d?g.Tk:g.hm)>>17&32767]=h);else{m=void 0;h=g.Ce;for(p=1+g.tn;p<h;p++)m=
uq(m,e[p].Ka,d);m=uq(m,g.attrs,d)}}void 0!==m&&(d?g.Eo=m:g.Fo=m);m=a;g=d?f.Tk:f.hm;a=g>>17&32767;g=(g&131068)>>2;e[c]=b;h=!1;if(Array.isArray(b)){if(p=b[1],null===p||0<$f(b,p))h=!0}else p=b;m?0!==g?(b=e[a+1]>>17&32767,e[c+1]=b<<17|a<<2,0!==b&&(e[b+1]=e[b+1]&-131069|c<<2),e[a+1]=e[a+1]&131071|c<<17):(e[c+1]=a<<17|0,0!==a&&(e[a+1]=e[a+1]&-131069|c<<2),a=c):(e[c+1]=g<<17|0,0===a?a=c:e[g+1]=e[g+1]&-131069|c<<2,g=c);h&&(e[c+1]|=2);Op(e,p,c,!0);Op(e,p,c,!1);b=p;m=d?f.Eo:f.Fo;null!=m&&"string"==typeof b&&
0<=$f(m,b)&&(e[c+1]|=1);g=a<<17|g<<2;d?f.Tk=g:f.hm=g}}function tq(a,b,c,d,e){let f;const g=c.Ce;let h=c.tn;for(-1===h?h=c.nd:h++;h<g;){f=b[h];d=uq(d,f.Ka,e);if(f===a)break;h++}null!==a&&(c.tn=h);return d}function uq(a,b,c){const d=c?1:2;let e=-1;if(null!==b)for(let f=0;f<b.length;f++){const g=b[f];"number"===typeof g?e=g:e===d&&(Array.isArray(a)||(a=void 0===a?[]:["",a]),Zf(a,g,c?!0:b[++f]))}return void 0===a?null:a}function vq(a,b,c){b=String(b);""===b||b.includes(" ")||Zf(a,b,c)}
function rq(a,b,c,d,e,f,g,h){if(b.type&3){a=a.data;var m=a[h+1];void 0===(1===(m&1)?wq(a,b,c,e,(m&131068)>>2,g):void 0)&&(void 0!==f||2==(m&2)&&(f=wq(a,null,c,e,h,g)),b=Mg(c[Wg.selectedIndex]),g?f?d.Wi(b,e):d.Do(b,e):(g=-1===e.indexOf("-")?void 0:2,null==f?d.ks(b,e,g):("string"===typeof f&&f.endsWith("!important")&&(f=f.slice(0,-10),g|=1),d.setStyle(b,e,f,g))))}}
function wq(a,b,c,d,e,f){const g=null===b;let h=void 0;for(;0<e;){const p=a[e],u=Array.isArray(p);var m=u?p[1]:p;const w=null===m;let B=c[e+1];B===Bl&&(B=w?If:void 0);m=w?ag(B,d):m===d?B:void 0;u&&void 0===m&&(m=ag(p,d));if(void 0!==m&&(h=m,g))return h;e=a[e+1];e=g?e>>17&32767:(e&131068)>>2}null!==b&&(a=f?b.Eo:b.Fo,null!=a&&(h=ag(a,d)));return h};function xq(a){a=zp(Zg(),Wg.cd++,a)?"mat-mdc-select-panel mdc-menu-surface mdc-menu-surface--open "+ie(a):Bl;sq(Zf,a)};function yq(a,b,c){const d=Math.min(b,c);b=Math.max(b,c);c=a.detach(b);if(1<b-d){const e=a.detach(d);a.attach(d,c);a.attach(b,e)}else a.attach(d,c)}var zq=class{destroy(){}C(){}move(a,b){this.attach(b,this.detach(a))}};function Aq(a,b,c,d,e){return a===c&&Object.is(b,d)?1:Object.is(e(a,b),e(c,d))?-1:0}
function Bq(a,b,c){let d=void 0,e=void 0,f=0,g=a.length-1;if(Array.isArray(b)){for(var h=b.length-1;f<=g&&f<=h;){var m=a.at(f),p=b[f],u=Aq(f,m,f,p,c);if(0!==u)0>u&&a.C(f,p),f++;else{var w=a.at(g);u=b[h];var B=Aq(g,w,h,u,c);0!==B?(0>B&&a.C(g,u),g--,h--):(m=c(f,m),w=c(g,w),B=c(f,p),Object.is(B,w)?(w=c(h,u),Object.is(w,m)?(yq(a,f,g),a.C(g,u),h--,g--):a.move(g,f),a.C(f,p),f++):(null!=d||(d=new Cq),null!=e||(e=Dq(a,f,g,c)),Eq(a,d,f,B)?(a.C(f,p),f++,g++):e.has(B)?(d.set(m,a.detach(f)),g--):(p=a.create(f,
b[f]),a.attach(f,p),f++,g++)))}}for(;f<=h;)Fq(a,d,c,f,b[f]),f++}else if(null!=b){b=b[Symbol.iterator]();for(h=b.next();!h.done&&f<=g;)p=a.at(f),u=h.value,m=Aq(f,p,f,u,c),0!==m?(0>m&&a.C(f,u),f++,h=b.next()):(null!=d||(d=new Cq),null!=e||(e=Dq(a,f,g,c)),m=c(f,u),Eq(a,d,f,m)?(a.C(f,u),f++,g++,h=b.next()):e.has(m)?(p=c(f,p),d.set(p,a.detach(f)),g--):(a.attach(f,a.create(f,u)),f++,g++,h=b.next()));for(;!h.done;)Fq(a,d,c,a.length,h.value),h=b.next()}for(;f<=g;)a.destroy(a.detach(g--));let G;null==(G=d)||
G.forEach(L=>{a.destroy(L)})}function Eq(a,b,c,d){return void 0!==b&&b.has(d)?(a.attach(c,b.get(d)),b.delete(d),!0):!1}function Fq(a,b,c,d,e){Eq(a,b,d,c(d,e))?a.C(d,e):(b=a.create(d,e),a.attach(d,b))}function Dq(a,b,c,d){const e=new Set;for(;b<=c;b++)e.add(d(b,a.at(b)));return e}
var Cq=class{constructor(){this.D=new Map;this.C=void 0}has(a){return this.D.has(a)}delete(a){if(!this.has(a))return!1;const b=this.D.get(a);void 0!==this.C&&this.C.has(b)?(this.D.set(a,this.C.get(b)),this.C.delete(b)):this.D.delete(a);return!0}get(a){return this.D.get(a)}set(a,b){if(this.D.has(a)){a=this.D.get(a);void 0===this.C&&(this.C=new Map);const c=this.C;for(;c.has(a);)a=c.get(a);c.set(a,b)}else this.D.set(a,b)}forEach(a){for(let [b,c]of this.D){let d=b,e=c;a(e,d);if(void 0!==this.C){const f=
this.C;for(;f.has(e);)e=f.get(e),a(e,d)}}}};function Gq(a,b){Ii("NgControlFlow");var c=Zg(),d=Wg.cd++,e=c[d]!==Bl?c[d]:-1;e=-1!==e?c[25+e]:void 0;if(zp(c,d,a)){d=se(null);try{if(void 0!==e){const f=sl(e,0);void 0!==f&&tl(f[1],f)}if(-1!==a){a=25+a;const f=c[a],g=c[1].data[a],h=jp(c,g,b,{yq:null});mp(f,h,0,lp(g,null))}}finally{se(d)}}else void 0!==e&&(c=kp(e,0),void 0!==c&&(c[8]=b))}var Hq=class{constructor(a,b,c){this.C=a;this.da=b;this.Cb=c}get hz(){return this.C.length-10}};function Iq(a){return a}function Jq(a,b){return b}
class Kq{constructor(a){this.D=a;this.C=void 0}}function Lq(a,b,c,d,e,f,g){Ii("NgControlFlow");const h=Zg(),m=Wg.ub;Zg()[25+a]=new Kq(g);Bp(h,m,a+1,b,c,d,e,Pg(m.Ba,f))}
class Mq extends zq{constructor(a,b,c){super();this.D=a;this.N=b;this.J=c;this.G=void 0;this.F=!1}get length(){return this.D.length-10}at(a){return kp(this.D,a)[8].da}attach(a,b){const c=b[6];this.F||(this.F=a!==this.length);mp(this.D,b,a,lp(this.J,c))}detach(a){this.F||(this.F=a!==this.length-1);return sl(this.D,a)}create(a,b){a=jp(this.N,this.J,new Hq(this.D,b,a),{yq:null});let c;null==(c=this.G)||c.C();return a}destroy(a){tl(a[1],a);let b;null==(b=this.G)||b.D()}C(a,b){kp(this.D,a)[8].da=b}reset(){this.F=
!1;let a;null==(a=this.G)||a.reset()}}function Nq(a){const b=se(null);var c=Wg.selectedIndex;try{const d=Zg(),e=d[1],f=d[c];c+=1;const g=d[c];void 0===f.C?f.C=new Mq(g,d,e.data[c]):f.C.reset();const h=f.C;Bq(h,a,f.D);if(h.F)for(a=0;a<h.length;a++)kp(h.D,a)[8].Cb=a}finally{se(b)}};function R(a,b,c,d){const e=Zg(),f=Wg.ub;var g=25+a;a=e[11];if(f.Cc){var h=f.Ba;c=Pg(h,c);c=zm(f,g,2,b,c);Nm(f,e,c,Pg(h,d));null!==c.attrs&&Xm(c,c.attrs,!1);null!==c.Je&&Xm(c,c.Je,!0);null!==f.queries&&f.queries.gl(f,c);h=c}else h=f.data[g];lh=!0;b=a.createElement(b,Wg.al);e[g]=b;g=1===(h.flags&1);ah(h,!0);Al(a,b,h);32!==(h.flags&32)&&lh&&wl(f,e,b,h);0===Wg.vn&&li(b,e);Wg.vn++;g&&(Dm(f,e,h),Cm(f,h,e));null!==d&&Hm(e,h);return R}
function U(){let a=$g();Wg.Sf?Wg.Sf=!1:(a=a.parent,ah(a,!1));const b=a;Xg===b&&(Xg=null);Wg.vn--;const c=Wg.ub;c.Cc&&(mh(c,a),0!==(a.flags&4)&&c.queries.wn(a));null!=b.jn&&0!==(b.flags&8)&&Np(c,b,Zg(),b.jn,!0);null!=b.Is&&0!==(b.flags&16)&&Np(c,b,Zg(),b.Is,!1);return U}function Oq(a,b,c,d){R(a,b,c,d);U();return Oq};function Pq(a,b,c){const d=Zg(),e=Wg.ub;a+=25;if(e.Cc){var f=e.Ba;const g=Pg(f,b);b=zm(e,a,8,"ng-container",g);null!==g&&Xm(b,g,!0);f=Pg(f,c);Nm(e,d,b,f);null!==e.queries&&e.queries.gl(e,b);f=b}else f=e.data[a];ah(f,!0);lh=!0;b=d[11].createComment(ml());d[a]=b;lh&&wl(e,d,b,f);li(b,d);1===(f.flags&1)&&(Dm(e,d,f),Cm(e,f,d));null!=c&&Hm(d,f);return Pq}function Qq(){let a=$g();const b=Wg.ub;Wg.Sf?Wg.Sf=!1:(a=a.parent,ah(a,!1));b.Cc&&(mh(b,a),0!==(a.flags&4)&&b.queries.wn(a));return Qq}
function Rq(a,b,c){Pq(a,b,c);Qq();return Rq};function Sq(a,b,c){const d=Zg();var e=Wg.cd++;if(zp(d,e,b)){e=Wg.ub;const f=jh();Lm(e,f,d,a,b,d[11],c,!0)}return Sq}function Tq(a,b,c){const d=Zg();var e=Wg.cd++;if(zp(d,e,b)){e=Wg.ub;const f=jh();Lm(e,f,d,a,b,Sm(dh(e.data),f,d),c,!0)}return Tq};function V(a,b,c,d){c=Zg();Uq(Wg.ub,c,c[11],$g(),a,b,d);return V}function Vq(a,b){const c=$g(),d=Zg(),e=Wg.ub;Uq(e,d,Sm(dh(e.data),c,d),c,a,b);return Vq}function Wq(a,b,c,d){a=a.fe;if(null!=a)for(let e=0;e<a.length-1;e+=2){const f=a[e];if(f===c&&a[e+1]===d)return b=b[7],c=a[e+2],b.length>c?b[c]:null;"string"===typeof f&&(e+=2)}return null}
function Uq(a,b,c,d,e,f,g){var h=1===(d.flags&1);const m=a.Cc&&(a.fe||(a.fe=[])),p=b[7]||(b[7]=[]);var u=!0;if(d.type&3||g){var w=Ng(d,b);const G=g?g(w):w;w=p.length;const L=g?ba=>g(Mg(ba[d.index])):d.index;let O=null;!g&&h&&(O=Wq(a,b,e,d.index));null!==O?((O.Mz||O).vt=f,O.Mz=f,u=!1):(f=Xq(d,b,f),a=c.Id(G,e,f),p.push(f,a),m&&m.push(e,L,w,w+1))}else f=Xq(d,b,f);a=d.outputs;let B;if(u&&null!==a&&(B=a[e])&&(u=B.length))for(a=0;a<u;a+=2)c=b[B[a]][B[a+1]].subscribe(f),h=p.length,p.push(f,c),m&&m.push(e,
d.index,h,-(h+1))}function Yq(a,b,c){const d=se(null);try{return!1!==b(c)}catch(e){return Tm(a,e),!1}finally{se(d)}}function Xq(a,b,c){return function f(e){if(e===Function)return c;var g=-1<a.Kf?Og(a.index,b):b;gl(g,5);g=Yq(b,c,e);let h=f.vt;for(;h;)g=Yq(b,h,e)&&g,h=h.vt;return g}};function Y(a=1){for(var b=Wg,c=Wg.Dh;0<a;)c=c[14],a--;return(b.Dh=c)[8]};function Zq(a){var b=Zg()[15][5];if(!b.Me){var c=a?a.length:1,d=[];for(var e=0;e<c;e++)d.push(null);c=b.Me=d;d=c.slice();for(b=b.If;null!==b;){if(a)a:{e=b;var f=a;let p=null;b:{var g=e.attrs;if(null!=g){var h=g.indexOf(5);if(0===(h&1)){g=g[h+1];break b}}g=null}for(h=0;h<f.length;h++){var m=f[h];if("*"===m)p=h;else{if(null===g)m=Cf(e,m,!0);else b:{c:for(let u=0;u<m.length;u++){const w=m[u];if(g.length===w.length){for(let B=0;B<g.length;B++)if(g[B]!==w[B])continue c;m=!0;break b}}m=!1}if(m){e=h;break a}}}e=
p}else e=0;null!==e&&(d[e]?d[e].es=b:c[e]=b,d[e]=b);b=b.next}}}function $q(a,b=0){const c=Zg();var d=Wg.ub;a=zm(d,25+a,16,null,null);null===a.Me&&(a.Me=b);Wg.Sf=!1;if((!c[6]||null!==Xg)&&32!==(a.flags&32)){b=c[11];d=vl(d,a.parent,c);var e=a.parent||c[5];e=e.type&40?Ng(e,c):null;zl(b,0,c,a,d,e)}};function ar(){return this.D[Symbol.iterator]()}function br(a){void 0===a.F||!a.G&&a.J||a.F.emit(a)}
var cr=class{get C(){let a;return null!=(a=this.F)?a:this.F=new Jk}constructor(a=!1){this.J=a;this.sc=!0;this.N=void 0;this.D=[];this.G=!1;this.F=void 0;this.length=0;this.first=void 0;const b=cr.prototype;b[Symbol.iterator]||(b[Symbol.iterator]=ar)}get(a){return this.D[a]}map(a){return this.D.map(a)}filter(a){return this.D.filter(a)}find(a){return this.D.find(a)}reduce(a,b){return this.D.reduce(a,b)}forEach(a){this.D.forEach(a)}some(a){return this.D.some(a)}toArray(){return this.D.slice()}toString(){return this.D.toString()}reset(a,b){this.sc=
!1;a=a.flat(Number.POSITIVE_INFINITY);a:{var c=this.D;if(c.length!==a.length)b=!1;else{for(let d=0;d<c.length;d++){let e=c[d],f=a[d];b&&(e=b(e),f=b(f));if(f!==e){b=!1;break a}}b=!0}}if(this.G=!b)this.D=a,this.length=a.length,this.first=a[0]}Bs(){this.sc=!0;let a;null==(a=this.N)||a.call(this)}destroy(){void 0!==this.F&&(this.F.complete(),this.F.unsubscribe())}};Symbol.iterator;var dr=class{};dr.xf=er;function up(a,b,c,d){a=jp(a.Yz,a.It,b,{WB:c,yq:d});return new gn(a)}const fr=class extends dr{constructor(a,b,c){super();this.Yz=a;this.It=b;this.Ub=c}get py(){let a;return(null==(a=this.It.ub)?void 0:a.py)||null}xd(a,b){return up(this,a,b)}};function er(){return gr($g(),Zg())}function gr(a,b){return a.type&4?new fr(b,a,vi(a,b)):null};class hr{constructor(a){this.gs=a;this.matches=null}clone(){return new hr(this.gs)}Bs(){this.gs.Bs()}}function Wm(a,b){for(let c=0;c<a.queries.length;c++)null!==b.queries.queries[c].matches&&a.queries[c].Bs()}class ir{constructor(a=[]){this.queries=a}xd(a){const b=a.queries;if(null!==b){a=null!==a.hc?a.hc[0]:b.length;const c=[];for(let d=0;d<a;d++)c.push(this.queries[b.queries[d].vw].clone());return new ir(c)}return null}kj(a){Wm(this,a)}}
var kr=class{constructor(a,b){this.flags=b;this.read=null;"string"===typeof a?this.pf=jr(a):this.pf=a}};
class lr{constructor(a=[]){this.queries=a}gl(a,b){for(let c=0;c<this.queries.length;c++)this.queries[c].gl(a,b)}wn(a){for(let b=0;b<this.queries.length;b++)this.queries[b].wn(a)}Jq(a){let b=null;for(let c=0;c<this.length;c++){const d=this.queries[c].Jq(a,null!==b?b.length:0);d&&(d.vw=c,null!==b?b.push(d):b=[d])}return null!==b?new lr(b):null}sa(a,b){for(let c=0;c<this.queries.length;c++)this.queries[c].sa(a,b)}get length(){return this.queries.length}track(a){this.queries.push(a)}}
function mr(a,b){if(a.C&&1!==(a.metadata.flags&1)){a=a.D;for(b=b.parent;null!==b&&b.type&8&&b.index!==a;)b=b.parent;return a===(null!==b?b.index:-1)}return a.C}function nr(a,b,c){null===a.matches?a.matches=[b,c]:a.matches.push(b,c)}function or(a,b,c,d){if(null!==d){const e=a.metadata.read;null!==e?e===wi||e===np||e===dr&&c.type&4?nr(a,c.index,-2):(b=Ih(c,b,e,!1,!1),null!==b&&nr(a,c.index,b)):nr(a,c.index,d)}}
class pr{constructor(a,b=-1){this.metadata=a;this.matches=null;this.vw=-1;this.Av=!1;this.C=!0;this.D=b}gl(a,b){if(mr(this,b)){const d=this.metadata.pf;if(Array.isArray(d))for(let e=0;e<d.length;e++){const f=d[e];a:{var c=b.wr;if(null!==c)for(let g=0;g<c.length;g+=2)if(c[g]===f){c=c[g+1];break a}c=null}or(this,a,b,c);or(this,a,b,Ih(b,a,f,!1,!1))}else d===dr?b.type&4&&or(this,a,b,-1):or(this,a,b,Ih(b,a,d,!1,!1))}}wn(a){this.D===a.index&&(this.C=!1)}sa(a,b){this.gl(a,b)}Jq(a,b){return mr(this,a)?(this.Av=
!0,nr(this,-a.index,b),new pr(this.metadata)):null}}function qr(a,b,c,d){d=b[18].queries[d];if(null===d.matches){const p=a.data,u=c.matches,w=[];for(let B=0;null!==u&&B<u.length;B+=2){var e=u[B];if(0>e)w.push(null);else{a=w;var f=a.push,g=b,h=p[e],m=u[B+1];e=c.metadata.read;-1===m?(e=h,g=e.type&11?vi(e,g):e.type&4?gr(e,g):null):-2===m?g=e===wi?vi(h,g):e===dr?gr(h,g):e===np?pp(h,g):void 0:g=Jh(g,g[1],m,h);f.call(a,g)}}d.matches=w}return d.matches}
function rr(a,b,c,d){var e=a.queries.queries[c];const f=e.matches;if(null!==f)for(a=qr(a,b,e,c),c=0;c<f.length;c+=2){var g=f[c];if(0<g)d.push(a[c/2]);else{e=f[c+1];g=b[-g];for(var h=10;h<g.length;h++){var m=g[h];m[16]===m[3]&&rr(m[1],m,e,d)}if(null!==g[9])for(g=g[9],h=0;h<g.length;h++)m=g[h],rr(m[1],m,e,d)}}return d}
function sr(a,b){var c=Zg();b=new cr(4===(b&4));var d=b.destroy;const e=c[7]||(c[7]=[]);e.push(b);a.Cc&&(a.fe||(a.fe=[])).push(d,e.length-1);let f;(null!=(f=c[18])?f:c[18]=new ir).queries.push(new hr(b))}function jr(a){return a.split(",").map(b=>b.trim())}function tr(a,b,c){null===a.queries&&(a.queries=new lr);a.queries.track(new pr(b,c))};function ur(a,b,c){const d=Wg.ub;if(d.Cc){const e=$g();tr(d,new kr(b,c),e.index);b=d.hc||(d.hc=[]);a!==(b.length?b[b.length-1]:-1)&&b.push(d.queries.length-1,a);2===(c&2)&&(d.qy=!0)}sr(d,c)}function vr(a,b){const c=Wg.ub;c.Cc&&(tr(c,new kr(a,b),-1),2===(b&2)&&(c.sy=!0));sr(c,b)}
function wr(a){var b=Zg(),c=Wg.ub;const d=Wg.ge;Wg.ge=d+1;c=c.queries.queries[d];if(a.sc&&4===(b[2]&4)===(2===(c.metadata.flags&2))){if(null===c.matches)a.reset([]);else{c=b[1];const e=c.queries.queries[d];b=e.Av?rr(c,b,d,[]):qr(c,b,e,d);a.reset(b,xi);br(a)}return!0}return!1}function xr(){return Zg()[18].queries[Wg.ge].gs};function yr(a){Ii("NgSignals");a=Pe(a);const b=a[re];a.set=c=>Re(b,c);a.update=c=>{Ee()||Oe();Re(b,c(b.value))};a.LF=zr.bind(a);return a}function zr(){const a=this[re];if(void 0===a.Fx){const b=()=>this();b[re]=a;a.Fx=b}return a.Fx};function Ar(a){return Wg.Dh[25+a]};function Z(a,b=""){const c=Zg(),d=Wg.ub;a+=25;const e=d.Cc?zm(d,a,1,b,null):d.data[a];lh=!0;b=c[11].tq(b);c[a]=b;lh&&wl(d,c,b,e);ah(e,!1)};function Br(a){Cr("",a,"");return Br}function Cr(a,b,c){const d=Zg();a=zp(d,Wg.cd++,b)?a+ie(b)+c:Bl;a!==Bl&&Um(d,a);return Cr}function Dr(a,b,c,d,e){const f=Zg(),g=Ap(f,Wg.cd,b,d);ch(2);a=g?a+ie(b)+c+ie(d)+e:Bl;a!==Bl&&Um(f,a);return Dr}function Er(a,b,c,d,e,f,g,h,m){const p=Zg();var u=Wg.cd;const w=Ap(p,u,b,d);u=Ap(p,u+2,f,h)||w;ch(4);a=u?a+ie(b)+c+ie(d)+e+ie(f)+g+ie(h)+m:Bl;a!==Bl&&Um(p,a);return Er};function Fr(a,b,c,d,e){a=ef(a);if(Array.isArray(a))for(var f=0;f<a.length;f++)Fr(a[f],b,c,d,e);else{f=Wg.ub;const w=Zg(),B=$g();let G=ig(a)?a:ef(a.pa);var g=Bg(a),h=B.Lg&1048575,m=B.nd,p=B.Lg>>20;if(ig(a)||!a.Eb)d=new rh(g,e),g=Gr(G,b,e?h:h+p,m),-1===g?(g=yh(B,w),xh(g,f,G),Hr(f,a,b.length),b.push(G),B.nd++,B.Ce++,e&&(B.Lg+=1048576),c.push(d),w.push(d)):(c[g]=d,w[g]=d);else{m=Gr(G,b,h+p,m);h=Gr(G,b,h,h+p);var u=0<=m&&c[m];p=0<=h&&c[h];e&&!p||!e&&!u?(m=yh(B,w),xh(m,f,G),m=c.length,u=new rh(e?Ir:Jr,
e),u.Eb=[],u.index=m,u.C=0,d&&!e&&u.C++,u.Eb.push(g),g=u,!e&&p&&(c[h].D=g),Hr(f,a,b.length,0),b.push(G),B.nd++,B.Ce++,e&&(B.Lg+=1048576),c.push(g),w.push(g)):(b=c[e?h:m],!e&&d&&b.C++,b=b.Eb.push(g)-1,Hr(f,a,-1<m?m:h,b));!e&&d&&p&&c[h].C++}}}function Hr(a,b,c,d){const e=ig(b);var f=!!b.Fc;if(e||f)if(f=(f?ef(b.Fc):b).prototype.va)a=a.bl||(a.bl=[]),!e&&b.Eb?(b=a.indexOf(c),-1===b?a.push(c,[d,f]):a[b+1].push(d,f)):a.push(c,f)}function Gr(a,b,c,d){for(;c<d;c++)if(b[c]===a)return c;return-1}
function Jr(){return Kr(this.Eb,[])}function Ir(a,b,c,d){b=this.Eb;if(this.D){const e=this.D.C;d=Jh(c,c[1],this.D.index,d);c=d.slice(0,e);Kr(b,c);for(b=e;b<d.length;b++)c.push(d[b])}else c=[],Kr(b,c);return c}function Kr(a,b){for(let c=0;c<a.length;c++)b.push((0,a[c])());return b};function Lr(a){var b=[];return c=>{c.fs=(d,e)=>{e=e?e(a):a;const f=Wg.ub;f.Cc&&(d=!!d.sa,Fr(b,f.data,f.sg,d,!0),Fr(e,f.data,f.sg,d,!1))}}};class Mr{constructor(a){this.Ra=a;this.C=new Map}va(){try{for(const a of this.C.values())null!==a&&a.destroy()}finally{this.C.clear()}}}Mr.na=Pd({ma:Mr,fa:"environment",ba:()=>new Mr(r(og))});function Nr(a){Ii("NgStandalone");a.Rq=b=>{b=b.get(Mr);if(a.ka){if(!b.C.has(a)){var c=cg(a.type);c=0<c.length?un([c],b.Ra,`Standalone[${a.type.name}]`):null;b.C.set(a,c)}b=b.C.get(a)}else b=null;return b}};function Or(){var a=Pr,b=bh()+6;const c=Zg();c[b]===Bl?(a=a(),b=c[b]=a):b=c[b];return b}function Qr(a,b){var c=Zg(),d=bh()+2;zp(c,d,b)?(d+=1,a=a(b),c=c[d]=a):(c=c[d+1],c=c===Bl?void 0:c);return c};function Rr(a,b){return gr(a,b)};var Sr=class{};Sr.K=function(a){return new (a||Sr)(r(ip))};Sr.Ya=Sf({type:Sr});Sr.Xa=Qd({});Object.assign({},Qe,{Dy:void 0,Wu(a,b){Re(a,b)}});var Tr=class{};Tr.xf=Ur;function Ur(a){var b=$g(),c=Zg();-1<b.Kf&&16!==(a&16)?(a=Og(b.index,c),a=new gn(a,a)):a=b.type&47?new gn(c[15],c):null;return a};var Wr=class{supports(a){return xp(a)}create(a){return new Vr(a)}};const Xr=(a,b)=>b;function Yr(a,b){let c=a.N;a=a.P;let d=0,e=null;for(;c||a;){const f=!a||c&&c.dd<Zr(a,d,e)?c:a,g=Zr(f,d,e),h=f.dd;if(f===a)d--,a=a.D;else if(c=c.Oa,null==f.Le)d++;else{e||(e=[]);const m=g-d,p=h-d;if(m!=p){for(let u=0;u<m;u++){const w=u<e.length?e[u]:e[u]=0,B=w+u;p<=B&&B<m&&(e[u]=w+1)}e[f.Le]=p-m}}g!==h&&b(f,g,h)}}function $r(a,b){for(a=a.ea;null!==a;a=a.P)b(a)}
function as(a,b){bs(a);let c=a.N,d=!1,e,f,g;if(Array.isArray(b)){a.length=b.length;for(let h=0;h<a.length;h++)f=b[h],g=a.wa(h,f),null!==c&&Object.is(c.G,g)?(d&&(c=a.Ta(c,f,g,h)),Object.is(c.item,f)||a.ca(c,f)):(c=a.Ja(c,f,g,h),d=!0),c=c.Oa}else e=0,yp(b,h=>{g=a.wa(e,h);null!==c&&Object.is(c.G,g)?(d&&(c=a.Ta(c,h,g,e)),Object.is(c.item,h)||a.ca(c,h)):(c=a.Ja(c,h,g,e),d=!0);c=c.Oa;e++}),a.length=e;a.sb(c);a.collection=b;return a.Yn}
function bs(a){if(a.Yn){let b;for(b=a.gb=a.N;null!==b;b=b.Oa)b.aa=b.Oa;for(b=a.aa;null!==b;b=b.O)b.Le=b.dd;a.aa=a.G=null;for(b=a.ja;null!==b;b=b.T)b.Le=b.dd;a.ja=a.O=null;a.P=a.D=null;a.ea=a.J=null}}
var Vr=class{constructor(a){this.length=0;this.J=this.ea=this.D=this.P=this.O=this.ja=this.G=this.aa=this.T=this.N=this.gb=this.C=this.F=null;this.wa=a||Xr}sn(a){null==a&&(a=[]);if(!xp(a))throw new ae(900,!1);return as(this,a)?this:null}ob(){}get Yn(){return null!==this.aa||null!==this.ja||null!==this.P||null!==this.ea}Ja(a,b,c,d){let e;null===a?e=this.T:(e=a.F,this.lb(a));a=null===this.C?null:this.C.get(c,null);null!==a?(Object.is(a.item,b)||this.ca(a,b),this.Qa(a,e,d)):(a=null===this.F?null:this.F.get(c,
d),null!==a?(Object.is(a.item,b)||this.ca(a,b),this.fb(a,e,d)):a=this.Va(new cs(b,c),e,d));return a}Ta(a,b,c,d){b=null===this.C?null:this.C.get(c,null);null!==b?a=this.Qa(b,a.F,d):a.dd!=d&&(a.dd=d,this.oa(a,d));return a}sb(a){for(;null!==a;){const b=a.Oa;this.Ea(this.ua(a));a=b}null!==this.C&&this.C.clear();null!==this.G&&(this.G.O=null);null!==this.O&&(this.O.T=null);null!==this.T&&(this.T.Oa=null);null!==this.D&&(this.D.D=null);null!==this.J&&(this.J.P=null)}Qa(a,b,c){null!==this.C&&this.C.remove(a);
const d=a.N,e=a.D;null===d?this.P=e:d.D=e;null===e?this.D=d:e.N=d;this.ta(a,b,c);this.oa(a,c);return a}fb(a,b,c){this.ua(a);this.ta(a,b,c);this.oa(a,c);return a}Va(a,b,c){this.ta(a,b,c);this.G=null===this.G?this.aa=a:this.G.O=a;return a}ta(a,b,c){const d=null===b?this.N:b.Oa;a.Oa=d;a.F=b;null===d?this.T=a:d.F=a;null===b?this.N=a:b.Oa=a;null===this.F&&(this.F=new ds);es(this.F,a);a.dd=c;return a}lb(a){return this.Ea(this.ua(a))}ua(a){null!==this.F&&this.F.remove(a);const b=a.F,c=a.Oa;null===b?this.N=
c:b.Oa=c;null===c?this.T=b:c.F=b;return a}oa(a,b){if(a.Le===b)return a;this.O=null===this.O?this.ja=a:this.O.T=a;return a}Ea(a){null===this.C&&(this.C=new ds);es(this.C,a);a.dd=null;a.D=null;null===this.D?(this.D=this.P=a,a.N=null):(a.N=this.D,this.D=this.D.D=a);return a}ca(a,b){a.item=b;this.J=null===this.J?this.ea=a:this.J.P=a;return a}},cs=class{constructor(a,b){this.item=a;this.G=b;this.P=this.T=this.O=this.D=this.N=this.C=this.J=this.Oa=this.F=this.aa=this.Le=this.dd=null}};
class fs{constructor(){this.D=this.C=null}add(a){null===this.C?(this.C=this.D=a,a.C=null,a.J=null):(this.D.C=a,a.J=this.D,a.C=null,this.D=a)}get(a,b){let c;for(c=this.C;null!==c;c=c.C)if((null===b||b<=c.dd)&&Object.is(c.G,a))return c;return null}remove(a){const b=a.J;a=a.C;null===b?this.C=a:b.C=a;null===a?this.D=b:a.J=b;return null===this.C}}function es(a,b){const c=b.G;let d=a.map.get(c);d||(d=new fs,a.map.set(c,d));d.add(b)}
class ds{constructor(){this.map=new Map}get(a,b){const c=this.map.get(a);return c?c.get(a,b):null}remove(a){const b=a.G;this.map.get(b).remove(a)&&this.map.delete(b);return a}clear(){this.map.clear()}}function Zr(a,b,c){a=a.Le;if(null===a)return a;let d=0;c&&a<c.length&&(d=c[a]);return a+b+d};var gs=class{constructor(a){this.C=a}find(a){const b=this.C.find(c=>c.supports(a));if(null!=b)return b;throw new ae(901,!1);}};gs.na=Pd({ma:gs,fa:"root",ba:function(){return new gs([new Wr])}});function hs(a,b){if(!a.C&&!a.G.C){var c=a.F.add();try{a.ea.run(()=>{a.C=!0;a.G.J(b)},void 0,a.ja)}catch(d){throw a.F.remove(c),d;}finally{a.fe()}a.aa=!0;Mk(()=>{a.aa=!1;a.F.remove(c)})}}
var is=class{constructor(){this.G=of(ip);this.F=of(An);this.ea=of(Tk);this.O=of(gi);let a;this.P=null!=(a=of(hi,{optional:!0}))?a:!1;this.ca="undefined"!==typeof Zone&&!!Zone.root.run;this.ja=[{data:{__scheduler_tick__:!0}}];this.T=new Si;this.J=null;this.C=this.aa=this.N=!1;this.D=null;this.T.add(this.G.O.subscribe(()=>{this.C||this.fe()}));this.T.add(this.ea.ro.subscribe(()=>{this.C||this.fe()}));this.P||(this.P=!this.O&&!this.ca)}notify(a){if(this.O||5!==a){switch(a){case 3:case 2:case 0:case 4:case 5:case 1:this.N=
!0}if(!(this.P||null!==this.D||this.C||this.G.G||!this.O&&this.ca&&Ok())){var b=this.aa?Mk:Lk;this.D=this.F.add();this.ca?Zone.root.run(()=>{this.J=b(()=>{hs(this,this.N)})}):this.J=b(()=>{hs(this,this.N)})}}}va(){this.T.unsubscribe();this.fe()}fe(){this.C=this.N=!1;var a;null==(a=this.J)||a.call(this);this.J=null;null!==this.D&&(a=this.D,this.D=null,this.F.remove(a))}};is.K=function(a){return new (a||is)};is.na=Pd({ma:is,ba:is.K,fa:"root"});function js(a){a.C||(a.C=a.zone.ex.subscribe({next:()=>{a.cj.C||a.zone.run(()=>{a.D.J(!0)})}}))}var ks=class{constructor(){this.zone=of(Tk);this.cj=of(fi);this.D=of(ip)}va(){let a;null==(a=this.C)||a.unsubscribe()}};ks.K=function(a){return new (a||ks)};ks.na=Pd({ma:ks,ba:ks.K,fa:"root"});new gf("",{ba:()=>!1});
function ls(){({pG:b,gG:a}={});var a,b;null!=b||(b=()=>new Tk({XB:!1,Io:!1,cm:!1}));return[{pa:Tk,Gc:b},{pa:tf,Eb:!0,Gc:()=>{const c=of(ks,{optional:!0});return()=>js(c)}},{pa:tf,Eb:!0,Gc:()=>{const c=of(ms);return()=>{ns(c)}}},{pa:Bi,Gc:os},!0===a?{pa:hi,nb:!0}:[]]}function os(){const a=of(Tk),b=of(Ai);return c=>Qk(a,()=>b.handleError(c))}
function ns(a){if(!a.F){a.F=!0;var b=null;a.zone.isStable||a.zone.Nn||a.zone.pl||(b=a.C.add());Qk(a.zone,()=>{a.D.add(a.zone.qo.subscribe(()=>{Pk();queueMicrotask(()=>{null===b||a.zone.Nn||a.zone.pl||(a.C.remove(b),b=null)})}))});a.D.add(a.zone.ro.subscribe(()=>{if(!Ok())throw new ae(909,!1);null!=b||(b=a.C.add())}))}}var ms=class{constructor(){this.D=new Si;this.F=!1;this.zone=of(Tk);this.C=of(An)}va(){this.D.unsubscribe()}};ms.K=function(a){return new (a||ms)};ms.na=Pd({ma:ms,ba:ms.K,fa:"root"});var ps=new gf("",{fa:"root",ba:()=>of(ps,12)||"undefined"!==typeof $localize&&$localize.locale||"en-US"});new gf("",{fa:"root",ba:()=>"USD"});new gf("");new gf("");var qs=class{};qs.K=function(a){return new (a||qs)};qs.na=Pd({ma:qs,ba:qs.K,fa:"root"});new gf("");var rs=new gf(""),ss=class{constructor(a){this.Ra=a;this.F=[];this.D=[];this.Ga=!1}G(a){const b=a.xa.get(ip);if(0<a.np.length)a.np.forEach(c=>b.Pk(c));else if(a.instance.kD)a.instance.kD(b);else throw new ae(-403,!1);this.F.push(a)}ob(a){this.D.push(a)}get xa(){return this.Ra}destroy(){if(this.Ga)throw new ae(404,!1);this.F.slice().forEach(b=>b.destroy());this.D.forEach(b=>b());const a=this.Ra.get(rs,null);a&&(a.forEach(b=>b()),a.clear());this.Ga=!0}get C(){return this.Ga}};
ss.K=function(a){return new (a||ss)(r(ai))};ss.na=Pd({ma:ss,ba:ss.K,fa:"platform"});let ts=null;var us=new gf("");function vs(a,b,c=[]){const d=`Platform: ${b}`,e=new gf(d);return(f=[])=>{let g=ws();if(!g||g.xa.get(us,!1))if(f=[...c,...f,{pa:e,nb:!0}],a)a(f);else{f=xs(f,d);if(ts&&!ts.get(us,!1))throw new ae(400,!1);cp();ts=f;f.get(ss);ys(f)}f=ws();if(!f)throw new ae(401,!1);return f}}function xs(a=[],b){return $h({name:b,Wa:[{pa:jg,nb:"platform"},{pa:rs,nb:new Set([()=>ts=null])},...a]})}function ws(){let a,b;return null!=(b=null==(a=ts)?void 0:a.get(ss))?b:null}
function zs(a=[]){if(ts)return ts;ts=a=xs(a);cp();ys(a);return a}function ys(a){let b;null==(b=a.get(oi,null))||b.forEach(c=>c())};/*

 Copyright 2024 Google, Inc
 SPDX-License-Identifier: MIT
*/
Symbol.for("propagationStopped");function As(a){return"boolean"===typeof a?a:null!=a&&"false"!==a}function Bs(a,b=NaN){return isNaN(parseFloat(a))||isNaN(Number(a))?b:Number(a)};function Cs(a){Ii("NgSignals");return Je(a)};var Es=new gf("",{fa:"root",ba:()=>of(Ds)}),Ds=class{};Ds.na=Pd({ma:Ds,fa:"root",ba:()=>new Fs});function Gs(a,b){var c=b.DB;a.C.has(c)||a.C.set(c,new Set);c=a.C.get(c);c.has(b)||(a.F++,c.add(b))}function Hs(a,b){Gs(a,b);if(null===a.D){const c=a.D=a.G.add();queueMicrotask(()=>{a.flush();a.G.remove(c);a.D=null})}}function Is(a,b){for(const c of b)b.delete(c),a.F--,c.run()}
var Fs=class{constructor(){this.F=0;this.C=new Map;this.G=of(An);this.D=null}flush(){for(;0<this.F;)for(const [a,b]of this.C){const c=a,d=b;null===c?Is(this,d):c.run(()=>Is(this,d))}}};
class Js{constructor(a,b,c,d,e,f){this.scheduler=a;this.D=b;this.DB=c;this.xa=e;this.C=Se(g=>{try{this.D(g)}catch(h){let m;null==(m=this.xa.get(Ai,null,{optional:!0}))||m.handleError(h)}},()=>this.Oc(),f);this.F=null==d?void 0:d.ob(()=>this.destroy())}run(){this.C.run()}Oc(){Hs(this.scheduler,this)}destroy(){this.C.destroy();let a;null==(a=this.F)||a.call(this)}}
function Ks(a,b){Ii("NgSignals");(null==b?0:b.xa)||Dg();var c,d=null!=(c=null==b?void 0:b.xa)?c:of(ai);c=!0!==(null==b?void 0:b.ZC)?d.get(Ci):null;let e;a=new Js(d.get(Es),a,"undefined"===typeof Zone?null:Zone.current,c,d,null!=(e=null==b?void 0:b.fq)?e:!1);if((d=d.get(Tr,null,{optional:!0}))&&d.D[2]&8){let f,g;(null!=(g=(f=d.D)[22])?g:f[22]=[]).push(a.C.notify)}else a.C.notify();return a};function Ls(a){const b=se(null);try{return a()}finally{se(b)}};var Ms=vs(null,"core",[]);function Ns(a){const b=Os[be]||null,c=a.VF||ng();return(new ln(b)).create(c,a.Bo,a.ff,a.Pf)};var Ps=new gf("GMAT_AUTOCOMPLETE_OPTIONS",{fa:"root",ba:()=>({tc:!1})});function Qs(a){return Array.isArray(a)?a:[a]};function Rs(a){return null!=a&&"false"!==`${a}`};function Ss(a){return null==a?"":"string"===typeof a?a:`${a}px`};function Ts(a){return a instanceof wi?a.R:a};function Us(a){return isNaN(parseFloat(a))||isNaN(Number(a))?0:Number(a)};const Vs=/\s+/,Ws=[];
var Xs=class{constructor(a,b){this.F=a;this.G=b;this.N=Ws;this.C=new Map}set VC(a){this.N=null!=a?a.trim().split(Vs):Ws}set jD(a){this.P="string"===typeof a?a.trim().split(Vs):a}Cj(){for(var a of this.N)this.D(a,!0);a=this.P;if(Array.isArray(a)||a instanceof Set)for(const b of a)this.D(b,!0);else if(null!=a)for(const b of Object.keys(a))this.D(b,!!a[b]);this.O()}D(a,b){const c=this.C.get(a);void 0!==c?(c.enabled!==b&&(c.Ch=!0,c.enabled=b),c.Vc=!0):this.C.set(a,{enabled:b,Ch:!0,Vc:!0})}O(){for(const a of this.C){const b=
a[0],c=a[1];c.Ch?(this.J(b,c.enabled),c.Ch=!1):c.Vc||(c.enabled&&this.J(b,!1),this.C.delete(b));c.Vc=!1}}J(a,b){a=a.trim();0<a.length&&a.split(Vs).forEach(c=>{b?this.G.Wi(this.F.R,c):this.G.Do(this.F.R,c)})}};Xs.K=function(a){return new (a||Xs)(y(wi),y(zi))};Xs.Fa=Uf({type:Xs,ha:[["","ngClass",""]],inputs:{VC:[0,"class","klass"],jD:"ngClass"},ka:!0});var Ys=class{constructor(a,b){this.da=a;this.Dr=b;this.count=this.index=-1}get first(){return 0===this.index}};function Zs(a,b){const c=a.O;Yr(b,(d,e,f)=>{null==d.Le?c.xd(a.G,new Ys(d.item,a.C),null===f?void 0:f):null==f?c.remove(null===e?void 0:e):null!==e&&(e=c.get(e),c.move(e,f),e.context.da=d.item)});for(let d=0,e=c.length;d<e;d++){const f=c.get(d).context;f.index=d;f.count=e;f.Dr=a.C}$r(b,d=>{c.get(d.dd).context.da=d.item})}
var $s=class{set Dr(a){this.C=a;this.F=!0}set Er(a){this.N=a}get Er(){return this.N}constructor(a,b,c){this.O=a;this.G=b;this.J=c;this.C=null;this.F=!0;this.D=null}set lD(a){a&&(this.G=a)}Cj(){if(this.F){this.F=!1;var a=this.C;!this.D&&a&&(this.D=this.J.find(a).create(this.Er))}this.D&&(a=this.D.sn(this.C))&&Zs(this,a)}};$s.K=function(a){return new (a||$s)(y(np),y(dr),y(gs))};$s.Fa=Uf({type:$s,ha:[["","ngFor","","ngForOf",""]],inputs:{Dr:"ngForOf",Er:"ngForTrackBy",lD:"ngForTemplate"},ka:!0});function at(a){a.Db.da?a.F||(a.C.clear(),a.D=null,a.J&&(a.F=a.C.xd(a.J,a.Db))):a.D||(a.C.clear(),a.F=null,a.G&&(a.D=a.C.xd(a.G,a.Db)))}var dt=class{constructor(a,b){this.C=a;this.Db=new bt;this.D=this.F=this.G=null;this.J=b}set Fr(a){this.Db.da=this.Db.Fr=a;at(this)}set nD(a){ct("ngIfThen",a);this.J=a;this.F=null;at(this)}set mD(a){ct("ngIfElse",a);this.G=a;this.D=null;at(this)}};dt.K=function(a){return new (a||dt)(y(np),y(dr))};
dt.Fa=Uf({type:dt,ha:[["","ngIf",""]],inputs:{Fr:"ngIf",nD:"ngIfThen",mD:"ngIfElse"},ka:!0});var bt=class{constructor(){this.Fr=this.da=null}};function ct(a,b){if(b&&!b.xd)throw Error(`${a} must be a TemplateRef, but received '${je(b)}'.`);};var et=class{constructor(a,b){this.D=a;this.F=b;this.C=!1}create(){this.C=!0;this.D.xd(this.F)}destroy(){this.C=!1;this.D.clear()}},ft=class{constructor(){this.F=[];this.J=!1;this.G=this.D=0;this.C=!1}set Gr(a){this.T=a;0===this.D&&this.N(!0)}O(){return this.D++}aa(a){this.F.push(a)}P(a){a=a==this.T;this.C||(this.C=a);this.G++;this.G===this.D&&(this.N(!this.C),this.G=0,this.C=!1);return a}N(a){if(0<this.F.length&&a!==this.J){this.J=a;for(const d of this.F){var b=d,c=a;c&&!b.C?b.create():!c&&b.C&&
b.destroy()}}}};ft.K=function(a){return new (a||ft)};ft.Fa=Uf({type:ft,ha:[["","ngSwitch",""]],inputs:{Gr:"ngSwitch"},ka:!0});var gt=class{constructor(a,b,c){this.Gr=c;c.O();this.C=new et(a,b)}Cj(){var a=this.C,b=this.Gr.P(this.oD);b&&!a.C?a.create():!b&&a.C&&a.destroy()}};gt.K=function(a){return new (a||gt)(y(np),y(dr),y(ft,9))};gt.Fa=Uf({type:gt,ha:[["","ngSwitchCase",""]],inputs:{oD:"ngSwitchCase"},ka:!0});var ht=class{constructor(a){this.D=a;this.bx=this.Hr=this.Nl=this.C=null}uc(a){if(this.G(a)){const c=this.D;this.C&&c.remove(c.indexOf(this.C));if(this.Hr){a=this.F();var b;this.C=c.xd(this.Hr,a,{xa:null!=(b=this.bx)?b:void 0})}else this.C=null}}G(a){return!!a.ngTemplateOutlet||!!a.ngTemplateOutletInjector}F(){return new Proxy({},{set:(a,b,c)=>this.Nl?Reflect.set(this.Nl,b,c):!1,get:(a,b,c)=>{if(this.Nl)return Reflect.get(this.Nl,b,c)}})}};ht.K=function(a){return new (a||ht)(y(np))};
ht.Fa=Uf({type:ht,ha:[["","ngTemplateOutlet",""]],inputs:{Nl:"ngTemplateOutletContext",Hr:"ngTemplateOutlet",bx:"ngTemplateOutletInjector"},ka:!0,features:[Hg]});new gf("");new gf("");var it=class{};it.K=function(a){return new (a||it)};it.Ya=Sf({type:it});it.Xa=Qd({});var jt=a=>a.src;new gf("",{fa:"root",ba:()=>jt});var kt=new gf("");new gf("");new gf("NG_OPTIMIZED_PRELOADED_IMAGES",{fa:"root",ba:()=>new Set});let lt=null;var mt=class{};var nt=class{F(){throw Error("");}};nt.K=function(a){return new (a||nt)};nt.na=Pd({ma:nt,ba:()=>of(ot),fa:"platform"});new gf("");function pt(a,b){const c=qt(a.J,"window");c.addEventListener("popstate",b,!1);return()=>c.removeEventListener("popstate",b)}function rt(a,b){const c=qt(a.J,"window");c.addEventListener("hashchange",b,!1);return()=>c.removeEventListener("hashchange",b)}
var ot=class extends nt{constructor(){super();this.J=of(kt);this.C=window.location;this.D=window.history}get port(){return this.C.port}get G(){return this.C.pathname}get N(){return this.C.search}get hash(){return this.C.hash}set G(a){this.C.pathname=a}F(a=0){this.D.go(a)}getState(){return this.D.state}};ot.K=function(a){return new (a||ot)};ot.na=Pd({ma:ot,ba:()=>new ot,fa:"platform"});function st(a,b){if(0==a.length)return b;if(0==b.length)return a;let c=0;a.endsWith("/")&&c++;b.startsWith("/")&&c++;return 2==c?a+b.substring(1):1==c?a+b:a+"/"+b}function tt(a){var b=a.match(/#|\?|$/);b=b&&b.index||a.length;return a.slice(0,b-("/"===a[b-1]?1:0))+a.slice(b)}function ut(a){return a&&"?"!==a[0]?"?"+a:a};var vt=class{G(){throw Error("");}};vt.K=function(a){return new (a||vt)};vt.na=Pd({ma:vt,ba:()=>of(wt),fa:"root"});var xt=new gf("");function yt(a,b){a.F.push(pt(a.C,b),rt(a.C,b))}
var wt=class extends vt{constructor(a,b){super();this.C=a;this.F=[];let c,d,e;null==b&&(b=(zt=zt||document.querySelector("base"))?zt.getAttribute("href"):null,b=null==b?null:(new URL(b,document.baseURI)).pathname);this.D=null!=(e=null!=(d=b)?d:null==(c=of(kt).location)?void 0:c.origin)?e:""}va(){for(;this.F.length;)this.F.pop()()}path(a=!1){const b=this.C.G+ut(this.C.N),c=this.C.hash;return c&&a?`${b}${c}`:b}getState(){return this.C.getState()}G(a=0){let b,c;null==(c=(b=this.C).F)||c.call(b,a)}};
wt.K=function(a){return new (a||wt)(r(nt),r(xt,8))};wt.na=Pd({ma:wt,ba:wt.K,fa:"root"});function At(a,b){b&&"/"!==b[0]&&(b="/"+b);return st(a.C.D,b)}function Bt(a,b,c=null){var d=a.C;d.C.D.pushState(c,"",st(d.D,b+ut("")));a.D(At(a,b+ut("")),c)}function Ct(a,b,c=null){var d=a.C;d.C.D.replaceState(c,"",st(d.D,b+ut("")));a.D(At(a,b+ut("")),c)}function Dt(a,b=0){let c,d;null==(d=(c=a.C).G)||d.call(c,b)}
var Ft=class{constructor(a){this.yh=new Jk;this.F=[];this.J=null;this.C=a;this.G=Et(tt(this.C.D.replace(/\/index.html$/,"")));yt(this.C,b=>{this.yh.emit({url:this.path(!0),pop:!0,state:b.state,type:b.type})})}va(){let a;null==(a=this.J)||a.unsubscribe();this.F=[]}path(a=!1){return this.normalize(this.C.path(a))}getState(){return this.C.getState()}normalize(a){var b=this.G;a=a.replace(/\/index.html$/,"");b&&a.startsWith(b)?(b=a.substring(b.length),b=""===b||["/",";","?","#"].includes(b[0])?b:a):b=
a;return tt(b)}D(a="",b){this.F.forEach(c=>c(a,b))}subscribe(a,b,c){return this.yh.subscribe({next:a,error:b,complete:c})}};Ft.K=function(a){return new (a||Ft)(r(vt))};Ft.na=Pd({ma:Ft,ba:()=>new Ft(r(vt)),fa:"root"});function Et(a){return RegExp("^(https?:)?//").test(a)?([,a]=a.split(/\/\/[^\/]+/),a):a};var Gt=class{};var Ht=class{constructor(a){this.D=[];this.ia=a}va(){this.detach()}add(a){this.remove(a);this.D.push(a)}remove(a){a=this.D.indexOf(a);-1<a&&this.D.splice(a,1);0===this.D.length&&this.detach()}};Ht.K=function(a){return new (a||Ht)(r(kt))};Ht.na=Pd({ma:Ht,ba:Ht.K,fa:"root"});var It=class extends Ht{constructor(a,b){super(a);this.qa=b;this.F=c=>{const d=this.D;for(let e=d.length-1;-1<e;e--)if(0<d[e].aa.nc.length){const f=d[e].aa;this.qa?this.qa.run(()=>f.next(c)):f.next(c);break}}}add(a){super.add(a);this.C||(this.qa?Qk(this.qa,()=>this.ia.body.addEventListener("keydown",this.F)):this.ia.body.addEventListener("keydown",this.F),this.C=!0)}detach(){this.C&&(this.ia.body.removeEventListener("keydown",this.F),this.C=!1)}};It.K=function(a){return new (a||It)(r(kt),r(Tk,8))};
It.na=Pd({ma:It,ba:It.K,fa:"root"});let Jt;function Kt(){if(null==Jt&&"undefined"!==typeof window)try{window.addEventListener("test",null,Object.defineProperty({},"passive",{get:()=>Jt=!0}))}finally{Jt=Jt||!1}return Jt}function Lt(a){return Kt()?a:!!a.capture};let Mt,Nt;function Ot(){if(null==Nt){if("object"!==typeof document||!document||"function"!==typeof Element||!Element)return Nt=!1;if("scrollBehavior"in document.documentElement.style)Nt=!0;else{const a=Element.prototype.scrollTo;Nt=a?!/\{\s*\[native code\]\s*\}/.test(a.toString()):!1}}return Nt}
function Pt(){if("object"!==typeof document||!document)return 0;if(null==Mt){const b=document.createElement("div");var a=b.style;b.dir="rtl";a.width="1px";a.overflow="auto";a.visibility="hidden";a.pointerEvents="none";a.position="absolute";a=document.createElement("div");const c=a.style;c.width="2px";c.height="1px";b.appendChild(a);document.body.appendChild(b);Mt=0;0===b.scrollLeft&&(b.scrollLeft=1,Mt=0===b.scrollLeft?1:2);b.remove()}return Mt};let Qt;function Rt(a){if(null==Qt){const b="undefined"!==typeof document?document.head:null;Qt=!(!b||!b.createShadowRoot&&!b.attachShadow)}return Qt&&(a=a.getRootNode?a.getRootNode():null,"undefined"!==typeof ShadowRoot&&ShadowRoot&&a instanceof ShadowRoot)?a:null}function St(){let a="undefined"!==typeof document&&document?document.activeElement:null;for(;a&&a.shadowRoot;){const b=a.shadowRoot.activeElement;if(b===a)break;else a=b}return a}
function Tt(a){return a.composedPath?a.composedPath()[0]:a.target};function Ut(){return"undefined"!==typeof __karma__&&!!__karma__||"undefined"!==typeof jasmine&&!!jasmine||"undefined"!==typeof jest&&!!jest||"undefined"!==typeof Mocha&&!!Mocha};let Vt;try{Vt="undefined"!==typeof Intl&&Intl.v8BreakIterator}catch(a){Vt=!1}
var Wt=class{constructor(a){this.J=(this.isBrowser=(this.O=a)?"browser"===this.O:"object"===typeof document&&!!document)&&/(edge)/i.test(navigator.userAgent);this.D=this.isBrowser&&/(msie|trident)/i.test(navigator.userAgent);this.G=this.isBrowser&&!(!window.chrome&&!Vt)&&"undefined"!==typeof CSS&&!this.J&&!this.D;this.F=this.isBrowser&&/AppleWebKit/i.test(navigator.userAgent)&&!this.G&&!this.J&&!this.D;this.C=this.isBrowser&&/iPad|iPhone|iPod/.test(navigator.userAgent)&&!("MSStream"in window);this.N=
this.isBrowser&&/(firefox|minefield)/i.test(navigator.userAgent);this.P=this.isBrowser&&/android/i.test(navigator.userAgent)&&!this.D;this.T=this.isBrowser&&/safari/i.test(navigator.userAgent)&&this.F}};Wt.K=function(a){return new (a||Wt)(r(pi))};Wt.na=Pd({ma:Wt,ba:Wt.K,fa:"root"});var Xt=class extends Ht{constructor(a,b,c){super(a);this.Ma=b;this.qa=c;this.G=!1;this.O=d=>{this.J=Tt(d)};this.F=d=>{const e=Tt(d),f="click"===d.type&&this.J?this.J:e;this.J=null;const g=this.D.slice();for(let h=g.length-1;-1<h;h--){const m=g[h];if(1>m.ca.nc.length||!m.jc())continue;if(m.N.contains(e)||m.N.contains(f))break;const p=m.ca;this.qa?this.qa.run(()=>p.next(d)):p.next(d)}}}add(a){super.add(a);if(!this.C){const b=this.ia.body;this.qa?Qk(this.qa,()=>this.N(b)):this.N(b);this.Ma.C&&!this.G&&
(this.P=b.style.cursor,b.style.cursor="pointer",this.G=!0);this.C=!0}}detach(){if(this.C){const a=this.ia.body;a.removeEventListener("pointerdown",this.O,!0);a.removeEventListener("click",this.F,!0);a.removeEventListener("auxclick",this.F,!0);a.removeEventListener("contextmenu",this.F,!0);this.Ma.C&&this.G&&(a.style.cursor=this.P,this.G=!1);this.C=!1}}N(a){a.addEventListener("pointerdown",this.O,!0);a.addEventListener("click",this.F,!0);a.addEventListener("auxclick",this.F,!0);a.addEventListener("contextmenu",
this.F,!0)}};Xt.K=function(a){return new (a||Xt)(r(kt),r(Wt),r(Tk,8))};Xt.na=Pd({ma:Xt,ba:Xt.K,fa:"root"});function Yt(a){a.Ii||a.C();return a.Ii}
var Zt=class{constructor(a,b){this.Ma=b;this.ia=a}va(){let a;null==(a=this.Ii)||a.remove()}C(){if(this.Ma.isBrowser||Ut()){var a=this.ia.querySelectorAll('.cdk-overlay-container[platform="server"], .cdk-overlay-container[platform="test"]');for(let b=0;b<a.length;b++)a[b].remove()}a=this.ia.createElement("div");a.classList.add("cdk-overlay-container");Ut()?a.setAttribute("platform","test"):this.Ma.isBrowser||a.setAttribute("platform","server");this.ia.body.appendChild(a);this.Ii=a}};
Zt.K=function(a){return new (a||Zt)(r(kt),r(Wt))};Zt.na=Pd({ma:Zt,ba:Zt.K,fa:"root"});const $t=Ot();
var cu=class{constructor(a,b){this.G=a;this.D={top:"",left:""};this.C=!1;this.ia=b}attach(){}enable(){if(this.J()){const a=this.ia.documentElement;this.F=au(this.G);this.D.left=a.style.left||"";this.D.top=a.style.top||"";a.style.left=Ss(-this.F.left);a.style.top=Ss(-this.F.top);a.classList.add("cdk-global-scrollblock");this.C=!0}}disable(){if(this.C){const a=this.ia.documentElement,b=a.style,c=this.ia.body.style,d=b.C||"",e=c.C||"";this.C=!1;b.left=this.D.left;b.top=this.D.top;a.classList.remove("cdk-global-scrollblock");$t&&
(b.C=c.C="auto");window.scroll(this.F.left,this.F.top);$t&&(b.C=d,c.C=e)}}J(){if(this.ia.documentElement.classList.contains("cdk-global-scrollblock")||this.C)return!1;const a=this.ia.body,b=bu(this.G);return a.scrollHeight>b.height||a.scrollWidth>b.width}};var eu=class{constructor(a,b,c,d){this.G=a;this.qa=b;this.F=c;this.Aa=d;this.C=null;this.D=()=>{this.disable();this.la.jc()&&this.qa.run(()=>this.la.detach())}}attach(a){this.la=a}enable(){if(!this.C){var a=C(du(this.G,0),xk(b=>!b||!this.la.N.contains(b.Ub.R)));this.Aa&&this.Aa.threshold&&1<this.Aa.threshold?(this.J=au(this.F).top,this.C=a.subscribe(()=>{const b=au(this.F).top;Math.abs(b-this.J)>this.Aa.threshold?this.D():this.la.Wc()})):this.C=a.subscribe(this.D)}}disable(){this.C&&(this.C.unsubscribe(),
this.C=null)}detach(){this.disable();this.la=null}};var fu=class{enable(){}disable(){}attach(){}};function gu(a,b){return b.some(c=>{const d=a.top>c.bottom,e=a.right<c.left,f=a.left>c.right;return a.bottom<c.top||d||e||f})}function hu(a,b){return b.some(c=>{const d=a.bottom>c.bottom,e=a.left<c.left,f=a.right>c.right;return a.top<c.top||d||e||f})};var iu=class{constructor(a,b,c,d){this.D=a;this.F=b;this.qa=c;this.Aa=d;this.C=null}attach(a){this.la=a}enable(){this.C||(this.C=du(this.D,this.Aa?this.Aa.Px:0).subscribe(()=>{this.la.Wc();if(this.Aa&&this.Aa.autoClose){const a=this.la.N.getBoundingClientRect(),{width:b,height:c}=bu(this.F);gu(a,[{width:b,height:c,bottom:c,right:b,top:0,left:0}])&&(this.disable(),this.qa.run(()=>this.la.detach()))}}))}disable(){this.C&&(this.C.unsubscribe(),this.C=null)}detach(){this.disable();this.la=null}};new gf("VIRTUAL_SCROLL_STRATEGY");function du(a,b=20){return a.Ma.isBrowser?new bj(c=>{a.D||a.N();const d=0<b?C(a.F,On(b)).subscribe(c):a.F.subscribe(c);a.G++;return()=>{d.unsubscribe();a.G--;a.G||a.J()}}):oj()}function ju(a,b){const c=a.C.get(b);c&&(c.unsubscribe(),a.C.delete(b))}function ku(a,b){const c=[];a.C.forEach((d,e)=>{a.O(e,b)&&c.push(e)});return c}
var lu=class{constructor(a,b,c){this.qa=a;this.Ma=b;this.F=new hj;this.D=null;this.G=0;this.C=new Map;this.ia=c}register(a){this.C.has(a)||this.C.set(a,a.G.subscribe(()=>this.F.next(a)))}va(){this.J();this.C.forEach((a,b)=>ju(this,b));this.F.complete()}O(a,b){b=Ts(b);a=a.Ub.R;do if(b==a)return!0;while(b=b.parentElement);return!1}N(){this.D=Qk(this.qa,()=>qk((this.ia.defaultView||window).document,"scroll").subscribe(()=>this.F.next()))}J(){this.D&&(this.D.unsubscribe(),this.D=null)}};
lu.K=function(a){return new (a||lu)(r(Tk),r(Wt),r(kt,8))};lu.na=Pd({ma:lu,ba:lu.K,fa:"root"});var nu=new gf("cdk-dir-doc",{fa:"root",ba:mu});function mu(){return of(kt)};const ou=/^(ar|ckb|dv|he|iw|fa|nqo|ps|sd|ug|ur|yi|.*[-_](Adlm|Arab|Hebr|Nkoo|Rohg|Thaa))(?!.*[-_](Latn|Cyrl)($|-|_))($|-|_)/i;var pu=class{constructor(a){this.value="ltr";this.xb=new Jk;if(a){var b=a.documentElement?a.documentElement.dir:null;b=(a.body?a.body.dir:null)||b||"ltr";b=(null==b?void 0:b.toLowerCase())||"";let c;this.value="auto"===b&&"undefined"!==typeof navigator&&(null==(c=navigator)?0:c.language)?ou.test(navigator.language)?"rtl":"ltr":"rtl"===b?"rtl":"ltr"}}va(){this.xb.complete()}};
pu.K=function(a){return new (a||pu)(r(nu,8))};pu.na=Pd({ma:pu,ba:pu.K,fa:"root"});var qu=class{};qu.K=function(a){return new (a||qu)};qu.Ya=Sf({type:qu});qu.Xa=Qd({});var ru=class{constructor(a,b,c,d){this.Ub=a;this.C=b;this.D=c;this.dir=d;this.Ga=new hj;this.G=new bj(e=>Qk(this.D,()=>C(qk(this.Ub.R,"scroll"),Oo(this.Ga)).subscribe(e)))}Fb(){this.C.register(this)}va(){ju(this.C,this);this.Ga.next();this.Ga.complete()}scrollTo(a){const b=this.Ub.R,c=this.dir&&"rtl"==this.dir.value;null==a.left&&(a.left=c?a.end:a.start);null==a.right&&(a.right=c?a.start:a.end);null!=a.bottom&&(a.top=b.scrollHeight-b.clientHeight-a.bottom);c&&0!=Pt()?(null!=a.left&&(a.right=b.scrollWidth-
b.clientWidth-a.left),2==Pt()?a.left=a.right:1==Pt()&&(a.left=a.right?-a.right:a.right)):null!=a.right&&(a.left=b.scrollWidth-b.clientWidth-a.right);this.F(a)}F(a){const b=this.Ub.R;Ot()?b.scrollTo(a):(null!=a.top&&(b.scrollTop=a.top),null!=a.left&&(b.scrollLeft=a.left))}};ru.K=function(a){return new (a||ru)(y(wi),y(lu),y(Tk),y(pu,8))};ru.Fa=Uf({type:ru,ha:[["","cdk-scrollable",""],["","cdkScrollable",""]],ka:!0});var su=class{};function tu(a){return a&&"function"===typeof a.connect&&!(a instanceof zj)};new gf("_ViewRepeater");function uu(a,...b){b.forEach(c=>a.T(c));b=a.N();a.Li();return b}function vu(a,b){return a.C.has(a.J(b))}
var wu=class{get selected(){this.G||(this.G=Array.from(this.C.values()));return this.G}constructor(a=!1){this.O=a;this.ca=!0;this.Uk=void 0;this.C=new Set;this.D=[];this.F=[];this.Ch=new hj}select(...a){a.forEach(b=>this.aa(b));a=this.N();this.Li();return a}toggle(a){return vu(this,a)?uu(this,a):this.select(a)}clear(a=!0){this.P();const b=this.N();a&&this.Li();return b}sort(a){this.O&&this.selected&&this.G.sort(a)}Li(){this.G=null;if(this.F.length||this.D.length)this.Ch.next({source:this,added:this.F,
removed:this.D}),this.D=[],this.F=[]}aa(a){a=this.J(a);vu(this,a)||(this.O||this.P(),vu(this,a)||this.C.add(a),this.F.push(a))}T(a){a=this.J(a);vu(this,a)&&(this.C.delete(a),this.D.push(a))}P(){0===this.C.size||this.C.forEach(a=>this.T(a))}ea(){}N(){return!(!this.D.length&&!this.F.length)}J(a,b){if(this.Uk){let c;b=null!=(c=b)?c:this.C;for(let d of b)if(this.Uk(a,d))return d}return a}};function au(a){if(!a.Ma.isBrowser)return{top:0,left:0};const b=a.ia;a=a.ia.defaultView||window;const c=b.documentElement,d=c.getBoundingClientRect();return{top:-d.top||b.body.scrollTop||a.scrollY||c.scrollTop||0,left:-d.left||b.body.scrollLeft||a.scrollX||c.scrollLeft||0}}function bu(a){a.C||a.G();const b={width:a.C.width,height:a.C.height};a.Ma.isBrowser||(a.C=null);return b}
var xu=class{constructor(a,b,c){this.Ma=a;this.D=new hj;this.F=d=>{this.D.next(d)};this.ia=c;Qk(b,()=>{if(a.isBrowser){const d=this.ia.defaultView||window;d.addEventListener("resize",this.F);d.addEventListener("orientationchange",this.F)}this.xb().subscribe(()=>this.C=null)})}va(){if(this.Ma.isBrowser){const a=this.ia.defaultView||window;a.removeEventListener("resize",this.F);a.removeEventListener("orientationchange",this.F)}this.D.complete()}xb(a=20){return 0<a?C(this.D,On(a)):this.D}G(){const a=
this.ia.defaultView||window;this.C=this.Ma.isBrowser?{width:a.innerWidth,height:a.innerHeight}:{width:0,height:0}}};xu.K=function(a){return new (a||xu)(r(Wt),r(Tk),r(kt,8))};xu.na=Pd({ma:xu,ba:xu.K,fa:"root"});new gf("VIRTUAL_SCROLLABLE");var yu=class{};yu.K=function(a){return new (a||yu)};yu.Ya=Sf({type:yu});yu.Xa=Qd({});var zu=class{};zu.K=function(a){return new (a||zu)};zu.Ya=Sf({type:zu});zu.Xa=Qd({imports:[qu,yu,qu,yu]});var Au=class{constructor(a,b,c,d){this.F=a;this.D=b;this.qa=c;this.G=()=>new fu;this.close=e=>new eu(this.F,this.qa,this.D,e);this.block=()=>new cu(this.D,this.ia);this.C=e=>new iu(this.F,this.D,this.qa,e);this.ia=d}};Au.K=function(a){return new (a||Au)(r(lu),r(xu),r(Tk),r(kt))};Au.na=Pd({ma:Au,ba:Au.K,fa:"root"});var Bu=class{constructor(a){this.wc=new fu;this.hb="";this.Vb=!1;this.fc="cdk-overlay-dark-backdrop";this.Jh=!1;if(a){const b=Object.keys(a);for(const c of b)void 0!==a[c]&&(this[c]=a[c])}}};function Cu(a,...b){return b.length?b.some(c=>a[c]):a.altKey||a.shiftKey||a.ctrlKey||a.metaKey};function Du(a){const b=a.G;b&&(a.ea?a.T(b):(b.classList.remove("cdk-overlay-backdrop-showing"),Qk(a.qa,()=>{b.addEventListener("transitionend",a.Ta)}),b.style.pointerEvents="none",a.ua=Qk(a.qa,()=>setTimeout(()=>{a.T(b)},500))))}function Eu(a){return(a=a.Aa.direction)?"string"===typeof a?a:a.value:"ltr"}
var Fu=class{constructor(a,b,c,d,e,f,g,h,m,p=!1,u){this.Ye=a;this.C=b;this.D=c;this.Aa=d;this.qa=e;this.ja=f;this.ia=g;this.gb=h;this.oa=m;this.ea=p;this.Ra=u;this.G=null;this.ta=new hj;this.P=new hj;this.J=new hj;this.wa=Si.EMPTY;this.Qa=w=>this.ta.next(w);this.Ta=w=>{this.T(w.target)};this.aa=new hj;this.ca=new hj;this.Ja=new hj;d.wc&&(this.Kc=d.wc,this.Kc.attach(this));this.F=d.vc;this.lb=Ls(()=>al(()=>{this.Ja.next()},{xa:this.Ra}))}get N(){return this.D}get ff(){return this.C}attach(a){!this.C.parentElement&&
this.Ea&&this.Ea.appendChild(this.C);a=this.Ye.attach(a);this.F&&this.F.attach(this);this.Pb();this.fb();this.Ob();this.Kc&&this.Kc.enable();el(()=>{this.jc()&&this.Wc()},{xa:this.Ra});this.Va(!0);this.Aa.Vb&&this.sb();this.Aa.hb&&this.O(this.D,this.Aa.hb,!0);this.P.next();this.ja.add(this);this.Aa.Jh&&(this.wa=this.gb.subscribe(()=>this.dispose()));this.oa.add(this);"function"===typeof(null==a?void 0:a.ob)&&a.ob(()=>{this.jc()&&Qk(this.qa,()=>Promise.resolve().then(()=>this.detach()))});return a}detach(){if(this.jc()){Du(this);
this.Va(!1);this.F&&this.F.detach&&this.F.detach();this.Kc&&this.Kc.disable();var a=this.Ye.detach();this.J.next();this.ja.remove(this);this.Kb();this.wa.unsubscribe();this.oa.remove(this);return a}}dispose(){const a=this.jc();this.F&&this.F.dispose();this.Lb();this.T(this.G);this.wa.unsubscribe();this.ja.remove(this);this.Ye.dispose();this.P.complete();this.ta.complete();this.aa.complete();this.ca.complete();this.oa.remove(this);let b;null==(b=this.C)||b.remove();this.Ea=this.D=this.C=null;a&&this.J.next();
this.J.complete();this.lb.destroy();this.Ja.complete()}jc(){return this.Ye.jc()}bd(){return this.ta}gf(){return this.aa}Pl(){return this.ca}Wc(){this.F&&this.F.apply()}te(a){this.Aa=Object.assign({},this.Aa,a);this.fb()}qg(a){this.D&&this.O(this.D,a,!0)}Oj(a){this.D&&this.O(this.D,a,!1)}Ob(){this.C.setAttribute("dir",Eu(this))}fb(){if(this.D){var a=this.D.style;a.width=Ss(this.Aa.width);a.height=Ss(this.Aa.height);a.minWidth=Ss(this.Aa.minWidth);a.minHeight=Ss(this.Aa.minHeight);a.maxWidth=Ss(this.Aa.maxWidth);
a.maxHeight=Ss(this.Aa.maxHeight)}}Va(a){this.D.style.pointerEvents=a?"":"none"}sb(){this.G=this.ia.createElement("div");this.G.classList.add("cdk-overlay-backdrop");this.ea&&this.G.classList.add("cdk-overlay-backdrop-noop-animation");this.Aa.fc&&this.O(this.G,this.Aa.fc,!0);this.C.parentElement.insertBefore(this.G,this.C);this.G.addEventListener("click",this.Qa);this.ea||"undefined"===typeof requestAnimationFrame?this.G.classList.add("cdk-overlay-backdrop-showing"):Qk(this.qa,()=>{requestAnimationFrame(()=>
{this.G&&this.G.classList.add("cdk-overlay-backdrop-showing")})})}Pb(){this.C.nextSibling&&this.C.parentNode.appendChild(this.C)}O(a,b,c){b=Qs(b||[]).filter(d=>!!d);b.length&&(c?a.classList.add(...b):a.classList.remove(...b))}Kb(){Qk(this.qa,()=>{const a=C(this.Ja,Oo(wk(this.P,this.J))).subscribe(()=>{this.D&&this.C&&0!==this.D.children.length||(this.D&&this.Aa.hb&&this.O(this.D,this.Aa.hb,!1),this.C&&this.C.parentElement&&(this.Ea=this.C.parentElement,this.C.remove()),a.unsubscribe())})})}Lb(){const a=
this.Kc;a&&(a.disable(),a.detach&&a.detach())}T(a){a&&(a.removeEventListener("click",this.Qa),a.removeEventListener("transitionend",this.Ta),a.remove(),this.G===a&&(this.G=null));this.ua&&(clearTimeout(this.ua),this.ua=void 0)}};var Gu=class{constructor(a,b){this.C=a;this.D=b}};const Hu=/([A-Za-z%]+)$/;function Iu(a,b){a.sb=b;return a}function Ju(a){if(!a.We&&a.Ma.isBrowser){var b=a.G;b?(a.Ta=a.Ja(),a.F=a.D.getBoundingClientRect(),a.ua=a.cc(),a.Ji=Yt(a.Kb).getBoundingClientRect(),a.aa(b,a.Ec(a.Ta,a.Ji,b))):a.apply()}}function Ku(a,b){a.kd=b;return a}function Lu(a,b){a.fb=b;-1===b.indexOf(a.G)&&(a.G=null);return a}function Mu(a,b){a.C=b;return a}function Nu(a,b=!0){a.ja=b;return a}function Ou(a,b=!0){a.ea=b;return a}function Pu(a,b=!0){a.Ob=b;return a}
function Qu(a,b=!0){a.Va=b;return a}function Ru(a,b){a.lb=b;return a}function Su(a,b){a.md=b;return a}function Jv(a){return"rtl"===Eu(a.la)}
var Nv=class{get oe(){return this.fb}constructor(a,b,c,d,e){this.Ea=b;this.ia=c;this.Ma=d;this.Kb=e;this.P={width:0,height:0};this.O=!1;this.Ob=!0;this.ea=!1;this.ja=!0;this.Va=!1;this.C=0;this.kd=[];this.fb=[];this.oa=new hj;this.wa=Si.EMPTY;this.lb=0;this.T=[];this.gb=this.oa;Iu(this,a)}attach(a){a.ff.classList.add("cdk-overlay-connected-position-bounding-box");this.la=a;this.J=a.ff;this.D=a.N;this.We=!1;this.N=!0;this.G=null;this.wa.unsubscribe();this.wa=this.Ea.xb().subscribe(()=>{this.N=!0;this.apply()})}apply(){if(!this.We&&
this.Ma.isBrowser)if(!this.N&&this.Va&&this.G)Ju(this);else{this.Pb();this.Qc();this.Wd();this.ua=this.cc();this.Ta=this.Ja();this.F=this.D.getBoundingClientRect();this.Ji=Yt(this.Kb).getBoundingClientRect();var a=this.Ta,b=this.F,c=this.ua,d=this.Ji,e=[];for(let g of this.fb){let h=this.Ec(a,d,g),m=this.Qa(h,b,g),p=this.Ud(m,b,c,g);if(p.JC){this.O=!1;this.aa(g,h);return}if(this.td(p,m,c))e.push({position:g,origin:h,CD:b,av:this.Lb(h,g)});else if(!f||f.BD.Ry<p.Ry)var f={BD:p,sG:m,yD:h,position:g,
CD:b}}if(e.length){a=null;b=-1;for(const g of e)e=g.av.width*g.av.height*(g.position.weight||1),e>b&&(b=e,a=g);this.O=!1;this.aa(a.position,a.origin)}else this.Ob&&(this.O=!0),this.aa(f.position,f.yD)}}detach(){this.Pb();this.ta=this.G=null;this.wa.unsubscribe()}dispose(){this.We||(this.J&&Kv(this.J.style,{top:"",left:"",right:"",bottom:"",height:"",width:"",alignItems:"",justifyContent:""}),this.D&&this.Qc(),this.la&&this.la.ff.classList.remove("cdk-overlay-connected-position-bounding-box"),this.detach(),
this.oa.complete(),this.la=this.J=null,this.We=!0)}Ec(a,b,c){if("center"==c.bb)var d=a.left+a.width/2;else{d=Jv(this)?a.right:a.left;const e=Jv(this)?a.left:a.right;d="start"==c.bb?d:e}0>b.left&&(d-=b.left);a="center"==c.cb?a.top+a.height/2:"top"==c.cb?a.top:a.bottom;0>b.top&&(a-=b.top);return{x:d,y:a}}Qa(a,b,c){return{x:a.x+("center"==c.Na?-b.width/2:"start"===c.Na?Jv(this)?-b.width:0:Jv(this)?0:-b.width),y:a.y+("center"==c.Ua?-b.height/2:"top"==c.Ua?0:-b.height)}}Ud(a,b,c,d){b=Lv(b);let {x:e,y:f}=
a;a=this.ca(d,"x");d=this.ca(d,"y");a&&(e+=a);d&&(f+=d);d=-f;a=f+b.height-c.height;c=this.ld(b.width,-e,e+b.width-c.width);d=this.ld(b.height,d,a);a=c*d;return{Ry:a,JC:b.width*b.height===a,fC:d===b.height,eC:c==b.width}}td(a,b,c){if(this.ja){const d=c.bottom-b.y;c=c.right-b.x;b=Mv(this.la.Aa.minHeight);const e=Mv(this.la.Aa.minWidth);c=a.eC||null!=e&&e<=c;return(a.fC||null!=b&&b<=d)&&c}return!1}Jc(a,b,c){if(this.ta&&this.Va)return{x:a.x+this.ta.x,y:a.y+this.ta.y};b=Lv(b);const d=this.ua;var e=Math.max(a.x+
b.width-d.width,0);const f=Math.max(a.y+b.height-d.height,0),g=Math.max(d.top-c.top-a.y,0),h=Math.max(d.left-c.left-a.x,0);e=b.width<=d.width?h||-e:a.x<this.C?d.left-c.left-a.x:0;c=b.height<=d.height?g||-f:a.y<this.C?d.top-c.top-a.y:0;this.ta={x:e,y:c};return{x:a.x+e,y:a.y+c}}aa(a,b){this.Re(a);this.ae(b,a);this.Zd(b,a);a.hb&&this.sd(a.hb);if(this.oa.nc.length){b=this.Vd();var c;if(c=a===this.G&&this.Ic)c=this.Ic,c=c===b?!0:c.Gw===b.Gw&&c.Hw===b.Hw&&c.rr===b.rr&&c.Iw===b.Iw;c||this.oa.next(new Gu(a,
b));this.Ic=b}this.G=a;this.N=!1}Re(a){if(this.md){var b=this.J.querySelectorAll(this.md),c=a.Ua;a="center"===a.Na?"center":Jv(this)?"start"===a.Na?"right":"left":"start"===a.Na?"left":"right";for(let d=0;d<b.length;d++)b[d].style.transformOrigin=`${a} ${c}`}}Lb(a,b){var c=this.ua;const d=Jv(this);let e;let f;if("top"===b.Ua){var g=a.y;e=c.height-g+this.C}else if("bottom"===b.Ua)f=c.height-a.y+2*this.C,e=c.height-f+this.C;else{g=Math.min(c.bottom-a.y+c.top,a.y);var h=this.P.height;e=2*g;g=a.y-g;e>
h&&!this.N&&!this.ea&&(g=a.y-h/2)}h="start"===b.Na&&!d||"end"===b.Na&&d;let m;if("end"===b.Na&&!d||"start"===b.Na&&d)m=c.width-a.x+2*this.C,b=a.x-this.C;else if(h){var p=a.x;b=c.right-a.x}else p=Math.min(c.right-a.x+c.left,a.x),c=this.P.width,b=2*p,p=a.x-p,b>c&&!this.N&&!this.ea&&(p=a.x-c/2);return{top:g,left:p,bottom:f,right:m,width:b,height:e}}Zd(a,b){a=this.Lb(a,b);this.N||this.ea||(a.height=Math.min(a.height,this.P.height),a.width=Math.min(a.width,this.P.width));const c={};if(this.Hc())c.top=
c.left="0",c.bottom=c.right=c.maxHeight=c.maxWidth="",c.width=c.height="100%";else{const d=this.la.Aa.maxHeight,e=this.la.Aa.maxWidth;c.height=Ss(a.height);c.top=Ss(a.top);c.bottom=Ss(a.bottom);c.width=Ss(a.width);c.left=Ss(a.left);c.right=Ss(a.right);c.alignItems="center"===b.Na?"center":"end"===b.Na?"flex-end":"flex-start";c.justifyContent="center"===b.Ua?"center":"bottom"===b.Ua?"flex-end":"flex-start";d&&(c.maxHeight=Ss(d));e&&(c.maxWidth=Ss(e))}this.P=a;Kv(this.J.style,c)}Wd(){Kv(this.J.style,
{top:"0",left:"0",right:"0",bottom:"0",height:"",width:"",alignItems:"",justifyContent:""})}Qc(){Kv(this.D.style,{top:"",left:"",bottom:"",right:"",position:"",transform:""})}ae(a,b){const c={},d=this.Hc(),e=this.ja,f=this.la.Aa;if(d){var g=au(this.Ea);Kv(c,this.Td(b,a,g));Kv(c,this.ud(b,a,g))}else c.position="static";a="";g=this.ca(b,"x");b=this.ca(b,"y");g&&(a+=`translateX(${g}px) `);b&&(a+=`translateY(${b}px)`);c.transform=a.trim();f.maxHeight&&(d?c.maxHeight=Ss(f.maxHeight):e&&(c.maxHeight=""));
f.maxWidth&&(d?c.maxWidth=Ss(f.maxWidth):e&&(c.maxWidth=""));Kv(this.D.style,c)}Td(a,b,c){let d={top:"",bottom:""};b=this.Qa(b,this.F,a);this.O&&(b=this.Jc(b,this.F,c));"bottom"===a.Ua?d.bottom=`${this.ia.documentElement.clientHeight-(b.y+this.F.height)}px`:d.top=Ss(b.y);return d}ud(a,b,c){let d={left:"",right:""};b=this.Qa(b,this.F,a);this.O&&(b=this.Jc(b,this.F,c));"right"===(Jv(this)?"end"===a.Na?"left":"right":"end"===a.Na?"right":"left")?d.right=`${this.ia.documentElement.clientWidth-(b.x+this.F.width)}px`:
d.left=Ss(b.x);return d}Vd(){const a=this.Ja(),b=this.D.getBoundingClientRect(),c=this.kd.map(d=>d.Ub.R.getBoundingClientRect());return{Gw:hu(a,c),Hw:gu(a,c),rr:hu(b,c),Iw:gu(b,c)}}ld(a,...b){return b.reduce((c,d)=>c-Math.max(d,0),a)}cc(){const a=this.ia.documentElement.clientWidth,b=this.ia.documentElement.clientHeight,c=au(this.Ea);return{top:c.top+this.C,left:c.left+this.C,right:c.left+a-this.C,bottom:c.top+b-this.C,width:a-2*this.C,height:b-2*this.C}}Hc(){return!this.ja||this.O}ca(a,b){return"x"===
b?null==a.offsetX?this.lb:a.offsetX:null==a.offsetY?0:a.offsetY}hg(){}sd(a){this.D&&Qs(a).forEach(b=>{""!==b&&-1===this.T.indexOf(b)&&(this.T.push(b),this.D.classList.add(b))})}Pb(){this.D&&(this.T.forEach(a=>{this.D.classList.remove(a)}),this.T=[])}Ja(){const a=this.sb;if(a instanceof wi)return a.R.getBoundingClientRect();if(a instanceof Element)return a.getBoundingClientRect();const b=a.width||0,c=a.height||0;return{top:a.y,bottom:a.y+c,left:a.x,right:a.x+b,height:c,width:b}}};
function Kv(a,b){for(let c in b)b.hasOwnProperty(c)&&(a[c]=b[c])}function Mv(a){if("number"!==typeof a&&null!=a){const [b,c]=a.split(Hu);return c&&"px"!==c?null:parseFloat(b)}return a||null}function Lv(a){return{top:Math.floor(a.top),right:Math.floor(a.right),bottom:Math.floor(a.bottom),left:Math.floor(a.left),width:Math.floor(a.width),height:Math.floor(a.height)}};function Ov(a){a.left("");a.Gf="center";return a}function Pv(a){a.top("");a.vm="center";return a}
var Qv=class{constructor(){this.Xz="static";this.Ep=this.eq=this.Kk=this.Gf=this.vm=this.pp=this.Wp="";this.We=!1}attach(a){const b=a.Aa;this.la=a;this.eq&&!b.width&&a.te({width:this.eq});this.Ep&&!b.height&&a.te({height:this.Ep});a.ff.classList.add("cdk-global-overlay-wrapper");this.We=!1}top(a=""){this.pp="";this.Wp=a;this.vm="flex-start";return this}left(a=""){this.Kk=a;this.Gf="left";return this}bottom(a=""){this.Wp="";this.pp=a;this.vm="flex-end";return this}right(a=""){this.Kk=a;this.Gf="right";
return this}start(a=""){this.Kk=a;this.Gf="start";return this}end(a=""){this.Kk=a;this.Gf="end";return this}width(a=""){this.la?this.la.te({width:a}):this.eq=a;return this}height(a=""){this.la?this.la.te({height:a}):this.Ep=a;return this}apply(){if(this.la&&this.la.jc()){var a=this.la.N.style,b=this.la.ff.style,c=this.la.Aa,d=c.width,e=c.height,f=c.maxWidth;c=c.maxHeight;d=("100%"===d||"100vw"===d)&&(!f||"100%"===f||"100vw"===f);e=("100%"===e||"100vh"===e)&&(!c||"100%"===c||"100vh"===c);f=this.Gf;
c=this.Kk;var g="rtl"===this.la.Aa.direction,h="",m="",p="";if(d)p="flex-start";else if("center"===f)p="center",g?m=c:h=c;else if(g)if("left"===f||"end"===f)p="flex-end",h=c;else{if("right"===f||"start"===f)p="flex-start",m=c}else if("left"===f||"start"===f)p="flex-start",h=c;else if("right"===f||"end"===f)p="flex-end",m=c;a.position=this.Xz;a.marginLeft=d?"0":h;a.marginTop=e?"0":this.Wp;a.marginBottom=this.pp;a.marginRight=d?"0":m;b.justifyContent=p;b.alignItems=e?"flex-start":this.vm}}dispose(){if(!this.We&&
this.la){var a=this.la.N.style,b=this.la.ff,c=b.style;b.classList.remove("cdk-global-overlay-wrapper");c.justifyContent=c.alignItems=a.marginTop=a.marginBottom=a.marginLeft=a.marginRight=a.position="";this.la=null;this.We=!0}}};function Rv(){return new Qv}function Sv(a,b){return new Nv(b,a.D,a.ia,a.Ma,a.C)}var Tv=class{constructor(a,b,c,d){this.D=a;this.ia=b;this.Ma=c;this.C=d}};Tv.K=function(a){return new (a||Tv)(r(xu),r(kt),r(Wt),r(Zt))};Tv.na=Pd({ma:Tv,ba:Tv.K,fa:"root"});var Uv=class{attach(a){this.D=a;return a.attach(this)}detach(){let a=this.D;null!=a&&(this.D=null,a.detach())}},Vv=class extends Uv{constructor(a,b,c,d){super();this.component=a;this.yb=b;this.xa=c;this.C=d;this.Bo=void 0}},Wv=class extends Uv{constructor(a,b,c,d){super();this.eg=a;this.yb=b;this.context=c;this.xa=d}get origin(){return this.eg.Ub}attach(a,b=this.context){this.context=b;return super.attach(a)}detach(){this.context=void 0;return super.detach()}};function Xv(a,b){a.wp=b}
var Yv=class{constructor(){this.We=!1}jc(){return!!this.ce}attach(a){if(a instanceof Vv)return this.ce=a,this.ee(a);if(a instanceof Wv)return this.ce=a,this.df(a)}detach(){this.ce&&(this.ce=this.ce.D=null);this.ku()}dispose(){this.jc()&&this.detach();this.ku();this.We=!0}ku(){this.wp&&(this.wp(),this.wp=null)}};var Zv=class extends Yv{constructor(a,b,c,d,e){super();this.D=a;this.F=b;this.C=c;this.G=d;this.ia=e}ee(a){const b=kn(a.C||this.F,a.component);let c;a.yb?(c=tp(a.yb,b,a.yb.length,a.xa||a.yb.xa,a.Bo||void 0),Xv(this,()=>c.destroy())):(c=b.create(a.xa||this.G||bi),gp(this.C,c.C),Xv(this,()=>{0<this.C.ua&&this.C.kj(c.C);c.destroy()}));this.D.appendChild(this.J(c));this.ce=a;return c}df(a){let b=a.yb,c=b.xd(a.eg,a.context,{xa:a.xa});c.rootNodes.forEach(d=>this.D.appendChild(d));fn(c);Xv(this,()=>{let d=
b.indexOf(c);-1!==d&&b.remove(d)});this.ce=a;return c}dispose(){super.dispose();this.D.remove()}J(a){return a.C.rootNodes[0]}};var $v=class extends Yv{constructor(a,b,c){super();this.G=a;this.C=b;this.D=!1;this.iq=new Jk;this.ia=c}get Od(){return this.ce}set Od(a){if(!this.jc()||a||this.D)this.jc()&&super.detach(),a&&super.attach(a),this.ce=a||null}Fb(){this.D=!0}va(){super.dispose();this.F=this.ce=null}ee(a){a.D=this;const b=null!=a.yb?a.yb:this.C,c=kn(a.C||this.G,a.component),d=tp(b,c,b.length,a.xa||b.xa,a.Bo||void 0);b!==this.C&&this.J().appendChild(d.C.rootNodes[0]);Xv(this,()=>d.destroy());this.ce=a;this.F=d;this.iq.emit(d);
return d}df(a){a.D=this;const b=this.C.xd(a.eg,a.context,{xa:a.xa});Xv(this,()=>this.C.clear());this.ce=a;this.F=b;this.iq.emit(b);return b}J(){const a=this.C.element.R;return a.nodeType===a.ELEMENT_NODE?a:a.parentNode}};$v.K=function(a){return new (a||$v)(y(ci),y(np),y(kt))};$v.Fa=Uf({type:$v,ha:[["","cdkPortalOutlet",""]],inputs:{Od:[0,"cdkPortalOutlet","portal"]},outputs:{iq:"attached"},ib:["cdkPortalOutlet"],ka:!0,features:[Ep]});var aw=class{};aw.K=function(a){return new (a||aw)};aw.Ya=Sf({type:aw});
aw.Xa=Qd({});let bw=0;
var cw=class{constructor(a,b,c,d,e,f,g,h,m,p,u,w){this.C=a;this.P=b;this.G=c;this.oa=d;this.J=e;this.Ra=f;this.qa=g;this.ia=h;this.ja=m;this.N=p;this.O=u;this.T=w}create(a){const b=this.aa(),c=this.ca(b),d=this.ea(c);a=new Bu(a);a.direction=a.direction||this.ja.value;return new Fu(d,b,c,a,this.qa,this.J,this.ia,this.N,this.O,"NoopAnimations"===this.T,this.Ra.get(og))}position(){return this.oa}ca(a){const b=this.ia.createElement("div");b.id=`cdk-overlay-${bw++}`;b.classList.add("cdk-overlay-pane");a.appendChild(b);
return b}aa(){const a=this.ia.createElement("div");Yt(this.P).appendChild(a);return a}ea(a){this.F||(this.F=this.Ra.get(ip));return new Zv(a,this.G,this.F,this.Ra,this.ia)}};cw.K=function(a){return new (a||cw)(r(Au),r(Zt),r(ci),r(Tv),r(It),r(ai),r(Tk),r(kt),r(pu),r(Ft),r(Xt),r(qi,8))};cw.na=Pd({ma:cw,ba:cw.K,fa:"root"});const dw=[{bb:"start",cb:"bottom",Na:"start",Ua:"top"},{bb:"start",cb:"top",Na:"start",Ua:"bottom"},{bb:"end",cb:"top",Na:"end",Ua:"bottom"},{bb:"end",cb:"bottom",Na:"end",Ua:"top"}];var ew=new gf("cdk-connected-overlay-scroll-strategy",{fa:"root",ba:()=>{const a=of(cw);return()=>a.C.C()}}),fw=class{constructor(a){this.Ub=a}};fw.K=function(a){return new (a||fw)(y(wi))};fw.Fa=Uf({type:fw,ha:[["","cdk-overlay-origin",""],["","overlay-origin",""],["","cdkOverlayOrigin",""]],ib:["cdkOverlayOrigin"],ka:!0});
function gw(a){a.la?a.la.Aa.Vb=a.Vb:hw(a);a.la.jc()||a.la.attach(a.ua);a.Vb?a.F=a.la.bd().subscribe(b=>{a.bd.emit(b)}):a.F.unsubscribe();a.D.unsubscribe();0<a.Ij.nc.length&&(a.D=C(a.C.gb,Po(()=>0<a.Ij.nc.length)).subscribe(b=>{a.qa.run(()=>a.Ij.emit(b));0===a.Ij.nc.length&&a.D.unsubscribe()}))}
function hw(a){a.oe&&a.oe.length||(a.oe=dw);const b=a.la=a.N.create(a.ja());a.O=b.P.subscribe(()=>a.attach.emit());a.Dm=b.J.subscribe(()=>a.detach.emit());b.gf().subscribe(c=>{a.kx.next(c);27!==c.keyCode||a.yd||Cu(c)||(c.preventDefault(),a.P())});a.la.Pl().subscribe(c=>{const d=a.ta(),e=Tt(c);d&&(d===e||d.contains(e))||a.lx.next(c)})}
var iw=class{get offsetX(){return this.aa}set offsetX(a){this.aa=a;this.C&&this.G(this.C)}get offsetY(){return this.ca}set offsetY(a){this.ca=a;this.C&&this.G(this.C)}get Jh(){return this.T}set Jh(a){this.T=a}constructor(a,b,c,d,e){this.N=a;this.J=e;this.D=this.Dm=this.O=this.F=Si.EMPTY;this.T=!1;this.qa=of(Tk);this.Qy=0;this.push=this.nw=this.fw=this.Qw=this.Vb=this.yd=this.open=!1;this.bd=new Jk;this.Ij=new Jk;this.attach=new Jk;this.detach=new Jk;this.kx=new Jk;this.lx=new Jk;this.ua=new Wv(b,
c);this.ea=d;this.wc=this.ea()}get Xb(){return this.la}get dir(){return this.J?this.J.value:"ltr"}va(){this.O.unsubscribe();this.Dm.unsubscribe();this.F.unsubscribe();this.D.unsubscribe();this.la&&this.la.dispose()}uc(a){this.C&&(this.G(this.C),this.la.te({width:this.width,minWidth:this.minWidth,height:this.height,minHeight:this.minHeight}),a.origin&&this.open&&this.C.apply());a.open&&(this.open?gw(this):this.P())}ja(){var a=this.C=this.vc||this.oa();a=new Bu({direction:this.J,vc:a,wc:this.wc,Vb:this.Vb,
Jh:this.Jh});if(this.width||0===this.width)a.width=this.width;if(this.height||0===this.height)a.height=this.height;if(this.minWidth||0===this.minWidth)a.minWidth=this.minWidth;if(this.minHeight||0===this.minHeight)a.minHeight=this.minHeight;this.fc&&(a.fc=this.fc);this.hb&&(a.hb=this.hb);return a}G(a){const b=this.oe.map(c=>({bb:c.bb,cb:c.cb,Na:c.Na,Ua:c.Ua,offsetX:c.offsetX||this.offsetX,offsetY:c.offsetY||this.offsetY,hb:c.hb||void 0}));return Su(Qu(Mu(Ou(Pu(Nu(Lu(Iu(a,this.origin instanceof fw?
this.origin.Ub:this.origin),b),this.fw),this.push),this.nw),this.Qy),this.Qw),this.TE)}oa(){const a=Sv(this.N.position(),this.origin instanceof fw?this.origin.Ub:this.origin);this.G(a);return a}ta(){return this.origin instanceof fw?this.origin.Ub.R:this.origin instanceof wi?this.origin.R:"undefined"!==typeof Element&&this.origin instanceof Element?this.origin:null}P(){this.la&&this.la.detach();this.F.unsubscribe();this.D.unsubscribe()}};
iw.K=function(a){return new (a||iw)(y(cw),y(dr),y(np),y(ew),y(pu,8))};
iw.Fa=Uf({type:iw,ha:[["","cdk-connected-overlay",""],["","connected-overlay",""],["","cdkConnectedOverlay",""]],inputs:{origin:[0,"cdkConnectedOverlayOrigin","origin"],oe:[0,"cdkConnectedOverlayPositions","positions"],vc:[0,"cdkConnectedOverlayPositionStrategy","positionStrategy"],offsetX:[0,"cdkConnectedOverlayOffsetX","offsetX"],offsetY:[0,"cdkConnectedOverlayOffsetY","offsetY"],width:[0,"cdkConnectedOverlayWidth","width"],height:[0,"cdkConnectedOverlayHeight","height"],minWidth:[0,"cdkConnectedOverlayMinWidth",
"minWidth"],minHeight:[0,"cdkConnectedOverlayMinHeight","minHeight"],fc:[0,"cdkConnectedOverlayBackdropClass","backdropClass"],hb:[0,"cdkConnectedOverlayPanelClass","panelClass"],Qy:[0,"cdkConnectedOverlayViewportMargin","viewportMargin"],wc:[0,"cdkConnectedOverlayScrollStrategy","scrollStrategy"],open:[0,"cdkConnectedOverlayOpen","open"],yd:[0,"cdkConnectedOverlayDisableClose","disableClose"],TE:[0,"cdkConnectedOverlayTransformOriginOn","transformOriginSelector"],Vb:[2,"cdkConnectedOverlayHasBackdrop",
"hasBackdrop",As],Qw:[2,"cdkConnectedOverlayLockPosition","lockPosition",As],fw:[2,"cdkConnectedOverlayFlexibleDimensions","flexibleDimensions",As],nw:[2,"cdkConnectedOverlayGrowAfterOpen","growAfterOpen",As],push:[2,"cdkConnectedOverlayPush","push",As],Jh:[2,"cdkConnectedOverlayDisposeOnNavigation","disposeOnNavigation",As]},outputs:{bd:"backdropClick",Ij:"positionChange",attach:"attach",detach:"detach",kx:"overlayKeydown",lx:"overlayOutsideClick"},ib:["cdkConnectedOverlay"],ka:!0,features:[Lp,Hg]});
var jw={pa:ew,Bc:[cw],Gc:function(a){return()=>a.C.C()}};var kw=class{};kw.K=function(a){return new (a||kw)};kw.Ya=Sf({type:kw});kw.Xa=Qd({Wa:[cw,jw],imports:[qu,aw,zu,zu]});function lw(a,b){a.D.includes(b)||a.D.push(b)}var mw=class extends cw{constructor(){super(...arguments);this.D=[]}create(a){return super.create(Object.assign({},a,{hb:(null==a?0:a.hb)?Qs(a.hb).concat(this.D):this.D}))}};mw.K=(()=>{let a;return function(b){return(a||(a=Lh(mw)))(b||mw)}})();mw.na=Pd({ma:mw,ba:mw.K});var nw=class{constructor(a,b){a.tc||lw(b,"gmat-mdc-autocomplete")}};nw.K=function(a){return new (a||nw)(y(Ps),y(cw))};nw.Fa=Uf({type:nw,ha:[["input","matAutocomplete",""],["textarea","matAutocomplete",""]],features:[Lr([{pa:cw,Fc:mw}])]});var ow=class{constructor(a,b){this.F=a;this.ra=b;this.oo=()=>{};this.Or=()=>{}}setProperty(a,b){this.F.setProperty(this.ra.R,a,b)}Nj(a){this.Or=a}Ng(a){this.oo=a}ji(a){this.setProperty("disabled",a)}};ow.K=function(a){return new (a||ow)(y(zi),y(wi))};ow.Fa=Uf({type:ow});var pw=class extends ow{};pw.K=(()=>{let a;return function(b){return(a||(a=Lh(pw)))(b||pw)}})();pw.Fa=Uf({type:pw,features:[Ep]});var qw=new gf("");const sw={pa:qw,Ab:df(()=>rw),Eb:!0};var rw=class extends pw{ti(a){this.setProperty("checked",a)}};rw.K=(()=>{let a;return function(b){return(a||(a=Lh(rw)))(b||rw)}})();rw.Fa=Uf({type:rw,ha:[["input","type","checkbox","formControlName",""],["input","type","checkbox","formControl",""],["input","type","checkbox","ngModel",""]],Ia:function(a,b){a&1&&V("change",function(c){return b.oo(c.target.checked)})("blur",function(){return b.Or()})},features:[Lr([sw]),Ep]});var uw={pa:qw,Ab:df(()=>tw),Eb:!0},vw=new gf(""),tw=class extends ow{constructor(a,b,c){super(a,b);this.C=c;this.D=!1;null==this.C&&(this.C=!/android (\d+)/.test((lt?window.navigator.userAgent:"").toLowerCase()))}ti(a){this.setProperty("value",null==a?"":a)}Bp(a){(!this.C||this.C&&!this.D)&&this.oo(a)}Wz(){this.D=!0}Vz(a){this.D=!1;this.C&&this.oo(a)}};tw.K=function(a){return new (a||tw)(y(zi),y(wi),y(vw,8))};
tw.Fa=Uf({type:tw,ha:[["input","formControlName","",3,"type","checkbox"],["textarea","formControlName",""],["input","formControl","",3,"type","checkbox"],["textarea","formControl",""],["input","ngModel","",3,"type","checkbox"],["textarea","ngModel",""],["","ngDefaultControl",""]],Ia:function(a,b){a&1&&V("input",function(c){return b.Bp(c.target.value)})("blur",function(){return b.Or()})("compositionstart",function(){return b.Wz()})("compositionend",function(c){return b.Vz(c.target.value)})},features:[Lr([uw]),
Ep]});var ww=new gf(""),xw=new gf("");function yw(a){a=a.value;return null==a||("string"===typeof a||Array.isArray(a))&&0===a.length?{required:!0}:null}function zw(a){return null!=a}function Aw(a){return vn(a)?Xj(a):a}function Bw(a){let b={};a.forEach(c=>{b=null!=c?Object.assign({},b,c):b});return 0===Object.keys(b).length?null:b}function Cw(a,b){return b.map(c=>c(a))}function Dw(a){return a.map(b=>b.validate?c=>b.validate(c):b)}
function Ew(a){if(!a)return null;const b=a.filter(zw);return 0==b.length?null:function(c){return Bw(Cw(c,b))}}function Fw(a){return null!=a?Ew(Dw(a)):null}function Gw(a){if(!a)return null;const b=a.filter(zw);return 0==b.length?null:function(c){c=Cw(c,b).map(Aw);return C(ok(c),Cj(Bw))}}function Hw(a){return null!=a?Gw(Dw(a)):null}function Iw(a,b){return null===a?[b]:Array.isArray(a)?[...a,b]:[a,b]};var Jw=class{constructor(){this.Ze=[];this.Ef=[];this.F=[]}get value(){return this.control?this.control.value:null}get valid(){return this.control?this.control.valid:null}get Vn(){return this.control?this.control.Vn:null}get pending(){return this.control?this.control.pending:null}get disabled(){return this.control?this.control.disabled:null}get enabled(){return this.control?this.control.enabled:null}get errors(){return this.control?this.control.errors:null}get pe(){return this.control?this.control.pe:
null}get sc(){return this.control?this.control.sc:null}get Vc(){return this.control?this.control.Vc:null}get status(){return this.control?this.control.status:null}get Ts(){return this.control?this.control.Ts:null}get gm(){return this.control?this.control.gm:null}get dh(){return this.control?this.control.dh:null}get path(){return null}Au(a){this.Ze=a||[];this.Bm=Fw(this.Ze)}zu(a){this.Ef=a||[];this.Am=Hw(this.Ef)}get bh(){return this.Bm||null}get cf(){return this.Am||null}G(a){this.F.push(a)}J(){this.F.forEach(a=>
a());this.F=[]}reset(a){this.control&&this.control.reset(a)}Fe(a,b){return this.control?this.control.Fe(a,b):!1}getError(a,b){return this.control?this.control.getError(a,b):null}};var Kw=class extends Jw{constructor(){super(...arguments);this.C=this.name=this.Sa=null}};var Lw=class extends Jw{get path(){return null}};var Mw=class{constructor(a){this.C=a}get SC(){let a,b;return!!(null==(a=this.C)?0:null==(b=a.control)?0:b.Vc)}get TC(){let a,b;return!!(null==(a=this.C)?0:null==(b=a.control)?0:b.Ts)}get QC(){let a,b;return!!(null==(a=this.C)?0:null==(b=a.control)?0:b.pe)}get Yn(){let a,b;return!!(null==(a=this.C)?0:null==(b=a.control)?0:b.sc)}get tr(){let a,b;return!!(null==(a=this.C)?0:null==(b=a.control)?0:b.valid)}get NC(){let a,b;return!!(null==(a=this.C)?0:null==(b=a.control)?0:b.Vn)}get PC(){let a,b;return!!(null==
(a=this.C)?0:null==(b=a.control)?0:b.pending)}};Object.assign({},{"[class.ng-untouched]":"isUntouched","[class.ng-touched]":"isTouched","[class.ng-pristine]":"isPristine","[class.ng-dirty]":"isDirty","[class.ng-valid]":"isValid","[class.ng-invalid]":"isInvalid","[class.ng-pending]":"isPending"},{"[class.ng-submitted]":"isSubmitted"});var Nw=class extends Mw{};Nw.K=function(a){return new (a||Nw)(y(Kw,2))};
Nw.Fa=Uf({type:Nw,ha:[["","formControlName",""],["","ngModel",""],["","formControl",""]],Za:14,Ia:function(a,b){a&2&&Wp("ng-untouched",b.TC)("ng-touched",b.SC)("ng-pristine",b.QC)("ng-dirty",b.Yn)("ng-valid",b.tr)("ng-invalid",b.NC)("ng-pending",b.PC)},features:[Ep]});var Ow=new gf("CallSetDisabledState",{fa:"root",ba:()=>"whenDisabledForLegacyCode"});function Pw(a,b,c="whenDisabledForLegacyCode"){Qw(a,b);b.C.ti(a.value);if(a.disabled||"always"===c){let d,e;null==(e=(d=b.C).ji)||e.call(d,a.disabled)}Rw(a,b);Sw(a,b);Tw(a,b);Uw(a,b)}function Vw(a,b){const c=()=>{};b.C&&(b.C.Ng(c),b.C.Nj(c));Ww(a,b);a&&(b.J(),a.Fk(()=>{}))}function Xw(a,b){a.forEach(c=>{c.hs&&c.hs(b)})}function Uw(a,b){if(b.C.ji){const c=d=>{b.C.ji(d)};a.Km.push(c);b.G(()=>{a.fB(c)})}}
function Qw(a,b){var c=a.Ze;null!==b.bh?Yw(a,Iw(c,b.bh)):"function"===typeof c&&Yw(a,[c]);c=a.Ef;null!==b.cf?Zw(a,Iw(c,b.cf)):"function"===typeof c&&Zw(a,[c]);c=()=>$w(a);Xw(b.Ze,c);Xw(b.Ef,c)}
function Ww(a,b){let c=!1;if(null!==a){if(null!==b.bh){var d=a.Ze;if(Array.isArray(d)&&0<d.length){var e=d.filter(f=>f!==b.bh);e.length!==d.length&&(c=!0,a.mp(e))}}null!==b.cf&&(d=a.Ef,Array.isArray(d)&&0<d.length&&(e=d.filter(f=>f!==b.cf),e.length!==d.length&&(c=!0,a.lp(e))))}a=()=>{};Xw(b.Ze,a);Xw(b.Ef,a);return c}function Rw(a,b){b.C.Ng(c=>{a.vh=c;a.uh=!0;a.Om=!0;"change"===a.uf&&ax(a,b)})}function Tw(a,b){b.C.Nj(()=>{a.Op=!0;"blur"===a.uf&&a.uh&&ax(a,b);"submit"!==a.uf&&bx(a)})}
function ax(a,b){a.Om&&cx(a);a.setValue(a.vh,{Zv:!1});var c=a.vh;b.D=c;b.update.emit(c);a.uh=!1}function Sw(a,b){const c=(d,e)=>{b.C.ti(d);e&&(b.D=d,b.update.emit(d))};a.Ng(c);b.G(()=>{a.eB(c)})}function dx(a,b){a.Vp();b.forEach(c=>{const d=c.control;if("submit"===d.uf&&d.uh){var e=d.vh;c.D=e;c.update.emit(e);d.uh=!1}})}function ex(a){if(!a)return null;let b=void 0,c=void 0,d=void 0;a.forEach(e=>{e.constructor===tw?b=e:Object.getPrototypeOf(e.constructor)===pw?c=e:d=e});return d?d:c?c:b?b:null};var fx=class{},gx=class extends fx{constructor(a,b){super();this.value=a;this.source=b}},hx=class extends fx{constructor(a,b){super();this.pe=a;this.source=b}},ix=class extends fx{constructor(a,b){super();this.Vc=a;this.source=b}},jx=class extends fx{constructor(a,b){super();this.status=a;this.source=b}},kx=class extends fx{constructor(a){super();this.source=a}},lx=class extends fx{constructor(a){super();this.source=a}};function mx(a){return null!=a&&!Array.isArray(a)&&"object"===typeof a}
function nx(a,b){a.Ue((c,d)=>{if(void 0===b[d])throw new ae(1002,"");})}function Yw(a,b){a.mp(b)}function Zw(a,b){a.lp(b)}function $w(a,b={}){a.UA();a.bq();if(a.enabled){var c=a.Qz();a.errors=a.SA();a.status=a.Dt();"VALID"!==a.status&&"PENDING"!==a.status||a.RA(c,b.wb)}let d;c=null!=(d=b.Zf)?d:a;!1!==b.wb&&(a.Bb.next(new gx(a.value,c)),a.Bb.next(new jx(a.status,c)),a.dh.emit(a.value),a.gm.emit(a.status));a.Sa&&!b.Wb&&$w(a.Sa,Object.assign({},b,{Zf:c}))}
function bx(a,b={}){const c=!1===a.Vc;a.Vc=!0;let d;const e=null!=(d=b.Zf)?d:a;a.Sa&&!b.Wb&&bx(a.Sa,Object.assign({},b,{Zf:e}));c&&!1!==b.wb&&a.Bb.next(new ix(!0,e))}function cx(a,b={}){const c=!0===a.pe;a.pe=!1;let d;const e=null!=(d=b.Zf)?d:a;a.Sa&&!b.Wb&&cx(a.Sa,Object.assign({},b,{Zf:e}));c&&!1!==b.wb&&a.Bb.next(new hx(!1,e))}
function ox(a,b={}){const c=!0===a.Vc;a.Vc=!1;a.Op=!1;let d;const e=null!=(d=b.Zf)?d:a;a.Ue(f=>{ox(f,{Wb:!0,wb:b.wb,Zf:e})});a.Sa&&!b.Wb&&a.Sa.Wm(b,e);c&&!1!==b.wb&&a.Bb.next(new ix(!1,e))}function px(a,b={}){const c=!1===a.pe;a.pe=!0;a.Om=!1;let d;const e=null!=(d=b.Zf)?d:a;a.Ue(f=>{px(f,{Wb:!0,wb:b.wb})});a.Sa&&!b.Wb&&a.Sa.Vm(b,e);c&&!1!==b.wb&&a.Bb.next(new hx(!0,e))}function qx(a,b,c={}){a.errors=b;a.Iu(!1!==c.wb,a,c.gE)}
var rx=class{constructor(a,b){this.Om=!1;this.uk=null;this.Op=!1;this.Ri=()=>{};this.Sa=null;this.pe=!0;this.Vc=!1;this.Bb=new hj;this.ic=fj(this.Bb);this.Km=[];this.mp(a);this.lp(b)}get bh(){return this.Bm}set bh(a){this.Ze=this.Bm=a}get cf(){return this.Am}set cf(a){this.Ef=this.Am=a}get parent(){return this.Sa}get valid(){return"VALID"===this.status}get Vn(){return"INVALID"===this.status}get pending(){return"PENDING"==this.status}get disabled(){return"DISABLED"===this.status}get enabled(){return"DISABLED"!==
this.status}get sc(){return!this.pe}get Ts(){return!this.Vc}get uf(){return this.aq?this.aq:this.parent?this.parent.uf:"change"}disable(a={}){const b=this.uu(a.Wb);this.status="DISABLED";this.errors=null;this.Ue(e=>{e.disable(Object.assign({},a,{Wb:!0}))});this.bq();let c;const d=null!=(c=a.Zf)?c:this;!1!==a.wb&&(this.Bb.next(new gx(this.value,d)),this.Bb.next(new jx(this.status,d)),this.dh.emit(this.value),this.gm.emit(this.status));this.Hu(Object.assign({},a,{hy:b}),this);this.Km.forEach(e=>e(!0))}enable(a=
{}){const b=this.uu(a.Wb);this.status="VALID";this.Ue(c=>{c.enable(Object.assign({},a,{Wb:!0}))});$w(this,{Wb:!0,wb:a.wb});this.Hu(Object.assign({},a,{hy:b}),this);this.Km.forEach(c=>c(!1))}Hu(a,b){this.Sa&&!a.Wb&&($w(this.Sa,a),a.hy||this.Sa.Vm({},b),this.Sa.Wm({},b))}Ku(a={wb:!0}){this.Ue(b=>b.Ku(a));$w(this,{Wb:!0,wb:a.wb})}UA(){this.status=this.jp()?"DISABLED":"VALID"}SA(){return this.bh?this.bh(this):null}RA(a,b){this.cf&&(this.status="PENDING",this.uk={wb:!1!==b},this.Ct=Aw(this.cf(this)).subscribe(c=>
{this.uk=null;qx(this,c,{wb:b,gE:a})}))}Qz(){if(this.Ct){this.Ct.unsubscribe();let a,b;const c=null!=(b=null==(a=this.uk)?void 0:a.wb)?b:!1;this.uk=null;return c}return!1}get(a){if(null==a)return null;Array.isArray(a)||(a=a.split("."));return 0===a.length?null:a.reduce((b,c)=>b&&b.Nt(c),this)}getError(a,b){return(b=b?this.get(b):this)&&b.errors?b.errors[a]:null}Fe(a,b){return!!this.getError(a,b)}get root(){let a=this;for(;a.Sa;)a=a.Sa;return a}Iu(a,b,c){this.status=this.Dt();a&&this.gm.emit(this.status);
(a||c)&&this.Bb.next(new jx(this.status,b));this.Sa&&this.Sa.Iu(a,b,c)}gu(){this.dh=new Jk;this.gm=new Jk}Dt(){return this.jp()?"DISABLED":this.errors?"INVALID":this.uk||this.Bt("PENDING")?"PENDING":this.Bt("INVALID")?"INVALID":"VALID"}Bt(a){return this.wm(b=>b.status===a)}At(){return this.wm(a=>a.sc)}Pz(){return this.wm(a=>a.Vc)}Vm(a,b){const c=!this.At(),d=this.pe!==c;this.pe=c;this.Sa&&!a.Wb&&this.Sa.Vm(a,b);d&&this.Bb.next(new hx(this.pe,b))}Wm(a={},b){this.Vc=this.Pz();this.Bb.next(new ix(this.Vc,
b));this.Sa&&!a.Wb&&this.Sa.Wm(a,b)}Fk(a){this.Ri=a}Rm(a){mx(a)&&null!=a.uf&&(this.aq=a.uf)}uu(a){const b=this.Sa&&this.Sa.sc;return!a&&!!b&&!this.Sa.At()}Nt(){return null}mp(a){a=this.Ze=Array.isArray(a)?a.slice():a;this.Bm=Array.isArray(a)?Fw(a):a||null}lp(a){a=this.Ef=Array.isArray(a)?a.slice():a;this.Am=Array.isArray(a)?Hw(a):a||null}};var sx=class extends rx{constructor(a,b){super((mx(a)?a.cF:a)||null,(mx(a)?a.nB:b)||null);this.controls={};this.gu();this.Rm(a);this.WA();$w(this,{Wb:!0,wb:!!this.cf})}contains(a){return this.controls.hasOwnProperty(a)&&this.controls[a].enabled}setValue(a,b={}){nx(this,a);Object.keys(a).forEach(c=>{const d=this.controls;if(!Object.keys(d).length)throw new ae(1E3,"");if(!d[c])throw new ae(1001,"");this.controls[c].setValue(a[c],{Wb:!0,wb:b.wb})});$w(this,b)}reset(a={},b={}){this.Ue((c,d)=>{c.reset(a?
a[d]:null,{Wb:!0,wb:b.wb})});this.Vm(b,this);this.Wm(b,this);$w(this,b)}Vp(){let a=this.vu(!1,(b,c)=>c.Vp()?!0:b);a&&$w(this,{Wb:!0});return a}Ue(a){Object.keys(this.controls).forEach(b=>{const c=this.controls[b];c&&a(c,b)})}WA(){this.Ue(a=>{a.Sa=this;a.Fk(this.Ri)})}bq(){this.value=this.PA()}wm(a){for(const [b,c]of Object.entries(this.controls)){const d=c;if(this.contains(b)&&a(d))return!0}return!1}PA(){return this.vu({},(a,b,c)=>{if(b.enabled||this.disabled)a[c]=b.value;return a})}vu(a,b){let c=
a;this.Ue((d,e)=>{c=b(c,d,e)});return c}jp(){for(const a of Object.keys(this.controls))if(this.controls[a].enabled)return!1;return 0<Object.keys(this.controls).length||this.disabled}Nt(a){return this.controls.hasOwnProperty(a)?this.controls[a]:null}};const ux={pa:Lw,Ab:df(()=>tx)};Promise.resolve();
var tx=class extends Lw{constructor(a,b,c){super();this.dn=c;this.Zj=!1;this.Zz=new Set;this.Ml=new Jk;this.form=new sx(Fw(a),Hw(b))}pd(){this.Rm()}get control(){return this.form}get path(){return[]}get controls(){return this.form.controls}setValue(a){this.control.setValue(a)}Nr(a){this.Zj=!0;dx(this.form,this.Zz);this.Ml.emit(a);let b;return"dialog"===(null==a?void 0:null==(b=a.target)?void 0:b.method)}Lr(){this.os()}os(){this.form.reset(void 0);this.Zj=!1}Rm(){this.options&&null!=this.options.uf&&
(this.form.aq=this.options.uf)}DF(a){a.pop();return a.length?this.form.get(a):this.form}};tx.K=function(a){return new (a||tx)(y(ww,10),y(xw,10),y(Ow,8))};tx.Fa=Uf({type:tx,ha:[["form",3,"ngNoForm","",3,"formGroup",""],["ng-form"],["","ngForm",""]],Ia:function(a,b){a&1&&V("submit",function(c){return b.Nr(c)})("reset",function(){return b.Lr()})},inputs:{options:[0,"ngFormOptions","options"]},outputs:{Ml:"ngSubmit"},ib:["ngForm"],features:[Lr([ux]),Ep]});function vx(a){return"object"===typeof a&&null!==a&&2===Object.keys(a).length&&"value"in a&&"disabled"in a}
var wx=class extends rx{constructor(a=null){super((mx()?(void 0).cF:void 0)||null,(mx()?(void 0).nB:void 0)||null);this.defaultValue=null;this.C=[];this.uh=!1;this.D(a);this.Rm(void 0);this.gu();$w(this,{Wb:!0,wb:!!this.cf});mx()&&((void 0).qG||(void 0).hG)&&(vx(a)?this.defaultValue=a.value:this.defaultValue=a)}setValue(a,b={}){this.value=this.vh=a;this.C.length&&!1!==b.Zv&&this.C.forEach(c=>c(this.value,!1!==b.WF));$w(this,b)}reset(a=this.defaultValue,b={}){this.D(a);px(this,b);ox(this,b);this.setValue(this.value,
b);this.uh=!1}bq(){}wm(){return!1}jp(){return this.disabled}Ng(a){this.C.push(a)}eB(a){var b=this.C;a=b.indexOf(a);-1<a&&b.splice(a,1)}fB(a){var b=this.Km;a=b.indexOf(a);-1<a&&b.splice(a,1)}Ue(){}Vp(){return"submit"===this.uf&&(this.Om&&cx(this),this.Op&&bx(this),this.uh)?(this.setValue(this.vh,{Wb:!0,Zv:!1}),!0):!1}D(a){vx(a)?(this.value=this.vh=a.value,a.disabled?this.disable({Wb:!0,wb:!1}):this.enable({Wb:!0,wb:!1})):this.value=this.vh=a}};Promise.resolve();var xx=new gf("");const zx={pa:Kw,Ab:df(()=>yx)};var yx=class extends Kw{set qr(a){}constructor(a,b,c,d,e){super();this.dn=e;this.update=new Jk;this.O=!1;this.Au(a);this.zu(b);this.C=ex(c)}uc(a){if(this.N(a)){var b=a.form.xx;b&&Vw(b,this);Pw(this.form,this,this.dn);$w(this.form,{wb:!1})}b=this.D;a.hasOwnProperty("model")?(a=a.model,a=a.Bn?!0:!Object.is(b,a.xq)):a=!1;a&&(this.form.setValue(this.model),this.D=this.model)}va(){this.form&&Vw(this.form,this)}get path(){return[]}get control(){return this.form}N(a){return a.hasOwnProperty("form")}};
yx.K=function(a){return new (a||yx)(y(ww,10),y(xw,10),y(qw,10),y(xx,8),y(Ow,8))};yx.Fa=Uf({type:yx,ha:[["","formControl",""]],inputs:{form:[0,"formControl","form"],qr:[0,"disabled","isDisabled"],model:[0,"ngModel","model"]},outputs:{update:"ngModelChange"},ib:["ngForm"],features:[Lr([zx]),Ep,Hg]});const Bx={pa:Lw,Ab:df(()=>Ax)};
var Ax=class extends Lw{constructor(a,b,c){super();this.dn=c;this.Zj=!1;this.Ri=()=>this.Ju();this.Uv=[];this.form=null;this.Ml=new Jk;this.Au(a);this.zu(b)}uc(a){a.hasOwnProperty("form")&&(this.hB(),this.Ju(),this.gB(),this.Jm=this.form)}va(){this.form&&(Ww(this.form,this),this.form.Ri===this.Ri&&this.form.Fk(()=>{}))}get control(){return this.form}get path(){return[]}Nr(a){this.Zj=!0;dx(this.form,this.Uv);this.Ml.emit(a);this.form.Bb.next(new kx(this.control));let b;return"dialog"===(null==a?void 0:
null==(b=a.target)?void 0:b.method)}Lr(){this.os()}os(){this.form.reset(void 0);this.Zj=!1;this.form.Bb.next(new lx(this.form))}Ju(){this.Uv.forEach(a=>{const b=a.control,c=this.form.get(a.path);b!==c&&(Vw(b||null,a),c instanceof wx&&(Pw(c,a,this.dn),a.control=c))});this.form.Ku({wb:!1})}JF(a){const b=this.form.get(a.path);Qw(b,a);$w(b,{wb:!1})}BF(a){if(this.form){const b=this.form.get(a.path);b&&Ww(b,a)&&$w(b,{wb:!1})}}gB(){this.form.Fk(this.Ri);this.Jm&&this.Jm.Fk(()=>{})}hB(){Qw(this.form,this);
this.Jm&&Ww(this.Jm,this)}AF(){}};Ax.K=function(a){return new (a||Ax)(y(ww,10),y(xw,10),y(Ow,8))};Ax.Fa=Uf({type:Ax,ha:[["","formGroup",""]],Ia:function(a,b){a&1&&V("submit",function(c){return b.Nr(c)})("reset",function(){return b.Lr()})},inputs:{form:[0,"formGroup","form"]},outputs:{Ml:"ngSubmit"},ib:["ngForm"],features:[Lr([Bx]),Ep,Hg]});var Cx=class{};Cx.K=function(a){return new (a||Cx)};Cx.Ya=Sf({type:Cx});Cx.Xa=Qd({});var Dx=class{};Dx.K=function(a){return new (a||Dx)};Dx.Ya=Sf({type:Dx});Dx.Xa=Qd({imports:[Cx]});function Ex(a){return 0===a.buttons||0===a.detail}function Fx(a){a=a.touches&&a.touches[0]||a.changedTouches&&a.changedTouches[0];return!!a&&-1===a.identifier&&(null==a.radiusX||1===a.radiusX)&&(null==a.radiusY||1===a.radiusY)};var Gx=new gf("cdk-input-modality-detector-options"),Hx={zC:[18,17,224,91,16]};const Ix=Lt({passive:!0,capture:!0});
var Jx=class{get O(){return this.C.value}constructor(a,b,c,d){this.Ma=a;this.D=null;this.C=new jj(null);this.F=0;this.G=e=>{let f,g;(null==(f=this.Si)?0:null==(g=f.zC)?0:g.some(h=>h===e.keyCode))||(this.C.next("keyboard"),this.D=Tt(e))};this.Mm=e=>{650>Date.now()-this.F||(this.C.next(Ex(e)?"keyboard":"mouse"),this.D=Tt(e))};this.J=e=>{Fx(e)?this.C.next("keyboard"):(this.F=Date.now(),this.C.next("touch"),this.D=Tt(e))};this.Si=Object.assign({},Hx,d);this.N=C(this.C,Ho());C(this.N,fo());a.isBrowser&&
Qk(b,()=>{c.addEventListener("keydown",this.G,Ix);c.addEventListener("mousedown",this.Mm,Ix);c.addEventListener("touchstart",this.J,Ix)})}va(){this.C.complete();this.Ma.isBrowser&&(document.removeEventListener("keydown",this.G,Ix),document.removeEventListener("mousedown",this.Mm,Ix),document.removeEventListener("touchstart",this.J,Ix))}};Jx.K=function(a){return new (a||Jx)(r(Wt),r(Tk),r(kt),r(Gx,8))};Jx.na=Pd({ma:Jx,ba:Jx.K,fa:"root"});var Kx=new gf("cdk-focus-monitor-default-options");const Lx=Lt({passive:!0,capture:!0});function Mx(a,b,c=!1){b=Ts(b);if(!a.Ma.isBrowser||1!==b.nodeType)return oj();const d=Rt(b)||a.G(),e=a.C.get(b);if(e)return c&&(e.gn=!0),e.Vg;c={gn:c,Vg:new hj,rootNode:d};a.C.set(b,c);a.Va(c);return c.Vg}function Nx(a,b){b=Ts(b);const c=a.C.get(b);c&&(c.Vg.complete(),a.P(b),a.C.delete(b),a.fb(c))}
function Ox(a,b,c,d){b=Ts(b);b===a.G().activeElement?a.Ja(b).forEach(([e,f])=>a.oa(e,c,f)):(a.ua(c),"function"===typeof b.focus&&b.focus(d))}
var Px=class{constructor(a,b,c,d,e){this.qa=a;this.Ma=b;this.J=c;this.F=null;this.O=this.T=!1;this.C=new Map;this.ja=0;this.D=new Map;this.Ea=()=>{this.T=!0;this.lb=window.setTimeout(()=>this.T=!1)};this.wa=new hj;this.N=f=>{for(var g=Tt(f);g;g=g.parentElement)"focus"===f.type?this.Np(f,g):this.Ck(f,g)};this.ia=d;this.aa=(null==e?void 0:e.PF)||0}va(){this.C.forEach((a,b)=>Nx(this,b))}G(){return this.ia||document}Qa(a){return this.F?this.O?this.gb(a)?"touch":"program":this.F:this.T&&this.ea?this.ea:
a&&this.Ta(a)?"mouse":"program"}gb(a){return 1===this.aa||!(null==a||!a.contains(this.J.D))}P(a,b){a.classList.toggle("cdk-focused",!!b);a.classList.toggle("cdk-touch-focused","touch"===b);a.classList.toggle("cdk-keyboard-focused","keyboard"===b);a.classList.toggle("cdk-mouse-focused","mouse"===b);a.classList.toggle("cdk-program-focused","program"===b)}ua(a,b=!1){Qk(this.qa,()=>{this.F=a;this.O="touch"===a&&b;0===this.aa&&(clearTimeout(this.ta),this.ta=setTimeout(()=>this.F=null,this.O?650:1))})}Np(a,
b){const c=this.C.get(b);a=Tt(a);c&&(c.gn||b===a)&&this.oa(b,this.Qa(a),c)}Ck(a,b){const c=this.C.get(b);!c||c.gn&&a.relatedTarget instanceof Node&&b.contains(a.relatedTarget)||(this.P(b),this.ca(c,null))}ca(a,b){a.Vg.nc.length&&this.qa.run(()=>a.Vg.next(b))}Va(a){if(this.Ma.isBrowser){var b=a.rootNode;(a=this.D.get(b)||0)||Qk(this.qa,()=>{b.addEventListener("focus",this.N,Lx);b.addEventListener("blur",this.N,Lx)});this.D.set(b,a+1);1===++this.ja&&(Qk(this.qa,()=>{(this.G().defaultView||window).addEventListener("focus",
this.Ea)}),C(this.J.N,Oo(this.wa)).subscribe(c=>{this.ua(c,!0)}))}}fb(a){a=a.rootNode;if(this.D.has(a)){const b=this.D.get(a);1<b?this.D.set(a,b-1):(a.removeEventListener("focus",this.N,Lx),a.removeEventListener("blur",this.N,Lx),this.D.delete(a))}--this.ja||((this.G().defaultView||window).removeEventListener("focus",this.Ea),this.wa.next(),clearTimeout(this.lb),clearTimeout(this.ta))}oa(a,b,c){this.P(a,b);this.ca(c,b);this.ea=b}Ja(a){const b=[];this.C.forEach((c,d)=>{(d===a||c.gn&&d.contains(a))&&
b.push([d,c])});return b}Ta(a){const b=this.J.D;if("mouse"!==this.J.O||!b||b===a||"INPUT"!==a.nodeName&&"TEXTAREA"!==a.nodeName||a.disabled)return!1;if(a=a.labels)for(let c=0;c<a.length;c++)if(a[c].contains(b))return!0;return!1}};Px.K=function(a){return new (a||Px)(r(Tk),r(Wt),r(Jx),r(kt,8),r(Kx,8))};Px.na=Pd({ma:Px,ba:Px.K,fa:"root"});function Qx(a,b){if(!a.Ma.isBrowser)return!1;try{var c=(b.ownerDocument&&b.ownerDocument.defaultView||window).frameElement}catch(f){c=null}if(c&&(-1===Rx(c)||!a.isVisible(c)))return!1;c=b.nodeName.toLowerCase();let d=Rx(b);if(b.hasAttribute("contenteditable"))a=-1!==d;else{var e;if(!(e="iframe"===c||"object"===c)&&(e=a.Ma.F&&a.Ma.C)){e=b.nodeName.toLowerCase();let f="input"===e&&b.type;e=!("text"===f||"password"===f||"select"===e||"textarea"===e)}a=e?!1:"audio"===c?b.hasAttribute("controls")?-1!==
d:!1:"video"===c?-1===d?!1:null!==d?!0:a.Ma.N||b.hasAttribute("controls"):0<=b.tabIndex}return a}function Sx(a,b){if("input"==b.nodeName.toLowerCase()&&"hidden"==b.type)var c=!1;else c=b.nodeName.toLowerCase(),c="input"===c||"select"===c||"button"===c||"textarea"===c||"a"==b.nodeName.toLowerCase()&&b.hasAttribute("href")||b.hasAttribute("contenteditable")||Tx(b);return c&&!a.qr(b)&&a.isVisible(b)}
var Ux=class{constructor(a){this.Ma=a}qr(a){return a.hasAttribute("disabled")}isVisible(a){return!!(a.offsetWidth||a.offsetHeight||"function"===typeof a.getClientRects&&a.getClientRects().length)&&"visible"===getComputedStyle(a).visibility}};Ux.K=function(a){return new (a||Ux)(r(Wt))};Ux.na=Pd({ma:Ux,ba:Ux.K,fa:"root"});function Tx(a){if(!a.hasAttribute("tabindex")||void 0===a.tabIndex)return!1;a=a.getAttribute("tabindex");return!(!a||isNaN(parseInt(a,10)))}
function Rx(a){if(!Tx(a))return null;a=parseInt(a.getAttribute("tabindex")||"",10);return isNaN(a)?-1:a};function Vx(a){(a=a.ja("start"))&&a.focus(void 0);return!!a}function Wx(a){a.P||(Qk(a.qa,()=>{a.F||(a.F=a.ca(),a.F.addEventListener("focus",a.ta));a.D||(a.D=a.ca(),a.D.addEventListener("focus",a.oa))}),a.C.parentNode&&(a.C.parentNode.insertBefore(a.F,a.C),a.C.parentNode.insertBefore(a.D,a.C.nextSibling),a.P=!0))}
function Xx(a){return new Promise(b=>{a.ua(()=>{var c;(c=a.C.querySelector("[cdk-focus-initial], [cdkFocusInitial]"))?Sx(a.J,c)?(c.focus(void 0),c=!0):(c=a.T(c),null==c||c.focus(void 0),c=!!c):c=Vx(a);return b(c)})})}function Yx(a,b){a.F&&a.D&&(a.N(b,a.F),a.N(b,a.D))}
var Zx=class{get enabled(){return this.G}set enabled(a){this.G=a;this.F&&this.D&&(this.N(a,this.F),this.N(a,this.D))}constructor(a,b,c,d,e=!1,f){this.C=a;this.J=b;this.qa=c;this.ia=d;this.Ra=f;this.P=!1;this.ta=()=>{const g=this.ja("end");g&&g.focus(void 0);return!!g};this.oa=()=>Vx(this);this.G=!0;e||Wx(this)}destroy(){const a=this.F,b=this.D;a&&(a.removeEventListener("focus",this.ta),a.remove());b&&(b.removeEventListener("focus",this.oa),b.remove());this.F=this.D=null;this.P=!1}ja(a){const b=this.C.querySelectorAll(`[cdk-focus-region-${a}], `+
`[cdkFocusRegion${a}], `+`[cdk-focus-${a}]`);return"start"==a?b.length?b[0]:this.T(this.C):b.length?b[b.length-1]:this.ea(this.C)}jc(){return this.P}T(a){if(Sx(this.J,a)&&Qx(this.J,a))return a;a=a.children;for(let b=0;b<a.length;b++){const c=a[b].nodeType===this.ia.ELEMENT_NODE?this.T(a[b]):null;if(c)return c}return null}ea(a){if(Sx(this.J,a)&&Qx(this.J,a))return a;a=a.children;for(let b=a.length-1;0<=b;b--){const c=a[b].nodeType===this.ia.ELEMENT_NODE?this.ea(a[b]):null;if(c)return c}return null}ca(){const a=
this.ia.createElement("div");this.N(this.G,a);a.classList.add("cdk-visually-hidden");a.classList.add("cdk-focus-trap-anchor");a.setAttribute("aria-hidden","true");return a}N(a,b){a?b.setAttribute("tabindex","0"):b.removeAttribute("tabindex")}ua(a){this.qa.isStable?this.Ra?el(a,{xa:this.Ra}):a():C(this.qa.qo,io()).subscribe(a)}},$x=class{constructor(a,b,c){this.C=a;this.qa=b;this.Ra=of(ai);this.ia=c}create(a,b=!1){return new Zx(a,this.C,this.qa,this.ia,b,this.Ra)}};
$x.K=function(a){return new (a||$x)(r(Ux),r(Tk),r(kt))};$x.na=Pd({ma:$x,ba:$x.K,fa:"root"});const ay=new Set;let by;function cy(a,b){if(a.Ma.F||a.Ma.G){var c=a.D;if(!ay.has(b))try{by||(by=document.createElement("style"),c&&by.setAttribute("nonce",c),by.setAttribute("type","text/css"),document.head.appendChild(by)),by.sheet&&(by.sheet.insertRule(`@media ${b} {body{ }}`,0),ay.add(b))}catch(d){console.error(d)}}return a.C(b)}var ey=class{constructor(a,b){this.Ma=a;this.D=b;this.C=this.Ma.isBrowser&&window.matchMedia?window.matchMedia.bind(window):dy}};
ey.K=function(a){return new (a||ey)(r(Wt),r(ri,8))};ey.na=Pd({ma:ey,ba:ey.K,fa:"root"});function dy(a){return{matches:"all"===a||""===a,media:a,addListener:()=>{},removeListener:()=>{}}};var gy=class{constructor(a,b){this.F=a;this.zb=b;this.D=new Map;this.C=new hj}va(){this.C.next();this.C.complete()}observe(a){a=fy(Qs(a)).map(b=>this.G(b).observable);a=Uj(a);a=mk(C(a,io()),C(a,Ho(),Rn(0)));return C(a,Cj(b=>{const c={matches:!1,bv:{}};b.forEach(({matches:d,query:e})=>{c.matches=c.matches||d;c.bv[e]=d});return c}))}G(a){if(this.D.has(a))return this.D.get(a);const b=cy(this.F,a),c={observable:C(new bj(d=>{const e=f=>this.zb.run(()=>d.next(f));b.addListener(e);return()=>{b.removeListener(e)}}),
Ko(b),Cj(({matches:d})=>({query:a,matches:d})),Oo(this.C)),lG:b};this.D.set(a,c);return c}};gy.K=function(a){return new (a||gy)(r(ey),r(Tk))};gy.na=Pd({ma:gy,ba:gy.K,fa:"root"});function fy(a){return a.map(b=>b.split(",")).reduce((b,c)=>b.concat(c)).map(b=>b.trim())};function hy(a){if(!a.Ma.isBrowser)return 0;const b=a.ia.createElement("div");b.style.backgroundColor="rgb(1,2,3)";b.style.position="absolute";a.ia.body.appendChild(b);a=(a=a.ia.defaultView||window)&&a.getComputedStyle?a.getComputedStyle(b):null;a=(a&&a.backgroundColor||"").replace(/ /g,"");b.remove();switch(a){case "rgb(0,0,0)":case "rgb(45,50,54)":case "rgb(32,32,32)":return 2;case "rgb(255,255,255)":case "rgb(255,250,239)":return 1}return 0}
var iy=class{constructor(a,b){this.Ma=a;this.ia=b;this.F=of(gy).observe("(forced-colors: active)").subscribe(()=>{this.D&&(this.D=!1,this.C())})}va(){this.F.unsubscribe()}C(){if(!this.D&&this.Ma.isBrowser&&this.ia.body){const a=this.ia.body.classList;a.remove("cdk-high-contrast-active","cdk-high-contrast-black-on-white","cdk-high-contrast-white-on-black");this.D=!0;const b=hy(this);1===b?a.add("cdk-high-contrast-active","cdk-high-contrast-black-on-white"):2===b&&a.add("cdk-high-contrast-active","cdk-high-contrast-white-on-black")}}};
iy.K=function(a){return new (a||iy)(r(Wt),r(kt))};iy.na=Pd({ma:iy,ba:iy.K,fa:"root"});var ky=new gf("liveAnnouncerElement",{fa:"root",ba:jy});function jy(){return null}var ly=new gf("LIVE_ANNOUNCER_DEFAULT_OPTIONS");var my=class{create(a){return"undefined"===typeof MutationObserver?null:new MutationObserver(a)}};my.K=function(a){return new (a||my)};my.na=Pd({ma:my,ba:my.K,fa:"root"});var ny=class{};ny.K=function(a){return new (a||ny)};ny.Ya=Sf({type:ny});ny.Xa=Qd({Wa:[my]});let oy=0;function py(a,b,...c){const d=a.J;let e,f;1===c.length&&"number"===typeof c[0]?f=c[0]:[e,f]=c;a.clear();clearTimeout(a.G);e||(e=d&&d.Tl?d.Tl:"polite");null==f&&d&&(f=d.duration);a.C.setAttribute("aria-live",e);a.C.id&&a.O(a.C.id);Qk(a.qa,()=>{a.D||(a.D=new Promise(g=>a.F=g));clearTimeout(a.G);a.G=setTimeout(()=>{a.C.textContent=b;"number"===typeof f&&(a.G=setTimeout(()=>a.clear(),f));let g;null==(g=a.F)||g.call(a);a.D=a.F=void 0},100);return a.D})}
var qy=class{constructor(a,b,c,d){this.qa=b;this.J=d;this.ia=c;this.C=a||this.N()}clear(){this.C&&(this.C.textContent="")}va(){clearTimeout(this.G);let a;null==(a=this.C)||a.remove();this.C=null;let b;null==(b=this.F)||b.call(this);this.D=this.F=void 0}N(){const a=this.ia.getElementsByClassName("cdk-live-announcer-element"),b=this.ia.createElement("div");for(let c=0;c<a.length;c++)a[c].remove();b.classList.add("cdk-live-announcer-element");b.classList.add("cdk-visually-hidden");b.setAttribute("aria-atomic",
"true");b.setAttribute("aria-live","polite");b.id=`cdk-live-announcer-${oy++}`;this.ia.body.appendChild(b);return b}O(a){const b=this.ia.querySelectorAll('body > .cdk-overlay-container [aria-modal="true"]');for(let c=0;c<b.length;c++){const d=b[c],e=d.getAttribute("aria-owns");e?-1===e.indexOf(a)&&d.setAttribute("aria-owns",e+" "+a):d.setAttribute("aria-owns",a)}}};qy.K=function(a){return new (a||qy)(r(ky,8),r(Tk),r(kt),r(ly,8))};qy.na=Pd({ma:qy,ba:qy.K,fa:"root"});var ry=class{constructor(a){a.C()}};ry.K=function(a){return new (a||ry)(r(iy))};ry.Ya=Sf({type:ry});ry.Xa=Qd({imports:[ny]});function sy(a,b,c){const d=ty(a,b);c=c.trim();d.some(e=>e.trim()===c)||(d.push(c),a.setAttribute(b,d.join(" ")))}function uy(a,b,c){var d=ty(a,b);c=c.trim();d=d.filter(e=>e!==c);d.length?a.setAttribute(b,d.join(" ")):a.removeAttribute(b)}function ty(a,b){let c,d;return null!=(d=null==(c=a.getAttribute(b))?void 0:c.match(/\S+/g))?d:[]};let vy=0;function wy(a,b,c,d){if(a.N(b,c)){var e=xy(c,d);"string"!==typeof c?(yy(c,a.Zb),a.C.set(e,{Jl:c,referenceCount:0})):a.C.has(e)||a.O(c,d);a.F(b,e)||a.J(b,e)}}function zy(a,b,c){if(c&&a.G(b)){var d=xy(c,"tooltip");a.F(b,d)&&a.ca(b,d);"string"===typeof c&&(b=a.C.get(d))&&0===b.referenceCount&&a.T(d);var e;0===(null==(e=a.D)?void 0:e.childNodes.length)&&(a.D.remove(),a.D=null)}}
var Ay=class{constructor(a,b){this.Ma=b;this.C=new Map;this.D=null;this.Zb=`${vy++}`;this.ia=a;this.Zb=of(ni)+"-"+vy++}va(){const a=this.ia.querySelectorAll(`[${"cdk-describedby-host"}="${this.Zb}"]`);for(let c=0;c<a.length;c++)this.aa(a[c]),a[c].removeAttribute("cdk-describedby-host");let b;null==(b=this.D)||b.remove();this.D=null;this.C.clear()}O(a,b){const c=this.ia.createElement("div");yy(c,this.Zb);c.textContent=a;b&&c.setAttribute("role",b);this.P();this.D.appendChild(c);this.C.set(xy(a,b),
{Jl:c,referenceCount:0})}T(a){let b,c;null==(b=this.C.get(a))||null==(c=b.Jl)||c.remove();this.C.delete(a)}P(){if(!this.D){var a=this.ia.querySelectorAll('.cdk-describedby-message-container[platform="server"]');for(let b=0;b<a.length;b++)a[b].remove();a=this.ia.createElement("div");a.style.visibility="hidden";a.classList.add("cdk-describedby-message-container");a.classList.add("cdk-visually-hidden");this.Ma&&!this.Ma.isBrowser&&a.setAttribute("platform","server");this.ia.body.appendChild(a);this.D=
a}}aa(a){const b=ty(a,"aria-describedby").filter(c=>0!=c.indexOf("cdk-describedby-message"));a.setAttribute("aria-describedby",b.join(" "))}J(a,b){b=this.C.get(b);sy(a,"aria-describedby",b.Jl.id);a.setAttribute("cdk-describedby-host",this.Zb);b.referenceCount++}ca(a,b){b=this.C.get(b);b.referenceCount--;uy(a,"aria-describedby",b.Jl.id);a.removeAttribute("cdk-describedby-host")}F(a,b){a=ty(a,"aria-describedby");b=(b=this.C.get(b))&&b.Jl.id;return!!b&&-1!=a.indexOf(b)}N(a,b){if(!this.G(a))return!1;
if(b&&"object"===typeof b)return!0;b=null==b?"":`${b}`.trim();a=a.getAttribute("aria-label");return b?!a||a.trim()!==b:!1}G(a){return a.nodeType===this.ia.ELEMENT_NODE}};Ay.K=function(a){return new (a||Ay)(r(kt),r(Wt))};Ay.na=Pd({ma:Ay,ba:Ay.K,fa:"root"});function xy(a,b){return"string"===typeof a?`${b||""}/${a}`:a}function yy(a,b){a.id||(a.id=`${"cdk-describedby-message"}-${b}-${vy++}`)};var Dy=class extends Zx{get enabled(){return this.G}set enabled(a){(this.G=a)?this.O.register(this):By(this.O,this)}constructor(a,b,c,d,e,f,g){super(a,b,c,d,g.defer);this.O=e;this.aa=f;this.O.register(this)}destroy(){By(this.O,this);super.destroy()}Mt(){Cy(this.aa,this);Yx(this,!0)}Lt(){var a=this.aa;a.C&&(this.ia.removeEventListener("focus",a.C,!0),a.C=null);Yx(this,!1)}};function Cy(a,b){a.C&&b.ia.removeEventListener("focus",a.C,!0);a.C=c=>Ey(b,c);Qk(b.qa,()=>{b.ia.addEventListener("focus",a.C,!0)})}function Ey(a,b){b=b.target;const c=a.C;let d;!b||c.contains(b)||(null==(d=b.closest)?0:d.call(b,"div.cdk-overlay-pane"))||setTimeout(()=>{a.enabled&&!c.contains(a.ia.activeElement)&&Vx(a)})}var Fy=class{constructor(){this.C=null}};var Gy=new gf("FOCUS_TRAP_INERT_STRATEGY");function By(a,b){b.Lt();a=a.C;b=a.indexOf(b);-1!==b&&(a.splice(b,1),a.length&&a[a.length-1].Mt())}var Hy=class{constructor(){this.C=[]}register(a){let b=this.C=this.C.filter(c=>c!==a);b.length&&b[b.length-1].Lt();b.push(a);a.Mt()}};Hy.K=function(a){return new (a||Hy)};Hy.na=Pd({ma:Hy,ba:Hy.K,fa:"root"});var Iy=class{constructor(a,b,c,d,e){this.C=a;this.qa=b;this.D=c;this.ia=d;this.F=e||new Fy}create(a,b={defer:!1}){let c;"boolean"===typeof b?c={defer:b}:c=b;return new Dy(a,this.C,this.qa,this.ia,this.D,this.F,c)}};Iy.K=function(a){return new (a||Iy)(r(Ux),r(Tk),r(Hy),r(kt),r(Gy,8))};Iy.na=Pd({ma:Iy,ba:Iy.K,fa:"root"});function Jy(a,b){a.oa=b;return a}function Ky(a){a.Ja=!0;return a}function Ly(a){a.Ea=!0;return a}function My(a,b){a.ea=b;return a}function Ny(a){a.Qa=["shiftKey"];return a}function Oy(a,b=200){a.wa.unsubscribe();a.wa=C(a.ja,So(c=>a.G.push(c)),Rn(b),xk(()=>0<a.G.length),Cj(()=>a.G.join(""))).subscribe(c=>{const d=a.N();for(let e=1;e<d.length+1;e++){const f=(a.F+e)%d.length,g=d[f];if(!a.oa(g)&&0===g.jw().toUpperCase().trim().indexOf(c)){a.D(f);break}}a.G=[]});return a}
function Py(a){a.ua=!0;return a}function Qy(a){a.aa={enabled:!0,delta:10};return a}function Ry(a,b){var c=a.N();b="number"===typeof b?b:c.indexOf(b);c=c[b];a.P=null==c?null:c;a.F=b}function Sy(a){0>a.F?a.J(0,1):a.Va(1)}function Ty(a){0>a.F&&a.Ja?a.J(a.N().length-1,-1):a.Va(-1)}
function Uy(a,b){var c=b.keyCode,d=["altKey","ctrlKey","metaKey","shiftKey"].every(e=>!b[e]||-1<a.Qa.indexOf(e));switch(c){case 9:a.ca.next();return;case 40:if(a.Ea&&d){Sy(a);break}else return;case 38:if(a.Ea&&d){Ty(a);break}else return;case 39:if(a.ea&&d){"rtl"===a.ea?Ty(a):Sy(a);break}else return;case 37:if(a.ea&&d){"rtl"===a.ea?Sy(a):Ty(a);break}else return;case 36:if(a.ua&&d){a.J(0,1);break}else return;case 35:if(a.ua&&d){a.J(a.N().length-1,-1);break}else return;case 33:if(a.aa.enabled&&d){c=
a.F-a.aa.delta;a.J(0<c?c:0,1);break}else return;case 34:if(a.aa.enabled&&d){c=a.F+a.aa.delta;d=a.N().length;a.J(c<d?c:d-1,-1);break}else return;default:if(d||Cu(b,"shiftKey"))b.key&&1===b.key.length?a.ja.next(b.key.toLocaleUpperCase()):(65<=c&&90>=c||48<=c&&57>=c)&&a.ja.next(String.fromCharCode(c));return}a.G=[];b.preventDefault()}
var Vy=class{constructor(a,b){this.T=a;this.F=-1;this.P=null;this.Ja=!1;this.ja=new hj;this.wa=Si.EMPTY;this.Ea=!0;this.Qa=[];this.ua=!1;this.aa={enabled:!1,delta:10};this.oa=c=>c.disabled;this.G=[];this.ca=new hj;this.xb=new hj;a instanceof cr?this.gb=a.C.subscribe(c=>this.Ta(c.toArray())):Bn(a)&&(this.fb=Ks(()=>this.Ta(a()),{xa:b}))}D(a){const b=this.P;Ry(this,a);this.P!==b&&this.xb.next(this.F)}get O(){return this.F}get C(){return this.P}destroy(){this.wa.unsubscribe();let a;null==(a=this.gb)||
a.unsubscribe();let b;null==(b=this.fb)||b.destroy();this.ja.complete();this.ca.complete();this.xb.complete();this.G=[]}Va(a){this.Ja?this.sb(a):this.lb(a)}sb(a){const b=this.N();for(let c=1;c<=b.length;c++){const d=(this.F+a*c+b.length)%b.length;if(!this.oa(b[d])){this.D(d);break}}}lb(a){this.J(this.F+a,a)}J(a,b){const c=this.N();if(c[a]){for(;this.oa(c[a]);)if(a+=b,!c[a])return;this.D(a)}}N(){return Bn(this.T)?this.T():this.T instanceof cr?this.T.toArray():this.T}Ta(a){this.P&&(a=a.indexOf(this.P),
-1<a&&a!==this.F&&(this.F=a))}};var Wy=class extends Vy{D(a){if(this.C){var b=this.C;b.D&&(b.D=!1,en(b.Da))}super.D(a);this.C&&(a=this.C,a.D||(a.D=!0,en(a.Da)))}};var Xy=class extends Vy{constructor(){super(...arguments);this.ta="program"}D(a){super.D(a);this.C&&this.C.focus(this.ta)}};function Yy(a,b){return{type:7,name:a,MB:b,options:{}}}function Zy(a,b=null){return{type:4,styles:b,fg:a}}function $y(a){return{type:6,styles:a,offset:null}}function az(a,b){return{type:0,name:a,styles:b,options:void 0}}function bz(a,b){return{type:1,bC:a,animation:b,options:null}};function cz(a){a.G||(a.G=!0,a.C.forEach(b=>b()),a.C=[])}function dz(a){a.D.forEach(b=>b());a.D=[]}function ez(a){queueMicrotask(()=>cz(a))}
var fz=class{constructor(a=0,b=0){this.C=[];this.D=[];this.J=[];this.O=[];this.P=[];this.G=this.Ga=this.N=!1;this.F=null;this.totalTime=a+b}onStart(a){this.P.push(a);this.D.push(a)}Md(a){this.O.push(a);this.C.push(a)}ob(a){this.J.push(a)}Rf(){return this.N}init(){}play(){this.Rf()||(dz(this),ez(this));this.N=!0}pause(){}re(){}finish(){cz(this)}destroy(){this.Ga||(this.Ga=!0,this.Rf()||dz(this),this.finish(),this.J.forEach(a=>a()),this.J=[])}reset(){this.G=this.N=!1;this.D=this.P;this.C=this.O}setPosition(){}si(a){a=
"start"==a?this.D:this.C;a.forEach(b=>b());a.length=0}};function gz(a){a.J||(a.J=!0,a.D.forEach(b=>b()),a.D=[])}function hz(a){a.Ga||(a.Ga=!0,gz(a),a.C.forEach(b=>b.destroy()),a.N.forEach(b=>b()),a.N=[])}function iz(a){a.Rf()||(a.O=!0,a.G.forEach(b=>b()),a.G=[])}
var jz=class{constructor(a){this.D=[];this.G=[];this.Ga=this.O=this.J=!1;this.N=[];this.F=null;this.totalTime=0;this.C=a;let b=0,c=0,d=0;const e=this.C.length;0==e?queueMicrotask(()=>gz(this)):this.C.forEach(f=>{f.Md(()=>{++b==e&&gz(this)});f.ob(()=>{++c==e&&hz(this)});f.onStart(()=>{++d==e&&iz(this)})});this.totalTime=this.C.reduce((f,g)=>Math.max(f,g.totalTime),0)}init(){this.C.forEach(a=>a.init())}onStart(a){this.G.push(a)}Md(a){this.D.push(a)}ob(a){this.N.push(a)}Rf(){return this.O}play(){this.F||
this.init();iz(this);this.C.forEach(a=>a.play())}pause(){this.C.forEach(a=>a.pause())}re(){this.C.forEach(a=>a.re())}finish(){gz(this);this.C.forEach(a=>a.finish())}destroy(){hz(this)}reset(){this.C.forEach(a=>a.reset());this.O=this.J=this.Ga=!1}setPosition(a){const b=a*this.totalTime;this.C.forEach(c=>{c.setPosition(c.totalTime?Math.min(1,b/c.totalTime):1)})}Ok(){this.C.forEach(a=>{a.Ok&&a.Ok()})}si(a){a="start"==a?this.G:this.D;a.forEach(b=>b());a.length=0}};var kz=Yy("panelAnimation",[az("void, hidden",$y({opacity:0,transform:"scaleY(0.8)"})),bz(":enter, hidden => visible",[function(a,b=null){return{type:3,steps:a,options:b}}([Zy("0.03s linear",$y({opacity:1})),Zy("0.12s cubic-bezier(0, 0, 0.2, 1)",$y({transform:"scaleY(1)"}))])]),bz(":leave, visible => hidden",[Zy("0.075s linear",$y({opacity:0}))])]);var lz=new gf("mat-sanity-checks",{fa:"root",ba:function(){return!0}}),mz=class{constructor(a,b,c){this.C=b;this.ia=c;this.D=!1;a.C();this.D||(this.D=!0)}F(a){return Ut()?!1:"boolean"===typeof this.C?this.C:!!this.C[a]}};mz.K=function(a){return new (a||mz)(r(iy),r(lz,8),r(kt))};mz.Ya=Sf({type:mz});mz.Xa=Qd({imports:[qu,qu]});var nz=class{constructor(a,b,c,d,e){this.D=a;this.Ld=b;this.G=c;this.F=d;this.C=e;this.De=!1}};new gf("MAT_DATE_LOCALE",{fa:"root",ba:oz});function oz(){return of(ps)};new gf("mat-date-formats");var pz=class{};pz.K=function(a){return new (a||pz)};pz.na=Pd({ma:pz,ba:pz.K,fa:"root"});var qz=class{};qz.K=function(a){return new (a||qz)};
qz.Ca=Jf({type:qz,ha:[["div","mat-internal-form-field",""]],Ka:[1,"mdc-form-field","mat-internal-form-field"],Za:2,Ia:function(a,b){a&2&&Wp("mdc-form-field--align-end","before"===b.Oh)},inputs:{Oh:"labelPosition"},ka:!0,features:[Nr],attrs:["mat-internal-form-field",""],bc:["*"],ya:1,za:0,sa:function(a){a&1&&(Zq(),$q(0))},styles:[".mdc-form-field{display:inline-flex;align-items:center;vertical-align:middle}.mdc-form-field[hidden]{display:none}.mdc-form-field>label{margin-left:0;margin-right:auto;padding-left:4px;padding-right:0;order:0}[dir=rtl] .mdc-form-field>label,.mdc-form-field>label[dir=rtl]{margin-left:auto;margin-right:0}[dir=rtl] .mdc-form-field>label,.mdc-form-field>label[dir=rtl]{padding-left:0;padding-right:4px}.mdc-form-field--nowrap>label{text-overflow:ellipsis;overflow:hidden;white-space:nowrap}.mdc-form-field--align-end>label{margin-left:auto;margin-right:0;padding-left:0;padding-right:4px;order:-1}[dir=rtl] .mdc-form-field--align-end>label,.mdc-form-field--align-end>label[dir=rtl]{margin-left:0;margin-right:auto}[dir=rtl] .mdc-form-field--align-end>label,.mdc-form-field--align-end>label[dir=rtl]{padding-left:4px;padding-right:0}.mdc-form-field--space-between{justify-content:space-between}.mdc-form-field--space-between>label{margin:0}[dir=rtl] .mdc-form-field--space-between>label,.mdc-form-field--space-between>label[dir=rtl]{margin:0}.mat-internal-form-field{-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased}\n"],pb:2,
Pa:0});var rz=new gf("MAT_OPTION_PARENT_COMPONENT");const sz=["*",[["mat-option"],["ng-container"]]];let tz=0;var uz=new gf("MatOptgroup"),vz=class{constructor(a){this.disabled=!1;this.Xe=`mat-optgroup-label-${tz++}`;let b;this.Fm=null!=(b=null==a?void 0:a.EC)?b:!1}};vz.K=function(a){return new (a||vz)(y(rz,8))};
vz.Ca=Jf({type:vz,ha:[["mat-optgroup"]],Ka:[1,"mat-mdc-optgroup"],Za:3,Ia:function(a,b){a&2&&Mp("role",b.Fm?null:"group")("aria-disabled",b.Fm?null:b.disabled.toString())("aria-labelledby",b.Fm?null:b.Xe)},inputs:{label:"label",disabled:[2,"disabled","disabled",As]},ib:["matOptgroup"],ka:!0,features:[Lr([{pa:uz,Ab:vz}]),Lp,Nr],bc:["*","mat-option, ng-container"],ya:5,za:4,Ba:[["role","presentation",1,"mat-mdc-optgroup-label",3,"id"],[1,"mdc-list-item__primary-text"]],sa:function(a,b){a&1&&(Zq(sz),
R(0,"span",0)(1,"span",1),Z(2),$q(3),U()(),$q(4,1));a&2&&(Wp("mdc-list-item--disabled",b.disabled),Q("id",b.Xe),D(2),Cr("",b.label," "))},styles:[".mat-mdc-optgroup{color:var(--mat-optgroup-label-text-color);font-family:var(--mat-optgroup-label-text-font);line-height:var(--mat-optgroup-label-text-line-height);font-size:var(--mat-optgroup-label-text-size);letter-spacing:var(--mat-optgroup-label-text-tracking);font-weight:var(--mat-optgroup-label-text-weight)}.mat-mdc-optgroup-label{display:flex;position:relative;align-items:center;justify-content:flex-start;overflow:hidden;padding:0;padding-left:16px;padding-right:16px;min-height:48px}.mat-mdc-optgroup-label:focus{outline:none}[dir=rtl] .mat-mdc-optgroup-label,.mat-mdc-optgroup-label[dir=rtl]{padding-left:16px;padding-right:16px}.mat-mdc-optgroup-label.mdc-list-item--disabled{opacity:.38}.mat-mdc-optgroup-label .mdc-list-item__primary-text{font-size:inherit;font-weight:inherit;letter-spacing:inherit;line-height:inherit;font-family:inherit;text-decoration:inherit;text-transform:inherit;white-space:normal}\n"],
pb:2,Pa:0});const wz=Lt({passive:!0,capture:!0});function xz(a,b,c,d){var e=yz;const f=e.Bb.get(b);f?(a=f.get(c))?a.add(d):f.set(c,new Set([d])):(e.Bb.set(b,new Map([[c,new Set([d])]])),Qk(a,()=>{document.addEventListener(b,e.C,wz)}))}var zz=class{constructor(){this.Bb=new Map;this.C=a=>{const b=Tt(a);if(b){let c;null==(c=this.Bb.get(a.type))||c.forEach((d,e)=>{(e===b||e.contains(b))&&d.forEach(f=>f.handleEvent(a))})}}}};var Az=class{constructor(a,b,c,d=!1){this.C=a;this.element=b;this.config=c;this.D=d;this.state=3}};var Bz={aw:225,zn:150};const Cz=Lt({passive:!0,capture:!0}),Dz=["mousedown","touchstart"],Ez=["mouseup","mouseleave","touchend","touchcancel"];function Fz(a,b){if(2!==b.state&&3!==b.state){var c=b.element,d=Object.assign({},Bz,b.config.animation);c.style.transitionDuration=`${d.zn}ms`;c.style.opacity="0";b.state=2;!b.D&&d.zn||a.xp(b)}}
function Gz(a,b,c,d={}){var e=a.Ji=a.Ji||a.Ii.getBoundingClientRect();const f=Object.assign({},Bz,d.animation);d.bj&&(b=e.left+e.width/2,c=e.top+e.height/2);var g=d.radius||Hz(b,c,e);b-=e.left;const h=c-e.top;c=f.aw;const m=document.createElement("div");m.classList.add("mat-ripple-element");m.style.left=`${b-g}px`;m.style.top=`${h-g}px`;m.style.height=`${2*g}px`;m.style.width=`${2*g}px`;null!=d.color&&(m.style.backgroundColor=d.color);m.style.transitionDuration=`${c}ms`;a.Ii.appendChild(m);g=window.getComputedStyle(m);
b=g.transitionDuration;e="none"===g.transitionProperty||"0s"===b||"0s, 0s"===b||0===e.width&&0===e.height;const p=new Az(a,m,d,e);m.style.transform="scale3d(1, 1, 1)";p.state=0;d.Wr||(a.Kp=p);let u=null;e||!c&&!f.zn||Qk(a.qa,()=>{const w=()=>a.xp(p),B=()=>a.Kt(p);m.addEventListener("transitionend",w);m.addEventListener("transitioncancel",B);u={xD:w,wD:B}});a.pk.set(p,u);!e&&c||a.xp(p)}function Iz(a,b){const c=Ts(b);a.Ma.isBrowser&&c&&c!==a.Ui&&(a.wu(),a.Ui=c,Dz.forEach(d=>{xz(a.qa,d,c,a)}))}
var Jz=class{constructor(a,b,c,d){this.Um=a;this.qa=b;this.Ma=d;this.zk=!1;this.pk=new Map;this.Pm=!1;d.isBrowser&&(this.Ii=Ts(c))}Nq(){this.Qt().forEach(a=>{a.config.Wr||Fz(a.C,a)})}handleEvent(a){"mousedown"===a.type?this.Mm(a):"touchstart"===a.type?this.GA(a):this.FA();this.Pm||(Qk(this.qa,()=>{Ez.forEach(b=>{this.Ui.addEventListener(b,this,Cz)})}),this.Pm=!0)}xp(a){0===a.state?this.XA(a):2===a.state&&this.Kt(a)}XA(a){const b=a===this.Kp,c=a.config.Wr;a.state=1;c||b&&this.zk||Fz(a.C,a)}Kt(a){let b;
const c=null!=(b=this.pk.get(a))?b:null;this.pk.delete(a);this.pk.size||(this.Ji=null);a===this.Kp&&(this.Kp=null);a.state=3;null!==c&&(a.element.removeEventListener("transitionend",c.xD),a.element.removeEventListener("transitioncancel",c.wD));a.element.remove()}Mm(a){const b=Ex(a),c=this.ou&&Date.now()<this.ou+800;this.Um.Kx||b||c||(this.zk=!0,Gz(this,a.clientX,a.clientY,this.Um.Jx))}GA(a){if(!this.Um.Kx&&!Fx(a)&&(this.ou=Date.now(),this.zk=!0,a=a.changedTouches))for(let b=0;b<a.length;b++)Gz(this,
a[b].clientX,a[b].clientY,this.Um.Jx)}FA(){this.zk&&(this.zk=!1,this.Qt().forEach(a=>{const b=1===a.state||a.config.zy&&0===a.state;!a.config.Wr&&b&&Fz(a.C,a)}))}Qt(){return Array.from(this.pk.keys())}wu(){const a=this.Ui;a&&(Dz.forEach(b=>{var c=yz;const d=c.Bb.get(b);if(d){var e=d.get(a);e&&(e.delete(this),0===e.size&&d.delete(a),0===d.size&&(c.Bb.delete(b),document.removeEventListener(b,c.C,wz)))}}),this.Pm&&(Ez.forEach(b=>a.removeEventListener(b,this,Cz)),this.Pm=!1))}},yz=new zz;
function Hz(a,b,c){a=Math.max(Math.abs(a-c.left),Math.abs(a-c.right));b=Math.max(Math.abs(b-c.top),Math.abs(b-c.bottom));return Math.sqrt(a*a+b*b)};var Kz=new gf("mat-ripple-global-options"),Lz=class{get disabled(){return this.Ib}set disabled(a){a&&this.Nq();this.Ib=a;this.G()}get Oe(){return this.J||this.ra.R}set Oe(a){this.J=a;this.G()}constructor(a,b,c,d,e){this.ra=a;this.Yc=e;this.radius=0;this.C=this.Ib=!1;this.D=d||{};this.F=new Jz(this,b,a,c)}Fb(){this.C=!0;this.G()}va(){this.F.wu()}Nq(){this.F.Nq()}get Jx(){return{bj:this.bj,radius:this.radius,color:this.color,animation:Object.assign({},this.D.animation,"NoopAnimations"===this.Yc?{aw:0,
zn:0}:{},this.animation),zy:this.D.zy}}get Kx(){return this.disabled||!!this.D.disabled}G(){!this.disabled&&this.C&&Iz(this.F,this.Oe)}};Lz.K=function(a){return new (a||Lz)(y(wi),y(Tk),y(Wt),y(Kz,8),y(qi,8))};
Lz.Fa=Uf({type:Lz,ha:[["","mat-ripple",""],["","matRipple",""]],Ka:[1,"mat-ripple"],Za:2,Ia:function(a,b){a&2&&Wp("mat-ripple-unbounded",b.YE)},inputs:{color:[0,"matRippleColor","color"],YE:[0,"matRippleUnbounded","unbounded"],bj:[0,"matRippleCentered","centered"],radius:[0,"matRippleRadius","radius"],animation:[0,"matRippleAnimation","animation"],disabled:[0,"matRippleDisabled","disabled"],Oe:[0,"matRippleTrigger","trigger"]},ib:["matRipple"],ka:!0});var Mz=class{constructor(a){this.Yc=a;this.state="unchecked";this.disabled=!1;this.Gd="full"}};Mz.K=function(a){return new (a||Mz)(y(qi,8))};
Mz.Ca=Jf({type:Mz,ha:[["mat-pseudo-checkbox"]],Ka:[1,"mat-pseudo-checkbox"],Za:12,Ia:function(a,b){a&2&&Wp("mat-pseudo-checkbox-indeterminate","indeterminate"===b.state)("mat-pseudo-checkbox-checked","checked"===b.state)("mat-pseudo-checkbox-disabled",b.disabled)("mat-pseudo-checkbox-minimal","minimal"===b.Gd)("mat-pseudo-checkbox-full","full"===b.Gd)("_mat-animation-noopable","NoopAnimations"===b.Yc)},inputs:{state:"state",disabled:"disabled",Gd:"appearance"},ka:!0,features:[Nr],ya:0,za:0,sa:function(){},
styles:['.mat-pseudo-checkbox{border-radius:2px;cursor:pointer;display:inline-block;vertical-align:middle;box-sizing:border-box;position:relative;flex-shrink:0;transition:border-color 90ms cubic-bezier(0, 0, 0.2, 0.1),background-color 90ms cubic-bezier(0, 0, 0.2, 0.1)}.mat-pseudo-checkbox::after{position:absolute;opacity:0;content:"";border-bottom:2px solid currentColor;transition:opacity 90ms cubic-bezier(0, 0, 0.2, 0.1)}.mat-pseudo-checkbox._mat-animation-noopable{transition:none !important;animation:none !important}.mat-pseudo-checkbox._mat-animation-noopable::after{transition:none}.mat-pseudo-checkbox-disabled{cursor:default}.mat-pseudo-checkbox-indeterminate::after{left:1px;opacity:1;border-radius:2px}.mat-pseudo-checkbox-checked::after{left:1px;border-left:2px solid currentColor;transform:rotate(-45deg);opacity:1;box-sizing:content-box}.mat-pseudo-checkbox-minimal.mat-pseudo-checkbox-checked::after,.mat-pseudo-checkbox-minimal.mat-pseudo-checkbox-indeterminate::after{color:var(--mat-minimal-pseudo-checkbox-selected-checkmark-color)}.mat-pseudo-checkbox-minimal.mat-pseudo-checkbox-checked.mat-pseudo-checkbox-disabled::after,.mat-pseudo-checkbox-minimal.mat-pseudo-checkbox-indeterminate.mat-pseudo-checkbox-disabled::after{color:var(--mat-minimal-pseudo-checkbox-disabled-selected-checkmark-color)}.mat-pseudo-checkbox-full{border-color:var(--mat-full-pseudo-checkbox-unselected-icon-color);border-width:2px;border-style:solid}.mat-pseudo-checkbox-full.mat-pseudo-checkbox-disabled{border-color:var(--mat-full-pseudo-checkbox-disabled-unselected-icon-color)}.mat-pseudo-checkbox-full.mat-pseudo-checkbox-checked,.mat-pseudo-checkbox-full.mat-pseudo-checkbox-indeterminate{background-color:var(--mat-full-pseudo-checkbox-selected-icon-color);border-color:rgba(0,0,0,0)}.mat-pseudo-checkbox-full.mat-pseudo-checkbox-checked::after,.mat-pseudo-checkbox-full.mat-pseudo-checkbox-indeterminate::after{color:var(--mat-full-pseudo-checkbox-selected-checkmark-color)}.mat-pseudo-checkbox-full.mat-pseudo-checkbox-checked.mat-pseudo-checkbox-disabled,.mat-pseudo-checkbox-full.mat-pseudo-checkbox-indeterminate.mat-pseudo-checkbox-disabled{background-color:var(--mat-full-pseudo-checkbox-disabled-selected-icon-color)}.mat-pseudo-checkbox-full.mat-pseudo-checkbox-checked.mat-pseudo-checkbox-disabled::after,.mat-pseudo-checkbox-full.mat-pseudo-checkbox-indeterminate.mat-pseudo-checkbox-disabled::after{color:var(--mat-full-pseudo-checkbox-disabled-selected-checkmark-color)}.mat-pseudo-checkbox{width:18px;height:18px}.mat-pseudo-checkbox-minimal.mat-pseudo-checkbox-checked::after{width:14px;height:6px;transform-origin:center;top:-4.2426406871px;left:0;bottom:0;right:0;margin:auto}.mat-pseudo-checkbox-minimal.mat-pseudo-checkbox-indeterminate::after{top:8px;width:16px}.mat-pseudo-checkbox-full.mat-pseudo-checkbox-checked::after{width:10px;height:4px;transform-origin:center;top:-2.8284271247px;left:0;bottom:0;right:0;margin:auto}.mat-pseudo-checkbox-full.mat-pseudo-checkbox-indeterminate::after{top:6px;width:12px}\n'],
pb:2,Pa:0});const Nz=["text"],Oz=[[["mat-icon"]],"*"];function Pz(a){a&1&&Oq(0,"mat-pseudo-checkbox",1);a&2&&(a=Y(),Q("disabled",a.disabled)("state",a.selected?"checked":"unchecked"))}function Qz(a){a&1&&Oq(0,"mat-pseudo-checkbox",3);a&2&&(a=Y(),Q("disabled",a.disabled))}function Rz(a){a&1&&(R(0,"span",4),Z(1),U());a&2&&(a=Y(),D(),Cr("(",a.group.label,")"))}let Sz=0;var Tz=class{constructor(a,b=!1){this.source=a;this.C=b}};function Uz(a,b=!0){a.C&&(a.C=!1,en(a.Da),b&&a.J())}
var Vz=class{get multiple(){return this.Sa&&this.Sa.multiple}get selected(){return this.C}get disabled(){return this.group&&this.group.disabled||this.Ib}set disabled(a){this.Ib=a}get Nc(){return!(!this.Sa||!this.Sa.Nc)}get Ge(){return!(!this.Sa||!this.Sa.Ge)}constructor(a,b,c,d){this.O=a;this.Da=b;this.Sa=c;this.group=d;this.Ib=this.D=this.C=!1;this.N="";this.id=`mat-option-${Sz++}`;this.po=new Jk;this.G=new hj}get active(){return this.D}get F(){let a;return((null==(a=this.bB)?void 0:a.R.textContent)||
"").trim()}select(a=!0){this.C||(this.C=!0,en(this.Da),a&&this.J())}focus(a,b){const c=this.we();"function"===typeof c.focus&&c.focus(b)}jw(){return this.F}Ve(a){13!==a.keyCode&&32!==a.keyCode||Cu(a)||(this.wh(),a.preventDefault())}wh(){this.disabled||(this.C=this.multiple?!this.C:!0,en(this.Da),this.J(!0))}Ut(){return this.disabled?"-1":"0"}we(){return this.O.R}Zw(){if(this.C){const a=this.F;a!==this.N&&(this.N&&this.G.next(),this.N=a)}}va(){this.G.complete()}J(a=!1){this.po.emit(new Tz(this,a))}};
Vz.K=function(a){return new (a||Vz)(y(wi),y(Tr),y(rz,8),y(uz,8))};
Vz.Ca=Jf({type:Vz,ha:[["mat-option"]],eb:function(a,b){a&1&&vr(Nz,7);if(a&2){let c;wr(c=xr())&&(b.bB=c.first)}},Ka:["role","option",1,"mat-mdc-option","mdc-list-item"],Za:11,Ia:function(a,b){a&1&&V("click",function(){return b.wh()})("keydown",function(c){return b.Ve(c)});a&2&&(Sq("id",b.id),Mp("aria-selected",b.selected)("aria-disabled",b.disabled.toString()),Wp("mdc-list-item--selected",b.selected)("mat-mdc-option-multiple",b.multiple)("mat-mdc-option-active",b.active)("mdc-list-item--disabled",
b.disabled))},inputs:{value:"value",id:"id",disabled:[2,"disabled","disabled",As]},outputs:{po:"onSelectionChange"},ib:["matOption"],ka:!0,features:[Lp,Nr],bc:["mat-icon","*"],ya:8,za:5,Ba:[["text",""],["aria-hidden","true",1,"mat-mdc-option-pseudo-checkbox",3,"disabled","state"],[1,"mdc-list-item__primary-text"],["state","checked","aria-hidden","true","appearance","minimal",1,"mat-mdc-option-pseudo-checkbox",3,"disabled"],[1,"cdk-visually-hidden"],["aria-hidden","true","mat-ripple","",1,"mat-mdc-option-ripple",
"mat-mdc-focus-indicator",3,"matRippleTrigger","matRippleDisabled"]],sa:function(a,b){a&1&&(Zq(Oz),H(0,Pz,1,2,"mat-pseudo-checkbox",1),$q(1),R(2,"span",2,0),$q(4,1),U(),H(5,Qz,1,1,"mat-pseudo-checkbox",3)(6,Rz,2,1,"span",4),Oq(7,"div",5));a&2&&(Gq(b.multiple?0:-1),D(5),Gq(b.multiple||!b.selected||b.Ge?-1:5),D(),Gq(b.group&&b.group.Fm?6:-1),D(),Q("matRippleTrigger",b.we())("matRippleDisabled",b.disabled||b.Nc))},Ha:[Mz,Lz],styles:['.mat-mdc-option{display:flex;position:relative;align-items:center;justify-content:flex-start;overflow:hidden;padding:0;padding-left:16px;padding-right:16px;-webkit-user-select:none;user-select:none;-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;cursor:pointer;-webkit-tap-highlight-color:rgba(0,0,0,0);color:var(--mat-option-label-text-color);font-family:var(--mat-option-label-text-font);line-height:var(--mat-option-label-text-line-height);font-size:var(--mat-option-label-text-size);letter-spacing:var(--mat-option-label-text-tracking);font-weight:var(--mat-option-label-text-weight);min-height:48px}.mat-mdc-option:focus{outline:none}[dir=rtl] .mat-mdc-option,.mat-mdc-option[dir=rtl]{padding-left:16px;padding-right:16px}.mat-mdc-option:hover:not(.mdc-list-item--disabled){background-color:var(--mat-option-hover-state-layer-color)}.mat-mdc-option:focus.mdc-list-item,.mat-mdc-option.mat-mdc-option-active.mdc-list-item{background-color:var(--mat-option-focus-state-layer-color)}.mat-mdc-option.mdc-list-item--selected:not(.mdc-list-item--disabled) .mdc-list-item__primary-text{color:var(--mat-option-selected-state-label-text-color)}.mat-mdc-option.mdc-list-item--selected:not(.mdc-list-item--disabled):not(.mat-mdc-option-multiple){background-color:var(--mat-option-selected-state-layer-color)}.mat-mdc-option.mdc-list-item{align-items:center;background:rgba(0,0,0,0)}.mat-mdc-option.mdc-list-item--disabled{cursor:default;pointer-events:none}.mat-mdc-option.mdc-list-item--disabled .mat-mdc-option-pseudo-checkbox,.mat-mdc-option.mdc-list-item--disabled .mdc-list-item__primary-text,.mat-mdc-option.mdc-list-item--disabled>mat-icon{opacity:.38}.mat-mdc-optgroup .mat-mdc-option:not(.mat-mdc-option-multiple){padding-left:32px}[dir=rtl] .mat-mdc-optgroup .mat-mdc-option:not(.mat-mdc-option-multiple){padding-left:16px;padding-right:32px}.mat-mdc-option .mat-icon,.mat-mdc-option .mat-pseudo-checkbox-full{margin-right:16px;flex-shrink:0}[dir=rtl] .mat-mdc-option .mat-icon,[dir=rtl] .mat-mdc-option .mat-pseudo-checkbox-full{margin-right:0;margin-left:16px}.mat-mdc-option .mat-pseudo-checkbox-minimal{margin-left:16px;flex-shrink:0}[dir=rtl] .mat-mdc-option .mat-pseudo-checkbox-minimal{margin-right:16px;margin-left:0}.mat-mdc-option .mat-mdc-option-ripple{top:0;left:0;right:0;bottom:0;position:absolute;pointer-events:none}.mat-mdc-option .mdc-list-item__primary-text{white-space:normal;font-size:inherit;font-weight:inherit;letter-spacing:inherit;line-height:inherit;font-family:inherit;text-decoration:inherit;text-transform:inherit;margin-right:auto}[dir=rtl] .mat-mdc-option .mdc-list-item__primary-text{margin-right:0;margin-left:auto}.cdk-high-contrast-active .mat-mdc-option.mdc-list-item--selected:not(.mat-mdc-option-multiple)::after{content:"";position:absolute;top:50%;right:16px;transform:translateY(-50%);width:10px;height:0;border-bottom:solid 10px;border-radius:10px}[dir=rtl] .cdk-high-contrast-active .mat-mdc-option.mdc-list-item--selected:not(.mat-mdc-option-multiple)::after{right:auto;left:16px}.mat-mdc-option-multiple{--mdc-list-list-item-selected-container-color:var(--mdc-list-list-item-container-color, transparent)}.mat-mdc-option-active .mat-mdc-focus-indicator::before{content:""}\n'],
pb:2,Pa:0});function Wz(a,b,c){if(c.length){b=b.toArray();c=c.toArray();let d=0;for(let e=0;e<a+1;e++)b[e].group&&b[e].group===c[d]&&d++;return d}return 0}function Xz(a,b,c,d){return a<c?a:a+b>c+d?Math.max(0,a-d+b):c};var Yz=class{};Yz.K=function(a){return new (a||Yz)};Yz.Ya=Sf({type:Yz});Yz.Xa=Qd({imports:[mz,mz]});var Zz=class{};Zz.K=function(a){return new (a||Zz)};Zz.Ya=Sf({type:Zz});Zz.Xa=Qd({imports:[mz]});var $z=class{};$z.K=function(a){return new (a||$z)};$z.Ya=Sf({type:$z});$z.Xa=Qd({imports:[Yz,mz,Zz,Vz]});const aA={capture:!0},bA=["focus","mousedown","mouseenter","touchstart"];function cA(a,b){const c=a.C.get(b);c&&(c.va(),a.C.delete(b))}function dA(a,b,c){let d,e;b.setAttribute("mat-ripple-loader-uninitialized",null!=(e=null==(d=a.D)?void 0:d.namespace)?e:"");!c.className&&b.hasAttribute("mat-ripple-loader-class-name")||b.setAttribute("mat-ripple-loader-class-name",c.className||"");c.bj&&b.setAttribute("mat-ripple-loader-centered","");c.disabled&&b.setAttribute("mat-ripple-loader-disabled","")}
function eA(a,b,c){b.removeAttribute("mat-ripple-loader-uninitialized");a.C.set(b,c)}
var fA=class{constructor(){this.ia=of(kt,{optional:!0});this.Yc=of(qi,{optional:!0});this.D=of(Kz,{optional:!0});this.Ma=of(Wt);this.qa=of(Tk);this.C=new Map;this.G=a=>{a=Tt(a);if(a instanceof HTMLElement){let b,c;(a=a.closest(`[${"mat-ripple-loader-uninitialized"}="${null!=(c=null==(b=this.D)?void 0:b.namespace)?c:""}"]`))&&this.F(a)}};Qk(this.qa,()=>{for(const a of bA){let b;null==(b=this.ia)||b.addEventListener(a,this.G,aA)}})}va(){const a=this.C.keys();for(const b of a)cA(this,b);for(const b of bA){let c;
null==(c=this.ia)||c.removeEventListener(b,this.G,aA)}}F(a){if(this.ia){var b=this.C.get(a);if(b)return b;var c;null==(c=a.querySelector(".mat-ripple"))||c.remove();b=this.ia.createElement("span");b.classList.add("mat-ripple",a.getAttribute("mat-ripple-loader-class-name"));a.append(b);b=new Lz(new wi(b),this.qa,this.Ma,this.D?this.D:void 0,this.Yc?this.Yc:void 0);b.C=!0;b.Oe=a;b.bj=a.hasAttribute("mat-ripple-loader-centered");b.disabled=a.hasAttribute("mat-ripple-loader-disabled");eA(this,a,b);return b}}};
fA.K=function(a){return new (a||fA)};fA.na=Pd({ma:fA,ba:fA.K,fa:"root"});const gA=["panel"];function hA(a,b){if(a&1){const c=Zg();R(0,"div",1,0);V("@panelAnimation.done",function(d){t(c);const e=Y();return x(e.jg.next(d))});$q(2);U()}a&2&&(a=b.id,b=Y(),sq(vq,b.Se),Wp("mat-mdc-autocomplete-visible",b.em)("mat-mdc-autocomplete-hidden",!b.em)("mat-primary","primary"===b.oh)("mat-accent","accent"===b.oh)("mat-warn","warn"===b.oh),Q("id",b.id)("@panelAnimation",b.Fw?"visible":"hidden"),Mp("aria-label",b.ariaLabel||null)("aria-labelledby",b.zp(a)))}let iA=0;
var jA=class{constructor(a,b){this.source=a;this.gx=b}},lA=new gf("mat-autocomplete-default-options",{fa:"root",ba:kA});function kA(){return{Xm:!1,Zm:!1,Ge:!1,Pj:!1}}
var mA=class{get Fw(){return this.D&&this.em}ea(a){this.oh=a;en(this.Da)}set classList(a){this.Se=a;this.ra.R.className=""}get Ge(){return this.F}set Ge(a){this.F=a;if(this.options)for(const b of this.options)en(b.Da)}constructor(a,b,c,d){this.Da=a;this.ra=b;this.P=c;this.G=Si.EMPTY;this.jg=new Jk;this.D=this.em=!1;this.Fq=null;this.ix=new Jk;this.kf=new Jk;this.closed=new Jk;this.hx=new Jk;this.id=`mat-autocomplete-${iA++}`;this.EC=(null==d?void 0:d.T)||!1;this.Xm=!!c.Xm;this.Zm=!!c.Zm;this.Pj=!!c.Pj;
let e;this.F=null!=(e=this.P.Ge)?e:!1}Bj(){this.C=Jy(Ky(new Wy(this.options)),this.T);this.G=this.C.xb.subscribe(a=>{this.Fw&&this.hx.emit({source:this,gx:this.options.toArray()[a]||null})});this.O()}va(){let a;null==(a=this.C)||a.destroy();this.G.unsubscribe();this.jg.complete()}N(a){this.Xf&&(this.Xf.R.scrollTop=a)}ca(){return this.Xf?this.Xf.R.scrollTop:0}O(){this.em=!!this.options.length;en(this.Da)}aa(a){this.ix.emit(new jA(this,a))}zp(a){return this.ariaLabel?null:this.ad?(a?a+" ":"")+this.ad:
a}T(){return!1}};mA.K=function(a){return new (a||mA)(y(Tr),y(wi),y(lA),y(Wt))};
mA.Ca=Jf({type:mA,ha:[["mat-autocomplete"]],hc:function(a,b,c){a&1&&(ur(c,Vz,5),ur(c,uz,5));if(a&2){let d;wr(d=xr())&&(b.options=d);wr(d=xr())&&(b.Qr=d)}},eb:function(a,b){a&1&&(vr(dr,7),vr(gA,5));if(a&2){let c;wr(c=xr())&&(b.sa=c.first);wr(c=xr())&&(b.Xf=c.first)}},Ka:[1,"mat-mdc-autocomplete"],inputs:{ariaLabel:[0,"aria-label","ariaLabel"],ad:[0,"aria-labelledby","ariaLabelledby"],Fq:"displayWith",Xm:[2,"autoActiveFirstOption","autoActiveFirstOption",As],Zm:[2,"autoSelectActiveOption","autoSelectActiveOption",
As],Pj:[2,"requireSelection","requireSelection",As],Ig:"panelWidth",Nc:[2,"disableRipple","disableRipple",As],classList:[0,"class","classList"],Ge:[2,"hideSingleSelectionIndicator","hideSingleSelectionIndicator",As]},outputs:{ix:"optionSelected",kf:"opened",closed:"closed",hx:"optionActivated"},ib:["matAutocomplete"],ka:!0,features:[Lr([{pa:rz,Ab:mA}]),Lp,Nr],bc:["*"],ya:1,za:0,Ba:[["panel",""],["role","listbox",1,"mat-mdc-autocomplete-panel","mdc-menu-surface","mdc-menu-surface--open",3,"id"]],sa:function(a){a&
1&&(Zq(),H(0,hA,3,16,"ng-template"))},styles:["div.mat-mdc-autocomplete-panel{width:100%;max-height:256px;visibility:hidden;transform-origin:center top;overflow:auto;padding:8px 0;box-sizing:border-box;position:static;border-radius:var(--mat-autocomplete-container-shape);box-shadow:var(--mat-autocomplete-container-elevation-shadow);background-color:var(--mat-autocomplete-background-color)}.cdk-high-contrast-active div.mat-mdc-autocomplete-panel{outline:solid 1px}.cdk-overlay-pane:not(.mat-mdc-autocomplete-panel-above) div.mat-mdc-autocomplete-panel{border-top-left-radius:0;border-top-right-radius:0}.mat-mdc-autocomplete-panel-above div.mat-mdc-autocomplete-panel{border-bottom-left-radius:0;border-bottom-right-radius:0;transform-origin:center bottom}div.mat-mdc-autocomplete-panel.mat-mdc-autocomplete-visible{visibility:visible}div.mat-mdc-autocomplete-panel.mat-mdc-autocomplete-hidden{visibility:hidden;pointer-events:none}mat-autocomplete{display:none}\n"],
pb:2,data:{animation:[kz]},Pa:0});var nA=new gf("MatError");let oA=0;var pA=class{constructor(){this.align="start";this.id=`mat-mdc-hint-${oA++}`}};pA.K=function(a){return new (a||pA)};pA.Fa=Uf({type:pA,ha:[["mat-hint"]],Ka:[1,"mat-mdc-form-field-hint","mat-mdc-form-field-bottom-align"],Za:4,Ia:function(a,b){a&2&&(Sq("id",b.id),Mp("align",null),Wp("mat-mdc-form-field-hint-end","end"===b.align))},inputs:{align:"align",id:"id"},ka:!0});var qA=class{};qA.K=function(a){return new (a||qA)};qA.Fa=Uf({type:qA,ha:[["mat-label"]],ka:!0});var rA=new gf("MatPrefix");var sA=new gf("MatSuffix");var tA=Yy("transitionMessages",[az("enter",$y({opacity:1,transform:"translateY(0%)"})),bz("void => enter",[$y({opacity:0,transform:"translateY(-5px)"}),Zy("300ms cubic-bezier(0.55, 0, 0.55, 0.2)")])]);var uA=class{};uA.K=function(a){return new (a||uA)};uA.Fa=Uf({type:uA});class vA{constructor(a){this.G=a;this.Ga=new hj;this.D=new hj;this.C=new Map;"undefined"!==typeof ResizeObserver&&(this.F=new ResizeObserver(b=>this.D.next(b)))}observe(a){this.C.has(a)||this.C.set(a,C(new bj(b=>{const c=this.D.subscribe(b);let d;null==(d=this.F)||d.observe(a,{box:this.G});return()=>{let e;null==(e=this.F)||e.unobserve(a);c.unsubscribe();this.C.delete(a)}}),xk(b=>b.some(c=>c.target===a)),Fo(),Oo(this.Ga)));return this.C.get(a)}destroy(){this.Ga.next();this.Ga.complete();this.D.complete();
this.C.clear()}}var wA=class{constructor(){this.C=new Map;this.qa=of(Tk)}va(){for(const [,a]of this.C)a.destroy();this.C.clear()}observe(a,b){b=(null==b?void 0:b.box)||"content-box";this.C.has(b)||this.C.set(b,new vA(b));return this.C.get(b).observe(a)}};wA.K=function(a){return new (a||wA)};wA.na=Pd({ma:wA,ba:wA.K,fa:"root"});var xA=new gf("FloatingLabelParent"),yA=class{get Pq(){return this.F}set Pq(a){this.F=a;this.Cr&&this.G()}get Cr(){return this.D}set Cr(a){(this.D=a)?this.N():this.C.unsubscribe()}constructor(a){this.ra=a;this.D=this.F=!1;this.J=of(wA);this.qa=of(Tk);this.Sa=of(xA);this.C=new Si}va(){this.C.unsubscribe()}get element(){return this.ra.R}G(){setTimeout(()=>this.Sa.nA())}N(){this.C.unsubscribe();Qk(this.qa,()=>{this.C=this.J.observe(this.ra.R,{box:"border-box"}).subscribe(()=>this.G())})}};
yA.K=function(a){return new (a||yA)(y(wi))};yA.Fa=Uf({type:yA,ha:[["label","matFormFieldFloatingLabel",""]],Ka:[1,"mdc-floating-label","mat-mdc-floating-label"],Za:2,Ia:function(a,b){a&2&&Wp("mdc-floating-label--float-above",b.Pq)},inputs:{Pq:"floating",Cr:"monitorResize"},ka:!0});var zA=class{constructor(a,b){this.ra=a;this.C=c=>{const d=this.ra.R.classList,e=d.contains("mdc-line-ripple--deactivating");"opacity"===c.propertyName&&e&&d.remove("mdc-line-ripple--active","mdc-line-ripple--deactivating")};Qk(b,()=>{a.R.addEventListener("transitionend",this.C)})}pn(){this.ra.R.classList.add("mdc-line-ripple--deactivating")}va(){this.ra.R.removeEventListener("transitionend",this.C)}};zA.K=function(a){return new (a||zA)(y(wi),y(Tk))};
zA.Fa=Uf({type:zA,ha:[["div","matFormFieldLineRipple",""]],Ka:[1,"mdc-line-ripple"],ka:!0});const AA=["notch"];
var BA=class{constructor(a,b){this.ra=a;this.qa=b;this.open=!1}pd(){const a=this.ra.R.querySelector(".mdc-floating-label");a?(this.ra.R.classList.add("mdc-notched-outline--upgraded"),"function"===typeof requestAnimationFrame&&(a.style.transitionDuration="0s",Qk(this.qa,()=>{requestAnimationFrame(()=>a.style.transitionDuration="")}))):this.ra.R.classList.add("mdc-notched-outline--no-label")}C(a){this.yA.R.style.width=this.open&&a?`calc(${a}px * var(--mat-mdc-form-field-floating-label-scale, 0.75) + ${9}px)`:""}};
BA.K=function(a){return new (a||BA)(y(wi),y(Tk))};
BA.Ca=Jf({type:BA,ha:[["div","matFormFieldNotchedOutline",""]],eb:function(a,b){a&1&&vr(AA,5);if(a&2){let c;wr(c=xr())&&(b.yA=c.first)}},Ka:[1,"mdc-notched-outline"],Za:2,Ia:function(a,b){a&2&&Wp("mdc-notched-outline--notched",b.open)},inputs:{open:[0,"matFormFieldNotchedOutlineOpen","open"]},ka:!0,features:[Nr],attrs:["matFormFieldNotchedOutline",""],bc:["*"],ya:5,za:0,Ba:[["notch",""],[1,"mdc-notched-outline__leading"],[1,"mdc-notched-outline__notch"],[1,"mdc-notched-outline__trailing"]],sa:function(a){a&
1&&(Zq(),Oq(0,"div",1),R(1,"div",2,0),$q(3),U(),Oq(4,"div",3))},pb:2,Pa:0});const CA=["textField"],DA=["iconPrefixContainer"],EA=["textPrefixContainer"],FA=["*",[["mat-label"]],[["","matPrefix",""],["","matIconPrefix",""]],[["","matTextPrefix",""]],[["","matTextSuffix",""]],[["","matSuffix",""],["","matIconSuffix",""]],[["mat-error"],["","matError",""]],[["mat-hint",3,"align","end"]],[["mat-hint","align","end"]]];function GA(a){a&1&&Oq(0,"span",19)}
function HA(a){a&1&&(R(0,"label",18),$q(1,1),H(2,GA,1,0,"span",19),U());a&2&&(a=Y(2),Q("floating",a.Gk())("monitorResize",a.Af())("id",a.Xe),Mp("for",a.Hb.NB?null:a.Hb.id),D(2),Gq(!a.Pn&&a.Hb.required?2:-1))}function IA(a){a&1&&H(0,HA,3,5,"label",18);a&2&&(a=Y(),Gq(a.rh()?0:-1))}function JA(a){a&1&&Oq(0,"div",5)}function KA(){}function LA(a){a&1&&H(0,KA,0,0,"ng-template",11);a&2&&(Y(2),a=Ar(1),Q("ngTemplateOutlet",a))}
function MA(a){a&1&&(R(0,"div",7),H(1,LA,1,1,null,11),U());a&2&&(a=Y(),Q("matFormFieldNotchedOutlineOpen",a.Gk()),D(),Gq(a.Ot()?-1:1))}function NA(a){a&1&&(R(0,"div",8,2),$q(2,2),U())}function OA(a){a&1&&(R(0,"div",9,3),$q(2,3),U())}function PA(){}function QA(a){a&1&&H(0,PA,0,0,"ng-template",11);a&2&&(Y(),a=Ar(1),Q("ngTemplateOutlet",a))}function RA(a){a&1&&(R(0,"div",12),$q(1,4),U())}function SA(a){a&1&&(R(0,"div",13),$q(1,5),U())}function TA(a){a&1&&Oq(0,"div",14)}
function UA(a){a&1&&(R(0,"div",16),$q(1,6),U());a&2&&(a=Y(),Q("@transitionMessages",a.Sp))}function VA(a){a&1&&(R(0,"mat-hint",20),Z(1),U());a&2&&(a=Y(2),Q("id",a.eu),D(),Br(a.Rn))}function WA(a){a&1&&(R(0,"div",17),H(1,VA,2,2,"mat-hint",20),$q(2,7),Oq(3,"div",21),$q(4,8),U());a&2&&(a=Y(),Q("@transitionMessages",a.Sp),D(),Gq(a.Rn?1:-1))}var XA=new gf("MatFormField"),YA=new gf("MAT_FORM_FIELD_DEFAULT_OPTIONS");let ZA=0;function $A(a){return a.Fu||a.ra}
var bB=class{get Pn(){return this.N}set Pn(a){this.N=Rs(a)}get yg(){let a;return this.T||(null==(a=this.G)?void 0:a.yg)||"auto"}set yg(a){a!==this.T&&(this.T=a,en(this.Da))}get Gd(){return this.D}set Gd(a){const b=this.D;let c;this.D=a||(null==(c=this.G)?void 0:c.Gd)||"fill";"outline"===this.D&&this.D!==b&&(this.C=!0)}get im(){let a;return this.ea||(null==(a=this.G)?void 0:a.im)||"fixed"}set im(a){let b;this.ea=a||(null==(b=this.G)?void 0:b.im)||"fixed"}get Rn(){return this.O}set Rn(a){this.O=a;this.ca()}get Hb(){return this.ta||
this.fA}set Hb(a){this.ta=a}constructor(a,b,c,d,e,f,g){this.ra=a;this.Da=b;this.P=d;this.Ma=e;this.G=f;this.Yc=g;this.N=!1;this.color="primary";this.D="fill";this.ea=null;this.O="";this.cu=this.Dp=this.bu=this.Cp=!1;this.Xe=`mat-mdc-form-field-label-${ZA++}`;this.eu=`mat-mdc-hint-${ZA++}`;this.Sp="";this.Ga=new hj;this.F=null;this.C=!1;this.Ra=of(ai);f&&(f.Gd&&(this.Gd=f.Gd),this.N=!(null==f||!f.Pn),f.color&&(this.color=f.color))}pd(){this.ja();this.Sp="enter";fn(this.Da)}Bj(){this.ua();this.Ja();
this.Ea();this.wa()}Yw(){}va(){this.Ga.next();this.Ga.complete()}oa(){this.rh()&&(this.yg="always")}ua(){const a=this.Hb;a.zv&&this.ra.R.classList.add(`mat-mdc-form-field-type-${a.zv}`);a.Tc.subscribe(()=>{this.ja();this.J();en(this.Da)});a.Ld&&a.Ld.dh&&C(a.Ld.dh,Oo(this.Ga)).subscribe(()=>en(this.Da))}aa(){this.Cp=!!this.Dk.find(a=>!a.C);this.bu=!!this.Dk.find(a=>a.C);this.Dp=!!this.Tp.find(a=>!a.C);this.cu=!!this.Tp.find(a=>a.C)}Ea(){this.aa();wk(this.Dk.C,this.Tp.C).subscribe(()=>{this.aa();en(this.Da)})}Ja(){this.wk.C.subscribe(()=>
{this.ca();en(this.Da)});this.sk.C.subscribe(()=>{this.J();en(this.Da)});this.J()}fb(){}ja(){if(this.Hb.focused&&!this.F){this.F=!0;var a;null!=(a=this.pu)&&(a=a.ra.R.classList,a.remove("mdc-line-ripple--deactivating"),a.add("mdc-line-ripple--active"))}else this.Hb.focused||!this.F&&null!==this.F||(this.F=!1,null==(a=this.pu)||a.pn());let b;null==(b=this.Fu)||b.R.classList.toggle("mdc-text-field--focused",this.Hb.focused)}wa(){this.Dk.C.subscribe(()=>this.C=!0);al(()=>{this.C&&(this.C=!1,this.Va())},
{xa:this.Ra});C(this.P.xb,Oo(this.Ga)).subscribe(()=>this.C=!0)}Bu(){return"always"===this.yg}Af(){return"outline"===this.Gd}Ot(){return!this.Ma.isBrowser&&this.Dk.length&&!this.Gk()}rh(){return!!this.vA||!!this.wA}Gk(){return this.Hb.hE||this.Bu()}xh(a){const b=this.Hb?this.Hb.Ld:null;return b&&b[a]}Tt(){return this.sk&&0<this.sk.length&&this.Hb.De?"error":"hint"}nA(){this.Ta()}Ta(){if(this.Af()&&this.Em&&this.Gk()){var a;if(null!=(a=this.qu)){var b=a.C;var c=this.Em.ra.R;if(null!==c.offsetParent)c=
c.scrollWidth;else{c=c.cloneNode(!0);c.style.setProperty("position","absolute");c.style.setProperty("transform","translate(-9999px, -9999px)");document.documentElement.appendChild(c);var d=c.scrollWidth;c.remove();c=d}b.call(a,c)}}else null==(a=this.qu)||a.C(0)}ca(){this.J()}gb(){}J(){if(this.Hb){let a=[];this.Hb.Vs&&"string"===typeof this.Hb.Vs&&a.push(...this.Hb.Vs.split(" "));if("hint"===this.Tt()){const b=this.wk?this.wk.find(d=>"start"===d.align):null,c=this.wk?this.wk.find(d=>"end"===d.align):
null;b?a.push(b.id):this.O&&a.push(this.eu);c&&a.push(c.id)}else this.sk&&a.push(...this.sk.map(b=>b.id));aB(this.Hb,a)}}Va(){if(this.Af()&&this.Em){var a=this.Em.element;if(this.fu||this.Gu)if(this.Qa()){var b,c=null==(b=this.fu)?void 0:b.R,d;b=null==(d=this.Gu)?void 0:d.R;var e;c=null!=(e=null==c?void 0:c.getBoundingClientRect().width)?e:0;var f;e=null!=(f=null==b?void 0:b.getBoundingClientRect().width)?f:0;a.style.transform=`var(
        --mat-mdc-form-field-label-transform,
        ${"translateY(-50%)"} translateX(${`calc(${"rtl"===this.P.value?"-1":"1"} * (${`${c+e}px`} + ${"var(--mat-mdc-form-field-label-offset-x, 0px)"}))`})
    )`}else this.C=!0;else a.style.transform=""}}Qa(){const a=this.ra.R;if(a.getRootNode){const b=a.getRootNode();return b&&b!==a}return document.documentElement.contains(a)}};bB.K=function(a){return new (a||bB)(y(wi),y(Tr),y(Tk),y(pu),y(Wt),y(YA,8),y(qi,8),y(kt))};
bB.Ca=Jf({type:bB,ha:[["mat-form-field"]],hc:function(a,b,c){a&1&&(ur(c,qA,5),ur(c,qA,7),ur(c,uA,5),ur(c,rA,5),ur(c,sA,5),ur(c,nA,5),ur(c,pA,5));if(a&2){let d;wr(d=xr())&&(b.vA=d.first);wr(d=xr())&&(b.wA=d.first);wr(d=xr())&&(b.fA=d.first);wr(d=xr())&&(b.Dk=d);wr(d=xr())&&(b.Tp=d);wr(d=xr())&&(b.sk=d);wr(d=xr())&&(b.wk=d)}},eb:function(a,b){a&1&&(vr(CA,5),vr(DA,5),vr(EA,5),vr(yA,5),vr(BA,5),vr(zA,5));if(a&2){let c;wr(c=xr())&&(b.Fu=c.first);wr(c=xr())&&(b.fu=c.first);wr(c=xr())&&(b.Gu=c.first);wr(c=
xr())&&(b.Em=c.first);wr(c=xr())&&(b.qu=c.first);wr(c=xr())&&(b.pu=c.first)}},Ka:[1,"mat-mdc-form-field"],Za:42,Ia:function(a,b){a&2&&Wp("mat-mdc-form-field-label-always-float",b.Bu())("mat-mdc-form-field-has-icon-prefix",b.Cp)("mat-mdc-form-field-has-icon-suffix",b.Dp)("mat-form-field-invalid",b.Hb.De)("mat-form-field-disabled",b.Hb.disabled)("mat-form-field-autofilled",b.Hb.MF)("mat-form-field-no-animations","NoopAnimations"===b.Yc)("mat-form-field-appearance-fill","fill"==b.Gd)("mat-form-field-appearance-outline",
"outline"==b.Gd)("mat-form-field-hide-placeholder",b.rh()&&!b.Gk())("mat-focused",b.Hb.focused)("mat-primary","accent"!==b.color&&"warn"!==b.color)("mat-accent","accent"===b.color)("mat-warn","warn"===b.color)("ng-untouched",b.xh("untouched"))("ng-touched",b.xh("touched"))("ng-pristine",b.xh("pristine"))("ng-dirty",b.xh("dirty"))("ng-valid",b.xh("valid"))("ng-invalid",b.xh("invalid"))("ng-pending",b.xh("pending"))},inputs:{Pn:"hideRequiredMarker",color:"color",yg:"floatLabel",Gd:"appearance",im:"subscriptSizing",
Rn:"hintLabel"},ib:["matFormField"],ka:!0,features:[Lr([{pa:XA,Ab:bB},{pa:xA,Ab:bB}]),Nr],bc:"*;mat-label;[matPrefix], [matIconPrefix];[matTextPrefix];[matTextSuffix];[matSuffix], [matIconSuffix];mat-error, [matError];mat-hint:not([align='end']);mat-hint[align='end']".split(";"),ya:18,za:21,Ba:[["labelTemplate",""],["textField",""],["iconPrefixContainer",""],["textPrefixContainer",""],[1,"mat-mdc-text-field-wrapper","mdc-text-field",3,"click"],[1,"mat-mdc-form-field-focus-overlay"],[1,"mat-mdc-form-field-flex"],
["matFormFieldNotchedOutline","",3,"matFormFieldNotchedOutlineOpen"],[1,"mat-mdc-form-field-icon-prefix"],[1,"mat-mdc-form-field-text-prefix"],[1,"mat-mdc-form-field-infix"],[3,"ngTemplateOutlet"],[1,"mat-mdc-form-field-text-suffix"],[1,"mat-mdc-form-field-icon-suffix"],["matFormFieldLineRipple",""],[1,"mat-mdc-form-field-subscript-wrapper","mat-mdc-form-field-bottom-align"],[1,"mat-mdc-form-field-error-wrapper"],[1,"mat-mdc-form-field-hint-wrapper"],["matFormFieldFloatingLabel","",3,"floating","monitorResize",
"id"],["aria-hidden","true",1,"mat-mdc-form-field-required-marker","mdc-floating-label--required"],[3,"id"],[1,"mat-mdc-form-field-hint-spacer"]],sa:function(a,b){if(a&1){const c=Zg();Zq(FA);H(0,IA,1,1,"ng-template",null,0,Rr);R(2,"div",4,1);V("click",function(){t(c);var d=b.Hb;d.focus();d.open();return x()});H(4,JA,1,0,"div",5);R(5,"div",6);H(6,MA,2,2,"div",7)(7,NA,3,0,"div",8)(8,OA,3,0,"div",9);R(9,"div",10);H(10,QA,1,1,null,11);$q(11);U();H(12,RA,2,0,"div",12)(13,SA,2,0,"div",13);U();H(14,TA,1,
0,"div",14);U();R(15,"div",15);H(16,UA,2,1,"div",16)(17,WA,5,2,"div",17);U()}if(a&2){let c;D(2);Wp("mdc-text-field--filled",!b.Af())("mdc-text-field--outlined",b.Af())("mdc-text-field--no-label",!b.rh())("mdc-text-field--disabled",b.Hb.disabled)("mdc-text-field--invalid",b.Hb.De);D(2);Gq(b.Af()||b.Hb.disabled?-1:4);D(2);Gq(b.Af()?6:-1);D();Gq(b.Cp?7:-1);D();Gq(b.bu?8:-1);D(2);Gq(!b.Af()||b.Ot()?10:-1);D(2);Gq(b.cu?12:-1);D();Gq(b.Dp?13:-1);D();Gq(b.Af()?-1:14);D();Wp("mat-mdc-form-field-subscript-dynamic-size",
"dynamic"===b.im);D();Gq("error"===(c=b.Tt())?16:"hint"===c?17:-1)}},Ha:[yA,BA,ht,zA,pA],styles:['.mdc-text-field{border-top-left-radius:4px;border-top-left-radius:var(--mdc-shape-small, 4px);border-top-right-radius:4px;border-top-right-radius:var(--mdc-shape-small, 4px);border-bottom-right-radius:0;border-bottom-left-radius:0;display:inline-flex;align-items:baseline;padding:0 16px;position:relative;box-sizing:border-box;overflow:hidden;will-change:opacity,transform,color}.mdc-text-field .mdc-floating-label{top:50%;transform:translateY(-50%);pointer-events:none}.mdc-text-field__input{height:28px;width:100%;min-width:0;border:none;border-radius:0;background:none;appearance:none;padding:0}.mdc-text-field__input::-ms-clear{display:none}.mdc-text-field__input::-webkit-calendar-picker-indicator{display:none}.mdc-text-field__input:focus{outline:none}.mdc-text-field__input:invalid{box-shadow:none}@media all{.mdc-text-field__input::placeholder{opacity:0}}@media all{.mdc-text-field__input:-ms-input-placeholder{opacity:0}}@media all{.mdc-text-field--no-label .mdc-text-field__input::placeholder,.mdc-text-field--focused .mdc-text-field__input::placeholder{opacity:1}}@media all{.mdc-text-field--no-label .mdc-text-field__input:-ms-input-placeholder,.mdc-text-field--focused .mdc-text-field__input:-ms-input-placeholder{opacity:1}}.mdc-text-field__affix{height:28px;opacity:0;white-space:nowrap}.mdc-text-field--label-floating .mdc-text-field__affix,.mdc-text-field--no-label .mdc-text-field__affix{opacity:1}@supports(-webkit-hyphens: none){.mdc-text-field--outlined .mdc-text-field__affix{align-items:center;align-self:center;display:inline-flex;height:100%}}.mdc-text-field__affix--prefix{padding-left:0;padding-right:2px}[dir=rtl] .mdc-text-field__affix--prefix,.mdc-text-field__affix--prefix[dir=rtl]{padding-left:2px;padding-right:0}.mdc-text-field--end-aligned .mdc-text-field__affix--prefix{padding-left:0;padding-right:12px}[dir=rtl] .mdc-text-field--end-aligned .mdc-text-field__affix--prefix,.mdc-text-field--end-aligned .mdc-text-field__affix--prefix[dir=rtl]{padding-left:12px;padding-right:0}.mdc-text-field__affix--suffix{padding-left:12px;padding-right:0}[dir=rtl] .mdc-text-field__affix--suffix,.mdc-text-field__affix--suffix[dir=rtl]{padding-left:0;padding-right:12px}.mdc-text-field--end-aligned .mdc-text-field__affix--suffix{padding-left:2px;padding-right:0}[dir=rtl] .mdc-text-field--end-aligned .mdc-text-field__affix--suffix,.mdc-text-field--end-aligned .mdc-text-field__affix--suffix[dir=rtl]{padding-left:0;padding-right:2px}.mdc-text-field--filled{height:56px}.mdc-text-field--filled::before{display:inline-block;width:0;height:40px;content:"";vertical-align:0}.mdc-text-field--filled .mdc-floating-label{left:16px;right:initial}[dir=rtl] .mdc-text-field--filled .mdc-floating-label,.mdc-text-field--filled .mdc-floating-label[dir=rtl]{left:initial;right:16px}.mdc-text-field--filled .mdc-floating-label--float-above{transform:translateY(-106%) scale(0.75)}.mdc-text-field--filled.mdc-text-field--no-label .mdc-text-field__input{height:100%}.mdc-text-field--filled.mdc-text-field--no-label .mdc-floating-label{display:none}.mdc-text-field--filled.mdc-text-field--no-label::before{display:none}@supports(-webkit-hyphens: none){.mdc-text-field--filled.mdc-text-field--no-label .mdc-text-field__affix{align-items:center;align-self:center;display:inline-flex;height:100%}}.mdc-text-field--outlined{height:56px;overflow:visible}.mdc-text-field--outlined .mdc-floating-label--float-above{transform:translateY(-37.25px) scale(1)}.mdc-text-field--outlined.mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-text-field--outlined .mdc-notched-outline--upgraded .mdc-floating-label--float-above{transform:translateY(-34.75px) scale(0.75)}.mdc-text-field--outlined .mdc-floating-label--float-above{font-size:.75rem}.mdc-text-field--outlined.mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-text-field--outlined .mdc-notched-outline--upgraded .mdc-floating-label--float-above{font-size:1rem}.mdc-text-field--outlined .mdc-text-field__input{height:100%}.mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__leading{border-top-left-radius:4px;border-top-left-radius:var(--mdc-shape-small, 4px);border-top-right-radius:0;border-bottom-right-radius:0;border-bottom-left-radius:4px;border-bottom-left-radius:var(--mdc-shape-small, 4px)}[dir=rtl] .mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__leading,.mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__leading[dir=rtl]{border-top-left-radius:0;border-top-right-radius:4px;border-top-right-radius:var(--mdc-shape-small, 4px);border-bottom-right-radius:4px;border-bottom-right-radius:var(--mdc-shape-small, 4px);border-bottom-left-radius:0}@supports(top: max(0%)){.mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__leading{width:max(12px,var(--mdc-shape-small, 4px))}}@supports(top: max(0%)){.mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__notch{max-width:calc(100% - max(12px,var(--mdc-shape-small, 4px))*2)}}.mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__trailing{border-top-left-radius:0;border-top-right-radius:4px;border-top-right-radius:var(--mdc-shape-small, 4px);border-bottom-right-radius:4px;border-bottom-right-radius:var(--mdc-shape-small, 4px);border-bottom-left-radius:0}[dir=rtl] .mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__trailing,.mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__trailing[dir=rtl]{border-top-left-radius:4px;border-top-left-radius:var(--mdc-shape-small, 4px);border-top-right-radius:0;border-bottom-right-radius:0;border-bottom-left-radius:4px;border-bottom-left-radius:var(--mdc-shape-small, 4px)}@supports(top: max(0%)){.mdc-text-field--outlined{padding-left:max(16px,calc(var(--mdc-shape-small, 4px) + 4px))}}@supports(top: max(0%)){.mdc-text-field--outlined{padding-right:max(16px,var(--mdc-shape-small, 4px))}}@supports(top: max(0%)){.mdc-text-field--outlined+.mdc-text-field-helper-line{padding-left:max(16px,calc(var(--mdc-shape-small, 4px) + 4px))}}@supports(top: max(0%)){.mdc-text-field--outlined+.mdc-text-field-helper-line{padding-right:max(16px,var(--mdc-shape-small, 4px))}}.mdc-text-field--outlined.mdc-text-field--with-leading-icon{padding-left:0}@supports(top: max(0%)){.mdc-text-field--outlined.mdc-text-field--with-leading-icon{padding-right:max(16px,var(--mdc-shape-small, 4px))}}[dir=rtl] .mdc-text-field--outlined.mdc-text-field--with-leading-icon,.mdc-text-field--outlined.mdc-text-field--with-leading-icon[dir=rtl]{padding-right:0}@supports(top: max(0%)){[dir=rtl] .mdc-text-field--outlined.mdc-text-field--with-leading-icon,.mdc-text-field--outlined.mdc-text-field--with-leading-icon[dir=rtl]{padding-left:max(16px,var(--mdc-shape-small, 4px))}}.mdc-text-field--outlined.mdc-text-field--with-trailing-icon{padding-right:0}@supports(top: max(0%)){.mdc-text-field--outlined.mdc-text-field--with-trailing-icon{padding-left:max(16px,calc(var(--mdc-shape-small, 4px) + 4px))}}[dir=rtl] .mdc-text-field--outlined.mdc-text-field--with-trailing-icon,.mdc-text-field--outlined.mdc-text-field--with-trailing-icon[dir=rtl]{padding-left:0}@supports(top: max(0%)){[dir=rtl] .mdc-text-field--outlined.mdc-text-field--with-trailing-icon,.mdc-text-field--outlined.mdc-text-field--with-trailing-icon[dir=rtl]{padding-right:max(16px,calc(var(--mdc-shape-small, 4px) + 4px))}}.mdc-text-field--outlined.mdc-text-field--with-leading-icon.mdc-text-field--with-trailing-icon{padding-left:0;padding-right:0}.mdc-text-field--outlined .mdc-notched-outline--notched .mdc-notched-outline__notch{padding-top:1px}.mdc-text-field--outlined .mdc-floating-label{left:4px;right:initial}[dir=rtl] .mdc-text-field--outlined .mdc-floating-label,.mdc-text-field--outlined .mdc-floating-label[dir=rtl]{left:initial;right:4px}.mdc-text-field--outlined .mdc-text-field__input{display:flex;border:none !important;background-color:rgba(0,0,0,0)}.mdc-text-field--outlined .mdc-notched-outline{z-index:1}.mdc-text-field--textarea{flex-direction:column;align-items:center;width:auto;height:auto;padding:0}.mdc-text-field--textarea .mdc-floating-label{top:19px}.mdc-text-field--textarea .mdc-floating-label:not(.mdc-floating-label--float-above){transform:none}.mdc-text-field--textarea .mdc-text-field__input{flex-grow:1;height:auto;min-height:1.5rem;overflow-x:hidden;overflow-y:auto;box-sizing:border-box;resize:none;padding:0 16px}.mdc-text-field--textarea.mdc-text-field--filled::before{display:none}.mdc-text-field--textarea.mdc-text-field--filled .mdc-floating-label--float-above{transform:translateY(-10.25px) scale(0.75)}.mdc-text-field--textarea.mdc-text-field--filled .mdc-text-field__input{margin-top:23px;margin-bottom:9px}.mdc-text-field--textarea.mdc-text-field--filled.mdc-text-field--no-label .mdc-text-field__input{margin-top:16px;margin-bottom:16px}.mdc-text-field--textarea.mdc-text-field--outlined .mdc-notched-outline--notched .mdc-notched-outline__notch{padding-top:0}.mdc-text-field--textarea.mdc-text-field--outlined .mdc-floating-label--float-above{transform:translateY(-27.25px) scale(1)}.mdc-text-field--textarea.mdc-text-field--outlined.mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-text-field--textarea.mdc-text-field--outlined .mdc-notched-outline--upgraded .mdc-floating-label--float-above{transform:translateY(-24.75px) scale(0.75)}.mdc-text-field--textarea.mdc-text-field--outlined .mdc-floating-label--float-above{font-size:.75rem}.mdc-text-field--textarea.mdc-text-field--outlined.mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-text-field--textarea.mdc-text-field--outlined .mdc-notched-outline--upgraded .mdc-floating-label--float-above{font-size:1rem}.mdc-text-field--textarea.mdc-text-field--outlined .mdc-text-field__input{margin-top:16px;margin-bottom:16px}.mdc-text-field--textarea.mdc-text-field--outlined .mdc-floating-label{top:18px}.mdc-text-field--textarea.mdc-text-field--with-internal-counter .mdc-text-field__input{margin-bottom:2px}.mdc-text-field--textarea.mdc-text-field--with-internal-counter .mdc-text-field-character-counter{align-self:flex-end;padding:0 16px}.mdc-text-field--textarea.mdc-text-field--with-internal-counter .mdc-text-field-character-counter::after{display:inline-block;width:0;height:16px;content:"";vertical-align:-16px}.mdc-text-field--textarea.mdc-text-field--with-internal-counter .mdc-text-field-character-counter::before{display:none}.mdc-text-field__resizer{align-self:stretch;display:inline-flex;flex-direction:column;flex-grow:1;max-height:100%;max-width:100%;min-height:56px;min-width:fit-content;min-width:-moz-available;min-width:-webkit-fill-available;overflow:hidden;resize:both}.mdc-text-field--filled .mdc-text-field__resizer{transform:translateY(-1px)}.mdc-text-field--filled .mdc-text-field__resizer .mdc-text-field__input,.mdc-text-field--filled .mdc-text-field__resizer .mdc-text-field-character-counter{transform:translateY(1px)}.mdc-text-field--outlined .mdc-text-field__resizer{transform:translateX(-1px) translateY(-1px)}[dir=rtl] .mdc-text-field--outlined .mdc-text-field__resizer,.mdc-text-field--outlined .mdc-text-field__resizer[dir=rtl]{transform:translateX(1px) translateY(-1px)}.mdc-text-field--outlined .mdc-text-field__resizer .mdc-text-field__input,.mdc-text-field--outlined .mdc-text-field__resizer .mdc-text-field-character-counter{transform:translateX(1px) translateY(1px)}[dir=rtl] .mdc-text-field--outlined .mdc-text-field__resizer .mdc-text-field__input,[dir=rtl] .mdc-text-field--outlined .mdc-text-field__resizer .mdc-text-field-character-counter,.mdc-text-field--outlined .mdc-text-field__resizer .mdc-text-field__input[dir=rtl],.mdc-text-field--outlined .mdc-text-field__resizer .mdc-text-field-character-counter[dir=rtl]{transform:translateX(-1px) translateY(1px)}.mdc-text-field--with-leading-icon{padding-left:0;padding-right:16px}[dir=rtl] .mdc-text-field--with-leading-icon,.mdc-text-field--with-leading-icon[dir=rtl]{padding-left:16px;padding-right:0}.mdc-text-field--with-leading-icon.mdc-text-field--filled .mdc-floating-label{max-width:calc(100% - 48px);left:48px;right:initial}[dir=rtl] .mdc-text-field--with-leading-icon.mdc-text-field--filled .mdc-floating-label,.mdc-text-field--with-leading-icon.mdc-text-field--filled .mdc-floating-label[dir=rtl]{left:initial;right:48px}.mdc-text-field--with-leading-icon.mdc-text-field--filled .mdc-floating-label--float-above{max-width:calc(100%/0.75 - 64px/0.75)}.mdc-text-field--with-leading-icon.mdc-text-field--outlined .mdc-floating-label{left:36px;right:initial}[dir=rtl] .mdc-text-field--with-leading-icon.mdc-text-field--outlined .mdc-floating-label,.mdc-text-field--with-leading-icon.mdc-text-field--outlined .mdc-floating-label[dir=rtl]{left:initial;right:36px}.mdc-text-field--with-leading-icon.mdc-text-field--outlined :not(.mdc-notched-outline--notched) .mdc-notched-outline__notch{max-width:calc(100% - 60px)}.mdc-text-field--with-leading-icon.mdc-text-field--outlined .mdc-floating-label--float-above{transform:translateY(-37.25px) translateX(-32px) scale(1)}[dir=rtl] .mdc-text-field--with-leading-icon.mdc-text-field--outlined .mdc-floating-label--float-above,.mdc-text-field--with-leading-icon.mdc-text-field--outlined .mdc-floating-label--float-above[dir=rtl]{transform:translateY(-37.25px) translateX(32px) scale(1)}.mdc-text-field--with-leading-icon.mdc-text-field--outlined.mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-text-field--with-leading-icon.mdc-text-field--outlined .mdc-notched-outline--upgraded .mdc-floating-label--float-above{transform:translateY(-34.75px) translateX(-32px) scale(0.75)}[dir=rtl] .mdc-text-field--with-leading-icon.mdc-text-field--outlined.mdc-notched-outline--upgraded .mdc-floating-label--float-above,[dir=rtl] .mdc-text-field--with-leading-icon.mdc-text-field--outlined .mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-text-field--with-leading-icon.mdc-text-field--outlined.mdc-notched-outline--upgraded .mdc-floating-label--float-above[dir=rtl],.mdc-text-field--with-leading-icon.mdc-text-field--outlined .mdc-notched-outline--upgraded .mdc-floating-label--float-above[dir=rtl]{transform:translateY(-34.75px) translateX(32px) scale(0.75)}.mdc-text-field--with-leading-icon.mdc-text-field--outlined .mdc-floating-label--float-above{font-size:.75rem}.mdc-text-field--with-leading-icon.mdc-text-field--outlined.mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-text-field--with-leading-icon.mdc-text-field--outlined .mdc-notched-outline--upgraded .mdc-floating-label--float-above{font-size:1rem}.mdc-text-field--with-trailing-icon{padding-left:16px;padding-right:0}[dir=rtl] .mdc-text-field--with-trailing-icon,.mdc-text-field--with-trailing-icon[dir=rtl]{padding-left:0;padding-right:16px}.mdc-text-field--with-trailing-icon.mdc-text-field--filled .mdc-floating-label{max-width:calc(100% - 64px)}.mdc-text-field--with-trailing-icon.mdc-text-field--filled .mdc-floating-label--float-above{max-width:calc(100%/0.75 - 64px/0.75)}.mdc-text-field--with-trailing-icon.mdc-text-field--outlined :not(.mdc-notched-outline--notched) .mdc-notched-outline__notch{max-width:calc(100% - 60px)}.mdc-text-field--with-leading-icon.mdc-text-field--with-trailing-icon{padding-left:0;padding-right:0}.mdc-text-field--with-leading-icon.mdc-text-field--with-trailing-icon.mdc-text-field--filled .mdc-floating-label{max-width:calc(100% - 96px)}.mdc-text-field--with-leading-icon.mdc-text-field--with-trailing-icon.mdc-text-field--filled .mdc-floating-label--float-above{max-width:calc(100%/0.75 - 96px/0.75)}.mdc-text-field-helper-line{display:flex;justify-content:space-between;box-sizing:border-box}.mdc-text-field+.mdc-text-field-helper-line{padding-right:16px;padding-left:16px}.mdc-form-field>.mdc-text-field+label{align-self:flex-start}.mdc-text-field--focused .mdc-notched-outline__leading,.mdc-text-field--focused .mdc-notched-outline__notch,.mdc-text-field--focused .mdc-notched-outline__trailing{border-width:2px}.mdc-text-field--focused+.mdc-text-field-helper-line .mdc-text-field-helper-text:not(.mdc-text-field-helper-text--validation-msg){opacity:1}.mdc-text-field--focused.mdc-text-field--outlined .mdc-notched-outline--notched .mdc-notched-outline__notch{padding-top:2px}.mdc-text-field--focused.mdc-text-field--outlined.mdc-text-field--textarea .mdc-notched-outline--notched .mdc-notched-outline__notch{padding-top:0}.mdc-text-field--invalid+.mdc-text-field-helper-line .mdc-text-field-helper-text--validation-msg{opacity:1}.mdc-text-field--disabled{pointer-events:none}@media screen and (forced-colors: active){.mdc-text-field--disabled .mdc-text-field__input{background-color:Window}.mdc-text-field--disabled .mdc-floating-label{z-index:1}}.mdc-text-field--disabled .mdc-floating-label{cursor:default}.mdc-text-field--disabled.mdc-text-field--filled .mdc-text-field__ripple{display:none}.mdc-text-field--disabled .mdc-text-field__input{pointer-events:auto}.mdc-text-field--end-aligned .mdc-text-field__input{text-align:right}[dir=rtl] .mdc-text-field--end-aligned .mdc-text-field__input,.mdc-text-field--end-aligned .mdc-text-field__input[dir=rtl]{text-align:left}[dir=rtl] .mdc-text-field--ltr-text .mdc-text-field__input,[dir=rtl] .mdc-text-field--ltr-text .mdc-text-field__affix,.mdc-text-field--ltr-text[dir=rtl] .mdc-text-field__input,.mdc-text-field--ltr-text[dir=rtl] .mdc-text-field__affix{direction:ltr}[dir=rtl] .mdc-text-field--ltr-text .mdc-text-field__affix--prefix,.mdc-text-field--ltr-text[dir=rtl] .mdc-text-field__affix--prefix{padding-left:0;padding-right:2px}[dir=rtl] .mdc-text-field--ltr-text .mdc-text-field__affix--suffix,.mdc-text-field--ltr-text[dir=rtl] .mdc-text-field__affix--suffix{padding-left:12px;padding-right:0}[dir=rtl] .mdc-text-field--ltr-text .mdc-text-field__icon--leading,.mdc-text-field--ltr-text[dir=rtl] .mdc-text-field__icon--leading{order:1}[dir=rtl] .mdc-text-field--ltr-text .mdc-text-field__affix--suffix,.mdc-text-field--ltr-text[dir=rtl] .mdc-text-field__affix--suffix{order:2}[dir=rtl] .mdc-text-field--ltr-text .mdc-text-field__input,.mdc-text-field--ltr-text[dir=rtl] .mdc-text-field__input{order:3}[dir=rtl] .mdc-text-field--ltr-text .mdc-text-field__affix--prefix,.mdc-text-field--ltr-text[dir=rtl] .mdc-text-field__affix--prefix{order:4}[dir=rtl] .mdc-text-field--ltr-text .mdc-text-field__icon--trailing,.mdc-text-field--ltr-text[dir=rtl] .mdc-text-field__icon--trailing{order:5}[dir=rtl] .mdc-text-field--ltr-text.mdc-text-field--end-aligned .mdc-text-field__input,.mdc-text-field--ltr-text.mdc-text-field--end-aligned[dir=rtl] .mdc-text-field__input{text-align:right}[dir=rtl] .mdc-text-field--ltr-text.mdc-text-field--end-aligned .mdc-text-field__affix--prefix,.mdc-text-field--ltr-text.mdc-text-field--end-aligned[dir=rtl] .mdc-text-field__affix--prefix{padding-right:12px}[dir=rtl] .mdc-text-field--ltr-text.mdc-text-field--end-aligned .mdc-text-field__affix--suffix,.mdc-text-field--ltr-text.mdc-text-field--end-aligned[dir=rtl] .mdc-text-field__affix--suffix{padding-left:2px}.mdc-floating-label{position:absolute;left:0;-webkit-transform-origin:left top;transform-origin:left top;line-height:1.15rem;text-align:left;text-overflow:ellipsis;white-space:nowrap;cursor:text;overflow:hidden;will-change:transform}[dir=rtl] .mdc-floating-label,.mdc-floating-label[dir=rtl]{right:0;left:auto;-webkit-transform-origin:right top;transform-origin:right top;text-align:right}.mdc-floating-label--float-above{cursor:auto}.mdc-floating-label--required:not(.mdc-floating-label--hide-required-marker)::after{margin-left:1px;margin-right:0px;content:"*"}[dir=rtl] .mdc-floating-label--required:not(.mdc-floating-label--hide-required-marker)::after,.mdc-floating-label--required:not(.mdc-floating-label--hide-required-marker)[dir=rtl]::after{margin-left:0;margin-right:1px}.mdc-notched-outline{display:flex;position:absolute;top:0;right:0;left:0;box-sizing:border-box;width:100%;max-width:100%;height:100%;text-align:left;pointer-events:none}[dir=rtl] .mdc-notched-outline,.mdc-notched-outline[dir=rtl]{text-align:right}.mdc-notched-outline__leading,.mdc-notched-outline__notch,.mdc-notched-outline__trailing{box-sizing:border-box;height:100%;pointer-events:none}.mdc-notched-outline__trailing{flex-grow:1}.mdc-notched-outline__notch{flex:0 0 auto;width:auto}.mdc-notched-outline .mdc-floating-label{display:inline-block;position:relative;max-width:100%}.mdc-notched-outline .mdc-floating-label--float-above{text-overflow:clip}.mdc-notched-outline--upgraded .mdc-floating-label--float-above{max-width:133.3333333333%}.mdc-notched-outline--notched .mdc-notched-outline__notch{padding-left:0;padding-right:8px;border-top:none}[dir=rtl] .mdc-notched-outline--notched .mdc-notched-outline__notch,.mdc-notched-outline--notched .mdc-notched-outline__notch[dir=rtl]{padding-left:8px;padding-right:0}.mdc-notched-outline--no-label .mdc-notched-outline__notch{display:none}.mdc-line-ripple::before,.mdc-line-ripple::after{position:absolute;bottom:0;left:0;width:100%;border-bottom-style:solid;content:""}.mdc-line-ripple::before{z-index:1}.mdc-line-ripple::after{transform:scaleX(0);opacity:0;z-index:2}.mdc-line-ripple--active::after{transform:scaleX(1);opacity:1}.mdc-line-ripple--deactivating::after{opacity:0}.mdc-floating-label--float-above{transform:translateY(-106%) scale(0.75)}.mdc-notched-outline__leading,.mdc-notched-outline__notch,.mdc-notched-outline__trailing{border-top:1px solid;border-bottom:1px solid}.mdc-notched-outline__leading{border-left:1px solid;border-right:none;width:12px}[dir=rtl] .mdc-notched-outline__leading,.mdc-notched-outline__leading[dir=rtl]{border-left:none;border-right:1px solid}.mdc-notched-outline__trailing{border-left:none;border-right:1px solid}[dir=rtl] .mdc-notched-outline__trailing,.mdc-notched-outline__trailing[dir=rtl]{border-left:1px solid;border-right:none}.mdc-notched-outline__notch{max-width:calc(100% - 12px*2)}.mdc-line-ripple::before{border-bottom-width:1px}.mdc-line-ripple::after{border-bottom-width:2px}.mdc-text-field--filled{border-top-left-radius:var(--mdc-filled-text-field-container-shape);border-top-right-radius:var(--mdc-filled-text-field-container-shape);border-bottom-right-radius:0;border-bottom-left-radius:0}.mdc-text-field--filled:not(.mdc-text-field--disabled) .mdc-text-field__input{caret-color:var(--mdc-filled-text-field-caret-color)}.mdc-text-field--filled.mdc-text-field--invalid:not(.mdc-text-field--disabled) .mdc-text-field__input{caret-color:var(--mdc-filled-text-field-error-caret-color)}.mdc-text-field--filled:not(.mdc-text-field--disabled) .mdc-text-field__input{color:var(--mdc-filled-text-field-input-text-color)}.mdc-text-field--filled.mdc-text-field--disabled .mdc-text-field__input{color:var(--mdc-filled-text-field-disabled-input-text-color)}.mdc-text-field--filled:not(.mdc-text-field--disabled) .mdc-floating-label,.mdc-text-field--filled:not(.mdc-text-field--disabled) .mdc-floating-label--float-above{color:var(--mdc-filled-text-field-label-text-color)}.mdc-text-field--filled:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-floating-label,.mdc-text-field--filled:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-floating-label--float-above{color:var(--mdc-filled-text-field-focus-label-text-color)}.mdc-text-field--filled:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-floating-label,.mdc-text-field--filled:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-floating-label--float-above{color:var(--mdc-filled-text-field-hover-label-text-color)}.mdc-text-field--filled.mdc-text-field--disabled .mdc-floating-label,.mdc-text-field--filled.mdc-text-field--disabled .mdc-floating-label--float-above{color:var(--mdc-filled-text-field-disabled-label-text-color)}.mdc-text-field--filled.mdc-text-field--invalid:not(.mdc-text-field--disabled) .mdc-floating-label,.mdc-text-field--filled.mdc-text-field--invalid:not(.mdc-text-field--disabled) .mdc-floating-label--float-above{color:var(--mdc-filled-text-field-error-label-text-color)}.mdc-text-field--filled.mdc-text-field--invalid:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-floating-label,.mdc-text-field--filled.mdc-text-field--invalid:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-floating-label--float-above{color:var(--mdc-filled-text-field-error-focus-label-text-color)}.mdc-text-field--filled.mdc-text-field--invalid:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-floating-label,.mdc-text-field--filled.mdc-text-field--invalid:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-floating-label--float-above{color:var(--mdc-filled-text-field-error-hover-label-text-color)}.mdc-text-field--filled .mdc-floating-label{font-family:var(--mdc-filled-text-field-label-text-font);font-size:var(--mdc-filled-text-field-label-text-size);font-weight:var(--mdc-filled-text-field-label-text-weight);letter-spacing:var(--mdc-filled-text-field-label-text-tracking)}@media all{.mdc-text-field--filled:not(.mdc-text-field--disabled) .mdc-text-field__input::placeholder{color:var(--mdc-filled-text-field-input-text-placeholder-color)}}@media all{.mdc-text-field--filled:not(.mdc-text-field--disabled) .mdc-text-field__input:-ms-input-placeholder{color:var(--mdc-filled-text-field-input-text-placeholder-color)}}.mdc-text-field--filled:not(.mdc-text-field--disabled){background-color:var(--mdc-filled-text-field-container-color)}.mdc-text-field--filled.mdc-text-field--disabled{background-color:var(--mdc-filled-text-field-disabled-container-color)}.mdc-text-field--filled:not(.mdc-text-field--disabled) .mdc-line-ripple::before{border-bottom-color:var(--mdc-filled-text-field-active-indicator-color)}.mdc-text-field--filled:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-line-ripple::before{border-bottom-color:var(--mdc-filled-text-field-hover-active-indicator-color)}.mdc-text-field--filled:not(.mdc-text-field--disabled) .mdc-line-ripple::after{border-bottom-color:var(--mdc-filled-text-field-focus-active-indicator-color)}.mdc-text-field--filled.mdc-text-field--disabled .mdc-line-ripple::before{border-bottom-color:var(--mdc-filled-text-field-disabled-active-indicator-color)}.mdc-text-field--filled.mdc-text-field--invalid:not(.mdc-text-field--disabled) .mdc-line-ripple::before{border-bottom-color:var(--mdc-filled-text-field-error-active-indicator-color)}.mdc-text-field--filled.mdc-text-field--invalid:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-line-ripple::before{border-bottom-color:var(--mdc-filled-text-field-error-hover-active-indicator-color)}.mdc-text-field--filled.mdc-text-field--invalid:not(.mdc-text-field--disabled) .mdc-line-ripple::after{border-bottom-color:var(--mdc-filled-text-field-error-focus-active-indicator-color)}.mdc-text-field--filled .mdc-line-ripple::before{border-bottom-width:var(--mdc-filled-text-field-active-indicator-height)}.mdc-text-field--filled .mdc-line-ripple::after{border-bottom-width:var(--mdc-filled-text-field-focus-active-indicator-height)}.mdc-text-field--outlined:not(.mdc-text-field--disabled) .mdc-text-field__input{caret-color:var(--mdc-outlined-text-field-caret-color)}.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled) .mdc-text-field__input{caret-color:var(--mdc-outlined-text-field-error-caret-color)}.mdc-text-field--outlined:not(.mdc-text-field--disabled) .mdc-text-field__input{color:var(--mdc-outlined-text-field-input-text-color)}.mdc-text-field--outlined.mdc-text-field--disabled .mdc-text-field__input{color:var(--mdc-outlined-text-field-disabled-input-text-color)}.mdc-text-field--outlined:not(.mdc-text-field--disabled) .mdc-floating-label,.mdc-text-field--outlined:not(.mdc-text-field--disabled) .mdc-floating-label--float-above{color:var(--mdc-outlined-text-field-label-text-color)}.mdc-text-field--outlined:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-floating-label,.mdc-text-field--outlined:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-floating-label--float-above{color:var(--mdc-outlined-text-field-focus-label-text-color)}.mdc-text-field--outlined:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-floating-label,.mdc-text-field--outlined:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-floating-label--float-above{color:var(--mdc-outlined-text-field-hover-label-text-color)}.mdc-text-field--outlined.mdc-text-field--disabled .mdc-floating-label,.mdc-text-field--outlined.mdc-text-field--disabled .mdc-floating-label--float-above{color:var(--mdc-outlined-text-field-disabled-label-text-color)}.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled) .mdc-floating-label,.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled) .mdc-floating-label--float-above{color:var(--mdc-outlined-text-field-error-label-text-color)}.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-floating-label,.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-floating-label--float-above{color:var(--mdc-outlined-text-field-error-focus-label-text-color)}.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-floating-label,.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-floating-label--float-above{color:var(--mdc-outlined-text-field-error-hover-label-text-color)}.mdc-text-field--outlined .mdc-floating-label{font-family:var(--mdc-outlined-text-field-label-text-font);font-size:var(--mdc-outlined-text-field-label-text-size);font-weight:var(--mdc-outlined-text-field-label-text-weight);letter-spacing:var(--mdc-outlined-text-field-label-text-tracking)}@media all{.mdc-text-field--outlined:not(.mdc-text-field--disabled) .mdc-text-field__input::placeholder{color:var(--mdc-outlined-text-field-input-text-placeholder-color)}}@media all{.mdc-text-field--outlined:not(.mdc-text-field--disabled) .mdc-text-field__input:-ms-input-placeholder{color:var(--mdc-outlined-text-field-input-text-placeholder-color)}}.mdc-text-field--outlined.mdc-text-field--textarea.mdc-text-field--outlined .mdc-floating-label--float-above{font-size:calc(.75*var(--mdc-outlined-text-field-label-text-size))}.mdc-text-field--outlined.mdc-text-field--textarea.mdc-text-field--outlined.mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-text-field--outlined.mdc-text-field--textarea.mdc-text-field--outlined .mdc-notched-outline--upgraded .mdc-floating-label--float-above{font-size:var(--mdc-outlined-text-field-label-text-size)}.mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__leading{border-top-left-radius:var(--mdc-outlined-text-field-container-shape);border-top-right-radius:0;border-bottom-right-radius:0;border-bottom-left-radius:var(--mdc-outlined-text-field-container-shape)}[dir=rtl] .mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__leading,.mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__leading[dir=rtl]{border-top-left-radius:0;border-top-right-radius:var(--mdc-outlined-text-field-container-shape);border-bottom-right-radius:var(--mdc-outlined-text-field-container-shape);border-bottom-left-radius:0}@supports(top: max(0%)){.mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__leading{width:max(12px,var(--mdc-outlined-text-field-container-shape))}}@supports(top: max(0%)){.mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__notch{max-width:calc(100% - max(12px,var(--mdc-outlined-text-field-container-shape))*2)}}.mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__trailing{border-top-left-radius:0;border-top-right-radius:var(--mdc-outlined-text-field-container-shape);border-bottom-right-radius:var(--mdc-outlined-text-field-container-shape);border-bottom-left-radius:0}[dir=rtl] .mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__trailing,.mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__trailing[dir=rtl]{border-top-left-radius:var(--mdc-outlined-text-field-container-shape);border-top-right-radius:0;border-bottom-right-radius:0;border-bottom-left-radius:var(--mdc-outlined-text-field-container-shape)}@supports(top: max(0%)){.mdc-text-field--outlined{padding-left:max(16px,calc(var(--mdc-outlined-text-field-container-shape) + 4px))}}@supports(top: max(0%)){.mdc-text-field--outlined{padding-right:max(16px,var(--mdc-outlined-text-field-container-shape))}}@supports(top: max(0%)){.mdc-text-field--outlined+.mdc-text-field-helper-line{padding-left:max(16px,calc(var(--mdc-outlined-text-field-container-shape) + 4px))}}@supports(top: max(0%)){.mdc-text-field--outlined+.mdc-text-field-helper-line{padding-right:max(16px,var(--mdc-outlined-text-field-container-shape))}}.mdc-text-field--outlined.mdc-text-field--with-leading-icon{padding-left:0}@supports(top: max(0%)){.mdc-text-field--outlined.mdc-text-field--with-leading-icon{padding-right:max(16px,var(--mdc-outlined-text-field-container-shape))}}[dir=rtl] .mdc-text-field--outlined.mdc-text-field--with-leading-icon,.mdc-text-field--outlined.mdc-text-field--with-leading-icon[dir=rtl]{padding-right:0}@supports(top: max(0%)){[dir=rtl] .mdc-text-field--outlined.mdc-text-field--with-leading-icon,.mdc-text-field--outlined.mdc-text-field--with-leading-icon[dir=rtl]{padding-left:max(16px,var(--mdc-outlined-text-field-container-shape))}}.mdc-text-field--outlined.mdc-text-field--with-trailing-icon{padding-right:0}@supports(top: max(0%)){.mdc-text-field--outlined.mdc-text-field--with-trailing-icon{padding-left:max(16px,calc(var(--mdc-outlined-text-field-container-shape) + 4px))}}[dir=rtl] .mdc-text-field--outlined.mdc-text-field--with-trailing-icon,.mdc-text-field--outlined.mdc-text-field--with-trailing-icon[dir=rtl]{padding-left:0}@supports(top: max(0%)){[dir=rtl] .mdc-text-field--outlined.mdc-text-field--with-trailing-icon,.mdc-text-field--outlined.mdc-text-field--with-trailing-icon[dir=rtl]{padding-right:max(16px,calc(var(--mdc-outlined-text-field-container-shape) + 4px))}}.mdc-text-field--outlined.mdc-text-field--with-leading-icon.mdc-text-field--with-trailing-icon{padding-left:0;padding-right:0}.mdc-text-field--outlined:not(.mdc-text-field--disabled) .mdc-notched-outline__leading,.mdc-text-field--outlined:not(.mdc-text-field--disabled) .mdc-notched-outline__notch,.mdc-text-field--outlined:not(.mdc-text-field--disabled) .mdc-notched-outline__trailing{border-color:var(--mdc-outlined-text-field-outline-color)}.mdc-text-field--outlined:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-notched-outline .mdc-notched-outline__leading,.mdc-text-field--outlined:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-notched-outline .mdc-notched-outline__notch,.mdc-text-field--outlined:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-notched-outline .mdc-notched-outline__trailing{border-color:var(--mdc-outlined-text-field-hover-outline-color)}.mdc-text-field--outlined:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-notched-outline__leading,.mdc-text-field--outlined:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-notched-outline__notch,.mdc-text-field--outlined:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-notched-outline__trailing{border-color:var(--mdc-outlined-text-field-focus-outline-color)}.mdc-text-field--outlined.mdc-text-field--disabled .mdc-notched-outline__leading,.mdc-text-field--outlined.mdc-text-field--disabled .mdc-notched-outline__notch,.mdc-text-field--outlined.mdc-text-field--disabled .mdc-notched-outline__trailing{border-color:var(--mdc-outlined-text-field-disabled-outline-color)}.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled) .mdc-notched-outline__leading,.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled) .mdc-notched-outline__notch,.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled) .mdc-notched-outline__trailing{border-color:var(--mdc-outlined-text-field-error-outline-color)}.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-notched-outline .mdc-notched-outline__leading,.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-notched-outline .mdc-notched-outline__notch,.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-notched-outline .mdc-notched-outline__trailing{border-color:var(--mdc-outlined-text-field-error-hover-outline-color)}.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-notched-outline__leading,.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-notched-outline__notch,.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-notched-outline__trailing{border-color:var(--mdc-outlined-text-field-error-focus-outline-color)}.mdc-text-field--outlined:not(.mdc-text-field--disabled) .mdc-notched-outline .mdc-notched-outline__leading,.mdc-text-field--outlined:not(.mdc-text-field--disabled) .mdc-notched-outline .mdc-notched-outline__notch,.mdc-text-field--outlined:not(.mdc-text-field--disabled) .mdc-notched-outline .mdc-notched-outline__trailing{border-width:var(--mdc-outlined-text-field-outline-width)}.mdc-text-field--outlined:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-notched-outline .mdc-notched-outline__leading,.mdc-text-field--outlined:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-notched-outline .mdc-notched-outline__notch,.mdc-text-field--outlined:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-notched-outline .mdc-notched-outline__trailing{border-width:var(--mdc-outlined-text-field-focus-outline-width)}.mat-mdc-form-field-textarea-control{vertical-align:middle;resize:vertical;box-sizing:border-box;height:auto;margin:0;padding:0;border:none;overflow:auto}.mat-mdc-form-field-input-control.mat-mdc-form-field-input-control{-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;font:inherit;letter-spacing:inherit;text-decoration:inherit;text-transform:inherit;border:none}.mat-mdc-form-field .mat-mdc-floating-label.mdc-floating-label{-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;line-height:normal;pointer-events:all}.mat-mdc-form-field:not(.mat-form-field-disabled) .mat-mdc-floating-label.mdc-floating-label{cursor:inherit}.mdc-text-field--no-label:not(.mdc-text-field--textarea) .mat-mdc-form-field-input-control.mdc-text-field__input,.mat-mdc-text-field-wrapper .mat-mdc-form-field-input-control{height:auto}.mat-mdc-text-field-wrapper .mat-mdc-form-field-input-control.mdc-text-field__input[type=color]{height:23px}.mat-mdc-text-field-wrapper{height:auto;flex:auto}.mat-mdc-form-field-has-icon-prefix .mat-mdc-text-field-wrapper{padding-left:0;--mat-mdc-form-field-label-offset-x: -16px}.mat-mdc-form-field-has-icon-suffix .mat-mdc-text-field-wrapper{padding-right:0}[dir=rtl] .mat-mdc-text-field-wrapper{padding-left:16px;padding-right:16px}[dir=rtl] .mat-mdc-form-field-has-icon-suffix .mat-mdc-text-field-wrapper{padding-left:0}[dir=rtl] .mat-mdc-form-field-has-icon-prefix .mat-mdc-text-field-wrapper{padding-right:0}.mat-form-field-disabled .mdc-text-field__input::placeholder{color:var(--mat-form-field-disabled-input-text-placeholder-color)}.mat-form-field-disabled .mdc-text-field__input::-moz-placeholder{color:var(--mat-form-field-disabled-input-text-placeholder-color)}.mat-form-field-disabled .mdc-text-field__input::-webkit-input-placeholder{color:var(--mat-form-field-disabled-input-text-placeholder-color)}.mat-form-field-disabled .mdc-text-field__input:-ms-input-placeholder{color:var(--mat-form-field-disabled-input-text-placeholder-color)}.mat-mdc-form-field-label-always-float .mdc-text-field__input::placeholder{transition-delay:40ms;transition-duration:110ms;opacity:1}.mat-mdc-text-field-wrapper .mat-mdc-form-field-infix .mat-mdc-floating-label{left:auto;right:auto}.mat-mdc-text-field-wrapper.mdc-text-field--outlined .mdc-text-field__input{display:inline-block}.mat-mdc-form-field .mat-mdc-text-field-wrapper.mdc-text-field .mdc-notched-outline__notch{padding-top:0}.mat-mdc-text-field-wrapper::before{content:none}.mat-mdc-form-field.mat-mdc-form-field.mat-mdc-form-field.mat-mdc-form-field.mat-mdc-form-field.mat-mdc-form-field .mdc-notched-outline__notch{border-left:1px solid rgba(0,0,0,0)}[dir=rtl] .mat-mdc-form-field.mat-mdc-form-field.mat-mdc-form-field.mat-mdc-form-field.mat-mdc-form-field.mat-mdc-form-field .mdc-notched-outline__notch{border-left:none;border-right:1px solid rgba(0,0,0,0)}.mat-mdc-form-field-infix{min-height:var(--mat-form-field-container-height);padding-top:var(--mat-form-field-filled-with-label-container-padding-top);padding-bottom:var(--mat-form-field-filled-with-label-container-padding-bottom)}.mdc-text-field--outlined .mat-mdc-form-field-infix,.mdc-text-field--no-label .mat-mdc-form-field-infix{padding-top:var(--mat-form-field-container-vertical-padding);padding-bottom:var(--mat-form-field-container-vertical-padding)}.mat-mdc-text-field-wrapper .mat-mdc-form-field-flex .mat-mdc-floating-label{top:calc(var(--mat-form-field-container-height)/2)}.mdc-text-field--filled .mat-mdc-floating-label{display:var(--mat-form-field-filled-label-display, block)}.mat-mdc-text-field-wrapper.mdc-text-field--outlined .mdc-notched-outline--upgraded .mdc-floating-label--float-above{--mat-mdc-form-field-label-transform: translateY(calc(calc(6.75px + var(--mat-form-field-container-height) / 2) * -1)) scale(var(--mat-mdc-form-field-floating-label-scale, 0.75));transform:var(--mat-mdc-form-field-label-transform)}.mat-mdc-form-field-subscript-wrapper{box-sizing:border-box;width:100%;position:relative}.mat-mdc-form-field-hint-wrapper,.mat-mdc-form-field-error-wrapper{position:absolute;top:0;left:0;right:0;padding:0 16px}.mat-mdc-form-field-subscript-dynamic-size .mat-mdc-form-field-hint-wrapper,.mat-mdc-form-field-subscript-dynamic-size .mat-mdc-form-field-error-wrapper{position:static}.mat-mdc-form-field-bottom-align::before{content:"";display:inline-block;height:16px}.mat-mdc-form-field-bottom-align.mat-mdc-form-field-subscript-dynamic-size::before{content:unset}.mat-mdc-form-field-hint-end{order:1}.mat-mdc-form-field-hint-wrapper{display:flex}.mat-mdc-form-field-hint-spacer{flex:1 0 1em}.mat-mdc-form-field-error{display:block;color:var(--mat-form-field-error-text-color)}.mat-mdc-form-field-subscript-wrapper,.mat-mdc-form-field-bottom-align::before{-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;font-family:var(--mat-form-field-subscript-text-font);line-height:var(--mat-form-field-subscript-text-line-height);font-size:var(--mat-form-field-subscript-text-size);letter-spacing:var(--mat-form-field-subscript-text-tracking);font-weight:var(--mat-form-field-subscript-text-weight)}.mat-mdc-form-field-focus-overlay{top:0;left:0;right:0;bottom:0;position:absolute;opacity:0;pointer-events:none;background-color:var(--mat-form-field-state-layer-color)}.mat-mdc-text-field-wrapper:hover .mat-mdc-form-field-focus-overlay{opacity:var(--mat-form-field-hover-state-layer-opacity)}.mat-mdc-form-field.mat-focused .mat-mdc-form-field-focus-overlay{opacity:var(--mat-form-field-focus-state-layer-opacity)}select.mat-mdc-form-field-input-control{-moz-appearance:none;-webkit-appearance:none;background-color:rgba(0,0,0,0);display:inline-flex;box-sizing:border-box}select.mat-mdc-form-field-input-control:not(:disabled){cursor:pointer}select.mat-mdc-form-field-input-control:not(.mat-mdc-native-select-inline) option{color:var(--mat-form-field-select-option-text-color)}select.mat-mdc-form-field-input-control:not(.mat-mdc-native-select-inline) option:disabled{color:var(--mat-form-field-select-disabled-option-text-color)}.mat-mdc-form-field-type-mat-native-select .mat-mdc-form-field-infix::after{content:"";width:0;height:0;border-left:5px solid rgba(0,0,0,0);border-right:5px solid rgba(0,0,0,0);border-top:5px solid;position:absolute;right:0;top:50%;margin-top:-2.5px;pointer-events:none;color:var(--mat-form-field-enabled-select-arrow-color)}[dir=rtl] .mat-mdc-form-field-type-mat-native-select .mat-mdc-form-field-infix::after{right:auto;left:0}.mat-mdc-form-field-type-mat-native-select.mat-focused .mat-mdc-form-field-infix::after{color:var(--mat-form-field-focus-select-arrow-color)}.mat-mdc-form-field-type-mat-native-select.mat-form-field-disabled .mat-mdc-form-field-infix::after{color:var(--mat-form-field-disabled-select-arrow-color)}.mat-mdc-form-field-type-mat-native-select .mat-mdc-form-field-input-control{padding-right:15px}[dir=rtl] .mat-mdc-form-field-type-mat-native-select .mat-mdc-form-field-input-control{padding-right:0;padding-left:15px}.cdk-high-contrast-active .mat-form-field-appearance-fill .mat-mdc-text-field-wrapper{outline:solid 1px}.cdk-high-contrast-active .mat-form-field-appearance-fill.mat-form-field-disabled .mat-mdc-text-field-wrapper{outline-color:GrayText}.cdk-high-contrast-active .mat-form-field-appearance-fill.mat-focused .mat-mdc-text-field-wrapper{outline:dashed 3px}.cdk-high-contrast-active .mat-mdc-form-field.mat-focused .mdc-notched-outline{border:dashed 3px}.mat-mdc-form-field-input-control[type=date],.mat-mdc-form-field-input-control[type=datetime],.mat-mdc-form-field-input-control[type=datetime-local],.mat-mdc-form-field-input-control[type=month],.mat-mdc-form-field-input-control[type=week],.mat-mdc-form-field-input-control[type=time]{line-height:1}.mat-mdc-form-field-input-control::-webkit-datetime-edit{line-height:1;padding:0;margin-bottom:-2px}.mat-mdc-form-field{--mat-mdc-form-field-floating-label-scale: 0.75;display:inline-flex;flex-direction:column;min-width:0;text-align:left;-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;font-family:var(--mat-form-field-container-text-font);line-height:var(--mat-form-field-container-text-line-height);font-size:var(--mat-form-field-container-text-size);letter-spacing:var(--mat-form-field-container-text-tracking);font-weight:var(--mat-form-field-container-text-weight)}[dir=rtl] .mat-mdc-form-field{text-align:right}.mat-mdc-form-field .mdc-text-field--outlined .mdc-floating-label--float-above{font-size:calc(var(--mat-form-field-outlined-label-text-populated-size)*var(--mat-mdc-form-field-floating-label-scale))}.mat-mdc-form-field .mdc-text-field--outlined .mdc-notched-outline--upgraded .mdc-floating-label--float-above{font-size:var(--mat-form-field-outlined-label-text-populated-size)}.mat-mdc-form-field-flex{display:inline-flex;align-items:baseline;box-sizing:border-box;width:100%}.mat-mdc-text-field-wrapper{width:100%;z-index:0}.mat-mdc-form-field-icon-prefix,.mat-mdc-form-field-icon-suffix{align-self:center;line-height:0;pointer-events:auto;position:relative;z-index:1}.mat-mdc-form-field-icon-prefix>.mat-icon,.mat-mdc-form-field-icon-suffix>.mat-icon{padding:0 12px;box-sizing:content-box}.mat-mdc-form-field-icon-prefix{color:var(--mat-form-field-leading-icon-color)}.mat-form-field-disabled .mat-mdc-form-field-icon-prefix{color:var(--mat-form-field-disabled-leading-icon-color)}.mat-mdc-form-field-icon-suffix{color:var(--mat-form-field-trailing-icon-color)}.mat-form-field-disabled .mat-mdc-form-field-icon-suffix{color:var(--mat-form-field-disabled-trailing-icon-color)}.mat-form-field-invalid .mat-mdc-form-field-icon-suffix{color:var(--mat-form-field-error-trailing-icon-color)}.mat-form-field-invalid:not(.mat-focused):not(.mat-form-field-disabled) .mat-mdc-text-field-wrapper:hover .mat-mdc-form-field-icon-suffix{color:var(--mat-form-field-error-hover-trailing-icon-color)}.mat-form-field-invalid.mat-focused .mat-mdc-text-field-wrapper .mat-mdc-form-field-icon-suffix{color:var(--mat-form-field-error-focus-trailing-icon-color)}.mat-mdc-form-field-icon-prefix,[dir=rtl] .mat-mdc-form-field-icon-suffix{padding:0 4px 0 0}.mat-mdc-form-field-icon-suffix,[dir=rtl] .mat-mdc-form-field-icon-prefix{padding:0 0 0 4px}.mat-mdc-form-field-subscript-wrapper .mat-icon,.mat-mdc-form-field label .mat-icon{width:1em;height:1em;font-size:inherit}.mat-mdc-form-field-infix{flex:auto;min-width:0;width:180px;position:relative;box-sizing:border-box}.mat-mdc-form-field .mdc-notched-outline__notch{margin-left:-1px;-webkit-clip-path:inset(-9em -999em -9em 1px);clip-path:inset(-9em -999em -9em 1px)}[dir=rtl] .mat-mdc-form-field .mdc-notched-outline__notch{margin-left:0;margin-right:-1px;-webkit-clip-path:inset(-9em 1px -9em -999em);clip-path:inset(-9em 1px -9em -999em)}.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field__input{transition:opacity 150ms 0ms cubic-bezier(0.4, 0, 0.2, 1)}@media all{.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field__input::placeholder{transition:opacity 67ms 0ms cubic-bezier(0.4, 0, 0.2, 1)}}@media all{.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field__input:-ms-input-placeholder{transition:opacity 67ms 0ms cubic-bezier(0.4, 0, 0.2, 1)}}@media all{.mdc-text-field--no-label .mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field__input::placeholder,.mdc-text-field--focused .mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field__input::placeholder{transition-delay:40ms;transition-duration:110ms}}@media all{.mdc-text-field--no-label .mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field__input:-ms-input-placeholder,.mdc-text-field--focused .mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field__input:-ms-input-placeholder{transition-delay:40ms;transition-duration:110ms}}.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field__affix{transition:opacity 150ms 0ms cubic-bezier(0.4, 0, 0.2, 1)}.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field--filled.mdc-ripple-upgraded--background-focused .mdc-text-field__ripple::before,.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field--filled:not(.mdc-ripple-upgraded):focus .mdc-text-field__ripple::before{transition-duration:75ms}.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field--outlined .mdc-floating-label--shake{animation:mdc-floating-label-shake-float-above-text-field-outlined 250ms 1}@keyframes mdc-floating-label-shake-float-above-text-field-outlined{0%{transform:translateX(calc(0% - 0%)) translateY(calc(0% - 34.75px)) scale(0.75)}33%{animation-timing-function:cubic-bezier(0.5, 0, 0.701732, 0.495819);transform:translateX(calc(4% - 0%)) translateY(calc(0% - 34.75px)) scale(0.75)}66%{animation-timing-function:cubic-bezier(0.302435, 0.381352, 0.55, 0.956352);transform:translateX(calc(-4% - 0%)) translateY(calc(0% - 34.75px)) scale(0.75)}100%{transform:translateX(calc(0% - 0%)) translateY(calc(0% - 34.75px)) scale(0.75)}}.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field--textarea{transition:none}.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field--textarea.mdc-text-field--filled .mdc-floating-label--shake{animation:mdc-floating-label-shake-float-above-textarea-filled 250ms 1}@keyframes mdc-floating-label-shake-float-above-textarea-filled{0%{transform:translateX(calc(0% - 0%)) translateY(calc(0% - 10.25px)) scale(0.75)}33%{animation-timing-function:cubic-bezier(0.5, 0, 0.701732, 0.495819);transform:translateX(calc(4% - 0%)) translateY(calc(0% - 10.25px)) scale(0.75)}66%{animation-timing-function:cubic-bezier(0.302435, 0.381352, 0.55, 0.956352);transform:translateX(calc(-4% - 0%)) translateY(calc(0% - 10.25px)) scale(0.75)}100%{transform:translateX(calc(0% - 0%)) translateY(calc(0% - 10.25px)) scale(0.75)}}.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field--textarea.mdc-text-field--outlined .mdc-floating-label--shake{animation:mdc-floating-label-shake-float-above-textarea-outlined 250ms 1}@keyframes mdc-floating-label-shake-float-above-textarea-outlined{0%{transform:translateX(calc(0% - 0%)) translateY(calc(0% - 24.75px)) scale(0.75)}33%{animation-timing-function:cubic-bezier(0.5, 0, 0.701732, 0.495819);transform:translateX(calc(4% - 0%)) translateY(calc(0% - 24.75px)) scale(0.75)}66%{animation-timing-function:cubic-bezier(0.302435, 0.381352, 0.55, 0.956352);transform:translateX(calc(-4% - 0%)) translateY(calc(0% - 24.75px)) scale(0.75)}100%{transform:translateX(calc(0% - 0%)) translateY(calc(0% - 24.75px)) scale(0.75)}}.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field--with-leading-icon.mdc-text-field--outlined .mdc-floating-label--shake{animation:mdc-floating-label-shake-float-above-text-field-outlined-leading-icon 250ms 1}@keyframes mdc-floating-label-shake-float-above-text-field-outlined-leading-icon{0%{transform:translateX(calc(0% - 32px)) translateY(calc(0% - 34.75px)) scale(0.75)}33%{animation-timing-function:cubic-bezier(0.5, 0, 0.701732, 0.495819);transform:translateX(calc(4% - 32px)) translateY(calc(0% - 34.75px)) scale(0.75)}66%{animation-timing-function:cubic-bezier(0.302435, 0.381352, 0.55, 0.956352);transform:translateX(calc(-4% - 32px)) translateY(calc(0% - 34.75px)) scale(0.75)}100%{transform:translateX(calc(0% - 32px)) translateY(calc(0% - 34.75px)) scale(0.75)}}[dir=rtl] .mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field--with-leading-icon.mdc-text-field--outlined .mdc-floating-label--shake,.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field--with-leading-icon.mdc-text-field--outlined[dir=rtl] .mdc-floating-label--shake{animation:mdc-floating-label-shake-float-above-text-field-outlined-leading-icon 250ms 1}@keyframes mdc-floating-label-shake-float-above-text-field-outlined-leading-icon-rtl{0%{transform:translateX(calc(0% - -32px)) translateY(calc(0% - 34.75px)) scale(0.75)}33%{animation-timing-function:cubic-bezier(0.5, 0, 0.701732, 0.495819);transform:translateX(calc(4% - -32px)) translateY(calc(0% - 34.75px)) scale(0.75)}66%{animation-timing-function:cubic-bezier(0.302435, 0.381352, 0.55, 0.956352);transform:translateX(calc(-4% - -32px)) translateY(calc(0% - 34.75px)) scale(0.75)}100%{transform:translateX(calc(0% - -32px)) translateY(calc(0% - 34.75px)) scale(0.75)}}.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-floating-label{transition:transform 150ms cubic-bezier(0.4, 0, 0.2, 1),color 150ms cubic-bezier(0.4, 0, 0.2, 1)}.mdc-floating-label--shake{animation:mdc-floating-label-shake-float-above-standard 250ms 1}@keyframes mdc-floating-label-shake-float-above-standard{0%{transform:translateX(calc(0% - 0%)) translateY(calc(0% - 106%)) scale(0.75)}33%{animation-timing-function:cubic-bezier(0.5, 0, 0.701732, 0.495819);transform:translateX(calc(4% - 0%)) translateY(calc(0% - 106%)) scale(0.75)}66%{animation-timing-function:cubic-bezier(0.302435, 0.381352, 0.55, 0.956352);transform:translateX(calc(-4% - 0%)) translateY(calc(0% - 106%)) scale(0.75)}100%{transform:translateX(calc(0% - 0%)) translateY(calc(0% - 106%)) scale(0.75)}}.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-line-ripple::after{transition:transform 180ms cubic-bezier(0.4, 0, 0.2, 1),opacity 180ms cubic-bezier(0.4, 0, 0.2, 1)}.mdc-notched-outline .mdc-floating-label{max-width:calc(100% + 1px)}.mdc-notched-outline--upgraded .mdc-floating-label--float-above{max-width:calc(133.3333333333% + 1px)}\n'],
pb:2,data:{animation:[tA]},Pa:0});var cB=class{};cB.K=function(a){return new (a||cB)};cB.Ya=Sf({type:cB});cB.Xa=Qd({imports:[mz,it,ny,mz]});var eB={pa:qw,Ab:df(()=>dB),Eb:!0},fB=new gf("mat-autocomplete-scroll-strategy",{fa:"root",ba:()=>{const a=of(cw);return()=>a.C.C()}}),gB={pa:fB,Bc:[cw],Gc:function(a){return()=>a.C.C()}};function hB(a){let b;return(null==(b=a.ia)?void 0:b.defaultView)||window}
function iB(a){a.ud();a.G&&(a.qb&&a.zb.run(()=>{a.autocomplete.closed.emit()}),a.autocomplete.J===a&&(a.autocomplete.D=!1,a.autocomplete.J=null),a.G=!1,a.J=null,a.la&&a.la.jc()&&(a.la.detach(),a.Ec.unsubscribe()),a.Va(),a.gb||fn(a.Da),a.F&&uy(a.F,"aria-owns",a.autocomplete.id))}function jB(a){const b=a.C.R;return!b.readOnly&&!b.disabled&&!a.Yi}
function kB(a,b){let c=a.la;if(c)Iu(a.P,a.ta()),c.te({width:a.ua()});else{let e;a.Jc=new Wv(a.autocomplete.sa,a.Qc,{id:null==(e=a.D)?void 0:e.rh()?e.Xe:null});c=a.fb.create(lB(a));a.la=c;a.Ob=a.kd.xb().subscribe(()=>{a.qb&&c&&c.te({width:a.ua()})})}c&&!c.jc()&&(c.attach(a.Jc),a.Zd=b,a.O=null,a.Ec=a.Vd());b=a.qb;a.autocomplete.D=a.G=!0;a.autocomplete.J=a;let d;a.autocomplete.ea(null==(d=a.D)?void 0:d.color);a.Va();mB(a);a.qb&&b!==a.qb&&a.lb()}
function nB(a,b=!1){a.D&&"auto"===a.D.yg&&(b?a.D.oa():a.D.yg="always",a.Ea=!0)}function lB(a){let b,c;const d=Pu(Nu(Sv(a.fb.position(),a.ta()),!1),!1);a.Kb(d);a.P=d;return new Bu({vc:d,wc:a.Kc(),width:a.ua(),direction:null!=(c=a.Ic)?c:void 0,hb:null==(b=a.Hc)?void 0:b.Zh})}function mB(a){const b=a.C.R.closest('body > .cdk-overlay-container [aria-modal="true"]');if(b){var c=a.autocomplete.id;a.F&&uy(a.F,"aria-owns",c);sy(b,"aria-owns",c);a.F=b}}
var dB=class{constructor(a,b,c,d,e,f,g,h,m,p,u){this.C=a;this.fb=b;this.Qc=c;this.zb=d;this.Da=e;this.Ic=g;this.D=h;this.ia=m;this.kd=p;this.Hc=u;this.Ea=this.gb=!1;this.Ob=Si.EMPTY;this.ja=!0;this.oa=new hj;this.Pb=()=>{this.ja=this.ia.activeElement!==this.C.R||this.qb};this.N=()=>{};this.de=()=>{};this.position="auto";this.Zu="off";this.ea=new hj;this.Ra=of(ai);this.Re="mat-mdc-autocomplete-panel-above";this.G=!1;this.cc=nk(()=>{const w=this.autocomplete?this.autocomplete.options:null;return w?
C(w.C,Ko(w),Lo(()=>wk(...w.map(B=>B.po)))):C(this.ea,Lo(()=>this.cc))});this.td=w=>{if(27===w.keyCode&&!Cu(w)||38===w.keyCode&&Cu(w,"altKey")){if(this.J){let B;this.Lb(null!=(B=this.Wd)?B:"");this.J=null}this.oa.next();this.Ta();w.stopPropagation();w.preventDefault()}};this.F=null;this.Kc=f}pd(){this.ea.next();this.ea.complete();const a=hB(this);"undefined"!==typeof a&&Qk(this.zb,()=>a.addEventListener("blur",this.Pb))}uc(a){a.position&&this.P&&(this.Kb(this.P),this.qb&&this.la.Wc())}va(){const a=
hB(this);"undefined"!==typeof a&&a.removeEventListener("blur",this.Pb);this.Ob.unsubscribe();this.gb=!0;this.ld();this.oa.complete();this.F&&(uy(this.F,"aria-owns",this.autocomplete.id),this.F=null)}get qb(){return this.G&&this.autocomplete.em}Wc(){this.G&&this.la.Wc()}get ae(){return C(wk(this.cc,C(this.autocomplete.C.ca,xk(()=>this.G)),this.oa,this.sd(),this.la?C(this.la.J,xk(()=>this.G)):oj()),Cj(a=>a instanceof Tz?a:null))}get zh(){return this.autocomplete&&this.autocomplete.C?this.autocomplete.C.C:
null}sd(){return C(wk(qk(this.ia,"click"),qk(this.ia,"auxclick"),qk(this.ia,"touchend")),xk(a=>{a=Tt(a);const b=this.D?$A(this.D).R:null,c=this.ln?this.ln.Ub.R:null;return this.G&&a!==this.C.R&&this.ia.activeElement!==this.C.R&&(!b||!b.contains(a))&&(!c||!c.contains(a))&&!!this.la&&!this.la.N.contains(a)}))}ti(a){Promise.resolve(null).then(()=>this.T(a))}Ng(a){this.N=a}Nj(a){this.de=a}ji(a){this.C.R.disabled=a}Ve(a){const b=a.keyCode,c=Cu(a);27!==b||c||a.preventDefault();this.O=this.C.R.value;if(this.zh&&
13===b&&this.qb&&!c)this.zh.wh(),this.Ta(),a.preventDefault();else if(this.autocomplete){const d=this.autocomplete.C.C,e=38===b||40===b;9===b||e&&!c&&this.qb?Uy(this.autocomplete.C,a):e&&jB(this)&&this.Ja(this.O);if(e||this.autocomplete.C.C!==d)this.Td(this.autocomplete.C.O||0),this.autocomplete.Zm&&this.zh&&(this.J||(this.Wd=this.O),this.J=this.zh,this.T(this.zh.value))}}Bp(a){var b=a.target;let c=b.value;"number"===b.type&&(c=""==c?null:parseFloat(c));if(this.ca!==c){this.ca=c;this.J=null;this.autocomplete&&
this.autocomplete.Pj||this.N(c);if(!c)this.aa(null,!1);else if(this.qb&&!this.autocomplete.Pj){var d;if(b=null==(d=this.autocomplete.options)?void 0:d.find(e=>e.selected))d=this.sb(b.value),c!==d&&Uz(b,!1)}if(jB(this)&&this.ia.activeElement===a.target){let e;a=null!=(e=this.O)?e:this.C.R.value;this.O=null;this.Ja(a)}}}mA(){this.ja?jB(this)&&(this.ca=this.C.R.value,kB(this,this.ca),nB(this,!0)):this.ja=!0}tk(){jB(this)&&!this.qb&&this.Ja()}ud(){this.Ea&&(this.D&&(this.D.yg="auto"),this.Ea=!1)}Vd(){const a=
new bj(c=>{el(()=>{c.next()},{xa:this.Ra})}),b=C(this.autocomplete.options.C,So(()=>Ju(this.P)),$n());return C(wk(a,b),Lo(()=>this.zb.run(()=>{const c=this.qb;this.Ta();this.Va();fn(this.Da);this.qb&&this.la.Wc();c!==this.qb&&(this.qb?this.lb():this.autocomplete.closed.emit());return this.ae})),io()).subscribe(c=>this.Ud(c))}lb(){this.autocomplete.kf.emit()}ld(){this.la&&(iB(this),this.la.dispose(),this.la=null)}sb(a){const b=this.autocomplete;return b&&b.Fq?b.Fq(a):a}T(a){const b=this.sb(a);null==
a&&this.aa(null,!1);this.Lb(null!=b?b:"")}Lb(a){this.D?this.D.Hb.value=a:this.C.R.value=a;this.ca=a}Ud(a){const b=this.autocomplete;(a=a?a.source:this.J)?(this.aa(a),this.T(a.value),this.N(a.value),b.aa(a),this.C.R.focus()):b.Pj&&this.C.R.value!==this.Zd&&(this.aa(null),this.T(null),b.jg?C(b.jg,io()).subscribe(()=>this.N(null)):this.N(null));iB(this)}aa(a,b){let c,d;null==(c=this.autocomplete)||null==(d=c.options)||d.forEach(e=>{e!==a&&e.selected&&Uz(e,b)})}Ja(a=this.C.R.value){kB(this,a);nB(this);
this.F&&sy(this.F,"aria-owns",this.autocomplete.id)}Va(){this.autocomplete.O();if(this.qb){var a=this.la;this.wa||(this.wa=a.gf().subscribe(this.td));this.Qa||(this.Qa=a.Pl().subscribe())}else{null==(a=this.wa)||a.unsubscribe();let b;null==(b=this.Qa)||b.unsubscribe();this.wa=this.Qa=null}}Kb(a){var b=[{bb:"start",cb:"bottom",Na:"start",Ua:"top"},{bb:"end",cb:"bottom",Na:"end",Ua:"top"}];const c=[{bb:"start",cb:"top",Na:"start",Ua:"bottom",hb:"mat-mdc-autocomplete-panel-above"},{bb:"end",cb:"top",
Na:"end",Ua:"bottom",hb:"mat-mdc-autocomplete-panel-above"}];b="above"===this.position?c:"below"===this.position?b:[...b,...c];Lu(a,b)}ta(){return this.ln?this.ln.Ub:this.D?$A(this.D):this.C}ua(){return this.autocomplete.Ig||this.md()}md(){return this.ta().R.getBoundingClientRect().width}Ta(){const a=this.autocomplete;if(a.Xm){let b=-1;for(let c=0;c<a.options.length;c++)if(!a.options.get(c).disabled){b=c;break}a.C.D(b)}else a.C.D(-1)}Td(a){const b=this.autocomplete,c=Wz(a,b.options,b.Qr);0===a&&1===
c?b.N(0):b.Xf&&(a=b.options.toArray()[a])&&(a=a.we(),b.N(Xz(a.offsetTop,a.offsetHeight,b.ca(),b.Xf.R.offsetHeight)))}};dB.K=function(a){return new (a||dB)(y(wi),y(cw),y(np),y(Tk),y(Tr),y(fB),y(pu,8),y(XA,9),y(kt,8),y(xu),y(lA,8))};
dB.Fa=Uf({type:dB,ha:[["input","matAutocomplete",""],["textarea","matAutocomplete",""]],Ka:[1,"mat-mdc-autocomplete-trigger"],Za:7,Ia:function(a,b){a&1&&V("focusin",function(){return b.mA()})("blur",function(){return b.de()})("input",function(c){return b.Bp(c)})("keydown",function(c){return b.Ve(c)})("click",function(){return b.tk()});a&2&&Mp("autocomplete",b.Zu)("role",b.Yi?null:"combobox")("aria-autocomplete",b.Yi?null:"list")("aria-activedescendant",b.qb&&b.zh?b.zh.id:null)("aria-expanded",b.Yi?
null:b.qb.toString())("aria-controls",b.Yi||!b.qb?null:null==b.autocomplete?null:b.autocomplete.id)("aria-haspopup",b.Yi?null:"listbox")},inputs:{autocomplete:[0,"matAutocomplete","autocomplete"],position:[0,"matAutocompletePosition","position"],ln:[0,"matAutocompleteConnectedTo","connectedTo"],Zu:[0,"autocomplete","autocompleteAttribute"],Yi:[2,"matAutocompleteDisabled","autocompleteDisabled",As]},ib:["matAutocompleteTrigger"],ka:!0,features:[Lr([eB]),Lp,Hg]});var oB=class{};oB.K=function(a){return new (a||oB)};oB.Ya=Sf({type:oB});oB.Xa=Qd({Wa:[gB],imports:[kw,$z,mz,it,yu,$z,mz]});var pB=class{};pB.K=function(a){return new (a||pB)};pB.Ya=Sf({type:pB});pB.Xa=Qd({imports:[oB]});var qB=new gf("GMAT_BUTTON_OPTIONS",{fa:"root",ba:()=>({tc:!1})});var rB=new gf("MAT_BUTTON_CONFIG");
const sB=[{Ah:"mat-button",Th:["mdc-button","mat-mdc-button"]},{Ah:"mat-flat-button",Th:["mdc-button","mdc-button--unelevated","mat-mdc-unelevated-button"]},{Ah:"mat-raised-button",Th:["mdc-button","mdc-button--raised","mat-mdc-raised-button"]},{Ah:"mat-stroked-button",Th:["mdc-button","mdc-button--outlined","mat-mdc-outlined-button"]},{Ah:"mat-fab",Th:["mdc-fab","mat-mdc-fab"]},{Ah:"mat-mini-fab",Th:["mdc-fab","mdc-fab--mini","mat-mdc-mini-fab"]},{Ah:"mat-icon-button",Th:["mdc-icon-button","mat-mdc-icon-button"]}];
var tB=class{get Ix(){var a;if(null==(a=this.C))a=void 0;else{var b=this.ra.R;a=a.C.get(b)||a.F(b)}return a}set Ix(a){let b;null==(b=this.C)||eA(b,this.ra.R,a)}get Nc(){return this.F}set Nc(a){this.F=a;this.G()}get disabled(){return this.Ib}set disabled(a){this.Ib=a;this.G()}constructor(a,b,c,d){this.ra=a;this.Ma=b;this.qa=c;this.Yc=d;this.D=of(Px);this.C=of(fA);this.Ib=this.F=this.mu=!1;const e=of(rB,{optional:!0}),f=a.R,g=f.classList;let h;this.lj=null!=(h=null==e?void 0:e.lj)?h:!1;let m;null==
(m=this.C)||dA(m,f,{className:"mat-mdc-button-ripple"});for(const {Ah:p,Th:u}of sB)f.hasAttribute(p)&&g.add(...u)}pd(){Mx(this.D,this.ra,!0)}va(){Nx(this.D,this.ra);let a;null==(a=this.C)||cA(a,this.ra.R)}focus(a="program",b){a?Ox(this.D,this.ra.R,a,b):this.ra.R.focus(b)}Rt(){return null!=this.ariaDisabled?this.ariaDisabled:this.disabled&&this.lj?!0:null}St(){return this.lj||!this.disabled?null:!0}G(){var a;if(null!=(a=this.C)){var b=this.ra.R,c=this.Nc||this.disabled;(a=a.C.get(b))?a.disabled=c:
c?b.setAttribute("mat-ripple-loader-disabled",""):b.removeAttribute("mat-ripple-loader-disabled")}}};tB.K=function(){throw Error("invalid");};tB.Fa=Uf({type:tB,inputs:{color:"color",Nc:[2,"disableRipple","disableRipple",As],disabled:[2,"disabled","disabled",As],ariaDisabled:[2,"aria-disabled","ariaDisabled",As],lj:[2,"disabledInteractive","disabledInteractive",As]},features:[Lp]});const uB=[[["",8,"material-icons",3,"iconPositionEnd",""],["mat-icon",3,"iconPositionEnd",""],["","matButtonIcon","",3,"iconPositionEnd",""]],"*",[["","iconPositionEnd","",8,"material-icons"],["mat-icon","iconPositionEnd",""],["","matButtonIcon","","iconPositionEnd",""]]];var vB=class extends tB{};vB.K=function(a){return new (a||vB)(y(wi),y(Wt),y(Tk),y(qi,8))};
vB.Ca=Jf({type:vB,ha:[["button","mat-button",""],["button","mat-raised-button",""],["button","mat-flat-button",""],["button","mat-stroked-button",""]],Za:14,Ia:function(a,b){a&2&&(Mp("disabled",b.St())("aria-disabled",b.Rt()),sq(vq,b.color?"mat-"+b.color:""),Wp("mat-mdc-button-disabled",b.disabled)("mat-mdc-button-disabled-interactive",b.lj)("_mat-animation-noopable","NoopAnimations"===b.Yc)("mat-unthemed",!b.color)("mat-mdc-button-base",!0))},ib:["matButton"],ka:!0,features:[Ep,Nr],attrs:["mat-button",
""],bc:[".material-icons:not([iconPositionEnd]), mat-icon:not([iconPositionEnd]), [matButtonIcon]:not([iconPositionEnd])","*",".material-icons[iconPositionEnd], mat-icon[iconPositionEnd], [matButtonIcon][iconPositionEnd]"],ya:7,za:4,Ba:[[1,"mat-mdc-button-persistent-ripple"],[1,"mdc-button__label"],[1,"mat-mdc-focus-indicator"],[1,"mat-mdc-button-touch-target"]],sa:function(a,b){a&1&&(Zq(uB),Oq(0,"span",0),$q(1),R(2,"span",1),$q(3,1),U(),$q(4,2),Oq(5,"span",2)(6,"span",3));a&2&&Wp("mdc-button__ripple",
!b.mu)("mdc-fab__ripple",b.mu)},styles:['.mdc-button{position:relative;display:inline-flex;align-items:center;justify-content:center;box-sizing:border-box;min-width:64px;border:none;outline:none;line-height:inherit;user-select:none;-webkit-appearance:none;overflow:visible;vertical-align:middle;background:rgba(0,0,0,0)}.mdc-button .mdc-elevation-overlay{width:100%;height:100%;top:0;left:0}.mdc-button::-moz-focus-inner{padding:0;border:0}.mdc-button:active{outline:none}.mdc-button:hover{cursor:pointer}.mdc-button:disabled{cursor:default;pointer-events:none}.mdc-button[hidden]{display:none}.mdc-button .mdc-button__icon{margin-left:0;margin-right:8px;display:inline-block;position:relative;vertical-align:top}[dir=rtl] .mdc-button .mdc-button__icon,.mdc-button .mdc-button__icon[dir=rtl]{margin-left:8px;margin-right:0}.mdc-button .mdc-button__progress-indicator{font-size:0;position:absolute;transform:translate(-50%, -50%);top:50%;left:50%;line-height:initial}.mdc-button .mdc-button__label{position:relative}.mdc-button .mdc-button__focus-ring{pointer-events:none;border:2px solid rgba(0,0,0,0);border-radius:6px;box-sizing:content-box;position:absolute;top:50%;left:50%;transform:translate(-50%, -50%);height:calc(100% + 4px);width:calc(100% + 4px);display:none}@media screen and (forced-colors: active){.mdc-button .mdc-button__focus-ring{border-color:CanvasText}}.mdc-button .mdc-button__focus-ring::after{content:"";border:2px solid rgba(0,0,0,0);border-radius:8px;display:block;position:absolute;top:50%;left:50%;transform:translate(-50%, -50%);height:calc(100% + 4px);width:calc(100% + 4px)}@media screen and (forced-colors: active){.mdc-button .mdc-button__focus-ring::after{border-color:CanvasText}}@media screen and (forced-colors: active){.mdc-button.mdc-ripple-upgraded--background-focused .mdc-button__focus-ring,.mdc-button:not(.mdc-ripple-upgraded):focus .mdc-button__focus-ring{display:block}}.mdc-button .mdc-button__touch{position:absolute;top:50%;height:48px;left:0;right:0;transform:translateY(-50%)}.mdc-button__label+.mdc-button__icon{margin-left:8px;margin-right:0}[dir=rtl] .mdc-button__label+.mdc-button__icon,.mdc-button__label+.mdc-button__icon[dir=rtl]{margin-left:0;margin-right:8px}svg.mdc-button__icon{fill:currentColor}.mdc-button--touch{margin-top:6px;margin-bottom:6px}.mdc-button{padding:0 8px 0 8px}.mdc-button--unelevated{transition:box-shadow 280ms cubic-bezier(0.4, 0, 0.2, 1);padding:0 16px 0 16px}.mdc-button--unelevated.mdc-button--icon-trailing{padding:0 12px 0 16px}.mdc-button--unelevated.mdc-button--icon-leading{padding:0 16px 0 12px}.mdc-button--raised{transition:box-shadow 280ms cubic-bezier(0.4, 0, 0.2, 1);padding:0 16px 0 16px}.mdc-button--raised.mdc-button--icon-trailing{padding:0 12px 0 16px}.mdc-button--raised.mdc-button--icon-leading{padding:0 16px 0 12px}.mdc-button--outlined{border-style:solid;transition:border 280ms cubic-bezier(0.4, 0, 0.2, 1)}.mdc-button--outlined .mdc-button__ripple{border-style:solid;border-color:rgba(0,0,0,0)}.mat-mdc-button{font-family:var(--mdc-text-button-label-text-font);font-size:var(--mdc-text-button-label-text-size);letter-spacing:var(--mdc-text-button-label-text-tracking);font-weight:var(--mdc-text-button-label-text-weight);text-transform:var(--mdc-text-button-label-text-transform);height:var(--mdc-text-button-container-height);border-radius:var(--mdc-text-button-container-shape);padding:0 var(--mat-text-button-horizontal-padding, 8px)}.mat-mdc-button:not(:disabled){color:var(--mdc-text-button-label-text-color)}.mat-mdc-button:disabled{color:var(--mdc-text-button-disabled-label-text-color)}.mat-mdc-button .mdc-button__ripple{border-radius:var(--mdc-text-button-container-shape)}.mat-mdc-button:has(.material-icons,mat-icon,[matButtonIcon]){padding:0 var(--mat-text-button-with-icon-horizontal-padding, 8px)}.mat-mdc-button>.mat-icon{margin-right:var(--mat-text-button-icon-spacing, 8px);margin-left:var(--mat-text-button-icon-offset, 0)}[dir=rtl] .mat-mdc-button>.mat-icon{margin-right:var(--mat-text-button-icon-offset, 0);margin-left:var(--mat-text-button-icon-spacing, 8px)}.mat-mdc-button .mdc-button__label+.mat-icon{margin-right:var(--mat-text-button-icon-offset, 0);margin-left:var(--mat-text-button-icon-spacing, 8px)}[dir=rtl] .mat-mdc-button .mdc-button__label+.mat-icon{margin-right:var(--mat-text-button-icon-spacing, 8px);margin-left:var(--mat-text-button-icon-offset, 0)}.mat-mdc-button .mat-ripple-element{background-color:var(--mat-text-button-ripple-color)}.mat-mdc-button .mat-mdc-button-persistent-ripple::before{background-color:var(--mat-text-button-state-layer-color)}.mat-mdc-button.mat-mdc-button-disabled .mat-mdc-button-persistent-ripple::before{background-color:var(--mat-text-button-disabled-state-layer-color)}.mat-mdc-button:hover .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-text-button-hover-state-layer-opacity)}.mat-mdc-button.cdk-program-focused .mat-mdc-button-persistent-ripple::before,.mat-mdc-button.cdk-keyboard-focused .mat-mdc-button-persistent-ripple::before,.mat-mdc-button.mat-mdc-button-disabled-interactive:focus .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-text-button-focus-state-layer-opacity)}.mat-mdc-button:active .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-text-button-pressed-state-layer-opacity)}.mat-mdc-button .mat-mdc-button-touch-target{position:absolute;top:50%;height:48px;left:0;right:0;transform:translateY(-50%);display:var(--mat-text-button-touch-target-display)}.mat-mdc-button[disabled],.mat-mdc-button.mat-mdc-button-disabled{cursor:default;pointer-events:none;color:var(--mdc-text-button-disabled-label-text-color)}.mat-mdc-button.mat-mdc-button-disabled-interactive{pointer-events:auto}.mat-mdc-unelevated-button{font-family:var(--mdc-filled-button-label-text-font);font-size:var(--mdc-filled-button-label-text-size);letter-spacing:var(--mdc-filled-button-label-text-tracking);font-weight:var(--mdc-filled-button-label-text-weight);text-transform:var(--mdc-filled-button-label-text-transform);height:var(--mdc-filled-button-container-height);border-radius:var(--mdc-filled-button-container-shape);padding:0 var(--mat-filled-button-horizontal-padding, 16px)}.mat-mdc-unelevated-button:not(:disabled){background-color:var(--mdc-filled-button-container-color)}.mat-mdc-unelevated-button:disabled{background-color:var(--mdc-filled-button-disabled-container-color)}.mat-mdc-unelevated-button:not(:disabled){color:var(--mdc-filled-button-label-text-color)}.mat-mdc-unelevated-button:disabled{color:var(--mdc-filled-button-disabled-label-text-color)}.mat-mdc-unelevated-button .mdc-button__ripple{border-radius:var(--mdc-filled-button-container-shape)}.mat-mdc-unelevated-button>.mat-icon{margin-right:var(--mat-filled-button-icon-spacing, 8px);margin-left:var(--mat-filled-button-icon-offset, -4px)}[dir=rtl] .mat-mdc-unelevated-button>.mat-icon{margin-right:var(--mat-filled-button-icon-offset, -4px);margin-left:var(--mat-filled-button-icon-spacing, 8px)}.mat-mdc-unelevated-button .mdc-button__label+.mat-icon{margin-right:var(--mat-filled-button-icon-offset, -4px);margin-left:var(--mat-filled-button-icon-spacing, 8px)}[dir=rtl] .mat-mdc-unelevated-button .mdc-button__label+.mat-icon{margin-right:var(--mat-filled-button-icon-spacing, 8px);margin-left:var(--mat-filled-button-icon-offset, -4px)}.mat-mdc-unelevated-button .mat-ripple-element{background-color:var(--mat-filled-button-ripple-color)}.mat-mdc-unelevated-button .mat-mdc-button-persistent-ripple::before{background-color:var(--mat-filled-button-state-layer-color)}.mat-mdc-unelevated-button.mat-mdc-button-disabled .mat-mdc-button-persistent-ripple::before{background-color:var(--mat-filled-button-disabled-state-layer-color)}.mat-mdc-unelevated-button:hover .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-filled-button-hover-state-layer-opacity)}.mat-mdc-unelevated-button.cdk-program-focused .mat-mdc-button-persistent-ripple::before,.mat-mdc-unelevated-button.cdk-keyboard-focused .mat-mdc-button-persistent-ripple::before,.mat-mdc-unelevated-button.mat-mdc-button-disabled-interactive:focus .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-filled-button-focus-state-layer-opacity)}.mat-mdc-unelevated-button:active .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-filled-button-pressed-state-layer-opacity)}.mat-mdc-unelevated-button .mat-mdc-button-touch-target{position:absolute;top:50%;height:48px;left:0;right:0;transform:translateY(-50%);display:var(--mat-filled-button-touch-target-display)}.mat-mdc-unelevated-button[disabled],.mat-mdc-unelevated-button.mat-mdc-button-disabled{cursor:default;pointer-events:none;color:var(--mdc-filled-button-disabled-label-text-color);background-color:var(--mdc-filled-button-disabled-container-color)}.mat-mdc-unelevated-button.mat-mdc-button-disabled-interactive{pointer-events:auto}.mat-mdc-raised-button{font-family:var(--mdc-protected-button-label-text-font);font-size:var(--mdc-protected-button-label-text-size);letter-spacing:var(--mdc-protected-button-label-text-tracking);font-weight:var(--mdc-protected-button-label-text-weight);text-transform:var(--mdc-protected-button-label-text-transform);height:var(--mdc-protected-button-container-height);border-radius:var(--mdc-protected-button-container-shape);padding:0 var(--mat-protected-button-horizontal-padding, 16px);box-shadow:var(--mdc-protected-button-container-elevation-shadow)}.mat-mdc-raised-button:not(:disabled){background-color:var(--mdc-protected-button-container-color)}.mat-mdc-raised-button:disabled{background-color:var(--mdc-protected-button-disabled-container-color)}.mat-mdc-raised-button:not(:disabled){color:var(--mdc-protected-button-label-text-color)}.mat-mdc-raised-button:disabled{color:var(--mdc-protected-button-disabled-label-text-color)}.mat-mdc-raised-button .mdc-button__ripple{border-radius:var(--mdc-protected-button-container-shape)}.mat-mdc-raised-button>.mat-icon{margin-right:var(--mat-protected-button-icon-spacing, 8px);margin-left:var(--mat-protected-button-icon-offset, -4px)}[dir=rtl] .mat-mdc-raised-button>.mat-icon{margin-right:var(--mat-protected-button-icon-offset, -4px);margin-left:var(--mat-protected-button-icon-spacing, 8px)}.mat-mdc-raised-button .mdc-button__label+.mat-icon{margin-right:var(--mat-protected-button-icon-offset, -4px);margin-left:var(--mat-protected-button-icon-spacing, 8px)}[dir=rtl] .mat-mdc-raised-button .mdc-button__label+.mat-icon{margin-right:var(--mat-protected-button-icon-spacing, 8px);margin-left:var(--mat-protected-button-icon-offset, -4px)}.mat-mdc-raised-button .mat-ripple-element{background-color:var(--mat-protected-button-ripple-color)}.mat-mdc-raised-button .mat-mdc-button-persistent-ripple::before{background-color:var(--mat-protected-button-state-layer-color)}.mat-mdc-raised-button.mat-mdc-button-disabled .mat-mdc-button-persistent-ripple::before{background-color:var(--mat-protected-button-disabled-state-layer-color)}.mat-mdc-raised-button:hover .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-protected-button-hover-state-layer-opacity)}.mat-mdc-raised-button.cdk-program-focused .mat-mdc-button-persistent-ripple::before,.mat-mdc-raised-button.cdk-keyboard-focused .mat-mdc-button-persistent-ripple::before,.mat-mdc-raised-button.mat-mdc-button-disabled-interactive:focus .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-protected-button-focus-state-layer-opacity)}.mat-mdc-raised-button:active .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-protected-button-pressed-state-layer-opacity)}.mat-mdc-raised-button .mat-mdc-button-touch-target{position:absolute;top:50%;height:48px;left:0;right:0;transform:translateY(-50%);display:var(--mat-protected-button-touch-target-display)}.mat-mdc-raised-button:hover{box-shadow:var(--mdc-protected-button-hover-container-elevation-shadow)}.mat-mdc-raised-button:focus{box-shadow:var(--mdc-protected-button-focus-container-elevation-shadow)}.mat-mdc-raised-button:active,.mat-mdc-raised-button:focus:active{box-shadow:var(--mdc-protected-button-pressed-container-elevation-shadow)}.mat-mdc-raised-button[disabled],.mat-mdc-raised-button.mat-mdc-button-disabled{cursor:default;pointer-events:none;color:var(--mdc-protected-button-disabled-label-text-color);background-color:var(--mdc-protected-button-disabled-container-color)}.mat-mdc-raised-button[disabled].mat-mdc-button-disabled,.mat-mdc-raised-button.mat-mdc-button-disabled.mat-mdc-button-disabled{box-shadow:var(--mdc-protected-button-disabled-container-elevation-shadow)}.mat-mdc-raised-button.mat-mdc-button-disabled-interactive{pointer-events:auto}.mat-mdc-outlined-button{font-family:var(--mdc-outlined-button-label-text-font);font-size:var(--mdc-outlined-button-label-text-size);letter-spacing:var(--mdc-outlined-button-label-text-tracking);font-weight:var(--mdc-outlined-button-label-text-weight);text-transform:var(--mdc-outlined-button-label-text-transform);height:var(--mdc-outlined-button-container-height);border-radius:var(--mdc-outlined-button-container-shape);padding:0 15px 0 15px;border-width:var(--mdc-outlined-button-outline-width);padding:0 var(--mat-outlined-button-horizontal-padding, 15px)}.mat-mdc-outlined-button:not(:disabled){color:var(--mdc-outlined-button-label-text-color)}.mat-mdc-outlined-button:disabled{color:var(--mdc-outlined-button-disabled-label-text-color)}.mat-mdc-outlined-button .mdc-button__ripple{border-radius:var(--mdc-outlined-button-container-shape)}.mat-mdc-outlined-button:not(:disabled){border-color:var(--mdc-outlined-button-outline-color)}.mat-mdc-outlined-button:disabled{border-color:var(--mdc-outlined-button-disabled-outline-color)}.mat-mdc-outlined-button.mdc-button--icon-trailing{padding:0 11px 0 15px}.mat-mdc-outlined-button.mdc-button--icon-leading{padding:0 15px 0 11px}.mat-mdc-outlined-button .mdc-button__ripple{top:-1px;left:-1px;bottom:-1px;right:-1px;border-width:var(--mdc-outlined-button-outline-width)}.mat-mdc-outlined-button .mdc-button__touch{left:calc(-1 * var(--mdc-outlined-button-outline-width));width:calc(100% + 2 * var(--mdc-outlined-button-outline-width))}.mat-mdc-outlined-button>.mat-icon{margin-right:var(--mat-outlined-button-icon-spacing, 8px);margin-left:var(--mat-outlined-button-icon-offset, -4px)}[dir=rtl] .mat-mdc-outlined-button>.mat-icon{margin-right:var(--mat-outlined-button-icon-offset, -4px);margin-left:var(--mat-outlined-button-icon-spacing, 8px)}.mat-mdc-outlined-button .mdc-button__label+.mat-icon{margin-right:var(--mat-outlined-button-icon-offset, -4px);margin-left:var(--mat-outlined-button-icon-spacing, 8px)}[dir=rtl] .mat-mdc-outlined-button .mdc-button__label+.mat-icon{margin-right:var(--mat-outlined-button-icon-spacing, 8px);margin-left:var(--mat-outlined-button-icon-offset, -4px)}.mat-mdc-outlined-button .mat-ripple-element{background-color:var(--mat-outlined-button-ripple-color)}.mat-mdc-outlined-button .mat-mdc-button-persistent-ripple::before{background-color:var(--mat-outlined-button-state-layer-color)}.mat-mdc-outlined-button.mat-mdc-button-disabled .mat-mdc-button-persistent-ripple::before{background-color:var(--mat-outlined-button-disabled-state-layer-color)}.mat-mdc-outlined-button:hover .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-outlined-button-hover-state-layer-opacity)}.mat-mdc-outlined-button.cdk-program-focused .mat-mdc-button-persistent-ripple::before,.mat-mdc-outlined-button.cdk-keyboard-focused .mat-mdc-button-persistent-ripple::before,.mat-mdc-outlined-button.mat-mdc-button-disabled-interactive:focus .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-outlined-button-focus-state-layer-opacity)}.mat-mdc-outlined-button:active .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-outlined-button-pressed-state-layer-opacity)}.mat-mdc-outlined-button .mat-mdc-button-touch-target{position:absolute;top:50%;height:48px;left:0;right:0;transform:translateY(-50%);display:var(--mat-outlined-button-touch-target-display)}.mat-mdc-outlined-button[disabled],.mat-mdc-outlined-button.mat-mdc-button-disabled{cursor:default;pointer-events:none;color:var(--mdc-outlined-button-disabled-label-text-color);border-color:var(--mdc-outlined-button-disabled-outline-color)}.mat-mdc-outlined-button.mat-mdc-button-disabled-interactive{pointer-events:auto}.mat-mdc-button-base{text-decoration:none}.mat-mdc-button,.mat-mdc-unelevated-button,.mat-mdc-raised-button,.mat-mdc-outlined-button{-webkit-tap-highlight-color:rgba(0,0,0,0)}.mat-mdc-button .mat-mdc-button-ripple,.mat-mdc-button .mat-mdc-button-persistent-ripple,.mat-mdc-button .mat-mdc-button-persistent-ripple::before,.mat-mdc-unelevated-button .mat-mdc-button-ripple,.mat-mdc-unelevated-button .mat-mdc-button-persistent-ripple,.mat-mdc-unelevated-button .mat-mdc-button-persistent-ripple::before,.mat-mdc-raised-button .mat-mdc-button-ripple,.mat-mdc-raised-button .mat-mdc-button-persistent-ripple,.mat-mdc-raised-button .mat-mdc-button-persistent-ripple::before,.mat-mdc-outlined-button .mat-mdc-button-ripple,.mat-mdc-outlined-button .mat-mdc-button-persistent-ripple,.mat-mdc-outlined-button .mat-mdc-button-persistent-ripple::before{top:0;left:0;right:0;bottom:0;position:absolute;pointer-events:none;border-radius:inherit}.mat-mdc-button .mat-mdc-button-ripple,.mat-mdc-unelevated-button .mat-mdc-button-ripple,.mat-mdc-raised-button .mat-mdc-button-ripple,.mat-mdc-outlined-button .mat-mdc-button-ripple{overflow:hidden}.mat-mdc-button .mat-mdc-button-persistent-ripple::before,.mat-mdc-unelevated-button .mat-mdc-button-persistent-ripple::before,.mat-mdc-raised-button .mat-mdc-button-persistent-ripple::before,.mat-mdc-outlined-button .mat-mdc-button-persistent-ripple::before{content:"";opacity:0}.mat-mdc-button .mdc-button__label,.mat-mdc-unelevated-button .mdc-button__label,.mat-mdc-raised-button .mdc-button__label,.mat-mdc-outlined-button .mdc-button__label{z-index:1}.mat-mdc-button .mat-mdc-focus-indicator,.mat-mdc-unelevated-button .mat-mdc-focus-indicator,.mat-mdc-raised-button .mat-mdc-focus-indicator,.mat-mdc-outlined-button .mat-mdc-focus-indicator{top:0;left:0;right:0;bottom:0;position:absolute}.mat-mdc-button:focus .mat-mdc-focus-indicator::before,.mat-mdc-unelevated-button:focus .mat-mdc-focus-indicator::before,.mat-mdc-raised-button:focus .mat-mdc-focus-indicator::before,.mat-mdc-outlined-button:focus .mat-mdc-focus-indicator::before{content:""}.mat-mdc-button._mat-animation-noopable,.mat-mdc-unelevated-button._mat-animation-noopable,.mat-mdc-raised-button._mat-animation-noopable,.mat-mdc-outlined-button._mat-animation-noopable{transition:none !important;animation:none !important}.mat-mdc-button>.mat-icon,.mat-mdc-unelevated-button>.mat-icon,.mat-mdc-raised-button>.mat-icon,.mat-mdc-outlined-button>.mat-icon{display:inline-block;position:relative;vertical-align:top;font-size:1.125rem;height:1.125rem;width:1.125rem}.mat-mdc-outlined-button .mat-mdc-button-ripple,.mat-mdc-outlined-button .mdc-button__ripple{top:-1px;left:-1px;bottom:-1px;right:-1px;border-width:-1px}.mat-mdc-unelevated-button .mat-mdc-focus-indicator::before,.mat-mdc-raised-button .mat-mdc-focus-indicator::before{margin:calc(calc(var(--mat-mdc-focus-indicator-border-width, 3px) + 2px)*-1)}.mat-mdc-outlined-button .mat-mdc-focus-indicator::before{margin:calc(calc(var(--mat-mdc-focus-indicator-border-width, 3px) + 3px)*-1)}\n',
".cdk-high-contrast-active .mat-mdc-button:not(.mdc-button--outlined),.cdk-high-contrast-active .mat-mdc-unelevated-button:not(.mdc-button--outlined),.cdk-high-contrast-active .mat-mdc-raised-button:not(.mdc-button--outlined),.cdk-high-contrast-active .mat-mdc-outlined-button:not(.mdc-button--outlined),.cdk-high-contrast-active .mat-mdc-icon-button{outline:solid 1px}\n"],pb:2,Pa:0});var xB=new gf("mat-mdc-fab-default-options",{fa:"root",ba:wB});function wB(){return{color:"accent"}};var yB=class extends tB{constructor(a,b,c,d){super(a,b,c,d);dA(this.C,this.ra.R,{bj:!0})}};yB.K=function(a){return new (a||yB)(y(wi),y(Wt),y(Tk),y(qi,8))};
yB.Ca=Jf({type:yB,ha:[["button","mat-icon-button",""]],Za:14,Ia:function(a,b){a&2&&(Mp("disabled",b.St())("aria-disabled",b.Rt()),sq(vq,b.color?"mat-"+b.color:""),Wp("mat-mdc-button-disabled",b.disabled)("mat-mdc-button-disabled-interactive",b.lj)("_mat-animation-noopable","NoopAnimations"===b.Yc)("mat-unthemed",!b.color)("mat-mdc-button-base",!0))},ib:["matButton"],ka:!0,features:[Ep,Nr],attrs:["mat-icon-button",""],bc:["*"],ya:4,za:0,Ba:[[1,"mat-mdc-button-persistent-ripple","mdc-icon-button__ripple"],
[1,"mat-mdc-focus-indicator"],[1,"mat-mdc-button-touch-target"]],sa:function(a){a&1&&(Zq(),Oq(0,"span",0),$q(1),Oq(2,"span",1)(3,"span",2))},styles:['.mdc-icon-button{display:inline-block;position:relative;box-sizing:border-box;border:none;outline:none;background-color:rgba(0,0,0,0);fill:currentColor;color:inherit;text-decoration:none;cursor:pointer;user-select:none;z-index:0;overflow:visible}.mdc-icon-button .mdc-icon-button__touch{position:absolute;top:50%;height:48px;left:50%;width:48px;transform:translate(-50%, -50%)}@media screen and (forced-colors: active){.mdc-icon-button.mdc-ripple-upgraded--background-focused .mdc-icon-button__focus-ring,.mdc-icon-button:not(.mdc-ripple-upgraded):focus .mdc-icon-button__focus-ring{display:block}}.mdc-icon-button:disabled{cursor:default;pointer-events:none}.mdc-icon-button[hidden]{display:none}.mdc-icon-button--display-flex{align-items:center;display:inline-flex;justify-content:center}.mdc-icon-button__focus-ring{pointer-events:none;border:2px solid rgba(0,0,0,0);border-radius:6px;box-sizing:content-box;position:absolute;top:50%;left:50%;transform:translate(-50%, -50%);height:100%;width:100%;display:none}@media screen and (forced-colors: active){.mdc-icon-button__focus-ring{border-color:CanvasText}}.mdc-icon-button__focus-ring::after{content:"";border:2px solid rgba(0,0,0,0);border-radius:8px;display:block;position:absolute;top:50%;left:50%;transform:translate(-50%, -50%);height:calc(100% + 4px);width:calc(100% + 4px)}@media screen and (forced-colors: active){.mdc-icon-button__focus-ring::after{border-color:CanvasText}}.mdc-icon-button__icon{display:inline-block}.mdc-icon-button__icon.mdc-icon-button__icon--on{display:none}.mdc-icon-button--on .mdc-icon-button__icon{display:none}.mdc-icon-button--on .mdc-icon-button__icon.mdc-icon-button__icon--on{display:inline-block}.mdc-icon-button__link{height:100%;left:0;outline:none;position:absolute;top:0;width:100%}.mat-mdc-icon-button{color:var(--mdc-icon-button-icon-color)}.mat-mdc-icon-button .mdc-button__icon{font-size:var(--mdc-icon-button-icon-size)}.mat-mdc-icon-button svg,.mat-mdc-icon-button img{width:var(--mdc-icon-button-icon-size);height:var(--mdc-icon-button-icon-size)}.mat-mdc-icon-button:disabled{color:var(--mdc-icon-button-disabled-icon-color)}.mat-mdc-icon-button{border-radius:50%;flex-shrink:0;text-align:center;width:var(--mdc-icon-button-state-layer-size, 48px);height:var(--mdc-icon-button-state-layer-size, 48px);padding:calc(calc(var(--mdc-icon-button-state-layer-size, 48px) - var(--mdc-icon-button-icon-size, 24px)) / 2);font-size:var(--mdc-icon-button-icon-size);-webkit-tap-highlight-color:rgba(0,0,0,0)}.mat-mdc-icon-button svg{vertical-align:baseline}.mat-mdc-icon-button[disabled],.mat-mdc-icon-button.mat-mdc-button-disabled{cursor:default;pointer-events:none;color:var(--mdc-icon-button-disabled-icon-color)}.mat-mdc-icon-button.mat-mdc-button-disabled-interactive{pointer-events:auto}.mat-mdc-icon-button .mat-mdc-button-ripple,.mat-mdc-icon-button .mat-mdc-button-persistent-ripple,.mat-mdc-icon-button .mat-mdc-button-persistent-ripple::before{top:0;left:0;right:0;bottom:0;position:absolute;pointer-events:none;border-radius:inherit}.mat-mdc-icon-button .mat-mdc-button-ripple{overflow:hidden}.mat-mdc-icon-button .mat-mdc-button-persistent-ripple::before{content:"";opacity:0}.mat-mdc-icon-button .mdc-button__label{z-index:1}.mat-mdc-icon-button .mat-mdc-focus-indicator{top:0;left:0;right:0;bottom:0;position:absolute}.mat-mdc-icon-button:focus .mat-mdc-focus-indicator::before{content:""}.mat-mdc-icon-button .mat-ripple-element{background-color:var(--mat-icon-button-ripple-color)}.mat-mdc-icon-button .mat-mdc-button-persistent-ripple::before{background-color:var(--mat-icon-button-state-layer-color)}.mat-mdc-icon-button.mat-mdc-button-disabled .mat-mdc-button-persistent-ripple::before{background-color:var(--mat-icon-button-disabled-state-layer-color)}.mat-mdc-icon-button:hover .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-icon-button-hover-state-layer-opacity)}.mat-mdc-icon-button.cdk-program-focused .mat-mdc-button-persistent-ripple::before,.mat-mdc-icon-button.cdk-keyboard-focused .mat-mdc-button-persistent-ripple::before,.mat-mdc-icon-button.mat-mdc-button-disabled-interactive:focus .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-icon-button-focus-state-layer-opacity)}.mat-mdc-icon-button:active .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-icon-button-pressed-state-layer-opacity)}.mat-mdc-icon-button .mat-mdc-button-touch-target{position:absolute;top:50%;height:48px;left:50%;width:48px;transform:translate(-50%, -50%);display:var(--mat-icon-button-touch-target-display)}.mat-mdc-icon-button._mat-animation-noopable{transition:none !important;animation:none !important}.mat-mdc-icon-button .mat-mdc-button-persistent-ripple{border-radius:50%}.mat-mdc-icon-button.mat-unthemed:not(.mdc-ripple-upgraded):focus::before,.mat-mdc-icon-button.mat-primary:not(.mdc-ripple-upgraded):focus::before,.mat-mdc-icon-button.mat-accent:not(.mdc-ripple-upgraded):focus::before,.mat-mdc-icon-button.mat-warn:not(.mdc-ripple-upgraded):focus::before{background:rgba(0,0,0,0);opacity:1}\n',
".cdk-high-contrast-active .mat-mdc-button:not(.mdc-button--outlined),.cdk-high-contrast-active .mat-mdc-unelevated-button:not(.mdc-button--outlined),.cdk-high-contrast-active .mat-mdc-raised-button:not(.mdc-button--outlined),.cdk-high-contrast-active .mat-mdc-outlined-button:not(.mdc-button--outlined),.cdk-high-contrast-active .mat-mdc-icon-button{outline:solid 1px}\n"],pb:2,Pa:0});var zB=class{};zB.K=function(a){return new (a||zB)};zB.Ya=Sf({type:zB});zB.Xa=Qd({imports:[mz,Yz,mz]});var AB=class{constructor(a,b){this.Ub=a;this.options=b}Fb(){this.Ub.R.querySelector(".mat-icon, .material-icons")&&this.Ub.R.classList.add("gmat-mdc-button-with-prefix")}};AB.K=function(a){return new (a||AB)(y(wi),y(qB))};
AB.Fa=Uf({type:AB,ha:[["button","mat-button",""],["a","mat-button",""],["button","mat-raised-button",""],["a","mat-raised-button",""],["button","mat-flat-button",""],["a","mat-flat-button",""],["button","mat-stroked-button",""],["a","mat-stroked-button",""],["button","mat-icon-button",""],["a","mat-icon-button",""],[""]],Za:2,Ia:function(a,b){a&2&&Wp("gmat-mdc-button",!b.options.tc)}});var BB=class{};BB.K=function(a){return new (a||BB)};BB.Ya=Sf({type:BB});BB.Xa=Qd({Wa:[{pa:xB,nb:{color:"primary"}}],imports:[zB]});var CB=new gf("GMAT_CHECKBOX_OPTIONS",{fa:"root",ba:()=>({tc:!1})});var DB=class{constructor(a){this.options=a}};DB.K=function(a){return new (a||DB)(y(CB))};DB.Fa=Uf({type:DB,ha:[["mat-checkbox"]],Za:2,Ia:function(a,b){a&2&&Wp("gmat-mdc-checkbox",!b.options.tc)}});var FB=new gf("mat-checkbox-default-options",{fa:"root",ba:EB});function EB(){return{color:"accent",wB:"check-indeterminate"}};const GB=["input"],HB=["label"];var JB={pa:qw,Ab:df(()=>IB),Eb:!0},KB=class{};let LB=0;const MB=EB();
var IB=class{focus(){this.kg.R.focus()}tp(a){const b=new KB;b.source=this;b.checked=a;return b}hA(){let a;return null==(a=this.kg)?void 0:a.R}get ir(){return`${this.id||this.Ik}-input`}constructor(a,b,c,d,e,f){this.ra=a;this.Da=b;this.qa=c;this.Yc=e;this.Si=f;this.yf={Jy:"mdc-checkbox--anim-unchecked-checked",Ky:"mdc-checkbox--anim-unchecked-indeterminate",rB:"mdc-checkbox--anim-checked-unchecked",jv:"mdc-checkbox--anim-checked-indeterminate",CC:"mdc-checkbox--anim-indeterminate-checked",DC:"mdc-checkbox--anim-indeterminate-unchecked"};
this.ariaLabel="";this.ad=null;this.Oh="after";this.name=null;this.xb=new Jk;this.fr=new Jk;this.de=()=>{};this.Ki="";this.Ht=0;this.sp=()=>{};this.Lu=()=>{};this.Bf=this.Ib=this.zf=!1;this.Si=this.Si||MB;this.color=this.Si.color||MB.color;this.tabIndex=parseInt(d)||0;this.id=this.Ik=`mat-mdc-checkbox-${++LB}`}uc(a){a.required&&this.Lu()}pd(){this.Eu(this.Bf)}get checked(){return this.zf}set checked(a){a!=this.checked&&(this.zf=a,en(this.Da))}get disabled(){return this.Ib}set disabled(a){a!==this.disabled&&
(this.Ib=a,en(this.Da))}get indeterminate(){return this.Bf}set indeterminate(a){const b=a!=this.Bf;this.Bf=a;b&&(this.Bf?this.Yp(3):this.Yp(this.checked?1:2),this.fr.emit(this.Bf));this.Eu(this.Bf)}FF(){return this.Nc||this.disabled}HF(){fn(this.Da)}ti(a){this.checked=!!a}Ng(a){this.sp=a}Nj(a){this.de=a}ji(a){this.disabled=a}validate(a){return this.required&&!0!==a.value?{required:!0}:null}hs(a){this.Lu=a}Yp(a){let b=this.Ht,c=this.hA();if(b!==a&&c&&(this.Ki&&c.classList.remove(this.Ki),this.Ki=this.gA(b,
a),this.Ht=a,0<this.Ki.length)){c.classList.add(this.Ki);const d=this.Ki;Qk(this.qa,()=>{setTimeout(()=>{c.classList.remove(d)},1E3)})}}Li(){this.sp(this.checked);this.xb.emit(this.tp(this.checked));this.kg&&(this.kg.R.checked=this.checked)}toggle(){this.checked=!this.checked;this.sp(this.checked)}Xt(){let a;const b=null==(a=this.Si)?void 0:a.wB;this.disabled||"noop"===b?this.disabled||"noop"!==b||(this.kg.R.checked=this.checked,this.kg.R.indeterminate=this.indeterminate):(this.indeterminate&&"check"!==
b&&Promise.resolve().then(()=>{this.Bf=!1;this.fr.emit(this.Bf)}),this.zf=!this.zf,this.Yp(this.zf?1:2),this.Li())}EA(a){a.stopPropagation()}Ck(){Promise.resolve().then(()=>{this.de();en(this.Da)})}gA(a,b){if("NoopAnimations"===this.Yc)return"";switch(a){case 0:if(1===b)return this.yf.Jy;if(3==b)return this.zf?this.yf.jv:this.yf.Ky;break;case 2:return 1===b?this.yf.Jy:this.yf.Ky;case 1:return 2===b?this.yf.rB:this.yf.jv;case 3:return 1===b?this.yf.CC:this.yf.DC}return""}Eu(a){const b=this.kg;b&&(b.R.indeterminate=
a)}DA(){this.Xt()}HA(){this.Xt();this.disabled||this.kg.R.focus()}OA(a){a.target&&this.xA.R.contains(a.target)&&a.stopPropagation()}};IB.K=function(a){return new (a||IB)(y(wi),y(Tr),y(Tk),Dh("tabindex"),y(qi,8),y(FB,8))};
IB.Ca=Jf({type:IB,ha:[["mat-checkbox"]],eb:function(a,b){a&1&&(vr(GB,5),vr(HB,5),vr(Lz,5));if(a&2){let c;wr(c=xr())&&(b.kg=c.first);wr(c=xr())&&(b.xA=c.first);wr(c=xr())&&(b.Ix=c.first)}},Ka:[1,"mat-mdc-checkbox"],Za:14,Ia:function(a,b){a&2&&(Sq("id",b.id),Mp("tabindex",null)("aria-label",null)("aria-labelledby",null),sq(vq,b.color?"mat-"+b.color:"mat-accent"),Wp("_mat-animation-noopable","NoopAnimations"===b.Yc)("mdc-checkbox--disabled",b.disabled)("mat-mdc-checkbox-disabled",b.disabled)("mat-mdc-checkbox-checked",
b.checked))},inputs:{ariaLabel:[0,"aria-label","ariaLabel"],ad:[0,"aria-labelledby","ariaLabelledby"],Mk:[0,"aria-describedby","ariaDescribedby"],id:"id",required:[2,"required","required",As],Oh:"labelPosition",name:"name",value:"value",Nc:[2,"disableRipple","disableRipple",As],tabIndex:[2,"tabIndex","tabIndex",a=>null==a?void 0:Bs(a)],color:"color",checked:[2,"checked","checked",As],disabled:[2,"disabled","disabled",As],indeterminate:[2,"indeterminate","indeterminate",As]},outputs:{xb:"change",fr:"indeterminateChange"},
ib:["matCheckbox"],ka:!0,features:[Lr([JB,{pa:ww,Ab:IB,Eb:!0}]),Lp,Hg,Nr],bc:["*"],ya:15,za:19,Ba:[["checkbox",""],["input",""],["label",""],["mat-internal-form-field","",3,"click","labelPosition"],[1,"mdc-checkbox"],[1,"mat-mdc-checkbox-touch-target",3,"click"],["type","checkbox",1,"mdc-checkbox__native-control",3,"blur","click","change","checked","indeterminate","disabled","id","required","tabIndex"],[1,"mdc-checkbox__ripple"],[1,"mdc-checkbox__background"],["focusable","false","viewBox","0 0 24 24",
"aria-hidden","true",1,"mdc-checkbox__checkmark"],["fill","none","d","M1.73,12.91 8.1,19.28 22.79,4.59",1,"mdc-checkbox__checkmark-path"],[1,"mdc-checkbox__mixedmark"],["mat-ripple","",1,"mat-mdc-checkbox-ripple","mat-mdc-focus-indicator",3,"matRippleTrigger","matRippleDisabled","matRippleCentered"],[1,"mdc-label",3,"for"]],sa:function(a,b){if(a&1){const c=Zg();Zq();R(0,"div",3);V("click",function(d){t(c);return x(b.OA(d))});R(1,"div",4,0)(3,"div",5);V("click",function(){t(c);return x(b.HA())});U();
R(4,"input",6,1);V("blur",function(){t(c);return x(b.Ck())})("click",function(){t(c);return x(b.DA())})("change",function(d){t(c);return x(b.EA(d))});U();Oq(6,"div",7);R(7,"div",8);kh();R(8,"svg",9);Oq(9,"path",10);U();Wg.al=null;Oq(10,"div",11);U();Oq(11,"div",12);U();R(12,"label",13,2);$q(14);U()()}a&2&&(a=Ar(2),Q("labelPosition",b.Oh),D(4),Wp("mdc-checkbox--selected",b.checked),Q("checked",b.checked)("indeterminate",b.indeterminate)("disabled",b.disabled)("id",b.ir)("required",b.required)("tabIndex",
b.disabled?-1:b.tabIndex),Mp("aria-label",b.ariaLabel||null)("aria-labelledby",b.ad)("aria-describedby",b.Mk)("aria-checked",b.indeterminate?"mixed":null)("name",b.name)("value",b.value),D(7),Q("matRippleTrigger",a)("matRippleDisabled",b.Nc||b.disabled)("matRippleCentered",!0),D(),Q("for",b.ir))},Ha:[Lz,qz],styles:['@keyframes mdc-checkbox-unchecked-checked-checkmark-path{0%,50%{stroke-dashoffset:29.7833385}50%{animation-timing-function:cubic-bezier(0, 0, 0.2, 1)}100%{stroke-dashoffset:0}}@keyframes mdc-checkbox-unchecked-indeterminate-mixedmark{0%,68.2%{transform:scaleX(0)}68.2%{animation-timing-function:cubic-bezier(0, 0, 0, 1)}100%{transform:scaleX(1)}}@keyframes mdc-checkbox-checked-unchecked-checkmark-path{from{animation-timing-function:cubic-bezier(0.4, 0, 1, 1);opacity:1;stroke-dashoffset:0}to{opacity:0;stroke-dashoffset:-29.7833385}}@keyframes mdc-checkbox-checked-indeterminate-checkmark{from{animation-timing-function:cubic-bezier(0, 0, 0.2, 1);transform:rotate(0deg);opacity:1}to{transform:rotate(45deg);opacity:0}}@keyframes mdc-checkbox-indeterminate-checked-checkmark{from{animation-timing-function:cubic-bezier(0.14, 0, 0, 1);transform:rotate(45deg);opacity:0}to{transform:rotate(360deg);opacity:1}}@keyframes mdc-checkbox-checked-indeterminate-mixedmark{from{animation-timing-function:mdc-animation-deceleration-curve-timing-function;transform:rotate(-45deg);opacity:0}to{transform:rotate(0deg);opacity:1}}@keyframes mdc-checkbox-indeterminate-checked-mixedmark{from{animation-timing-function:cubic-bezier(0.14, 0, 0, 1);transform:rotate(0deg);opacity:1}to{transform:rotate(315deg);opacity:0}}@keyframes mdc-checkbox-indeterminate-unchecked-mixedmark{0%{animation-timing-function:linear;transform:scaleX(1);opacity:1}32.8%,100%{transform:scaleX(0);opacity:0}}.mdc-checkbox{display:inline-block;position:relative;flex:0 0 18px;box-sizing:content-box;width:18px;height:18px;line-height:0;white-space:nowrap;cursor:pointer;vertical-align:bottom}.mdc-checkbox[hidden]{display:none}.mdc-checkbox.mdc-ripple-upgraded--background-focused .mdc-checkbox__focus-ring,.mdc-checkbox:not(.mdc-ripple-upgraded):focus .mdc-checkbox__focus-ring{pointer-events:none;border:2px solid rgba(0,0,0,0);border-radius:6px;box-sizing:content-box;position:absolute;top:50%;left:50%;transform:translate(-50%, -50%);height:100%;width:100%}@media screen and (forced-colors: active){.mdc-checkbox.mdc-ripple-upgraded--background-focused .mdc-checkbox__focus-ring,.mdc-checkbox:not(.mdc-ripple-upgraded):focus .mdc-checkbox__focus-ring{border-color:CanvasText}}.mdc-checkbox.mdc-ripple-upgraded--background-focused .mdc-checkbox__focus-ring::after,.mdc-checkbox:not(.mdc-ripple-upgraded):focus .mdc-checkbox__focus-ring::after{content:"";border:2px solid rgba(0,0,0,0);border-radius:8px;display:block;position:absolute;top:50%;left:50%;transform:translate(-50%, -50%);height:calc(100% + 4px);width:calc(100% + 4px)}@media screen and (forced-colors: active){.mdc-checkbox.mdc-ripple-upgraded--background-focused .mdc-checkbox__focus-ring::after,.mdc-checkbox:not(.mdc-ripple-upgraded):focus .mdc-checkbox__focus-ring::after{border-color:CanvasText}}@media all and (-ms-high-contrast: none){.mdc-checkbox .mdc-checkbox__focus-ring{display:none}}@media screen and (forced-colors: active),(-ms-high-contrast: active){.mdc-checkbox__mixedmark{margin:0 1px}}.mdc-checkbox--disabled{cursor:default;pointer-events:none}.mdc-checkbox__background{display:inline-flex;position:absolute;align-items:center;justify-content:center;box-sizing:border-box;width:18px;height:18px;border:2px solid currentColor;border-radius:2px;background-color:rgba(0,0,0,0);pointer-events:none;will-change:background-color,border-color;transition:background-color 90ms 0ms cubic-bezier(0.4, 0, 0.6, 1),border-color 90ms 0ms cubic-bezier(0.4, 0, 0.6, 1)}.mdc-checkbox__checkmark{position:absolute;top:0;right:0;bottom:0;left:0;width:100%;opacity:0;transition:opacity 180ms 0ms cubic-bezier(0.4, 0, 0.6, 1)}.mdc-checkbox--upgraded .mdc-checkbox__checkmark{opacity:1}.mdc-checkbox__checkmark-path{transition:stroke-dashoffset 180ms 0ms cubic-bezier(0.4, 0, 0.6, 1);stroke:currentColor;stroke-width:3.12px;stroke-dashoffset:29.7833385;stroke-dasharray:29.7833385}.mdc-checkbox__mixedmark{width:100%;height:0;transform:scaleX(0) rotate(0deg);border-width:1px;border-style:solid;opacity:0;transition:opacity 90ms 0ms cubic-bezier(0.4, 0, 0.6, 1),transform 90ms 0ms cubic-bezier(0.4, 0, 0.6, 1)}.mdc-checkbox--anim-unchecked-checked .mdc-checkbox__background,.mdc-checkbox--anim-unchecked-indeterminate .mdc-checkbox__background,.mdc-checkbox--anim-checked-unchecked .mdc-checkbox__background,.mdc-checkbox--anim-indeterminate-unchecked .mdc-checkbox__background{animation-duration:180ms;animation-timing-function:linear}.mdc-checkbox--anim-unchecked-checked .mdc-checkbox__checkmark-path{animation:mdc-checkbox-unchecked-checked-checkmark-path 180ms linear 0s;transition:none}.mdc-checkbox--anim-unchecked-indeterminate .mdc-checkbox__mixedmark{animation:mdc-checkbox-unchecked-indeterminate-mixedmark 90ms linear 0s;transition:none}.mdc-checkbox--anim-checked-unchecked .mdc-checkbox__checkmark-path{animation:mdc-checkbox-checked-unchecked-checkmark-path 90ms linear 0s;transition:none}.mdc-checkbox--anim-checked-indeterminate .mdc-checkbox__checkmark{animation:mdc-checkbox-checked-indeterminate-checkmark 90ms linear 0s;transition:none}.mdc-checkbox--anim-checked-indeterminate .mdc-checkbox__mixedmark{animation:mdc-checkbox-checked-indeterminate-mixedmark 90ms linear 0s;transition:none}.mdc-checkbox--anim-indeterminate-checked .mdc-checkbox__checkmark{animation:mdc-checkbox-indeterminate-checked-checkmark 500ms linear 0s;transition:none}.mdc-checkbox--anim-indeterminate-checked .mdc-checkbox__mixedmark{animation:mdc-checkbox-indeterminate-checked-mixedmark 500ms linear 0s;transition:none}.mdc-checkbox--anim-indeterminate-unchecked .mdc-checkbox__mixedmark{animation:mdc-checkbox-indeterminate-unchecked-mixedmark 300ms linear 0s;transition:none}.mdc-checkbox__native-control:checked~.mdc-checkbox__background,.mdc-checkbox__native-control:indeterminate~.mdc-checkbox__background,.mdc-checkbox__native-control[data-indeterminate=true]~.mdc-checkbox__background{transition:border-color 90ms 0ms cubic-bezier(0, 0, 0.2, 1),background-color 90ms 0ms cubic-bezier(0, 0, 0.2, 1)}.mdc-checkbox__native-control:checked~.mdc-checkbox__background .mdc-checkbox__checkmark-path,.mdc-checkbox__native-control:indeterminate~.mdc-checkbox__background .mdc-checkbox__checkmark-path,.mdc-checkbox__native-control[data-indeterminate=true]~.mdc-checkbox__background .mdc-checkbox__checkmark-path{stroke-dashoffset:0}.mdc-checkbox__native-control{position:absolute;margin:0;padding:0;opacity:0;cursor:inherit}.mdc-checkbox__native-control:disabled{cursor:default;pointer-events:none}.mdc-checkbox--touch{margin:calc((var(--mdc-checkbox-state-layer-size) - var(--mdc-checkbox-state-layer-size)) / 2)}.mdc-checkbox--touch .mdc-checkbox__native-control{top:calc((var(--mdc-checkbox-state-layer-size) - var(--mdc-checkbox-state-layer-size)) / 2);right:calc((var(--mdc-checkbox-state-layer-size) - var(--mdc-checkbox-state-layer-size)) / 2);left:calc((var(--mdc-checkbox-state-layer-size) - var(--mdc-checkbox-state-layer-size)) / 2);width:var(--mdc-checkbox-state-layer-size);height:var(--mdc-checkbox-state-layer-size)}.mdc-checkbox__native-control:checked~.mdc-checkbox__background .mdc-checkbox__checkmark{transition:opacity 180ms 0ms cubic-bezier(0, 0, 0.2, 1),transform 180ms 0ms cubic-bezier(0, 0, 0.2, 1);opacity:1}.mdc-checkbox__native-control:checked~.mdc-checkbox__background .mdc-checkbox__mixedmark{transform:scaleX(1) rotate(-45deg)}.mdc-checkbox__native-control:indeterminate~.mdc-checkbox__background .mdc-checkbox__checkmark,.mdc-checkbox__native-control[data-indeterminate=true]~.mdc-checkbox__background .mdc-checkbox__checkmark{transform:rotate(45deg);opacity:0;transition:opacity 90ms 0ms cubic-bezier(0.4, 0, 0.6, 1),transform 90ms 0ms cubic-bezier(0.4, 0, 0.6, 1)}.mdc-checkbox__native-control:indeterminate~.mdc-checkbox__background .mdc-checkbox__mixedmark,.mdc-checkbox__native-control[data-indeterminate=true]~.mdc-checkbox__background .mdc-checkbox__mixedmark{transform:scaleX(1) rotate(0deg);opacity:1}.mdc-checkbox.mdc-checkbox--upgraded .mdc-checkbox__background,.mdc-checkbox.mdc-checkbox--upgraded .mdc-checkbox__checkmark,.mdc-checkbox.mdc-checkbox--upgraded .mdc-checkbox__checkmark-path,.mdc-checkbox.mdc-checkbox--upgraded .mdc-checkbox__mixedmark{transition:none}.mdc-checkbox{padding:calc((var(--mdc-checkbox-state-layer-size) - 18px) / 2);margin:calc((var(--mdc-checkbox-state-layer-size) - var(--mdc-checkbox-state-layer-size)) / 2)}.mdc-checkbox .mdc-checkbox__native-control[disabled]:not(:checked):not(:indeterminate):not([data-indeterminate=true])~.mdc-checkbox__background{border-color:var(--mdc-checkbox-disabled-unselected-icon-color);background-color:transparent}.mdc-checkbox .mdc-checkbox__native-control[disabled]:checked~.mdc-checkbox__background,.mdc-checkbox .mdc-checkbox__native-control[disabled]:indeterminate~.mdc-checkbox__background,.mdc-checkbox .mdc-checkbox__native-control[data-indeterminate=true][disabled]~.mdc-checkbox__background{border-color:transparent;background-color:var(--mdc-checkbox-disabled-selected-icon-color)}.mdc-checkbox .mdc-checkbox__native-control:enabled~.mdc-checkbox__background .mdc-checkbox__checkmark{color:var(--mdc-checkbox-selected-checkmark-color)}.mdc-checkbox .mdc-checkbox__native-control:enabled~.mdc-checkbox__background .mdc-checkbox__mixedmark{border-color:var(--mdc-checkbox-selected-checkmark-color)}.mdc-checkbox .mdc-checkbox__native-control:disabled~.mdc-checkbox__background .mdc-checkbox__checkmark{color:var(--mdc-checkbox-disabled-selected-checkmark-color)}.mdc-checkbox .mdc-checkbox__native-control:disabled~.mdc-checkbox__background .mdc-checkbox__mixedmark{border-color:var(--mdc-checkbox-disabled-selected-checkmark-color)}.mdc-checkbox .mdc-checkbox__native-control:enabled:not(:checked):not(:indeterminate):not([data-indeterminate=true])~.mdc-checkbox__background{border-color:var(--mdc-checkbox-unselected-icon-color);background-color:transparent}.mdc-checkbox .mdc-checkbox__native-control:enabled:checked~.mdc-checkbox__background,.mdc-checkbox .mdc-checkbox__native-control:enabled:indeterminate~.mdc-checkbox__background,.mdc-checkbox .mdc-checkbox__native-control[data-indeterminate=true]:enabled~.mdc-checkbox__background{border-color:var(--mdc-checkbox-selected-icon-color);background-color:var(--mdc-checkbox-selected-icon-color)}@keyframes mdc-checkbox-fade-in-background-8A000000FFF4433600000000FFF44336{0%{border-color:var(--mdc-checkbox-unselected-icon-color);background-color:transparent}50%{border-color:var(--mdc-checkbox-selected-icon-color);background-color:var(--mdc-checkbox-selected-icon-color)}}@keyframes mdc-checkbox-fade-out-background-8A000000FFF4433600000000FFF44336{0%,80%{border-color:var(--mdc-checkbox-selected-icon-color);background-color:var(--mdc-checkbox-selected-icon-color)}100%{border-color:var(--mdc-checkbox-unselected-icon-color);background-color:transparent}}.mdc-checkbox.mdc-checkbox--anim-unchecked-checked .mdc-checkbox__native-control:enabled~.mdc-checkbox__background,.mdc-checkbox.mdc-checkbox--anim-unchecked-indeterminate .mdc-checkbox__native-control:enabled~.mdc-checkbox__background{animation-name:mdc-checkbox-fade-in-background-8A000000FFF4433600000000FFF44336}.mdc-checkbox.mdc-checkbox--anim-checked-unchecked .mdc-checkbox__native-control:enabled~.mdc-checkbox__background,.mdc-checkbox.mdc-checkbox--anim-indeterminate-unchecked .mdc-checkbox__native-control:enabled~.mdc-checkbox__background{animation-name:mdc-checkbox-fade-out-background-8A000000FFF4433600000000FFF44336}.mdc-checkbox:hover .mdc-checkbox__native-control:enabled:not(:checked):not(:indeterminate):not([data-indeterminate=true])~.mdc-checkbox__background{border-color:var(--mdc-checkbox-unselected-hover-icon-color);background-color:transparent}.mdc-checkbox:hover .mdc-checkbox__native-control:enabled:checked~.mdc-checkbox__background,.mdc-checkbox:hover .mdc-checkbox__native-control:enabled:indeterminate~.mdc-checkbox__background,.mdc-checkbox:hover .mdc-checkbox__native-control[data-indeterminate=true]:enabled~.mdc-checkbox__background{border-color:var(--mdc-checkbox-selected-hover-icon-color);background-color:var(--mdc-checkbox-selected-hover-icon-color)}@keyframes mdc-checkbox-fade-in-background-FF212121FFF4433600000000FFF44336{0%{border-color:var(--mdc-checkbox-unselected-hover-icon-color);background-color:transparent}50%{border-color:var(--mdc-checkbox-selected-hover-icon-color);background-color:var(--mdc-checkbox-selected-hover-icon-color)}}@keyframes mdc-checkbox-fade-out-background-FF212121FFF4433600000000FFF44336{0%,80%{border-color:var(--mdc-checkbox-selected-hover-icon-color);background-color:var(--mdc-checkbox-selected-hover-icon-color)}100%{border-color:var(--mdc-checkbox-unselected-hover-icon-color);background-color:transparent}}.mdc-checkbox:hover.mdc-checkbox--anim-unchecked-checked .mdc-checkbox__native-control:enabled~.mdc-checkbox__background,.mdc-checkbox:hover.mdc-checkbox--anim-unchecked-indeterminate .mdc-checkbox__native-control:enabled~.mdc-checkbox__background{animation-name:mdc-checkbox-fade-in-background-FF212121FFF4433600000000FFF44336}.mdc-checkbox:hover.mdc-checkbox--anim-checked-unchecked .mdc-checkbox__native-control:enabled~.mdc-checkbox__background,.mdc-checkbox:hover.mdc-checkbox--anim-indeterminate-unchecked .mdc-checkbox__native-control:enabled~.mdc-checkbox__background{animation-name:mdc-checkbox-fade-out-background-FF212121FFF4433600000000FFF44336}.mdc-checkbox:not(:disabled):active .mdc-checkbox__native-control:enabled:not(:checked):not(:indeterminate):not([data-indeterminate=true])~.mdc-checkbox__background{border-color:var(--mdc-checkbox-unselected-pressed-icon-color);background-color:transparent}.mdc-checkbox:not(:disabled):active .mdc-checkbox__native-control:enabled:checked~.mdc-checkbox__background,.mdc-checkbox:not(:disabled):active .mdc-checkbox__native-control:enabled:indeterminate~.mdc-checkbox__background,.mdc-checkbox:not(:disabled):active .mdc-checkbox__native-control[data-indeterminate=true]:enabled~.mdc-checkbox__background{border-color:var(--mdc-checkbox-selected-pressed-icon-color);background-color:var(--mdc-checkbox-selected-pressed-icon-color)}@keyframes mdc-checkbox-fade-in-background-8A000000FFF4433600000000FFF44336{0%{border-color:var(--mdc-checkbox-unselected-pressed-icon-color);background-color:transparent}50%{border-color:var(--mdc-checkbox-selected-pressed-icon-color);background-color:var(--mdc-checkbox-selected-pressed-icon-color)}}@keyframes mdc-checkbox-fade-out-background-8A000000FFF4433600000000FFF44336{0%,80%{border-color:var(--mdc-checkbox-selected-pressed-icon-color);background-color:var(--mdc-checkbox-selected-pressed-icon-color)}100%{border-color:var(--mdc-checkbox-unselected-pressed-icon-color);background-color:transparent}}.mdc-checkbox:not(:disabled):active.mdc-checkbox--anim-unchecked-checked .mdc-checkbox__native-control:enabled~.mdc-checkbox__background,.mdc-checkbox:not(:disabled):active.mdc-checkbox--anim-unchecked-indeterminate .mdc-checkbox__native-control:enabled~.mdc-checkbox__background{animation-name:mdc-checkbox-fade-in-background-8A000000FFF4433600000000FFF44336}.mdc-checkbox:not(:disabled):active.mdc-checkbox--anim-checked-unchecked .mdc-checkbox__native-control:enabled~.mdc-checkbox__background,.mdc-checkbox:not(:disabled):active.mdc-checkbox--anim-indeterminate-unchecked .mdc-checkbox__native-control:enabled~.mdc-checkbox__background{animation-name:mdc-checkbox-fade-out-background-8A000000FFF4433600000000FFF44336}.mdc-checkbox .mdc-checkbox__background{top:calc((var(--mdc-checkbox-state-layer-size) - 18px) / 2);left:calc((var(--mdc-checkbox-state-layer-size) - 18px) / 2)}.mdc-checkbox .mdc-checkbox__native-control{top:calc((var(--mdc-checkbox-state-layer-size) - var(--mdc-checkbox-state-layer-size)) / 2);right:calc((var(--mdc-checkbox-state-layer-size) - var(--mdc-checkbox-state-layer-size)) / 2);left:calc((var(--mdc-checkbox-state-layer-size) - var(--mdc-checkbox-state-layer-size)) / 2);width:var(--mdc-checkbox-state-layer-size);height:var(--mdc-checkbox-state-layer-size)}.mdc-checkbox .mdc-checkbox__native-control:enabled:focus:focus:not(:checked):not(:indeterminate)~.mdc-checkbox__background{border-color:var(--mdc-checkbox-unselected-focus-icon-color)}.mdc-checkbox .mdc-checkbox__native-control:enabled:focus:checked~.mdc-checkbox__background,.mdc-checkbox .mdc-checkbox__native-control:enabled:focus:indeterminate~.mdc-checkbox__background{border-color:var(--mdc-checkbox-selected-focus-icon-color);background-color:var(--mdc-checkbox-selected-focus-icon-color)}.mdc-checkbox:hover .mdc-checkbox__ripple{opacity:var(--mdc-checkbox-unselected-hover-state-layer-opacity);background-color:var(--mdc-checkbox-unselected-hover-state-layer-color)}.mdc-checkbox:hover .mat-mdc-checkbox-ripple .mat-ripple-element{background-color:var(--mdc-checkbox-unselected-hover-state-layer-color)}.mdc-checkbox .mdc-checkbox__native-control:focus~.mdc-checkbox__ripple{opacity:var(--mdc-checkbox-unselected-focus-state-layer-opacity);background-color:var(--mdc-checkbox-unselected-focus-state-layer-color)}.mdc-checkbox .mdc-checkbox__native-control:focus~.mat-mdc-checkbox-ripple .mat-ripple-element{background-color:var(--mdc-checkbox-unselected-focus-state-layer-color)}.mdc-checkbox:active .mdc-checkbox__native-control~.mdc-checkbox__ripple{opacity:var(--mdc-checkbox-unselected-pressed-state-layer-opacity);background-color:var(--mdc-checkbox-unselected-pressed-state-layer-color)}.mdc-checkbox:active .mdc-checkbox__native-control~.mat-mdc-checkbox-ripple .mat-ripple-element{background-color:var(--mdc-checkbox-unselected-pressed-state-layer-color)}.mdc-checkbox:hover .mdc-checkbox__native-control:checked~.mdc-checkbox__ripple{opacity:var(--mdc-checkbox-selected-hover-state-layer-opacity);background-color:var(--mdc-checkbox-selected-hover-state-layer-color)}.mdc-checkbox:hover .mdc-checkbox__native-control:checked~.mat-mdc-checkbox-ripple .mat-ripple-element{background-color:var(--mdc-checkbox-selected-hover-state-layer-color)}.mdc-checkbox .mdc-checkbox__native-control:focus:checked~.mdc-checkbox__ripple{opacity:var(--mdc-checkbox-selected-focus-state-layer-opacity);background-color:var(--mdc-checkbox-selected-focus-state-layer-color)}.mdc-checkbox .mdc-checkbox__native-control:focus:checked~.mat-mdc-checkbox-ripple .mat-ripple-element{background-color:var(--mdc-checkbox-selected-focus-state-layer-color)}.mdc-checkbox:active .mdc-checkbox__native-control:checked~.mdc-checkbox__ripple{opacity:var(--mdc-checkbox-selected-pressed-state-layer-opacity);background-color:var(--mdc-checkbox-selected-pressed-state-layer-color)}.mdc-checkbox:active .mdc-checkbox__native-control:checked~.mat-mdc-checkbox-ripple .mat-ripple-element{background-color:var(--mdc-checkbox-selected-pressed-state-layer-color)}.mat-mdc-checkbox{display:inline-block;position:relative;-webkit-tap-highlight-color:rgba(0,0,0,0)}.mat-mdc-checkbox .mdc-checkbox__background{-webkit-print-color-adjust:exact;color-adjust:exact}.mat-mdc-checkbox._mat-animation-noopable *,.mat-mdc-checkbox._mat-animation-noopable *::before{transition:none !important;animation:none !important}.mat-mdc-checkbox label{cursor:pointer}.mat-mdc-checkbox .mat-internal-form-field{color:var(--mat-checkbox-label-text-color);font-family:var(--mat-checkbox-label-text-font);line-height:var(--mat-checkbox-label-text-line-height);font-size:var(--mat-checkbox-label-text-size);letter-spacing:var(--mat-checkbox-label-text-tracking);font-weight:var(--mat-checkbox-label-text-weight)}.mat-mdc-checkbox.mat-mdc-checkbox-disabled label{cursor:default;color:var(--mat-checkbox-disabled-label-color)}.mat-mdc-checkbox label:empty{display:none}.cdk-high-contrast-active .mat-mdc-checkbox.mat-mdc-checkbox-disabled{opacity:.5}.cdk-high-contrast-active .mat-mdc-checkbox .mdc-checkbox__checkmark{--mdc-checkbox-selected-checkmark-color: CanvasText;--mdc-checkbox-disabled-selected-checkmark-color: CanvasText}.mat-mdc-checkbox .mdc-checkbox__ripple{opacity:0}.mat-mdc-checkbox-ripple,.mdc-checkbox__ripple{top:0;left:0;right:0;bottom:0;position:absolute;border-radius:50%;pointer-events:none}.mat-mdc-checkbox-ripple:not(:empty),.mdc-checkbox__ripple:not(:empty){transform:translateZ(0)}.mat-mdc-checkbox-ripple .mat-ripple-element{opacity:.1}.mat-mdc-checkbox-touch-target{position:absolute;top:50%;height:48px;left:50%;width:48px;transform:translate(-50%, -50%);display:var(--mat-checkbox-touch-target-display)}.mat-mdc-checkbox-ripple::before{border-radius:50%}.mdc-checkbox__native-control:focus~.mat-mdc-focus-indicator::before{content:""}\n'],
pb:2,Pa:0});var NB=class{};NB.K=function(a){return new (a||NB)};NB.Ya=Sf({type:NB});NB.Xa=Qd({imports:[IB,mz,mz]});var OB=class{};OB.K=function(a){return new (a||OB)};OB.Ya=Sf({type:OB});OB.Xa=Qd({Wa:[{pa:FB,nb:{color:"primary"}}],imports:[NB]});var PB=new gf("GMAT_DIALOG_OPTIONS",{fa:"root",ba:()=>({tc:!1})});var QB=class{constructor(){this.role="dialog";this.hb="";this.Vb=!0;this.fc="";this.yd=!1;this.height=this.width="";this.ariaLabel=this.hq=this.gq=this.data=null;this.ariaModal=!0;this.Ym="first-tabbable";this.ov=this.Pv=this.Pg=!0}};var RB=class{constructor(){this.role="dialog";this.hb="";this.Vb=!0;this.fc="";this.yd=!1;this.height=this.width="";this.ariaLabel=this.hq=this.gq=this.data=null;this.ariaModal=!0;this.Ym="first-tabbable";this.pv=this.nv=this.ov=this.Pg=!0}};function SB(){}function TB(a){const b=a.ra.R;switch(a.Aa.Ym){case !1:case "dialog":a.ea()||b.focus();break;case !0:case "first-tabbable":let c;null==(c=a.O)||Xx(c).then(d=>{d||a.Ta()});break;case "first-heading":a.ta('h1, h2, h3, h4, h5, h6, [role="heading"]');break;default:a.ta(a.Aa.Ym)}}
var UB=class extends Yv{constructor(a,b,c,d,e,f,g,h){super();this.ra=a;this.Ja=b;this.Aa=d;this.lb=e;this.qa=f;this.la=g;this.P=h;this.Ma=of(Wt);this.Gi=this.ja=this.O=null;this.Fi=[];this.Da=of(Tr);this.ia=c;this.Aa.hq&&this.Fi.push(this.Aa.hq)}Qa(a){this.Fi.push(a);en(this.Da)}Lb(a){a=this.Fi.indexOf(a);-1<a&&(this.Fi.splice(a,1),en(this.Da))}G(){this.gb();this.fb();this.aa()}aa(){TB(this)}va(){this.Ob()}ee(a){a=this.Ye.ee(a);this.G();return a}df(a){a=this.Ye.df(a);this.G();return a}Kb(){this.ea()||
TB(this)}Va(a,b){Sx(this.lb,a)||(a.tabIndex=-1,Qk(this.qa,()=>{const c=()=>{a.removeEventListener("blur",c);a.removeEventListener("mousedown",c);a.removeAttribute("tabindex")};a.addEventListener("blur",c);a.addEventListener("mousedown",c)}));a.focus(b)}ta(a,b){(a=this.ra.R.querySelector(a))&&this.Va(a,b)}Ob(){var a=this.Aa.Pg;let b=null;"string"===typeof a?b=this.ia.querySelector(a):"boolean"===typeof a?b=a?this.ja:null:a&&(b=a);if(this.Aa.Pg&&b&&"function"===typeof b.focus){a=St();const c=this.ra.R;
if(!a||a===this.ia.body||a===c||c.contains(a))this.P?(Ox(this.P,b,this.Gi),this.Gi=null):b.focus()}this.O&&this.O.destroy()}Ta(){this.ra.R.focus&&this.ra.R.focus()}ea(){const a=this.ra.R,b=St();return a===b||a.contains(b)}gb(){this.Ma.isBrowser&&(this.O=this.Ja.create(this.ra.R),this.ia&&(this.ja=St()))}fb(){this.la.bd().subscribe(()=>{this.Aa.yd&&this.Kb()})}};UB.K=function(a){return new (a||UB)(y(wi),y($x),y(kt,8),y(RB),y(Ux),y(Tk),y(Fu),y(Px))};
UB.Ca=Jf({type:UB,ha:[["cdk-dialog-container"]],eb:function(a,b){a&1&&vr($v,7);if(a&2){let c;wr(c=xr())&&(b.Ye=c.first)}},Ka:["tabindex","-1",1,"cdk-dialog-container"],Za:6,Ia:function(a,b){a&2&&Mp("id",b.Aa.id||null)("role",b.Aa.role)("aria-modal",b.Aa.ariaModal)("aria-labelledby",b.Aa.ariaLabel?null:b.Fi[0])("aria-label",b.Aa.ariaLabel)("aria-describedby",b.Aa.gq||null)},ka:!0,features:[Ep,Nr],ya:1,za:0,Ba:[["cdkPortalOutlet",""]],sa:function(a){a&1&&H(0,SB,0,0,"ng-template",0)},Ha:[$v],styles:[".cdk-dialog-container{display:block;width:100%;height:100%;min-height:inherit;max-height:inherit}\n"],
pb:2});var VB=new gf("DialogScrollStrategy",{fa:"root",ba:()=>{const a=of(cw);return()=>a.C.block()}}),WB=new gf("DialogData"),XB=new gf("DefaultDialogConfig");var YB=class{constructor(a,b){this.Xb=a;this.config=b;this.closed=new hj;this.yd=b.yd;this.bd=a.bd();this.gf=a.gf();this.Pl=a.Pl();this.id=b.id;this.gf.subscribe(c=>{27!==c.keyCode||this.yd||Cu(c)||(c.preventDefault(),this.close(void 0,{Qq:"keyboard"}))});this.bd.subscribe(()=>{this.yd||this.close(void 0,{Qq:"mouse"})});this.Dm=a.J.subscribe(()=>{!1!==b.pv&&this.close()})}close(a,b){if(this.tg){const c=this.closed;this.tg.Gi=(null==b?void 0:b.Qq)||"program";this.Dm.unsubscribe();this.Xb.dispose();
c.next(a);c.complete();this.kn=this.tg=null}}Wc(){this.Xb.Wc()}te(a="",b=""){this.Xb.te({width:a,height:b})}qg(a){this.Xb.qg(a)}Oj(a){this.Xb.Oj(a)}};let ZB=0;function $B(a,b){a.C.find(c=>c.id===b)}function aC(a,b){a=new Bu({vc:b.vc||Pv(Ov(Rv(a.O.position()))),wc:b.wc||a.Kc(),hb:b.hb,Vb:b.Vb,direction:b.direction,minWidth:b.minWidth,minHeight:b.minHeight,maxWidth:b.maxWidth,maxHeight:b.maxHeight,width:b.width,height:b.height,Jh:b.ov});b.fc&&(a.fc=b.fc);return a}
function bC(a,b,c,d){var e;const f=a.xa||(null==(e=a.yb)?void 0:e.xa);e=[{pa:WB,nb:a.data},{pa:YB,nb:b}];a.Wa&&("function"===typeof a.Wa?e.push(...a.Wa(b,a,c)):e.push(...a.Wa));!a.direction||f&&f.get(pu,null,{optional:!0})||e.push({pa:pu,nb:{value:a.direction,xb:oj()}});return $h({parent:f||d,Wa:e})}function cC(a){const b=a.D;return b?cC(b):a.J}
var eC=class{get C(){return this.D?this.D.C:this.F}get rg(){return this.D?this.D.rg:this.N}constructor(a,b,c,d,e,f){this.O=a;this.Ra=b;this.T=c;this.D=d;this.aa=e;this.F=[];this.J=new hj;this.N=new hj;this.G=new Map;this.Kc=f}open(a,b){b=Object.assign({},this.T||new RB,b);b.id=b.id||`cdk-dialog-${ZB++}`;b.id&&$B(this,b.id);var c=aC(this,b);c=this.O.create(c);const d=new YB(c,b);c=this.ca(c,d,b);d.tg=c;this.ea(a,d,c,b);this.C.length||this.ja();this.C.push(d);d.closed.subscribe(()=>this.P(d,!0));this.rg.next(d);
return d}va(){dC(this.F,a=>{!1===a.config.nv&&this.P(a,!1)});dC(this.F,a=>a.close());this.J.complete();this.N.complete();this.F=[]}ca(a,b,c){var d;const e=c.xa||(null==(d=c.yb)?void 0:d.xa);b=[{pa:RB,nb:c},{pa:YB,nb:b},{pa:Fu,nb:a}];c.ab?"function"===typeof c.ab?d=c.ab:(d=c.ab.type,b.push(...c.ab.Wa(c))):d=UB;c=new Vv(d,c.yb,$h({parent:e||this.Ra,Wa:b}),c.C);return a.attach(c).instance}ea(a,b,c,d){if(a instanceof dr){var e=bC(d,b,c);b={da:d.data,dl:b};d.Po&&(b=Object.assign({},b,"function"===typeof d.Po?
d.Po():d.Po));c.df(new Wv(a,null,b,e))}else e=bC(d,b,c,this.Ra),a=c.ee(new Vv(a,d.yb,e,d.C)),b.vv=a,b.kn=a.instance}P(a,b){a=this.C.indexOf(a);-1<a&&(this.C.splice(a,1),this.C.length||(this.G.forEach((c,d)=>{c?d.setAttribute("aria-hidden",c):d.removeAttribute("aria-hidden")}),this.G.clear(),b&&cC(this).next()))}ja(){const a=Yt(this.aa);if(a.parentElement){const b=a.parentElement.children;for(let c=b.length-1;-1<c;c--){const d=b[c];d===a||"SCRIPT"===d.nodeName||"STYLE"===d.nodeName||d.hasAttribute("aria-live")||
(this.G.set(d,d.getAttribute("aria-hidden")),d.setAttribute("aria-hidden","true"))}}}};eC.K=function(a){return new (a||eC)(r(cw),r(ai),r(XB,8),r(eC,12),r(Zt),r(VB))};eC.na=Pd({ma:eC,ba:eC.K,fa:"root"});function dC(a,b){let c=a.length;for(;c--;)b(a[c])};var fC=class{};fC.K=function(a){return new (a||fC)};fC.Ya=Sf({type:fC});fC.Xa=Qd({Wa:[eC],imports:[kw,aw,ry,aw]});function gC(){}
var iC=class extends UB{constructor(a,b,c,d,e,f,g,h,m){super(a,b,c,d,e,f,g,m);this.Yc=h;this.C=new Jk;this.qk="NoopAnimations"!==this.Yc;this.hp=0;this.D=this.ra.R;let p;this.J=this.qk?null!=(p=hC(this.Aa.XF))?p:150:0;let u;this.N=this.qk?null!=(u=hC(this.Aa.aG))?u:75:0;this.F=null;this.T=!1;this.oa=()=>{this.ca();this.sb(this.J)};this.Mi=()=>{this.ca();this.C.emit({state:"closed",totalTime:this.N})}}G(){super.G();this.cc()}cc(){this.C.emit({state:"opening",totalTime:this.J});this.qk?(this.D.style.setProperty("--mat-dialog-transition-duration",
`${this.J}ms`),this.ua(()=>this.D.classList.add("mdc-dialog--opening","mdc-dialog--open")),this.Ea(this.J,this.oa)):(this.D.classList.add("mdc-dialog--open"),Promise.resolve().then(()=>this.oa()))}Pb(){this.C.emit({state:"closing",totalTime:this.N});this.D.classList.remove("mdc-dialog--open");this.qk?(this.D.style.setProperty("--mat-dialog-transition-duration",`${this.N}ms`),this.ua(()=>this.D.classList.add("mdc-dialog--closing")),this.Ea(this.N,this.Mi)):Promise.resolve().then(()=>this.Mi())}wa(a){this.hp+=
a;en(this.Da)}ca(){this.D.classList.remove("mdc-dialog--opening","mdc-dialog--closing")}Ea(a,b){null!==this.F&&clearTimeout(this.F);this.F=setTimeout(b,a)}ua(a){Qk(this.qa,()=>{"function"===typeof requestAnimationFrame?requestAnimationFrame(a):a()})}aa(){this.Aa.Pv||TB(this)}sb(a){this.T||(this.Aa.Pv&&TB(this),this.C.next({state:"opened",totalTime:a}))}va(){super.va();null!==this.F&&clearTimeout(this.F);this.T=!0}ee(a){a=super.ee(a);a.location.R.classList.add("mat-mdc-dialog-component-host");return a}};
iC.K=function(a){return new (a||iC)(y(wi),y($x),y(kt,8),y(QB),y(Ux),y(Tk),y(Fu),y(qi,8),y(Px))};
iC.Ca=Jf({type:iC,ha:[["mat-dialog-container"]],Ka:["tabindex","-1",1,"mat-mdc-dialog-container","mdc-dialog"],Za:10,Ia:function(a,b){a&2&&(Sq("id",b.Aa.id),Mp("aria-modal",b.Aa.ariaModal)("role",b.Aa.role)("aria-labelledby",b.Aa.ariaLabel?null:b.Fi[0])("aria-label",b.Aa.ariaLabel)("aria-describedby",b.Aa.gq||null),Wp("_mat-animation-noopable",!b.qk)("mat-mdc-dialog-container-with-actions",0<b.hp))},ka:!0,features:[Ep,Nr],ya:3,za:0,Ba:[[1,"mat-mdc-dialog-inner-container","mdc-dialog__container"],
[1,"mat-mdc-dialog-surface","mdc-dialog__surface"],["cdkPortalOutlet",""]],sa:function(a){a&1&&(R(0,"div",0)(1,"div",1),H(2,gC,0,0,"ng-template",2),U()())},Ha:[$v],styles:['.mat-mdc-dialog-container{width:100%;height:100%;display:block;box-sizing:border-box;max-height:inherit;min-height:inherit;min-width:inherit;max-width:inherit;outline:0}.cdk-overlay-pane.mat-mdc-dialog-panel{max-width:var(--mat-dialog-container-max-width, 80vw);min-width:var(--mat-dialog-container-min-width, 0)}@media(max-width: 599px){.cdk-overlay-pane.mat-mdc-dialog-panel{max-width:var(--mat-dialog-container-small-max-width, 80vw)}}.mat-mdc-dialog-inner-container{display:flex;flex-direction:row;align-items:center;justify-content:space-around;box-sizing:border-box;height:100%;opacity:0;transition:opacity linear var(--mat-dialog-transition-duration, 0ms);max-height:inherit;min-height:inherit;min-width:inherit;max-width:inherit}.mdc-dialog--closing .mat-mdc-dialog-inner-container{transition:opacity 75ms linear;transform:none}.mdc-dialog--open .mat-mdc-dialog-inner-container{opacity:1}._mat-animation-noopable .mat-mdc-dialog-inner-container{transition:none}.mat-mdc-dialog-surface{display:flex;flex-direction:column;flex-grow:0;flex-shrink:0;box-sizing:border-box;width:100%;height:100%;position:relative;overflow-y:auto;outline:0;transform:scale(0.8);transition:transform var(--mat-dialog-transition-duration, 0ms) cubic-bezier(0, 0, 0.2, 1);max-height:inherit;min-height:inherit;min-width:inherit;max-width:inherit;box-shadow:var(--mat-dialog-container-elevation-shadow, 0px 11px 15px -7px rgba(0, 0, 0, 0.2), 0px 24px 38px 3px rgba(0, 0, 0, 0.14), 0px 9px 46px 8px rgba(0, 0, 0, 0.12));border-radius:var(--mdc-dialog-container-shape, 4px);background-color:var(--mdc-dialog-container-color, white)}[dir=rtl] .mat-mdc-dialog-surface{text-align:right}.mdc-dialog--open .mat-mdc-dialog-surface,.mdc-dialog--closing .mat-mdc-dialog-surface{transform:none}._mat-animation-noopable .mat-mdc-dialog-surface{transition:none}.mat-mdc-dialog-surface::before{position:absolute;box-sizing:border-box;width:100%;height:100%;top:0;left:0;border:2px solid rgba(0,0,0,0);border-radius:inherit;content:"";pointer-events:none}.mat-mdc-dialog-title{display:block;margin-top:0;position:relative;flex-shrink:0;box-sizing:border-box;margin:0 0 1px;padding:var(--mat-dialog-headline-padding, 0 24px 9px)}.mat-mdc-dialog-title::before{display:inline-block;width:0;height:40px;content:"";vertical-align:0}[dir=rtl] .mat-mdc-dialog-title{text-align:right}.mat-mdc-dialog-container .mat-mdc-dialog-title{color:var(--mdc-dialog-subhead-color, rgba(0, 0, 0, 0.87));font-family:var(--mdc-dialog-subhead-font, Roboto, sans-serif);line-height:var(--mdc-dialog-subhead-line-height, 1.5rem);font-size:var(--mdc-dialog-subhead-size, 1rem);font-weight:var(--mdc-dialog-subhead-weight, 400);letter-spacing:var(--mdc-dialog-subhead-tracking, 0.03125em)}.mat-mdc-dialog-content{display:block;flex-grow:1;box-sizing:border-box;margin:0;overflow:auto;max-height:65vh}.mat-mdc-dialog-content>:first-child{margin-top:0}.mat-mdc-dialog-content>:last-child{margin-bottom:0}.mat-mdc-dialog-container .mat-mdc-dialog-content{color:var(--mdc-dialog-supporting-text-color, rgba(0, 0, 0, 0.6));font-family:var(--mdc-dialog-supporting-text-font, Roboto, sans-serif);line-height:var(--mdc-dialog-supporting-text-line-height, 1.5rem);font-size:var(--mdc-dialog-supporting-text-size, 1rem);font-weight:var(--mdc-dialog-supporting-text-weight, 400);letter-spacing:var(--mdc-dialog-supporting-text-tracking, 0.03125em)}.mat-mdc-dialog-container .mat-mdc-dialog-content{padding:var(--mat-dialog-content-padding, 20px 24px)}.mat-mdc-dialog-container-with-actions .mat-mdc-dialog-content{padding:var(--mat-dialog-with-actions-content-padding, 20px 24px)}.mat-mdc-dialog-container .mat-mdc-dialog-title+.mat-mdc-dialog-content{padding-top:0}.mat-mdc-dialog-actions{display:flex;position:relative;flex-shrink:0;flex-wrap:wrap;align-items:center;justify-content:flex-end;box-sizing:border-box;min-height:52px;margin:0;padding:8px;border-top:1px solid rgba(0,0,0,0);padding:var(--mat-dialog-actions-padding, 8px);justify-content:var(--mat-dialog-actions-alignment, start)}.cdk-high-contrast-active .mat-mdc-dialog-actions{border-top-color:CanvasText}.mat-mdc-dialog-actions.mat-mdc-dialog-actions-align-start,.mat-mdc-dialog-actions[align=start]{justify-content:start}.mat-mdc-dialog-actions.mat-mdc-dialog-actions-align-center,.mat-mdc-dialog-actions[align=center]{justify-content:center}.mat-mdc-dialog-actions.mat-mdc-dialog-actions-align-end,.mat-mdc-dialog-actions[align=end]{justify-content:flex-end}.mat-mdc-dialog-actions .mat-button-base+.mat-button-base,.mat-mdc-dialog-actions .mat-mdc-button-base+.mat-mdc-button-base{margin-left:8px}[dir=rtl] .mat-mdc-dialog-actions .mat-button-base+.mat-button-base,[dir=rtl] .mat-mdc-dialog-actions .mat-mdc-button-base+.mat-mdc-button-base{margin-left:0;margin-right:8px}.mat-mdc-dialog-component-host{display:contents}\n'],
pb:2});function hC(a){return null==a?null:"number"===typeof a?a:a.endsWith("ms")?Us(a.substring(0,a.length-2)):a.endsWith("s")?1E3*Us(a.substring(0,a.length-1)):"0"===a?0:null};var jC=class{constructor(a,b,c){this.af=a;this.ph=c;this.nh=new hj;this.xm=new hj;this.qc=0;this.yd=b.yd;this.id=a.id;a.qg("mat-mdc-dialog-panel");C(c.C,xk(d=>"opened"===d.state),io()).subscribe(()=>{this.nh.next();this.nh.complete()});C(c.C,xk(d=>"closed"===d.state),io()).subscribe(()=>{clearTimeout(this.Tz);this.Mi()});a.Xb.J.subscribe(()=>{this.xm.next(this.xu);this.xm.complete();this.Mi()});wk(this.bd(),C(this.gf(),xk(d=>27===d.keyCode&&!this.yd&&!Cu(d)))).subscribe(d=>{this.yd||(d.preventDefault(),
this.Gi="keydown"===d.type?"keyboard":"mouse",this.close(void 0))})}close(a){this.xu=a;C(this.ph.C,xk(b=>"closing"===b.state),io()).subscribe(b=>{this.xm.next(a);this.xm.complete();Du(this.af.Xb);this.Tz=setTimeout(()=>this.Mi(),b.totalTime+100)});this.qc=1;this.ph.Pb()}rg(){return this.nh}bd(){return this.af.bd}gf(){return this.af.gf}Wc(a){let b=this.af.config.vc;a&&(a.left||a.right)?a.left?b.left(a.left):b.right(a.right):Ov(b);a&&(a.top||a.bottom)?a.top?b.top(a.top):b.bottom(a.bottom):Pv(b);this.af.Wc()}te(a=
"",b=""){this.af.te(a,b)}qg(a){this.af.qg(a)}Oj(a){this.af.Oj(a)}getState(){return this.qc}Mi(){this.qc=2;this.af.close(this.xu,{Qq:this.Gi});this.kn=null}};var kC=new gf("MatMdcDialogData"),lC=new gf("mat-mdc-dialog-default-options"),mC=new gf("mat-mdc-dialog-scroll-strategy",{fa:"root",ba:()=>{const a=of(cw);return()=>a.C.block()}});let nC=0;function oC(a){const b=a.D;return b?oC(b):a.F}
var pC=class{get C(){return this.D?this.D.C:this.J}get rg(){return this.D?this.D.rg:this.G}constructor(a,b,c,d,e,f){this.aa=a;this.P=d;this.Kc=e;this.D=f;this.J=[];this.F=new hj;this.G=new hj;this.ja=QB;this.T=b.get(eC);this.O=jC;this.N=iC;this.ea=kC}open(a,b){let c;b=Object.assign({},this.P||new QB,b);b.id=b.id||`mat-mdc-dialog-${nC++}`;b.wc=b.wc||this.Kc();a=this.T.open(a,Object.assign({},b,{vc:Pv(Ov(Rv(this.aa.position()))),yd:!0,nv:!1,pv:!1,ab:{type:this.N,Wa:()=>[{pa:this.ja,nb:b},{pa:RB,nb:b}]},
Po:()=>({dl:c}),Wa:(d,e,f)=>{c=new this.O(d,b,f);let g;c.Wc(null==(g=b)?void 0:g.position);return[{pa:this.N,nb:f},{pa:this.ea,nb:e.data},{pa:this.O,nb:c}]}}));c.vv=a.vv;c.kn=a.kn;this.C.push(c);this.rg.next(c);c.af.closed.subscribe(()=>{const d=this.C.indexOf(c);-1<d&&(this.C.splice(d,1),this.C.length||oC(this).next())});return c}va(){this.ca(this.J);this.F.complete();this.G.complete()}ca(a){let b=a.length;for(;b--;)a[b].close()}};
pC.K=function(a){return new (a||pC)(r(cw),r(ai),r(Ft,8),r(lC,8),r(mC),r(pC,12),r(Zt),r(qi,8))};pC.na=Pd({ma:pC,ba:pC.K,fa:"root"});let qC=0;var sC=class{constructor(a,b,c){this.dl=a;this.ra=b;this.C=c;this.type="button"}Fb(){this.dl||(this.dl=rC(this.ra,this.C.C))}uc(a){if(a=a._matDialogClose||a._matDialogCloseResult)this.Rv=a.xq}CA(a){var b=this.dl,c=this.Rv;b.Gi=0===a.screenX&&0===a.screenY?"keyboard":"mouse";b.close(c)}};sC.K=function(a){return new (a||sC)(y(jC,8),y(wi),y(pC))};
sC.Fa=Uf({type:sC,ha:[["","mat-dialog-close",""],["","matDialogClose",""]],Za:2,Ia:function(a,b){a&1&&V("click",function(c){return b.CA(c)});a&2&&Mp("aria-label",b.ariaLabel||null)("type",b.type)},inputs:{ariaLabel:[0,"aria-label","ariaLabel"],type:"type",Rv:[0,"mat-dialog-close","dialogResult"],GF:[0,"matDialogClose","_matDialogClose"]},ib:["matDialogClose"],ka:!0,features:[Hg]});
var tC=class{constructor(a,b,c){this.C=a;this.ra=b;this.G=c}Fb(){this.C||(this.C=rC(this.ra,this.G.C));this.C&&Promise.resolve().then(()=>{this.D()})}va(){let a;(null==(a=this.C)?0:a.ph)&&Promise.resolve().then(()=>{this.F()})}};tC.K=function(a){return new (a||tC)(y(jC,8),y(wi),y(pC))};tC.Fa=Uf({type:tC,ka:!0});
var uC=class extends tC{constructor(){super(...arguments);this.id=`mat-mdc-dialog-title-${qC++}`}D(){let a,b;null==(a=this.C.ph)||null==(b=a.Qa)||b.call(a,this.id)}F(){let a,b,c;null==(a=this.C)||null==(b=a.ph)||null==(c=b.Lb)||c.call(b,this.id)}};uC.K=(()=>{let a;return function(b){return(a||(a=Lh(uC)))(b||uC)}})();
uC.Fa=Uf({type:uC,ha:[["","mat-dialog-title",""],["","matDialogTitle",""]],Ka:[1,"mat-mdc-dialog-title","mdc-dialog__title"],Za:1,Ia:function(a,b){a&2&&Sq("id",b.id)},inputs:{id:"id"},ib:["matDialogTitle"],ka:!0,features:[Ep]});var vC=class{};vC.K=function(a){return new (a||vC)};
vC.Fa=Uf({type:vC,ha:[["","mat-dialog-content",""],["mat-dialog-content"],["","matDialogContent",""]],Ka:[1,"mat-mdc-dialog-content","mdc-dialog__content"],ka:!0,features:[function(a){const b=c=>{const d=(Array.isArray(a)?a:a()).map(e=>"function"===typeof e?{zq:ef(e),inputs:Hf,outputs:Hf}:{zq:ef(e.zq),inputs:Kp(e.inputs),outputs:Kp(e.outputs)});null===c.tl?(c.sj=Jp,c.tl=d):c.tl.unshift(...d)};b.ax=!0;return b}([ru])]});
var wC=class extends tC{D(){let a,b;null==(a=this.C.ph)||null==(b=a.wa)||b.call(a,1)}F(){let a,b;null==(a=this.C.ph)||null==(b=a.wa)||b.call(a,-1)}};wC.K=(()=>{let a;return function(b){return(a||(a=Lh(wC)))(b||wC)}})();
wC.Fa=Uf({type:wC,ha:[["","mat-dialog-actions",""],["mat-dialog-actions"],["","matDialogActions",""]],Ka:[1,"mat-mdc-dialog-actions","mdc-dialog__actions"],Za:6,Ia:function(a,b){a&2&&Wp("mat-mdc-dialog-actions-align-start","start"===b.align)("mat-mdc-dialog-actions-align-center","center"===b.align)("mat-mdc-dialog-actions-align-end","end"===b.align)},inputs:{align:"align"},ka:!0,features:[Ep]});
function rC(a,b){let c=a.R.parentElement;for(;c&&!c.classList.contains("mat-mdc-dialog-container");)c=c.parentElement;return c?b.find(d=>d.id===c.id):null};var xC=class{};xC.K=function(a){return new (a||xC)};xC.Ya=Sf({type:xC});xC.Xa=Qd({Wa:[pC],imports:[fC,kw,aw,mz,mz]});var yC=class extends pC{constructor(a,b,c,d,e,f,g,h){h.tc||lw(a,"gmat-mdc-dialog");super(a,b,c,d,e,f,g)}};yC.K=function(a){return new (a||yC)(r(cw),r(ai),r(Ft,8),r(lC,8),r(mC),r(pC,12),r(Zt),r(PB))};yC.na=Pd({ma:yC,ba:yC.K});var zC=class{};zC.K=function(a){return new (a||zC)};zC.Ya=Sf({type:zC});zC.Xa=Qd({Wa:[{pa:pC,Fc:yC},{pa:cw,Fc:mw}],imports:[xC]});new gf("GMAT_FORM_FIELD_OPTIONS",{fa:"root",ba:()=>({tc:!1})});var AC=class{};AC.K=function(a){return new (a||AC)};AC.Ya=Sf({type:AC});AC.Xa=Qd({imports:[cB]});var BC=new gf("GMAT_MENU_OPTIONS",{fa:"root",ba:()=>({tc:!1})});var CC=class{constructor(a,b){this.options=a;a.tc||lw(b,"gmat-mdc-menu")}};CC.K=function(a){return new (a||CC)(y(BC),y(cw))};CC.Fa=Uf({type:CC,ha:[["","mat-menu-trigger-for",""],["","matMenuTriggerFor",""]],features:[Lr([{pa:cw,Fc:mw}])]});var DC=Yy("transformMenu",[az("void",$y({opacity:0,transform:"scale(0.8)"})),bz("void => enter",Zy("120ms cubic-bezier(0, 0, 0.2, 1)",$y({opacity:1,transform:"scale(1)"}))),bz("* => void",Zy("100ms 25ms linear",$y({opacity:0})))]),EC=Yy("fadeInItems",[az("showing",$y({opacity:1})),bz("void => *",[$y({opacity:0}),Zy("400ms 100ms cubic-bezier(0.55, 0, 0.55, 0.2)")])]);var FC=new gf("MatMenuContent");var GC=new gf("MAT_MENU_PANEL");const HC=[[["mat-icon"],["","matMenuItemIcon",""]],"*"];function IC(a){a&1&&(kh(),R(0,"svg",2),Oq(1,"polygon",3),U())}
var JC=class{constructor(a,b,c,d,e){this.ra=a;this.ia=b;this.C=c;this.D=d;this.Da=e;this.role="menuitem";this.Nc=this.disabled=!1;this.F=new hj;this.Te=new hj;this.Zp=this.du=!1;let f;null==d||null==(f=d.jB)||f.call(d,this)}focus(a,b){this.C&&a?Ox(this.C,this.we(),a,b):this.we().focus(b);this.Te.next(this)}pd(){this.C&&Mx(this.C,this.ra,!1)}va(){this.C&&Nx(this.C,this.ra);this.D&&this.D.removeItem&&this.D.removeItem(this);this.F.complete();this.Te.complete()}Ut(){return this.disabled?"-1":"0"}we(){return this.ra.R}Rz(a){this.disabled&&
(a.preventDefault(),a.stopPropagation())}oA(){this.F.next(this)}jw(){const a=this.ra.R.cloneNode(!0),b=a.querySelectorAll("mat-icon, .material-icons");for(let d=0;d<b.length;d++)b[d].remove();let c;return(null==(c=a.textContent)?void 0:c.trim())||""}G(a){this.du=a;let b;null==(b=this.Da)||en(b)}J(a){this.Zp=a;let b;null==(b=this.Da)||en(b)}rA(){return this.ia&&this.ia.activeElement===this.we()}};JC.K=function(a){return new (a||JC)(y(wi),y(kt),y(Px),y(GC,8),y(Tr))};
JC.Ca=Jf({type:JC,ha:[["","mat-menu-item",""]],Ka:[1,"mat-mdc-menu-item","mat-mdc-focus-indicator"],Za:8,Ia:function(a,b){a&1&&V("click",function(c){return b.Rz(c)})("mouseenter",function(){return b.oA()});a&2&&(Mp("role",b.role)("tabindex",b.Ut())("aria-disabled",b.disabled)("disabled",b.disabled||null),Wp("mat-mdc-menu-item-highlighted",b.du)("mat-mdc-menu-item-submenu-trigger",b.Zp))},inputs:{role:"role",disabled:[2,"disabled","disabled",As],Nc:[2,"disableRipple","disableRipple",As]},ib:["matMenuItem"],
ka:!0,features:[Lp,Nr],attrs:["mat-menu-item",""],bc:["mat-icon, [matMenuItemIcon]","*"],ya:5,za:3,Ba:[[1,"mat-mdc-menu-item-text"],["matRipple","",1,"mat-mdc-menu-ripple",3,"matRippleDisabled","matRippleTrigger"],["viewBox","0 0 5 10","focusable","false","aria-hidden","true",1,"mat-mdc-menu-submenu-icon"],["points","0,0 5,5 0,10"]],sa:function(a,b){a&1&&(Zq(HC),$q(0),R(1,"span",0),$q(2,1),U(),Oq(3,"div",1),H(4,IC,2,0,":svg:svg",2));a&2&&(D(3),Q("matRippleDisabled",b.Nc||b.disabled)("matRippleTrigger",
b.we()),D(),Gq(b.Zp?4:-1))},Ha:[Lz],pb:2,Pa:0});function KC(a){if(a&1){const b=Zg();R(0,"div",0);V("keydown",function(c){t(b);const d=Y();return x(d.Ve(c))})("click",function(){t(b);const c=Y();return x(c.closed.emit("click"))})("@transformMenu.start",function(c){t(b);const d=Y();return x(d.AA(c))})("@transformMenu.done",function(c){t(b);const d=Y();return x(d.zA(c))});R(1,"div",1);$q(2);U()()}a&2&&(a=Y(),sq(vq,a.Se),Q("id",a.nx)("@transformMenu",a.Nm),Mp("aria-label",a.ariaLabel||null)("aria-labelledby",a.ad||null)("aria-describedby",a.Mk||null))}
let LC=0;var NC=new gf("mat-menu-default-options",{fa:"root",ba:MC});function MC(){return{uo:!1,ui:"after",ek:"below",fc:"cdk-overlay-transparent-backdrop"}}function OC(a){return C(a.C.C,Ko(a.C),Lo(b=>wk(...b.map(c=>c.F))))}function PC(a,b="program"){let c;null==(c=a.G)||c.destroy();a.G=el(()=>{let d=null;a.C.length&&(d=a.C.first.we().closest('[role="menu"]'));if(!d||!d.contains(document.activeElement)){const e=a.D;e.ta=b;e.J(0,1);!e.C&&d&&d.focus()}},{xa:a.Ra})}
var QC=class{get ui(){return this.Gf}set ui(a){this.Gf=a;this.Vj()}get ek(){return this.N}set ek(a){this.N=a;this.Vj()}set hb(a){const b=this.O,c=Object.assign({},this.Se);b&&b.length&&b.split(" ").forEach(d=>{c[d]=!1});(this.O=a)&&a.length&&(a.split(" ").forEach(d=>{c[d]=!0}),this.ra.R.className="");this.Se=c}get classList(){return this.hb}set classList(a){this.hb=a}constructor(a,b,c,d){this.ra=a;this.Da=d;this.ea="mat-elevation-z";this.ca=8;this.C=new cr;this.Se={};this.Nm="void";this.jg=new hj;
this.close=this.closed=new Jk;this.nx=`mat-menu-panel-${LC++}`;this.Ra=of(ai);this.Zh=c.Zh||"";this.Gf=c.ui;this.N=c.ek;this.fc=c.fc;this.uo=c.uo;this.Vb=c.Vb}Fb(){this.Vj()}Bj(){this.aa();this.D=Py(Oy(Ky(new Xy(this.C))));this.D.ca.subscribe(()=>this.closed.emit("tab"));C(this.C.C,Ko(this.C),Lo(a=>wk(...a.map(b=>b.Te)))).subscribe(a=>Ry(this.D,a));this.C.C.subscribe(a=>{const b=this.D;var c;"enter"===this.Nm&&(null==(c=b.C)?0:c.rA())&&(a=a.toArray(),c=Math.max(0,Math.min(a.length-1,b.O||0)),a[c]&&
!a[c].disabled?b.D(c):Sy(b))})}va(){let a;null==(a=this.D)||a.destroy();this.C.destroy();this.closed.complete();let b;null==(b=this.G)||b.destroy()}jB(){}removeItem(){}Ve(a){const b=a.keyCode,c=this.D;switch(b){case 27:Cu(a)||(a.preventDefault(),this.closed.emit("keydown"));break;case 37:this.vo&&"ltr"===this.direction&&this.closed.emit("keydown");break;case 39:this.vo&&"rtl"===this.direction&&this.closed.emit("keydown");break;default:if(38===b||40===b)c.ta="keyboard";Uy(c,a);return}a.stopPropagation()}Ux(a){a=
`${"mat-elevation-z"}${Math.min(8+a,24)}`;var b=Object.keys(this.Se).find(c=>c.startsWith("mat-elevation-z"));b&&b!==this.F||(b=Object.assign({},this.Se),this.F&&(b[this.F]=!1),b[a]=!0,this.F=a,this.Se=b)}Vj(a=this.ui,b=this.ek){this.Se=Object.assign({},this.Se,{["mat-menu-before"]:"before"===a,["mat-menu-after"]:"after"===a,["mat-menu-above"]:"above"===b,["mat-menu-below"]:"below"===b});let c;null==(c=this.Da)||en(c)}T(){this.Nm="enter"}P(){this.Nm="void"}zA(a){this.jg.next(a);this.J=!1}AA(a){this.J=
!0;"enter"===a.se&&0===this.D.O&&(a.element.scrollTop=0)}aa(){C(this.yt.C,Ko(this.yt)).subscribe(a=>{this.C.reset(a.filter(b=>b.D===this));br(this.C)})}};QC.K=function(a){return new (a||QC)(y(wi),y(Tk),y(NC),y(Tr))};
QC.Ca=Jf({type:QC,ha:[["mat-menu"]],hc:function(a,b,c){a&1&&(ur(c,FC,5),ur(c,JC,5),ur(c,JC,4));if(a&2){let d;wr(d=xr())&&(b.xj=d.first);wr(d=xr())&&(b.yt=d);wr(d=xr())&&(b.items=d)}},eb:function(a,b){a&1&&vr(dr,5);if(a&2){let c;wr(c=xr())&&(b.eg=c.first)}},Za:3,Ia:function(a){a&2&&Mp("aria-label",null)("aria-labelledby",null)("aria-describedby",null)},inputs:{fc:"backdropClass",ariaLabel:[0,"aria-label","ariaLabel"],ad:[0,"aria-labelledby","ariaLabelledby"],Mk:[0,"aria-describedby","ariaDescribedby"],
ui:"xPosition",ek:"yPosition",uo:[2,"overlapTrigger","overlapTrigger",As],Vb:[2,"hasBackdrop","hasBackdrop",a=>null==a?null:As(a)],hb:[0,"class","panelClass"],classList:"classList"},outputs:{closed:"closed",close:"close"},ib:["matMenu"],ka:!0,features:[Lr([{pa:GC,Ab:QC}]),Lp,Nr],bc:["*"],ya:1,za:0,Ba:[["tabindex","-1","role","menu",1,"mat-mdc-menu-panel","mat-mdc-elevation-specific",3,"keydown","click","id"],[1,"mat-mdc-menu-content"]],sa:function(a){a&1&&(Zq(),H(0,KC,3,7,"ng-template"))},styles:['mat-menu{display:none}.mat-mdc-menu-content{margin:0;padding:8px 0;list-style-type:none}.mat-mdc-menu-content:focus{outline:none}.mat-mdc-menu-content,.mat-mdc-menu-content .mat-mdc-menu-item .mat-mdc-menu-item-text{-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;flex:1;white-space:normal;font-family:var(--mat-menu-item-label-text-font);line-height:var(--mat-menu-item-label-text-line-height);font-size:var(--mat-menu-item-label-text-size);letter-spacing:var(--mat-menu-item-label-text-tracking);font-weight:var(--mat-menu-item-label-text-weight)}.mat-mdc-menu-panel{min-width:112px;max-width:280px;overflow:auto;-webkit-overflow-scrolling:touch;box-sizing:border-box;outline:0;border-radius:var(--mat-menu-container-shape);background-color:var(--mat-menu-container-color);will-change:transform,opacity}.mat-mdc-menu-panel.ng-animating{pointer-events:none}.cdk-high-contrast-active .mat-mdc-menu-panel{outline:solid 1px}.mat-mdc-menu-panel .mat-divider{color:var(--mat-menu-divider-color);margin-bottom:var(--mat-menu-divider-bottom-spacing);margin-top:var(--mat-menu-divider-top-spacing)}.mat-mdc-menu-item{display:flex;position:relative;align-items:center;justify-content:flex-start;overflow:hidden;padding:0;padding-left:var(--mat-menu-item-leading-spacing);padding-right:var(--mat-menu-item-trailing-spacing);-webkit-user-select:none;user-select:none;cursor:pointer;outline:none;border:none;-webkit-tap-highlight-color:rgba(0,0,0,0);cursor:pointer;width:100%;text-align:left;box-sizing:border-box;color:inherit;font-size:inherit;background:none;text-decoration:none;margin:0;align-items:center;min-height:48px}.mat-mdc-menu-item:focus{outline:none}[dir=rtl] .mat-mdc-menu-item,.mat-mdc-menu-item[dir=rtl]{padding-left:var(--mat-menu-item-trailing-spacing);padding-right:var(--mat-menu-item-leading-spacing)}.mat-mdc-menu-item:has(.material-icons,mat-icon,[matButtonIcon]){padding-left:var(--mat-menu-item-with-icon-leading-spacing);padding-right:var(--mat-menu-item-with-icon-trailing-spacing)}[dir=rtl] .mat-mdc-menu-item:has(.material-icons,mat-icon,[matButtonIcon]),.mat-mdc-menu-item:has(.material-icons,mat-icon,[matButtonIcon])[dir=rtl]{padding-left:var(--mat-menu-item-with-icon-trailing-spacing);padding-right:var(--mat-menu-item-with-icon-leading-spacing)}.mat-mdc-menu-item::-moz-focus-inner{border:0}.mat-mdc-menu-item,.mat-mdc-menu-item:visited,.mat-mdc-menu-item:link{color:var(--mat-menu-item-label-text-color)}.mat-mdc-menu-item .mat-icon-no-color,.mat-mdc-menu-item .mat-mdc-menu-submenu-icon{color:var(--mat-menu-item-icon-color)}.mat-mdc-menu-item[disabled]{cursor:default;opacity:.38}.mat-mdc-menu-item[disabled]::after{display:block;position:absolute;content:"";top:0;left:0;bottom:0;right:0}.mat-mdc-menu-item .mat-icon{flex-shrink:0;margin-right:var(--mat-menu-item-spacing);height:var(--mat-menu-item-icon-size);width:var(--mat-menu-item-icon-size)}[dir=rtl] .mat-mdc-menu-item{text-align:right}[dir=rtl] .mat-mdc-menu-item .mat-icon{margin-right:0;margin-left:var(--mat-menu-item-spacing)}.mat-mdc-menu-item:not([disabled]):hover{background-color:var(--mat-menu-item-hover-state-layer-color)}.mat-mdc-menu-item:not([disabled]).cdk-program-focused,.mat-mdc-menu-item:not([disabled]).cdk-keyboard-focused,.mat-mdc-menu-item:not([disabled]).mat-mdc-menu-item-highlighted{background-color:var(--mat-menu-item-focus-state-layer-color)}.cdk-high-contrast-active .mat-mdc-menu-item{margin-top:1px}.mat-mdc-menu-submenu-icon{width:var(--mat-menu-item-icon-size);height:10px;fill:currentColor;padding-left:var(--mat-menu-item-spacing)}[dir=rtl] .mat-mdc-menu-submenu-icon{padding-right:var(--mat-menu-item-spacing);padding-left:0}[dir=rtl] .mat-mdc-menu-submenu-icon polygon{transform:scaleX(-1)}.cdk-high-contrast-active .mat-mdc-menu-submenu-icon{fill:CanvasText}.mat-mdc-menu-item .mat-mdc-menu-ripple{top:0;left:0;right:0;bottom:0;position:absolute;pointer-events:none}\n'],
pb:2,data:{animation:[DC,EC]},Pa:0});var RC=new gf("mat-menu-scroll-strategy",{fa:"root",ba:()=>{const a=of(cw);return()=>a.C.C()}}),SC={pa:RC,Bc:[cw],Gc:function(a){return()=>a.C.C()}};const TC=Lt({passive:!0});function UC(a){return!!(a.G&&a.C&&a.Jd)}function VC(a){let b;null==(b=a.Jd)||b.close.emit()}
function WC(a){const b=a.Jd;if(!a.F&&b){var c=XC(a,b),d=c.Aa,e=d.vc;a.gb(b,e);d.Vb=null==b.Vb?!UC(a):b.Vb;c.attach(a.Ja(b));b.xj&&b.xj.attach(a.bD);a.O=a.Va().subscribe(()=>VC(a));a.Ta(b);b instanceof QC&&(b.T(),C(b.C.C,Oo(b.close)).subscribe(()=>{Ju(Qu(e,!1));Qu(e,!0)}))}}
function XC(a,b){if(!a.la){var c=new Bu({vc:Su(Ou(Qu(Sv(a.ja.position(),a.J))),".mat-menu-panel, .mat-mdc-menu-panel"),fc:b.fc||"cdk-overlay-transparent-backdrop",hb:b.Zh,wc:a.Kc(),direction:a.P});a.lb(b,c.vc);a.la=a.ja.create(c);a.la.gf().subscribe()}return a.la}
var YC=class{get Jt(){return this.Jd}set Jt(a){this.Jd=a}get Jd(){return this.ua}set Jd(a){if(a!==this.ua){this.ua=a;this.aa.unsubscribe();a&&(this.aa=a.close.subscribe(c=>{this.Ea(c);"click"!==c&&"tab"!==c||!this.C||this.C.closed.emit(c)}));var b;null==(b=this.G)||b.J(UC(this))}}constructor(a,b,c,d,e,f,g,h,m){this.ja=a;this.J=b;this.wa=c;this.G=f;this.P=g;this.ea=h;this.qa=m;this.la=null;this.F=!1;this.aa=this.ta=this.O=Si.EMPTY;this.Da=of(Tr);this.oa=p=>{Fx(p)||(this.D="touch")};this.D=void 0;this.Pg=
!0;this.vD=this.Br=new Jk;this.uD=this.zr=new Jk;this.Kc=d;this.C=e instanceof QC?e:void 0;b.R.addEventListener("touchstart",this.oa,TC)}Bj(){this.Qa()}va(){this.la&&(this.la.dispose(),this.la=null);this.J.R.removeEventListener("touchstart",this.oa,TC);this.aa.unsubscribe();this.O.unsubscribe();this.ta.unsubscribe()}get Ar(){return this.F}get dir(){return this.P&&"rtl"===this.P.value?"rtl":"ltr"}focus(a,b){this.ea&&a?Ox(this.ea,this.J,a,b):this.J.R.focus(b)}Wc(){let a;null==(a=this.la)||a.Wc()}Ea(a){if(this.la&&
this.Ar){var b=this.Jd;this.O.unsubscribe();this.la.detach();!this.Pg||"keydown"!==a&&this.D&&UC(this)||this.focus(this.D);this.D=void 0;if(b instanceof QC)b.P(),b.xj?C(b.jg,xk(c=>"void"===c.se),io(),Oo(b.xj.C)).subscribe({next:()=>b.xj.detach(),complete:()=>this.N(!1)}):this.N(!1);else{this.N(!1);let c;null==b||null==(c=b.xj)||c.detach()}}}Ta(a){a.vo=UC(this)?this.C:void 0;a.direction=this.dir;this.fb(a);PC(a,this.D||"program");this.N(!0)}fb(a){if(a.Ux){let b=0,c=a.vo;for(;c;)b++,c=c.vo;a.Ux(b)}}N(a){a!==
this.F&&((this.F=a)?this.Br.emit():this.zr.emit(),UC(this)&&this.G.G(a),en(this.Da))}lb(a,b){a.Vj&&b.gb.subscribe(c=>{const d="start"===c.C.Na?"after":"before",e="top"===c.C.Ua?"below":"above";this.qa?this.qa.run(()=>a.Vj(d,e)):a.Vj(d,e)})}gb(a,b){let [c,d]="before"===a.ui?["end","start"]:["start","end"],e=c,f=d,[g,h]="above"===a.ek?["bottom","top"]:["top","bottom"],[m,p]=[g,h],u=m,w=p,[B,G]=[e,f],L=B,O=G,ba=0;UC(this)?(O=e="before"===a.ui?"start":"end",f=L="end"===e?"start":"end",this.C&&(null==
this.ca&&(this.ca=(a=this.C.items.first)?a.we().offsetTop:0),ba="bottom"===g?this.ca:-this.ca)):a.uo||(u="top"===g?"bottom":"top",w="top"===h?"bottom":"top");Lu(b,[{bb:e,cb:u,Na:L,Ua:g,offsetY:ba},{bb:f,cb:u,Na:O,Ua:g,offsetY:ba},{bb:e,cb:w,Na:L,Ua:h,offsetY:-ba},{bb:f,cb:w,Na:O,Ua:h,offsetY:-ba}])}Va(){const a=this.la.bd(),b=this.la.J,c=this.C?this.C.closed:oj(),d=this.C?C(OC(this.C),xk(e=>e!==this.G),xk(()=>this.F)):oj();return wk(a,c,d,b)}qA(a){Ex(a)||(this.D=0===a.button?"mouse":void 0,UC(this)&&
a.preventDefault())}Ve(a){a=a.keyCode;if(13===a||32===a)this.D="keyboard";UC(this)&&(39===a&&"ltr"===this.dir||37===a&&"rtl"===this.dir)&&(this.D="keyboard",WC(this))}tk(a){UC(this)?(a.stopPropagation(),WC(this)):this.F?VC(this):WC(this)}Qa(){UC(this)&&this.C&&(this.ta=C(OC(this.C),xk(a=>a===this.G&&!a.disabled),$n(Hk)).subscribe(()=>{this.D="mouse";this.Jd instanceof QC&&this.Jd.J?C(this.Jd.jg,io(),$n(Hk),Oo(OC(this.C))).subscribe(()=>WC(this)):WC(this)}))}Ja(a){this.T&&this.T.eg===a.eg||(this.T=
new Wv(a.eg,this.wa));return this.T}};YC.K=function(a){return new (a||YC)(y(cw),y(wi),y(np),y(RC),y(GC,8),y(JC,10),y(pu,8),y(Px),y(Tk))};
YC.Fa=Uf({type:YC,ha:[["","mat-menu-trigger-for",""],["","matMenuTriggerFor",""]],Ka:[1,"mat-mdc-menu-trigger"],Za:3,Ia:function(a,b){a&1&&V("click",function(c){return b.tk(c)})("mousedown",function(c){return b.qA(c)})("keydown",function(c){return b.Ve(c)});a&2&&Mp("aria-haspopup",b.Jd?"menu":null)("aria-expanded",b.Ar)("aria-controls",b.Ar?b.Jd.nx:null)},inputs:{Jt:[0,"mat-menu-trigger-for","_deprecatedMatMenuTriggerFor"],Jd:[0,"matMenuTriggerFor","menu"],bD:[0,"matMenuTriggerData","menuData"],Pg:[0,
"matMenuTriggerRestoreFocus","restoreFocus"]},outputs:{Br:"menuOpened",vD:"onMenuOpen",zr:"menuClosed",uD:"onMenuClose"},ib:["matMenuTrigger"],ka:!0});var ZC=class{};ZC.K=function(a){return new (a||ZC)};ZC.Ya=Sf({type:ZC});ZC.Xa=Qd({Wa:[SC],imports:[it,Yz,mz,kw,yu,mz]});var $C=class{};$C.K=function(a){return new (a||$C)};$C.Ya=Sf({type:$C});$C.Xa=Qd({Wa:[{pa:cw,Fc:mw}],imports:[ZC]});var aD={AED:[2,"dh","\u062f.\u0625."],ALL:[0,"Lek","Lek"],AUD:[2,"$","AU$"],BDT:[2,"\u09f3","Tk"],BGN:[2,"lev","lev"],BRL:[2,"R$","R$"],CAD:[2,"$","C$"],CDF:[2,"FrCD","CDF"],CHF:[2,"CHF","CHF"],CLP:[0,"$","CL$"],CNY:[2,"\u00a5","RMB\u00a5"],COP:[32,"$","COL$"],CRC:[0,"\u20a1","CR\u20a1"],CZK:[50,"K\u010d","K\u010d"],DKK:[50,"kr.","kr."],DOP:[2,"RD$","RD$"],EGP:[2,"\u00a3","LE"],ETB:[2,"Birr","Birr"],EUR:[2,"\u20ac","\u20ac"],GBP:[2,"\u00a3","GB\u00a3"],HKD:[2,"$","HK$"],HRK:[2,"kn","kn"],HUF:[34,
"Ft","Ft"],IDR:[0,"Rp","Rp"],ILS:[34,"\u20aa","IL\u20aa"],INR:[2,"\u20b9","Rs"],IRR:[0,"Rial","IRR"],ISK:[0,"kr","kr"],JMD:[2,"$","JA$"],JPY:[0,"\u00a5","JP\u00a5"],KRW:[0,"\u20a9","KR\u20a9"],LKR:[2,"Rs","SLRs"],LTL:[2,"Lt","Lt"],MNT:[0,"\u20ae","MN\u20ae"],MVR:[2,"Rf","MVR"],MXN:[2,"$","Mex$"],MYR:[2,"RM","RM"],NOK:[50,"kr","NOkr"],PAB:[2,"B/.","B/."],PEN:[2,"S/.","S/."],PHP:[2,"\u20b1","PHP"],PKR:[0,"Rs","PKRs."],PLN:[50,"z\u0142","z\u0142"],RON:[2,"RON","RON"],RSD:[0,"din","RSD"],RUB:[50,"\u20bd",
"RUB"],SAR:[2,"SAR","SAR"],SEK:[50,"kr","kr"],SGD:[2,"$","S$"],THB:[2,"\u0e3f","THB"],TRY:[2,"\u20ba","TRY"],TWD:[2,"$","NT$"],TZS:[0,"TSh","TSh"],UAH:[2,"\u0433\u0440\u043d.","UAH"],USD:[2,"$","US$"],UYU:[2,"$","$U"],VND:[48,"\u20ab","VN\u20ab"],YER:[0,"Rial","Rial"],ZAR:[2,"R","ZAR"]};var bD={jt:".",Xo:",",qt:"%",gp:"0",zz:"+",pt:"-",mt:"E",rt:"\u2030",Zo:"\u221e",uz:"NaN",mF:"#,##0.###",vF:"#E0",yz:"#,##0%",lF:"\u00a4#,##0.00",kt:"USD"},cD=bD;cD=bD;function dD(){this.aa=40;this.D=1;this.F=3;this.ea=this.G=0;this.ta=!1;this.ca=this.P="";this.J=cD.pt;this.T="";this.C=1;this.O=!1;this.N=[];this.ja=this.oa=!1;var a=cD.yz;const b=[0];this.P=eD(this,a,b);var c=b[0];let d=-1,e=0;var f=0,g=0;let h=-1;const m=a.length;for(let p=!0;b[0]<m&&p;b[0]++)switch(a.charAt(b[0])){case "#":0<f?g++:e++;0<=h&&0>d&&h++;break;case "0":if(0<g)throw Error('Unexpected "0" in pattern "'+a+'"');f++;0<=h&&0>d&&h++;break;case ",":0<h&&this.N.push(h);h=0;break;case ".":if(0<=
d)throw Error('Multiple decimal separators in pattern "'+a+'"');d=e+f+g;break;case "E":if(this.ja)throw Error('Multiple exponential symbols in pattern "'+a+'"');this.ja=!0;this.ea=0;b[0]+1<m&&"+"==a.charAt(b[0]+1)&&(b[0]++,this.ta=!0);for(;b[0]+1<m&&"0"==a.charAt(b[0]+1);)b[0]++,this.ea++;if(1>e+f||1>this.ea)throw Error('Malformed exponential pattern "'+a+'"');p=!1;break;default:b[0]--,p=!1}0==f&&0<e&&0<=d&&(f=d,0==f&&f++,g=e-f,e=f-1,f=1);if(0>d&&0<g||0<=d&&(d<e||d>e+f)||0==h)throw Error('Malformed pattern "'+
a+'"');g=e+f+g;this.F=0<=d?g-d:0;0<=d&&(this.G=e+f-d,0>this.G&&(this.G=0));this.D=(0<=d?d:g)-e;this.ja&&(this.aa=e+this.D,0==this.F&&0==this.D&&(this.D=1));this.N.push(Math.max(0,h));this.oa=0==d||d==g;c=b[0]-c;this.ca=eD(this,a,b);b[0]<a.length&&";"==a.charAt(b[0])?(b[0]++,1!=this.C&&(this.O=!0),this.J=eD(this,a,b),b[0]+=c,this.T=eD(this,a,b)):(this.J+=this.P,this.T+=this.ca)}
dD.prototype.parse=function(a,b){b=b||[0];a=a.replace(/ |\u202f/g,"\u00a0");let c=a.indexOf(this.P,b[0])==b[0],d=a.indexOf(this.J,b[0])==b[0];c&&d&&(this.P.length>this.J.length?d=!1:this.P.length<this.J.length&&(c=!1));c?b[0]+=this.P.length:d&&(b[0]+=this.J.length);if(a.indexOf(cD.Zo,b[0])==b[0]){b[0]+=cD.Zo.length;var e=Infinity}else{e=a;let f=!1,g=!1,h=!1,m=-1,p=1;const u=cD.jt;let w=cD.Xo;const B=cD.mt;w=w.replace(/\u202f/g,"\u00a0");let G="";for(;b[0]<e.length;b[0]++){const L=e.charAt(b[0]),O=
fD(L);if(0<=O&&9>=O)G+=O,h=!0;else if(L==u.charAt(0)){if(f||g)break;G+=".";f=!0}else if(L==w.charAt(0)&&("\u00a0"!=w.charAt(0)||b[0]+1<e.length&&0<=fD(e.charAt(b[0]+1)))){if(f||g)break}else if(L==B.charAt(0)){if(g)break;G+="E";g=!0;m=b[0]}else if("+"==L||"-"==L){if(h&&m!=b[0]-1)break;G+=L}else if(1==this.C&&L==cD.qt.charAt(0)){if(1!=p)break;p=100;if(h){b[0]++;break}}else if(1==this.C&&L==cD.rt.charAt(0)){if(1!=p)break;p=1E3;if(h){b[0]++;break}}else break}1!=this.C&&(p=this.C);e=parseFloat(G)/p}if(c){if(a.indexOf(this.ca,
b[0])!=b[0])return NaN;b[0]+=this.ca.length}else if(d){if(a.indexOf(this.T,b[0])!=b[0])return NaN;b[0]+=this.T.length}return d?-e:e};
dD.prototype.format=function(a){if(this.G>this.F)throw Error("Min value must be less than max value");if(isNaN(a))return cD.uz;const b=[];var c=gD;a=hD(a,-c.SB);const d=0>a||0==a&&0>1/a;d?c.Ww?b.push(c.Ww):(b.push(c.prefix),b.push(this.J)):(b.push(c.prefix),b.push(this.P));if(isFinite(a))if(a*=d?-1:1,a*=this.C,this.ja){var e=a;if(0==e)iD(this,e,this.D,b),jD(this,0,b);else{var f=Math.floor(Math.log(e)/Math.log(10)+2E-15);e=hD(e,-f);var g=this.D;1<this.aa&&this.aa>this.D?(g=f%this.aa,0>g&&(g=this.aa+
g),e=hD(e,g),f-=g,g=1):1>this.D?(f++,e=hD(e,-1)):(f-=this.D-1,e=hD(e,this.D-1));iD(this,e,g,b);jD(this,f,b)}}else iD(this,a,this.D,b);else b.push(cD.Zo);d?c.Xw?b.push(c.Xw):(isFinite(a)&&b.push(c.wy),b.push(this.T)):(isFinite(a)&&b.push(c.wy),b.push(this.ca));return b.join("")};
function iD(a,b,c,d){if(a.G>a.F)throw Error("Min value must be less than max value");d||(d=[]);var e=hD(b,a.F);e=Math.round(e);isFinite(e)?(b=Math.floor(hD(e,-a.F)),e=Math.floor(e-hD(b,a.F))):e=0;var f=b,g=e,h=0<a.G||0<g||!1;e=a.G;h&&(e=a.G);var m="";for(b=f;1E20<b;)m="0"+m,b=Math.round(hD(b,-1));m=b+m;var p=cD.jt;b=cD.gp.codePointAt(0);var u=m.length,w=0;if(0<f||0<c){for(f=u;f<c;f++)d.push(String.fromCodePoint(b));if(2<=a.N.length)for(c=1;c<a.N.length;c++)w+=a.N[c];c=u-w;if(0<c){f=a.N;w=u=0;var B=
cD.Xo,G=m.length;for(let O=0;O<G;O++)if(d.push(String.fromCodePoint(b+Number(m.charAt(O)))),1<G-O){var L=f[w];if(O<c){let ba=c-O;(1===L||0<L&&1===ba%L)&&d.push(B)}else w<f.length&&(O===c?w+=1:L===O-c-u+1&&(d.push(B),u+=L,w+=1))}}else{c=m;m=a.N;f=cD.Xo;L=c.length;B=[];for(u=m.length-1;0<=u&&0<L;u--){w=m[u];for(G=0;G<w&&0<=L-G-1;G++)B.push(String.fromCodePoint(b+Number(c.charAt(L-G-1))));L-=w;0<L&&B.push(f)}d.push.apply(d,B.reverse())}}else h||d.push(String.fromCodePoint(b));(a.oa||h)&&d.push(p);h=
String(g);g=h.split("e+");if(2==g.length){if(h=parseFloat(g[0])){p=h;if(isFinite(p)){for(c=0;1<=(p/=10);)c++;p=c}else p=0<p?p:0;p=-p-1;h=-1>p?h&&isFinite(h)?hD(Math.round(hD(h,-1)),1):h:h&&isFinite(h)?hD(Math.round(hD(h,p)),-p):h}h=String(h);h=h.replace(".","");h+=yd("0",parseInt(g[1],10)-h.length+1)}a.F+1>h.length&&(h="1"+yd("0",a.F-h.length)+h);for(a=h.length;"0"==h.charAt(a-1)&&a>e+1;)a--;for(e=1;e<a;e++)d.push(String.fromCodePoint(b+Number(h.charAt(e))))}
function jD(a,b,c){c.push(cD.mt);0>b?(b=-b,c.push(cD.pt)):a.ta&&c.push(cD.zz);b=""+b;var d=cD.gp;for(let e=b.length;e<a.ea;e++)c.push(d);a=d.codePointAt(0)-kD;for(d=0;d<b.length;d++)c.push(String.fromCodePoint(a+b.codePointAt(d)))}function fD(a){a=a.codePointAt(0);if(48<=a&&58>a)return a-48;const b=cD.gp.codePointAt(0);return b<=a&&a<b+10?a-b:-1}var kD="0".codePointAt(0);
function eD(a,b,c){let d="",e=!1;const f=b.length;for(;c[0]<f;c[0]++){var g=b.charAt(c[0]);if("'"==g)c[0]+1<f&&"'"==b.charAt(c[0]+1)?(c[0]++,d+="'"):e=!e;else if(e)d+=g;else switch(g){case "#":case "0":case ",":case ".":case ";":return d;case "\u00a4":c[0]+1<f&&"\u00a4"==b.charAt(c[0]+1)?(c[0]++,d+=cD.kt):(g=cD.kt,d+=g in aD?aD[g][1]:g);break;case "%":if(!a.O&&1!=a.C)throw Error("Too many percent/permill");if(a.O&&100!=a.C)throw Error("Inconsistent use of percent/permill characters");a.C=100;a.O=
!1;d+=cD.qt;break;case "\u2030":if(!a.O&&1!=a.C)throw Error("Too many percent/permill");if(a.O&&1E3!=a.C)throw Error("Inconsistent use of percent/permill characters");a.C=1E3;a.O=!1;d+=cD.rt;break;default:d+=g}}return d}var gD={SB:0,Ww:"",Xw:"",prefix:"",wy:""};function hD(a,b){if(!a||!isFinite(a)||0==b)return a;a=String(a).split("e");return parseFloat(a[0]+"e"+(parseInt(a[1]||0,10)+b))};var lD=new dD;function mD(a){const b={attributeFilter:["aria-label","aria-valuenow"]};return new bj(c=>{(new MutationObserver(d=>{const e=d[0].target;var f;d=null!=(f=e.getAttribute("aria-label"))?f:"";let g;f=null!=(g=e.getAttribute("aria-valuenow"))?g:"indeterminate";c.next([d,f])})).observe(a,b)})};var nD=new gf("GMAT_PROGRESS_SPINNER_OPTIONS",{fa:"root",ba:()=>({tc:!1})});const oD=["determinateSpinner"];function pD(a){a&1&&(kh(),R(0,"svg",11),Oq(1,"circle",12),U());a&2&&(a=Y(),Mp("viewBox",a.Nu()),D(),Up("stroke-dasharray",a.Sm(),"px")("stroke-dashoffset",a.Sm()/2,"px")("stroke-width",a.Et(),"%"),Mp("r",a.zm()))}var rD=new gf("mat-progress-spinner-default-options",{fa:"root",ba:qD});function qD(){return{diameter:100}}
var sD=class{get color(){return this.oh||this.C}set color(a){this.oh=a}constructor(a,b,c){this.ra=a;this.C="primary";this.Lc=0;this.D=100;this.Lp="NoopAnimations"===b&&!!c&&!c.EF;this.mode="mat-spinner"===a.R.nodeName.toLowerCase()?"indeterminate":"determinate";c&&(c.color&&(this.color=this.C=c.color),c.diameter&&(this.diameter=c.diameter),c.strokeWidth&&(this.strokeWidth=c.strokeWidth))}get value(){return"determinate"===this.mode?this.Lc:0}set value(a){this.Lc=Math.max(0,Math.min(100,a||0))}get diameter(){return this.D}set diameter(a){this.D=
a||0}get strokeWidth(){let a;return null!=(a=this.F)?a:this.diameter/10}set strokeWidth(a){this.F=a||0}zm(){return(this.diameter-10)/2}Nu(){const a=2*this.zm()+this.strokeWidth;return`0 0 ${a} ${a}`}Sm(){return 2*Math.PI*this.zm()}YA(){return"determinate"===this.mode?this.Sm()*(100-this.Lc)/100:null}Et(){return this.strokeWidth/this.diameter*100}};sD.K=function(a){return new (a||sD)(y(wi),y(qi,8),y(rD))};
sD.Ca=Jf({type:sD,ha:[["mat-progress-spinner"],["mat-spinner"]],eb:function(a,b){a&1&&vr(oD,5);if(a&2){let c;wr(c=xr())&&(b.CF=c.first)}},Ka:["role","progressbar","tabindex","-1",1,"mat-mdc-progress-spinner","mdc-circular-progress"],Za:18,Ia:function(a,b){a&2&&(Mp("aria-valuemin",0)("aria-valuemax",100)("aria-valuenow","determinate"===b.mode?b.value:null)("mode",b.mode),sq(vq,"mat-"+b.color),Up("width",b.diameter,"px")("height",b.diameter,"px")("--mdc-circular-progress-size",b.diameter+"px")("--mdc-circular-progress-active-indicator-width",
b.diameter+"px"),Wp("_mat-animation-noopable",b.Lp)("mdc-circular-progress--indeterminate","indeterminate"===b.mode))},inputs:{color:"color",mode:"mode",value:[2,"value","value",Bs],diameter:[2,"diameter","diameter",Bs],strokeWidth:[2,"strokeWidth","strokeWidth",Bs]},ib:["matProgressSpinner"],ka:!0,features:[Lp,Nr],ya:14,za:11,Ba:[["circle",""],["determinateSpinner",""],["aria-hidden","true",1,"mdc-circular-progress__determinate-container"],["xmlns","http://www.w3.org/2000/svg","focusable","false",
1,"mdc-circular-progress__determinate-circle-graphic"],["cx","50%","cy","50%",1,"mdc-circular-progress__determinate-circle"],["aria-hidden","true",1,"mdc-circular-progress__indeterminate-container"],[1,"mdc-circular-progress__spinner-layer"],[1,"mdc-circular-progress__circle-clipper","mdc-circular-progress__circle-left"],[3,"ngTemplateOutlet"],[1,"mdc-circular-progress__gap-patch"],[1,"mdc-circular-progress__circle-clipper","mdc-circular-progress__circle-right"],["xmlns","http://www.w3.org/2000/svg",
"focusable","false",1,"mdc-circular-progress__indeterminate-circle-graphic"],["cx","50%","cy","50%"]],sa:function(a,b){a&1&&(H(0,pD,2,8,"ng-template",null,0,Rr),R(2,"div",2,1),kh(),R(4,"svg",3),Oq(5,"circle",4),U()(),Wg.al=null,R(6,"div",5)(7,"div",6)(8,"div",7),Rq(9,8),U(),R(10,"div",9),Rq(11,8),U(),R(12,"div",10),Rq(13,8),U()()());a&2&&(a=Ar(1),D(4),Mp("viewBox",b.Nu()),D(),Up("stroke-dasharray",b.Sm(),"px")("stroke-dashoffset",b.YA(),"px")("stroke-width",b.Et(),"%"),Mp("r",b.zm()),D(4),Q("ngTemplateOutlet",
a),D(2),Q("ngTemplateOutlet",a),D(2),Q("ngTemplateOutlet",a))},Ha:[ht],styles:["@keyframes mdc-circular-progress-container-rotate{to{transform:rotate(360deg)}}@keyframes mdc-circular-progress-spinner-layer-rotate{12.5%{transform:rotate(135deg)}25%{transform:rotate(270deg)}37.5%{transform:rotate(405deg)}50%{transform:rotate(540deg)}62.5%{transform:rotate(675deg)}75%{transform:rotate(810deg)}87.5%{transform:rotate(945deg)}100%{transform:rotate(1080deg)}}@keyframes mdc-circular-progress-color-1-fade-in-out{from{opacity:.99}25%{opacity:.99}26%{opacity:0}89%{opacity:0}90%{opacity:.99}to{opacity:.99}}@keyframes mdc-circular-progress-color-2-fade-in-out{from{opacity:0}15%{opacity:0}25%{opacity:.99}50%{opacity:.99}51%{opacity:0}to{opacity:0}}@keyframes mdc-circular-progress-color-3-fade-in-out{from{opacity:0}40%{opacity:0}50%{opacity:.99}75%{opacity:.99}76%{opacity:0}to{opacity:0}}@keyframes mdc-circular-progress-color-4-fade-in-out{from{opacity:0}65%{opacity:0}75%{opacity:.99}90%{opacity:.99}to{opacity:0}}@keyframes mdc-circular-progress-left-spin{from{transform:rotate(265deg)}50%{transform:rotate(130deg)}to{transform:rotate(265deg)}}@keyframes mdc-circular-progress-right-spin{from{transform:rotate(-265deg)}50%{transform:rotate(-130deg)}to{transform:rotate(-265deg)}}.mdc-circular-progress{display:inline-flex;position:relative;direction:ltr;line-height:0;transition:opacity 250ms 0ms cubic-bezier(0.4, 0, 0.6, 1)}.mdc-circular-progress__determinate-container,.mdc-circular-progress__indeterminate-circle-graphic,.mdc-circular-progress__indeterminate-container,.mdc-circular-progress__spinner-layer{position:absolute;width:100%;height:100%}.mdc-circular-progress__determinate-container{transform:rotate(-90deg)}.mdc-circular-progress__indeterminate-container{font-size:0;letter-spacing:0;white-space:nowrap;opacity:0}.mdc-circular-progress__determinate-circle-graphic,.mdc-circular-progress__indeterminate-circle-graphic{fill:rgba(0,0,0,0)}.mdc-circular-progress__determinate-circle{transition:stroke-dashoffset 500ms 0ms cubic-bezier(0, 0, 0.2, 1)}.mdc-circular-progress__gap-patch{position:absolute;top:0;left:47.5%;box-sizing:border-box;width:5%;height:100%;overflow:hidden}.mdc-circular-progress__gap-patch .mdc-circular-progress__indeterminate-circle-graphic{left:-900%;width:2000%;transform:rotate(180deg)}.mdc-circular-progress__circle-clipper{display:inline-flex;position:relative;width:50%;height:100%;overflow:hidden}.mdc-circular-progress__circle-clipper .mdc-circular-progress__indeterminate-circle-graphic{width:200%}.mdc-circular-progress__circle-right .mdc-circular-progress__indeterminate-circle-graphic{left:-100%}.mdc-circular-progress--indeterminate .mdc-circular-progress__determinate-container{opacity:0}.mdc-circular-progress--indeterminate .mdc-circular-progress__indeterminate-container{opacity:1}.mdc-circular-progress--indeterminate .mdc-circular-progress__indeterminate-container{animation:mdc-circular-progress-container-rotate 1568.2352941176ms linear infinite}.mdc-circular-progress--indeterminate .mdc-circular-progress__spinner-layer{animation:mdc-circular-progress-spinner-layer-rotate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both}.mdc-circular-progress--indeterminate .mdc-circular-progress__color-1{animation:mdc-circular-progress-spinner-layer-rotate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both,mdc-circular-progress-color-1-fade-in-out 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both}.mdc-circular-progress--indeterminate .mdc-circular-progress__color-2{animation:mdc-circular-progress-spinner-layer-rotate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both,mdc-circular-progress-color-2-fade-in-out 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both}.mdc-circular-progress--indeterminate .mdc-circular-progress__color-3{animation:mdc-circular-progress-spinner-layer-rotate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both,mdc-circular-progress-color-3-fade-in-out 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both}.mdc-circular-progress--indeterminate .mdc-circular-progress__color-4{animation:mdc-circular-progress-spinner-layer-rotate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both,mdc-circular-progress-color-4-fade-in-out 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both}.mdc-circular-progress--indeterminate .mdc-circular-progress__circle-left .mdc-circular-progress__indeterminate-circle-graphic{animation:mdc-circular-progress-left-spin 1333ms cubic-bezier(0.4, 0, 0.2, 1) infinite both}.mdc-circular-progress--indeterminate .mdc-circular-progress__circle-right .mdc-circular-progress__indeterminate-circle-graphic{animation:mdc-circular-progress-right-spin 1333ms cubic-bezier(0.4, 0, 0.2, 1) infinite both}.mdc-circular-progress--closed{opacity:0}.mat-mdc-progress-spinner .mdc-circular-progress__determinate-circle,.mat-mdc-progress-spinner .mdc-circular-progress__indeterminate-circle-graphic{stroke:var(--mdc-circular-progress-active-indicator-color)}@media screen and (forced-colors: active),(-ms-high-contrast: active){.mat-mdc-progress-spinner .mdc-circular-progress__determinate-circle,.mat-mdc-progress-spinner .mdc-circular-progress__indeterminate-circle-graphic{stroke:CanvasText}}.mat-mdc-progress-spinner circle{stroke-width:var(--mdc-circular-progress-active-indicator-width)}@media screen and (forced-colors: active),(-ms-high-contrast: active){.mat-mdc-progress-spinner .mdc-circular-progress--four-color .mdc-circular-progress__color-1 .mdc-circular-progress__indeterminate-circle-graphic{stroke:CanvasText}}@media screen and (forced-colors: active),(-ms-high-contrast: active){.mat-mdc-progress-spinner .mdc-circular-progress--four-color .mdc-circular-progress__color-2 .mdc-circular-progress__indeterminate-circle-graphic{stroke:CanvasText}}@media screen and (forced-colors: active),(-ms-high-contrast: active){.mat-mdc-progress-spinner .mdc-circular-progress--four-color .mdc-circular-progress__color-3 .mdc-circular-progress__indeterminate-circle-graphic{stroke:CanvasText}}@media screen and (forced-colors: active),(-ms-high-contrast: active){.mat-mdc-progress-spinner .mdc-circular-progress--four-color .mdc-circular-progress__color-4 .mdc-circular-progress__indeterminate-circle-graphic{stroke:CanvasText}}.mat-mdc-progress-spinner .mdc-circular-progress{width:var(--mdc-circular-progress-size) !important;height:var(--mdc-circular-progress-size) !important}.mat-mdc-progress-spinner{display:block;overflow:hidden;line-height:0}.mat-mdc-progress-spinner._mat-animation-noopable,.mat-mdc-progress-spinner._mat-animation-noopable .mdc-circular-progress__determinate-circle{transition:none}.mat-mdc-progress-spinner._mat-animation-noopable .mdc-circular-progress__indeterminate-circle-graphic,.mat-mdc-progress-spinner._mat-animation-noopable .mdc-circular-progress__spinner-layer,.mat-mdc-progress-spinner._mat-animation-noopable .mdc-circular-progress__indeterminate-container{animation:none}.mat-mdc-progress-spinner._mat-animation-noopable .mdc-circular-progress__indeterminate-container circle{stroke-dasharray:0 !important}.cdk-high-contrast-active .mat-mdc-progress-spinner .mdc-circular-progress__indeterminate-circle-graphic,.cdk-high-contrast-active .mat-mdc-progress-spinner .mdc-circular-progress__determinate-circle{stroke:currentColor;stroke:CanvasText}\n"],
pb:2,Pa:0});var tD=class{};tD.K=function(a){return new (a||tD)};tD.Ya=Sf({type:tD});tD.Xa=Qd({imports:[it,mz]});function uD(a,b,c){b="indeterminate"===c?b:b+" "+lD.format(Number(c)/100);a.D.textContent=b}
var vD=class{constructor(a,b){this.options=a;this.F=b;this.C=new hj;this.D=document.createElement("label");this.D.classList.add("cdk-visually-hidden");this.D.classList.add("gmat-mdc-progress-spinner-accessible-label");C(mD(b.ra.R),Oo(this.C)).subscribe(c=>{uD(this,c[0],c[1])})}Fb(){this.F.ra.R.insertAdjacentElement("afterend",this.D);const a=this.F.ra.R.getAttribute("aria-label");uD(this,null!=a?a:"","indeterminate"===this.F.mode?"indeterminate":this.F.value.toString())}va(){this.D.remove();this.C.next();
this.C.complete()}};vD.K=function(a){return new (a||vD)(y(nD),y(sD,10))};vD.Fa=Uf({type:vD,ha:[["mat-progress-spinner"],["mat-spinner"]],Ka:["aria-hidden","true"],Za:3,Ia:function(a,b){a&2&&(Mp("tabindex",null),Wp("gmat-mdc-progress-spinner",!b.options.tc))}});var wD=class{};wD.K=function(a){return new (a||wD)};wD.Ya=Sf({type:wD});wD.Xa=Qd({imports:[tD]});var xD=new gf("GMAT_SELECT_OPTIONS",{fa:"root",ba:()=>({tc:!1})});var yD=Yy("transformPanel",[az("void",$y({opacity:0,transform:"scale(1, 0.8)"})),bz("void => showing",Zy("120ms cubic-bezier(0, 0, 0.2, 1)",$y({opacity:1,transform:"scale(1, 1)"}))),bz("* => void",Zy("100ms linear",$y({opacity:0})))]);const zD=["trigger"],AD=["panel"],BD=[[["mat-select-trigger"]],"*"];function CD(a){a&1&&(R(0,"span",4),Z(1),U());a&2&&(a=Y(),D(),Br(a.placeholder))}function DD(a){a&1&&$q(0)}function ED(a){a&1&&(R(0,"span",11),Z(1),U());a&2&&(a=Y(2),D(),Br(a.WE))}function FD(a){a&1&&(R(0,"span",5),H(1,DD,1,0)(2,ED,2,1,"span",11),U());a&2&&(a=Y(),D(),Gq(a.HB?1:2))}
function GD(a){if(a&1){const b=Zg();R(0,"div",12,1);V("@transformPanel.done",function(c){t(b);const d=Y();return x(d.tu.next(c.se))})("keydown",function(c){t(b);const d=Y();return x(d.Ve(c))});$q(2,1);U()}a&2&&(a=Y(),xq(a.kA()),Q("ngClass",a.hb)("@transformPanel","showing"),Mp("id",a.id+"-panel")("aria-multiselectable",a.multiple)("aria-label",a.ariaLabel||null)("aria-labelledby",a.zp()))}let HD=0;
var ID=new gf("mat-select-scroll-strategy",{fa:"root",ba:()=>{const a=of(cw);return()=>a.C.C()}}),JD=new gf("MAT_SELECT_CONFIG"),KD={pa:ID,Bc:[cw],Gc:function(a){return()=>a.C.C()}},LD=new gf("MatSelectTrigger"),MD=class{constructor(a,b){this.source=a;this.value=b}};function aB(a,b){b.length?a.ra.R.setAttribute("aria-describedby",b.join(" ")):a.ra.R.removeAttribute("aria-describedby")}function ND(a){return a.ja?"rtl"===a.ja.value:!1}
var OD=class{Qa(a){var b=this.options.toArray()[a];if(b){const c=this.Xf.R,d=Wz(a,this.options,this.Qr);b=b.we();c.scrollTop=0===a&&1===d?0:Xz(b.offsetTop,b.offsetHeight,c.scrollTop,c.offsetHeight)}}md(){this.Qa(this.C.O||0)}cc(a){return new MD(this,a)}get focused(){return this.Te||this.F}get Ge(){return this.oa}set Ge(a){this.oa=a;if(this.options)for(const b of this.options)en(b.Da)}get placeholder(){return this.Kb}set placeholder(a){this.Kb=a;this.Tc.next()}get required(){var a;let b;null==(a=this.sd)&&
(null==(a=this.Ld)?a=void 0:null==(a=a.control)?a=void 0:(a=a.Ze,a=Array.isArray(a)?a.includes(yw):a===yw));return null!=(b=a)?b:!1}set required(a){this.sd=a;this.Tc.next()}get multiple(){return this.G}set multiple(a){this.G=a}get Uk(){return this.aa}set Uk(a){this.aa=a;this.D&&this.wa()}get value(){return this.Lc}set value(a){this.ta(a)&&this.ea(a)}get bw(){return this.T.yr}set bw(a){this.T.yr=a}get id(){return this.Zb}set id(a){this.Zb=a||this.ud;this.Tc.next()}get De(){return this.T.De}set De(a){this.T.De=
a}constructor(a,b,c,d,e,f,g,h,m,p,u,w,B,G){this.Pb=a;this.Da=b;this.ra=e;this.ja=f;this.P=m;this.Ld=p;this.sb=B;this.N=G;this.NA=[{bb:"start",cb:"bottom",Na:"start",Ua:"top"},{bb:"end",cb:"bottom",Na:"end",Ua:"top"},{bb:"start",cb:"top",Na:"start",Ua:"bottom",hb:"mat-mdc-select-panel-above"},{bb:"end",cb:"top",Na:"end",Ua:"bottom",hb:"mat-mdc-select-panel-above"}];this.F=!1;this.aa=(K,W)=>K===W;this.ud=`mat-select-${HD++}`;this.gb=null;this.J=new hj;this.Tc=new hj;this.NB=!0;this.ea=()=>{};this.de=
()=>{};this.Mu=`mat-select-value-${HD++}`;this.tu=new hj;let L;this.LA=(null==(L=this.N)?void 0:L.Zh)||"";this.Te=!1;this.zv="mat-select";this.Nc=this.disabled=!1;this.tabIndex=0;let O,ba;this.oa=null!=(ba=null==(O=this.N)?void 0:O.Ge)?ba:!1;this.G=!1;let ca,na;this.Wv=null!=(na=null==(ca=this.N)?void 0:ca.Wv)?na:!1;this.ariaLabel="";this.Ig=this.N&&"undefined"!==typeof this.N.Ig?this.N.Ig:"auto";this.ca=new hj;this.lb=nk(()=>{const K=this.options;return K?C(K.C,Ko(K),Lo(()=>wk(...K.map(W=>W.po)))):
C(this.ca,Lo(()=>this.lb))});this.so=new Jk;this.IA=C(this.so,xk(K=>K),Cj(()=>{}));this.Uz=C(this.so,xk(K=>!K),Cj(()=>{}));this.Tx=new Jk;this.Ny=new Jk;this.O=null;this.Ob=K=>this.qb?!1:K.disabled;this.Ld&&(this.Ld.C=this);null!=(null==G?void 0:G.nm)&&(this.nm=G.nm);this.T=new nz(d,p,h,g,this.Tc);this.Lb=w;this.Kc=this.Lb();this.tabIndex=parseInt(u)||0;this.id=this.id}Fb(){this.D=new wu(this.multiple);this.Tc.next();C(this.tu,fo(),Oo(this.J)).subscribe(()=>this.ld(this.qb));C(this.Pb.xb(),Oo(this.J)).subscribe(()=>
{this.qb&&(this.su=this.ua(this.Pp),fn(this.Da))})}Bj(){this.ca.next();this.ca.complete();this.Qc();C(this.D.Ch,Oo(this.J)).subscribe(a=>{a.added.forEach(b=>b.select());a.removed.forEach(b=>Uz(b))});C(this.options.C,Ko(null),Oo(this.J)).subscribe(()=>{this.td();this.wa()})}Cj(){var a=this.Ec(),b=this.Ld;if(a!==this.gb){var c=this.ra.R;(this.gb=a)?c.setAttribute("aria-labelledby",a):c.removeAttribute("aria-labelledby")}if(b){this.Ea!==b.control&&(void 0!==this.Ea&&null!==b.disabled&&b.disabled!==this.disabled&&
(this.disabled=b.disabled),this.Ea=b.control);a=this.T;b=a.De;c=a.G||a.F;const d=a.Ld?a.Ld.control:null;let e;c=null!=(e=null==(a.yr||a.D)?void 0:!!(d&&d.Vn&&(d.Vc||c&&c.Zj)))?e:!1;c!==b&&(a.De=c,a.C.next())}}uc(a){(a.disabled||a.userAriaDescribedBy)&&this.Tc.next();a.typeaheadDebounceInterval&&this.C&&Oy(this.C,this.nm)}va(){let a;null==(a=this.C)||a.destroy();this.J.next();this.J.complete();this.Tc.complete();this.O&&(uy(this.O,"aria-owns",`${this.id}-panel`),this.O=null)}toggle(){this.qb?this.close():
this.open()}open(){var a;if(!this.F&&!this.disabled&&0<(null==(a=this.options)?void 0:a.length)){this.P&&(this.Pp=$A(this.P));this.su=this.ua(this.Pp);const b=this.ra.R.closest('body > .cdk-overlay-container [aria-modal="true"]');b&&(a=`${this.id}-panel`,this.O&&uy(this.O,"aria-owns",a),sy(b,"aria-owns",a),this.O=b);this.F=!0;My(this.C,null);this.Jc();en(this.Da);this.Tc.next()}}close(){this.F&&(this.F=!1,My(this.C,ND(this)?"rtl":"ltr"),en(this.Da),this.de(),this.Tc.next())}ti(a){this.ta(a)}Ng(a){this.ea=
a}Nj(a){this.de=a}ji(a){this.disabled=a;en(this.Da);this.Tc.next()}get qb(){return this.F}get selected(){let a,b;return this.multiple?(null==(a=this.D)?void 0:a.selected)||[]:null==(b=this.D)?void 0:b.selected[0]}get WE(){if(this.empty)return"";if(this.G){const a=this.D.selected.map(b=>b.F);ND(this)&&a.reverse();return a.join(", ")}return this.D.selected[0].F}Ve(a){this.disabled||(this.qb?this.Ic(a):this.Hc(a))}Hc(a){var b=a.keyCode;const c=40===b||38===b||37===b||39===b,d=this.C;!(0<d.G.length||
13!==b&&32!==b||Cu(a))||(this.multiple||a.altKey)&&c?(a.preventDefault(),this.open()):this.multiple||(b=this.selected,Uy(d,a),(a=this.selected)&&b!==a&&py(this.sb,a.F,1E4))}Ic(a){const b=this.C;var c=a.keyCode;const d=40===c||38===c,e=0<b.G.length;if(d&&a.altKey)a.preventDefault(),this.close();else if(e||13!==c&&32!==c||!b.C||Cu(a))if(!e&&this.G&&65===c&&a.ctrlKey){a.preventDefault();const f=this.options.some(g=>!g.disabled&&!g.selected);this.options.forEach(g=>{g.disabled||(f?g.select():Uz(g))})}else c=
b.O,Uy(b,a),this.G&&d&&a.shiftKey&&b.C&&b.O!==c&&b.C.wh();else a.preventDefault(),b.C.wh()}Np(){this.disabled||(this.Te=!0,this.Tc.next())}Ck(){this.Te=!1;let a;null!=(a=this.C)&&(a.G=[]);this.disabled||this.qb||(this.de(),en(this.Da),this.Tc.next())}BA(){C(this.KA.Ij,io()).subscribe(()=>{fn(this.Da);this.md()})}kA(){return this.P?`mat-${this.P.color}`:""}get empty(){return!this.D||0===this.D.C.size}wa(){Promise.resolve().then(()=>{this.Ld&&(this.Lc=this.Ld.value);this.Va(this.Lc);this.Tc.next()})}Va(a){this.options.forEach(b=>
{b.D&&(b.D=!1,en(b.Da))});this.D.clear();this.multiple&&a?(a.forEach(b=>this.Ta(b)),this.fb()):(a=this.Ta(a))?Ry(this.C,a):this.qb||Ry(this.C,-1);en(this.Da)}Ta(a){const b=this.options.find(c=>{if(vu(this.D,c))return!1;try{return null!=c.value&&this.aa(c.value,a)}catch(d){return!1}});b&&this.D.select(b);return b}ta(a){return a!==this.Lc||this.G&&Array.isArray(a)?(this.options&&this.Va(a),this.Lc=a,!0):!1}ua(a){return"auto"===this.Ig?(a instanceof fw?a.Ub:a||this.ra).R.getBoundingClientRect().width:
null===this.Ig?"":this.Ig}Qc(){this.C=Jy(Ny(Qy(Py(My(Ly(Oy(new Wy(this.options),this.nm)),ND(this)?"rtl":"ltr")))),this.Ob);this.C.ca.subscribe(()=>{this.qb&&(!this.multiple&&this.C.C&&this.C.C.wh(),this.focus(),this.close())});this.C.xb.subscribe(()=>{this.F&&this.Xf?this.Qa(this.C.O||0):this.F||this.multiple||!this.C.C||this.C.C.wh()})}td(){const a=wk(this.options.C,this.J);C(this.lb,Oo(a)).subscribe(b=>{this.kd(b.source,b.C);b.C&&!this.multiple&&this.F&&(this.close(),this.focus())});C(wk(...this.options.map(b=>
b.G)),Oo(a)).subscribe(()=>{fn(this.Da);this.Tc.next()})}kd(a,b){const c=vu(this.D,a);null!=a.value||this.G?(c!==a.selected&&(a.selected?this.D.select(a):uu(this.D,a)),b&&this.C.D(a),this.multiple&&(this.fb(),b&&this.focus())):(Uz(a),this.D.clear(),null!=this.value&&this.Ja(a.value));c!==vu(this.D,a)&&this.Ja();this.Tc.next()}fb(){if(this.multiple){const a=this.options.toArray();this.D.sort((b,c)=>this.ny?this.ny(b,c,a):a.indexOf(b)-a.indexOf(c));this.Tc.next()}}Ja(a){let b;this.multiple?b=this.selected.map(c=>
c.value):b=this.selected?this.selected.value:a;this.Lc=b;this.Ny.emit(b);this.ea(b);this.Tx.emit(this.cc(b));en(this.Da)}Jc(){if(this.C)if(this.empty){let a=-1;for(let b=0;b<this.options.length;b++)if(!this.options.get(b).disabled){a=b;break}this.C.D(a)}else this.C.D(this.D.selected[0])}focus(a){this.ra.R.focus(a)}zp(){if(this.ariaLabel)return null;let a;const b=null==(a=this.P)?void 0:a.rh()?a.Xe:null;return this.ad?(b?b+" ":"")+this.ad:b}iA(){return this.qb&&this.C&&this.C.C?this.C.C.id:null}Ec(){if(this.ariaLabel)return null;
var a;const b=null==(a=this.P)?void 0:a.rh()?a.Xe:null;a=(b?b+" ":"")+this.Mu;this.ad&&(a+=" "+this.ad);return a}ld(a){this.so.emit(a)}get hE(){return this.qb||!this.empty||this.focused&&!!this.placeholder}};OD.K=function(a){return new (a||OD)(y(xu),y(Tr),y(Tk),y(pz),y(wi),y(pu,8),y(tx,8),y(Ax,8),y(XA,8),y(Kw,10),Dh("tabindex"),y(ID),y(qy),y(JD,8))};
OD.Ca=Jf({type:OD,ha:[["mat-select"]],hc:function(a,b,c){a&1&&(ur(c,LD,5),ur(c,Vz,5),ur(c,uz,5));if(a&2){let d;wr(d=xr())&&(b.HB=d.first);wr(d=xr())&&(b.options=d);wr(d=xr())&&(b.Qr=d)}},eb:function(a,b){a&1&&(vr(zD,5),vr(AD,5),vr(iw,5));if(a&2){let c;wr(c=xr())&&(b.Oe=c.first);wr(c=xr())&&(b.Xf=c.first);wr(c=xr())&&(b.KA=c.first)}},Ka:["role","combobox","aria-autocomplete","none","aria-haspopup","listbox",1,"mat-mdc-select"],Za:19,Ia:function(a,b){a&1&&V("keydown",function(c){return b.Ve(c)})("focus",
function(){return b.Np()})("blur",function(){return b.Ck()});a&2&&(Mp("id",b.id)("tabindex",b.disabled?-1:b.tabIndex)("aria-controls",b.qb?b.id+"-panel":null)("aria-expanded",b.qb)("aria-label",b.ariaLabel||null)("aria-required",b.required.toString())("aria-disabled",b.disabled.toString())("aria-invalid",b.De)("aria-activedescendant",b.iA()),Wp("mat-mdc-select-disabled",b.disabled)("mat-mdc-select-invalid",b.De)("mat-mdc-select-required",b.required)("mat-mdc-select-empty",b.empty)("mat-mdc-select-multiple",
b.multiple))},inputs:{Vs:[0,"aria-describedby","userAriaDescribedBy"],hb:"panelClass",disabled:[2,"disabled","disabled",As],Nc:[2,"disableRipple","disableRipple",As],tabIndex:[2,"tabIndex","tabIndex",a=>null==a?0:Bs(a)],Ge:[2,"hideSingleSelectionIndicator","hideSingleSelectionIndicator",As],placeholder:"placeholder",required:[2,"required","required",As],multiple:[2,"multiple","multiple",As],Wv:[2,"disableOptionCentering","disableOptionCentering",As],Uk:"compareWith",value:"value",ariaLabel:[0,"aria-label",
"ariaLabel"],ad:[0,"aria-labelledby","ariaLabelledby"],bw:"errorStateMatcher",nm:[2,"typeaheadDebounceInterval","typeaheadDebounceInterval",Bs],ny:"sortComparator",id:"id",Ig:"panelWidth"},outputs:{so:"openedChange",IA:"opened",Uz:"closed",Tx:"selectionChange",Ny:"valueChange"},ib:["matSelect"],ka:!0,features:[Lr([{pa:uA,Ab:OD},{pa:rz,Ab:OD}]),Lp,Hg,Nr],bc:["mat-select-trigger","*"],ya:11,za:8,Ba:[["fallbackOverlayOrigin","cdkOverlayOrigin","trigger",""],["panel",""],["cdk-overlay-origin","",1,"mat-mdc-select-trigger",
3,"click"],[1,"mat-mdc-select-value"],[1,"mat-mdc-select-placeholder","mat-mdc-select-min-line"],[1,"mat-mdc-select-value-text"],[1,"mat-mdc-select-arrow-wrapper"],[1,"mat-mdc-select-arrow"],"viewBox;0 0 24 24;width;24px;height;24px;focusable;false;aria-hidden;true".split(";"),["d","M7 10l5 5 5-5z"],["cdk-connected-overlay","","cdkConnectedOverlayLockPosition","","cdkConnectedOverlayHasBackdrop","","cdkConnectedOverlayBackdropClass","cdk-overlay-transparent-backdrop",3,"backdropClick","attach","detach",
"cdkConnectedOverlayPanelClass","cdkConnectedOverlayScrollStrategy","cdkConnectedOverlayOrigin","cdkConnectedOverlayOpen","cdkConnectedOverlayPositions","cdkConnectedOverlayWidth"],[1,"mat-mdc-select-min-line"],["role","listbox","tabindex","-1",3,"keydown","ngClass"]],sa:function(a,b){if(a&1){const c=Zg();Zq(BD);R(0,"div",2,0);V("click",function(){t(c);return x(b.open())});R(3,"div",3);H(4,CD,2,1,"span",4)(5,FD,3,1,"span",5);U();R(6,"div",6)(7,"div",7);kh();R(8,"svg",8);Oq(9,"path",9);U()()()();H(10,
GD,3,9,"ng-template",10);V("backdropClick",function(){t(c);return x(b.close())})("attach",function(){t(c);return x(b.BA())})("detach",function(){t(c);return x(b.close())})}a&2&&(a=Ar(1),D(3),Mp("id",b.Mu),D(),Gq(b.empty?4:5),D(6),Q("cdkConnectedOverlayPanelClass",b.LA)("cdkConnectedOverlayScrollStrategy",b.Kc)("cdkConnectedOverlayOrigin",b.Pp||a)("cdkConnectedOverlayOpen",b.qb)("cdkConnectedOverlayPositions",b.NA)("cdkConnectedOverlayWidth",b.su))},Ha:[fw,iw,Xs],styles:['.mat-mdc-select{display:inline-block;width:100%;outline:none;-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;color:var(--mat-select-enabled-trigger-text-color);font-family:var(--mat-select-trigger-text-font);line-height:var(--mat-select-trigger-text-line-height);font-size:var(--mat-select-trigger-text-size);font-weight:var(--mat-select-trigger-text-weight);letter-spacing:var(--mat-select-trigger-text-tracking)}div.mat-mdc-select-panel{box-shadow:var(--mat-select-container-elevation-shadow)}.mat-mdc-select-disabled{color:var(--mat-select-disabled-trigger-text-color)}.mat-mdc-select-trigger{display:inline-flex;align-items:center;cursor:pointer;position:relative;box-sizing:border-box;width:100%}.mat-mdc-select-disabled .mat-mdc-select-trigger{-webkit-user-select:none;user-select:none;cursor:default}.mat-mdc-select-value{width:100%;overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.mat-mdc-select-value-text{white-space:nowrap;overflow:hidden;text-overflow:ellipsis}.mat-mdc-select-arrow-wrapper{height:24px;flex-shrink:0;display:inline-flex;align-items:center}.mat-form-field-appearance-fill .mdc-text-field--no-label .mat-mdc-select-arrow-wrapper{transform:none}.mat-mdc-form-field .mat-mdc-select.mat-mdc-select-invalid .mat-mdc-select-arrow,.mat-form-field-invalid:not(.mat-form-field-disabled) .mat-mdc-form-field-infix::after{color:var(--mat-select-invalid-arrow-color)}.mat-mdc-select-arrow{width:10px;height:5px;position:relative;color:var(--mat-select-enabled-arrow-color)}.mat-mdc-form-field.mat-focused .mat-mdc-select-arrow{color:var(--mat-select-focused-arrow-color)}.mat-mdc-form-field .mat-mdc-select.mat-mdc-select-disabled .mat-mdc-select-arrow{color:var(--mat-select-disabled-arrow-color)}.mat-mdc-select-arrow svg{fill:currentColor;position:absolute;top:50%;left:50%;transform:translate(-50%, -50%)}.cdk-high-contrast-active .mat-mdc-select-arrow svg{fill:CanvasText}.mat-mdc-select-disabled .cdk-high-contrast-active .mat-mdc-select-arrow svg{fill:GrayText}div.mat-mdc-select-panel{width:100%;max-height:275px;outline:0;overflow:auto;padding:8px 0;border-radius:4px;box-sizing:border-box;position:static;background-color:var(--mat-select-panel-background-color)}.cdk-high-contrast-active div.mat-mdc-select-panel{outline:solid 1px}.cdk-overlay-pane:not(.mat-mdc-select-panel-above) div.mat-mdc-select-panel{border-top-left-radius:0;border-top-right-radius:0;transform-origin:top center}.mat-mdc-select-panel-above div.mat-mdc-select-panel{border-bottom-left-radius:0;border-bottom-right-radius:0;transform-origin:bottom center}div.mat-mdc-select-panel .mat-mdc-option{--mdc-list-list-item-container-color: var(--mat-select-panel-background-color)}.mat-mdc-select-placeholder{transition:color 400ms 133.3333333333ms cubic-bezier(0.25, 0.8, 0.25, 1);color:var(--mat-select-placeholder-text-color)}._mat-animation-noopable .mat-mdc-select-placeholder{transition:none}.mat-form-field-hide-placeholder .mat-mdc-select-placeholder{color:rgba(0,0,0,0);-webkit-text-fill-color:rgba(0,0,0,0);transition:none;display:block}.mat-mdc-form-field-type-mat-select:not(.mat-form-field-disabled) .mat-mdc-text-field-wrapper{cursor:pointer}.mat-mdc-form-field-type-mat-select.mat-form-field-appearance-fill .mat-mdc-floating-label{max-width:calc(100% - 18px)}.mat-mdc-form-field-type-mat-select.mat-form-field-appearance-fill .mdc-floating-label--float-above{max-width:calc(100%/0.75 - 24px)}.mat-mdc-form-field-type-mat-select.mat-form-field-appearance-outline .mdc-notched-outline__notch{max-width:calc(100% - 60px)}.mat-mdc-form-field-type-mat-select.mat-form-field-appearance-outline .mdc-text-field--label-floating .mdc-notched-outline__notch{max-width:calc(100% - 24px)}.mat-mdc-select-min-line:empty::before{content:" ";white-space:pre;width:1px;display:inline-block;visibility:hidden}.mat-form-field-appearance-fill .mat-mdc-select-arrow-wrapper{transform:var(--mat-select-arrow-transform)}\n'],
pb:2,data:{animation:[yD]},Pa:0});var PD=class{};PD.K=function(a){return new (a||PD)};PD.Ya=Sf({type:PD});PD.Xa=Qd({Wa:[KD],imports:[it,kw,$z,mz,yu,cB,$z,mz]});var QD={pa:JD,Bc:[[new Wh,new Xh,new Vh(JD)],[new Wh,new Vh(xD)]],Gc:(a,b)=>!b||b.tc?a||{}:a?Object.assign({},a,{Zh:Qs(a.Zh).concat("gmat-mdc-select")}):{Zh:"gmat-mdc-select"}},RD=class{};RD.K=function(a){return new (a||RD)};RD.Ya=Sf({type:RD});RD.Xa=Qd({Wa:[QD],imports:[PD,AC]});var SD=new gf("GMAT_SLIDE_TOGGLE_OPTIONS",{fa:"root",ba:()=>({tc:!1})});var TD=class{constructor(a){this.options=a}};TD.K=function(a){return new (a||TD)(y(SD))};TD.Fa=Uf({type:TD,ha:[["mat-slide-toggle"]],Za:2,Ia:function(a,b){a&2&&Wp("gmat-mdc-slide-toggle",!b.options.tc)}});var UD=new gf("mat-slide-toggle-default-options",{fa:"root",ba:()=>({RB:!1,On:!1})});const VD=["switch"];function WD(a){a&1&&(R(0,"div",10),kh(),R(1,"svg",12),Oq(2,"path",13),U(),R(3,"svg",14),Oq(4,"path",15),U()())}var YD={pa:qw,Ab:df(()=>XD),Eb:!0},ZD=class{constructor(a,b){this.source=a;this.checked=b}};let $D=0;
var XD=class{tp(a){return new ZD(this,a)}get cv(){return`${this.id||this.Ik}-button`}focus(){this.ZA.R.focus()}get checked(){return this.zf}set checked(a){this.zf=a;en(this.Da)}get ir(){return`${this.id||this.Ik}-input`}constructor(a,b,c,d,e,f){this.ra=a;this.D=b;this.Da=c;this.defaults=e;this.C=()=>{};this.de=()=>{};this.F=()=>{};this.zf=!1;this.name=null;this.Oh="after";this.ad=this.ariaLabel=null;this.Nc=this.disabled=!1;this.tabIndex=0;this.xb=new Jk;this.By=new Jk;this.tabIndex=parseInt(d)||
0;this.color=e.color||"accent";this.Lp="NoopAnimations"===f;this.id=this.Ik=`mat-mdc-slide-toggle-${++$D}`;let g;this.On=null!=(g=e.On)?g:!1;this.Xe=this.Ik+"-label"}Bj(){Mx(this.D,this.ra,!0).subscribe(a=>{"keyboard"===a||"program"===a?(this.Te=!0,en(this.Da)):a||Promise.resolve().then(()=>{this.Te=!1;this.de();en(this.Da)})})}uc(a){a.required&&this.F()}va(){Nx(this.D,this.ra)}ti(a){this.checked=!!a}Ng(a){this.C=a}Nj(a){this.de=a}validate(a){return this.required&&!0!==a.value?{required:!0}:null}hs(a){this.F=
a}ji(a){this.disabled=a;en(this.Da)}toggle(){this.checked=!this.checked;this.C(this.checked)}Li(){this.C(this.checked);this.xb.emit(this.tp(this.checked))}tk(){this.By.emit();this.defaults.RB||(this.checked=!this.checked,this.C(this.checked),this.xb.emit(new ZD(this,this.checked)))}jA(){return this.ad?this.ad:this.ariaLabel?null:this.Xe}};XD.K=function(a){return new (a||XD)(y(wi),y(Px),y(Tr),Dh("tabindex"),y(UD),y(qi,8))};
XD.Ca=Jf({type:XD,ha:[["mat-slide-toggle"]],eb:function(a,b){a&1&&vr(VD,5);if(a&2){let c;wr(c=xr())&&(b.ZA=c.first)}},Ka:[1,"mat-mdc-slide-toggle"],Za:13,Ia:function(a,b){a&2&&(Sq("id",b.id),Mp("tabindex",null)("aria-label",null)("name",null)("aria-labelledby",null),sq(vq,b.color?"mat-"+b.color:""),Wp("mat-mdc-slide-toggle-focused",b.Te)("mat-mdc-slide-toggle-checked",b.checked)("_mat-animation-noopable",b.Lp))},inputs:{name:"name",id:"id",Oh:"labelPosition",ariaLabel:[0,"aria-label","ariaLabel"],
ad:[0,"aria-labelledby","ariaLabelledby"],Mk:[0,"aria-describedby","ariaDescribedby"],required:[2,"required","required",As],color:"color",disabled:[2,"disabled","disabled",As],Nc:[2,"disableRipple","disableRipple",As],tabIndex:[2,"tabIndex","tabIndex",a=>null==a?0:Bs(a)],checked:[2,"checked","checked",As],On:[2,"hideIcon","hideIcon",As]},outputs:{xb:"change",By:"toggleChange"},ib:["matSlideToggle"],ka:!0,features:[Lr([YD,{pa:ww,Ab:XD,Eb:!0}]),Lp,Hg,Nr],bc:["*"],ya:13,za:24,Ba:[["switch",""],["mat-internal-form-field",
"",3,"labelPosition"],["role","switch","type","button",1,"mdc-switch",3,"click","tabIndex","disabled"],[1,"mdc-switch__track"],[1,"mdc-switch__handle-track"],[1,"mdc-switch__handle"],[1,"mdc-switch__shadow"],[1,"mdc-elevation-overlay"],[1,"mdc-switch__ripple"],["mat-ripple","",1,"mat-mdc-slide-toggle-ripple","mat-mdc-focus-indicator",3,"matRippleTrigger","matRippleDisabled","matRippleCentered"],[1,"mdc-switch__icons"],[1,"mdc-label",3,"click","for"],["viewBox","0 0 24 24","aria-hidden","true",1,"mdc-switch__icon",
"mdc-switch__icon--on"],["d","M19.69,5.23L8.96,15.96l-4.23-4.23L2.96,13.5l6,6L21.46,7L19.69,5.23z"],["viewBox","0 0 24 24","aria-hidden","true",1,"mdc-switch__icon","mdc-switch__icon--off"],["d","M20 13H4v-2h16v2z"]],sa:function(a,b){if(a&1){const c=Zg();Zq();R(0,"div",1)(1,"button",2,0);V("click",function(){t(c);return x(b.tk())});Oq(3,"div",3);R(4,"div",4)(5,"div",5)(6,"div",6);Oq(7,"div",7);U();R(8,"div",8);Oq(9,"div",9);U();H(10,WD,5,0,"div",10);U()()();R(11,"label",11);V("click",function(d){t(c);
return x(d.stopPropagation())});$q(12);U()()}a&2&&(a=Ar(2),Q("labelPosition",b.Oh),D(),Wp("mdc-switch--selected",b.checked)("mdc-switch--unselected",!b.checked)("mdc-switch--checked",b.checked)("mdc-switch--disabled",b.disabled),Q("tabIndex",b.disabled?-1:b.tabIndex)("disabled",b.disabled),Mp("id",b.cv)("name",b.name)("aria-label",b.ariaLabel)("aria-labelledby",b.jA())("aria-describedby",b.Mk)("aria-required",b.required||null)("aria-checked",b.checked),D(8),Q("matRippleTrigger",a)("matRippleDisabled",
b.Nc||b.disabled)("matRippleCentered",!0),D(),Gq(b.On?-1:10),D(),Q("for",b.cv),Mp("id",b.Xe))},Ha:[Lz,qz],styles:['.mdc-switch{align-items:center;background:none;border:none;cursor:pointer;display:inline-flex;flex-shrink:0;margin:0;outline:none;overflow:visible;padding:0;position:relative}.mdc-switch[hidden]{display:none}.mdc-switch:disabled{cursor:default;pointer-events:none}.mdc-switch__track{overflow:hidden;position:relative;width:100%}.mdc-switch__track::before,.mdc-switch__track::after{border:1px solid rgba(0,0,0,0);border-radius:inherit;box-sizing:border-box;content:"";height:100%;left:0;position:absolute;width:100%}@media screen and (forced-colors: active){.mdc-switch__track::before,.mdc-switch__track::after{border-color:currentColor}}.mdc-switch__track::before{transition:transform 75ms 0ms cubic-bezier(0, 0, 0.2, 1);transform:translateX(0)}.mdc-switch__track::after{transition:transform 75ms 0ms cubic-bezier(0.4, 0, 0.6, 1);transform:translateX(-100%)}[dir=rtl] .mdc-switch__track::after,.mdc-switch__track[dir=rtl]::after{transform:translateX(100%)}.mdc-switch--selected .mdc-switch__track::before{transition:transform 75ms 0ms cubic-bezier(0.4, 0, 0.6, 1);transform:translateX(100%)}[dir=rtl] .mdc-switch--selected .mdc-switch__track::before,.mdc-switch--selected .mdc-switch__track[dir=rtl]::before{transform:translateX(-100%)}.mdc-switch--selected .mdc-switch__track::after{transition:transform 75ms 0ms cubic-bezier(0, 0, 0.2, 1);transform:translateX(0)}.mdc-switch__handle-track{height:100%;pointer-events:none;position:absolute;top:0;transition:transform 75ms 0ms cubic-bezier(0.4, 0, 0.2, 1);left:0;right:auto;transform:translateX(0)}[dir=rtl] .mdc-switch__handle-track,.mdc-switch__handle-track[dir=rtl]{left:auto;right:0}.mdc-switch--selected .mdc-switch__handle-track{transform:translateX(100%)}[dir=rtl] .mdc-switch--selected .mdc-switch__handle-track,.mdc-switch--selected .mdc-switch__handle-track[dir=rtl]{transform:translateX(-100%)}.mdc-switch__handle{display:flex;pointer-events:auto;position:absolute;top:50%;transform:translateY(-50%);left:0;right:auto}[dir=rtl] .mdc-switch__handle,.mdc-switch__handle[dir=rtl]{left:auto;right:0}.mdc-switch__handle::before,.mdc-switch__handle::after{border:1px solid rgba(0,0,0,0);border-radius:inherit;box-sizing:border-box;content:"";width:100%;height:100%;left:0;position:absolute;top:0;transition:background-color 75ms 0ms cubic-bezier(0.4, 0, 0.2, 1),border-color 75ms 0ms cubic-bezier(0.4, 0, 0.2, 1);z-index:-1}@media screen and (forced-colors: active){.mdc-switch__handle::before,.mdc-switch__handle::after{border-color:currentColor}}.mdc-switch__shadow{border-radius:inherit;bottom:0;left:0;position:absolute;right:0;top:0}.mdc-elevation-overlay{bottom:0;left:0;right:0;top:0}.mdc-switch__ripple{left:50%;position:absolute;top:50%;transform:translate(-50%, -50%);z-index:-1}.mdc-switch:disabled .mdc-switch__ripple{display:none}.mdc-switch__icons{height:100%;position:relative;width:100%;z-index:1}.mdc-switch__icon{bottom:0;left:0;margin:auto;position:absolute;right:0;top:0;opacity:0;transition:opacity 30ms 0ms cubic-bezier(0.4, 0, 1, 1)}.mdc-switch--selected .mdc-switch__icon--on,.mdc-switch--unselected .mdc-switch__icon--off{opacity:1;transition:opacity 45ms 30ms cubic-bezier(0, 0, 0.2, 1)}.mat-mdc-slide-toggle .mat-internal-form-field{color:var(--mat-switch-label-text-color);font-family:var(--mat-switch-label-text-font);line-height:var(--mat-switch-label-text-line-height);font-size:var(--mat-switch-label-text-size);letter-spacing:var(--mat-switch-label-text-tracking);font-weight:var(--mat-switch-label-text-weight)}.mat-mdc-slide-toggle .mdc-switch--disabled+label{color:var(--mdc-switch-disabled-label-text-color)}.mdc-switch{width:var(--mdc-switch-track-width)}.mdc-switch.mdc-switch--selected:enabled .mdc-switch__handle::after{background:var(--mdc-switch-selected-handle-color)}.mdc-switch.mdc-switch--selected:enabled:hover:not(:focus):not(:active) .mdc-switch__handle::after{background:var(--mdc-switch-selected-hover-handle-color)}.mdc-switch.mdc-switch--selected:enabled:focus:not(:active) .mdc-switch__handle::after{background:var(--mdc-switch-selected-focus-handle-color)}.mdc-switch.mdc-switch--selected:enabled:active .mdc-switch__handle::after{background:var(--mdc-switch-selected-pressed-handle-color)}.mdc-switch.mdc-switch--selected:disabled .mdc-switch__handle::after{background:var(--mdc-switch-disabled-selected-handle-color)}.mdc-switch.mdc-switch--unselected:enabled .mdc-switch__handle::after{background:var(--mdc-switch-unselected-handle-color)}.mdc-switch.mdc-switch--unselected:enabled:hover:not(:focus):not(:active) .mdc-switch__handle::after{background:var(--mdc-switch-unselected-hover-handle-color)}.mdc-switch.mdc-switch--unselected:enabled:focus:not(:active) .mdc-switch__handle::after{background:var(--mdc-switch-unselected-focus-handle-color)}.mdc-switch.mdc-switch--unselected:enabled:active .mdc-switch__handle::after{background:var(--mdc-switch-unselected-pressed-handle-color)}.mdc-switch.mdc-switch--unselected:disabled .mdc-switch__handle::after{background:var(--mdc-switch-disabled-unselected-handle-color)}.mdc-switch .mdc-switch__handle::before{background:var(--mdc-switch-handle-surface-color)}.mdc-switch:enabled .mdc-switch__shadow{box-shadow:var(--mdc-switch-handle-elevation)}.mdc-switch:disabled .mdc-switch__shadow{box-shadow:var(--mdc-switch-disabled-handle-elevation)}.mdc-switch .mdc-switch__focus-ring-wrapper,.mdc-switch .mdc-switch__handle{height:var(--mdc-switch-handle-height)}.mdc-switch .mdc-switch__handle{border-radius:var(--mdc-switch-handle-shape)}.mdc-switch .mdc-switch__handle{width:var(--mdc-switch-handle-width)}.mdc-switch .mdc-switch__handle-track{width:calc(100% - var(--mdc-switch-handle-width))}.mdc-switch.mdc-switch--selected:enabled .mdc-switch__icon{fill:var(--mdc-switch-selected-icon-color)}.mdc-switch.mdc-switch--selected:disabled .mdc-switch__icon{fill:var(--mdc-switch-disabled-selected-icon-color)}.mdc-switch.mdc-switch--unselected:enabled .mdc-switch__icon{fill:var(--mdc-switch-unselected-icon-color)}.mdc-switch.mdc-switch--unselected:disabled .mdc-switch__icon{fill:var(--mdc-switch-disabled-unselected-icon-color)}.mdc-switch.mdc-switch--selected:disabled .mdc-switch__icons{opacity:var(--mdc-switch-disabled-selected-icon-opacity)}.mdc-switch.mdc-switch--unselected:disabled .mdc-switch__icons{opacity:var(--mdc-switch-disabled-unselected-icon-opacity)}.mdc-switch.mdc-switch--selected .mdc-switch__icon{width:var(--mdc-switch-selected-icon-size);height:var(--mdc-switch-selected-icon-size)}.mdc-switch.mdc-switch--unselected .mdc-switch__icon{width:var(--mdc-switch-unselected-icon-size);height:var(--mdc-switch-unselected-icon-size)}.mdc-switch.mdc-switch--selected:enabled:hover:not(:focus) .mdc-switch__ripple::before,.mdc-switch.mdc-switch--selected:enabled:hover:not(:focus) .mdc-switch__ripple::after{background-color:var(--mdc-switch-selected-hover-state-layer-color)}.mdc-switch.mdc-switch--selected:enabled:focus .mdc-switch__ripple::before,.mdc-switch.mdc-switch--selected:enabled:focus .mdc-switch__ripple::after{background-color:var(--mdc-switch-selected-focus-state-layer-color)}.mdc-switch.mdc-switch--selected:enabled:active .mdc-switch__ripple::before,.mdc-switch.mdc-switch--selected:enabled:active .mdc-switch__ripple::after{background-color:var(--mdc-switch-selected-pressed-state-layer-color)}.mdc-switch.mdc-switch--unselected:enabled:hover:not(:focus) .mdc-switch__ripple::before,.mdc-switch.mdc-switch--unselected:enabled:hover:not(:focus) .mdc-switch__ripple::after{background-color:var(--mdc-switch-unselected-hover-state-layer-color)}.mdc-switch.mdc-switch--unselected:enabled:focus .mdc-switch__ripple::before,.mdc-switch.mdc-switch--unselected:enabled:focus .mdc-switch__ripple::after{background-color:var(--mdc-switch-unselected-focus-state-layer-color)}.mdc-switch.mdc-switch--unselected:enabled:active .mdc-switch__ripple::before,.mdc-switch.mdc-switch--unselected:enabled:active .mdc-switch__ripple::after{background-color:var(--mdc-switch-unselected-pressed-state-layer-color)}.mdc-switch.mdc-switch--selected:enabled:hover:not(:focus):hover .mdc-switch__ripple::before,.mdc-switch.mdc-switch--selected:enabled:hover:not(:focus).mdc-ripple-surface--hover .mdc-switch__ripple::before{opacity:var(--mdc-switch-selected-hover-state-layer-opacity)}.mdc-switch.mdc-switch--selected:enabled:focus.mdc-ripple-upgraded--background-focused .mdc-switch__ripple::before,.mdc-switch.mdc-switch--selected:enabled:focus:not(.mdc-ripple-upgraded):focus .mdc-switch__ripple::before{transition-duration:75ms;opacity:var(--mdc-switch-selected-focus-state-layer-opacity)}.mdc-switch.mdc-switch--selected:enabled:active:not(.mdc-ripple-upgraded) .mdc-switch__ripple::after{transition:opacity 150ms linear}.mdc-switch.mdc-switch--selected:enabled:active:not(.mdc-ripple-upgraded):active .mdc-switch__ripple::after{transition-duration:75ms;opacity:var(--mdc-switch-selected-pressed-state-layer-opacity)}.mdc-switch.mdc-switch--selected:enabled:active.mdc-ripple-upgraded{--mdc-ripple-fg-opacity:var(--mdc-switch-selected-pressed-state-layer-opacity)}.mdc-switch.mdc-switch--unselected:enabled:hover:not(:focus):hover .mdc-switch__ripple::before,.mdc-switch.mdc-switch--unselected:enabled:hover:not(:focus).mdc-ripple-surface--hover .mdc-switch__ripple::before{opacity:var(--mdc-switch-unselected-hover-state-layer-opacity)}.mdc-switch.mdc-switch--unselected:enabled:focus.mdc-ripple-upgraded--background-focused .mdc-switch__ripple::before,.mdc-switch.mdc-switch--unselected:enabled:focus:not(.mdc-ripple-upgraded):focus .mdc-switch__ripple::before{transition-duration:75ms;opacity:var(--mdc-switch-unselected-focus-state-layer-opacity)}.mdc-switch.mdc-switch--unselected:enabled:active:not(.mdc-ripple-upgraded) .mdc-switch__ripple::after{transition:opacity 150ms linear}.mdc-switch.mdc-switch--unselected:enabled:active:not(.mdc-ripple-upgraded):active .mdc-switch__ripple::after{transition-duration:75ms;opacity:var(--mdc-switch-unselected-pressed-state-layer-opacity)}.mdc-switch.mdc-switch--unselected:enabled:active.mdc-ripple-upgraded{--mdc-ripple-fg-opacity:var(--mdc-switch-unselected-pressed-state-layer-opacity)}.mdc-switch .mdc-switch__ripple{height:var(--mdc-switch-state-layer-size);width:var(--mdc-switch-state-layer-size)}.mdc-switch .mdc-switch__track{height:var(--mdc-switch-track-height)}.mdc-switch:disabled .mdc-switch__track{opacity:var(--mdc-switch-disabled-track-opacity)}.mdc-switch:enabled .mdc-switch__track::after{background:var(--mdc-switch-selected-track-color)}.mdc-switch:enabled:hover:not(:focus):not(:active) .mdc-switch__track::after{background:var(--mdc-switch-selected-hover-track-color)}.mdc-switch:enabled:focus:not(:active) .mdc-switch__track::after{background:var(--mdc-switch-selected-focus-track-color)}.mdc-switch:enabled:active .mdc-switch__track::after{background:var(--mdc-switch-selected-pressed-track-color)}.mdc-switch:disabled .mdc-switch__track::after{background:var(--mdc-switch-disabled-selected-track-color)}.mdc-switch:enabled .mdc-switch__track::before{background:var(--mdc-switch-unselected-track-color)}.mdc-switch:enabled:hover:not(:focus):not(:active) .mdc-switch__track::before{background:var(--mdc-switch-unselected-hover-track-color)}.mdc-switch:enabled:focus:not(:active) .mdc-switch__track::before{background:var(--mdc-switch-unselected-focus-track-color)}.mdc-switch:enabled:active .mdc-switch__track::before{background:var(--mdc-switch-unselected-pressed-track-color)}.mdc-switch:disabled .mdc-switch__track::before{background:var(--mdc-switch-disabled-unselected-track-color)}.mdc-switch .mdc-switch__track{border-radius:var(--mdc-switch-track-shape)}.mdc-switch:enabled .mdc-switch__shadow{box-shadow:var(--mdc-switch-handle-elevation-shadow)}.mdc-switch:disabled .mdc-switch__shadow{box-shadow:var(--mdc-switch-disabled-handle-elevation-shadow)}.mat-mdc-slide-toggle{display:inline-block;-webkit-tap-highlight-color:rgba(0,0,0,0);outline:0}.mat-mdc-slide-toggle .mat-mdc-slide-toggle-ripple,.mat-mdc-slide-toggle .mdc-switch__ripple::after{top:0;left:0;right:0;bottom:0;position:absolute;border-radius:50%;pointer-events:none}.mat-mdc-slide-toggle .mat-mdc-slide-toggle-ripple:not(:empty),.mat-mdc-slide-toggle .mdc-switch__ripple::after:not(:empty){transform:translateZ(0)}.mat-mdc-slide-toggle .mdc-switch__ripple::after{content:"";opacity:0}.mat-mdc-slide-toggle .mdc-switch:hover .mdc-switch__ripple::after{opacity:.04;transition:opacity 75ms 0ms cubic-bezier(0, 0, 0.2, 1)}.mat-mdc-slide-toggle.mat-mdc-slide-toggle-focused .mdc-switch .mdc-switch__ripple::after{opacity:.12}.mat-mdc-slide-toggle.mat-mdc-slide-toggle-focused .mat-mdc-focus-indicator::before{content:""}.mat-mdc-slide-toggle .mat-ripple-element{opacity:.12}.mat-mdc-slide-toggle .mat-mdc-focus-indicator::before{border-radius:50%}.mat-mdc-slide-toggle._mat-animation-noopable .mdc-switch__handle-track,.mat-mdc-slide-toggle._mat-animation-noopable .mdc-elevation-overlay,.mat-mdc-slide-toggle._mat-animation-noopable .mdc-switch__icon,.mat-mdc-slide-toggle._mat-animation-noopable .mdc-switch__handle::before,.mat-mdc-slide-toggle._mat-animation-noopable .mdc-switch__handle::after,.mat-mdc-slide-toggle._mat-animation-noopable .mdc-switch__track::before,.mat-mdc-slide-toggle._mat-animation-noopable .mdc-switch__track::after{transition:none}.mat-mdc-slide-toggle .mdc-switch:enabled+.mdc-label{cursor:pointer}.mdc-switch__handle{transition:width 75ms cubic-bezier(0.4, 0, 0.2, 1),height 75ms cubic-bezier(0.4, 0, 0.2, 1),margin 75ms cubic-bezier(0.4, 0, 0.2, 1)}.mdc-switch--selected .mdc-switch__track::before{opacity:var(--mat-switch-hidden-track-opacity);transition:var(--mat-switch-hidden-track-transition)}.mdc-switch--selected .mdc-switch__track::after{opacity:var(--mat-switch-visible-track-opacity);transition:var(--mat-switch-visible-track-transition)}.mdc-switch--unselected .mdc-switch__track::before{opacity:var(--mat-switch-visible-track-opacity);transition:var(--mat-switch-visible-track-transition)}.mdc-switch--unselected .mdc-switch__track::after{opacity:var(--mat-switch-hidden-track-opacity);transition:var(--mat-switch-hidden-track-transition)}.mat-mdc-slide-toggle .mdc-switch--unselected .mdc-switch__handle{width:var(--mat-switch-unselected-handle-size);height:var(--mat-switch-unselected-handle-size)}.mat-mdc-slide-toggle .mdc-switch--selected .mdc-switch__handle{width:var(--mat-switch-selected-handle-size);height:var(--mat-switch-selected-handle-size)}.mat-mdc-slide-toggle .mdc-switch__handle:has(.mdc-switch__icons){width:var(--mat-switch-with-icon-handle-size);height:var(--mat-switch-with-icon-handle-size)}.mat-mdc-slide-toggle:active .mdc-switch:not(.mdc-switch--disabled) .mdc-switch__handle{width:var(--mat-switch-pressed-handle-size);height:var(--mat-switch-pressed-handle-size)}.mat-mdc-slide-toggle .mdc-switch--selected .mdc-switch__handle{margin:var(--mat-switch-selected-handle-horizontal-margin)}.mat-mdc-slide-toggle .mdc-switch--selected .mdc-switch__handle:has(.mdc-switch__icons){margin:var(--mat-switch-selected-with-icon-handle-horizontal-margin)}.mat-mdc-slide-toggle .mdc-switch--unselected .mdc-switch__handle{margin:var(--mat-switch-unselected-handle-horizontal-margin)}.mat-mdc-slide-toggle .mdc-switch--unselected .mdc-switch__handle:has(.mdc-switch__icons){margin:var(--mat-switch-unselected-with-icon-handle-horizontal-margin)}.mat-mdc-slide-toggle:active .mdc-switch--selected:not(.mdc-switch--disabled) .mdc-switch__handle{margin:var(--mat-switch-selected-pressed-handle-horizontal-margin)}.mat-mdc-slide-toggle:active .mdc-switch--unselected:not(.mdc-switch--disabled) .mdc-switch__handle{margin:var(--mat-switch-unselected-pressed-handle-horizontal-margin)}.mdc-switch__track::after,.mdc-switch__track::before{border-width:var(--mat-switch-track-outline-width);border-color:var(--mat-switch-track-outline-color)}.mdc-switch--selected .mdc-switch__track::after,.mdc-switch--selected .mdc-switch__track::before{border-width:var(--mat-switch-selected-track-outline-width);border-color:var(--mat-switch-selected-track-outline-color)}.mdc-switch--disabled .mdc-switch__track::after,.mdc-switch--disabled .mdc-switch__track::before{border-width:var(--mat-switch-disabled-unselected-track-outline-width);border-color:var(--mat-switch-disabled-unselected-track-outline-color)}.mdc-switch--disabled.mdc-switch--selected .mdc-switch__handle::after{opacity:var(--mat-switch-disabled-selected-handle-opacity)}.mdc-switch--disabled.mdc-switch--unselected .mdc-switch__handle::after{opacity:var(--mat-switch-disabled-unselected-handle-opacity)}\n'],
pb:2,Pa:0});var aE=class{};aE.K=function(a){return new (a||aE)};aE.Ya=Sf({type:aE});aE.Xa=Qd({imports:[XD,mz,mz]});var bE=class{};bE.K=function(a){return new (a||bE)};bE.Ya=Sf({type:bE});bE.Xa=Qd({Wa:[{pa:UD,nb:{color:"primary"}}],imports:[aE]});var cE=new gf("GMAT_SNACK_BAR_OPTIONS",{fa:"root",ba:()=>({tc:!1})});var dE=new gf("MatSnackBarData"),eE=class{constructor(){this.Tl="assertive";this.Xi="";this.duration=0;this.data=null;this.Sn="center";this.eF="bottom"}};var fE=class{};fE.K=function(a){return new (a||fE)};fE.Fa=Uf({type:fE,ha:[["","matSnackBarLabel",""]],Ka:[1,"mat-mdc-snack-bar-label","mdc-snackbar__label"],ka:!0});var gE=class{};gE.K=function(a){return new (a||gE)};gE.Fa=Uf({type:gE,ha:[["","matSnackBarActions",""]],Ka:[1,"mat-mdc-snack-bar-actions","mdc-snackbar__actions"],ka:!0});var hE=class{};hE.K=function(a){return new (a||hE)};hE.Fa=Uf({type:hE,ha:[["","matSnackBarAction",""]],Ka:[1,"mat-mdc-snack-bar-action","mdc-snackbar__action"],ka:!0});const iE=Math.pow(2,31)-1;function jE(a){a.C.closed||a.tg.Mq();clearTimeout(a.G)}var kE=class{constructor(a,b){this.la=b;this.C=new hj;this.nh=new hj;this.D=new hj;this.F=!1;this.tg=a;a.Lm.subscribe(()=>this.N())}J(a){this.G=setTimeout(()=>jE(this),Math.min(a,iE))}O(){this.nh.closed||(this.nh.next(),this.nh.complete())}N(){this.la.dispose();this.D.closed||this.D.complete();this.C.next({TF:this.F});this.C.complete();this.F=!1}rg(){return this.tg.ru}};function lE(a){if(a&1){const b=Zg();R(0,"div",1)(1,"button",2);V("click",function(){t(b);const c=Y();return x(c.action())});Z(2);U()()}a&2&&(a=Y(),D(2),Cr(" ",a.data.action," "))}var mE=class{constructor(a,b){this.jy=a;this.data=b}action(){var a=this.jy;a.D.closed||(a.F=!0,a.D.next(),a.D.complete(),jE(a));clearTimeout(a.G)}get nC(){return!!this.data.action}};mE.K=function(a){return new (a||mE)(y(kE),y(dE))};
mE.Ca=Jf({type:mE,ha:[["simple-snack-bar"]],Ka:[1,"mat-mdc-simple-snack-bar"],ib:["matSnackBar"],ka:!0,features:[Nr],ya:3,za:2,Ba:[["matSnackBarLabel",""],["matSnackBarActions",""],["mat-button","","matSnackBarAction","",3,"click"]],sa:function(a,b){a&1&&(R(0,"div",0),Z(1),U(),H(2,lE,3,1,"div",1));a&2&&(D(),Cr(" ",b.data.message,"\n"),D(),Gq(b.nC?2:-1))},Ha:[vB,fE,gE,hE],styles:[".mat-mdc-simple-snack-bar{display:flex}\n"],pb:2,Pa:0});var nE=Yy("state",[az("void, hidden",$y({transform:"scale(0.8)",opacity:0})),az("visible",$y({transform:"scale(1)",opacity:1})),bz("* => visible",Zy("150ms cubic-bezier(0, 0, 0.2, 1)")),bz("* => void, * => hidden",Zy("75ms cubic-bezier(0.4, 0.0, 1, 1)",$y({opacity:0})))]);const oE=["label"];function pE(){}let qE=0;function rE(a,b){const c=b.Bg;b=b.se;("void"===b&&"void"!==c||"hidden"===b)&&a.Ft();if("visible"===b){const d=a.ru;a.qa.run(()=>{d.next();d.complete()})}}
var sE=class extends Yv{constructor(a,b,c,d,e){super();this.qa=a;this.ra=b;this.Da=c;this.Ma=d;this.iy=e;this.ia=of(kt);this.Xp=new Set;this.Oz=150;this.Ga=!1;this.Mp=new hj;this.Lm=new hj;this.ru=new hj;this.kp="void";this.Jp=`mat-snack-bar-container-live-${qE++}`;this.Ni="assertive"!==e.Tl||e.Xi?"off"===e.Tl?"off":"polite":"assertive";this.Ma.N&&("polite"===this.Ni&&(this.yu="status"),"assertive"===this.Ni&&(this.yu="alert"))}ee(a){a=this.Ye.ee(a);this.xt();return a}df(a){a=this.Ye.df(a);this.xt();
return a}xn(){this.Ga||(this.kp="visible",en(this.Da),fn(this.Da),this.TA())}Mq(){this.qa.run(()=>{this.kp="hidden";en(this.Da);this.ra.R.setAttribute("mat-exit","");clearTimeout(this.zt)});return this.Lm}va(){this.Ga=!0;this.Sz();this.Ft()}Ft(){queueMicrotask(()=>{this.Lm.next();this.Lm.complete()})}xt(){const a=this.ra.R;var b=this.iy.hb;b&&(Array.isArray(b)?b.forEach(c=>a.classList.add(c)):a.classList.add(b));this.cA();b=this.uA.R;b.classList.toggle("mdc-snackbar__label",!b.querySelector(".mdc-snackbar__label"))}cA(){const a=
this.Jp,b=this.ia.querySelectorAll('body > .cdk-overlay-container [aria-modal="true"]');for(let c=0;c<b.length;c++){const d=b[c],e=d.getAttribute("aria-owns");this.Xp.add(d);e?-1===e.indexOf(a)&&d.setAttribute("aria-owns",e+" "+a):d.setAttribute("aria-owns",a)}}Sz(){this.Xp.forEach(a=>{var b=a.getAttribute("aria-owns");b&&(b=b.replace(this.Jp,"").trim(),0<b.length?a.setAttribute("aria-owns",b):a.removeAttribute("aria-owns"))});this.Xp.clear()}zF(){}TA(){this.zt||Qk(this.qa,()=>{this.zt=setTimeout(()=>
{const a=this.ra.R.querySelector("[aria-hidden]"),b=this.ra.R.querySelector("[aria-live]");if(a&&b){let c=null;this.Ma.isBrowser&&document.activeElement instanceof HTMLElement&&a.contains(document.activeElement)&&(c=document.activeElement);a.removeAttribute("aria-hidden");b.appendChild(a);let d;null==(d=c)||d.focus();this.Mp.next();this.Mp.complete()}},this.Oz)})}};sE.K=function(a){return new (a||sE)(y(Tk),y(wi),y(Tr),y(Wt),y(eE))};
sE.Ca=Jf({type:sE,ha:[["mat-snack-bar-container"]],eb:function(a,b){a&1&&(vr($v,7),vr(oE,7));if(a&2){let c;wr(c=xr())&&(b.Ye=c.first);wr(c=xr())&&(b.uA=c.first)}},Ka:[1,"mdc-snackbar","mat-mdc-snack-bar-container","mdc-snackbar--open"],Za:1,Ia:function(a,b){a&1&&Vq("@state.done",function(c){return rE(b,c)});a&2&&Tq("@state",b.kp)},ka:!0,features:[Ep,Nr],ya:6,za:3,Ba:[["label",""],[1,"mdc-snackbar__surface"],[1,"mat-mdc-snack-bar-label"],["aria-hidden","true"],["cdkPortalOutlet",""]],sa:function(a,
b){a&1&&(R(0,"div",1)(1,"div",2,0)(3,"div",3),H(4,pE,0,0,"ng-template",4),U(),Oq(5,"div"),U()());a&2&&(D(5),Mp("aria-live",b.Ni)("role",b.yu)("id",b.Jp))},Ha:[$v],styles:['.mdc-snackbar{display:none;position:fixed;right:0;bottom:0;left:0;align-items:center;justify-content:center;box-sizing:border-box;pointer-events:none;-webkit-tap-highlight-color:rgba(0,0,0,0)}.mdc-snackbar--opening,.mdc-snackbar--open,.mdc-snackbar--closing{display:flex}.mdc-snackbar--open .mdc-snackbar__label,.mdc-snackbar--open .mdc-snackbar__actions{visibility:visible}.mdc-snackbar__surface{padding-left:0;padding-right:8px;display:flex;align-items:center;justify-content:flex-start;box-sizing:border-box;transform:scale(0.8);opacity:0}.mdc-snackbar__surface::before{position:absolute;box-sizing:border-box;width:100%;height:100%;top:0;left:0;border:1px solid rgba(0,0,0,0);border-radius:inherit;content:"";pointer-events:none}@media screen and (forced-colors: active){.mdc-snackbar__surface::before{border-color:CanvasText}}[dir=rtl] .mdc-snackbar__surface,.mdc-snackbar__surface[dir=rtl]{padding-left:8px;padding-right:0}.mdc-snackbar--open .mdc-snackbar__surface{transform:scale(1);opacity:1;pointer-events:auto}.mdc-snackbar--closing .mdc-snackbar__surface{transform:scale(1)}.mdc-snackbar__label{padding-left:16px;padding-right:8px;width:100%;flex-grow:1;box-sizing:border-box;margin:0;visibility:hidden;padding-top:14px;padding-bottom:14px}[dir=rtl] .mdc-snackbar__label,.mdc-snackbar__label[dir=rtl]{padding-left:8px;padding-right:16px}.mdc-snackbar__label::before{display:inline;content:attr(data-mdc-snackbar-label-text)}.mdc-snackbar__actions{display:flex;flex-shrink:0;align-items:center;box-sizing:border-box;visibility:hidden}.mdc-snackbar__action+.mdc-snackbar__dismiss{margin-left:8px;margin-right:0}[dir=rtl] .mdc-snackbar__action+.mdc-snackbar__dismiss,.mdc-snackbar__action+.mdc-snackbar__dismiss[dir=rtl]{margin-left:0;margin-right:8px}.mat-mdc-snack-bar-container{margin:8px;position:static}.mat-mdc-snack-bar-container .mdc-snackbar__surface{min-width:344px}@media(max-width: 480px),(max-width: 344px){.mat-mdc-snack-bar-container .mdc-snackbar__surface{min-width:100%}}@media(max-width: 480px),(max-width: 344px){.mat-mdc-snack-bar-container{width:100vw}}.mat-mdc-snack-bar-container .mdc-snackbar__surface{max-width:672px}.mat-mdc-snack-bar-container .mdc-snackbar__surface{box-shadow:0px 3px 5px -1px rgba(0, 0, 0, 0.2), 0px 6px 10px 0px rgba(0, 0, 0, 0.14), 0px 1px 18px 0px rgba(0, 0, 0, 0.12)}.mat-mdc-snack-bar-container .mdc-snackbar__surface{background-color:var(--mdc-snackbar-container-color)}.mat-mdc-snack-bar-container .mdc-snackbar__surface{border-radius:var(--mdc-snackbar-container-shape)}.mat-mdc-snack-bar-container .mdc-snackbar__label{color:var(--mdc-snackbar-supporting-text-color)}.mat-mdc-snack-bar-container .mdc-snackbar__label{font-size:var(--mdc-snackbar-supporting-text-size);font-family:var(--mdc-snackbar-supporting-text-font);font-weight:var(--mdc-snackbar-supporting-text-weight);line-height:var(--mdc-snackbar-supporting-text-line-height)}.mat-mdc-snack-bar-container .mat-mdc-button.mat-mdc-snack-bar-action:not(:disabled){color:var(--mat-snack-bar-button-color);--mat-text-button-state-layer-color:currentColor;--mat-text-button-ripple-color:currentColor}.mat-mdc-snack-bar-container .mat-mdc-button.mat-mdc-snack-bar-action:not(:disabled) .mat-ripple-element{opacity:.1}.mat-mdc-snack-bar-container .mdc-snackbar__label::before{display:none}.mat-mdc-snack-bar-handset,.mat-mdc-snack-bar-container,.mat-mdc-snack-bar-label{flex:1 1 auto}.mat-mdc-snack-bar-handset .mdc-snackbar__surface{width:100%}\n'],
pb:2,data:{animation:[nE]}});/*

 Copyright Google LLC All Rights Reserved.

 Use of this source code is governed by an MIT-style license that can be
 found in the LICENSE file at https://angular.io/license
 @type {{XSmall: string, Small: string, Medium: string, Large: string, XLarge: string, Handset: string, Tablet: string, Web: string, HandsetPortrait: string, TabletPortrait: string, WebPortrait: string, HandsetLandscape: string, TabletLandscape: string, WebLandscape: string}}
*/
var tE=new gf("mat-snack-bar-default-options",{fa:"root",ba:function(){return new eE}});function uE(a,b){const c=new Bu;c.direction=b.direction;let d=Rv(a.G.position());var e="rtl"===b.direction;e="left"===b.Sn||"start"===b.Sn&&!e||"end"===b.Sn&&e;const f=!e&&"center"!==b.Sn;e?d.left("0"):f?d.right("0"):Ov(d);"top"===b.eF?d.top("0"):d.bottom("0");c.vc=d;return a.G.create(c)}
var vE=class{get C(){const a=this.F;return a?a.C:this.D}set C(a){this.F?this.F.C=a:this.D=a}constructor(a,b,c,d,e,f){this.G=a;this.Ni=b;this.Ra=c;this.T=d;this.F=e;this.J=f;this.D=null;this.aa=mE;this.ca=sE}open(a,b="",c){const d=Object.assign({},this.J,c);d.data={message:a,action:b};d.Xi===a&&(d.Xi=void 0);return this.O(this.aa,d)}va(){this.D&&jE(this.D)}P(a,b){const c=$h({parent:b&&b.yb&&b.yb.xa||this.Ra,Wa:[{pa:eE,nb:b}]});a=a.attach(new Vv(this.ca,b.yb,c));a.instance.iy=b;return a.instance}O(a,
b){const c=Object.assign({},new eE,this.J,b),d=uE(this,c);b=this.P(d,c);const e=new kE(b,d);if(a instanceof dr)e.instance=b.df(new Wv(a,null,{da:c.data,jy:e}));else{const f=$h({parent:c&&c.yb&&c.yb.xa||this.Ra,Wa:[{pa:kE,nb:e},{pa:dE,nb:c.data}]});a=b.ee(new Vv(a,void 0,f));e.instance=a.instance}C(this.T.observe("(max-width: 599.98px) and (orientation: portrait)"),Oo(d.J)).subscribe(f=>{d.N.classList.toggle("mat-mdc-snack-bar-handset",f.matches)});c.Xi&&b.Mp.subscribe(()=>{py(this.Ni,c.Xi,c.Tl)});
this.N(e,c);return this.C=e}N(a,b){a.C.subscribe(()=>{this.C==a&&(this.C=null);b.Xi&&this.Ni.clear()});this.C?(this.C.C.subscribe(()=>{a.tg.xn()}),jE(this.C)):a.tg.xn();b.duration&&0<b.duration&&a.rg().subscribe(()=>a.J(b.duration))}};vE.K=function(a){return new (a||vE)(r(cw),r(qy),r(ai),r(gy),r(vE,12),r(tE))};vE.na=Pd({ma:vE,ba:vE.K,fa:"root"});var wE=class{};wE.K=function(a){return new (a||wE)};wE.Ya=Sf({type:wE});wE.Xa=Qd({Wa:[vE],imports:[kw,aw,zB,mz,mE,mz]});var xE=class extends vE{constructor(a,b,c,d,e,f,g){g.tc||lw(a,"gmat-mdc-snack-bar");super(a,b,c,d,e,f)}};xE.K=function(a){return new (a||xE)(r(cw),r(qy),r(ai),r(gy),r(vE,12),r(tE),r(cE))};xE.na=Pd({ma:xE,ba:xE.K});var yE=class{};yE.K=function(a){return new (a||yE)};yE.Ya=Sf({type:yE});yE.Xa=Qd({Wa:[{pa:vE,Fc:xE},{pa:cw,Fc:mw}],imports:[wE]});var zE=new gf("GMAT_TOOLTIP_OPTIONS",{fa:"root",ba:()=>({tc:!1})});var AE=class{constructor(a,b){a.tc||lw(b,"gmat-mdc-tooltip")}};AE.K=function(a){return new (a||AE)(y(zE),y(cw))};AE.Fa=Uf({type:AE,ha:[["","matTooltip",""]],features:[Lr([{pa:cw,Fc:mw}])]});const BE=["tooltip"];var CE=new gf("mat-tooltip-scroll-strategy",{fa:"root",ba:()=>{const a=of(cw);return()=>a.C.C({Px:20})}}),DE={pa:CE,Bc:[cw],Gc:function(a){return()=>a.C.C({Px:20})}},EE=new gf("mat-tooltip-default-options",{fa:"root",ba:function(){return{dm:0,sl:0,JE:1500}}});const FE=Lt({passive:!0});function GE(a){a.la&&a.la.jc()&&a.la.detach();a.C=null}function HE(a,b=a.sl){const c=a.C;c&&(c.isVisible()?IE(c,b):(c.G(),GE(a)))}
function JE(a,b=a.dm,c){if(a.disabled||!a.message||a.G()){let d;null==(d=a.C)||d.G()}else c=KE(a,c),GE(a),a.ua=a.ua||new Vv(a.Hc,a.Ec),c=a.C=c.attach(a.ua).instance,c.Ui=a.ra.R,c.aa=a.J,C(c.P,Oo(a.Ga)).subscribe(()=>GE(a)),a.sb(a.ca),a.Ob(),LE(c,b)}
function KE(a,b){if(a.la){var c=a.la.Aa.vc;if((!a.ci||!b)&&c.sb instanceof wi)return a.la;GE(a)}c=ku(a.cc,a.ra);b=Ku(Mu(Nu(Su(Sv(a.ta.position(),a.ci?b||a.ra:a.ra),".mat-mdc-tooltip"),!1),a.wa),c);C(b.gb,Oo(a.Ga)).subscribe(e=>{a.Ic(e.C);a.C&&e.D.rr&&a.C.isVisible()&&a.qa.run(()=>HE(a,0))});a.la=a.ta.create({direction:a.D,vc:b,hb:"mat-mdc-tooltip-panel",wc:a.Kc()});a.ja(a.la);C(a.la.J,Oo(a.Ga)).subscribe(()=>GE(a));C(a.la.Pl(),Oo(a.Ga)).subscribe(()=>{let e;return null==(e=a.C)?void 0:e.ea()});C(a.la.gf(),
Oo(a.Ga)).subscribe(e=>{a.G()&&27===e.keyCode&&!Cu(e)&&(e.preventDefault(),e.stopPropagation(),a.qa.run(()=>HE(a,0)))});let d;(null==(d=a.O)?0:d.SF)&&a.la.qg("mat-mdc-tooltip-panel-non-interactive");return a.la}
var NE=class{get position(){return this.P}set position(a){if(a!==this.P&&(this.P=a,this.la)){this.ja(this.la);let b;null==(b=this.C)||LE(b,0);this.la.Wc()}}get ci(){return this.lb}set ci(a){this.lb=Rs(a);GE(this);this.la=null}get disabled(){return this.Ib}set disabled(a){(this.Ib=Rs(a))?HE(this,0):this.aa()}get dm(){return this.Lb}set dm(a){this.Lb=Us(a)}get sl(){return this.J}set sl(a){this.J=Us(a);this.C&&(this.C.aa=this.J)}get message(){return this.N}set message(a){zy(this.T,this.ra.R,this.N);
this.N=null!=a?String(a).trim():"";!this.N&&this.G()?HE(this,0):(this.aa(),this.Ob(),Qk(this.qa,()=>{Promise.resolve().then(()=>{wy(this.T,this.ra.R,this.message,"tooltip")})}))}get Qo(){return this.ca}set Qo(a){this.ca=a;this.C&&this.sb(this.ca)}constructor(a,b,c,d,e,f,g,h,m,p,u,w){this.ta=a;this.ra=b;this.cc=c;this.Ec=d;this.qa=e;this.Ma=f;this.T=g;this.oa=h;this.D=p;this.O=u;this.P="below";this.gb=this.Pb=this.Ib=this.lb=!1;this.Hc=ME;this.wa=8;this.Qc="mat-mdc";this.ri="auto";this.N="";this.F=
[];this.Ga=new hj;this.Ra=of(ai);this.Kc=m;this.ia=w;u&&(this.Lb=u.dm,this.J=u.sl,u.position&&(this.position=u.position),u.ci&&(this.ci=u.ci),u.ri&&(this.ri=u.ri));C(p.xb,Oo(this.Ga)).subscribe(()=>{this.la&&this.ja(this.la)});this.wa=8}pd(){this.Pb=!0;this.aa();C(Mx(this.oa,this.ra),Oo(this.Ga)).subscribe(a=>{a?"keyboard"===a&&this.qa.run(()=>JE(this)):this.qa.run(()=>HE(this,0))})}va(){const a=this.ra.R;clearTimeout(this.ea);this.la&&(this.la.dispose(),this.C=null);this.F.forEach(([b,c])=>{a.removeEventListener(b,
c,FE)});this.F.length=0;this.Ga.next();this.Ga.complete();zy(this.T,a,this.message);Nx(this.oa,a)}toggle(a){this.G()?HE(this):JE(this,void 0,a)}G(){return!!this.C&&this.C.isVisible()}ja(a){a=a.Aa.vc;var b=!this.D||"ltr"==this.D.value;var c=this.position;let d;if("above"==c||"below"==c)d={bb:"center",cb:"above"==c?"top":"bottom"};else if("before"==c||"left"==c&&b||"right"==c&&!b)d={bb:"start",cb:"center"};else if("after"==c||"right"==c&&b||"left"==c&&!b)d={bb:"end",cb:"center"};const {x:e,y:f}=this.Va(d.bb,
d.cb);b={bb:e,cb:f};c=!this.D||"ltr"==this.D.value;const g=this.position;let h;if("above"==g)h={Na:"center",Ua:"bottom"};else if("below"==g)h={Na:"center",Ua:"top"};else if("before"==g||"left"==g&&c||"right"==g&&!c)h={Na:"end",Ua:"center"};else if("after"==g||"right"==g&&c||"left"==g&&!c)h={Na:"start",Ua:"center"};const {x:m,y:p}=this.Va(h.Na,h.Ua);c={Na:m,Ua:p};Lu(a,[this.Ja(Object.assign({},d,h)),this.Ja(Object.assign({},b,c))])}Ja(a){const b=!this.D||"ltr"==this.D.value;"top"===a.cb?a.offsetY=
-8:"bottom"===a.cb?a.offsetY=8:"start"===a.bb?a.offsetX=b?-8:8:"end"===a.bb&&(a.offsetX=b?8:-8);return a}Ob(){this.C&&(this.C.message=this.message,this.C.O(),el(()=>{this.C&&this.la.Wc()},{xa:this.Ra}))}sb(a){this.C&&(this.C.Qo=a,this.C.O())}Va(a,b){"above"===this.position||"below"===this.position?"top"===b?b="bottom":"bottom"===b&&(b="top"):"end"===a?a="start":"start"===a&&(a="end");return{x:a,y:b}}Ic(a){var b=a.Ua,c=a.bb;a=a.cb;b="center"===b?this.D&&"rtl"===this.D.value?"end"===c?"left":"right":
"start"===c?"left":"right":"bottom"===b&&"top"===a?"above":"below";if(b!==this.Qa){if(c=this.la)c.Oj("mat-mdc-tooltip-panel-"+this.Qa),c.qg("mat-mdc-tooltip-panel-"+b);this.Qa=b}}aa(){!this.Ib&&this.message&&this.Pb&&!this.F.length&&(this.fb()?this.F.push(["mouseenter",a=>{this.Kb();let b=void 0;void 0!==a.x&&void 0!==a.y&&(b=a);JE(this,void 0,b)}]):"off"!==this.ri&&(this.Ta(),this.F.push(["touchstart",a=>{let b;const c=(a=null==(b=a.targetTouches)?void 0:b[0])?{x:a.clientX,y:a.clientY}:void 0;this.Kb();
clearTimeout(this.ea);let d;this.ea=setTimeout(()=>JE(this,void 0,c),null!=(d=this.O.NG)?d:500)}])),this.Ea(this.F))}Kb(){if(!this.gb){this.gb=!0;var a=[];if(this.fb())a.push(["mouseleave",b=>{b=b.relatedTarget;let c;b&&(null==(c=this.la)?0:c.N.contains(b))||HE(this)}],["wheel",b=>this.Jc(b)]);else if("off"!==this.ri){this.Ta();const b=()=>{clearTimeout(this.ea);HE(this,this.O.JE)};a.push(["touchend",b],["touchcancel",b])}this.Ea(a);this.F.push(...a)}}Ea(a){a.forEach(([b,c])=>{this.ra.R.addEventListener(b,
c,FE)})}fb(){return!this.Ma.C&&!this.Ma.P}Jc(a){if(this.G()){a=this.ia.elementFromPoint(a.clientX,a.clientY);const b=this.ra.R;a===b||b.contains(a)||HE(this)}}Ta(){const a=this.ri;if("off"!==a){const b=this.ra.R,c=b.style;if("on"===a||"INPUT"!==b.nodeName&&"TEXTAREA"!==b.nodeName)c.userSelect=c.msUserSelect=c.webkitUserSelect=c.MozUserSelect="none";"on"!==a&&b.draggable||(c.webkitUserDrag="none");c.touchAction="none";c.webkitTapHighlightColor="transparent"}}};
NE.K=function(a){return new (a||NE)(y(cw),y(wi),y(lu),y(np),y(Tk),y(Wt),y(Ay),y(Px),y(CE),y(pu),y(EE,8),y(kt))};
NE.Fa=Uf({type:NE,ha:[["","matTooltip",""]],Ka:[1,"mat-mdc-tooltip-trigger"],Za:2,Ia:function(a,b){a&2&&Wp("mat-mdc-tooltip-disabled",b.disabled)},inputs:{position:[0,"matTooltipPosition","position"],ci:[0,"matTooltipPositionAtOrigin","positionAtOrigin"],disabled:[0,"matTooltipDisabled","disabled"],dm:[0,"matTooltipShowDelay","showDelay"],sl:[0,"matTooltipHideDelay","hideDelay"],ri:[0,"matTooltipTouchGestures","touchGestures"],message:[0,"matTooltip","message"],Qo:[0,"matTooltipClass","tooltipClass"]},
ib:["matTooltip"],ka:!0});function LE(a,b){null!=a.C&&clearTimeout(a.C);a.D=setTimeout(()=>{a.ca(!0);a.D=void 0},b)}function IE(a,b){null!=a.D&&clearTimeout(a.D);a.C=setTimeout(()=>{a.ca(!1);a.C=void 0},b)}
var ME=class{constructor(a,b,c){this.Da=a;this.ra=b;this.N=this.T=this.nu=!1;this.P=new hj;this.ua="mat-mdc-tooltip-show";this.ta="mat-mdc-tooltip-hide";this.F="NoopAnimations"===c}isVisible(){return this.N}va(){this.G();this.P.complete();this.Ui=null}ea(){this.T&&IE(this,0)}O(){en(this.Da)}pA({relatedTarget:a}){a&&this.Ui.contains(a)||(this.isVisible()?IE(this,this.aa):this.J(!1))}oa(){this.nu=this.ja();this.O()}ja(){const a=this.ra.R.getBoundingClientRect();return 24<a.height&&200<=a.width}lA({animationName:a}){"mat-mdc-tooltip-show"!==
a&&"mat-mdc-tooltip-hide"!==a||this.J("mat-mdc-tooltip-show"===a)}G(){null!=this.D&&clearTimeout(this.D);null!=this.C&&clearTimeout(this.C);this.D=this.C=void 0}J(a){a?this.T=!0:this.isVisible()||this.P.next()}ca(a){const b=this.dB.R;b.classList.remove(a?"mat-mdc-tooltip-hide":"mat-mdc-tooltip-show");b.classList.add(a?"mat-mdc-tooltip-show":"mat-mdc-tooltip-hide");this.N!==a&&(this.N=a,en(this.Da));if(a&&!this.F&&"function"===typeof getComputedStyle){const c=getComputedStyle(b);if("0s"===c.getPropertyValue("animation-duration")||
"none"===c.getPropertyValue("animation-name"))this.F=!0}a&&this.oa();this.F&&(b.classList.add("_mat-animation-noopable"),this.J(a))}};ME.K=function(a){return new (a||ME)(y(Tr),y(wi),y(qi,8))};
ME.Ca=Jf({type:ME,ha:[["mat-tooltip-component"]],eb:function(a,b){a&1&&vr(BE,7);if(a&2){let c;wr(c=xr())&&(b.dB=c.first)}},Ka:["aria-hidden","true"],Za:2,Ia:function(a,b){a&1&&V("mouseleave",function(c){return b.pA(c)});a&2&&Up("zoom",b.isVisible()?1:null)},ka:!0,features:[Nr],ya:4,za:4,Ba:[["tooltip",""],[1,"mdc-tooltip","mdc-tooltip--shown","mat-mdc-tooltip",3,"animationend","ngClass"],[1,"mdc-tooltip__surface","mdc-tooltip__surface-animation"]],sa:function(a,b){if(a&1){const c=Zg();R(0,"div",1,
0);V("animationend",function(d){t(c);return x(b.lA(d))});R(2,"div",2);Z(3);U()()}a&2&&(Wp("mdc-tooltip--multiline",b.nu),Q("ngClass",b.Qo),D(3),Br(b.message))},Ha:[Xs],styles:['.mdc-tooltip__surface{word-break:break-all;word-break:var(--mdc-tooltip-word-break, normal);overflow-wrap:anywhere}.mdc-tooltip--showing-transition .mdc-tooltip__surface-animation{transition:opacity 150ms 0ms cubic-bezier(0, 0, 0.2, 1),transform 150ms 0ms cubic-bezier(0, 0, 0.2, 1)}.mdc-tooltip--hide-transition .mdc-tooltip__surface-animation{transition:opacity 75ms 0ms cubic-bezier(0.4, 0, 1, 1)}.mdc-tooltip{position:fixed;display:none;z-index:9}.mdc-tooltip-wrapper--rich{position:relative}.mdc-tooltip--shown,.mdc-tooltip--showing,.mdc-tooltip--hide{display:inline-flex}.mdc-tooltip--shown.mdc-tooltip--rich,.mdc-tooltip--showing.mdc-tooltip--rich,.mdc-tooltip--hide.mdc-tooltip--rich{display:inline-block;left:-320px;position:absolute}.mdc-tooltip__surface{line-height:16px;padding:4px 8px;min-width:40px;max-width:200px;min-height:24px;max-height:40vh;box-sizing:border-box;overflow:hidden;text-align:center}.mdc-tooltip__surface::before{position:absolute;box-sizing:border-box;width:100%;height:100%;top:0;left:0;border:1px solid rgba(0,0,0,0);border-radius:inherit;content:"";pointer-events:none}@media screen and (forced-colors: active){.mdc-tooltip__surface::before{border-color:CanvasText}}.mdc-tooltip--rich .mdc-tooltip__surface{align-items:flex-start;display:flex;flex-direction:column;min-height:24px;min-width:40px;max-width:320px;position:relative}.mdc-tooltip--multiline .mdc-tooltip__surface{text-align:left}[dir=rtl] .mdc-tooltip--multiline .mdc-tooltip__surface,.mdc-tooltip--multiline .mdc-tooltip__surface[dir=rtl]{text-align:right}.mdc-tooltip__surface .mdc-tooltip__title{margin:0 8px}.mdc-tooltip__surface .mdc-tooltip__content{max-width:calc(200px - 2*8px);margin:8px;text-align:left}[dir=rtl] .mdc-tooltip__surface .mdc-tooltip__content,.mdc-tooltip__surface .mdc-tooltip__content[dir=rtl]{text-align:right}.mdc-tooltip--rich .mdc-tooltip__surface .mdc-tooltip__content{max-width:calc(320px - 2*8px);align-self:stretch}.mdc-tooltip__surface .mdc-tooltip__content-link{text-decoration:none}.mdc-tooltip--rich-actions,.mdc-tooltip__content,.mdc-tooltip__title{z-index:1}.mdc-tooltip__surface-animation{opacity:0;transform:scale(0.8);will-change:transform,opacity}.mdc-tooltip--shown .mdc-tooltip__surface-animation{transform:scale(1);opacity:1}.mdc-tooltip--hide .mdc-tooltip__surface-animation{transform:scale(1)}.mdc-tooltip__caret-surface-top,.mdc-tooltip__caret-surface-bottom{position:absolute;height:24px;width:24px;transform:rotate(35deg) skewY(20deg) scaleX(0.9396926208)}.mdc-tooltip__caret-surface-top .mdc-elevation-overlay,.mdc-tooltip__caret-surface-bottom .mdc-elevation-overlay{width:100%;height:100%;top:0;left:0}.mdc-tooltip__caret-surface-bottom{box-shadow:0px 3px 1px -2px rgba(0, 0, 0, 0.2), 0px 2px 2px 0px rgba(0, 0, 0, 0.14), 0px 1px 5px 0px rgba(0, 0, 0, 0.12);outline:1px solid rgba(0,0,0,0);z-index:-1}@media screen and (forced-colors: active){.mdc-tooltip__caret-surface-bottom{outline-color:CanvasText}}.mat-mdc-tooltip .mdc-tooltip__surface{background-color:var(--mdc-plain-tooltip-container-color)}.mat-mdc-tooltip .mdc-tooltip__surface{border-radius:var(--mdc-plain-tooltip-container-shape)}.mat-mdc-tooltip .mdc-tooltip__caret-surface-top,.mat-mdc-tooltip .mdc-tooltip__caret-surface-bottom{border-radius:var(--mdc-plain-tooltip-container-shape)}.mat-mdc-tooltip .mdc-tooltip__surface{color:var(--mdc-plain-tooltip-supporting-text-color)}.mat-mdc-tooltip .mdc-tooltip__surface{font-family:var(--mdc-plain-tooltip-supporting-text-font);line-height:var(--mdc-plain-tooltip-supporting-text-line-height);font-size:var(--mdc-plain-tooltip-supporting-text-size);font-weight:var(--mdc-plain-tooltip-supporting-text-weight);letter-spacing:var(--mdc-plain-tooltip-supporting-text-tracking)}.mat-mdc-tooltip{position:relative;transform:scale(0)}.mat-mdc-tooltip::before{content:"";top:0;right:0;bottom:0;left:0;z-index:-1;position:absolute}.mat-mdc-tooltip-panel-below .mat-mdc-tooltip::before{top:-8px}.mat-mdc-tooltip-panel-above .mat-mdc-tooltip::before{bottom:-8px}.mat-mdc-tooltip-panel-right .mat-mdc-tooltip::before{left:-8px}.mat-mdc-tooltip-panel-left .mat-mdc-tooltip::before{right:-8px}.mat-mdc-tooltip._mat-animation-noopable{animation:none;transform:scale(1)}.mat-mdc-tooltip-panel.mat-mdc-tooltip-panel-non-interactive{pointer-events:none}@keyframes mat-mdc-tooltip-show{0%{opacity:0;transform:scale(0.8)}100%{opacity:1;transform:scale(1)}}@keyframes mat-mdc-tooltip-hide{0%{opacity:1;transform:scale(1)}100%{opacity:0;transform:scale(0.8)}}.mat-mdc-tooltip-show{animation:mat-mdc-tooltip-show 150ms cubic-bezier(0, 0, 0.2, 1) forwards}.mat-mdc-tooltip-hide{animation:mat-mdc-tooltip-hide 75ms cubic-bezier(0.4, 0, 1, 1) forwards}\n'],
pb:2,Pa:0});var OE=class{};OE.K=function(a){return new (a||OE)};OE.Ya=Sf({type:OE});OE.Xa=Qd({Wa:[DE],imports:[ry,it,kw,mz,mz,yu]});var PE=class{};PE.K=function(a){return new (a||PE)};PE.Ya=Sf({type:PE});PE.Xa=Qd({imports:[OE]});const iG=$y({opacity:0}),jG=$y({opacity:1});var kG=Yy("dialogContainer",[az("void, hidden",iG),az("visible",jG),bz("void => *, * => visible",Zy("200ms cubic-bezier(0.0,0.0,0.2,1)",jG)),bz("* => void, * => hidden",Zy("200ms cubic-bezier(0.4,0.0,0.2,1)",iG))]);function lG(a){a&1&&Oq(0,"a",1)}var mG=new gf("XAP_IDCO");function nG(a,b){a.Vu=b?"visible":"hidden";en(a.C)}var oG=class extends Yv{constructor(){super(...arguments);this.Vu="hidden";this.options=of(mG);this.C=of(Tr)}pd(){}ee(a){return this.tx.ee(a)}df(a){return this.tx.df(a)}};oG.K=(()=>{let a;return function(b){return(a||(a=Lh(oG)))(b||oG)}})();
oG.Ca=Jf({type:oG,ha:[["xap-inline-dialog-container"]],eb:function(a,b){a&1&&vr($v,7);if(a&2){let c;wr(c=xr())&&(b.tx=c.first)}},Ka:["role","dialog",1,"xap-inline-dialog-container","gmat-body-2"],Za:3,Ia:function(a,b){a&2&&(Tq("@dialogContainer",b.Vu),Mp("aria-label",b.options.qn)("tabindex",b.options.rn))},ka:!0,features:[Ep,Nr],ya:2,za:0,Ba:[["cdkPortalOutlet",""],["cdkFocusInitial","","tabindex","0"]],sa:function(a){a&1&&(R(0,"div"),H(1,lG,1,0,"ng-template",0),U())},Ha:[ry,aw,$v],styles:[".xap-inline-dialog-container{box-shadow:0 1px 2px 0 rgba(60,64,67,.3),0 1px 3px 1px rgba(60,64,67,.15);border-radius:8px;overflow:auto}"],
pb:2,data:{animation:[kG]},Pa:0});var pG=[{bb:"start",cb:"bottom",Na:"start",Ua:"top",offsetY:8},{bb:"start",cb:"top",Na:"start",Ua:"bottom",offsetY:-8},{bb:"end",cb:"top",Na:"end",Ua:"bottom",offsetY:-8},{bb:"end",cb:"bottom",Na:"end",Ua:"top",offsetY:8}],qG={minWidth:220,maxWidth:420,minHeight:64,maxHeight:420};
function rG(a){let b;null==(b=a.Xb)||!b.jc()||a.Yj&&!a.Yj.C(a.ua)||(nG(a.G.instance,!1),setTimeout(()=>{a.F.run(()=>{a.Xb&&a.Xb.detach();a.J&&(a.J.destroy(),a.J=void 0);a.Pg();a.G&&(a.G.destroy(),a.G=void 0);a.closed.emit()})},200))}function sG(a,b){Qk(a.F,()=>{C(qk(b,"keydown"),Oo(a.C)).subscribe(c=>{switch(c.keyCode){case 13:tG(a);c.preventDefault();break;case 32:c.preventDefault()}});C(qk(b,"keyup"),Oo(a.C)).subscribe(({keyCode:c})=>{switch(c){case 32:tG(a)}})})}
function uG(a,b){Qk(a.F,()=>{C(qk(b,"focus"),xk(()=>0<a.Bh.nc.length),Oo(a.C)).subscribe(()=>{a.F.run(()=>{a.Bh.emit()})})})}function tG(a){if(!a.disabled){var b;if(null==(b=a.Xb)||!b.jc()){var c=a.Od;null!=c&&(a.Yj&&(a.ua=a.Yj.register()),a.F.run(()=>{a.G=vG(a);a.G.instance.attach(c);const d=a.G.location.R;a.P(d);a.aa(d);wG(a,d);nG(a.G.instance,!0);a.kf.nc.length&&setTimeout(()=>{a.kf.emit()},200)}))}}}
function vG(a){var b=$h({parent:a.xa,Wa:[{pa:mG,nb:{qn:a.qn,rn:a.rn}}]});b=new Vv(oG,null,b);null==a.Xb&&(a.Xb=a.T.create(a.ja(pG)));a.mx&&a.Xb.te(a.mx);if(a.Sr&&0<a.Sr.length){var c=a.Xb,d=xG(a,a.Sr);d!==c.F&&(c.F&&c.F.dispose(),c.F=d,c.jc()&&(d.attach(c),c.Wc()))}return a.Xb.attach(b)}function wG(a,b){const c=a.document.activeElement,d=a.Ub.R;if(a.gw||c&&c===d)a.document&&(a.ea=a.document.activeElement),a.J=a.Ea.create(b),Wx(a.J),a.Dq||yG(a)}function yG(a){a.J&&Xx(a.J).then(()=>{})}
function xG(a,b){let c,d;return Iu(Lu(Sv(a.T.position(),null!=(c=a.Yu)?c:a.Ub),b),null!=(d=a.Yu)?d:a.Ub)}function zG(a){const b=a.T.C;switch(a.wc){case "noop":return b.G();case "block":return b.block();case "reposition":return b.C();default:return b.close()}}
var AG=class{set Ih(a){a&&(this.Od=a instanceof dr?new Wv(a,this.yb):new Vv(a,this.yb))}set disabled(a){(this.ca=a)&&rG(this)}get disabled(){return this.ca}set Dq(a){this.oa=Rs(a)}get Dq(){return this.oa}set hb(a){if(this.N!==a){if(this.N){let b;null==(b=this.Xb)||b.Oj(this.N)}if(a){let b;null==(b=this.Xb)||b.qg(a)}this.N=a}}constructor(a,b,c,d,e,f,g,h){this.F=a;this.T=b;this.Ub=c;this.yb=d;this.document=e;this.Ea=f;this.wa=g;this.xa=h;this.ca=!1;this.qn=null;this.rn=-1;this.Rr=qG;this.Gy="Press space for more information.";
this.gw=this.oa=!1;this.wc="close";this.ua=0;this.Bh=new Jk;this.kf=new Jk;this.closed=new Jk;this.C=new rj;this.D=new jj(!1);this.ta=C(this.D,xk(m=>m&&!this.disabled));this.ea=void 0;sG(this,c.R);this.aa(c.R);uG(this,c.R)}Fb(){var a=this.Ub.R;a.hasAttribute("aria-haspopup")||a.setAttribute("aria-haspopup","dialog");a.hasAttribute("aria-describedby")||wy(this.wa,a,this.Gy)}va(){rG(this);this.D.complete();this.C.next();this.C.complete();this.Xb&&this.Xb.dispose()}Pg(){const a=this.ea;if(a&&"function"===
typeof a.focus){var b,c=null==(b=this.G)?void 0:b.location.R;(b=this.document.activeElement)&&null!=c&&c.contains(b)&&(a.focus(),this.ea=void 0)}}};AG.K=function(a){return new (a||AG)(y(Tk),y(cw),y(wi),y(np),y(Document),y(Iy),y(Ay),y(ai))};
AG.Fa=Uf({type:AG,Za:1,Ia:function(a,b){a&2&&Mp("data-disabled",b.disabled)},inputs:{Yu:"attachedTo",qn:"dialogLabel",rn:"dialogTabIndex",mx:"overlaySize",Sr:"overlayPositions",Rr:"overlayDimensions",Gy:"triggerDescription",Dq:"disableAutoFocus",gw:"forceTrapFocus",hb:"panelClass",wc:"scrollStrategy",Yj:"stackManager"},outputs:{Bh:"beforeOpened",kf:"opened",closed:"closed"},ka:!0});function BG(a,b){const c=C(a.D,Kn(()=>C(a.O,Rn(b))));C(c,Oo(a.C)).subscribe(d=>{d?tG(a):rG(a)});C(a.ta,xk(d=>d&&0<a.Bh.nc.length),Wo(()=>C(c,xk(d=>!d))),Oo(a.C)).subscribe(()=>{a.F.run(()=>{a.Bh.emit()})})}function CG(a){a.disabled||(a.D.next(!0),a.O.next())}function DG(a){a.D.next(!1);a.O.next()}
var EG=class extends AG{constructor(a,b,c,d,e,f,g,h){super(a,b,c,d,e,f,g,h);this.dr=500;this.O=new hj;this.P(c.R)}Fb(){super.Fb();BG(this,this.dr)}P(a){this.Yj&&C(this.Yj.D(),Rn(this.dr),Oo(this.C)).subscribe(()=>{a.matches(":hover")||this.yb.element.R.matches(":hover")?CG(this):DG(this)});Qk(this.F,()=>{C(qk(a,"mouseenter"),Oo(this.C)).subscribe(()=>{CG(this)});C(qk(a,"click"),Oo(this.C)).subscribe(b=>{b.target.closest("[xapInlineDialogClose]")?DG(this):(CG(this),tG(this))});C(qk(a,"mouseleave"),
Oo(this.C)).subscribe(()=>{DG(this)});C(qk(a,"mousemove"),Oo(this.C)).subscribe(()=>{this.O.next()})})}aa(a){Qk(this.F,()=>{C(qk(a,"keydown"),Oo(this.C)).subscribe(b=>{switch(b.keyCode){case 27:let c;(null==(c=this.Xb)?0:c.jc())&&b.stopPropagation();rG(this)}})})}ja(a){return new Bu(Object.assign({},this.Rr,{vc:xG(this,a),wc:zG(this),hb:this.N}))}};EG.K=function(a){return new (a||EG)(y(Tk),y(cw),y(wi),y(np),y(kt),y(Iy),y(Ay),y(ai))};
EG.Fa=Uf({type:EG,ha:[["","xapInlineDialog",""]],Ka:[1,"xap-inline-dialog"],inputs:{Ih:[0,"xapInlineDialog","dialog"],disabled:[0,"xapInlineDialogDisabled","disabled"],dr:"hoverDelayMs"},ib:["xapInlineDialog"],ka:!0,features:[Ep]});function FG(a){C(a.D,Oo(a.C)).subscribe(b=>{b?tG(a):rG(a)});C(a.ta,xk(b=>b&&0<a.Bh.nc.length),Wo(()=>C(a.D,xk(b=>!b))))}
var GG=class extends AG{constructor(a,b,c,d,e,f,g,h){super(a,b,c,d,e,f,g,h);this.P(c.R)}Fb(){super.Fb();FG(this)}P(a){Qk(this.F,()=>{C(qk(a,"mouseenter"),Oo(this.C)).subscribe(()=>{this.F.run(()=>{this.Bh.emit()})});C(qk(a,"click"),Oo(this.C)).subscribe(c=>{c.target.closest("[xapInlineDialogClose]")?this.D.next(!1):this.disabled||this.D.next(!0)});let b;null==(b=this.Xb)||C(b.bd(),Oo(this.C)).subscribe(()=>{this.D.next(!1)})})}aa(a){Qk(this.F,()=>{C(qk(a,"keyup"),Oo(this.C)).subscribe(b=>{const c=
b.target;switch(b.keyCode){case 27:this.D.next(!1);break;case 32:c.closest("[xapInlineDialogClose]")&&this.D.next(!1)}});C(qk(a,"keydown"),Oo(this.C)).subscribe(b=>{const c=b.target;switch(b.keyCode){case 13:c.closest("[xapInlineDialogClose]")&&this.D.next(!1)}})})}ja(a){return new Bu(Object.assign({},this.Rr,{vc:xG(this,a),wc:zG(this),Vb:!0,fc:"cdk-overlay-transparent-backdrop",hb:this.N}))}};GG.K=function(a){return new (a||GG)(y(Tk),y(cw),y(wi),y(np),y(kt),y(Iy),y(Ay),y(ai))};
GG.Fa=Uf({type:GG,ha:[["","xapInlineDialogClick",""]],Ka:[1,"xap-inline-dialog-click"],inputs:{Ih:[0,"xapInlineDialogClick","dialog"],disabled:[0,"xapInlineDialogDisabled","disabled"]},ib:["xapInlineDialogClick"],ka:!0,features:[Ep]});var HG=class{};HG.K=function(a){return new (a||HG)};HG.Ya=Sf({type:HG});HG.Xa=Qd({});var IG=class extends mt{constructor(){super(...arguments)}};function qt(a,b){return"window"===b?window:"document"===b?a:"body"===b?a.body:null}function JG(a,b,c){a.addEventListener(b,c);return()=>{a.removeEventListener(b,c)}}var KG=class extends IG{dispatchEvent(a,b){a.dispatchEvent(b)}remove(a){a.parentNode&&a.parentNode.removeChild(a)}createElement(a,b){b=b||document;return b.createElement(a)}};let zt=null;function Hn(a){hf.getAngularTestability=(b,c=!0)=>{b=a.kl(b,c);if(null==b)throw new ae(5103,!1);return b};hf.getAllAngularTestabilities=()=>Array.from(a.C.values());hf.getAllAngularRootElements=()=>Array.from(a.C.keys());hf.frameworkStabilizers||(hf.frameworkStabilizers=[]);hf.frameworkStabilizers.push(b=>{function c(){e--;0==e&&b()}const d=hf.getAllAngularTestabilities();let e=d.length;d.forEach(f=>{f.whenStable(c)})})}
var LG=class{kl(a,b,c){if(null==b)return null;const d=a.C.get(b)||null;if(null!=d)a=d;else{var e;c?e=b instanceof DocumentFragment?this.kl(a,b.host,!0):this.kl(a,b.parentElement,!0):e=null;a=e}return a}};var MG=class{D(){return new XMLHttpRequest}};MG.K=function(a){return new (a||MG)};MG.na=Pd({ma:MG,ba:MG.K});var NG=new gf(""),OG=class{constructor(a,b){this.zb=b;this.C=new Map;a.forEach(c=>{c.manager=this});this.F=a.slice().reverse()}addEventListener(a,b,c){return this.D(b).addEventListener(a,b,c)}D(a){let b=this.C.get(a);if(b)return b;b=this.F.find(c=>c.supports(a));if(!b)throw new ae(5101,!1);this.C.set(a,b);return b}};OG.K=function(a){return new (a||OG)(r(NG),r(Tk))};OG.na=Pd({ma:OG,ba:OG.K});var PG=class{};function QG(a){let b;const c=null==(b=a.F.head)?void 0:b.querySelectorAll(`style[${"ng-app-id"}="${a.G}"]`);if(null==c?0:c.length){const d=new Map;c.forEach(e=>{null!=e.textContent&&d.set(e.textContent,e)});return d}return null}function RG(a){const b=a.D;b.clear();b.add(a.F.head)}function SG(a,b,c){a=a.C;if(a.has(b))return b=a.get(b),b.usage+=c,b.usage;a.set(b,{usage:c,elements:[]});return c}function TG(a,b){for(const c of b)if(1===SG(a,c,1)){b=a;for(const d of b.D)UG(b,d,c)}}
function VG(a,b){a=a.C;let c,d;null==(c=a.get(b))||null==(d=c.elements)||d.forEach(e=>e.remove());a.delete(b)}function UG(a,b,c){{var d=a.J;const f=null==d?void 0:d.get(c);(null==f?void 0:f.parentNode)===b?(d.delete(c),f.removeAttribute("ng-app-id"),b=f):(d=a.F.createElement("style"),a.nonce&&d.setAttribute("nonce",a.nonce),d.textContent=c,a.N&&d.setAttribute("ng-app-id",a.G),b.appendChild(d),b=d)}a=a.C;let e;(d=null==(e=a.get(c))?void 0:e.elements)?d.push(b):a.set(c,{elements:[b],usage:1})}
var WG=class{constructor(a,b,c,d={}){this.F=a;this.G=b;this.nonce=c;this.C=new Map;this.D=new Set;this.J=QG(this);this.N="server"===d;RG(this)}va(){const a=this.J;a&&(a.forEach(b=>b.remove()),a.clear());for(const b of this.C.keys())VG(this,b);RG(this)}};WG.K=function(a){return new (a||WG)(r(kt),r(ni),r(ri,8),r(pi))};WG.na=Pd({ma:WG,ba:WG.K});var XG={svg:"http://www.w3.org/2000/svg",xhtml:"http://www.w3.org/1999/xhtml",xlink:"http://www.w3.org/1999/xlink",xml:"http://www.w3.org/XML/1998/namespace",xmlns:"http://www.w3.org/2000/xmlns/",math:"http://www.w3.org/1998/Math/MathML"};const YG=/%COMP%/g;var ZG=new gf("",{fa:"root",ba:()=>!0});function $G(a,b){return b.map(c=>c.replace(YG,a))}
var eH=class{constructor(a,b,c,d,e,f,g,h=null){this.J=a;this.P=b;this.F=c;this.O=d;this.G=e;this.N=g;this.nonce=h;this.D=new Map;this.C="server"===f;this.defaultRenderer=new aH(a,e,g,this.C)}Zk(a,b){if(!a||!b)return this.defaultRenderer;this.C&&3===b.pb&&(b=Object.assign({},b,{pb:0}));a:{const d=this.D;var c=d.get(b.id);if(!c){c=this.G;const e=this.N,f=this.J,g=this.P,h=this.O,m=this.C;switch(b.pb){case 0:c=new bH(f,g,b,this.F,h,c,e,m);break;case 3:b=new cH(f,g,a,b,c,e,this.nonce,m);break a;default:c=
new dH(f,g,b,h,c,e,m)}d.set(b.id,c)}b=c}b instanceof bH?(TG(b.D,b.styles),b.setAttribute(a,b.P,"")):b instanceof dH&&TG(b.D,b.styles);return b}va(){this.D.clear()}};eH.K=function(a){return new (a||eH)(r(OG),r(WG),r(ni),r(ZG),r(kt),r(pi),r(Tk),r(ri))};eH.na=Pd({ma:eH,ba:eH.K});function fH(a,b){return c=>{if("__ngUnwrap__"===c)return b;!1===(a.J?a.G.runGuarded(()=>b(c)):b(c))&&c.preventDefault()}}
class aH{constructor(a,b,c,d){this.F=a;this.C=b;this.G=c;this.J=d;this.data=Object.create(null);this.cl=null}destroy(){}createElement(a,b){return b?this.C.createElementNS(XG[b]||b,a):this.C.createElement(a)}createComment(a){return this.C.createComment(a)}tq(a){return this.C.createTextNode(a)}appendChild(a,b){("TEMPLATE"===a.tagName&&void 0!==a.content?a.content:a).appendChild(b)}insertBefore(a,b,c){a&&("TEMPLATE"===a.tagName&&void 0!==a.content?a.content:a).insertBefore(b,c)}removeChild(a,b){a&&a.removeChild(b)}xs(a,
b){a="string"===typeof a?this.C.querySelector(a):a;if(!a)throw new ae(-5104,!1);b||(a.textContent="");return a}parentNode(a){return a.parentNode}nextSibling(a){return a.nextSibling}setAttribute(a,b,c,d){d?(b=d+":"+b,(d=XG[d])?a.setAttributeNS(d,b,c):a.setAttribute(b,c)):a.setAttribute(b,c)}removeAttribute(a,b,c){if(c){const d=XG[c];d?a.removeAttributeNS(d,b):a.removeAttribute(`${c}:${b}`)}else a.removeAttribute(b)}Wi(a,b){a.classList.add(b)}Do(a,b){a.classList.remove(b)}setStyle(a,b,c,d){d&3?a.style.setProperty(b,
c,d&1?"important":""):a.style[b]=c}ks(a,b,c){c&2?a.style.removeProperty(b):a.style[b]=""}setProperty(a,b,c){null!=a&&(a[b]=c)}setValue(a,b){a.nodeValue=b}Id(a,b,c){if("string"===typeof a&&(a=qt(this.C,a),!a))throw Error(`Unsupported event target ${a} for event ${b}`);return this.F.addEventListener(a,b,fH(this,c))}}function gH(a,b){return b===a.N?a.shadowRoot:b}
class cH extends aH{constructor(a,b,c,d,e,f,g,h){super(a,e,f,h);this.D=b;this.N=c;this.shadowRoot=c.attachShadow({mode:"open"});a=this.D;b=this.shadowRoot;a.D.add(b);for(const m of a.C.keys())UG(a,b,m);d=$G(d.id,d.styles);for(const m of d)d=document.createElement("style"),g&&d.setAttribute("nonce",g),d.textContent=m,this.shadowRoot.appendChild(d)}appendChild(a,b){return super.appendChild(gH(this,a),b)}insertBefore(a,b,c){return super.insertBefore(gH(this,a),b,c)}removeChild(a,b){return super.removeChild(gH(this,
a),b)}parentNode(a){return gH(this,super.parentNode(gH(this,a)))}destroy(){this.D.D.delete(this.shadowRoot)}}class dH extends aH{constructor(a,b,c,d,e,f,g,h){super(a,e,f,g);this.D=b;this.N=d;this.styles=h?$G(h,c.styles):c.styles}destroy(){if(this.N){var a=this.D,b=this.styles;for(const c of b)0>=SG(a,c,-1)&&VG(a,c)}}}
class bH extends dH{constructor(a,b,c,d,e,f,g,h){d=d+"-"+c.id;super(a,b,c,e,f,g,h,d);this.O="_ngcontent-%COMP%".replace(YG,d);this.P="_nghost-%COMP%".replace(YG,d)}createElement(a,b){a=super.createElement(a,b);super.setAttribute(a,this.O,"");return a}};var hH=class extends PG{supports(){return!0}addEventListener(a,b,c){a.addEventListener(b,c,!1);return()=>this.removeEventListener(a,b,c)}removeEventListener(a,b,c){return a.removeEventListener(b,c)}};hH.K=function(a){return new (a||hH)(r(kt))};hH.na=Pd({ma:hH,ba:hH.K});const iH=["alt","control","meta","shift"],jH={"\b":"Backspace","\t":"Tab","\u007f":"Delete","\u001b":"Escape",Del:"Delete",Esc:"Escape",Left:"ArrowLeft",Right:"ArrowRight",Up:"ArrowUp",Down:"ArrowDown",Menu:"ContextMenu",Scroll:"ScrollLock",Win:"OS"},kH={alt:a=>a.altKey,control:a=>a.ctrlKey,meta:a=>a.metaKey,shift:a=>a.shiftKey};
function lH(a){const b=a.toLowerCase().split(".");a=b.shift();if(0===b.length||"keydown"!==a&&"keyup"!==a)return null;const c=mH(b.pop());let d="",e=b.indexOf("code");-1<e&&(b.splice(e,1),d="code.");iH.forEach(f=>{const g=b.indexOf(f);-1<g&&(b.splice(g,1),d+=f+".")});d+=c;if(0!=b.length||0===c.length)return null;a={domEventName:a};a.fullKey=d;return a}function nH(a,b,c){return d=>{oH(d,a)&&c.runGuarded(()=>b(d))}}function mH(a){return"esc"===a?"escape":a}
function oH(a,b){let c=jH[a.key]||a.key,d="";-1<b.indexOf("code.")&&(c=a.code,d="code.");if(null==c||!c)return!1;c=c.toLowerCase();" "===c?c="space":"."===c&&(c="dot");iH.forEach(e=>{e!==c&&(0,kH[e])(a)&&(d+=e+".")});d+=c;return d===b}var pH=class extends PG{supports(a){return null!=lH(a)}addEventListener(a,b,c){const d=lH(b),e=nH(d.fullKey,c,this.manager.zb);return Qk(this.manager.zb,()=>JG(a,d.domEventName,e))}};pH.K=function(a){return new (a||pH)(r(kt))};pH.na=Pd({ma:pH,ba:pH.K});var qH=[{pa:pi,nb:"browser"},{pa:oi,nb:function(){var a=new KG;null!=lt||(lt=a)},Eb:!0},{pa:kt,Gc:function(){return ii=document},Bc:[]}];vs(Ms,"browser",qH);const rH=new gf(""),sH=[{pa:Dn,Fc:LG,Bc:[]},{pa:Cn,Fc:In,Bc:[Tk,Jn,Dn]},{pa:In,Fc:In,Bc:[Tk,Jn,Dn]}],tH=[{pa:jg,nb:"root"},{pa:Ai,Gc:function(){return new Ai},Bc:[]},{pa:NG,Fc:hH,Eb:!0,Bc:[kt,Tk,pi]},{pa:NG,Fc:pH,Eb:!0,Bc:[kt]},eH,WG,OG,{pa:yi,Ab:eH},{pa:Gt,Fc:MG,Bc:[]},[]];var uH=class{};uH.K=function(a){return new (a||uH)(r(rH,12))};uH.Ya=Sf({type:uH});
uH.Xa=Qd({Wa:[...tH,...sH],imports:[it,Sr]});var vH=class{constructor(a){this.C=a}};vH.K=function(a){return new (a||vH)(r(kt))};vH.na=Pd({ma:vH,ba:vH.K,fa:"root"});new gf("HammerGestureConfig");new gf("HammerLoader");var wH=class{};function xH(a,b,c){a.Vf.has(c)||a.Vf.set(c,b)}function yH(a,b,c){c=(Array.isArray(c)?c:[c]).map(e=>e.toString());const d=b.toLowerCase();a.headers.set(d,c);xH(a,b,d)}function zH(a,b){b.init();Array.from(b.headers.keys()).forEach(c=>{a.headers.set(c,b.headers.get(c));a.Vf.set(c,b.Vf.get(c))})}
function AH(a,b){const c=b.name.toLowerCase();switch(b.op){case "a":case "s":var d=b.value;"string"===typeof d&&(d=[d]);if(0===d.length)break;xH(a,b.name,c);b=("a"===b.op?a.headers.get(c):void 0)||[];b.push(...d);a.headers.set(c,b);break;case "d":const e=b.value;if(e){if(d=a.headers.get(c))d=d.filter(f=>-1===e.indexOf(f)),0===d.length?(a.headers.delete(c),a.Vf.delete(c)):a.headers.set(c,d)}else a.headers.delete(c),a.Vf.delete(c)}}
var BH=class{constructor(a){this.Vf=new Map;this.Dl=null;a?"string"===typeof a?this.hf=()=>{this.headers=new Map;a.split("\n").forEach(b=>{const c=b.indexOf(":");if(0<c){const d=b.slice(0,c),e=d.toLowerCase();b=b.slice(c+1).trim();xH(this,d,e);this.headers.has(e)?this.headers.get(e).push(b):this.headers.set(e,[b])}})}:"undefined"!==typeof Headers&&a instanceof Headers?(this.headers=new Map,a.forEach((b,c)=>{yH(this,c,b)})):this.hf=()=>{this.headers=new Map;Object.entries(a).forEach(([b,c])=>{yH(this,
b,c)})}:this.headers=new Map}has(a){this.init();return this.headers.has(a.toLowerCase())}get(a){this.init();return(a=this.headers.get(a.toLowerCase()))&&0<a.length?a[0]:null}keys(){this.init();return Array.from(this.Vf.values())}getAll(a){this.init();return this.headers.get(a.toLowerCase())||null}append(a,b){return this.clone({name:a,value:b,op:"a"})}set(a,b){return this.clone({name:a,value:b,op:"s"})}delete(a,b){return this.clone({name:a,value:b,op:"d"})}init(){this.hf&&(this.hf instanceof BH?zH(this,
this.hf):this.hf(),this.hf=null,this.Dl&&(this.Dl.forEach(a=>AH(this,a)),this.Dl=null))}clone(a){const b=new BH;b.hf=this.hf&&this.hf instanceof BH?this.hf:this;b.Dl=(this.Dl||[]).concat([a]);return b}forEach(a){this.init();Array.from(this.Vf.keys()).forEach(b=>a(this.Vf.get(b),this.headers.get(b)))}};var CH=class{};function DH(a){const b=new Map;0<a.length&&a.replace(/^\?/,"").split("&").forEach(c=>{const d=c.indexOf("="),[e,f]=-1==d?[decodeURIComponent(c),""]:[decodeURIComponent(c.slice(0,d)),decodeURIComponent(c.slice(d+1))];c=b.get(e)||[];c.push(f);b.set(e,c)});return b}const EH=/%(\d[a-f0-9])/gi,FH={40:"@","3A":":",24:"$","2C":",","3B":";","3D":"=","3F":"?","2F":"/"};function GH(a){return encodeURIComponent(a).replace(EH,(b,c)=>{let d;return null!=(d=FH[c])?d:b})}
function HH(a){return`${a}`}
var IH=class{constructor(a={}){this.C=this.D=null;this.Kq=a.Kq||new CH;if(a.iC){if(a.Gn)throw Error("Cannot specify both fromString and fromObject.");this.map=DH(a.iC)}else a.Gn?(this.map=new Map,Object.keys(a.Gn).forEach(b=>{var c=a.Gn[b];c=Array.isArray(c)?c.map(HH):[`${c}`];this.map.set(b,c)})):this.map=null}has(a){this.init();return this.map.has(a)}get(a){this.init();return(a=this.map.get(a))?a[0]:null}getAll(a){this.init();return this.map.get(a)||null}keys(){this.init();return Array.from(this.map.keys())}append(a,
b){return this.clone({Jg:a,value:b,op:"a"})}set(a,b){return this.clone({Jg:a,value:b,op:"s"})}delete(a,b){return this.clone({Jg:a,value:b,op:"d"})}toString(){this.init();return this.keys().map(a=>{const b=GH(a);return this.map.get(a).map(c=>b+"="+GH(c)).join("&")}).filter(a=>""!==a).join("&")}clone(a){const b=new IH({Kq:this.Kq});b.C=this.C||this;b.D=(this.D||[]).concat(a);return b}init(){null===this.map&&(this.map=new Map);null!==this.C&&(this.C.init(),this.C.keys().forEach(a=>this.map.set(a,this.C.map.get(a))),
this.D.forEach(a=>{switch(a.op){case "a":case "s":var b=("a"===a.op?this.map.get(a.Jg):void 0)||[];b.push(`${a.value}`);this.map.set(a.Jg,b);break;case "d":if(void 0!==a.value){b=this.map.get(a.Jg)||[];const c=b.indexOf(`${a.value}`);-1!==c&&b.splice(c,1);0<b.length?this.map.set(a.Jg,b):this.map.delete(a.Jg)}else this.map.delete(a.Jg)}}),this.C=this.D=null)}};var JH=class{constructor(){this.map=new Map}set(a,b){this.map.set(a,b);return this}get(a){this.map.has(a)||this.map.set(a,a.defaultValue());return this.map.get(a)}delete(a){this.map.delete(a);return this}has(a){return this.map.has(a)}keys(){return this.map.keys()}};var KH=class{constructor(a,b,c,d){this.url=b;this.body=null;this.withCredentials=this.hi=!1;this.responseType="json";this.method=a.toUpperCase();a:switch(this.method){case "DELETE":case "GET":case "HEAD":case "OPTIONS":case "JSONP":a=!1;break a;default:a=!0}if(a||d)this.body=void 0!==c?c:null,c=d;c&&(this.hi=!!c.hi,this.withCredentials=!!c.withCredentials,c.responseType&&(this.responseType=c.responseType),c.headers&&(this.headers=c.headers),c.context&&(this.context=c.context),c.params&&(this.params=
c.params),this.ak=c.ak);null!=this.headers||(this.headers=new BH);null!=this.context||(this.context=new JH);this.params||(this.params=new IH)}clone(a={}){const b=a.method||this.method,c=a.url||this.url,d=a.responseType||this.responseType;var e;const f=null!=(e=a.ak)?e:this.ak;e=void 0!==a.body?a.body:this.body;var g;const h=null!=(g=a.withCredentials)?g:this.withCredentials;var m;g=null!=(m=a.hi)?m:this.hi;m=a.headers||this.headers;let p=a.params||this.params,u;const w=null!=(u=a.context)?u:this.context;
void 0!==a.Vx&&(m=Object.keys(a.Vx).reduce((B,G)=>B.set(G,a.Vx[G]),m));a.Wx&&(p=Object.keys(a.Wx).reduce((B,G)=>B.set(G,a.Wx[G]),p));return new KH(b,c,e,{params:p,headers:m,context:w,hi:g,responseType:d,withCredentials:h,ak:f})}};function LH(a,b,c,d={}){if(b instanceof KH)c=b;else{var e=void 0;e=d.headers instanceof BH?d.headers:new BH(d.headers);let f=void 0;d.params&&(f=d.params instanceof IH?d.params:new IH({Gn:d.params}));c=new KH(b,c,void 0!==d.body?d.body:null,{headers:e,context:d.context,params:f,hi:d.hi,responseType:d.responseType||"json",withCredentials:d.withCredentials,ak:d.ak})}e=C(oj(c),Yj(f=>a.handler.handle(f),1));if(b instanceof KH||"events"===d.observe)return e;b=C(e,xk(()=>!1));switch(d.observe||"body"){case "body":switch(c.responseType){case "arraybuffer":return C(b,
Cj(f=>{if(null!==f.body&&!(f.body instanceof ArrayBuffer))throw Error("Response is not an ArrayBuffer.");return f.body}));case "blob":return C(b,Cj(f=>{if(null!==f.body&&!(f.body instanceof Blob))throw Error("Response is not a Blob.");return f.body}));case "text":return C(b,Cj(f=>{if(null!==f.body&&"string"!==typeof f.body)throw Error("Response is not a string.");return f.body}));default:return C(b,Cj(f=>f.body))}case "response":return b;default:throw Error(`Unreachable: unhandled observe type ${d.observe}}`);
}}var MH=class{constructor(a){this.handler=a}delete(a,b={}){return LH(this,"DELETE",a,b)}get(a,b={}){return LH(this,"GET",a,b)}head(a,b={}){return LH(this,"HEAD",a,b)}options(a,b={}){return LH(this,"OPTIONS",a,b)}};MH.K=function(a){return new (a||MH)(r(wH))};MH.na=Pd({ma:MH,ba:MH.K});new gf("");new gf("");new gf("");new gf("",{fa:"root",ba:()=>!0});new gf("");new gf("",{fa:"root",ba:()=>"XSRF-TOKEN"});new gf("",{fa:"root",ba:()=>"X-XSRF-TOKEN"});new gf("");new gf("");new gf("");var NH=class{};NH.K=function(a){return new (a||NH)};NH.na=Pd({ma:NH,ba:function(a){return a?new (a||NH):r(OH)},fa:"root"});
var OH=class extends NH{constructor(a){super();this.D=a}C(a,b){if(null==b)return null;switch(a){case 0:return b;case 1:if(Gl(b,"HTML"))var c=Fl(b);else{a:{a=this.D;var d=String(b);b=null;try{var e;if(!(e=hm)){var f=new Ll(a);try{var g=!!(new window.DOMParser).parseFromString(Jl(""),"text/html")}catch(u){g=!1}e=g?new Kl(f):f}hm=e;var h=d?String(d):"";b=hm.In(h);e=5;g=h;do{if(0===e)throw Error("Failed to sanitize html because the input is unstable");e--;h=g;g=b.innerHTML;b=hm.In(h)}while(h!==g);var m=
new bm;let p=(im(b)||b).firstChild;h=!0;for(e=[];p;){if(p.nodeType===Node.ELEMENT_NODE)b:{g=m;f=p;const u=dm(f).toLowerCase();if(!Wl.hasOwnProperty(u)){h=!am.hasOwnProperty(u);break b}g.C.push("<");g.C.push(u);const w=f.attributes;for(f=0;f<w.length;f++){const B=w.item(f),G=B.name,L=G.toLowerCase();if(!$l.hasOwnProperty(L))continue;let O=B.value;Xl[L]&&(O=Nl(O));g.C.push(" ",G,'="',cm(O),'"')}g.C.push(">");h=!0}else p.nodeType===Node.TEXT_NODE&&m.chars(p.nodeValue);if(h&&p.firstChild){e.push(p);const u=
p.firstChild;if(u&&(p.compareDocumentPosition(u)&Node.DOCUMENT_POSITION_CONTAINED_BY)!==Node.DOCUMENT_POSITION_CONTAINED_BY)throw em(u);p=u}else for(;p;){if(p.nodeType===Node.ELEMENT_NODE){g=m;const w=dm(p).toLowerCase();Wl.hasOwnProperty(w)&&!Ql.hasOwnProperty(w)&&(g.C.push("</"),g.C.push(w),g.C.push(">"))}const u=p.nextSibling;if(u&&p!==u.previousSibling)throw em(u);if(g=u){p=g;break}p=e.pop()}}c=Jl(m.C.join(""));break a}finally{if(b)for(m=im(b)||b;m.firstChild;)m.removeChild(m.firstChild)}c=void 0}c=
c.toString()}return c;case 2:return Gl(b,"Style")?Fl(b):b;case 3:if(Gl(b,"Script"))return Fl(b);throw new ae(5200,!1);case 4:return Gl(b,"URL")?Fl(b):Nl(String(b));case 5:if(Gl(b,"ResourceURL"))return Fl(b);throw new ae(5201,!1);default:throw new ae(5202,!1);}}G(a){return new Dl(a)}F(a){return new El(a)}};OH.K=function(a){return new (a||OH)(r(kt))};OH.na=Pd({ma:OH,ba:OH.K,fa:"root"});function PH(a,...b){if(0===b.length)return ld(a[0]);let c=a[0];for(let d=0;d<b.length;d++)c+=encodeURIComponent(b[d])+a[d+1];return ld(c)};function QH(a,b){switch(b){case 1:return!1;case 4:return a instanceof md||a instanceof fd;case 5:return a instanceof fd;case 3:return!1;default:return!1}}
var RH=class extends NH{constructor(a){super();this.D=new OH(a)}C(a,b){if(null==b)return null;if(b instanceof fd||b instanceof md){if(QH(b,a)){var c=this.D,d=c.C;a:{if(null!=b){if(b instanceof md){b=this.D.G(nd(b));break a}if(b instanceof fd){b=this.D.F(String(b));break a}}b=null}return d.call(c,a,b)}throw Error(`Required a safe value for ${jm[a]}, got ${b} `+"(see http://g.co/ng/security#xss)");}return this.D.C(a,b)}G(){throw Error("bypassSecurityTrust* functions are prohibited in the google3 version of Angular. Use LegacyDomSanitizer if you need to disable auto sanitization and get a security from ise-hardening-reviews.");
}F(){throw Error("bypassSecurityTrust* functions are prohibited in the google3 version of Angular. Use LegacyDomSanitizer if you need to disable auto sanitization and get a security from ise-hardening-reviews.");}};RH.K=function(a){return new (a||RH)(r(kt))};RH.na=Pd({ma:RH,ba:RH.K});const SH=[{pa:hn,Ab:NH},{pa:NH,Fc:RH}];var TH=class{};TH.K=function(a){return new (a||TH)};TH.Ya=Sf({type:TH});TH.Xa=Qd({Wa:SH,imports:[uH]});let UH;function VH(){if(void 0===UH&&(UH=null,"undefined"!==typeof window)){const a=window;void 0!==a.trustedTypes&&(UH=a.trustedTypes.createPolicy("angular#components",{createHTML:b=>b}))}return UH}function WH(a){let b;return(null==(b=VH())?void 0:b.createHTML(a))||a};function XH(a){return Error(`Unable to find icon with the name "${a}"`)}class YH{constructor(a,b){this.url=a;this.Wg=null;this.options=b}}function ZH(a,b){return a.wa.get(b)||b}function $H(a,b,c=""){const d=c+":"+b;let e=a.F.get(d);return e?a.aa(e):(e=a.Ea(c,b))?(a.F.set(d,e),a.aa(e)):(c=a.C.get(c))?a.Ja(b,c):pj(XH(d))}
var aI=class{constructor(a,b,c,d){this.ca=a;this.ea=b;this.ja=d;this.F=new Map;this.C=new Map;this.oa=new Map;this.G=new Map;this.wa=new Map;this.J=[];this.ta=["material-icons","mat-ligature-font"];this.ia=c}va(){this.J=[];this.F.clear();this.C.clear();this.oa.clear()}aa(a){return a.Wg?oj(this.N(a).cloneNode(!0)):C(this.Qa(a),Cj(b=>b.cloneNode(!0)))}Ja(a,b){var c=this.P(a,b);if(c)return oj(c);c=b.filter(d=>!d.Wg).map(d=>C(this.Ta(d),Pn(e=>{e=`Loading icon set URL: ${this.ea.C(5,d.url)} failed: ${e.message}`;
this.ja.handleError(Error(e));return oj(null)})));return C(ok(c),Cj(()=>{const d=this.P(a,b);if(!d)throw XH(a);return d}))}P(a,b){for(let d=b.length-1;0<=d;d--){var c=b[d];if(c.Wg&&-1<c.Wg.toString().indexOf(a)){const e=this.N(c);if(c=this.ua(e,a,c.options))return c}}return null}Qa(a){return C(this.T(a),So(b=>a.Wg=b),Cj(()=>this.N(a)))}Ta(a){return a.Wg?oj(null):C(this.T(a),So(b=>a.Wg=b))}ua(a,b,c){a=a.querySelector(`[id="${b}"]`);if(!a)return null;a=a.cloneNode(!0);a.removeAttribute("id");if("svg"===
a.nodeName.toLowerCase())return this.D(a,c);if("symbol"===a.nodeName.toLowerCase())return this.D(this.Va(a),c);b=this.O(WH("<svg></svg>"));b.appendChild(a);return this.D(b,c)}O(a){const b=this.ia.createElement("DIV");b.innerHTML=a;a=b.querySelector("svg");if(!a)throw Error("<svg> tag not found");return a}Va(a){const b=this.O(WH("<svg></svg>"));var c=a.attributes;for(let d=0;d<c.length;d++){const {name:e,value:f}=c[d];"id"!==e&&b.setAttribute(e,f)}for(c=0;c<a.childNodes.length;c++)a.childNodes[c].nodeType===
this.ia.ELEMENT_NODE&&b.appendChild(a.childNodes[c].cloneNode(!0));return b}D(a,b){a.setAttribute("fit","");a.setAttribute("height","100%");a.setAttribute("width","100%");a.setAttribute("preserveAspectRatio","xMidYMid meet");a.setAttribute("focusable","false");b&&b.viewBox&&a.setAttribute("viewBox",b.viewBox);return a}T(a){var b=a.url;let c,d;a=null!=(d=null==(c=a.options)?void 0:c.withCredentials)?d:!1;if(!this.ca)throw Error("Could not find HttpClient provider for use with Angular Material icons. Please include the HttpClientModule from @angular/common/http in your app imports.");
if(null==b)throw Error(`Cannot fetch icon from URL "${b}".`);const e=this.ea.C(5,b);if(!e)throw Error("The URL provided to MatIconRegistry was not trusted as a resource URL "+`via Angular's DomSanitizer. Attempted URL was "${b}".`);if(b=this.G.get(e))return b;b=C(this.ca.get(e,{responseType:"text",withCredentials:a}),Cj(f=>WH(f)),po(()=>this.G.delete(e)),Eo());this.G.set(e,b);return b}fb(a,b,c){this.F.set(a+":"+b,c);return this}gb(a,b){const c=this.C.get(a);c?c.push(b):this.C.set(a,[b]);return this}N(a){if(!a.xy){const b=
this.O(a.Wg);this.D(b,a.options);a.xy=b}return a.xy}Ea(a,b){for(let c=0;c<this.J.length;c++){const d=this.J[c](b,a);if(d)return d.url&&d.options?new YH(d.url,d.options):new YH(d)}}};aI.K=function(a){return new (a||aI)(r(MH,8),r(NH),r(kt,8),r(Ai))};aI.na=Pd({ma:aI,ba:aI.K,fa:"root"});new Wh;new Xh;new Wh;new Wh;var bI=new gf("MAT_ICON_DEFAULT_OPTIONS"),dI=new gf("mat-icon-location",{fa:"root",ba:cI});function cI(){const a=of(kt),b=a?a.location:null;return{kw:()=>b?b.pathname+b.search:""}}const eI="clip-path color-profile src cursor fill filter marker marker-start marker-mid marker-end mask stroke".split(" "),fI=eI.map(a=>`[${a}]`).join(", "),gI=/^url\(['"]?#(.*?)['"]?\)$/;
var hI=class{get color(){return this.oh||this.ta}set color(a){this.oh=a}get Ls(){return this.J}set Ls(a){a!==this.J&&(a?this.Qa(a):this.J&&this.T(),this.J=a)}get Ag(){return this.ca}set Ag(a){a=this.P(a);a!==this.ca&&(this.ca=a,this.N())}get zg(){return this.aa}set zg(a){a=this.P(a);a!==this.aa&&(this.aa=a,this.N())}constructor(a,b,c,d,e,f){this.ra=a;this.F=b;this.O=d;this.ua=e;this.inline=!1;this.ja=[];this.D=Si.EMPTY;f&&(f.color&&(this.color=this.ta=f.color),f.Ag&&(this.Ag=f.Ag));c||a.R.setAttribute("aria-hidden",
"true")}Ja(a){if(!a)return["",""];const b=a.split(":");switch(b.length){case 1:return["",b[0]];case 2:return b;default:throw Error(`Invalid icon name: "${a}"`);}}Fb(){this.N()}Zw(){var a=this.C;a&&a.size&&(a=this.O.kw(),a!==this.oa&&(this.oa=a,this.ea(a)))}va(){this.D.unsubscribe();this.C&&this.C.clear()}cq(){return!this.Ls}Ea(a){this.T();const b=this.O.kw();this.oa=b;this.wa(a);this.ea(b);this.ra.R.appendChild(a)}T(){const a=this.ra.R;let b=a.childNodes.length;for(this.C&&this.C.clear();b--;){const c=
a.childNodes[b];1===c.nodeType&&"svg"!==c.nodeName.toLowerCase()||c.remove()}}N(){if(this.cq()){var a=this.ra.R,b=(this.Ag?ZH(this.F,this.Ag).split(/ +/):this.F.ta).filter(c=>0<c.length);this.ja.forEach(c=>a.classList.remove(c));b.forEach(c=>a.classList.add(c));this.ja=b;this.zg===this.G||b.includes("mat-ligature-font")||(this.G&&a.classList.remove(this.G),this.zg&&a.classList.add(this.zg),this.G=this.zg)}}P(a){return"string"===typeof a?a.trim().split(" ")[0]:a}ea(a){const b=this.C;b&&b.forEach((c,
d)=>{c.forEach(e=>{d.setAttribute(e.name,`url('${a}#${e.value}')`)})})}wa(a){const b=a.querySelectorAll(fI),c=this.C=this.C||new Map;for(let d=0;d<b.length;d++)eI.forEach(e=>{const f=b[d];var g=f.getAttribute(e);if(g=g?g.match(gI):null){let h=c.get(f);h||(h=[],c.set(f,h));h.push({name:e,value:g[1]})}})}Qa(a){this.Cu=this.Du=null;this.D.unsubscribe();if(a){const [b,c]=this.Ja(a);b&&(this.Du=b);c&&(this.Cu=c);this.D=C($H(this.F,c,b),io()).subscribe(d=>this.Ea(d),d=>{this.ua.handleError(Error(`Error retrieving icon ${b}:${c}! ${d.message}`))})}}};
hI.K=function(a){return new (a||hI)(y(wi),y(aI),Dh("aria-hidden"),y(dI),y(Ai),y(bI,8))};
hI.Ca=Jf({type:hI,ha:[["mat-icon"]],Ka:["role","img",1,"mat-icon","notranslate"],Za:10,Ia:function(a,b){a&2&&(Mp("data-mat-icon-type",b.cq()?"font":"svg")("data-mat-icon-name",b.Cu||b.zg)("data-mat-icon-namespace",b.Du||b.Ag)("fontIcon",b.cq()?b.zg:null),sq(vq,b.color?"mat-"+b.color:""),Wp("mat-icon-inline",b.inline)("mat-icon-no-color","primary"!==b.color&&"accent"!==b.color&&"warn"!==b.color))},inputs:{color:"color",inline:[2,"inline","inline",As],Ls:"svgIcon",Ag:"fontSet",zg:"fontIcon"},ib:["matIcon"],
ka:!0,features:[Lp,Nr],bc:["*"],ya:1,za:0,sa:function(a){a&1&&(Zq(),$q(0))},styles:["mat-icon,mat-icon.mat-primary,mat-icon.mat-accent,mat-icon.mat-warn{color:var(--mat-icon-color)}.mat-icon{-webkit-user-select:none;user-select:none;background-repeat:no-repeat;display:inline-block;fill:currentColor;height:24px;width:24px;overflow:hidden}.mat-icon.mat-icon-inline{font-size:inherit;height:inherit;line-height:inherit;width:inherit}.mat-icon.mat-ligature-font[fontIcon]::before{content:attr(fontIcon)}[dir=rtl] .mat-icon-rtl-mirror{transform:scale(-1, 1)}.mat-form-field:not(.mat-form-field-appearance-legacy) .mat-form-field-prefix .mat-icon,.mat-form-field:not(.mat-form-field-appearance-legacy) .mat-form-field-suffix .mat-icon{display:block}.mat-form-field:not(.mat-form-field-appearance-legacy) .mat-form-field-prefix .mat-icon-button .mat-icon,.mat-form-field:not(.mat-form-field-appearance-legacy) .mat-form-field-suffix .mat-icon-button .mat-icon{margin:auto}\n"],
pb:2,Pa:0});var iI=class{};iI.K=function(a){return new (a||iI)};iI.Ya=Sf({type:iI});iI.Xa=Qd({imports:[mz,mz]});function jI(a){if(a&1){const b=Zg();R(0,"button",1);V("click",function(){t(b);Y();var c=new URL(window.location.href),d=new URLSearchParams(c.search);d.delete("show_open_in_new_tab");c.search=d.toString();d=window;c=td(c.toString());c=vd(c);void 0!==c&&d.open(c,"_blank",void 0);return x()});R(1,"mat-icon");Z(2,"open_in_new");U();Z(3," Open in new tab ");U()}}var kI=class{constructor(){this.Jo="1"===(new URLSearchParams(window.location.search)).get("show_open_in_new_tab")&&gb}};
kI.K=function(a){return new (a||kI)};kI.Ca=Jf({type:kI,ha:[["open-in-new-tab-button"]],ka:!0,features:[Nr],ya:1,za:1,Ba:[["mat-flat-button","","color","primary",1,"btn-open-in-tab"],["mat-flat-button","","color","primary",1,"btn-open-in-tab",3,"click"]],sa:function(a,b){a&1&&H(0,jI,4,0,"button",0);a&2&&Gq(b.Jo?0:-1)},Ha:[it,BB,AB,vB,iI,hI]});var lI={aliceblue:"#f0f8ff",antiquewhite:"#faebd7",aqua:"#00ffff",aquamarine:"#7fffd4",azure:"#f0ffff",beige:"#f5f5dc",bisque:"#ffe4c4",black:"#000000",blanchedalmond:"#ffebcd",blue:"#0000ff",blueviolet:"#8a2be2",brown:"#a52a2a",burlywood:"#deb887",cadetblue:"#5f9ea0",chartreuse:"#7fff00",chocolate:"#d2691e",coral:"#ff7f50",cornflowerblue:"#6495ed",cornsilk:"#fff8dc",crimson:"#dc143c",cyan:"#00ffff",darkblue:"#00008b",darkcyan:"#008b8b",darkgoldenrod:"#b8860b",darkgray:"#a9a9a9",darkgreen:"#006400",
darkkhaki:"#bdb76b",darkmagenta:"#8b008b",darkolivegreen:"#556b2f",darkorange:"#ff8c00",darkorchid:"#9932cc",darkred:"#8b0000",darksalmon:"#e9967a",darkseagreen:"#8fbc8f",darkslateblue:"#483d8b",darkslategray:"#2f4f4f",darkturquoise:"#00ced1",darkviolet:"#9400d3",deeppink:"#ff1493",deepskyblue:"#00bfff",dimgray:"#696969",dodgerblue:"#1e90ff",firebrick:"#b22222",floralwhite:"#fffaf0",forestgreen:"#228b22",fuchsia:"#ff00ff",gainsboro:"#dcdcdc",ghostwhite:"#f8f8ff",gold:"#ffd700",goldenrod:"#daa520",
gray:"#808080",green:"#008000",greenyellow:"#adff2f",honeydew:"#f0fff0",hotpink:"#ff69b4","indianred ":"#cd5c5c",indigo:"#4b0082",ivory:"#fffff0",khaki:"#f0e68c",lavender:"#e6e6fa",lavenderblush:"#fff0f5",lawngreen:"#7cfc00",lemonchiffon:"#fffacd",lightblue:"#add8e6",lightcoral:"#f08080",lightcyan:"#e0ffff",lightgoldenrodyellow:"#fafad2",lightgrey:"#d3d3d3",lightgreen:"#90ee90",lightpink:"#ffb6c1",lightsalmon:"#ffa07a",lightseagreen:"#20b2aa",lightskyblue:"#87cefa",lightslategray:"#778899",lightsteelblue:"#b0c4de",
lightyellow:"#ffffe0",lime:"#00ff00",limegreen:"#32cd32",linen:"#faf0e6",magenta:"#ff00ff",maroon:"#800000",mediumaquamarine:"#66cdaa",mediumblue:"#0000cd",mediumorchid:"#ba55d3",mediumpurple:"#9370d8",mediumseagreen:"#3cb371",mediumslateblue:"#7b68ee",mediumspringgreen:"#00fa9a",mediumturquoise:"#48d1cc",mediumvioletred:"#c71585",midnightblue:"#191970",mintcream:"#f5fffa",mistyrose:"#ffe4e1",moccasin:"#ffe4b5",navajowhite:"#ffdead",navy:"#000080",oldlace:"#fdf5e6",olive:"#808000",olivedrab:"#6b8e23",
orange:"#ffa500",orangered:"#ff4500",orchid:"#da70d6",palegoldenrod:"#eee8aa",palegreen:"#98fb98",paleturquoise:"#afeeee",palevioletred:"#d87093",papayawhip:"#ffefd5",peachpuff:"#ffdab9",peru:"#cd853f",pink:"#ffc0cb",plum:"#dda0dd",powderblue:"#b0e0e6",purple:"#800080",rebeccapurple:"#663399",red:"#ff0000",rosybrown:"#bc8f8f",royalblue:"#4169e1",saddlebrown:"#8b4513",salmon:"#fa8072",sandybrown:"#f4a460",seagreen:"#2e8b57",seashell:"#fff5ee",sienna:"#a0522d",silver:"#c0c0c0",skyblue:"#87ceeb",slateblue:"#6a5acd",
slategray:"#708090",snow:"#fffafa",springgreen:"#00ff7f",steelblue:"#4682b4",tan:"#d2b48c",teal:"#008080",thistle:"#d8bfd8",tomato:"#ff6347",turquoise:"#40e0d0",violet:"#ee82ee",wheat:"#f5deb3",white:"#ffffff",whitesmoke:"#f5f5f5",yellow:"#ffff00",yellowgreen:"#9acd32"};var mI={st:"regex",ft:"attr_value_range",vz:"node_type"};function nI(a){return 0===(null==a?void 0:a.nodeType)}function oI(a){return 1===(null==a?void 0:a.nodeType)}function pI(a){const b=[];for(const [c,d]of Object.entries(a))switch(a=d,c){case "Op node id":a.selected&&b.push("id")}return b}function qI(a){const b=[];for(const [c,d]of Object.entries(a))switch(a=d,c){case "Layer node children count":a.selected&&b.push("#children");break;case "Layer node descendants count":a.selected&&b.push("#descendants")}return b}
function rI(a,b){if(nI(a))switch(b.toLowerCase()){case "id":return a.id;case "namespace":return sI(a)}else if(oI(a))switch(b.toLowerCase()){case "namespace":return sI(a);case "#children":return String((a.nsChildrenIds||[]).length);case "#descendants":return String((a.descendantsNodeIds||[]).length)}return""}function sI(a){return a.savedNamespace||a.namespace||"<root>"}function tI(){return Math.random().toString(36).slice(-6)}
function uI(a,b,c){let d=[];d=null==a?b.rootNodes.map(e=>e.id):a.nsChildrenIds||[];for(const e of d)(a=b.nodesById[e])&&oI(a)&&a.expanded&&((a.nsChildrenIds||[]).filter(f=>oI(b.nodesById[f])).every(f=>!b.nodesById[f].expanded)&&c.push(a.id),uI(a,b,c))}
function vI(a){var b=wI,c=xI,d=THREE,e=[];if(2===a.length)e=a;else if(3===a.length&&a[0].x===a[1].x&&a[1].x===a[2].x)e=a;else{e=!0;var f=0;for(var g=0;g<a.length-1;g++){var h=a[g+1]>a[g]?1:-1;if(0!==f&&f!==h){e=!1;break}f=h}const p=d.Vector3;if(e){a=b().x(u=>u.x).y(u=>u.y).curve(c)(a).split(/M|C/).filter(u=>""!==u).map(u=>u.split(",").map(w=>Number(w)));b=new p(a[0][0],a[0][1],0);c=new d.CurvePath;for(e=1;e<a.length;e++){var m=a[e];6===m.length&&(f=b,g=new p(m[0],m[1]),h=new p(m[2],m[3]),b=m=new p(m[4],
m[5]),f=new d.CubicBezierCurve3(f,g,h,m),c.add(f))}e=c.getPoints(25)}else a=a.map(u=>new p(u.x,u.y,0)),e=(new d.CatmullRomCurve3(a,!1,"catmullrom",.1)).getPoints(25)}return e}function yI(){const a=zI();return a?"INPUT"===a.tagName||"SELECT"===a.tagName||"TEXTAREA"===a.tagName||"true"===a.contentEditable:!1}function zI(a=document){return(a=a.activeElement)?a.shadowRoot?zI(a.shadowRoot):a:null}function AI(a){a=((a||{}).shape||"").replace(/ /g,"").replace(/\u00d7/g,"x");""===a&&(a="?");return a}
function BI(a,b=""){a=a.attrs||{};const c=[],d=new RegExp(b,"i");for(const f of Object.keys(a)){const g=f,h=a[f];var e=[`${g}:${h}`,`${g}=${h}`];if(""===b.trim()||e.some(m=>d.test(m)))e=h,e="__value"===g?h.replace(/\s/gm,""):h.replace(/(\r\n|\n|\r)/gm,""),c.push({key:g,value:e})}return c}
function CI(a,b){const c=a.incomingEdges||[],d=[];for(let h=0;h<Math.min(10,c.length);h++){var e=c[h],f=b.nodesById[e.sourceNodeId];const m=AI((f.outputsMetadata||{})[e.sourceNodeOutputId]);var g=h;e=((a.inputsMetadata||{})[e.targetNodeInputId]||{}).__tensor_tag;d.push({key:e?`Input${g}:${e} (${f.label})`:`Input${g} (${f.label})`,value:m})}10<c.length&&(a=c.length-10,d.push({key:`(${a} more input${1===a?"":"s"} omitted)`,value:"..."}));return d}
function DI(a){const b=[],c=Object.values(a.outputsMetadata||{});for(let e=0;e<Math.min(10,c.length);e++){var d=c[e];const f=AI(d);let g=`Output${e}`;"GraphInputs"===a.label?(d=d.tensor_name,null!=d&&(g=`${g} (${d})`)):(d=d.__tensor_tag)&&(g=`Output${e}:${d}`);b.push({key:g,value:f})}10<c.length&&(a=c.length-10,b.push({key:`(${a} more output${1===a?"":"s"} omitted)`,value:"..."}));return b}
function EI(a,b,c,d){const e=[],f=Object.keys(c).filter(g=>c[g].selected).filter(g=>g.startsWith("Node data provider: ")).map(g=>g.replace("Node data provider: ",""));d=Object.values(d).filter(g=>f.includes(g.runName));for(const g of d){let h,m;d=(null==(h=g.results||{})?void 0:null==(m=h[b][a.id])?void 0:m.strValue)||"-";e.push({key:g.runName,value:d})}return e}
function FI(a,b){a=a.split("/");b=b.split("/");let c="";for(let d=Math.min(a.length,b.length);0<d;d--){const e=b.slice(0,d).join("/");if(a.slice(0,d).join("/")===e){c=e;break}}return c}function GI(){var a=HI;return ta(function*(){return new Promise(b=>{const c=document.createElement("script");xd(c,a);c.onload=()=>{c.remove();b()};document.body.appendChild(c)})})}
function II(a){return(new RegExp(/Only `SavedModel`s with \d+ MetaGraph are supported. Instead, it has \d+/)).test(a)?`${a}. Try using the "TF adapter (direct)" adapter.`:a}
function JI(a,b,c,d){const e=[],f=new Set;a.has("title")&&b.test(c.label)&&(e.push({type:"title"}),f.add("title"));if(a.has("list")){var g=KI(c);for(var h of Object.keys(g)){var m=g[h],p=`${h}=${m}`;if(b.test(`${h}:${m}`)||b.test(p))e.push({type:"list",Rw:h}),f.add("list")}}if(a.has("input")&&nI(c))for(const w of c.incomingEdges||[]){h=d.nodesById[w.sourceNodeId];b.test(h.label)&&(e.push({type:"input",jf:h.label}),f.add("input"));g=c.inputsMetadata||{};(m=(g[w.targetNodeInputId]||{}).__tensor_tag)&&
b.test(m)&&(e.push({type:"input",jf:m}),f.add("input"));h=(h.outputsMetadata||{})[w.sourceNodeOutputId]||{};for(const B of Object.keys(h)){if(B.startsWith("__"))continue;m=h[B];p=`${B}:${m}`;const G=`${B}=${m}`;if(b.test(m)||b.test(p)||b.test(G))e.push({type:"input",jf:m}),f.add("input")}g=g[w.targetNodeInputId]||{};for(const B of Object.keys(g))!B.startsWith("__")&&(h=g[B],m=`${B}:${h}`,p=`${B}=${h}`,b.test(h)||b.test(m)||b.test(p))&&(e.push({type:"input",jf:h}),f.add("input"))}if(a.has("output")&&
nI(c)){a=c.outputsMetadata||{};for(var u of c.outgoingEdges||[])c=d.nodesById[u.targetNodeId],b.test(c.label)&&(e.push({type:"output",jf:c.label}),f.add("output")),(c=(a[u.sourceNodeOutputId]||{}).__tensor_tag)&&b.test(c)&&(e.push({type:"output",jf:c}),f.add("output"));for(const w of Object.values(a))for(const B of Object.keys(w))!B.startsWith("__")&&(d=w[B],u=`${B}:${d}`,c=`${B}=${d}`,b.test(d)||b.test(u)||b.test(c))&&(e.push({type:"output",jf:d}),f.add("output"))}return{matches:e,matchTypes:f}}
function KI(a){let b={};nI(a)?(b=Object.assign({},a.attrs||{}),b.id=a.id):oI(a)&&(b={"#descendants":`${(a.descendantsNodeIds||[]).length}`,"#children":`${(a.nsChildrenIds||[]).length}`});return b}function LI(a){for(const b of a.filter(c=>"node_type"!==c.type))switch(b.type){case "regex":if(""!==b.queryRegex)return!0;break;case "attr_value_range":if(""!==b.attrName)return!0}return!1}
function MI(a){return a.map(b=>({queries:b.queries.map(c=>{switch(c.type){case "regex":let d=RegExp("","i");try{d=new RegExp(c.queryRegex,"i")}catch(e){console.warn("Failed to create regex",e)}return{type:"regex",queryRegex:d,matchTypes:new Set(c.matchTypes)};case "node_type":case "attr_value_range":return c}}),nodeType:b.nodeType,styles:b.styles}))}
function NI(a,b,c){let d=!0;for(const h of b)if("node_type"===h.type){b=!0;if(nI(a)&&"layer_nodes"===h.nodeType||oI(a)&&"op_nodes"===h.nodeType)b=!1;if(!b){d=!1;break}}else if("regex"===h.type){if(0===JI(h.matchTypes,h.queryRegex,a,c).matches.length){d=!1;break}}else if("attr_value_range"===h.type){if(""!==h.attrName){let m,p;b=h.attrName;var e=null!=(m=h.min)?m:Number.NEGATIVE_INFINITY,f=null!=(p=h.max)?p:Number.POSITIVE_INFINITY;const u=[];var g=KI(a)[b];null!=g&&(g=Number(g),!isNaN(g)&&g>=e&&g<=
f&&u.push({type:"list",Rw:b}));if(0===u.length){d=!1;break}}}else d=!1;return d};var OI="Processing nodes and edges;Processing layer namespaces;Processing layout data;Splitting large layers (if any);Laying out root layer;Finding identical layers".split(";");function PI(a,b){window.localStorage.setItem(a,b)}var QI=class{};QI.K=function(a){return new (a||QI)};QI.na=Pd({ma:QI,ba:QI.K,fa:"root"});function RI(){return{deepestExpandedGroupNodeIds:[],selectedNodeId:"",selectedGraphId:"",selectedCollectionLabel:"",widthFraction:1,selected:!1}}function SI(a,b,c){a.C.update(d=>{const e=d.paneStates[c];if(!e)return d;e.deepestExpandedGroupNodeIds=b;return Object.assign({},d)})}function TI(a,b,c){a.C.update(d=>{const e=d.paneStates[c];if(!e)return d;e.selectedNodeId=b;return Object.assign({},d)})}
function UI(a,b,c,d){a.C.update(e=>{const f=e.paneStates[d];if(!f)return e;f.selectedGraphId=b;f.selectedCollectionLabel=c;return Object.assign({},e)})}function VI(a,b,c){a.C.update(d=>{const e=d.paneStates[c];if(!e)return d;e.flattenLayers=b;return Object.assign({},d)})}function WI(a){a.C.update(b=>{1<b.paneStates.length&&(b.paneStates=[b.paneStates[0]]);b.paneStates.push(RI());for(const c of b.paneStates)c.widthFraction=.5;return Object.assign({},b)})}
function XI(a,b){a.C.update(c=>{c.paneStates.splice(b,1);1===c.paneStates.length&&(c.paneStates[0].widthFraction=1);return Object.assign({},c)})}function YI(a,b){a.C.update(c=>{2===c.paneStates.length&&(c.paneStates[0].widthFraction=b,c.paneStates[1].widthFraction=1-b);return Object.assign({},c)})}function ZI(a){a.C.update(b=>{2===b.paneStates.length&&(b.paneStates=[b.paneStates[1],b.paneStates[0]]);return Object.assign({},b)})}
function $I(a,b){a.C.update(c=>{for(let d=0;d<c.paneStates.length;d++)c.paneStates[d].selected=d===b;return Object.assign({},c)})}var aJ=class{constructor(){this.C=yr({paneStates:[RI()]})}};aJ.K=function(a){return new (a||aJ)};aJ.na=Pd({ma:aJ,ba:aJ.K});var bJ=class{constructor(){const a=PH`/static_files/worker_bin.js`;this.C=new Worker(gd(a),void 0)}};bJ.K=function(a){return new (a||bJ)};bJ.na=Pd({ma:bJ,ba:bJ.K});function cJ(a){a.aa.C.addEventListener("message",b=>{b=b.data;switch(b.eventType){case 1:dJ(a,b.modelGraph,b.paneId)}})}function eJ(a){let b={};a.C||(a=window.localStorage.getItem("model_explorer_show_on_node_item_types_v2"))&&(b=JSON.parse(a));return b}function fJ(a){let b={};a.C||(a=window.localStorage.getItem("model_explorer_show_on_edge_item_types_v2"))&&(b=JSON.parse(a));return b}
function gJ(a,b){a.O.update(c=>{c=[...c];const d={};for(const e of b){const f={},g={};for(const u of e.graphs){if(null==u.id||""===u.id)u.id="unnamed_graph";const w=u.id;let B=d[w];null==B?B=d[w]=0:(u.id=`${u.id} (${B+1})`,g[w]=u.id);d[w]=B+1;f[u.id]=u;u.collectionLabel=e.label}for(const u of e.graphs)for(const w of u.nodes)if(null!=w.subgraphIds&&0<w.subgraphIds.length){w.subgraphIds=w.subgraphIds.map(B=>g[B]||B);null==u.subGraphIds&&(u.subGraphIds=[]);u.subGraphIds.push(...w.subgraphIds);for(const B of w.subgraphIds)f[B].parentGraphId=
u.id}const h=e.graphs.filter(u=>null==u.parentGraphId),m=[],p=(u,w=0)=>{let B=[];B=null==u?h:(u.subGraphIds||[]).map(G=>f[G]);B.sort((G,L)=>L.nodes.length-G.nodes.length);for(const G of B)G.level=w,m.push(G),p(G,w+1)};p();e.graphs=m}c.push(...b);return c})}
function hJ(a,b,c=!1){const d=tI();a.T[d]=b;a.La.update(g=>{g[0].widthFraction=.5;g.push({id:d,widthFraction:.5,flattenLayers:c,showOnNodeItemTypes:{[d]:eJ(a)},Yb:{[d]:fJ(a)}});return[...g]});a.G.set({id:d,Tr:2});const e=iJ(a,d);WI(a.D);jJ(a,d);UI(a.D,b.id,b.collectionLabel||"",e);VI(a.D,c,e);let f;b={eventType:0,graph:b,showOnNodeItemTypes:kJ(a,d,d),nodeDataProviderRuns:{},config:a.config?a.config():void 0,paneId:d,groupNodeChildrenCountThreshold:lJ(a),flattenLayers:c,keepLayersWithASingleChild:null==
(f=a.config())?void 0:f.keepLayersWithASingleChild};a.aa.C.postMessage(b)}function iJ(a,b){return a.La().findIndex(c=>c.id===b)}function mJ(a,b,c=!1,d){if(null!=d){const f=nJ(a,b);f&&(f.Ko=d)}oJ(a,b);let e;b={eventType:0,graph:a.T[b],showOnNodeItemTypes:kJ(a,b,b),nodeDataProviderRuns:{},config:a.config?a.config():void 0,paneId:b,groupNodeChildrenCountThreshold:lJ(a),flattenLayers:c,keepLayersWithASingleChild:null==(e=a.config())?void 0:e.keepLayersWithASingleChild};a.aa.C.postMessage(b)}
function pJ(a,b,c,d=!1,e){if(1===c&&1===a.La().length)hJ(a,b);else{var f;if(((null==(f=a.La()[c].modelGraph)?void 0:f.id)||"")!==b.id){c=a.La()[c];c.Rg=void 0;c.As=void 0;f=c.id;a.T[f]=b;UI(a.D,b.id,b.collectionLabel||"",iJ(a,f));if(null!=c.bg&&0<c.bg.length){const g=[...c.bg];g[g.length-1].Cg=b.id;c.bg=g}mJ(a,f,d,e)}}}function qJ(a,b,c=!1,d){pJ(a,b,iJ(a,a.J()),c,d)}function jJ(a,b){a.J.set(b);$I(a.D,iJ(a,b))}
function kJ(a,b,c){return(a=nJ(a,b))?JSON.parse(JSON.stringify((a.showOnNodeItemTypes||{})[c]||{})):{}}function lJ(a){let b=1E3;if(a.config){let c;b=(null==(c=a.config())?void 0:c.artificialLayerNodeCountThreshold)||1E3}null!=a.ua&&(b=Number(a.ua));return b}function nJ(a,b){return a.La().find(c=>c.id===b)}function oJ(a,b){a.La.update(c=>{for(const d of c)if(d.id===b){d.modelGraph=void 0;break}return[...c]})}function rJ(a){return nJ(a,a.J())}
function sJ(a,b){const c=rJ(a);if(c){a.La.update(e=>{c.flattenLayers=b;c.Rg=void 0;return[...e]});var d=iJ(a,c.id);VI(a.D,b,d);SI(a.D,[],d)}}function tJ(a,b){const c=nJ(a,b);if(c){var d=!0===c.flattenLayers;a.La.update(e=>{c.flattenLayers=!d;c.Rg=void 0;return[...e]});b=iJ(a,b);VI(a.D,!d,b);SI(a.D,[],b)}}function uJ(a,b){let c;return!0===(null==(c=nJ(a,b))?void 0:c.flattenLayers)}function vJ(a,b){(b=a.La()[b])&&jJ(a,b.id)}
function wJ(a,b,c){a.La.update(f=>{const g=nJ(a,b);if(!g)return f;g.Sg=c;return[...f]});TI(a.D,(null==c?void 0:c.nodeId)||"",iJ(a,b));var d,e=null==(d=nJ(a,b))?void 0:d.modelGraph;if(e&&(d=(null==c?void 0:c.nodeId)||"",(e=e.nodesById[d])&&nI(e)&&window.parent)){e=e.outputsMetadata||{};const f=[];for(const g of Object.keys(e)){const h=e[g].tensor_name;h&&f.push(h)}window.parent.postMessage({cmd:"model-explorer-node-selected",nodeId:d,outputTensorNames:f},"*")}}
function xJ(a){let b;return null==(b=nJ(a,a.J()))?void 0:b.modelGraph}function yJ(a,b){let c;return null==(c=nJ(a,b))?void 0:c.modelGraph}function zJ(a){let b;return null==(b=a.La()[0])?void 0:b.modelGraph}function AJ(a,b){a.La.update(c=>{if(2!==c.length)return c;c[0].widthFraction=b;c[1].widthFraction=1-b;return[...c]});YI(a.D,b)}function BJ(a,b,c){a.La.update(d=>{const e=nJ(a,b);if(!e)return d;e.As=c;return[...d]})}function CJ(a,b){let c;return null==(c=nJ(a,b))?void 0:c.As}
function DJ(a,b,c){a.La.update(d=>{const e=nJ(a,b);if(!e)return d;e.Mn=c;return[...d]})}function EJ(a,b,c){a.La.update(d=>{const e=nJ(a,b);if(!e)return d;e.sD=c;return[...d]})}function FJ(a,b){delete a.T[b];delete a.P[b];a.La.update(c=>{const d=c.findIndex(e=>e.id===b);0<=d&&(c.splice(d,1),XI(a.D,d));c[0].widthFraction=1;return[...c]});jJ(a,a.La()[0].id)}function GJ(a){a.La.update(b=>2!==b.length?b:[b[1],b[0]]);ZI(a.D)}
function HJ(a,b,c,d){a.La.update(e=>{const f=nJ(a,d);f&&(null==f.Tg&&(f.Tg={}),null==f.Tg[c]&&(f.Tg[c]=[]),f.Tg[c].push(b));return[...e]})}function IJ(a,b,c,d){a.La.update(e=>{const f=nJ(a,d);f&&f.Tg&&f.Tg[c]&&f.Tg[c].splice(b,1);return[...e]})}function JJ(a,b){for(const c of a.O())for(const d of c.graphs)if(d.id===b)return d}
function KJ(a,b,c,d,e){a.La.update(f=>{const g=nJ(a,b);if(!g)return f;const h=[...(g.bg||[])];0===h.length?h.push({Cg:c,kb:e}):h[h.length-1]={Cg:c,kb:e};h.push({Cg:d});g.bg=h;return[...f]})}function LJ(a,b,c){a.La.update(d=>{const e=nJ(a,b);if(!e)return d;let f=[...(e.bg||[])];f.splice(c+1);1===f.length&&(f=[]);e.bg=f;return[...d]})}function MJ(a,b,c){a.La.update(d=>{const e=nJ(a,b);if(!e)return d;e.Rg=c;return[...d]})}
function NJ(a,b){a.La.update(c=>{const d=nJ(a,b);if(!d)return c;d.Rg={results:{}};return[...c]})}
function OJ(a,b,c,d){a.La.update(e=>{const f=nJ(a,b);if(!f)return e;f.showOnNodeItemTypes||(f.showOnNodeItemTypes={});null==f.showOnNodeItemTypes[c]&&(f.showOnNodeItemTypes[c]={});null==f.showOnNodeItemTypes[c][d]&&(f.showOnNodeItemTypes[c][d]={selected:!1});const g=f.showOnNodeItemTypes[c][d].selected;f.showOnNodeItemTypes[c]=Object.assign({},f.showOnNodeItemTypes[c]);f.showOnNodeItemTypes[c][d].selected=!g;f.showOnNodeItemTypes=Object.assign({},f.showOnNodeItemTypes);return[...e]})}
function PJ(a,b,c,d){a.La.update(e=>{const f=nJ(a,b);if(!f)return e;f.Yb||(f.Yb={});null==f.Yb[c]&&(f.Yb[c]={});null==f.Yb[c][d]&&(f.Yb[c][d]={selected:!1});const g=f.Yb[c][d].selected;f.Yb[c]=Object.assign({},f.Yb[c]);f.Yb[c][d].selected=!g;f.Yb=Object.assign({},f.Yb);return[...e]})}
function QJ(a,b,c,d,e){a.La.update(f=>{const g=nJ(a,b);if(!g)return f;g.showOnNodeItemTypes||(g.showOnNodeItemTypes={});null==g.showOnNodeItemTypes[c]&&(g.showOnNodeItemTypes[c]={});null==g.showOnNodeItemTypes[c][d]&&(g.showOnNodeItemTypes[c][d]={selected:!1});g.showOnNodeItemTypes[c][d].filterRegex=e;g.showOnNodeItemTypes=Object.assign({},g.showOnNodeItemTypes);return[...f]})}
function RJ(a,b,c,d){a.La.update(e=>{const f=nJ(a,b);if(!f)return e;f.showOnNodeItemTypes||(f.showOnNodeItemTypes={});f.showOnNodeItemTypes=Object.assign({},f.showOnNodeItemTypes);f.showOnNodeItemTypes[c]=d;return[...e]})}function SJ(a,b,c,d){a.La.update(e=>{const f=nJ(a,b);if(!f)return e;f.Yb||(f.Yb={});f.Yb=Object.assign({},f.Yb);f.Yb[c]=d;return[...e]})}
function TJ(a,b){a.La.update(c=>{for(const d of c){d.showOnNodeItemTypes=Object.assign({},d.showOnNodeItemTypes);for(const e of Object.keys(d.showOnNodeItemTypes))for(const f of b){const g=d.showOnNodeItemTypes[e][f];g&&(g.selected=!1)}}return[...c]})}function UJ(a,b,c){return(a=nJ(a,b))?JSON.parse(JSON.stringify((a.Yb||{})[c]||{})):{}}function dJ(a,b,c){a.La.update(d=>{for(const e of d)if(e.id===c){e.modelGraph=b;break}return[...d]});a.wa.next({tG:iJ(a,c),modelGraph:b})}
var VJ=class{constructor(a,b,c){this.D=b;this.aa=c;this.O=yr([]);this.F=yr();this.G=yr();this.ja=new hj;this.Ea=new hj;this.oa=new hj;this.ca=new hj;this.ea=new hj;this.ta=new hj;this.config=yr();this.N=yr();this.La=yr([{id:tI(),widthFraction:1}]);this.J=yr(this.La()[0].id);this.wa=new hj;this.Og=yr([]);this.T={};this.P={};cJ(this);this.G.set({id:this.La()[0].id,Tr:2});a=new URLSearchParams(document.location.search);this.C="1"===a.get("test_mode");this.ua=a.get("groupNodeChildrenCountThreshold");
a=this.La()[0];a.showOnNodeItemTypes={[a.id]:eJ(this)};a.Yb={[a.id]:fJ(this)}}};VJ.K=function(a){return new (a||VJ)(r(QI),r(aJ),r(bJ))};VJ.na=Pd({ma:VJ,ba:VJ.K});var WJ=class{constructor(){this.Ow=Math.random().toString(36).slice(-6);this.cC=`url(#${this.Ow})`}};WJ.K=function(a){return new (a||WJ)};
WJ.Ca=Jf({type:WJ,ha:[["me-logo"]],ka:!0,features:[Nr],ya:19,za:2,Ba:["width;24;height;24;viewBox;0 0 256 256;fill;none;xmlns;http://www.w3.org/2000/svg".split(";"),"width 256 height 256 fill none".split(" "),["d","M229 64.5V192.5L128 248.5L127.5 115.5L229 64.5Z"],["d","M208 76V99L128 141.5V116L208 76Z","fill","white"],["d","M208 122.5V141.5L128 182.5V161.5L208 122.5Z","fill","white"],["d","M208 164.5V183L128 225.5V206L208 164.5Z","fill","white"],["d","M149 106V214.5L127 226V117.5L149 106Z","fill",
"white"],["d","M26 61L47.5 72.5V200L26 187V61Z","fill","#FBBC04"],["d","M125.5 10.5L145.5 21V60.5L125.5 56.5V10.5Z","fill","#DCA810"],["d","M164 31L184 41.5V81L164 78V31Z","fill","#309C4D"],["d","M164 31L184 41.5L87 94L65 82L164 31Z","fill","#11792D"],["d","M208 54.5L229 64.5L128 116L107 105L208 54.5Z","fill","#2C5CAC"],["d","M125.5 10.5L145.5 21L47.5 72.5L26 61L125.5 10.5Z","fill","#C78B15"],["d","M65 82L87 94V224L65 210.5V82Z","fill","#34A853"],["d","M107 105L128 116V248.5L107 236V105Z","fill",
"#4285F4"],"x1 144.5 y1 232 x2 219 y2 69.5 gradientUnits userSpaceOnUse".split(" "),["stop-color","#4285F4"],["offset","1","stop-color","#2C5CAC"]],sa:function(a,b){a&1&&(kh(),R(0,"svg",0),Oq(1,"rect",1)(2,"path",2)(3,"path",3)(4,"path",4)(5,"path",5)(6,"path",6)(7,"path",7)(8,"path",8)(9,"path",9)(10,"path",10)(11,"path",11)(12,"path",12)(13,"path",13)(14,"path",14),R(15,"defs")(16,"linearGradient",15),Oq(17,"stop",16)(18,"stop",17),U()()());a&2&&(D(2),Mp("fill",b.cC),D(14),Mp("id",b.Ow))},styles:["[_nghost-%COMP%] {display: flex; align-items: center; justify-content: center}"]});function XJ(a){a&1&&Oq(0,"div",1);a&2&&(a=Y(),Up("top",a.top,"px")("left",a.left,"px")("width",a.width,"px")("height",a.height,"px")("border-color",a.borderColor)("background-color",a.bgColor))}
var YJ=class{constructor(a){this.G=a;this.F=this.D=this.C=this.ag=-1}start(a,b){a.preventDefault();a.stopPropagation();this.ag=a.offsetX;this.C=a.offsetY;this.D=this.ag;this.F=this.C;a=qk(document,"mousemove");const c=qk(window,"mouseup");let d=0,e=0;C(Uj([a]),Oo(c)).subscribe({next:([f])=>{d=f.offsetX-this.ag;e=f.offsetY-this.C;this.D=this.ag+d;this.F=this.C+e;fn(this.G)},complete:()=>{b(5>Math.abs(d)&&5>Math.abs(e),this.ag,this.C,this.D,this.F);this.F=this.D=this.C=this.ag=-1;fn(this.G)}})}get top(){return Math.min(this.C,
this.F)}get left(){return Math.min(this.ag,this.D)}get width(){return Math.abs(this.D-this.ag)}get height(){return Math.abs(this.F-this.C)}};YJ.K=function(a){return new (a||YJ)(y(Tr))};
YJ.Ca=Jf({type:YJ,ha:[["drag-area"]],inputs:{borderColor:"borderColor",bgColor:"bgColor"},ka:!0,features:[Nr],ya:1,za:1,Ba:[[1,"container",3,"top","left","width","height","border-color","background-color"],[1,"container"]],sa:function(a,b){a&1&&H(0,XJ,1,12,"div",0);a&2&&Gq(0<b.ag?0:-1)},Ha:[it],styles:[".container[_ngcontent-%COMP%]{position:absolute;z-index:5000;-moz-box-sizing:border-box;box-sizing:border-box;border-width:1px;border-style:solid;pointer-events:none}"],Pa:0});var ZJ=[Yy("transformPanel",[az("void",$y({opacity:0,transform:"scale(1, 0.8)"})),bz("void => showing",Zy("120ms cubic-bezier(0, 0, 0.2, 1)",$y({opacity:1,transform:"scale(1, 1)"}))),bz("* => void",Zy("100ms linear",$y({opacity:0})))])];function $J(a,b){let c=[b];c.push(...aK(a,b));uu(a.C,...c.map(d=>a.D(d)))}function bK(a,b){let c=[b];c.push(...aK(a,b));a.C.select(...c.map(d=>a.D(d)))}var cK=class{constructor(){this.C=new wu(!0)}toggle(a){this.C.toggle(this.D(a))}expand(a){this.C.select(this.D(a))}collapse(a){uu(this.C,this.D(a))}zl(a){return vu(this.C,this.D(a))}D(a){return this.mm?this.mm(a):a}};function aK(a,b){var c=a.F.indexOf(b);const d=[];for(c+=1;c<a.F.length&&a.Ee(b)<a.Ee(a.F[c]);c++)d.push(a.F[c]);return d}function dK(a){a.C.select(...a.F.map(b=>a.D(b)))}var eK=class extends cK{constructor(a,b){super();this.Ee=a;this.G=b;this.options=void 0}};var fK=new gf("CDK_TREE_NODE_OUTLET_NODE"),gK=class{constructor(a,b){this.gg=a;this.C=b}};gK.K=function(a){return new (a||gK)(y(np),y(fK,8))};gK.Fa=Uf({type:gK,ha:[["","cdkTreeNodeOutlet",""]],ka:!0});var hK=class{constructor(a){this.da=a}},iK=class{constructor(a){this.sa=a}};iK.K=function(a){return new (a||iK)(y(dr))};iK.Fa=Uf({type:iK,ha:[["","cdkTreeNodeDef",""]],inputs:{Uo:[0,"cdkTreeNodeDefWhen","when"]},ka:!0});function jK(a,b,c=a.O,d=a.Qi.gg,e){if(c=c.sn(b))Yr(c,(f,g,h)=>{if(null==f.Le){f=b[h];g=a.T(f,h);const m=new hK(f);a.rd.Ee?m.level=a.rd.Ee(f):"undefined"!==typeof e&&a.F.has(e)?m.level=a.F.get(e)+1:m.level=0;a.F.set(f,m.level);(d?d:a.Qi.gg).xd(g.sa,m,h);kK&&(kK.data=f)}else null==h?(d.remove(g),a.F.delete(f.item)):(f=d.get(g),d.move(f,h))}),fn(a.Da)}
var lK=class{get Hh(){return this.C}set Hh(a){this.C!==a&&this.aa(a)}constructor(a,b){this.J=a;this.Da=b;this.G=new hj;this.F=new Map;this.Py=new jj({start:0,end:Number.MAX_VALUE})}Fb(){this.O=this.J.find([]).create(this.mm)}va(){this.Qi.gg.clear();this.Py.complete();this.G.next();this.G.complete();this.D&&(this.D.unsubscribe(),this.D=null)}Yw(){this.P=this.Pi.filter(a=>!a.Uo)[0];this.Hh&&this.Pi&&!this.D&&this.N()}aa(a){this.D&&(this.D.unsubscribe(),this.D=null);a||this.Qi.gg.clear();this.C=a;this.Pi&&
this.N()}N(){let a;tu(this.C)?a=this.C.connect(this):Ik(this.C)?a=this.C:Array.isArray(this.C)&&(a=oj(this.C));a&&(this.D=C(a,Oo(this.G)).subscribe(b=>jK(this,b)))}T(a,b){return 1===this.Pi.length?this.Pi.first:this.Pi.find(c=>c.Uo&&c.Uo(b,a))||this.P}};lK.K=function(a){return new (a||lK)(y(gs),y(Tr))};
lK.Ca=Jf({type:lK,ha:[["cdk-tree"]],hc:function(a,b,c){a&1&&ur(c,iK,5);if(a&2){let d;wr(d=xr())&&(b.Pi=d)}},eb:function(a,b){a&1&&vr(gK,7);if(a&2){let c;wr(c=xr())&&(b.Qi=c.first)}},Ka:["role","tree",1,"cdk-tree"],inputs:{Hh:"dataSource",rd:"treeControl",mm:"trackBy"},ib:["cdkTree"],ka:!0,features:[Nr],ya:1,za:0,Ba:[["cdkTreeNodeOutlet",""]],sa:function(a){a&1&&Rq(0,0)},Ha:[gK],pb:2});
var nK=class{get role(){return"treeitem"}set role(a){this.ra.R.setAttribute("role",a)}get data(){return this.qh}set data(a){a!==this.qh&&(this.qh=a,this.VA(),this.vp.next())}get zl(){return this.Ff.rd.zl(this.qh)}get level(){return this.Ff.rd.Ee?this.Ff.rd.Ee(this.qh):this.MA}constructor(a,b){this.ra=a;this.Ff=b;this.Ga=new hj;this.vp=new hj;kK=this;this.role="treeitem"}Fb(){let a=this.ra.R.parentElement;for(;a&&!mK(a);)a=a.parentElement;this.MA=a?a.classList.contains("cdk-nested-tree-node")?Bs(a.getAttribute("aria-level")):
0:-1;this.ra.R.setAttribute("aria-level",`${this.level+1}`)}va(){kK===this&&(kK=null);this.vp.complete();this.Ga.next();this.Ga.complete()}focus(){this.ra.R.focus()}VA(){this.role="treeitem"}},kK=null;nK.K=function(a){return new (a||nK)(y(wi),y(lK))};nK.Fa=Uf({type:nK,ha:[["cdk-tree-node"]],Ka:[1,"cdk-tree-node"],Za:1,Ia:function(a,b){a&2&&Mp("aria-expanded",b.zl)},inputs:{role:"role"},ib:["cdkTreeNode"],ka:!0});
function mK(a){a=a.classList;return!!((null==a?0:a.contains("cdk-nested-tree-node"))||(null==a?0:a.contains("cdk-tree")))};const oK=/([A-Za-z%]+)$/;
var pK=class{get level(){return this.F}set level(a){this.O(a)}get yl(){return this.D}set yl(a){this.N(a)}constructor(a,b,c,d){this.J=a;this.Ff=b;this.T=c;this.G=d;this.Ga=new hj;this.P="px";this.D=40;this.C();d&&C(d.xb,Oo(this.Ga)).subscribe(()=>this.C(!0));a.vp.subscribe(()=>this.C())}va(){this.Ga.next();this.Ga.complete()}ca(){var a=this.J.data&&this.Ff.rd.Ee?this.Ff.rd.Ee(this.J.data):null;a=null==this.F?a:this.F;return"number"===typeof a?`${a*this.D}${this.P}`:null}C(a=!1){const b=this.ca();if(b!==
this.aa||a){a=this.T.R;const c=this.G&&"rtl"===this.G.value?"paddingRight":"paddingLeft";a.style[c]=b||"";a.style["paddingLeft"===c?"paddingRight":"paddingLeft"]="";this.aa=b}}O(a){this.F=isNaN(a)?null:a;this.C()}N(a){let b=a,c="px";"string"===typeof a&&(a=a.split(oK),b=a[0],c=a[1]||c);this.P=c;this.D=Bs(b);this.C()}};pK.K=function(a){return new (a||pK)(y(nK),y(lK),y(wi),y(pu,8))};
pK.Fa=Uf({type:pK,ha:[["","cdkTreeNodePadding",""]],inputs:{level:[2,"cdkTreeNodePadding","level",Bs],yl:[0,"cdkTreeNodePaddingIndent","indent"]},ka:!0,features:[Lp]});var qK=class{constructor(a,b){this.Ff=a;this.C=b;this.recursive=!1}cB(a){if(this.recursive){var b=this.Ff.rd,c=this.C.data;vu(b.C,b.D(c))?$J(b,c):bK(b,c)}else this.Ff.rd.toggle(this.C.data);a.stopPropagation()}};qK.K=function(a){return new (a||qK)(y(lK),y(nK))};qK.Fa=Uf({type:qK,ha:[["","cdkTreeNodeToggle",""]],Ia:function(a,b){a&1&&V("click",function(c){return b.cB(c)})},inputs:{recursive:[2,"cdkTreeNodeToggleRecursive","recursive",As]},ka:!0,features:[Lp]});var rK=class{};rK.K=function(a){return new (a||rK)};rK.Ya=Sf({type:rK});rK.Xa=Qd({});function sK(a,b){let c=[];b.forEach(d=>a.D(d,0,c,[]));return c}function tK(a,b,c){let d=[],e=[!0];b.forEach(f=>{let g=!0;for(let h=0;h<=a.Ee(f);h++)g=g&&e[h];g&&d.push(f);a.G(f)&&(e[a.Ee(f)+1]=c.zl(f))});return d}
var uK=class{constructor(a,b,c,d){this.J=a;this.Ee=b;this.G=c;this.F=d}D(a,b,c,d){const e=this.J(a,b);c.push(e);this.G(e)&&(a=this.F(a))&&(Array.isArray(a)?this.C(a,b,c,d):C(a,io()).subscribe(f=>{this.C(f,b,c,d)}));return c}C(a,b,c,d){a.forEach((e,f)=>{let g=d.slice();g.push(f!=a.length-1);this.D(e,b+1,c,g)})}},vK=class extends su{get data(){return this.qh.value}set data(a){this.qh.next(a);this.C.next(sK(this.G,this.data));this.D.F=this.C.value}constructor(a,b){super();this.D=a;this.G=b;this.C=new jj([]);
this.F=new jj([]);this.qh=new jj([])}connect(a){return C(wk(a.Py,this.D.C.Ch,this.C),Cj(()=>{this.F.next(tK(this.G,this.C.value,this.D));return this.F.value}))}};var wK=class extends nK{constructor(a,b,c){super(a,b);this.disabled=!1;this.tabIndex=Number(c)||0}Fb(){super.Fb()}va(){super.va()}};wK.K=function(a){return new (a||wK)(y(wi),y(lK),Dh("tabindex"))};wK.Fa=Uf({type:wK,ha:[["mat-tree-node"]],Ka:[1,"mat-tree-node"],inputs:{disabled:[2,"disabled","disabled",As],tabIndex:[2,"tabIndex","tabIndex",a=>null==a?0:Bs(a)]},ib:["matTreeNode"],ka:!0,features:[Lr([{pa:nK,Ab:wK}]),Lp,Ep]});var xK=class extends iK{};
xK.K=(()=>{let a;return function(b){return(a||(a=Lh(xK)))(b||xK)}})();xK.Fa=Uf({type:xK,ha:[["","matTreeNodeDef",""]],inputs:{Uo:[0,"matTreeNodeDefWhen","when"],data:[0,"matTreeNode","data"]},ka:!0,features:[Lr([{pa:iK,Ab:xK}]),Ep]});var yK=class{constructor(a,b){this.gg=a;this.C=b}};yK.K=function(a){return new (a||yK)(y(np),y(fK,8))};yK.Fa=Uf({type:yK,ha:[["","matTreeNodeOutlet",""]],ka:!0,features:[Lr([{pa:gK,Ab:yK}])]});var zK=class extends pK{get level(){return this.F}set level(a){this.O(a)}get yl(){return this.D}set yl(a){this.N(a)}};zK.K=(()=>{let a;return function(b){return(a||(a=Lh(zK)))(b||zK)}})();zK.Fa=Uf({type:zK,ha:[["","matTreeNodePadding",""]],inputs:{level:[2,"matTreeNodePadding","level",Bs],yl:[0,"matTreeNodePaddingIndent","indent"]},ka:!0,features:[Lr([{pa:pK,Ab:zK}]),Lp,Ep]});var AK=class extends qK{};AK.K=(()=>{let a;return function(b){return(a||(a=Lh(AK)))(b||AK)}})();AK.Fa=Uf({type:AK,ha:[["","matTreeNodeToggle",""]],inputs:{recursive:[0,"matTreeNodeToggleRecursive","recursive"]},ka:!0,features:[Lr([{pa:qK,Ab:AK}]),Ep]});var BK=class extends lK{constructor(){super(...arguments);this.Qi=void 0}};BK.K=(()=>{let a;return function(b){return(a||(a=Lh(BK)))(b||BK)}})();
BK.Ca=Jf({type:BK,ha:[["mat-tree"]],eb:function(a,b){a&1&&vr(yK,7);if(a&2){let c;wr(c=xr())&&(b.Qi=c.first)}},Ka:["role","tree",1,"mat-tree"],ib:["matTree"],ka:!0,features:[Lr([{pa:lK,Ab:BK}]),Ep,Nr],ya:1,za:0,Ba:[["matTreeNodeOutlet",""]],sa:function(a){a&1&&Rq(0,0)},Ha:[yK],styles:[".mat-tree{display:block;background-color:var(--mat-tree-container-background-color)}.mat-tree-node,.mat-nested-tree-node{color:var(--mat-tree-node-text-color);font-family:var(--mat-tree-node-text-font);font-size:var(--mat-tree-node-text-size);font-weight:var(--mat-tree-node-text-weight)}.mat-tree-node{display:flex;align-items:center;flex:1;word-wrap:break-word;min-height:var(--mat-tree-node-min-height)}.mat-nested-tree-node{border-bottom-width:0}\n"],pb:2});var CK=class{};CK.K=function(a){return new (a||CK)};CK.Ya=Sf({type:CK});CK.Xa=Qd({imports:[rK,mz,mz]});const DK=["tree"],EK=a=>({node:a});function FK(a){a&1&&Z(0);a&2&&(a=Y().da,Dr(" ",a.Zi," (",a.label,") "))}function GK(a){a&1&&Z(0);a&2&&(a=Y().da,Cr(" ",a.label," "))}function HK(a){a&1&&Rq(0)}function IK(a){a&1&&H(0,HK,1,0,"ng-container",14);if(a&2){a=Y().da;Y();const b=Ar(5);Q("ngTemplateOutlet",b)("ngTemplateOutletContext",Qr(EK,a))}}
function JK(a,b){a&1&&(R(0,"tr")(1,"td",17)(2,"mat-icon",18),Z(3,"circle"),U(),Z(4),U(),R(5,"td",19),Z(6),U()());if(a&2){a=b.da;b=Y(2).da;const c=Y();Wp("search-match",KK(c,b.metadata[a]));D(4);Cr(" ",a,"");D(2);Br(b.metadata[a])}}function LK(a){a&1&&(R(0,"table",15),H(1,JK,7,4,"tr",16),U());a&2&&(a=Y().da,Y(),D(),Q("ngForOf",MK(a)))}function NK(a){a&1&&(R(0,"div",20)(1,"mat-icon",21),Z(2,"my_location"),U()());a&2&&(a=Y(2),Q("matTooltip",a.Gl)("matTooltipPosition",a.Qs))}
function OK(a){a&1&&(R(0,"div",23),Z(1),U());a&2&&(a=Y(3),D(),Cr(" Showing at most ",a.ml()," elements "))}function PK(a){a&1&&(R(0,"div",22),H(1,OK,2,1,"div",23),R(2,"div",24),Z(3),U()());if(a&2){a=Y().da;const b=Y();D();Gq(0!==b.ml()?1:-1);D(2);a=a.node;Br(a?nI(a)?(a.attrs||{}).__value||"<empty>":"":"")}}
function QK(a,b){if(a&1){const c=Zg();R(0,"mat-tree-node",6)(1,"div",7);V("click",function(d){const e=t(c).da,f=Y();return x(f.nl(e,d.altKey))});R(2,"div",8)(3,"div",9)(4,"div",10)(5,"span",11);H(6,FK,1,2)(7,GK,1,1);U();H(8,IK,1,4,"ng-container");U()();H(9,LK,2,1,"table",12);U();H(10,NK,3,2,"div",13);U();H(11,PK,4,2,"ng-template",null,2,Rr);U()}if(a&2){a=b.da;b=Ar(12);const c=Y();Wp("has-locator",a.Dd)("has-values",RK(a))("highlight",a.Qn)("has-metadata",null!=a.metadata&&0<Object.keys(a.metadata).length);
Q("xapInlineDialog",b)("overlaySize",c.oq)("overlayPositions",c.xv)("hoverDelayMs",50)("xapInlineDialogDisabled",!RK(a))("matTreeNodePaddingIndent",c.Fy);Mp("data-id",a.nodeId);D(2);Wp("has-extra-data",null!=a.xg);D(3);Wp("color-bold",c.rv)("search-match",KK(c,a.label,a.Zi));D();Gq(a.Zi?6:7);D(2);Gq(a.xg?8:-1);D();Q("ngIf",null!=a.metadata&&0<Object.keys(a.metadata).length);D();Q("ngIf",a.Dd&&c.Dd)}}function SK(a){a&1&&Rq(0)}
function TK(a){a&1&&H(0,SK,1,0,"ng-container",14);if(a&2){a=Y().da;Y();const b=Ar(5);Q("ngTemplateOutlet",b)("ngTemplateOutletContext",Qr(EK,a))}}function UK(a){a&1&&(R(0,"div",20)(1,"mat-icon",21),Z(2,"my_location"),U()());a&2&&(a=Y(2),Q("matTooltip",a.Gl)("matTooltipPosition",a.Qs))}
function VK(a,b){if(a&1){const c=Zg();R(0,"mat-tree-node",25)(1,"div",26);V("click",function(d){const e=t(c).da,f=Y();return x(f.nl(e,d.altKey))});R(2,"div",9)(3,"button",27)(4,"mat-icon",28);Z(5);U()();R(6,"div",10);Z(7);H(8,TK,1,4,"ng-container");U()();H(9,UK,3,2,"div",13);U()()}a&2&&(a=b.da,b=Y(),Wp("has-locator",a.Dd),Q("matTreeNodePaddingIndent",b.Fy),Mp("data-id",a.nodeId),D(),Wp("highlight",a.eG),D(),Wp("has-extra-data",null!=a.xg),D(),Mp("aria-label","Toggle "+a.label),D(2),Cr(" ",b.rd.zl(a)?
"keyboard_arrow_down":"chevron_right"," "),D(2),Cr(" ",a.label," "),D(),Gq(a.xg?8:-1),D(),Q("ngIf",a.Dd&&b.Dd))}function WK(a,b){a&1&&(R(0,"div",30)(1,"mat-icon"),Z(2),U()());a&2&&(a=b.da,Y(2),Q("matTooltip",XK(a)),D(2),Br(a))}function YK(a,b){a&1&&(R(0,"div",29),Lq(1,WK,3,2,"div",30,Jq),U());a&2&&(a=b.node,D(),Nq(a.xg.matchTypes))}function KK(a,b,c){return a.D?null!=a.D.find(d=>d.jf===b||d.jf===c):!1}function MK(a){return Object.keys(a.metadata||{}).filter(b=>!b.startsWith("__")).sort()}
function RK(a){a=a.node;return a?nI(a)&&(a=a.attrs||{},a.__value)?"DATA_ELIDED"!==a.__value:!1:!1}function XK(a){switch(a){case "title":return"Node label matched";case "list":return"Node attribute(s) matched";case "input":return"Input(s) matched";case "output":return"Output(s) matched";default:return""}}function ZK(a,b){a.data=b;a.Hh.data=a.data;dK(a.rd);en(a.F)}
var $K=class{constructor(a,b){this.C=a;this.F=b;this.Lo=!1;this.rendererId="";this.Qs="left";this.Dd=!0;this.rv=!1;this.ne=new Jk;this.oq={minWidth:100};this.xv=[{bb:"start",cb:"top",Na:"end",Ua:"top"}];this.Gl="Click: locate\nAlt+click: select";this.G=(c,d)=>({nodeId:c.nodeId,node:c.node,An:!!c.children&&0<c.children.length,label:c.label,Zi:c.Zi,level:d,Dd:c.Dd,Qn:c.Qn,metadata:c.metadata,xg:c.xg});this.D=[];this.Fy=8;this.rd=new eK(c=>c.level,c=>c.An);this.J=new uK(this.G,c=>c.level,c=>c.An,c=>
c.children);this.Hh=new vK(this.rd,this.J)}uc(){null!=this.data&&(this.Hh.data=this.data,dK(this.rd))}nl(a,b){a.Dd&&(this.C.F.set({nodeId:a.nodeId||"",rendererId:this.rendererId,Cd:a.An,select:b}),this.ne.emit({}))}pC(a,b){return b.An}ml(){let a,b;return null!=(b=null==(a=this.C.config())?void 0:a.maxConstValueCount)?b:0}};$K.K=function(a){return new (a||$K)(y(VJ),y(Tr))};
$K.Ca=Jf({type:$K,ha:[["io-tree"]],eb:function(a,b){a&1&&vr(DK,5);if(a&2){let c;wr(c=xr())&&(b.Ey=c.first)}},inputs:{data:"data",Lo:"solidBackground",rendererId:"rendererId",Qs:"tooltipPosition",Dd:"showLocator",rv:"colorBoldNodeLabel"},outputs:{ne:"onClose"},ka:!0,features:[Hg,Nr],ya:6,za:7,Ba:[["tree",""],["matchTypes",""],["constValuesPopup",""],[1,"io-tree",3,"dataSource","treeControl"],["class","leaf","dialogLabel","const values popup","matTreeNodePadding","",3,"has-locator","has-values","highlight",
"has-metadata","xapInlineDialog","overlaySize","overlayPositions","hoverDelayMs","xapInlineDialogDisabled","matTreeNodePaddingIndent",4,"matTreeNodeDef"],["matTreeNodePadding","",3,"has-locator","matTreeNodePaddingIndent",4,"matTreeNodeDef","matTreeNodeDefWhen"],["dialogLabel","const values popup","matTreeNodePadding","",1,"leaf",3,"xapInlineDialog","overlaySize","overlayPositions","hoverDelayMs","xapInlineDialogDisabled","matTreeNodePaddingIndent"],[1,"edge-node",3,"click"],[1,"label-and-metadata"],
[1,"label-container"],[1,"label"],[1,"node-label"],["class","metadata-table",4,"ngIf"],["class","locator-container","matTooltipClass","multiline-tooltip-left",3,"matTooltip","matTooltipPosition",4,"ngIf"],[4,"ngTemplateOutlet","ngTemplateOutletContext"],[1,"metadata-table"],[3,"search-match",4,"ngFor","ngForOf"],[1,"key"],[1,"bullet"],[1,"value"],["matTooltipClass","multiline-tooltip-left",1,"locator-container",3,"matTooltip","matTooltipPosition"],[1,"locator-icon"],[1,"model-explorer-const-values-popup"],
[1,"values-count-msg"],[1,"content"],["matTreeNodePadding","",3,"matTreeNodePaddingIndent"],[1,"expandable-tree-node",3,"click"],["mat-icon-button","","matTreeNodeToggle","",1,"expand-icon-button"],[1,"expander"],[1,"match-types-container"],[1,"extra-label-container",3,"matTooltip"]],sa:function(a,b){a&1&&(R(0,"mat-tree",3,0),H(2,QK,13,25,"mat-tree-node",4)(3,VK,10,13,"mat-tree-node",5),U(),H(4,YK,3,0,"ng-template",null,1,Rr));a&2&&(Wp("solid-background",b.Lo),Q("dataSource",b.Hh)("treeControl",b.rd)("@transformPanel",
"showing")("@.disabled",!b.Lo),D(3),Q("matTreeNodeDefWhen",b.pC))},Ha:[it,$s,dt,ht,BB,AB,yB,PE,AE,NE,iI,hI,CK,xK,zK,AK,BK,wK,HG,EG],styles:[".io-tree[_ngcontent-%COMP%]{background-color:transparent;padding-top:4px}.io-tree.solid-background[_ngcontent-%COMP%]{-webkit-transform-origin:top center;transform-origin:top center;background-color:#fff;border:1px solid #ccc;border-radius:4px;padding-bottom:8px;min-width:270px;box-shadow:0 4px 6px -1px rgba(0,0,0,.1),0 2px 4px -2px rgba(0,0,0,.1)}.io-tree[_ngcontent-%COMP%]   mat-tree-node.has-locator[_ngcontent-%COMP%], .io-tree[_ngcontent-%COMP%]   mat-tree-node.has-values[_ngcontent-%COMP%]{cursor:pointer}.io-tree[_ngcontent-%COMP%]   mat-tree-node.has-locator[_ngcontent-%COMP%]:hover, .io-tree[_ngcontent-%COMP%]   mat-tree-node.has-values[_ngcontent-%COMP%]:hover{background-color:#f3f3f3}.io-tree[_ngcontent-%COMP%]   mat-tree-node.highlight[_ngcontent-%COMP%]   .label[_ngcontent-%COMP%]{color:#00639b;font-weight:500}.io-tree[_ngcontent-%COMP%]   mat-tree-node[_ngcontent-%COMP%]:not(:last-child).has-metadata.leaf{margin-bottom:6px}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:start;-webkit-align-items:flex-start;-moz-box-align:start;-ms-flex-align:start;align-items:flex-start;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;width:100%;-moz-box-sizing:border-box;box-sizing:border-box;padding-right:12px;padding-left:27px}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]:hover   .locator-container[_ngcontent-%COMP%]{opacity:.8}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]   .label-and-metadata.has-extra-data[_ngcontent-%COMP%]{width:100%}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]   .label-and-metadata.has-extra-data[_ngcontent-%COMP%]   .label[_ngcontent-%COMP%]{width:100%;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]   .label-and-metadata[_ngcontent-%COMP%]   .label-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]   .label-and-metadata[_ngcontent-%COMP%]   .label-container[_ngcontent-%COMP%]   .label[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]   .label-and-metadata[_ngcontent-%COMP%]   .label-container[_ngcontent-%COMP%]   .node-label.color-bold[_ngcontent-%COMP%]{color:#00639b;font-weight:500}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]   .label-and-metadata[_ngcontent-%COMP%]   .label-container[_ngcontent-%COMP%]   .node-label.search-match[_ngcontent-%COMP%]{background-color:#f5e25a}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]   .label-and-metadata[_ngcontent-%COMP%]   .hover-for-values-label[_ngcontent-%COMP%]{text-decoration:underline;-webkit-text-decoration-style:dotted;-moz-text-decoration-style:dotted;text-decoration-style:dotted;color:#999;margin-left:8px}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]   .label-and-metadata[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]{border-spacing:0;border-collapse:collapse;padding:0;font-size:11px;margin-top:2px}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]   .label-and-metadata[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   tr[_ngcontent-%COMP%]{vertical-align:text-top}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]   .label-and-metadata[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   tr.search-match[_ngcontent-%COMP%]{background-color:#f5e25a}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]   .label-and-metadata[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   tr[_ngcontent-%COMP%]   mat-icon.bullet[_ngcontent-%COMP%]{font-size:4px;width:5px;height:5px;color:#bbb;margin-right:2px}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]   .label-and-metadata[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   td.key[_ngcontent-%COMP%]{color:#999;padding-right:4px;white-space:nowrap;line-height:12px}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]   .label-and-metadata[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   td.value[_ngcontent-%COMP%]{line-height:12px}.io-tree[_ngcontent-%COMP%]   .expandable-tree-node[_ngcontent-%COMP%], .io-tree[_ngcontent-%COMP%]   mat-tree-node[_ngcontent-%COMP%]{font-size:13px;min-height:22px;word-break:break-word}.io-tree[_ngcontent-%COMP%]   .expandable-tree-node[_ngcontent-%COMP%]{color:#999;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;width:100%;-moz-box-sizing:border-box;box-sizing:border-box;padding-right:12px;padding-left:4px}.io-tree[_ngcontent-%COMP%]   .expandable-tree-node.highlight[_ngcontent-%COMP%]{color:rgba(0,0,0,.87)}.io-tree[_ngcontent-%COMP%]   .expandable-tree-node[_ngcontent-%COMP%]:hover   .locator-container[_ngcontent-%COMP%]{opacity:.8}.io-tree[_ngcontent-%COMP%]   .expandable-tree-node[_ngcontent-%COMP%]   mat-icon.expander[_ngcontent-%COMP%]{color:#999}.io-tree[_ngcontent-%COMP%]   .expandable-tree-node[_ngcontent-%COMP%]   .label-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.io-tree[_ngcontent-%COMP%]   .expandable-tree-node[_ngcontent-%COMP%]   .label-container.has-extra-data[_ngcontent-%COMP%]{width:100%}.io-tree[_ngcontent-%COMP%]   .expandable-tree-node[_ngcontent-%COMP%]   .label-container.has-extra-data[_ngcontent-%COMP%]   .label[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between}.io-tree[_ngcontent-%COMP%]   .expandable-tree-node[_ngcontent-%COMP%]   .label-container[_ngcontent-%COMP%]   .label[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.io-tree[_ngcontent-%COMP%]   .expand-icon-button[_ngcontent-%COMP%]{padding:0;width:24px;height:24px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center}.io-tree[_ngcontent-%COMP%]   .expand-icon-button[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:18px;margin-top:6px}.io-tree[_ngcontent-%COMP%]   .locator-container[_ngcontent-%COMP%]{height:18px;opacity:.5;cursor:pointer;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.io-tree[_ngcontent-%COMP%]   .locator-container[_ngcontent-%COMP%]   mat-icon.locator-icon[_ngcontent-%COMP%]{color:#333;font-size:16px;width:16px;height:16px}.io-tree[_ngcontent-%COMP%]   .locator-container[_ngcontent-%COMP%]:hover{opacity:.8}.io-tree[_ngcontent-%COMP%]   .match-types-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;justify-self:center;gap:2px;margin-left:8px}.io-tree[_ngcontent-%COMP%]   .extra-label-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;justify-self:center;padding:4px;border-radius:99px;background-color:#e8f0fe;-moz-box-sizing:border-box;box-sizing:border-box}.io-tree[_ngcontent-%COMP%]   .extra-label-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:12px;width:12px;height:12px;color:#9da3a9}.io-tree[_ngcontent-%COMP%]     .mat-mdc-button-touch-target{width:24px;height:24px}.io-tree-invisible[_ngcontent-%COMP%]{display:none}.io-tree[_ngcontent-%COMP%]   li[_ngcontent-%COMP%], .io-tree[_ngcontent-%COMP%]   ul[_ngcontent-%COMP%]{margin-top:0;margin-bottom:0;list-style-type:none}.io-tree[_ngcontent-%COMP%]   .mat-nested-tree-node[_ngcontent-%COMP%]   div[role=group][_ngcontent-%COMP%]{padding-left:8px}.io-tree[_ngcontent-%COMP%]   div[role=group][_ngcontent-%COMP%] > .mat-tree-node[_ngcontent-%COMP%]{padding-left:16px}  xap-inline-dialog-container:has(.model-explorer-const-values-popup){width:100%}  .model-explorer-const-values-popup{padding:8px;font-size:11px;line-height:12px;background-color:#fff}  .model-explorer-const-values-popup .values-count-msg{margin-bottom:8px;color:#999}  .model-explorer-const-values-popup .content{white-space:pre-wrap;font-family:monospace}"],
data:{animation:ZJ},Pa:0});
function aL(a,b,c,d=[]){var e=[];let f={label:"<root>",children:[],type:b,Cd:!0};const g=[];for(let m=0;m<a.length;m++){const p=a[m];var h=e[m];const u=d[m];if(nI(p)&&p.hideInLayout)g.push({node:p,index:m,metadata:h});else{const w=bL(p.savedNamespace||p.namespace,f);if(w){null==w.children&&(w.children=[]);const B=w.children.find(G=>G.label===p.label&&G.Cd);null==B?(h={label:p.label,Zi:cL(h),nodeId:p.id,node:p,type:b,Dd:!0,Qn:p.id===c,Cd:1===p.nodeType,metadata:h},null!=u&&(h.xg=u),w.children.push(h)):
(B.nodeId=p.id,B.Dd=!0,B.xg=u)}}}f=dL(f);a=[f];if(0<g.length){d="weights";for(const m of g)if(!m.node.label.toLowerCase().includes("const")){d="<hidden>";break}a.push({label:d,children:g.map(m=>({label:m.node.label,Zi:cL(m.metadata),nodeId:m.node.id,node:m.node,type:b,Dd:!1,Qn:m.node.id===c,Cd:!1,metadata:e[m.index]})),type:b,Cd:!1})}return a}function cL(a){if(a)return a.__tensor_tag}
function bL(a,b){var c=a?["<root>",...a.split("/")]:["<root>"];let d;a=[];for(const e of c)"<root>"!==e&&a.push(e),"<root>"===e?d=b:(d&&!d.children&&(d.children=[]),c=d.children.find(f=>f.label===e),null==c?(c={label:e,children:[],type:b.type,Cd:!0},d.children.push(c),d=c):d=c);return d}
function dL(a){let b=a;const c=[];for(;;){c.push(b.label);let d;if((null==(d=b)?0:d.children)&&1===b.children.length&&0<(b.children[0].children||[]).length&&!b.Dd)b=b.children[0];else break}b!==a&&(b.label=c.join(" / "));return b};var eL={COLOR:"COLOR",ep:"NUMBER"},fL=[{type:"COLOR",label:"Bg color",id:"node_bg_color",defaultValue:"#ffffff"},{type:"COLOR",label:"Border color",id:"node_border_color",defaultValue:"#777777"},{type:"COLOR",label:"Text color",id:"node_text_color",defaultValue:"#041e49"}];
function gL(a,b){const c=MI(b),d={};if(0<b.length&&b.some(e=>LI(e.queries))){const e=a.C.La();for(let f=0;f<e.length;f++){const g=e[f].modelGraph;if(g)for(const h of g.nodes)for(let m=0;m<c.length;m++){const p=c[m];if(LI(b[m].queries)&&NI(h,p.queries,g)){null==d[m]&&(d[m]={});null==d[m][f]&&(d[m][f]=[]);d[m][f].push(h);break}}}}a.D.set(d)}function hL(a,b){a.rules.set(iL(b))}
function jL(a){a.rules.update(b=>{b=[...b];b.push({queries:[{type:"node_type",nodeType:"op_nodes"},{type:"regex",queryRegex:"",matchTypes:["title"]}],nodeType:"op_nodes",styles:{},version:"v2"});return b})}function kL(a,b){a.rules.update(c=>{const d=[...c];c=c[b];d.splice(b,1);d.splice(b-1,0,c);return d})}function lL(a,b){a.rules.update(c=>{const d=[...c];c=c[b];d.splice(b,1);d.splice(b+1,0,c);return d})}
function mL(a,b){a.rules.update(c=>{const d=JSON.parse(JSON.stringify(c[b]));return[...c.slice(0,b),d,...c.slice(b)]})}function iL(a){return a.map(b=>{null==b.version&&(b.queries.unshift({type:"node_type",nodeType:b.nodeType}),b.version="v2");return b})}function nL(a,b,c){a.rules.update(d=>{d[b].queries=c;return[...d]})}
function oL(a,b,c,d){a.rules.update(e=>{const f=e[b];null==d?null==f.styles[c.id]?f.styles[c.id]={id:c.id,value:c.defaultValue}:delete f.styles[c.id]:d?f.styles[c.id]={id:c.id,value:c.defaultValue}:delete f.styles[c.id];return[...e]})}function pL(a,b,c){a.rules.update(d=>{d[b].nodeType=c;return[...d]})}function qL(a,b,c,d){a.rules.update(e=>{e[b].styles[c.id].value=d;return[...e]})}function rL(a,b){a.rules.update(c=>{c=[...c];c.splice(b,1);return c})}
var sL=class{constructor(a){this.C=a;this.rules=yr([]);this.D=yr({});this.ol=Cs(()=>0<this.rules().filter(b=>LI(b.queries)&&0<Object.keys(b.styles).length).length);Ks(()=>{const b=this.rules();this.C.C||PI("model_explorer_node_styler_rules",JSON.stringify(b));gL(this,b)},{fq:!0});this.C.C?(a=new URLSearchParams(document.location.search),a=decodeURIComponent(a.get("test_node_styler_rules")||""),hL(this,""===a?[]:JSON.parse(a))):(a=window.localStorage.getItem("model_explorer_node_styler_rules")||"",
a=""===a?[]:JSON.parse(a),hL(this,a))}};sL.K=function(a){return new (a||sL)(r(VJ),r(QI))};sL.na=Pd({ma:sL,ba:sL.K});function tL(a){a||(Dg(),a=of(Ci));const b=new bj(c=>a.ob(c.next.bind(c)));return c=>C(c,Oo(b))};function uL(a){Dg();const b=of(ai),c=new rj(1),d=Ks(()=>{let e;try{e=a()}catch(f){Ls(()=>c.error(f));return}Ls(()=>c.next(e))},{xa:b,ZC:!0});b.get(Ci).ob(()=>{d.destroy();c.complete()});return fj(c)};function vL(a,b,c){const d=wL(a,b);if(0<d.length){for(const e of d)null==e.results&&(e.results={}),null==e.results[b.id]&&null!=e.nodeDataProviderData&&(e.results[b.id]=xL(b,e.nodeDataProviderData));BJ(a.C,a.C.La()[c].id,d[0].runId)}}
function xL(a,b){yL(a);const c={};b=b[a.id];if(!b)return{};const d=[];for(var e of b.gradient||[]){var f={stop:e.stop};null!=e.bgColor&&(f.bgColor=zL(e.bgColor,"#ffffff"));null!=e.textColor&&(f.textColor=zL(e.textColor,"#000000"));d.push(f)}d.sort((m,p)=>m.stop-p.stop);e=Number.POSITIVE_INFINITY;f=Number.NEGATIVE_INFINITY;if(0<d.length)for(var {value:g}of Object.values(b.results))"number"===typeof g&&(e=Math.min(e,g),f=Math.max(f,g));for(const m of Object.keys(b.results)){g=b.results[m];"number"===
typeof g.value&&null==g.bgColor&&(g.bgColor=AL(g.value,b.thresholds||[],d,e,f));"number"===typeof g.value&&null==g.textColor&&(g.textColor=BL(g.value,b.thresholds||[],d,e,f));if((null==g.textColor||""===g.textColor)&&(g.bgColor||"").startsWith("#")){var h=zL(g.bgColor,"#ffffff");null!=h&&.38>.2126*Math.pow(h.r/255,2.2)+.7152*Math.pow(h.g/255,2.2)+.0722*Math.pow(h.b/255,2.2)&&(g.textColor="#ffffff")}h="-";h=g.value;h="number"===typeof h||"boolean"===typeof h||"string"===typeof h?`${h}`:JSON.stringify(h);
let p;const u=`${null!=(p=(a.outputTensorIdToNodeId||{})[m])?p:m}`,w=c[u];if(w){const B=w.allValues;B[m]=g.value;c[u]={value:g.value,bgColor:g.bgColor,textColor:g.textColor,allValues:B,strValue:`${w.strValue}, ${h}`}}else c[u]=Object.assign({},g,{strValue:h,allValues:{[m]:g.value}})}return c}
function CL(a,b,c,d,e,f=!1,g){const h=d.collectionLabel;a.D.update(m=>{if(f){var p=[];for(const [u,w]of Object.entries(m)){const B=u,G=w;f&&G.collectionId===h&&p.push(B)}for(const u of p)delete m[u]}m[b]={runId:b,runName:c,done:null==e?!1:!0,results:null==e?void 0:{[d.id]:xL(d,e)},extensionId:"",collectionId:h,remotePath:g,nodeDataProviderData:e};p=a.C.La();for(const u of p){let w,B;(null==(w=u.modelGraph)?void 0:w.id)===d.id&&(null==(B=u.modelGraph)?void 0:B.collectionLabel)===d.collectionLabel&&
BJ(a.C,u.id,b)}return Object.assign({},m)})}function DL(a,b,c,d,e){a.D.update(f=>{const g=f[b];g.done=!0;g.nodeDataProviderData=c;null==g.results&&(g.results={});g.results[d.id]=xL(d,c);e&&(g.error=e);return Object.assign({},f)})}function EL(a){const b=Object.values(a.D()).filter(c=>null!=c.remotePath).map(c=>c.remotePath);a.C.Og.set(b)}
function FL(a,b,c){return ta(function*(){a.gi.set(!0);var d=b.split("/"),e=d[d.length-1];let f;b.startsWith("node_data://")?(d=b.replace("node_data://","").split("/"),e=d[0],f=`${"/api/v1/load_node_data"}?node_data_index=${Number(d[1])}`):f=`${"/api/v1/read_text_file"}?path=${b}`;d=tI();CL(a,d,e,c,void 0,!1,b);e=yield fetch(f);if(e.ok)if(e=JSON.parse(yield e.text()),e.error)DL(a,d,{[c.id]:{results:{}}},c,`Failed to process JSON file. ${e.error}`);else try{const g=JSON.parse(e.content);e={};null!=
g.results&&null==g.results.results?c&&(e[c.id]=g):e=g;DL(a,d,e,c);EL(a)}catch(g){DL(a,d,{[c.id]:{results:{}}},c,`Failed to process JSON file. ${g}`)}else DL(a,d,{[c.id]:{results:{}}},c,`Failed to load JSON file "${b}"`);a.gi.set(!1)})}function wL(a,b){const c=[];a=a.D();for(const d of Object.values(a))d.collectionId===b.collectionLabel&&(a=d.nodeDataProviderData)&&null!=a[b.id]&&c.push(d);return c}
function GL(a,b){a.D.update(c=>{delete c[b];return Object.assign({},c)});EL(a);for(const c of a.C.La())if(c.As===b){const d=wL(a,c.modelGraph);BJ(a.C,c.id,0<d.length?d[0].runId:void 0)}}function HL(a,b,c){const d=CJ(a.C,b);if(d)return wL(a,c).find(e=>e.runId===d)}
function yL(a){if(null==a.outputTensorIdToNodeId){a.outputTensorIdToNodeId={};for(const b of a.nodes)if(nI(b)){const c=b.outputsMetadata||{};for(const d of Object.keys(c)){const e=c[d].tensor_name;null!=e&&(a.outputTensorIdToNodeId[e]=b.id)}}}}function zL(a,b){let c=a;a.startsWith("#")||(c=lI[a]);c||(c=b);c=c.replace("#","");return{r:IL(c.substring(0,2)),g:IL(c.substring(2,4)),b:IL(c.substring(4,6))}}
function AL(a,b,c,d,e){if(0<c.length)return JL(a,c,d,e,!0,"transparent");for(const f of b)if(a<=f.value)return f.bgColor;return"transparent"}function BL(a,b,c,d,e){if(0<c.length)return JL(a,c,d,e,!1,"");for(const f of b)if(a<=f.value)return f.textColor||"";return""}
function JL(a,b,c,d,e,f){d=(a-c)/(d-c);for(let g=0;g<b.length-1;g++){const h=b[g],m=b[g+1];a=e?h.bgColor:h.textColor;c=e?m.bgColor:m.textColor;if(d>=h.stop&&d<=m.stop){if(null==a||null==c)break;b=(d-h.stop)/(m.stop-h.stop);e=Math.floor(a.g+(c.g-a.g)*b);f=Math.floor(a.b+(c.b-a.b)*b);return`#${KL(Math.floor(a.r+(c.r-a.r)*b))}${KL(e)}${KL(f)}`}}return f}function KL(a){a=a.toString(16);return 1===a.length?`0${a}`:a}function IL(a){return/^[a-fA-F0-9]+$/.test(a)?parseInt(a,16):255}
var LL=class{constructor(a){this.C=a;this.D=yr({});this.gi=yr(!1);this.F=uL(Cs(()=>this.C.La()[0].modelGraph));this.G=uL(Cs(()=>{const b=this.C.La();if(!(1>=b.length))return b[1].modelGraph}));this.F.subscribe(b=>{null!=b&&vL(this,b,0)});this.G.subscribe(b=>{null!=b&&vL(this,b,1)})}};LL.K=function(a){return new (a||LL)(r(VJ))};LL.na=Pd({ma:LL,ba:LL.K});/*

Copyright 2010-2017 Mike Bostock
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

 Redistributions of source code must retain the above copyright notice, this
  list of conditions and the following disclaimer.

 Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.

 Neither the name of the author nor the names of contributors may be used to
  endorse or promote products derived from this software without specific prior
  written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
var ML,NL,OL,PL,QL,RL,wI,xI,SL,TL;
(function(){function a(l){return function(){return this.matches(l)}}function b(l,k){return l<k?-1:l>k?1:l>=k?0:NaN}function c(l){return function(k,n){return b(l(k),n)}}function d(){for(var l=0,k=arguments.length,n={},q;l<k;++l){if(!(q=arguments[l]+"")||q in n)throw Error("illegal type: "+q);n[q]=[]}return new e(n)}function e(l){this.Gb=l}function f(l,k){return l.trim().split(/^|\s+/).map(function(n){var q="",v=n.indexOf(".");0<=v&&(q=n.slice(v+1),n=n.slice(0,v));if(n&&!k.hasOwnProperty(n))throw Error("unknown type: "+
n);return{type:n,name:q}})}function g(l,k,n){for(var q=0,v=l.length;q<v;++q)if(l[q].name===k){l[q]=tU;l=l.slice(0,q).concat(l.slice(q+1));break}null!=n&&l.push({name:k,value:n});return l}function h(l){var k=l,n=k.indexOf(":");0<=n&&"xmlns"!==(k=l.slice(0,n))&&(l=l.slice(n+1));return Td.hasOwnProperty(k)?{space:Td[k],local:l}:l}function m(l){return function(){var k=this.ownerDocument,n=this.namespaceURI;return"http://www.w3.org/1999/xhtml"===n&&"http://www.w3.org/1999/xhtml"===k.documentElement.namespaceURI?
k.createElement(l):k.createElementNS(n,l)}}function p(l){return function(){return this.ownerDocument.createElementNS(l.space,l.local)}}function u(l){l=h(l);return(l.local?p:m)(l)}function w(){}function B(l){return null==l?w:function(){return this.querySelector(l)}}function G(l){return Array(l.length)}function L(l,k){this.ownerDocument=l.ownerDocument;this.namespaceURI=l.namespaceURI;this.Oa=null;this.Sa=l;this.__data__=k}function O(l){return function(){return l}}function ba(l,k,n,q,v,z){for(var A=
0,E,M=k.length,N=z.length;A<N;++A)(E=k[A])?(E.__data__=z[A],q[A]=E):n[A]=new L(l,z[A]);for(;A<M;++A)if(E=k[A])v[A]=E}function ca(l,k,n,q,v,z,A){var E,M,N={},P=k.length,X=z.length,la=Array(P),pa;for(E=0;E<P;++E)if(M=k[E])la[E]=pa="$"+A.call(M,M.__data__,E,k),pa in N?v[E]=M:N[pa]=M;for(E=0;E<X;++E)pa="$"+A.call(l,z[E],E,z),(M=N[pa])?(q[E]=M,M.__data__=z[E],N[pa]=null):n[E]=new L(l,z[E]);for(E=0;E<P;++E)(M=k[E])&&N[la[E]]===M&&(v[E]=M)}function na(l,k){return l<k?-1:l>k?1:l>=k?0:NaN}function K(l){return function(){this.removeAttribute(l)}}
function W(l){return function(){this.removeAttributeNS(l.space,l.local)}}function S(l,k){return function(){this.setAttribute(l,k)}}function ja(l,k){return function(){this.setAttributeNS(l.space,l.local,k)}}function F(l,k){return function(){var n=k.apply(this,arguments);null==n?this.removeAttribute(l):this.setAttribute(l,n)}}function I(l,k){return function(){var n=k.apply(this,arguments);null==n?this.removeAttributeNS(l.space,l.local):this.setAttributeNS(l.space,l.local,n)}}function aa(l){return function(){this.style.removeProperty(l)}}
function Aa(l,k,n){return function(){this.style.setProperty(l,k,n)}}function Za(l,k,n){return function(){var q=k.apply(this,arguments);null==q?this.style.removeProperty(l):this.style.setProperty(l,q,n)}}function vb(l,k){return l.style.getPropertyValue(k)||(l.ownerDocument&&l.ownerDocument.defaultView||l.document&&l||l.defaultView).getComputedStyle(l,null).getPropertyValue(k)}function kb(l){return function(){delete this[l]}}function jb(l,k){return function(){this[l]=k}}function bb(l,k){return function(){var n=
k.apply(this,arguments);null==n?delete this[l]:this[l]=n}}function ma(){this.textContent=""}function T(l){return function(){this.textContent=l}}function da(l){return function(){var k=l.apply(this,arguments);this.textContent=null==k?"":k}}function oa(){return null}function Ia(){var l=this.parentNode;l&&l.removeChild(this)}function db(){return this.parentNode.insertBefore(this.cloneNode(!1),this.nextSibling)}function Oa(){return this.parentNode.insertBefore(this.cloneNode(!0),this.nextSibling)}function tb(l,
k,n){l=wb(l,k,n);return function(q){var v=q.relatedTarget;v&&(v===this||v.compareDocumentPosition(this)&8)||l.call(this,q)}}function wb(l,k,n){return function(q){var v=ML;ML=q;try{l.call(this,this.__data__,k,n)}finally{ML=v}}}function sa(l){return l.trim().split(/^|\s+/).map(function(k){var n="",q=k.indexOf(".");0<=q&&(n=k.slice(q+1),k=k.slice(0,q));return{type:k,name:n}})}function Ha(l){return function(){var k=this.__on;if(k){for(var n=0,q=-1,v=k.length,z;n<v;++n)(z=k[n],l.type&&z.type!==l.type||
z.name!==l.name)?k[++q]=z:this.removeEventListener(z.type,z.listener,z.capture);++q?k.length=q:delete this.__on}}}function Wa(l,k,n){var q=QE.hasOwnProperty(l.type)?tb:wb;return function(v,z,A){v=this.__on;var E;z=q(k,z,A);if(v){A=0;for(var M=v.length;A<M;++A)if((E=v[A]).type===l.type&&E.name===l.name){this.removeEventListener(E.type,E.listener,E.capture);this.addEventListener(E.type,E.listener=z,E.capture=n);E.value=k;return}}this.addEventListener(l.type,z,n);E={type:l.type,name:l.name,value:k,listener:z,
capture:n};v?v.push(E):this.__on=[E]}}function cb(l,k){this.yc=l;this.ec=k}function Zb(){return new cb([[document.documentElement]],RE)}function Mc(l){return"string"===typeof l?new cb([[document.querySelector(l)]],[document.documentElement]):new cb([[l]],RE)}function pc(){this.Gb="@"+(++uU).toString(36)}function bc(){for(var l=ML,k;k=l.Mo;)l=k;return l}function Nc(l,k){var n=l.ownerSVGElement||l;if(n.createSVGPoint)return n=n.createSVGPoint(),n.x=k.clientX,n.y=k.clientY,n=n.matrixTransform(l.getScreenCTM().inverse()),
[n.x,n.y];n=l.getBoundingClientRect();return[k.clientX-n.left-l.clientLeft,k.clientY-n.top-l.clientTop]}function ve(l){var k=bc();k.changedTouches&&(k=k.changedTouches[0]);return Nc(l,k)}function Ed(l,k,n){3>arguments.length&&(n=k,k=bc().changedTouches);for(var q=0,v=k?k.length:0,z;q<v;++q)if((z=k[q]).identifier===n)return Nc(l,z);return null}function Dc(){ML.preventDefault();ML.stopImmediatePropagation()}function Fd(l){var k=l.document.documentElement;l=Mc(l).on("dragstart.drag",Dc,!0);if("onselectstart"in
k)l.on("selectstart.drag",Dc,!0);else k.wt=k.style.MozUserSelect,k.style.MozUserSelect="none"}function Mf(l,k){var n=l.document.documentElement,q=Mc(l).on("dragstart.drag",null);k&&(q.on("click.drag",Dc,!0),setTimeout(function(){q.on("click.drag",null)},0));if("onselectstart"in n)q.on("selectstart.drag",null);else n.style.MozUserSelect=n.wt,delete n.wt}function Nf(l,k,n){l.prototype=k.prototype=n;n.constructor=l}function Yc(l,k){l=Object.create(l.prototype);for(var n in k)l[n]=k[n];return l}function Ec(){}
function Oc(l){var k;l=(l+"").trim().toLowerCase();return(k=vU.exec(l))?(k=parseInt(k[1],16),new Wc(k>>8&15|k>>4&240,k>>4&15|k&240,(k&15)<<4|k&15,1)):(k=wU.exec(l))?Kg(parseInt(k[1],16)):(k=xU.exec(l))?new Wc(k[1],k[2],k[3],1):(k=yU.exec(l))?new Wc(255*k[1]/100,255*k[2]/100,255*k[3]/100,1):(k=zU.exec(l))?Uk(k[1],k[2],k[3],k[4]):(k=AU.exec(l))?Uk(255*k[1]/100,255*k[2]/100,255*k[3]/100,k[4]):(k=BU.exec(l))?SE(k[1],k[2]/100,k[3]/100,1):(k=CU.exec(l))?SE(k[1],k[2]/100,k[3]/100,k[4]):TE.hasOwnProperty(l)?
Kg(TE[l]):"transparent"===l?new Wc(NaN,NaN,NaN,0):null}function Kg(l){return new Wc(l>>16&255,l>>8&255,l&255,1)}function Uk(l,k,n,q){0>=q&&(l=k=n=NaN);return new Wc(l,k,n,q)}function jd(l){l instanceof Ec||(l=Oc(l));if(!l)return new Wc;l=l.qf();return new Wc(l.r,l.g,l.b,l.opacity)}function Of(l,k,n,q){return 1===arguments.length?jd(l):new Wc(l,k,n,null==q?1:q)}function Wc(l,k,n,q){this.r=+l;this.g=+k;this.b=+n;this.opacity=+q}function Tu(l){l=Math.max(0,Math.min(255,Math.round(l)||0));return(16>l?
"0":"")+l.toString(16)}function SE(l,k,n,q){0>=q?l=k=n=NaN:0>=n||1<=n?l=k=NaN:0>=k&&(l=NaN);return new Te(l,k,n,q)}function DU(l){if(l instanceof Te)return new Te(l.h,l.s,l.l,l.opacity);l instanceof Ec||(l=Oc(l));if(!l)return new Te;if(l instanceof Te)return l;l=l.qf();var k=l.r/255,n=l.g/255,q=l.b/255,v=Math.min(k,n,q),z=Math.max(k,n,q),A=NaN,E=z-v,M=(z+v)/2;E?(A=k===z?(n-q)/E+6*(n<q):n===z?(q-k)/E+2:(k-n)/E+4,E/=.5>M?z+v:2-z-v,A*=60):E=0<M&&1>M?0:A;return new Te(A,E,M,l.opacity)}function Uu(l,k,
n,q){return 1===arguments.length?DU(l):new Te(l,k,n,null==q?1:q)}function Te(l,k,n,q){this.h=+l;this.s=+k;this.l=+n;this.opacity=+q}function Vu(l,k,n){return 255*(60>l?k+(n-k)*l/60:180>l?n:240>l?k+(n-k)*(240-l)/60:k)}function Wu(l){if(l instanceof Ue)return new Ue(l.l,l.a,l.b,l.opacity);if(l instanceof Gf){if(isNaN(l.h))return new Ue(l.l,0,0,l.opacity);var k=l.h*UE;return new Ue(l.l,Math.cos(k)*l.c,Math.sin(k)*l.c,l.opacity)}l instanceof Wc||(l=jd(l));var n=Xu(l.r),q=Xu(l.g),v=Xu(l.b);k=Yu(.2225045*
n+.7168786*q+.0606169*v);if(n===q&&q===v)var z=n=k;else z=Yu((.4360747*n+.3850649*q+.1430804*v)/.96422),n=Yu((.0139322*n+.0971045*q+.7141733*v)/.82521);return new Ue(116*k-16,500*(z-k),200*(k-n),l.opacity)}function Ue(l,k,n,q){this.l=+l;this.a=+k;this.b=+n;this.opacity=+q}function Yu(l){return l>EU?Math.pow(l,1/3):l/VE+WE}function Zu(l){return l>ck?l*l*l:VE*(l-WE)}function $u(l){return 255*(.0031308>=l?12.92*l:1.055*Math.pow(l,1/2.4)-.055)}function Xu(l){return.04045>=(l/=255)?l/12.92:Math.pow((l+
.055)/1.055,2.4)}function FU(l){if(l instanceof Gf)return new Gf(l.h,l.c,l.l,l.opacity);l instanceof Ue||(l=Wu(l));if(0===l.a&&0===l.b)return new Gf(NaN,0,l.l,l.opacity);var k=Math.atan2(l.b,l.a)*XE;return new Gf(0>k?k+360:k,Math.sqrt(l.a*l.a+l.b*l.b),l.l,l.opacity)}function av(l,k,n,q){return 1===arguments.length?FU(l):new Gf(l,k,n,null==q?1:q)}function Gf(l,k,n,q){this.h=+l;this.c=+k;this.l=+n;this.opacity=+q}function Ve(l,k,n,q){if(1===arguments.length){var v=l;if(v instanceof Mh)v=new Mh(v.h,
v.s,v.l,v.opacity);else{v instanceof Wc||(v=jd(v));var z=v.g/255,A=v.b/255,E=(YE*A+v.r/255*-1.7884503806-3.5172982438*z)/(YE+-1.7884503806-3.5172982438);A-=E;var M=(1.97294*(z-E)- -.29227*A)/-.90649;A=(z=Math.sqrt(M*M+A*A)/(1.97294*E*(1-E)))?Math.atan2(M,A)*XE-120:NaN;v=new Mh(0>A?A+360:A,z,E,v.opacity)}}else v=new Mh(l,k,n,null==q?1:q);return v}function Mh(l,k,n,q){this.h=+l;this.s=+k;this.l=+n;this.opacity=+q}function ZE(l,k,n,q,v){var z=l*l,A=z*l;return((1-3*l+3*z-A)*k+(4-6*z+3*A)*n+(1+3*l+3*z-
3*A)*q+A*v)/6}function Yp(l){return function(){return l}}function $E(l,k){return function(n){return l+n*k}}function GU(l,k,n){return l=Math.pow(l,n),k=Math.pow(k,n)-l,n=1/n,function(q){return Math.pow(l+q*k,n)}}function bv(l,k){var n=k-l;return n?$E(l,180<n||-180>n?n-360*Math.round(n/360):n):Yp(isNaN(l)?k:l)}function HU(l){return 1===(l=+l)?pd:function(k,n){return n-k?GU(k,n,l):Yp(isNaN(k)?n:k)}}function pd(l,k){var n=k-l;return n?$E(l,n):Yp(isNaN(l)?k:l)}function aF(l){return function(k){var n=k.length,
q=Array(n),v=Array(n),z=Array(n),A;for(A=0;A<n;++A){var E=Of(k[A]);q[A]=E.r||0;v[A]=E.g||0;z[A]=E.b||0}q=l(q);v=l(v);z=l(z);E.opacity=1;return function(M){E.r=q(M);E.g=v(M);E.b=z(M);return E+""}}}function IU(l,k){var n=k?k.length:0,q=l?Math.min(n,l.length):0,v=Array(q),z=Array(n),A;for(A=0;A<q;++A)v[A]=cv(l[A],k[A]);for(;A<n;++A)z[A]=k[A];return function(E){for(A=0;A<q;++A)z[A]=v[A](E);return z}}function JU(l,k){var n=new Date;return l=+l,k-=l,function(q){return n.setTime(l+k*q),n}}function We(l,
k){return l=+l,k-=l,function(n){return l+k*n}}function KU(l,k){var n={},q={},v;if(null===l||"object"!==typeof l)l={};if(null===k||"object"!==typeof k)k={};for(v in k)v in l?n[v]=cv(l[v],k[v]):q[v]=k[v];return function(z){for(v in n)q[v]=n[v](z);return q}}function LU(l){return function(){return l}}function MU(l){return function(k){return l(k)+""}}function bF(l,k){var n=dv.lastIndex=ev.lastIndex=0,q,v,z,A=-1,E=[],M=[];l+="";for(k+="";(q=dv.exec(l))&&(v=ev.exec(k));)(z=v.index)>n&&(z=k.slice(n,z),E[A]?
E[A]+=z:E[++A]=z),(q=q[0])===(v=v[0])?E[A]?E[A]+=v:E[++A]=v:(E[++A]=null,M.push({i:A,x:We(q,v)})),n=ev.lastIndex;n<k.length&&(z=k.slice(n),E[A]?E[A]+=z:E[++A]=z);return 2>E.length?M[0]?MU(M[0].x):LU(k):(k=M.length,function(N){for(var P=0,X;P<k;++P)E[(X=M[P]).i]=X.x(N);return E.join("")})}function cv(l,k){var n=typeof k,q;return null==k||"boolean"===n?Yp(k):("number"===n?We:"string"===n?(q=Oc(k))?(k=q,Zp):bF:k instanceof Oc?Zp:k instanceof Date?JU:Array.isArray(k)?IU:"function"!==typeof k.valueOf&&
"function"!==typeof k.toString||isNaN(k)?KU:We)(l,k)}function cF(l,k,n,q,v,z){var A,E,M;if(A=Math.sqrt(l*l+k*k))l/=A,k/=A;if(M=l*n+k*q)n-=l*M,q-=k*M;if(E=Math.sqrt(n*n+q*q))n/=E,q/=E,M/=E;l*q<k*n&&(l=-l,k=-k,M=-M,A=-A);return{translateX:v,translateY:z,rotate:Math.atan2(k,l)*dF,skewX:Math.atan(M)*dF,ss:A,us:E}}function eF(l,k,n,q){function v(N){return N.length?N.pop()+" ":""}function z(N,P,X,la,pa,Ea){N!==X||P!==la?(pa=pa.push("translate(",null,k,null,n),Ea.push({i:pa-4,x:We(N,X)},{i:pa-2,x:We(P,la)})):
(X||la)&&pa.push("translate("+X+k+la+n)}function A(N,P,X,la){N!==P?(180<N-P?P+=360:180<P-N&&(N+=360),la.push({i:X.push(v(X)+"rotate(",null,q)-2,x:We(N,P)})):P&&X.push(v(X)+"rotate("+P+q)}function E(N,P,X,la){N!==P?la.push({i:X.push(v(X)+"skewX(",null,q)-2,x:We(N,P)}):P&&X.push(v(X)+"skewX("+P+q)}function M(N,P,X,la,pa,Ea){N!==X||P!==la?(pa=pa.push(v(pa)+"scale(",null,",",null,")"),Ea.push({i:pa-4,x:We(N,X)},{i:pa-2,x:We(P,la)})):1===X&&1===la||pa.push(v(pa)+"scale("+X+","+la+")")}return function(N,
P){var X=[],la=[];N=l(N);P=l(P);z(N.translateX,N.translateY,P.translateX,P.translateY,X,la);A(N.rotate,P.rotate,X,la);E(N.skewX,P.skewX,X,la);M(N.ss,N.us,P.ss,P.us,X,la);N=P=null;return function(pa){for(var Ea=-1,Sa=la.length,Va;++Ea<Sa;)X[(Va=la[Ea]).i]=Va.x(pa);return X.join("")}}}function fF(l){return((l=Math.exp(l))+1/l)/2}function NU(l,k){var n=l[0],q=l[1],v=l[2];l=k[2];var z=k[0]-n,A=k[1]-q,E=z*z+A*A;if(1E-12>E){var M=Math.log(l/v)/km;l=function(X){return[n+X*z,q+X*A,v*Math.exp(km*X*M)]}}else{var N=
Math.sqrt(E);k=(l*l-v*v+4*E)/(4*v*N);l=(l*l-v*v-4*E)/(4*l*N);var P=Math.log(Math.sqrt(k*k+1)-k);M=(Math.log(Math.sqrt(l*l+1)-l)-P)/km;l=function(X){X*=M;var la=fF(P),pa=km*X+P;var Ea=((pa=Math.exp(2*pa))-1)/(pa+1);var Sa=P;pa=((Sa=Math.exp(Sa))-1/Sa)/2;Ea=v/(2*N)*(la*Ea-pa);return[n+Ea*z,q+Ea*A,v*la/fF(km*X+P)]}}l.duration=1E3*M;return l}function gF(l){return function(k,n){var q=l((k=Uu(k)).h,(n=Uu(n)).h),v=pd(k.s,n.s),z=pd(k.l,n.l),A=pd(k.opacity,n.opacity);return function(E){k.h=q(E);k.s=v(E);k.l=
z(E);k.opacity=A(E);return k+""}}}function hF(l){return function(k,n){var q=l((k=av(k)).h,(n=av(n)).h),v=pd(k.c,n.c),z=pd(k.l,n.l),A=pd(k.opacity,n.opacity);return function(E){k.h=q(E);k.c=v(E);k.l=z(E);k.opacity=A(E);return k+""}}}function iF(l){return function q(n){function v(z,A){var E=l((z=Ve(z)).h,(A=Ve(A)).h),M=pd(z.s,A.s),N=pd(z.l,A.l),P=pd(z.opacity,A.opacity);return function(X){z.h=E(X);z.s=M(X);z.l=N(Math.pow(X,n));z.opacity=P(X);return z+""}}n=+n;v.gamma=q;return v}(1)}function $p(){return Nh||
(jF(OU),Nh=lm.now()+aq)}function OU(){Nh=0}function bq(){this.rk=this.Hk=this.Oa=null}function kF(l,k,n){var q=new bq;q.re(l,k,n);return q}function lF(){Nh=(cq=lm.now())+aq;dk=mm=0;try{$p();++dk;for(var l=dq,k;l;)0<=(k=Nh-l.Hk)&&l.rk.call(null,k),l=l.Oa;--dk}finally{dk=0;l=dq;for(var n=Infinity;l;)if(l.rk){n>l.Hk&&(n=l.Hk);var q=l;l=l.Oa}else k=l.Oa,l.Oa=null,l=q?q.Oa=k:dq=k;nm=q;fv(n);Nh=0}}function PU(){var l=lm.now(),k=l-cq;1E3<k&&(aq-=k,cq=l)}function fv(l){dk||(mm&&(mm=clearTimeout(mm)),24<l-
Nh?(Infinity>l&&(mm=setTimeout(lF,l-lm.now()-aq)),om&&(om=clearInterval(om))):(om||(cq=lm.now(),om=setInterval(PU,1E3)),dk=1,jF(lF)))}function mF(l,k,n){var q=new bq;k=null==k?0:+k;q.re(function(v){q.stop();l(v+k)},k,n);return q}function gv(l,k,n,q,v,z){var A=l.ig;if(!A)l.ig={};else if(n in A)return;QU(l,n,{name:k,index:q,group:v,on:RU,jd:SU,time:z.time,delay:z.delay,duration:z.duration,Of:z.Of,Xg:null,state:0})}function hv(l,k){l=Xe(l,k);if(0<l.state)throw Error("too late; already scheduled");return l}
function Oh(l,k){l=Xe(l,k);if(2<l.state)throw Error("too late; already started");return l}function Xe(l,k){l=l.ig;if(!l||!(l=l[k]))throw Error("transition not found");return l}function QU(l,k,n){function q(M){var N,P;if(1!==n.state)return z();for(la in A){var X=A[la];if(X.name===n.name){if(3===X.state)return mF(q);4===X.state?(X.state=6,X.Xg.stop(),X.on.call("interrupt",l,l.__data__,X.index,X.group),delete A[la]):+la<k&&(X.state=6,X.Xg.stop(),delete A[la])}}mF(function(){3===n.state&&(n.state=4,n.Xg.re(v,
n.delay,n.time),v(M))});n.state=2;n.on.call("start",l,l.__data__,n.index,n.group);if(2===n.state){n.state=3;E=Array(P=n.jd.length);var la=0;for(N=-1;la<P;++la)if(X=n.jd[la].value.call(l,l.__data__,n.index,n.group))E[++N]=X;E.length=N+1}}function v(M){M=M<n.duration?n.Of.call(null,M/n.duration):(n.Xg.re(z),n.state=5,1);for(var N=-1,P=E.length;++N<P;)E[N].call(null,M);5===n.state&&(n.on.call("end",l,l.__data__,n.index,n.group),z())}function z(){n.state=6;n.Xg.stop();delete A[k];for(var M in A)return;
delete l.ig}var A=l.ig,E;A[k]=n;n.Xg=kF(function(M){n.state=1;n.Xg.re(q,n.delay,n.time);n.delay<=M&&q(M-n.delay)},0,n.time)}function eq(l,k){var n=l.ig,q,v=!0,z;if(n){k=null==k?null:k+"";for(z in n)if((q=n[z]).name!==k)v=!1;else{var A=2<q.state&&5>q.state;q.state=6;q.Xg.stop();A&&q.on.call("interrupt",l,l.__data__,q.index,q.group);delete n[z]}v&&delete l.ig}}function TU(l,k){var n,q;return function(){var v=Oh(this,l),z=v.jd;if(z!==n){q=n=z;z=0;for(var A=q.length;z<A;++z)if(q[z].name===k){q=q.slice();
q.splice(z,1);break}}v.jd=q}}function UU(l,k,n){var q,v;if("function"!==typeof n)throw Error();return function(){var z=Oh(this,l),A=z.jd;if(A!==q){v=(q=A).slice();A={name:k,value:n};for(var E=0,M=v.length;E<M;++E)if(v[E].name===k){v[E]=A;break}E===M&&v.push(A)}z.jd=v}}function iv(l,k,n){var q=l.Zb;l.each(function(){var v=Oh(this,q);(v.value||(v.value={}))[k]=n.apply(this,arguments)});return function(v){return Xe(v,q).value[k]}}function nF(l,k){var n;return("number"===typeof k?We:k instanceof Oc?Zp:
(n=Oc(k))?(k=n,Zp):bF)(l,k)}function VU(l){return function(){this.removeAttribute(l)}}function WU(l){return function(){this.removeAttributeNS(l.space,l.local)}}function XU(l,k,n){var q,v;return function(){var z=this.getAttribute(l);return z===n?null:z===q?v:v=k(q=z,n)}}function YU(l,k,n){var q,v;return function(){var z=this.getAttributeNS(l.space,l.local);return z===n?null:z===q?v:v=k(q=z,n)}}function ZU(l,k,n){var q,v,z;return function(){var A=n(this);if(null==A)return void this.removeAttribute(l);
var E=this.getAttribute(l);return E===A?null:E===q&&A===v?z:z=k(q=E,v=A)}}function $U(l,k,n){var q,v,z;return function(){var A=n(this);if(null==A)return void this.removeAttributeNS(l.space,l.local);var E=this.getAttributeNS(l.space,l.local);return E===A?null:E===q&&A===v?z:z=k(q=E,v=A)}}function aV(l,k){function n(){var q=this,v=k.apply(q,arguments);return v&&function(z){q.setAttributeNS(l.space,l.local,v(z))}}n.Lc=k;return n}function bV(l,k){function n(){var q=this,v=k.apply(q,arguments);return v&&
function(z){q.setAttribute(l,v(z))}}n.Lc=k;return n}function cV(l,k){return function(){hv(this,l).delay=+k.apply(this,arguments)}}function dV(l,k){return k=+k,function(){hv(this,l).delay=k}}function eV(l,k){return function(){Oh(this,l).duration=+k.apply(this,arguments)}}function fV(l,k){return k=+k,function(){Oh(this,l).duration=k}}function gV(l,k){if("function"!==typeof k)throw Error();return function(){Oh(this,l).Of=k}}function hV(l){return(l+"").trim().split(/^|\s+/).every(function(k){var n=k.indexOf(".");
0<=n&&(k=k.slice(0,n));return!k||"start"===k})}function iV(l,k,n){var q,v,z=hV(k)?hv:Oh;return function(){var A=z(this,l),E=A.on;if(E!==q)(v=(q=E).copy()).on(k,n);A.on=v}}function jV(l){return function(){var k=this.parentNode,n;for(n in this.ig)if(+n!==l)return;k&&k.removeChild(this)}}function kV(l,k){var n,q,v;return function(){var z=vb(this,l),A=(this.style.removeProperty(l),vb(this,l));return z===A?null:z===n&&A===q?v:v=k(n=z,q=A)}}function lV(l){return function(){this.style.removeProperty(l)}}
function mV(l,k,n){var q,v;return function(){var z=vb(this,l);return z===n?null:z===q?v:v=k(q=z,n)}}function nV(l,k,n){var q,v,z;return function(){var A=vb(this,l),E=n(this);null==E&&(E=(this.style.removeProperty(l),vb(this,l)));return A===E?null:A===q&&E===v?z:z=k(q=A,v=E)}}function oV(l,k,n){function q(){var v=this,z=k.apply(v,arguments);return z&&function(A){v.style.setProperty(l,z(A),n)}}q.Lc=k;return q}function pV(l){return function(){this.textContent=l}}function qV(l){return function(){var k=
l(this);this.textContent=null==k?"":k}}function yg(l,k,n,q){this.yc=l;this.ec=k;this.th=n;this.Zb=q}function jv(l){return{type:l}}function kv(){this.rc=this.zc=this.Jb=this.Nb=null;this.Gb=""}function oF(){return new kv}function fq(){}function pF(l,k){var n=new fq;if(l instanceof fq)l.each(function(A,E){n.set(E,A)});else if(Array.isArray(l)){var q=-1,v=l.length,z;if(null==k)for(;++q<v;)n.set(q,l[q]);else for(;++q<v;)n.set(k(z=l[q],q,l),z)}else if(l)for(q in l)n.set(q,l[q]);return n}function gq(){}
function qF(l){return function(k){for(var n={},q=0;q<l.length;q++)n[l[q]]=k[q]||"";return n}}function rV(l,k){var n=qF(l);return function(q,v){return k(n(q),v,l)}}function sV(l){var k=Object.create(null),n=[];l.forEach(function(q){for(var v in q)v in k||n.push(k[v]=v)});return n}function rF(l){function k(A,E){function M(){if(Ea)return lv;if(Sa)return Sa=!1,sF;var kc,xb=X,lc;if(34===A.charCodeAt(xb)){for(;X++<P&&34!==A.charCodeAt(X)||34===A.charCodeAt(++X););(kc=X)>=P?Ea=!0:10===(lc=A.charCodeAt(X++))?
Sa=!0:13===lc&&(Sa=!0,10===A.charCodeAt(X)&&++X);return A.slice(xb+1,kc-1).replace(/""/g,'"')}for(;X<P;){if(10===(lc=A.charCodeAt(kc=X++)))Sa=!0;else if(13===lc)Sa=!0,10===A.charCodeAt(X)&&++X;else if(lc!==z)continue;return A.slice(xb,kc)}return Ea=!0,A.slice(xb,P)}var N=[],P=A.length,X=0,la=0,pa,Ea=0>=P,Sa=!1;10===A.charCodeAt(P-1)&&--P;for(13===A.charCodeAt(P-1)&&--P;(pa=M())!==lv;){for(var Va=[];pa!==sF&&pa!==lv;)Va.push(pa),pa=M();E&&null==(Va=E(Va,la++))||N.push(Va)}return N}function n(A){return A.map(q).join(l)}
function q(A){return null==A?"":v.test(A)?'"'+A.replace(/"/g,'""')+'"':A}var v=new RegExp('["'+l+"\n\r]"),z=l.charCodeAt(0);return{parse:function(A,E){var M,N;A=k(A,function(P,X){if(M)return M(P,X-1);N=P;M=E?rV(P,E):qF(P)});A.NF=N||[];return A},vG:k,format:function(A,E){null==E&&(E=sV(A));return[E.map(q).join(l)].concat(A.map(function(M){return E.map(function(N){return q(M[N])}).join(l)})).join("\n")},dG:function(A){return A.map(n).join("\n")}}}function tV(l){if(!l.ok)throw Error(l.status+" "+l.statusText);
return l.text()}function tF(l){return function(k,n,q){2===arguments.length&&"function"===typeof n&&(q=n,n=void 0);return fetch(k,n).then(tV).then(function(v){return l(v,q)})}}function uF(l,k,n,q){if(isNaN(k)||isNaN(n))return l;var v,z=l.vb;q={data:q};var A=l.rc,E=l.zc,M=l.Jb,N=l.Nb,P,X,la,pa,Ea;if(!z)return l.vb=q,l;for(;z.length;)if((la=k>=(P=(A+M)/2))?A=P:M=P,(pa=n>=(X=(E+N)/2))?E=X:N=X,v=z,!(z=z[Ea=pa<<1|la]))return v[Ea]=q,l;var Sa=+l.og.call(null,z.data);var Va=+l.pg.call(null,z.data);if(k===
Sa&&n===Va)return q.next=z,v?v[Ea]=q:l.vb=q,l;do v=v?v[Ea]=Array(4):l.vb=Array(4),(la=k>=(P=(A+M)/2))?A=P:M=P,(pa=n>=(X=(E+N)/2))?E=X:N=X;while((Ea=pa<<1|la)===(la=(Va>=X)<<1|Sa>=P));return v[la]=z,v[Ea]=q,l}function hd(l,k,n,q,v){this.node=l;this.Vo=k;this.Wo=n;this.x1=q;this.y1=v}function uV(l){return l[0]}function vV(l){return l[1]}function mv(l,k,n,q,v,z){this.og=l;this.pg=k;this.rc=n;this.zc=q;this.Jb=v;this.Nb=z;this.vb=void 0}function vF(l){for(var k={data:l.data},n=k;l=l.next;)n=n.next={data:l.data};
return k}function hq(l,k){if(0>(k=(l=k?l.toExponential(k-1):l.toExponential()).indexOf("e")))return null;var n=l.slice(0,k);return[1<n.length?n[0]+n.slice(2):n,+l.slice(k+1)]}function wV(l){return l=hq(Math.abs(l)),l?l[1]:NaN}function xV(l,k){return function(n,q){for(var v=n.length,z=[],A=0,E=l[0],M=0;0<v&&0<E;){M+E+1>q&&(E=Math.max(1,q-M));z.push(n.substring(v-=E,v+E));if((M+=E+1)>q)break;E=l[A=(A+1)%l.length]}return z.reverse().join(k)}}function yV(l){return function(k){return k.replace(/[0-9]/g,
function(n){return l[+n]})}}function nv(l){return new ov(l)}function ov(l){if(!(k=zV.exec(l)))throw Error("invalid format: "+l);var k;this.fill=k[1]||" ";this.align=k[2]||">";this.sign=k[3]||"-";this.symbol=k[4]||"";this.zero=!!k[5];this.width=k[6]&&+k[6];this.sv=!!k[7];this.precision=k[8]&&+k[8].slice(1);this.trim=!!k[9];this.type=k[10]||""}function wF(l,k){k=hq(l,k);if(!k)return l+"";l=k[0];k=k[1];return 0>k?"0."+Array(-k).join("0")+l:l.length>k+1?l.slice(0,k+1)+"."+l.slice(k+1):l+Array(k-l.length+
2).join("0")}function xF(l){return l}function AV(l){function k(E){function M(La){var Ib=lc,za=Kc,Fa;if("c"===xb)za=Gb(La)+za,La="";else{La=+La;var J=0>La;La=Gb(Math.abs(La),Va);if(kc){var ha=La.length;var Ga=1,Ra=-1,yb;a:for(;Ga<ha;++Ga)switch(La[Ga]){case ".":Ra=yb=Ga;break;case "0":0===Ra&&(Ra=Ga);yb=Ga;break;default:if(!+La[Ga])break a;0<Ra&&(Ra=0)}La=0<Ra?La.slice(0,Ra)+La.slice(yb+1):La}J&&0===+La&&(J=!1);Ib=(J?"("===X?X:"-":"-"===X||"("===X?"":X)+Ib;za=("s"===xb?yF[8+zF/3]:"")+za+(J&&"("===
X?")":"");if(ub)for(J=-1,ha=La.length;++J<ha;)if(Fa=La.charCodeAt(J),48>Fa||57<Fa){za=(46===Fa?v+La.slice(J+1):La.slice(J))+za;La=La.slice(0,J);break}}Sa&&!pa&&(La=n(La,Infinity));Fa=Ib.length+La.length+za.length;J=Fa<Ea?Array(Ea-Fa+1).join(N):"";Sa&&pa&&(La=n(J+La,J.length?Ea-za.length:Infinity),J="");switch(P){case "<":La=Ib+La+za+J;break;case "=":La=Ib+J+La+za;break;case "^":La=J.slice(0,Fa=J.length>>1)+Ib+La+za+J.slice(Fa);break;default:La=J+Ib+La+za}return z(La)}E=nv(E);var N=E.fill,P=E.align,
X=E.sign,la=E.symbol,pa=E.zero,Ea=E.width,Sa=E.sv,Va=E.precision,kc=E.trim,xb=E.type;"n"===xb?(Sa=!0,xb="g"):AF[xb]||(null==Va&&(Va=12),kc=!0,xb="g");if(pa||"0"===N&&"="===P)pa=!0,N="0",P="=";var lc="$"===la?q[0]:"#"===la&&/[boxX]/.test(xb)?"0"+xb.toLowerCase():"",Kc="$"===la?q[1]:/[%p]/.test(xb)?A:"",Gb=AF[xb],ub=/[defgprs%]/.test(xb);Va=null==Va?6:/[gprs]/.test(xb)?Math.max(1,Math.min(21,Va)):Math.max(0,Math.min(20,Va));M.toString=function(){return E+""};return M}var n=l.mw&&l.Ay?xV(l.mw,l.Ay):
xF,q=l.currency,v=l.KB,z=l.tD?yV(l.tD):xF,A=l.wG||"%";return{format:k,cG:function(E,M){var N=k((E=nv(E),E.type="f",E));E=3*Math.max(-8,Math.min(8,Math.floor(wV(M)/3)));var P=Math.pow(10,-E),X=yF[8+E/3];return function(la){return N(P*la)+X}}}}function ke(){this.reset()}function BF(l,k,n){var q=l.s=k+n,v=q-k;l.t=k-(q-v)+(n-v)}function iq(l){return 1<l?Ud:-1>l?-Ud:Math.asin(l)}function BV(){}function CF(l){var k=l[0];l=l[1];var n=Vd(l);return[n*Vd(k),n*ed(k),ed(l)]}function DF(l,k){return[l[1]*k[2]-
l[2]*k[1],l[2]*k[0]-l[0]*k[2],l[0]*k[1]-l[1]*k[0]]}function EF(l){var k=jq(l[0]*l[0]+l[1]*l[1]+l[2]*l[2]);l[0]/=k;l[1]/=k;l[2]/=k}function CV(){var l=[],k;return{point:function(n,q){k.push([n,q])},lc:function(){l.push(k=[])},kc:BV,AG:function(){1<l.length&&l.push(l.pop().concat(l.shift()))},result:function(){var n=l;l=[];k=null;return n}}}function kq(l,k,n,q){this.x=l;this.z=k;this.mo=n;this.e=q;this.v=!1;this.n=this.p=null}function DV(l,k,n,q,v){var z=[],A=[];l.forEach(function(X){if(!(0>=(la=X.length-
1))){var la,pa=X[0],Ea=X[la],Sa=pa;if(1E-6>zg(Sa[0]-Ea[0])&&1E-6>zg(Sa[1]-Ea[1])){v.lc();for(E=0;E<la;++E)v.point((pa=X[E])[0],pa[1]);v.kc()}else z.push(la=new kq(pa,X,null,!0)),A.push(la.mo=new kq(pa,null,la,!1)),z.push(la=new kq(Ea,X,null,!1)),A.push(la.mo=new kq(Ea,null,la,!0))}});if(z.length){A.sort(k);FF(z);FF(A);var E=0;for(l=A.length;E<l;++E)A[E].e=n=!n;n=z[0];for(var M;;){for(var N=n,P=!0;N.v;)if((N=N.n)===n)return;k=N.z;v.lc();do{N.v=N.mo.v=!0;if(N.e){if(P)for(E=0,l=k.length;E<l;++E)v.point((M=
k[E])[0],M[1]);else q(N.x,N.n.x,1,v);N=N.n}else{if(P)for(k=N.p.z,E=k.length-1;0<=E;--E)v.point((M=k[E])[0],M[1]);else q(N.x,N.p.x,-1,v);N=N.p}N=N.mo;k=N.z;P=!P}while(!N.v);v.kc()}}}function FF(l){if(k=l.length){for(var k,n=0,q=l[0],v;++n<k;)q.n=v=l[n],v.p=q,q=v;q.n=v=l[0];v.p=q}}function EV(l){return 1<l.length}function FV(l,k){return(0>(l=l.x)[0]?l[1]-Ud-1E-6:Ud-l[1])-(0>(k=k.x)[0]?k[1]-Ud-1E-6:Ud-k[1])}function pv(){}function GF(l){return function(k,n){var q=Vd(k),v=Vd(n);q=l(q*v);return[q*v*ed(k),
q*ed(n)]}}function pm(l){return function(k,n){var q=jq(k*k+n*n),v=l(q),z=ed(v);v=Vd(v);return[HF(k*z,q*v),iq(q&&n*z/q)]}}function GV(l){var k=0,n=l.children,q=n&&n.length;if(q)for(;0<=--q;)k+=n[q].value;else k=1;l.value=k}function IF(l,k){var n=new qm(l);l=+l.value&&(n.value=l.value);var q,v=[n],z,A,E,M;for(null==k&&(k=HV);q=v.pop();)if(l&&(q.value=+q.data.value),(A=k(q.data))&&(M=A.length))for(q.children=Array(M),E=M-1;0<=E;--E)v.push(z=q.children[E]=new qm(A[E])),z.parent=q,z.depth=q.depth+1;return n.Hq(IV)}
function HV(l){return l.children}function JV(l){l.data=l.data.data}function IV(l){var k=0;do l.height=k;while((l=l.parent)&&l.height<++k)}function qm(l){this.data=l;this.depth=this.height=0;this.parent=null}function Lc(l,k,n,q){function v(z){return l(z=new Date(+z)),z}v.floor=v;v.ceil=function(z){return l(z=new Date(z-1)),k(z,1),l(z),z};v.round=function(z){var A=v(z),E=v.ceil(z);return z-A<E-z?A:E};v.offset=function(z,A){return k(z=new Date(+z),null==A?1:Math.floor(A)),z};v.filter=function(z){return Lc(function(A){if(A>=
A)for(;l(A),!z(A);)A.setTime(A-1)},function(A,E){if(A>=A)if(0>E)for(;0>=++E;)for(;k(A,-1),!z(A););else for(;0<=--E;)for(;k(A,1),!z(A););})};n&&(v.count=function(z,A){qv.setTime(+z);rv.setTime(+A);l(qv);l(rv);return Math.floor(n(qv,rv))},v.every=function(z){z=Math.floor(z);return isFinite(z)&&0<z?1<z?v.filter(q?function(A){return 0===q(A)%z}:function(A){return 0===v.count(0,A)%z}):v:null});return v}function Ph(l){return Lc(function(k){k.setDate(k.getDate()-(k.getDay()+7-l)%7);k.setHours(0,0,0,0)},
function(k,n){k.setDate(k.getDate()+7*n)},function(k,n){return(n-k-6E4*(n.getTimezoneOffset()-k.getTimezoneOffset()))/6048E5})}function Qh(l){return Lc(function(k){k.setUTCDate(k.getUTCDate()-(k.getUTCDay()+7-l)%7);k.setUTCHours(0,0,0,0)},function(k,n){k.setUTCDate(k.getUTCDate()+7*n)},function(k,n){return(n-k)/6048E5})}function KV(l){if(0<=l.y&&100>l.y){var k=new Date(-1,l.me,l.d,l.vf,l.fh,l.mk,l.zi);k.setFullYear(l.y);return k}return new Date(l.y,l.me,l.d,l.vf,l.fh,l.mk,l.zi)}function lq(l){if(0<=
l.y&&100>l.y){var k=new Date(Date.UTC(-1,l.me,l.d,l.vf,l.fh,l.mk,l.zi));k.setUTCFullYear(l.y);return k}return new Date(Date.UTC(l.y,l.me,l.d,l.vf,l.fh,l.mk,l.zi))}function rm(l){return{y:l,me:0,d:1,vf:0,fh:0,mk:0,zi:0}}function LV(l){function k(za,Fa){return function(J){var ha=[],Ga=-1,Ra=0,yb=za.length,Pa,Tb;for(J instanceof Date||(J=new Date(+J));++Ga<yb;)if(37===za.charCodeAt(Ga)){ha.push(za.slice(Ra,Ga));null!=(Ra=JF[Pa=za.charAt(++Ga)])?Pa=za.charAt(++Ga):Ra="e"===Pa?" ":"0";if(Tb=Fa[Pa])Pa=
Tb(J,Ra);ha.push(Pa);Ra=Ga+1}ha.push(za.slice(Ra,Ga));return ha.join("")}}function n(za,Fa){return function(J){var ha=rm(1900);if(q(ha,za,J,0)!=J.length)return null;if("Q"in ha)return new Date(ha.Q);"p"in ha&&(ha.vf=ha.vf%12+12*ha.p);if("V"in ha){if(1>ha.um||53<ha.um)return null;"w"in ha||(ha.w=1);if("Z"in ha){J=lq(rm(ha.y));var Ga=J.getUTCDay();J=4<Ga||0===Ga?sv.ceil(J):sv(J);J=KF.offset(J,7*(ha.um-1));ha.y=J.getUTCFullYear();ha.me=J.getUTCMonth();ha.d=J.getUTCDate()+(ha.w+6)%7}else J=Fa(rm(ha.y)),
Ga=J.getDay(),J=4<Ga||0===Ga?tv.ceil(J):tv(J),J=LF.offset(J,7*(ha.um-1)),ha.y=J.getFullYear(),ha.me=J.getMonth(),ha.d=J.getDate()+(ha.w+6)%7}else if("W"in ha||"U"in ha)"w"in ha||(ha.w="u"in ha?ha.XE%7:"W"in ha?1:0),Ga="Z"in ha?lq(rm(ha.y)).getUTCDay():Fa(rm(ha.y)).getDay(),ha.me=0,ha.d="W"in ha?(ha.w+6)%7+7*ha.W-(Ga+5)%7:ha.w+7*ha.U-(Ga+6)%7;return"Z"in ha?(ha.vf+=ha.Z/100|0,ha.fh+=ha.Z%100,lq(ha)):Fa(ha)}}function q(za,Fa,J,ha){for(var Ga=0,Ra=Fa.length,yb=J.length,Pa;Ga<Ra;){if(ha>=yb)return-1;
Pa=Fa.charCodeAt(Ga++);if(37===Pa){if(Pa=Fa.charAt(Ga++),Pa=Ib[Pa in JF?Fa.charAt(Ga++):Pa],!Pa||0>(ha=Pa(za,J,ha)))return-1}else if(Pa!=J.charCodeAt(ha++))return-1}return ha}var v=l.dateTime,z=l.IB,A=l.time,E=l.GD,M=l.JB,N=l.eE,P=l.fD,X=l.fE,la=sm(E),pa=tm(E),Ea=sm(M),Sa=tm(M),Va=sm(N),kc=tm(N),xb=sm(P),lc=tm(P),Kc=sm(X),Gb=tm(X),ub={a:function(za){return N[za.getDay()]},A:function(za){return M[za.getDay()]},b:function(za){return X[za.getMonth()]},B:function(za){return P[za.getMonth()]},c:null,d:MF,
e:MF,f:MV,H:NV,I:OV,j:PV,L:NF,m:QV,M:RV,p:function(za){return E[+(12<=za.getHours())]},Q:OF,s:PF,S:SV,u:TV,U:UV,V:VV,w:WV,W:XV,x:null,X:null,y:YV,Y:ZV,Z:$V,"%":QF},La={a:function(za){return N[za.getUTCDay()]},A:function(za){return M[za.getUTCDay()]},b:function(za){return X[za.getUTCMonth()]},B:function(za){return P[za.getUTCMonth()]},c:null,d:RF,e:RF,f:aW,H:bW,I:cW,j:dW,L:SF,m:eW,M:fW,p:function(za){return E[+(12<=za.getUTCHours())]},Q:OF,s:PF,S:gW,u:hW,U:iW,V:jW,w:kW,W:lW,x:null,X:null,y:mW,Y:nW,
Z:oW,"%":QF},Ib={a:function(za,Fa,J){return(Fa=Va.exec(Fa.slice(J)))?(za.w=kc[Fa[0].toLowerCase()],J+Fa[0].length):-1},A:function(za,Fa,J){return(Fa=Ea.exec(Fa.slice(J)))?(za.w=Sa[Fa[0].toLowerCase()],J+Fa[0].length):-1},b:function(za,Fa,J){return(Fa=Kc.exec(Fa.slice(J)))?(za.me=Gb[Fa[0].toLowerCase()],J+Fa[0].length):-1},B:function(za,Fa,J){return(Fa=xb.exec(Fa.slice(J)))?(za.me=lc[Fa[0].toLowerCase()],J+Fa[0].length):-1},c:function(za,Fa,J){return q(za,v,Fa,J)},d:TF,e:TF,f:pW,H:UF,I:UF,j:qW,L:rW,
m:sW,M:tW,p:function(za,Fa,J){return(Fa=la.exec(Fa.slice(J)))?(za.p=pa[Fa[0].toLowerCase()],J+Fa[0].length):-1},Q:uW,s:vW,S:wW,u:xW,U:yW,V:zW,w:AW,W:BW,x:function(za,Fa,J){return q(za,z,Fa,J)},X:function(za,Fa,J){return q(za,A,Fa,J)},y:CW,Y:DW,Z:EW,"%":FW};ub.x=k(z,ub);ub.X=k(A,ub);ub.c=k(v,ub);La.x=k(z,La);La.X=k(A,La);La.c=k(v,La);return{format:function(za){var Fa=k(za,ub);Fa.toString=function(){return za};return Fa},parse:function(za){var Fa=n(za,KV);Fa.toString=function(){return za};return Fa},
aF:function(za){var Fa=k(za,La);Fa.toString=function(){return za};return Fa},bF:function(za){var Fa=n(za,lq);Fa.toString=function(){return za};return Fa}}}function Ub(l,k,n){var q=0>l?"-":"";l=(q?-l:l)+"";var v=l.length;return q+(v<n?Array(n-v+1).join(k)+l:l)}function GW(l){return l.replace(HW,"\\$&")}function sm(l){return new RegExp("^(?:"+l.map(GW).join("|")+")","i")}function tm(l){for(var k={},n=-1,q=l.length;++n<q;)k[l[n].toLowerCase()]=n;return k}function AW(l,k,n){return(k=Xc.exec(k.slice(n,
n+1)))?(l.w=+k[0],n+k[0].length):-1}function xW(l,k,n){return(k=Xc.exec(k.slice(n,n+1)))?(l.XE=+k[0],n+k[0].length):-1}function yW(l,k,n){return(k=Xc.exec(k.slice(n,n+2)))?(l.U=+k[0],n+k[0].length):-1}function zW(l,k,n){return(k=Xc.exec(k.slice(n,n+2)))?(l.um=+k[0],n+k[0].length):-1}function BW(l,k,n){return(k=Xc.exec(k.slice(n,n+2)))?(l.W=+k[0],n+k[0].length):-1}function DW(l,k,n){return(k=Xc.exec(k.slice(n,n+4)))?(l.y=+k[0],n+k[0].length):-1}function CW(l,k,n){return(k=Xc.exec(k.slice(n,n+2)))?
(l.y=+k[0]+(68<+k[0]?1900:2E3),n+k[0].length):-1}function EW(l,k,n){return(k=/^(Z)|([+-]\d\d)(?::?(\d\d))?/.exec(k.slice(n,n+6)))?(l.Z=k[1]?0:-(k[2]+(k[3]||"00")),n+k[0].length):-1}function sW(l,k,n){return(k=Xc.exec(k.slice(n,n+2)))?(l.me=k[0]-1,n+k[0].length):-1}function TF(l,k,n){return(k=Xc.exec(k.slice(n,n+2)))?(l.d=+k[0],n+k[0].length):-1}function qW(l,k,n){return(k=Xc.exec(k.slice(n,n+3)))?(l.me=0,l.d=+k[0],n+k[0].length):-1}function UF(l,k,n){return(k=Xc.exec(k.slice(n,n+2)))?(l.vf=+k[0],
n+k[0].length):-1}function tW(l,k,n){return(k=Xc.exec(k.slice(n,n+2)))?(l.fh=+k[0],n+k[0].length):-1}function wW(l,k,n){return(k=Xc.exec(k.slice(n,n+2)))?(l.mk=+k[0],n+k[0].length):-1}function rW(l,k,n){return(k=Xc.exec(k.slice(n,n+3)))?(l.zi=+k[0],n+k[0].length):-1}function pW(l,k,n){return(k=Xc.exec(k.slice(n,n+6)))?(l.zi=Math.floor(k[0]/1E3),n+k[0].length):-1}function FW(l,k,n){return(l=IW.exec(k.slice(n,n+1)))?n+l[0].length:-1}function uW(l,k,n){return(k=Xc.exec(k.slice(n)))?(l.Q=+k[0],n+k[0].length):
-1}function vW(l,k,n){return(k=Xc.exec(k.slice(n)))?(l.Q=1E3*+k[0],n+k[0].length):-1}function MF(l,k){return Ub(l.getDate(),k,2)}function NV(l,k){return Ub(l.getHours(),k,2)}function OV(l,k){return Ub(l.getHours()%12||12,k,2)}function PV(l,k){return Ub(1+LF.count(ek(l),l),k,3)}function NF(l,k){return Ub(l.getMilliseconds(),k,3)}function MV(l,k){return NF(l,k)+"000"}function QV(l,k){return Ub(l.getMonth()+1,k,2)}function RV(l,k){return Ub(l.getMinutes(),k,2)}function SV(l,k){return Ub(l.getSeconds(),
k,2)}function TV(l){l=l.getDay();return 0===l?7:l}function UV(l,k){return Ub(JW.count(ek(l),l),k,2)}function VV(l,k){var n=l.getDay();l=4<=n||0===n?uv(l):uv.ceil(l);return Ub(uv.count(ek(l),l)+(4===ek(l).getDay()),k,2)}function WV(l){return l.getDay()}function XV(l,k){return Ub(tv.count(ek(l),l),k,2)}function YV(l,k){return Ub(l.getFullYear()%100,k,2)}function ZV(l,k){return Ub(l.getFullYear()%1E4,k,4)}function $V(l){l=l.getTimezoneOffset();return(0<l?"-":(l*=-1,"+"))+Ub(l/60|0,"0",2)+Ub(l%60,"0",
2)}function RF(l,k){return Ub(l.getUTCDate(),k,2)}function bW(l,k){return Ub(l.getUTCHours(),k,2)}function cW(l,k){return Ub(l.getUTCHours()%12||12,k,2)}function dW(l,k){return Ub(1+KF.count(fk(l),l),k,3)}function SF(l,k){return Ub(l.getUTCMilliseconds(),k,3)}function aW(l,k){return SF(l,k)+"000"}function eW(l,k){return Ub(l.getUTCMonth()+1,k,2)}function fW(l,k){return Ub(l.getUTCMinutes(),k,2)}function gW(l,k){return Ub(l.getUTCSeconds(),k,2)}function hW(l){l=l.getUTCDay();return 0===l?7:l}function iW(l,
k){return Ub(KW.count(fk(l),l),k,2)}function jW(l,k){var n=l.getUTCDay();l=4<=n||0===n?vv(l):vv.ceil(l);return Ub(vv.count(fk(l),l)+(4===fk(l).getUTCDay()),k,2)}function kW(l){return l.getUTCDay()}function lW(l,k){return Ub(sv.count(fk(l),l),k,2)}function mW(l,k){return Ub(l.getUTCFullYear()%100,k,2)}function nW(l,k){return Ub(l.getUTCFullYear()%1E4,k,4)}function oW(){return"+0000"}function QF(){return"%"}function OF(l){return+l}function PF(l){return Math.floor(+l/1E3)}function qb(l){for(var k=l.length/
6|0,n=Array(k),q=0;q<k;)n[q]="#"+l.slice(6*q,6*++q);return n}function Yb(l){return LW(l[l.length-1])}function mq(l){var k=l.length;return function(n){return l[Math.max(0,Math.min(k-1,Math.floor(n*k)))]}}function wv(l){return function(){return l}}function VF(l){this.Db=l}function WF(l){return new VF(l)}function MW(l){return l[0]}function NW(l){return l[1]}function XF(l){this.Cm=l}function YF(l,k,n){var q=l.Jb-l.rc;k-=l.Jb;var v=(l.Nb-l.zc)/(q||0>k&&-0);l=(n-l.Nb)/(k||0>q&&-0);return((0>v?-1:1)+(0>
l?-1:1))*Math.min(Math.abs(v),Math.abs(l),.5*Math.abs((v*k+l*q)/(q+k)))||0}function ZF(l,k){var n=l.Jb-l.rc;return n?(3*(l.Nb-l.zc)/n-k)/2:k}function xv(l,k,n){var q=l.rc,v=l.Jb,z=l.Nb,A=(v-q)/3;l.Db.bezierCurveTo(q+A,l.zc+A*k,v-A,z-A*n,v,z)}function yv(l){this.Db=l}function $F(l){this.Db=new aG(l)}function aG(l){this.Db=l}function zv(l){return function(){return l}}function OW(l,k,n){this.target=l;this.type=k;this.transform=n}function Ag(l,k,n){this.k=l;this.x=k;this.y=n}function um(){ML.preventDefault();
ML.stopImmediatePropagation()}function PW(){return!ML.button}function QW(){var l=this;if(l instanceof SVGElement){l=l.ownerSVGElement||l;var k=l.width.baseVal.value;l=l.height.baseVal.value}else k=l.clientWidth,l=l.clientHeight;return[[0,0],[k,l]]}function bG(){return this.__zoom||cG}function RW(){return-ML.deltaY*(ML.deltaMode?120:1)/500}function SW(){return"ontouchstart"in this}function TW(l,k,n){var q=l.zw(k[0][0])-n[0][0],v=l.zw(k[1][0])-n[1][0],z=l.Aw(k[0][1])-n[0][1];k=l.Aw(k[1][1])-n[1][1];
return l.translate(v>q?(q+v)/2:Math.min(0,q)||Math.max(0,v),k>z?(z+k)/2:Math.min(0,z)||Math.max(0,k))}(function(l){1===l.length&&(l=c(l));return{left:function(k,n,q,v){null==q&&(q=0);null==v&&(v=k.length);for(;q<v;){var z=q+v>>>1;0>l(k[z],n)?q=z+1:v=z}return q},right:function(k,n,q,v){null==q&&(q=0);null==v&&(v=k.length);for(;q<v;){var z=q+v>>>1;0<l(k[z],n)?v=z:q=z+1}return q}}})(b);var tU={value:function(){}};e.prototype=d.prototype={constructor:e,on:function(l,k){var n=this.Gb,q=f(l+"",n),v,z=-1,
A=q.length;if(2>arguments.length)for(;++z<A;){var E;if(E=v=(l=q[z]).type){a:{E=n[v];for(var M=0,N=E.length;M<N;++M)if((v=E[M]).name===l.name){v=v.value;break a}v=void 0}E=v}if(E)return v}else{if(null!=k&&"function"!==typeof k)throw Error("invalid callback: "+k);for(;++z<A;)if(v=(l=q[z]).type)n[v]=g(n[v],l.name,k);else if(null==k)for(v in n)n[v]=g(n[v],l.name,null);return this}},copy:function(){var l={},k=this.Gb,n;for(n in k)l[n]=k[n].slice();return new e(l)},call:function(l,k){if(0<(v=arguments.length-
2))for(var n=Array(v),q=0,v,z;q<v;++q)n[q]=arguments[q+2];if(!this.Gb.hasOwnProperty(l))throw Error("unknown type: "+l);z=this.Gb[l];q=0;for(v=z.length;q<v;++q)z[q].value.apply(k,n)},apply:function(l,k,n){if(!this.Gb.hasOwnProperty(l))throw Error("unknown type: "+l);l=this.Gb[l];for(var q=0,v=l.length;q<v;++q)l[q].value.apply(k,n)}};var Td={svg:"http://www.w3.org/2000/svg",xhtml:"http://www.w3.org/1999/xhtml",xlink:"http://www.w3.org/1999/xlink",xml:"http://www.w3.org/XML/1998/namespace",xmlns:"http://www.w3.org/2000/xmlns/"};
if("undefined"!==typeof document){var vm=document.documentElement;if(!vm.matches){var UW=vm.webkitMatchesSelector||vm.msMatchesSelector||vm.mozMatchesSelector||vm.oMatchesSelector;a=function(l){return function(){return UW.call(this,l)}}}}var dG=a;L.prototype={constructor:L,appendChild:function(l){return this.Sa.insertBefore(l,this.Oa)},insertBefore:function(l,k){return this.Sa.insertBefore(l,k)},querySelector:function(l){return this.Sa.querySelector(l)},querySelectorAll:function(l){return this.Sa.querySelectorAll(l)}};
var QE={};ML=null;"undefined"!==typeof document&&("onmouseenter"in document.documentElement||(QE={mouseenter:"mouseover",mouseleave:"mouseout"}));var RE=[null];cb.prototype=Zb.prototype={constructor:cb,select:function(l){"function"!==typeof l&&(l=B(l));for(var k=this.yc,n=k.length,q=Array(n),v=0;v<n;++v)for(var z=k[v],A=z.length,E=q[v]=Array(A),M,N,P=0;P<A;++P)(M=z[P])&&(N=l.call(M,M.__data__,P,z))&&("__data__"in M&&(N.__data__=M.__data__),E[P]=N);return new cb(q,this.ec)},filter:function(l){"function"!==
typeof l&&(l=dG(l));for(var k=this.yc,n=k.length,q=Array(n),v=0;v<n;++v)for(var z=k[v],A=z.length,E=q[v]=[],M,N=0;N<A;++N)(M=z[N])&&l.call(M,M.__data__,N,z)&&E.push(M);return new cb(q,this.ec)},data:function(l,k){if(!l)return pa=Array(this.size()),N=-1,this.each(function(xb){pa[++N]=xb}),pa;var n=k?ca:ba,q=this.ec,v=this.yc;"function"!==typeof l&&(l=O(l));for(var z=v.length,A=Array(z),E=Array(z),M=Array(z),N=0;N<z;++N){var P=q[N],X=v[N],la=X.length,pa=l.call(P,P&&P.__data__,N,q),Ea=pa.length,Sa=E[N]=
Array(Ea),Va=A[N]=Array(Ea);la=M[N]=Array(la);n(P,X,Sa,Va,la,pa,k);X=P=0;for(var kc;P<Ea;++P)if(la=Sa[P]){for(P>=X&&(X=P+1);!(kc=Va[X])&&++X<Ea;);la.Oa=kc||null}}A=new cb(A,q);A.aA=E;A.bA=M;return A},xn:function(){return new cb(this.aA||this.yc.map(G),this.ec)},Mq:function(){return new cb(this.bA||this.yc.map(G),this.ec)},merge:function(l){var k=this.yc;l=l.yc;for(var n=k.length,q=Math.min(n,l.length),v=Array(n),z=0;z<q;++z)for(var A=k[z],E=l[z],M=A.length,N=v[z]=Array(M),P,X=0;X<M;++X)if(P=A[X]||
E[X])N[X]=P;for(;z<n;++z)v[z]=k[z];return new cb(v,this.ec)},order:function(){for(var l=this.yc,k=-1,n=l.length;++k<n;)for(var q=l[k],v=q.length-1,z=q[v],A;0<=--v;)if(A=q[v])z&&z!==A.nextSibling&&z.parentNode.insertBefore(A,z),z=A;return this},sort:function(l){function k(X,la){return X&&la?l(X.__data__,la.__data__):!X-!la}l||(l=na);for(var n=this.yc,q=n.length,v=Array(q),z=0;z<q;++z){for(var A=n[z],E=A.length,M=v[z]=Array(E),N,P=0;P<E;++P)if(N=A[P])M[P]=N;M.sort(k)}return(new cb(v,this.ec)).order()},
call:function(){var l=arguments[0];arguments[0]=this;l.apply(null,arguments);return this},nodes:function(){var l=Array(this.size()),k=-1;this.each(function(){l[++k]=this});return l},node:function(){for(var l=this.yc,k=0,n=l.length;k<n;++k)for(var q=l[k],v=0,z=q.length;v<z;++v){var A=q[v];if(A)return A}return null},size:function(){var l=0;this.each(function(){++l});return l},empty:function(){return!this.node()},each:function(l){for(var k=this.yc,n=0,q=k.length;n<q;++n)for(var v=k[n],z=0,A=v.length,
E;z<A;++z)(E=v[z])&&l.call(E,E.__data__,z,v);return this},attr:function(l,k){var n=h(l);if(2>arguments.length){var q=this.node();return n.local?q.getAttributeNS(n.space,n.local):q.getAttribute(n)}return this.each((null==k?n.local?W:K:"function"===typeof k?n.local?I:F:n.local?ja:S)(n,k))},style:function(l,k,n){return 1<arguments.length?this.each((null==k?aa:"function"===typeof k?Za:Aa)(l,k,null==n?"":n)):vb(this.node(),l)},Bx:function(l,k){return 1<arguments.length?this.each((null==k?kb:"function"===
typeof k?bb:jb)(l,k)):this.node()[l]},text:function(l){return arguments.length?this.each(null==l?ma:("function"===typeof l?da:T)(l)):this.node().textContent},append:function(l){var k="function"===typeof l?l:u(l);return this.select(function(){return this.appendChild(k.apply(this,arguments))})},kr:function(l,k){var n="function"===typeof l?l:u(l),q=null==k?oa:"function"===typeof k?k:B(k);return this.select(function(){return this.insertBefore(n.apply(this,arguments),q.apply(this,arguments)||null)})},
remove:function(){return this.each(Ia)},clone:function(l){return this.select(l?Oa:db)},on:function(l,k,n){var q=sa(l+""),v=q.length,z;if(2>arguments.length){var A=this.node().__on;if(A)for(var E=0,M=A.length,N;E<M;++E){var P=0;for(N=A[E];P<v;++P)if((z=q[P]).type===N.type&&z.name===N.name)return N.value}}else{A=k?Wa:Ha;null==n&&(n=!1);for(P=0;P<v;++P)this.each(A(q[P],k,n));return this}}};var uU=0;pc.prototype=function(){return new pc}.prototype={constructor:pc,get:function(l){for(var k=this.Gb;!(k in
l);)if(!(l=l.parentNode))return;return l[k]},set:function(l,k){return l[this.Gb]=k},remove:function(l){return this.Gb in l&&delete l[this.Gb]},toString:function(){return this.Gb}};var gk=1/.7,vU=/^#([0-9a-f]{3})$/,wU=/^#([0-9a-f]{6})$/,xU=RegExp("^rgb\\(\\s*([+-]?\\d+)\\s*,\\s*([+-]?\\d+)\\s*,\\s*([+-]?\\d+)\\s*\\)$"),yU=RegExp("^rgb\\(\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*,\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*,\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*\\)$"),zU=RegExp("^rgba\\(\\s*([+-]?\\d+)\\s*,\\s*([+-]?\\d+)\\s*,\\s*([+-]?\\d+)\\s*,\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)\\s*\\)$"),
AU=RegExp("^rgba\\(\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*,\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*,\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*,\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)\\s*\\)$"),BU=RegExp("^hsl\\(\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)\\s*,\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*,\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*\\)$"),CU=RegExp("^hsla\\(\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)\\s*,\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*,\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*,\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)\\s*\\)$"),
TE={aliceblue:15792383,antiquewhite:16444375,aqua:65535,aquamarine:8388564,azure:15794175,beige:16119260,bisque:16770244,black:0,blanchedalmond:16772045,blue:255,blueviolet:9055202,brown:10824234,burlywood:14596231,cadetblue:6266528,chartreuse:8388352,chocolate:13789470,coral:16744272,cornflowerblue:6591981,cornsilk:16775388,crimson:14423100,cyan:65535,darkblue:139,darkcyan:35723,darkgoldenrod:12092939,darkgray:11119017,darkgreen:25600,darkgrey:11119017,darkkhaki:12433259,darkmagenta:9109643,darkolivegreen:5597999,
darkorange:16747520,darkorchid:10040012,darkred:9109504,darksalmon:15308410,darkseagreen:9419919,darkslateblue:4734347,darkslategray:3100495,darkslategrey:3100495,darkturquoise:52945,darkviolet:9699539,deeppink:16716947,deepskyblue:49151,dimgray:6908265,dimgrey:6908265,dodgerblue:2003199,firebrick:11674146,floralwhite:16775920,forestgreen:2263842,fuchsia:16711935,gainsboro:14474460,ghostwhite:16316671,gold:16766720,goldenrod:14329120,gray:8421504,green:32768,greenyellow:11403055,grey:8421504,honeydew:15794160,
hotpink:16738740,indianred:13458524,indigo:4915330,ivory:16777200,khaki:15787660,lavender:15132410,lavenderblush:16773365,lawngreen:8190976,lemonchiffon:16775885,lightblue:11393254,lightcoral:15761536,lightcyan:14745599,lightgoldenrodyellow:16448210,lightgray:13882323,lightgreen:9498256,lightgrey:13882323,lightpink:16758465,lightsalmon:16752762,lightseagreen:2142890,lightskyblue:8900346,lightslategray:7833753,lightslategrey:7833753,lightsteelblue:11584734,lightyellow:16777184,lime:65280,limegreen:3329330,
linen:16445670,magenta:16711935,maroon:8388608,mediumaquamarine:6737322,mediumblue:205,mediumorchid:12211667,mediumpurple:9662683,mediumseagreen:3978097,mediumslateblue:8087790,mediumspringgreen:64154,mediumturquoise:4772300,mediumvioletred:13047173,midnightblue:1644912,mintcream:16121850,mistyrose:16770273,moccasin:16770229,navajowhite:16768685,navy:128,oldlace:16643558,olive:8421376,olivedrab:7048739,orange:16753920,orangered:16729344,orchid:14315734,palegoldenrod:15657130,palegreen:10025880,paleturquoise:11529966,
palevioletred:14381203,papayawhip:16773077,peachpuff:16767673,peru:13468991,pink:16761035,plum:14524637,powderblue:11591910,purple:8388736,rebeccapurple:6697881,red:16711680,rosybrown:12357519,royalblue:4286945,saddlebrown:9127187,salmon:16416882,sandybrown:16032864,seagreen:3050327,seashell:16774638,sienna:10506797,silver:12632256,skyblue:8900331,slateblue:6970061,slategray:7372944,slategrey:7372944,snow:16775930,springgreen:65407,steelblue:4620980,tan:13808780,teal:32896,thistle:14204888,tomato:16737095,
turquoise:4251856,violet:15631086,wheat:16113331,white:16777215,whitesmoke:16119285,yellow:16776960,yellowgreen:10145074};Nf(Ec,Oc,{Gq:function(){return this.qf().Gq()},sw:function(){return this.qf().sw()},toString:function(){return this.qf()+""}});Nf(Wc,Of,Yc(Ec,{cn:function(l){l=null==l?gk:Math.pow(gk,l);return new Wc(this.r*l,this.g*l,this.b*l,this.opacity)},nn:function(l){l=null==l?.7:Math.pow(.7,l);return new Wc(this.r*l,this.g*l,this.b*l,this.opacity)},qf:function(){return this},Gq:function(){return 0<=
this.r&&255>=this.r&&0<=this.g&&255>=this.g&&0<=this.b&&255>=this.b&&0<=this.opacity&&1>=this.opacity},sw:function(){return"#"+Tu(this.r)+Tu(this.g)+Tu(this.b)},toString:function(){var l=this.opacity;l=isNaN(l)?1:Math.max(0,Math.min(1,l));return(1===l?"rgb(":"rgba(")+Math.max(0,Math.min(255,Math.round(this.r)||0))+", "+Math.max(0,Math.min(255,Math.round(this.g)||0))+", "+Math.max(0,Math.min(255,Math.round(this.b)||0))+(1===l?")":", "+l+")")}}));Nf(Te,Uu,Yc(Ec,{cn:function(l){l=null==l?gk:Math.pow(gk,
l);return new Te(this.h,this.s,this.l*l,this.opacity)},nn:function(l){l=null==l?.7:Math.pow(.7,l);return new Te(this.h,this.s,this.l*l,this.opacity)},qf:function(){var l=this.h%360+360*(0>this.h),k=this.l,n=k+(.5>k?k:1-k)*(isNaN(l)||isNaN(this.s)?0:this.s);k=2*k-n;return new Wc(Vu(240<=l?l-240:l+120,k,n),Vu(l,k,n),Vu(120>l?l+240:l-120,k,n),this.opacity)},Gq:function(){return(0<=this.s&&1>=this.s||isNaN(this.s))&&0<=this.l&&1>=this.l&&0<=this.opacity&&1>=this.opacity}}));var UE=Math.PI/180,XE=180/
Math.PI,WE=4/29,ck=6/29,VE=3*ck*ck,EU=ck*ck*ck;Nf(Ue,function(l,k,n,q){return 1===arguments.length?Wu(l):new Ue(l,k,n,null==q?1:q)},Yc(Ec,{cn:function(l){return new Ue(this.l+18*(null==l?1:l),this.a,this.b,this.opacity)},nn:function(l){return new Ue(this.l-18*(null==l?1:l),this.a,this.b,this.opacity)},qf:function(){var l=(this.l+16)/116,k=isNaN(this.a)?l:l+this.a/500,n=isNaN(this.b)?l:l-this.b/200;k=.96422*Zu(k);l=1*Zu(l);n=.82521*Zu(n);return new Wc($u(3.1338561*k-1.6168667*l-.4906146*n),$u(-.9787684*
k+1.9161415*l+.033454*n),$u(.0719453*k-.2289914*l+1.4052427*n),this.opacity)}}));Nf(Gf,av,Yc(Ec,{cn:function(l){return new Gf(this.h,this.c,this.l+18*(null==l?1:l),this.opacity)},nn:function(l){return new Gf(this.h,this.c,this.l-18*(null==l?1:l),this.opacity)},qf:function(){return Wu(this).qf()}}));var YE=1.78277*-.29227-.1347134789;Nf(Mh,Ve,Yc(Ec,{cn:function(l){l=null==l?gk:Math.pow(gk,l);return new Mh(this.h,this.s,this.l*l,this.opacity)},nn:function(l){l=null==l?.7:Math.pow(.7,l);return new Mh(this.h,
this.s,this.l*l,this.opacity)},qf:function(){var l=isNaN(this.h)?0:(this.h+120)*UE,k=+this.l,n=isNaN(this.s)?0:this.s*k*(1-k),q=Math.cos(l);l=Math.sin(l);return new Wc(255*(k+n*(-.14861*q+1.78277*l)),255*(k+n*(-.29227*q+-.90649*l)),255*(k+1.97294*n*q),this.opacity)}}));var Zp=function n(k){function q(z,A){var E=v((z=Of(z)).r,(A=Of(A)).r),M=v(z.g,A.g),N=v(z.b,A.b),P=pd(z.opacity,A.opacity);return function(X){z.r=E(X);z.g=M(X);z.b=N(X);z.opacity=P(X);return z+""}}var v=HU(k);q.gamma=n;return q}(1),
LW=aF(function(k){var n=k.length-1;return function(q){var v=0>=q?q=0:1<=q?(q=1,n-1):Math.floor(q*n),z=k[v],A=k[v+1];return ZE((q-v/n)*n,0<v?k[v-1]:2*z-A,z,A,v<n-1?k[v+2]:2*A-z)}});aF(function(k){var n=k.length;return function(q){var v=Math.floor((0>(q%=1)?++q:q)*n);return ZE((q-v/n)*n,k[(v+n-1)%n],k[v%n],k[(v+1)%n],k[(v+2)%n])}});var dv=/[-+]?(?:\d+\.?\d*|\.?\d+)(?:[eE][-+]?\d+)?/g,ev=new RegExp(dv.source,"g"),dF=180/Math.PI,Av={translateX:0,translateY:0,rotate:0,skewX:0,ss:1,us:1},wm,Bv,eG,nq,VW=
eF(function(k){if("none"===k)return Av;wm||(wm=document.createElement("DIV"),Bv=document.documentElement,eG=document.defaultView);wm.style.transform=k;k=eG.getComputedStyle(Bv.appendChild(wm),null).getPropertyValue("transform");Bv.removeChild(wm);k=k.slice(7,-1).split(",");return cF(+k[0],+k[1],+k[2],+k[3],+k[4],+k[5])},"px, ","px)","deg)"),WW=eF(function(k){if(null==k)return Av;nq||(nq=document.createElementNS("http://www.w3.org/2000/svg","g"));nq.setAttribute("transform",k);if(!(k=nq.transform.baseVal.consolidate()))return Av;
k=k.matrix;return cF(k.a,k.b,k.c,k.d,k.e,k.f)},", ",")",")"),km=Math.SQRT2;gF(bv);gF(pd);hF(bv);hF(pd);iF(bv);var Cv=iF(pd),dk=0,mm=0,om=0,dq,nm,cq=0,Nh=0,aq=0,lm="object"===typeof performance&&performance.now?performance:Date,jF="object"===typeof window&&window.requestAnimationFrame?window.requestAnimationFrame.bind(window):function(k){setTimeout(k,17)};bq.prototype=kF.prototype={constructor:bq,re:function(k,n,q){if("function"!==typeof k)throw new TypeError("callback is not a function");q=(null==
q?$p():+q)+(null==n?0:+n);this.Oa||nm===this||(nm?nm.Oa=this:dq=this,nm=this);this.rk=k;this.Hk=q;fv()},stop:function(){this.rk&&(this.rk=null,this.Hk=Infinity,fv())}};var RU=d("start","end","interrupt"),SU=[],XW=Zb.prototype.constructor,fG=0,hk=Zb.prototype;yg.prototype=function(k){return Zb().transition(k)}.prototype={constructor:yg,select:function(k){var n=this.th,q=this.Zb;"function"!==typeof k&&(k=B(k));for(var v=this.yc,z=v.length,A=Array(z),E=0;E<z;++E)for(var M=v[E],N=M.length,P=A[E]=Array(N),
X,la,pa=0;pa<N;++pa)(X=M[pa])&&(la=k.call(X,X.__data__,pa,M))&&("__data__"in X&&(la.__data__=X.__data__),P[pa]=la,gv(P[pa],n,q,pa,P,Xe(X,q)));return new yg(A,this.ec,n,q)},filter:function(k){"function"!==typeof k&&(k=dG(k));for(var n=this.yc,q=n.length,v=Array(q),z=0;z<q;++z)for(var A=n[z],E=A.length,M=v[z]=[],N,P=0;P<E;++P)(N=A[P])&&k.call(N,N.__data__,P,A)&&M.push(N);return new yg(v,this.ec,this.th,this.Zb)},merge:function(k){if(k.Zb!==this.Zb)throw Error();var n=this.yc;k=k.yc;for(var q=n.length,
v=Math.min(q,k.length),z=Array(q),A=0;A<v;++A)for(var E=n[A],M=k[A],N=E.length,P=z[A]=Array(N),X,la=0;la<N;++la)if(X=E[la]||M[la])P[la]=X;for(;A<q;++A)z[A]=n[A];return new yg(z,this.ec,this.th,this.Zb)},selection:function(){return new XW(this.yc,this.ec)},transition:function(){for(var k=this.th,n=this.Zb,q=++fG,v=this.yc,z=v.length,A=0;A<z;++A)for(var E=v[A],M=E.length,N,P=0;P<M;++P)if(N=E[P]){var X=Xe(N,n);gv(N,k,q,P,E,{time:X.time+X.delay+X.duration,delay:0,duration:X.duration,Of:X.Of})}return new yg(v,
this.ec,k,q)},call:hk.call,nodes:hk.nodes,node:hk.node,size:hk.size,empty:hk.empty,each:hk.each,on:function(k,n){var q=this.Zb;return 2>arguments.length?Xe(this.node(),q).on.on(k):this.each(iV(q,k,n))},attr:function(k,n){var q=h(k),v="transform"===q?WW:nF;return this.oB(k,"function"===typeof n?(q.local?$U:ZU)(q,v,iv(this,"attr."+k,n)):null==n?(q.local?WU:VU)(q):(q.local?YU:XU)(q,v,n+""))},oB:function(k,n){var q="attr."+k;if(2>arguments.length)return(q=this.jd(q))&&q.Lc;if(null==n)return this.jd(q,
null);if("function"!==typeof n)throw Error();var v=h(k);return this.jd(q,(v.local?aV:bV)(v,n))},style:function(k,n,q){var v="transform"===k?VW:nF;return null==n?this.ty(k,kV(k,v)).on("end.style."+k,lV(k)):this.ty(k,"function"===typeof n?nV(k,v,iv(this,"style."+k,n)):mV(k,v,n+""),q)},ty:function(k,n,q){var v="style."+k;if(2>arguments.length)return(v=this.jd(v))&&v.Lc;if(null==n)return this.jd(v,null);if("function"!==typeof n)throw Error();return this.jd(v,oV(k,n,null==q?"":q))},text:function(k){return this.jd("text",
"function"===typeof k?qV(iv(this,"text",k)):pV(null==k?"":k+""))},remove:function(){return this.on("end.remove",jV(this.Zb))},jd:function(k,n){var q=this.Zb;if(2>arguments.length){q=Xe(this.node(),q).jd;for(var v=0,z=q.length,A;v<z;++v)if((A=q[v]).name===k)return A.value;return null}return this.each((null==n?TU:UU)(q,k,n))},delay:function(k){var n=this.Zb;return arguments.length?this.each(("function"===typeof k?cV:dV)(n,k)):Xe(this.node(),n).delay},duration:function(k){var n=this.Zb;return arguments.length?
this.each(("function"===typeof k?eV:fV)(n,k)):Xe(this.node(),n).duration},Of:function(k){var n=this.Zb;return arguments.length?this.each(gV(n,k)):Xe(this.node(),n).Of}};var Dv={time:null,delay:0,duration:250,Of:function(k){return(1>=(k*=2)?k*k*k:(k-=2)*k*k+2)/2}};Zb.prototype.HC=function(){return this.each(function(){eq(this,void 0)})};Zb.prototype.transition=function(k){var n;if(k instanceof yg){var q=k.Zb;k=k.th}else q=++fG,(n=Dv).time=$p(),k=null==k?null:k+"";for(var v=this.yc,z=v.length,A=0;A<
z;++A)for(var E=v[A],M=E.length,N,P=0;P<M;++P)if(N=E[P]){var X=N,la=k,pa=q,Ea=P,Sa=E,Va;if(!(Va=n))a:{Va=void 0;for(var kc=q;!(Va=N.ig)||!(Va=Va[kc]);)if(!(N=N.parentNode)){Va=(Dv.time=$p(),Dv);break a}}gv(X,la,pa,Ea,Sa,Va)}return new yg(v,this.ec,k,q)};["e","w"].map(jv);["n","s"].map(jv);"n e s w nw ne se sw".split(" ").map(jv);kv.prototype=oF.prototype={constructor:kv,moveTo:function(k,n){this.Gb+="M"+(this.rc=this.Jb=+k)+","+(this.zc=this.Nb=+n)},closePath:function(){null!==this.Jb&&(this.Jb=this.rc,
this.Nb=this.zc,this.Gb+="Z")},lineTo:function(k,n){this.Gb+="L"+(this.Jb=+k)+","+(this.Nb=+n)},bezierCurveTo:function(k,n,q,v,z,A){this.Gb+="C"+ +k+","+ +n+","+ +q+","+ +v+","+(this.Jb=+z)+","+(this.Nb=+A)},rect:function(k,n,q,v){this.Gb+="M"+(this.rc=this.Jb=+k)+","+(this.zc=this.Nb=+n)+"h"+ +q+"v"+ +v+"h"+-q+"Z"},toString:function(){return this.Gb}};fq.prototype=pF.prototype={constructor:fq,has:function(k){return" "+k in this},get:function(k){return this[" "+k]},set:function(k,n){this[" "+k]=n;
return this},remove:function(k){k=" "+k;return k in this&&delete this[k]},clear:function(){for(var k in this)" "===k[0]&&delete this[k]},keys:function(){var k=[],n;for(n in this)" "===n[0]&&k.push(n.slice(1));return k},values:function(){var k=[],n;for(n in this)" "===n[0]&&k.push(this[n]);return k},entries:function(){var k=[],n;for(n in this)" "===n[0]&&k.push({key:n.slice(1),value:this[n]});return k},size:function(){var k=0,n;for(n in this)" "===n[0]&&++k;return k},empty:function(){for(var k in this)if(" "===
k[0])return!1;return!0},each:function(k){for(var n in this)" "===n[0]&&k(this[n],n.slice(1),this)}};var Rh=pF.prototype;gq.prototype=function(k,n){var q=new gq;if(k instanceof gq)k.each(function(A){q.add(A)});else if(k){var v=-1,z=k.length;if(null==n)for(;++v<z;)q.add(k[v]);else for(;++v<z;)q.add(n(k[v],v,k))}return q}.prototype={constructor:gq,has:Rh.has,add:function(k){this[" "+k]=k;return this},remove:Rh.remove,clear:Rh.clear,values:Rh.keys,size:Rh.size,empty:Rh.empty,each:Rh.each};var sF={},lv=
{},YW=rF(",").parse,ZW=rF("\t").parse;tF(YW);tF(ZW);var id=function(k,n,q){n=new mv(null==n?uV:n,null==q?vV:q,NaN,NaN,NaN,NaN);return null==k?n:n.addAll(k)}.prototype=mv.prototype;id.copy=function(){var k=new mv(this.og,this.pg,this.rc,this.zc,this.Jb,this.Nb),n=this.vb,q,v;if(!n)return k;if(!n.length)return k.vb=vF(n),k;for(q=[{source:n,target:k.vb=Array(4)}];n=q.pop();)for(var z=0;4>z;++z)if(v=n.source[z])v.length?q.push({source:v,target:n.target[z]=Array(4)}):n.target[z]=vF(v);return k};id.add=
function(k){var n=+this.og.call(null,k),q=+this.pg.call(null,k);return uF(this.Yk(n,q),n,q,k)};id.addAll=function(k){var n,q,v=k.length,z,A,E=Array(v),M=Array(v),N=Infinity,P=Infinity,X=-Infinity,la=-Infinity;for(q=0;q<v;++q)isNaN(z=+this.og.call(null,n=k[q]))||isNaN(A=+this.pg.call(null,n))||(E[q]=z,M[q]=A,z<N&&(N=z),z>X&&(X=z),A<P&&(P=A),A>la&&(la=A));X<N&&(N=this.rc,X=this.Jb);la<P&&(P=this.zc,la=this.Nb);this.Yk(N,P).Yk(X,la);for(q=0;q<v;++q)uF(this,E[q],M[q],k[q]);return this};id.Yk=function(k,
n){if(isNaN(k=+k)||isNaN(n=+n))return this;var q=this.rc,v=this.zc,z=this.Jb,A=this.Nb;if(isNaN(q))z=(q=Math.floor(k))+1,A=(v=Math.floor(n))+1;else if(q>k||k>z||v>n||n>A){var E=z-q,M=this.vb,N;switch(N=(n<(v+A)/2)<<1|k<(q+z)/2){case 0:do{var P=Array(4);P[N]=M;M=P}while(E*=2,z=q+E,A=v+E,k>z||n>A);break;case 1:do P=Array(4),P[N]=M,M=P;while(E*=2,q=z-E,A=v+E,q>k||n>A);break;case 2:do P=Array(4),P[N]=M,M=P;while(E*=2,z=q+E,v=A-E,k>z||v>n);break;case 3:do P=Array(4),P[N]=M,M=P;while(E*=2,q=z-E,v=A-E,q>
k||v>n)}this.vb&&this.vb.length&&(this.vb=M)}else return this;this.rc=q;this.zc=v;this.Jb=z;this.Nb=A;return this};id.data=function(){var k=[];this.Sy(function(n){if(!n.length){do k.push(n.data);while(n=n.next)}});return k};id.hl=function(k){return arguments.length?this.Yk(+k[0][0],+k[0][1]).Yk(+k[1][0],+k[1][1]):isNaN(this.rc)?void 0:[[this.rc,this.zc],[this.Jb,this.Nb]]};id.find=function(k,n,q){var v=this.rc,z=this.zc,A,E,M,N,P=this.Jb,X=this.Nb,la=[],pa=this.vb,Ea;pa&&la.push(new hd(pa,v,z,P,X));
null==q?q=Infinity:(v=k-q,z=n-q,P=k+q,X=n+q,q*=q);for(;Ea=la.pop();)if(!(!(pa=Ea.node)||(A=Ea.Vo)>P||(E=Ea.Wo)>X||(M=Ea.x1)<v||(N=Ea.y1)<z))if(pa.length){Ea=(A+M)/2;var Sa=(E+N)/2;la.push(new hd(pa[3],Ea,Sa,M,N),new hd(pa[2],A,Sa,Ea,N),new hd(pa[1],Ea,E,M,Sa),new hd(pa[0],A,E,Ea,Sa));if(pa=(n>=Sa)<<1|k>=Ea)Ea=la[la.length-1],la[la.length-1]=la[la.length-1-pa],la[la.length-1-pa]=Ea}else if(Ea=k-+this.og.call(null,pa.data),Sa=n-+this.pg.call(null,pa.data),Ea=Ea*Ea+Sa*Sa,Ea<q){var Va=Math.sqrt(q=Ea);
v=k-Va;z=n-Va;P=k+Va;X=n+Va;Va=pa.data}return Va};id.remove=function(k){if(isNaN(N=+this.og.call(null,k))||isNaN(P=+this.pg.call(null,k)))return this;var n,q=this.vb,v,z=this.rc,A=this.zc,E=this.Jb,M=this.Nb,N,P,X,la,pa,Ea,Sa;if(!q)return this;if(q.length)for(;;){(pa=N>=(X=(z+E)/2))?z=X:E=X;(Ea=P>=(la=(A+M)/2))?A=la:M=la;if(!(n=q,q=q[Sa=Ea<<1|pa]))return this;if(!q.length)break;if(n[Sa+1&3]||n[Sa+2&3]||n[Sa+3&3]){var Va=n;var kc=Sa}}for(;q.data!==k;)if(!(v=q,q=q.next))return this;(k=q.next)&&delete q.next;
if(v)return k?v.next=k:delete v.next,this;if(!n)return this.vb=k,this;k?n[Sa]=k:delete n[Sa];(q=n[0]||n[1]||n[2]||n[3])&&q===(n[3]||n[2]||n[1]||n[0])&&!q.length&&(Va?Va[kc]=q:this.vb=q);return this};id.CG=function(k){for(var n=0,q=k.length;n<q;++n)this.remove(k[n]);return this};id.root=function(){return this.vb};id.size=function(){var k=0;this.Sy(function(n){if(!n.length){do++k;while(n=n.next)}});return k};id.Sy=function(k){var n=[],q,v=this.vb,z,A,E,M,N;for(v&&n.push(new hd(v,this.rc,this.zc,this.Jb,
this.Nb));q=n.pop();)if(!k(v=q.node,A=q.Vo,E=q.Wo,M=q.x1,N=q.y1)&&v.length){q=(A+M)/2;var P=(E+N)/2;(z=v[3])&&n.push(new hd(z,q,P,M,N));(z=v[2])&&n.push(new hd(z,A,P,q,N));(z=v[1])&&n.push(new hd(z,q,E,M,P));(z=v[0])&&n.push(new hd(z,A,E,q,P))}};id.QG=function(k){var n=[],q=[],v;for(this.vb&&n.push(new hd(this.vb,this.rc,this.zc,this.Jb,this.Nb));v=n.pop();){var z=v.node;if(z.length){var A,E=v.Vo,M=v.Wo,N=v.x1,P=v.y1,X=(E+N)/2,la=(M+P)/2;(A=z[0])&&n.push(new hd(A,E,M,X,la));(A=z[1])&&n.push(new hd(A,
X,M,N,la));(A=z[2])&&n.push(new hd(A,E,la,X,P));(A=z[3])&&n.push(new hd(A,X,la,N,P))}q.push(v)}for(;v=q.pop();)k(v.node,v.Vo,v.Wo,v.x1,v.y1);return this};id.x=function(k){return arguments.length?(this.og=k,this):this.og};id.y=function(k){return arguments.length?(this.pg=k,this):this.pg};var zV=/^(?:(.)?([<>=^]))?([+\-( ])?([$#])?(0)?(\d+)?(,)?(\.\d+)?(~)?([a-z%])?$/i;nv.prototype=ov.prototype;ov.prototype.toString=function(){return this.fill+this.align+this.sign+this.symbol+(this.zero?"0":"")+(null==
this.width?"":Math.max(1,this.width|0))+(this.sv?",":"")+(null==this.precision?"":"."+Math.max(0,this.precision|0))+(this.trim?"~":"")+this.type};var zF,AF={"%":function(k,n){return(100*k).toFixed(n)},b:function(k){return Math.round(k).toString(2)},c:function(k){return k+""},d:function(k){return Math.round(k).toString(10)},e:function(k,n){return k.toExponential(n)},f:function(k,n){return k.toFixed(n)},g:function(k,n){return k.toPrecision(n)},o:function(k){return Math.round(k).toString(8)},p:function(k,
n){return wF(100*k,n)},r:wF,s:function(k,n){var q=hq(k,n);if(!q)return k+"";var v=q[0];q=q[1];q=q-(zF=3*Math.max(-8,Math.min(8,Math.floor(q/3))))+1;var z=v.length;return q===z?v:q>z?v+Array(q-z+1).join("0"):0<q?v.slice(0,q)+"."+v.slice(q):"0."+Array(1-q).join("0")+hq(k,Math.max(0,n+q-1))[0]},X:function(k){return Math.round(k).toString(16).toUpperCase()},x:function(k){return Math.round(k).toString(16)}},yF="y z a f p n \u00b5 m  k M G T P E Z Y".split(" "),$W;(function(k){return $W=AV(k)})({KB:".",
Ay:",",mw:[3],currency:["$",""]});ke.prototype={constructor:ke,reset:function(){this.s=this.t=0},add:function(k){BF(oq,k,this.t);BF(this,oq.s,this.s);this.s?this.t+=oq.t:this.s=oq.t},valueOf:function(){return this.s}};var oq=new ke,vc=Math.PI,Ud=vc/2,gG=vc/4,aX=2*vc,Ev=vc/180,zg=Math.abs,Fv=Math.atan,HF=Math.atan2,Vd=Math.cos,ed=Math.sin,jq=Math.sqrt;new ke;new ke;new ke;var Gv=new ke;(function(k,n,q,v){return function(z){function A(Gb,ub){k(Gb,ub)&&z.point(Gb,ub)}function E(Gb,ub){pa.point(Gb,ub)}
function M(){Kc.point=E;pa.lc()}function N(){Kc.point=A;pa.kc()}function P(Gb,ub){lc.push([Gb,ub]);Sa.point(Gb,ub)}function X(){Sa.lc();lc=[]}function la(){P(lc[0][0],lc[0][1]);Sa.kc();var Gb=Sa.vB(),ub=Ea.result(),La=ub.length,Ib;lc.pop();kc.push(lc);lc=null;if(La)if(Gb&1){if(La=ub[0],0<(ub=La.length-1)){Va||(z.Hj(),Va=!0);z.lc();for(Gb=0;Gb<ub;++Gb)z.point((Ib=La[Gb])[0],Ib[1]);z.kc()}}else 1<La&&Gb&2&&ub.push(ub.pop().concat(ub.shift())),xb.push(ub.filter(EV))}var pa=n(z),Ea=CV(),Sa=n(Ea),Va=!1,
kc,xb,lc,Kc={point:A,lc:M,kc:N,Hj:function(){Kc.point=P;Kc.lc=X;Kc.kc=la;xb=[];kc=[]},xo:function(){Kc.point=A;Kc.lc=M;Kc.kc=N;var Gb=xb,ub=Gb.length;var La=-1;for(var Ib=0,za,Fa;++La<ub;)Ib+=Gb[La].length;for(za=Array(Ib);0<=--ub;)for(Fa=Gb[ub],La=Fa.length;0<=--La;)za[--Ib]=Fa[La];xb=za;Gb=kc;ub=v[0];Ib=v[1];var J=ed(Ib);La=[ed(ub),-Vd(ub),0];Fa=za=0;Gv.reset();1===J?Ib=Ud+1E-6:-1===J&&(Ib=-Ud-1E-6);J=0;for(var ha=Gb.length;J<ha;++J)if(Ra=(Ga=Gb[J]).length){var Ga,Ra,yb=Ga[Ra-1],Pa=yb[0],Tb=yb[1]/
2+gG,qd=ed(Tb),ik=Vd(Tb);for(Tb=0;Tb<Ra;++Tb,Pa=Ad,qd=jk,ik=xm,yb=kk){var kk=Ga[Tb],Ad=kk[0];xm=kk[1]/2+gG;var jk=ed(xm),xm=Vd(xm),lk=Ad-Pa,Hv=0<=lk?1:-1,Iv=Hv*lk,pq=Iv>vc;qd*=jk;Gv.add(HF(qd*Hv*ed(Iv),ik*xm+qd*Vd(Iv)));za+=pq?lk+Hv*aX:lk;pq^Pa>=ub^Ad>=ub&&(yb=DF(CF(yb),CF(kk)),EF(yb),Pa=DF(La,yb),EF(Pa),Pa=(pq^0<=lk?-1:1)*iq(Pa[2]),Ib>Pa||Ib===Pa&&(yb[0]||yb[1]))&&(Fa+=pq^0<=lk?1:-1)}}Gb=(-1E-6>za||1E-6>za&&-1E-6>Gv)^Fa&1;xb.length?(Va||(z.Hj(),Va=!0),DV(xb,FV,Gb,q,z)):Gb&&(Va||(z.Hj(),Va=!0),z.lc(),
q(null,null,1,z),z.kc());Va&&(z.xo(),Va=!1);xb=kc=null},oy:function(){z.Hj();z.lc();q(null,null,1,z);z.kc();z.xo()}};return Kc}})(function(){return!0},function(k){var n=NaN,q=NaN,v=NaN,z;return{lc:function(){k.lc();z=1},point:function(A,E){var M=0<A?vc:-vc,N=zg(A-n);if(1E-6>zg(N-vc))k.point(n,q=0<(q+E)/2?Ud:-Ud),k.point(v,q),k.kc(),k.lc(),k.point(M,q),k.point(A,q),z=0;else if(v!==M&&N>=vc){1E-6>zg(n-v)&&(n-=1E-6*v);1E-6>zg(A-M)&&(A-=1E-6*M);N=n;var P=q,X=A,la,pa,Ea=ed(N-X);q=1E-6<zg(Ea)?Fv((ed(P)*
(pa=Vd(E))*ed(X)-ed(E)*(la=Vd(P))*ed(N))/(la*pa*Ea)):(P+E)/2;k.point(v,q);k.kc();k.lc();k.point(M,q);z=0}k.point(n=A,q=E);v=M},kc:function(){k.kc();n=q=NaN},vB:function(){return 2-z}}},function(k,n,q,v){null==k?(q*=Ud,v.point(-vc,q),v.point(0,q),v.point(vc,q),v.point(vc,0),v.point(vc,-q),v.point(0,-q),v.point(-vc,-q),v.point(-vc,0),v.point(-vc,q)):1E-6<zg(k[0]-n[0])?(k=k[0]<n[0]?vc:-vc,q=q*k/2,v.point(-k,q),v.point(0,q),v.point(k,q)):v.point(n[0],n[1])},[-vc,-Ud]);new ke;new ke;new ke;new ke;pv.prototype=
{constructor:pv,point:function(k,n){this.stream.point(k,n)},oy:function(){this.stream.oy()},lc:function(){this.stream.lc()},kc:function(){this.stream.kc()},Hj:function(){this.stream.Hj()},xo:function(){this.stream.xo()}};Vd(30*Ev);(function(k){return function(n){var q=new pv,v;for(v in k)q[v]=k[v];q.stream=n;return q}})({point:function(k,n){this.stream.point(k*Ev,n*Ev)}});GF(function(k){return jq(2/(1+k))}).invert=pm(function(k){return 2*iq(k/2)});GF(function(k){return(k=1<k?0:-1>k?vc:Math.acos(k))&&
k/ed(k)}).invert=pm(function(k){return k});jq(3);pm(Fv);pm(iq);pm(function(k){return 2*Fv(k)});qm.prototype=IF.prototype={constructor:qm,count:function(){return this.Yv(GV)},each:function(k){var n,q=[this],v;do{var z=q.reverse();for(q=[];n=z.pop();)if(k(n),n=n.children){var A=0;for(v=n.length;A<v;++A)q.push(n[A])}}while(q.length);return this},Yv:function(k){for(var n,q=[this],v=[],z,A;n=q.pop();)if(v.push(n),n=n.children)for(z=0,A=n.length;z<A;++z)q.push(n[z]);for(;n=v.pop();)k(n);return this},Hq:function(k){for(var n,
q=[this],v;n=q.pop();)if(k(n),n=n.children)for(v=n.length-1;0<=v;--v)q.push(n[v]);return this},oi:function(k){return this.Yv(function(n){for(var q=+k(n.data)||0,v=n.children,z=v&&v.length;0<=--z;)q+=v[z].value;n.value=q})},sort:function(k){return this.Hq(function(n){n.children&&n.children.sort(k)})},path:function(k){var n=this;var q=n;var v=k;if(q!==v){var z=q.Uu(),A=v.Uu(),E=null;q=z.pop();for(v=A.pop();q===v;)E=q,q=z.pop(),v=A.pop();q=E}for(v=[n];n!==q;)n=n.parent,v.push(n);for(n=v.length;k!==q;)v.splice(n,
0,k),k=k.parent;return v},Uu:function(){for(var k=this,n=[k];k=k.parent;)n.push(k);return n},links:function(){var k=this,n=[];k.each(function(q){q!==k&&n.push({source:q.parent,target:q})});return n},copy:function(){return IF(this).Hq(JV)}};(function(k,n){this.Gb=k;this.jF=this.children=this.parent=null;this.a=this;this.s=this.c=this.me=this.z=0;this.t=null;this.i=n}).prototype=Object.create(qm.prototype);var qv=new Date,rv=new Date,hG=Lc(function(){},function(k,n){k.setTime(+k+n)},function(k,n){return n-
k});hG.every=function(k){k=Math.floor(k);return isFinite(k)&&0<k?1<k?Lc(function(n){n.setTime(Math.floor(n/k)*k)},function(n,q){n.setTime(+n+q*k)},function(n,q){return(q-n)/k}):hG:null};Lc(function(k){k.setTime(1E3*Math.floor(k/1E3))},function(k,n){k.setTime(+k+1E3*n)},function(k,n){return(n-k)/1E3},function(k){return k.getUTCSeconds()});Lc(function(k){k.setTime(6E4*Math.floor(k/6E4))},function(k,n){k.setTime(+k+6E4*n)},function(k,n){return(n-k)/6E4},function(k){return k.getMinutes()});Lc(function(k){var n=
6E4*k.getTimezoneOffset()%36E5;0>n&&(n+=36E5);k.setTime(36E5*Math.floor((+k-n)/36E5)+n)},function(k,n){k.setTime(+k+36E5*n)},function(k,n){return(n-k)/36E5},function(k){return k.getHours()});var LF=Lc(function(k){k.setHours(0,0,0,0)},function(k,n){k.setDate(k.getDate()+n)},function(k,n){return(n-k-6E4*(n.getTimezoneOffset()-k.getTimezoneOffset()))/864E5},function(k){return k.getDate()-1}),JW=Ph(0),tv=Ph(1);Ph(2);Ph(3);var uv=Ph(4);Ph(5);Ph(6);Lc(function(k){k.setDate(1);k.setHours(0,0,0,0)},function(k,
n){k.setMonth(k.getMonth()+n)},function(k,n){return n.getMonth()-k.getMonth()+12*(n.getFullYear()-k.getFullYear())},function(k){return k.getMonth()});var ek=Lc(function(k){k.setMonth(0,1);k.setHours(0,0,0,0)},function(k,n){k.setFullYear(k.getFullYear()+n)},function(k,n){return n.getFullYear()-k.getFullYear()},function(k){return k.getFullYear()});ek.every=function(k){return isFinite(k=Math.floor(k))&&0<k?Lc(function(n){n.setFullYear(Math.floor(n.getFullYear()/k)*k);n.setMonth(0,1);n.setHours(0,0,0,
0)},function(n,q){n.setFullYear(n.getFullYear()+q*k)}):null};Lc(function(k){k.setUTCSeconds(0,0)},function(k,n){k.setTime(+k+6E4*n)},function(k,n){return(n-k)/6E4},function(k){return k.getUTCMinutes()});Lc(function(k){k.setUTCMinutes(0,0,0)},function(k,n){k.setTime(+k+36E5*n)},function(k,n){return(n-k)/36E5},function(k){return k.getUTCHours()});var KF=Lc(function(k){k.setUTCHours(0,0,0,0)},function(k,n){k.setUTCDate(k.getUTCDate()+n)},function(k,n){return(n-k)/864E5},function(k){return k.getUTCDate()-
1}),KW=Qh(0),sv=Qh(1);Qh(2);Qh(3);var vv=Qh(4);Qh(5);Qh(6);Lc(function(k){k.setUTCDate(1);k.setUTCHours(0,0,0,0)},function(k,n){k.setUTCMonth(k.getUTCMonth()+n)},function(k,n){return n.getUTCMonth()-k.getUTCMonth()+12*(n.getUTCFullYear()-k.getUTCFullYear())},function(k){return k.getUTCMonth()});var fk=Lc(function(k){k.setUTCMonth(0,1);k.setUTCHours(0,0,0,0)},function(k,n){k.setUTCFullYear(k.getUTCFullYear()+n)},function(k,n){return n.getUTCFullYear()-k.getUTCFullYear()},function(k){return k.getUTCFullYear()});
fk.every=function(k){return isFinite(k=Math.floor(k))&&0<k?Lc(function(n){n.setUTCFullYear(Math.floor(n.getUTCFullYear()/k)*k);n.setUTCMonth(0,1);n.setUTCHours(0,0,0,0)},function(n,q){n.setUTCFullYear(n.getUTCFullYear()+q*k)}):null};var JF={"-":"",_:" ",0:"0"},Xc=/^\s*\d+/,IW=/^%/,HW=/[\\^$*+?|[\]().{}]/g,qq;(function(k){qq=LV(k);NL=qq.aF;OL=qq.bF;return qq})({dateTime:"%x, %X",IB:"%-m/%-d/%Y",time:"%-I:%M:%S %p",GD:["AM","PM"],JB:"Sunday Monday Tuesday Wednesday Thursday Friday Saturday".split(" "),
eE:"Sun Mon Tue Wed Thu Fri Sat".split(" "),fD:"January February March April May June July August September October November December".split(" "),fE:"Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec".split(" ")});Date.prototype.toISOString||NL("%Y-%m-%dT%H:%M:%S.%LZ");+new Date("2000-01-01T00:00:00.000Z")||OL("%Y-%m-%dT%H:%M:%S.%LZ");qb("1f77b4ff7f0e2ca02cd627289467bd8c564be377c27f7f7fbcbd2217becf");qb("393b795254a36b6ecf9c9ede6379398ca252b5cf6bcedb9c8c6d31bd9e39e7ba52e7cb94843c39ad494ad6616be7969c7b4173a55194ce6dbdde9ed6");
qb("3182bd6baed69ecae1c6dbefe6550dfd8d3cfdae6bfdd0a231a35474c476a1d99bc7e9c0756bb19e9ac8bcbddcdadaeb636363969696bdbdbdd9d9d9");qb("1f77b4aec7e8ff7f0effbb782ca02c98df8ad62728ff98969467bdc5b0d58c564bc49c94e377c2f7b6d27f7f7fc7c7c7bcbd22dbdb8d17becf9edae5");qb("7fc97fbeaed4fdc086ffff99386cb0f0027fbf5b17666666");qb("1b9e77d95f027570b3e7298a66a61ee6ab02a6761d666666");qb("a6cee31f78b4b2df8a33a02cfb9a99e31a1cfdbf6fff7f00cab2d66a3d9affff99b15928");qb("fbb4aeb3cde3ccebc5decbe4fed9a6ffffcce5d8bdfddaecf2f2f2");
qb("b3e2cdfdcdaccbd5e8f4cae4e6f5c9fff2aef1e2cccccccc");qb("e41a1c377eb84daf4a984ea3ff7f00ffff33a65628f781bf999999");qb("66c2a5fc8d628da0cbe78ac3a6d854ffd92fe5c494b3b3b3");qb("8dd3c7ffffb3bebadafb807280b1d3fdb462b3de69fccde5d9d9d9bc80bdccebc5ffed6f");var bX=Array(3).concat("d8b365f5f5f55ab4ac","a6611adfc27d80cdc1018571","a6611adfc27df5f5f580cdc1018571","8c510ad8b365f6e8c3c7eae55ab4ac01665e","8c510ad8b365f6e8c3f5f5f5c7eae55ab4ac01665e","8c510abf812ddfc27df6e8c3c7eae580cdc135978f01665e","8c510abf812ddfc27df6e8c3f5f5f5c7eae580cdc135978f01665e",
"5430058c510abf812ddfc27df6e8c3c7eae580cdc135978f01665e003c30","5430058c510abf812ddfc27df6e8c3f5f5f5c7eae580cdc135978f01665e003c30").map(qb);Yb(bX);var cX=Array(3).concat("af8dc3f7f7f77fbf7b","7b3294c2a5cfa6dba0008837","7b3294c2a5cff7f7f7a6dba0008837","762a83af8dc3e7d4e8d9f0d37fbf7b1b7837","762a83af8dc3e7d4e8f7f7f7d9f0d37fbf7b1b7837","762a839970abc2a5cfe7d4e8d9f0d3a6dba05aae611b7837","762a839970abc2a5cfe7d4e8f7f7f7d9f0d3a6dba05aae611b7837","40004b762a839970abc2a5cfe7d4e8d9f0d3a6dba05aae611b783700441b",
"40004b762a839970abc2a5cfe7d4e8f7f7f7d9f0d3a6dba05aae611b783700441b").map(qb);Yb(cX);var dX=Array(3).concat("e9a3c9f7f7f7a1d76a","d01c8bf1b6dab8e1864dac26","d01c8bf1b6daf7f7f7b8e1864dac26","c51b7de9a3c9fde0efe6f5d0a1d76a4d9221","c51b7de9a3c9fde0eff7f7f7e6f5d0a1d76a4d9221","c51b7dde77aef1b6dafde0efe6f5d0b8e1867fbc414d9221","c51b7dde77aef1b6dafde0eff7f7f7e6f5d0b8e1867fbc414d9221","8e0152c51b7dde77aef1b6dafde0efe6f5d0b8e1867fbc414d9221276419","8e0152c51b7dde77aef1b6dafde0eff7f7f7e6f5d0b8e1867fbc414d9221276419").map(qb);
Yb(dX);var eX=Array(3).concat("998ec3f7f7f7f1a340","5e3c99b2abd2fdb863e66101","5e3c99b2abd2f7f7f7fdb863e66101","542788998ec3d8daebfee0b6f1a340b35806","542788998ec3d8daebf7f7f7fee0b6f1a340b35806","5427888073acb2abd2d8daebfee0b6fdb863e08214b35806","5427888073acb2abd2d8daebf7f7f7fee0b6fdb863e08214b35806","2d004b5427888073acb2abd2d8daebfee0b6fdb863e08214b358067f3b08","2d004b5427888073acb2abd2d8daebf7f7f7fee0b6fdb863e08214b358067f3b08").map(qb);Yb(eX);var fX=Array(3).concat("ef8a62f7f7f767a9cf","ca0020f4a58292c5de0571b0",
"ca0020f4a582f7f7f792c5de0571b0","b2182bef8a62fddbc7d1e5f067a9cf2166ac","b2182bef8a62fddbc7f7f7f7d1e5f067a9cf2166ac","b2182bd6604df4a582fddbc7d1e5f092c5de4393c32166ac","b2182bd6604df4a582fddbc7f7f7f7d1e5f092c5de4393c32166ac","67001fb2182bd6604df4a582fddbc7d1e5f092c5de4393c32166ac053061","67001fb2182bd6604df4a582fddbc7f7f7f7d1e5f092c5de4393c32166ac053061").map(qb);Yb(fX);var gX=Array(3).concat("ef8a62ffffff999999","ca0020f4a582bababa404040","ca0020f4a582ffffffbababa404040","b2182bef8a62fddbc7e0e0e09999994d4d4d",
"b2182bef8a62fddbc7ffffffe0e0e09999994d4d4d","b2182bd6604df4a582fddbc7e0e0e0bababa8787874d4d4d","b2182bd6604df4a582fddbc7ffffffe0e0e0bababa8787874d4d4d","67001fb2182bd6604df4a582fddbc7e0e0e0bababa8787874d4d4d1a1a1a","67001fb2182bd6604df4a582fddbc7ffffffe0e0e0bababa8787874d4d4d1a1a1a").map(qb);Yb(gX);var hX=Array(3).concat("fc8d59ffffbf91bfdb","d7191cfdae61abd9e92c7bb6","d7191cfdae61ffffbfabd9e92c7bb6","d73027fc8d59fee090e0f3f891bfdb4575b4","d73027fc8d59fee090ffffbfe0f3f891bfdb4575b4","d73027f46d43fdae61fee090e0f3f8abd9e974add14575b4",
"d73027f46d43fdae61fee090ffffbfe0f3f8abd9e974add14575b4","a50026d73027f46d43fdae61fee090e0f3f8abd9e974add14575b4313695","a50026d73027f46d43fdae61fee090ffffbfe0f3f8abd9e974add14575b4313695").map(qb);Yb(hX);var iX=Array(3).concat("fc8d59ffffbf91cf60","d7191cfdae61a6d96a1a9641","d7191cfdae61ffffbfa6d96a1a9641","d73027fc8d59fee08bd9ef8b91cf601a9850","d73027fc8d59fee08bffffbfd9ef8b91cf601a9850","d73027f46d43fdae61fee08bd9ef8ba6d96a66bd631a9850","d73027f46d43fdae61fee08bffffbfd9ef8ba6d96a66bd631a9850",
"a50026d73027f46d43fdae61fee08bd9ef8ba6d96a66bd631a9850006837","a50026d73027f46d43fdae61fee08bffffbfd9ef8ba6d96a66bd631a9850006837").map(qb);Yb(iX);var jX=Array(3).concat("fc8d59ffffbf99d594","d7191cfdae61abdda42b83ba","d7191cfdae61ffffbfabdda42b83ba","d53e4ffc8d59fee08be6f59899d5943288bd","d53e4ffc8d59fee08bffffbfe6f59899d5943288bd","d53e4ff46d43fdae61fee08be6f598abdda466c2a53288bd","d53e4ff46d43fdae61fee08bffffbfe6f598abdda466c2a53288bd","9e0142d53e4ff46d43fdae61fee08be6f598abdda466c2a53288bd5e4fa2",
"9e0142d53e4ff46d43fdae61fee08bffffbfe6f598abdda466c2a53288bd5e4fa2").map(qb);Yb(jX);var kX=Array(3).concat("e5f5f999d8c92ca25f","edf8fbb2e2e266c2a4238b45","edf8fbb2e2e266c2a42ca25f006d2c","edf8fbccece699d8c966c2a42ca25f006d2c","edf8fbccece699d8c966c2a441ae76238b45005824","f7fcfde5f5f9ccece699d8c966c2a441ae76238b45005824","f7fcfde5f5f9ccece699d8c966c2a441ae76238b45006d2c00441b").map(qb);Yb(kX);var lX=Array(3).concat("e0ecf49ebcda8856a7","edf8fbb3cde38c96c688419d","edf8fbb3cde38c96c68856a7810f7c",
"edf8fbbfd3e69ebcda8c96c68856a7810f7c","edf8fbbfd3e69ebcda8c96c68c6bb188419d6e016b","f7fcfde0ecf4bfd3e69ebcda8c96c68c6bb188419d6e016b","f7fcfde0ecf4bfd3e69ebcda8c96c68c6bb188419d810f7c4d004b").map(qb);Yb(lX);var mX=Array(3).concat("e0f3dba8ddb543a2ca","f0f9e8bae4bc7bccc42b8cbe","f0f9e8bae4bc7bccc443a2ca0868ac","f0f9e8ccebc5a8ddb57bccc443a2ca0868ac","f0f9e8ccebc5a8ddb57bccc44eb3d32b8cbe08589e","f7fcf0e0f3dbccebc5a8ddb57bccc44eb3d32b8cbe08589e","f7fcf0e0f3dbccebc5a8ddb57bccc44eb3d32b8cbe0868ac084081").map(qb);
Yb(mX);var nX=Array(3).concat("fee8c8fdbb84e34a33","fef0d9fdcc8afc8d59d7301f","fef0d9fdcc8afc8d59e34a33b30000","fef0d9fdd49efdbb84fc8d59e34a33b30000","fef0d9fdd49efdbb84fc8d59ef6548d7301f990000","fff7ecfee8c8fdd49efdbb84fc8d59ef6548d7301f990000","fff7ecfee8c8fdd49efdbb84fc8d59ef6548d7301fb300007f0000").map(qb);Yb(nX);var oX=Array(3).concat("ece2f0a6bddb1c9099","f6eff7bdc9e167a9cf02818a","f6eff7bdc9e167a9cf1c9099016c59","f6eff7d0d1e6a6bddb67a9cf1c9099016c59","f6eff7d0d1e6a6bddb67a9cf3690c002818a016450",
"fff7fbece2f0d0d1e6a6bddb67a9cf3690c002818a016450","fff7fbece2f0d0d1e6a6bddb67a9cf3690c002818a016c59014636").map(qb);Yb(oX);var pX=Array(3).concat("ece7f2a6bddb2b8cbe","f1eef6bdc9e174a9cf0570b0","f1eef6bdc9e174a9cf2b8cbe045a8d","f1eef6d0d1e6a6bddb74a9cf2b8cbe045a8d","f1eef6d0d1e6a6bddb74a9cf3690c00570b0034e7b","fff7fbece7f2d0d1e6a6bddb74a9cf3690c00570b0034e7b","fff7fbece7f2d0d1e6a6bddb74a9cf3690c00570b0045a8d023858").map(qb);Yb(pX);var qX=Array(3).concat("e7e1efc994c7dd1c77","f1eef6d7b5d8df65b0ce1256",
"f1eef6d7b5d8df65b0dd1c77980043","f1eef6d4b9dac994c7df65b0dd1c77980043","f1eef6d4b9dac994c7df65b0e7298ace125691003f","f7f4f9e7e1efd4b9dac994c7df65b0e7298ace125691003f","f7f4f9e7e1efd4b9dac994c7df65b0e7298ace125698004367001f").map(qb);Yb(qX);var rX=Array(3).concat("fde0ddfa9fb5c51b8a","feebe2fbb4b9f768a1ae017e","feebe2fbb4b9f768a1c51b8a7a0177","feebe2fcc5c0fa9fb5f768a1c51b8a7a0177","feebe2fcc5c0fa9fb5f768a1dd3497ae017e7a0177","fff7f3fde0ddfcc5c0fa9fb5f768a1dd3497ae017e7a0177","fff7f3fde0ddfcc5c0fa9fb5f768a1dd3497ae017e7a017749006a").map(qb);
Yb(rX);var sX=Array(3).concat("edf8b17fcdbb2c7fb8","ffffcca1dab441b6c4225ea8","ffffcca1dab441b6c42c7fb8253494","ffffccc7e9b47fcdbb41b6c42c7fb8253494","ffffccc7e9b47fcdbb41b6c41d91c0225ea80c2c84","ffffd9edf8b1c7e9b47fcdbb41b6c41d91c0225ea80c2c84","ffffd9edf8b1c7e9b47fcdbb41b6c41d91c0225ea8253494081d58").map(qb);Yb(sX);var tX=Array(3).concat("f7fcb9addd8e31a354","ffffccc2e69978c679238443","ffffccc2e69978c67931a354006837","ffffccd9f0a3addd8e78c67931a354006837","ffffccd9f0a3addd8e78c67941ab5d238443005a32",
"ffffe5f7fcb9d9f0a3addd8e78c67941ab5d238443005a32","ffffe5f7fcb9d9f0a3addd8e78c67941ab5d238443006837004529").map(qb);Yb(tX);var uX=Array(3).concat("fff7bcfec44fd95f0e","ffffd4fed98efe9929cc4c02","ffffd4fed98efe9929d95f0e993404","ffffd4fee391fec44ffe9929d95f0e993404","ffffd4fee391fec44ffe9929ec7014cc4c028c2d04","ffffe5fff7bcfee391fec44ffe9929ec7014cc4c028c2d04","ffffe5fff7bcfee391fec44ffe9929ec7014cc4c02993404662506").map(qb);Yb(uX);var vX=Array(3).concat("ffeda0feb24cf03b20","ffffb2fecc5cfd8d3ce31a1c",
"ffffb2fecc5cfd8d3cf03b20bd0026","ffffb2fed976feb24cfd8d3cf03b20bd0026","ffffb2fed976feb24cfd8d3cfc4e2ae31a1cb10026","ffffccffeda0fed976feb24cfd8d3cfc4e2ae31a1cb10026","ffffccffeda0fed976feb24cfd8d3cfc4e2ae31a1cbd0026800026").map(qb);Yb(vX);var wX=Array(3).concat("deebf79ecae13182bd","eff3ffbdd7e76baed62171b5","eff3ffbdd7e76baed63182bd08519c","eff3ffc6dbef9ecae16baed63182bd08519c","eff3ffc6dbef9ecae16baed64292c62171b5084594","f7fbffdeebf7c6dbef9ecae16baed64292c62171b5084594","f7fbffdeebf7c6dbef9ecae16baed64292c62171b508519c08306b").map(qb);
Yb(wX);var xX=Array(3).concat("e5f5e0a1d99b31a354","edf8e9bae4b374c476238b45","edf8e9bae4b374c47631a354006d2c","edf8e9c7e9c0a1d99b74c47631a354006d2c","edf8e9c7e9c0a1d99b74c47641ab5d238b45005a32","f7fcf5e5f5e0c7e9c0a1d99b74c47641ab5d238b45005a32","f7fcf5e5f5e0c7e9c0a1d99b74c47641ab5d238b45006d2c00441b").map(qb);Yb(xX);var yX=Array(3).concat("f0f0f0bdbdbd636363","f7f7f7cccccc969696525252","f7f7f7cccccc969696636363252525","f7f7f7d9d9d9bdbdbd969696636363252525","f7f7f7d9d9d9bdbdbd969696737373525252252525",
"fffffff0f0f0d9d9d9bdbdbd969696737373525252252525","fffffff0f0f0d9d9d9bdbdbd969696737373525252252525000000").map(qb);Yb(yX);var zX=Array(3).concat("efedf5bcbddc756bb1","f2f0f7cbc9e29e9ac86a51a3","f2f0f7cbc9e29e9ac8756bb154278f","f2f0f7dadaebbcbddc9e9ac8756bb154278f","f2f0f7dadaebbcbddc9e9ac8807dba6a51a34a1486","fcfbfdefedf5dadaebbcbddc9e9ac8807dba6a51a34a1486","fcfbfdefedf5dadaebbcbddc9e9ac8807dba6a51a354278f3f007d").map(qb);Yb(zX);var AX=Array(3).concat("fee0d2fc9272de2d26","fee5d9fcae91fb6a4acb181d",
"fee5d9fcae91fb6a4ade2d26a50f15","fee5d9fcbba1fc9272fb6a4ade2d26a50f15","fee5d9fcbba1fc9272fb6a4aef3b2ccb181d99000d","fff5f0fee0d2fcbba1fc9272fb6a4aef3b2ccb181d99000d","fff5f0fee0d2fcbba1fc9272fb6a4aef3b2ccb181da50f1567000d").map(qb);Yb(AX);var BX=Array(3).concat("fee6cefdae6be6550d","feeddefdbe85fd8d3cd94701","feeddefdbe85fd8d3ce6550da63603","feeddefdd0a2fdae6bfd8d3ce6550da63603","feeddefdd0a2fdae6bfd8d3cf16913d948018c2d04","fff5ebfee6cefdd0a2fdae6bfd8d3cf16913d948018c2d04","fff5ebfee6cefdd0a2fdae6bfd8d3cf16913d94801a636037f2704").map(qb);
Yb(BX);Cv(Ve(300,.5,0),Ve(-240,.5,1));Cv(Ve(-100,.75,.35),Ve(80,1.5,.8));Cv(Ve(260,.75,.35),Ve(80,1.5,.8));Ve();Of();mq(qb("44015444025645045745055946075a46085c460a5d460b5e470d60470e6147106347116447136548146748166848176948186a481a6c481b6d481c6e481d6f481f70482071482173482374482475482576482677482878482979472a7a472c7a472d7b472e7c472f7d46307e46327e46337f463480453581453781453882443983443a83443b84433d84433e85423f854240864241864142874144874045884046883f47883f48893e49893e4a893e4c8a3d4d8a3d4e8a3c4f8a3c508b3b518b3b528b3a538b3a548c39558c39568c38588c38598c375a8c375b8d365c8d365d8d355e8d355f8d34608d34618d33628d33638d32648e32658e31668e31678e31688e30698e306a8e2f6b8e2f6c8e2e6d8e2e6e8e2e6f8e2d708e2d718e2c718e2c728e2c738e2b748e2b758e2a768e2a778e2a788e29798e297a8e297b8e287c8e287d8e277e8e277f8e27808e26818e26828e26828e25838e25848e25858e24868e24878e23888e23898e238a8d228b8d228c8d228d8d218e8d218f8d21908d21918c20928c20928c20938c1f948c1f958b1f968b1f978b1f988b1f998a1f9a8a1e9b8a1e9c891e9d891f9e891f9f881fa0881fa1881fa1871fa28720a38620a48621a58521a68522a78522a88423a98324aa8325ab8225ac8226ad8127ad8128ae8029af7f2ab07f2cb17e2db27d2eb37c2fb47c31b57b32b67a34b67935b77937b87838b9773aba763bbb753dbc743fbc7340bd7242be7144bf7046c06f48c16e4ac16d4cc26c4ec36b50c46a52c56954c56856c66758c7655ac8645cc8635ec96260ca6063cb5f65cb5e67cc5c69cd5b6ccd5a6ece5870cf5773d05675d05477d1537ad1517cd2507fd34e81d34d84d44b86d54989d5488bd6468ed64590d74393d74195d84098d83e9bd93c9dd93ba0da39a2da37a5db36a8db34aadc32addc30b0dd2fb2dd2db5de2bb8de29bade28bddf26c0df25c2df23c5e021c8e020cae11fcde11dd0e11cd2e21bd5e21ad8e219dae319dde318dfe318e2e418e5e419e7e419eae51aece51befe51cf1e51df4e61ef6e620f8e621fbe723fde725"));
mq(qb("00000401000501010601010802010902020b02020d03030f03031204041405041606051806051a07061c08071e0907200a08220b09240c09260d0a290e0b2b100b2d110c2f120d31130d34140e36150e38160f3b180f3d19103f1a10421c10441d11471e114920114b21114e22115024125325125527125829115a2a115c2c115f2d11612f116331116533106734106936106b38106c390f6e3b0f703d0f713f0f72400f74420f75440f764510774710784910784a10794c117a4e117b4f127b51127c52137c54137d56147d57157e59157e5a167e5c167f5d177f5f187f601880621980641a80651a80671b80681c816a1c816b1d816d1d816e1e81701f81721f817320817521817621817822817922827b23827c23827e24828025828125818326818426818627818827818928818b29818c29818e2a81902a81912b81932b80942c80962c80982d80992d809b2e7f9c2e7f9e2f7fa02f7fa1307ea3307ea5317ea6317da8327daa337dab337cad347cae347bb0357bb2357bb3367ab5367ab73779b83779ba3878bc3978bd3977bf3a77c03a76c23b75c43c75c53c74c73d73c83e73ca3e72cc3f71cd4071cf4070d0416fd2426fd3436ed5446dd6456cd8456cd9466bdb476adc4869de4968df4a68e04c67e24d66e34e65e44f64e55064e75263e85362e95462ea5661eb5760ec5860ed5a5fee5b5eef5d5ef05f5ef1605df2625df2645cf3655cf4675cf4695cf56b5cf66c5cf66e5cf7705cf7725cf8745cf8765cf9785df9795df97b5dfa7d5efa7f5efa815ffb835ffb8560fb8761fc8961fc8a62fc8c63fc8e64fc9065fd9266fd9467fd9668fd9869fd9a6afd9b6bfe9d6cfe9f6dfea16efea36ffea571fea772fea973feaa74feac76feae77feb078feb27afeb47bfeb67cfeb77efeb97ffebb81febd82febf84fec185fec287fec488fec68afec88cfeca8dfecc8ffecd90fecf92fed194fed395fed597fed799fed89afdda9cfddc9efddea0fde0a1fde2a3fde3a5fde5a7fde7a9fde9aafdebacfcecaefceeb0fcf0b2fcf2b4fcf4b6fcf6b8fcf7b9fcf9bbfcfbbdfcfdbf"));
mq(qb("00000401000501010601010802010a02020c02020e03021004031204031405041706041907051b08051d09061f0a07220b07240c08260d08290e092b10092d110a30120a32140b34150b37160b39180c3c190c3e1b0c411c0c431e0c451f0c48210c4a230c4c240c4f260c51280b53290b552b0b572d0b592f0a5b310a5c320a5e340a5f3609613809623909633b09643d09653e0966400a67420a68440a68450a69470b6a490b6a4a0c6b4c0c6b4d0d6c4f0d6c510e6c520e6d540f6d550f6d57106e59106e5a116e5c126e5d126e5f136e61136e62146e64156e65156e67166e69166e6a176e6c186e6d186e6f196e71196e721a6e741a6e751b6e771c6d781c6d7a1d6d7c1d6d7d1e6d7f1e6c801f6c82206c84206b85216b87216b88226a8a226a8c23698d23698f24699025689225689326679526679727669827669a28659b29649d29649f2a63a02a63a22b62a32c61a52c60a62d60a82e5fa92e5eab2f5ead305dae305cb0315bb1325ab3325ab43359b63458b73557b93556ba3655bc3754bd3853bf3952c03a51c13a50c33b4fc43c4ec63d4dc73e4cc83f4bca404acb4149cc4248ce4347cf4446d04545d24644d34743d44842d54a41d74b3fd84c3ed94d3dda4e3cdb503bdd513ade5238df5337e05536e15635e25734e35933e45a31e55c30e65d2fe75e2ee8602de9612bea632aeb6429eb6628ec6726ed6925ee6a24ef6c23ef6e21f06f20f1711ff1731df2741cf3761bf37819f47918f57b17f57d15f67e14f68013f78212f78410f8850ff8870ef8890cf98b0bf98c0af98e09fa9008fa9207fa9407fb9606fb9706fb9906fb9b06fb9d07fc9f07fca108fca309fca50afca60cfca80dfcaa0ffcac11fcae12fcb014fcb216fcb418fbb61afbb81dfbba1ffbbc21fbbe23fac026fac228fac42afac62df9c72ff9c932f9cb35f8cd37f8cf3af7d13df7d340f6d543f6d746f5d949f5db4cf4dd4ff4df53f4e156f3e35af3e55df2e661f2e865f2ea69f1ec6df1ed71f1ef75f1f179f2f27df2f482f3f586f3f68af4f88ef5f992f6fa96f8fb9af9fc9dfafda1fcffa4"));
mq(qb("0d088710078813078916078a19068c1b068d1d068e20068f2206902406912605912805922a05932c05942e05952f059631059733059735049837049938049a3a049a3c049b3e049c3f049c41049d43039e44039e46039f48039f4903a04b03a14c02a14e02a25002a25102a35302a35502a45601a45801a45901a55b01a55c01a65e01a66001a66100a76300a76400a76600a76700a86900a86a00a86c00a86e00a86f00a87100a87201a87401a87501a87701a87801a87a02a87b02a87d03a87e03a88004a88104a78305a78405a78606a68707a68808a68a09a58b0aa58d0ba58e0ca48f0da4910ea3920fa39410a29511a19613a19814a099159f9a169f9c179e9d189d9e199da01a9ca11b9ba21d9aa31e9aa51f99a62098a72197a82296aa2395ab2494ac2694ad2793ae2892b02991b12a90b22b8fb32c8eb42e8db52f8cb6308bb7318ab83289ba3388bb3488bc3587bd3786be3885bf3984c03a83c13b82c23c81c33d80c43e7fc5407ec6417dc7427cc8437bc9447aca457acb4679cc4778cc4977cd4a76ce4b75cf4c74d04d73d14e72d24f71d35171d45270d5536fd5546ed6556dd7566cd8576bd9586ada5a6ada5b69db5c68dc5d67dd5e66de5f65de6164df6263e06363e16462e26561e26660e3685fe4695ee56a5de56b5de66c5ce76e5be76f5ae87059e97158e97257ea7457eb7556eb7655ec7754ed7953ed7a52ee7b51ef7c51ef7e50f07f4ff0804ef1814df1834cf2844bf3854bf3874af48849f48948f58b47f58c46f68d45f68f44f79044f79143f79342f89441f89540f9973ff9983ef99a3efa9b3dfa9c3cfa9e3bfb9f3afba139fba238fca338fca537fca636fca835fca934fdab33fdac33fdae32fdaf31fdb130fdb22ffdb42ffdb52efeb72dfeb82cfeba2cfebb2bfebd2afebe2afec029fdc229fdc328fdc527fdc627fdc827fdca26fdcb26fccd25fcce25fcd025fcd225fbd324fbd524fbd724fad824fada24f9dc24f9dd25f8df25f8e125f7e225f7e425f6e626f6e826f5e926f5eb27f4ed27f3ee27f3f027f2f227f1f426f1f525f0f724f0f921"));
VF.prototype={lc:function(){this.xe=0},kc:function(){(this.Df||0!==this.Df&&1===this.xe)&&this.Db.closePath();this.Df=1-this.Df},point:function(k,n){k=+k;n=+n;switch(this.xe){case 0:this.xe=1;this.Df?this.Db.lineTo(k,n):this.Db.moveTo(k,n);break;case 1:this.xe=2;default:this.Db.lineTo(k,n)}}};(function(k){function n(q){return new XF(k(q))}n.Cm=k;return n})(WF);XF.prototype={lc:function(){this.Cm.lc()},kc:function(){this.Cm.kc()},point:function(k,n){this.Cm.point(n*Math.sin(k),n*-Math.cos(k))}};yv.prototype=
{lc:function(){this.rc=this.Jb=this.zc=this.Nb=this.Tm=NaN;this.xe=0},kc:function(){switch(this.xe){case 2:this.Db.lineTo(this.Jb,this.Nb);break;case 3:xv(this,this.Tm,ZF(this,this.Tm))}(this.Df||0!==this.Df&&1===this.xe)&&this.Db.closePath();this.Df=1-this.Df},point:function(k,n){var q=NaN;k=+k;n=+n;if(k!==this.Jb||n!==this.Nb){switch(this.xe){case 0:this.xe=1;this.Df?this.Db.lineTo(k,n):this.Db.moveTo(k,n);break;case 1:this.xe=2;break;case 2:this.xe=3;xv(this,ZF(this,q=YF(this,k,n)),q);break;default:xv(this,
this.Tm,q=YF(this,k,n))}this.rc=this.Jb;this.Jb=k;this.zc=this.Nb;this.Nb=n;this.Tm=q}}};($F.prototype=Object.create(yv.prototype)).point=function(k,n){yv.prototype.point.call(this,n,k)};aG.prototype={moveTo:function(k,n){this.Db.moveTo(n,k)},closePath:function(){this.Db.closePath()},lineTo:function(k,n){this.Db.lineTo(n,k)},bezierCurveTo:function(k,n,q,v,z,A){this.Db.bezierCurveTo(n,k,v,q,A,z)}};Ag.prototype={constructor:Ag,scale:function(k){return 1===k?this:new Ag(this.k*k,this.x,this.y)},translate:function(k,
n){return 0===k&0===n?this:new Ag(this.k,this.x+this.k*k,this.y+this.k*n)},apply:function(k){return[k[0]*this.k+this.x,k[1]*this.k+this.y]},invert:function(k){return[(k[0]-this.x)/this.k,(k[1]-this.y)/this.k]},zw:function(k){return(k-this.x)/this.k},Aw:function(k){return(k-this.y)/this.k},toString:function(){return"translate("+this.x+","+this.y+") scale("+this.k+")"}};var cG=new Ag(1,0,0);PL=function(k){return 1-Math.pow(2,-10*k)};QL=cv;RL=Mc;wI=function(){function k(M){var N,P=M.length,X,la=!1,pa;
null==z&&(E=A(pa=oF()));for(N=0;N<=P;++N)!(N<P&&v(X=M[N],N,M))===la&&((la=!la)?E.lc():E.kc()),la&&E.point(+n(X,N,M),+q(X,N,M));if(pa)return E=null,pa+""||null}var n=MW,q=NW,v=wv(!0),z=null,A=WF,E=null;k.x=function(M){return arguments.length?(n="function"===typeof M?M:wv(+M),k):n};k.y=function(M){return arguments.length?(q="function"===typeof M?M:wv(+M),k):q};k.curve=function(M){return arguments.length?(A=M,null!=z&&(E=A(z)),k):A};k.context=function(M){return arguments.length?(null==M?z=E=null:E=A(z=
M),k):z};return k};xI=function(k){return new $F(k)};SL=function(){function k(J){J.Bx("__zoom",bG).on("wheel.zoom",E).on("mousedown.zoom",M).on("dblclick.zoom",N).filter(kc).on("touchstart.zoom",P).on("touchmove.zoom",X).on("touchend.zoom touchcancel.zoom",la).style("touch-action","none").style("-webkit-tap-highlight-color","rgba(0,0,0,0)")}function n(J,ha){ha=Math.max(xb[0],Math.min(xb[1],ha));return ha===J.k?J:new Ag(ha,J.x,J.y)}function q(J,ha,Ga){var Ra=ha[0]-Ga[0]*J.k;ha=ha[1]-Ga[1]*J.k;return Ra===
J.x&&ha===J.y?J:new Ag(J.k,Ra,ha)}function v(J,ha,Ga){J.on("start.zoom",function(){z(this,arguments).start()}).on("interrupt.zoom end.zoom",function(){z(this,arguments).end()}).jd("zoom",function(){var Ra=arguments,yb=z(this,Ra),Pa=Ea.apply(this,Ra),Tb=Ga||[(+Pa[0][0]+ +Pa[1][0])/2,(+Pa[0][1]+ +Pa[1][1])/2],qd=Math.max(Pa[1][0]-Pa[0][0],Pa[1][1]-Pa[0][1]);Pa=this.__zoom;var ik="function"===typeof ha?ha.apply(this,Ra):ha,kk=Gb(Pa.invert(Tb).concat(qd/Pa.k),ik.invert(Tb).concat(qd/ik.k));return function(Ad){if(1===
Ad)Ad=ik;else{Ad=kk(Ad);var jk=qd/Ad[2];Ad=new Ag(jk,Tb[0]-Ad[0]*jk,Tb[1]-Ad[1]*jk)}yb.zoom(null,Ad)}})}function z(J,ha){for(var Ga=0,Ra=ub.length,yb;Ga<Ra;++Ga)if((yb=ub[Ga]).pi===J)return yb;return new A(J,ha)}function A(J,ha){this.pi=J;this.bf=ha;this.index=-1;this.active=0;this.hl=Ea.apply(J,ha)}function E(){if(pa.apply(this,arguments)){var J=z(this,arguments),ha=this.__zoom,Ga=Math.max(xb[0],Math.min(xb[1],ha.k*Math.pow(2,Va.apply(this,arguments)))),Ra=ve(this);if(J.ct){if(J.Kd[0][0]!==Ra[0]||
J.Kd[0][1]!==Ra[1])J.Kd[1]=ha.invert(J.Kd[0]=Ra);clearTimeout(J.ct)}else{if(ha.k===Ga)return;J.Kd=[Ra,ha.invert(Ra)];eq(this);J.start()}um();J.ct=setTimeout(function(){J.ct=null;J.end()},150);J.zoom("mouse",Sa(q(n(ha,Ga),J.Kd[0],J.Kd[1]),J.hl,lc))}}function M(){if(!za&&pa.apply(this,arguments)){var J=z(this,arguments),ha=Mc(ML.view).on("mousemove.zoom",function(){um();if(!J.Aj){var Pa=ML.clientX-Ra,Tb=ML.clientY-yb;J.Aj=Pa*Pa+Tb*Tb>Fa}J.zoom("mouse",Sa(q(J.pi.__zoom,J.Kd[0]=ve(J.pi),J.Kd[1]),J.hl,
lc))},!0).on("mouseup.zoom",function(){ha.on("mousemove.zoom mouseup.zoom",null);Mf(ML.view,J.Aj);um();J.end()},!0),Ga=ve(this),Ra=ML.clientX,yb=ML.clientY;Fd(ML.view);ML.stopImmediatePropagation();J.Kd=[Ga,this.__zoom.invert(Ga)];eq(this);J.start()}}function N(){if(pa.apply(this,arguments)){var J=this.__zoom,ha=ve(this),Ga=J.invert(ha);J=Sa(q(n(J,J.k*(ML.shiftKey?.5:2)),ha,Ga),Ea.apply(this,arguments),lc);um();0<Kc?Mc(this).transition().duration(Kc).call(v,J,ha):Mc(this).call(k.transform,J)}}function P(){if(pa.apply(this,
arguments)){var J=z(this,arguments),ha=ML.changedTouches,Ga=ha.length,Ra;ML.stopImmediatePropagation();for(Ra=0;Ra<Ga;++Ra){var yb=ha[Ra];var Pa=Ed(this,ha,yb.identifier);Pa=[Pa,this.__zoom.invert(Pa),yb.identifier];if(J.xc)J.Uc||(J.Uc=Pa);else{J.xc=Pa;var Tb=!0}}if(Ib&&(Ib=clearTimeout(Ib),!J.Uc)){J.end();(Pa=Mc(this).on("dblclick.zoom"))&&Pa.apply(this,arguments);return}Tb&&(Ib=setTimeout(function(){Ib=null},500),eq(this),J.start())}}function X(){var J=z(this,arguments),ha=ML.changedTouches,Ga=
ha.length,Ra;um();Ib&&(Ib=clearTimeout(Ib));for(Ra=0;Ra<Ga;++Ra){var yb=ha[Ra];var Pa=Ed(this,ha,yb.identifier);J.xc&&J.xc[2]===yb.identifier?J.xc[0]=Pa:J.Uc&&J.Uc[2]===yb.identifier&&(J.Uc[0]=Pa)}yb=J.pi.__zoom;if(J.Uc){Pa=J.xc[0];ha=J.xc[1];Ra=J.Uc[0];Ga=J.Uc[1];var Tb=(Tb=Ra[0]-Pa[0])*Tb+(Tb=Ra[1]-Pa[1])*Tb;var qd=(qd=Ga[0]-ha[0])*qd+(qd=Ga[1]-ha[1])*qd;yb=n(yb,Math.sqrt(Tb/qd));Pa=[(Pa[0]+Ra[0])/2,(Pa[1]+Ra[1])/2];Tb=[(ha[0]+Ga[0])/2,(ha[1]+Ga[1])/2]}else if(J.xc)Pa=J.xc[0],Tb=J.xc[1];else return;
J.zoom("touch",Sa(q(yb,Pa,Tb),J.hl,lc))}function la(){var J=z(this,arguments),ha=ML.changedTouches,Ga=ha.length,Ra;ML.stopImmediatePropagation();za&&clearTimeout(za);za=setTimeout(function(){za=null},500);for(Ra=0;Ra<Ga;++Ra){var yb=ha[Ra];J.xc&&J.xc[2]===yb.identifier?delete J.xc:J.Uc&&J.Uc[2]===yb.identifier&&delete J.Uc}J.Uc&&!J.xc&&(J.xc=J.Uc,delete J.Uc);J.xc?J.xc[1]=this.__zoom.invert(J.xc[0]):J.end()}var pa=PW,Ea=QW,Sa=TW,Va=RW,kc=SW,xb=[0,Infinity],lc=[[-Infinity,-Infinity],[Infinity,Infinity]],
Kc=250,Gb=NU,ub=[],La=d("start","zoom","end"),Ib,za,Fa=0;k.transform=function(J,ha){var Ga=J.selection?J.selection():J;Ga.Bx("__zoom",bG);J!==Ga?v(J,ha):Ga.HC().each(function(){z(this,arguments).start().zoom(null,"function"===typeof ha?ha.apply(this,arguments):ha).end()})};k.UE=function(J,ha,Ga){k.transform(J,function(){return Sa(this.__zoom.translate("function"===typeof ha?ha.apply(this,arguments):ha,"function"===typeof Ga?Ga.apply(this,arguments):Ga),Ea.apply(this,arguments),lc)})};A.prototype=
{start:function(){1===++this.active&&(this.index=ub.push(this)-1,this.emit("start"));return this},zoom:function(J,ha){this.Kd&&"mouse"!==J&&(this.Kd[1]=ha.invert(this.Kd[0]));this.xc&&"touch"!==J&&(this.xc[1]=ha.invert(this.xc[0]));this.Uc&&"touch"!==J&&(this.Uc[1]=ha.invert(this.Uc[0]));this.pi.__zoom=ha;this.emit("zoom");return this},end:function(){0===--this.active&&(ub.splice(this.index,1),this.index=-1,this.emit("end"));return this},emit:function(J){var ha=new OW(k,J,this.pi.__zoom),Ga=La.apply;
J=[J,this.pi,this.bf];var Ra=ML;ha.Mo=ML;ML=ha;try{Ga.apply(La,J)}finally{ML=Ra}}};k.wheelDelta=function(J){return arguments.length?(Va="function"===typeof J?J:zv(+J),k):Va};k.filter=function(J){return arguments.length?(pa="function"===typeof J?J:zv(!!J),k):pa};k.hl=function(J){return arguments.length?(Ea="function"===typeof J?J:zv([[+J[0][0],+J[0][1]],[+J[1][0],+J[1][1]]]),k):Ea};k.ZD=function(J){return arguments.length?(xb[0]=+J[0],xb[1]=+J[1],k):[xb[0],xb[1]]};k.duration=function(J){return arguments.length?
(Kc=+J,k):Kc};k.interpolate=function(J){return arguments.length?(Gb=J,k):Gb};k.on=function(){var J=La.on.apply(La,arguments);return J===La?k:J};return k};TL=cG;Td.KG=Td.svg;Td.SG=Td.xhtml;Td.TG=Td.xlink;Td.xml=Td.xml;Td.UG=Td.xmlns})();function UL(a){0<=a.Ud&&(clearTimeout(a.Ud),a.Ud=-1);var b=a.yy;b.C&&jE(b.C);fn(a.ta)}function VL(a,b,c,d=!1,e=!1){WL(a);a.O(b)?(UL(a),a.Ai(c,a.ga,b,[],d,e,!0)):a.vi.C.postMessage({eventType:6,modelGraphId:a.ga.id,showOnNodeItemTypes:a.F,nodeDataProviderRuns:a.lb,nodeId:b,rendererId:c,noNodeShake:d,select:e})}
function XL(a,b,c,d=!1,e,f=!1,g,h){WL(a);a.vi.C.postMessage({eventType:4,modelGraphId:a.ga.id,showOnNodeItemTypes:g||a.F,nodeDataProviderRuns:a.lb,selectedNodeId:b,targetDeepestGroupNodeIdsToExpand:c,rendererId:a.rendererId,forRestoringUiState:d,rectToZoomFit:e,clearAllExpandStates:f,forRestoringSnapshotAfterTogglingFlattenLayers:h})}
function YL(a,b,c=!1,d){WL(a);b={eventType:2,modelGraphId:a.ga.id,groupNodeId:null==b?void 0:b.id,expand:null==d?!(null==b?0:b.expanded):d,showOnNodeItemTypes:a.F,nodeDataProviderRuns:a.lb,rendererId:a.rendererId,paneId:a.paneId,all:c,ts:Date.now()};a.vi.C.postMessage(b)}
function ZL(a,b){const c=RL(b);let d=0,e=0;a.zoom.ZD([1E-4,20]).wheelDelta(()=>-ML.deltaY*(ML.deltaMode?120:1)/150).filter(()=>{"mousedown"===ML.type&&(d=a.kh,e=a.lh);if(2===ML.button||a.nk&&ML.ctrlKey&&0===ML.button&&"mousedown"===ML.type)return!1;if("dblclick"===ML.type)return ML.stopPropagation(),a.il(ML.altKey,ML.shiftKey),!1;if("wheel"===ML.type&&!ML.ctrlKey){const f=.5/a.N;a.zoom.UE(c,-Number(ML.deltaX)*f,-Number(ML.deltaY)*f);ML.preventDefault();return!1}return!0}).on("zoom",()=>{a.xl()}).on("end",
()=>{a.ht(d,e)});a.zoom.interpolate(QL);c.call(a.zoom)}function WL(a){UL(a);a.Ud=window.setTimeout(()=>{0>a.Ud||(a.yy.open("Processing. Please wait..."),fn(a.ta))},1E3)}function $L(a,b){b=a.ga.nodesById[b];null==b||b.expanded||YL(a,b,!1,!0)}function aM(a,b){wJ(a.C,a.paneId,{nodeId:b,rendererId:a.rendererId,Cd:""===b?!1:oI(a.ga.nodesById[b])})}function bM(a,b){a.ga=Object.assign({},b,{edgesByGroupNodeIds:Object.assign({},a.ga.edgesByGroupNodeIds,b.edgesByGroupNodeIds)});a.C.P[a.paneId]=a.ga}
function cM(a){if(a.ga){a.Va=[];a.J=[];a.oa={};a.Lb=[];var b=!0,c=!1,d=e=>{const f=a.ga.nodesById[e||""];if(b&&f){a.Va.push({type:0,id:f.id,node:f});var g={node:f,index:a.Va.length-1};a.J.push(g);a.oa[g.node.id]=g;oI(f)&&f.sectionContainer&&(c=!0)}b=!1;if(e&&f&&f.expanded||!e)for(const h of a.ga.edgesByGroupNodeIds[e||""]||[])a.Va.push({type:1,id:h.id,edge:h}),a.Lb.push({edge:h,index:a.Va.length-1});g=[];e?f.expanded&&(g=(f.nsChildrenIds||[]).map(h=>a.ga.nodesById[h])):g=a.ga.rootNodes;for(const h of g)e=
{type:0,id:h.id,node:h},!c&&oI(h)&&h.sectionContainer&&(c=!0),a.Va.push(e),e={node:h,index:a.Va.length-1},a.J.push(e),a.oa[e.node.id]=e,oI(h)&&h.expanded&&d(h.id)};d(a.am);DJ(a.C,a.paneId,c)}}function dM(a){return(a.x||0)+(a.globalX||0)}function eM(a){return(a.y||0)+(a.globalY||0)}function fM(a,b,c,d){b?setTimeout(()=>{const e=c.nodesById[b];a.Qc({x:dM(e),y:eM(e),width:e.width||0,height:e.height||0},.9,d,!1,!0,!0)},0):setTimeout(()=>{a.ae(.9,d)})}
function gM(a,b,c=!1,d){1===b.nodeType&&0!==(b.nsChildrenIds||[]).length&&YL(a,b,c,d)}function hM(a){if(!a.selectedNodeId||oI(a.ga.nodesById[a.selectedNodeId]))return!1;const b=a.ga.nodesById[a.am||""];return b&&oI(b)&&!(b.descendantsOpNodeIds||[]).includes(a.selectedNodeId)||!a.O(a.selectedNodeId)?!1:!0}function iM(a,b,c,d){return a.ga.edgesByGroupNodeIds[""===b?"":`${b}___group___`].find(e=>{var f=a.ga.nodesById[e.toNodeId];e=jM(a.ga.nodesById[e.fromNodeId],c);f=jM(f,d);return e&&f})}
function kM(a,b,c){let d=b;const e=[];for(;d;){oI(d)&&!d.expanded&&e.push(d);if(d.namespace===c)break;d=a.ga.nodesById[d.nsParentId||""]}return 0<e.length?e[e.length-1]:b}
function lM(a,b,c){var d=dM(c);const e=eM(c),f=c.width||0;c=c.height||0;d=[{point:{x:d+f/2,y:e},distance:0,direction:"horizontal"},{point:{x:d+f,y:e+c/2},distance:0,direction:"vertical"},{point:{x:d+f/2,y:e+c},distance:0,direction:"horizontal"},{point:{x:d,y:e+c/2},distance:0,direction:"vertical"}];for(const g of d)g.distance=Math.pow(a-g.point.x,2)+Math.pow(b-g.point.y,2);d.sort((g,h)=>g.distance-h.distance);return d[0].direction!==d[1].direction?mM(a,b,d[0].point.x,d[0].point.y,d[0].direction)>=
mM(a,b,d[1].point.x,d[1].point.y,d[1].direction)?d[0].point:d[1].point:d[0].point}
function nM(a,b){const c=a.ga.nodesById[a.selectedNodeId],d=[],e=[],f={},g=[];for(const u of c.incomingEdges||[]){if(b[u.sourceNodeId])continue;var h=a.ga.nodesById[u.sourceNodeId];if(!h)continue;var m=FI(h.namespace,c.namespace),p=iM(a,m,h.id,c.id);if(!p)continue;d.push(p);const w=kM(a,h,m);e.push(w);null==f[w.id]&&(f[w.id]=[]);f[w.id].push(h);h=[];const B=a.ga.nodesById[p.fromNodeId];p.fromNodeId!==w.id&&(m=lM(p.points[0].x+(B.globalX||0),p.points[0].y+(B.globalY||0),w),h.push({x:m.x-(w.globalX||
0),y:m.y-(w.globalY||0)}));h.push(...p.points.map(G=>({x:G.x-(w.globalX||0)+(B.globalX||0),y:G.y-(w.globalY||0)+(B.globalY||0)})));p.toNodeId!==a.selectedNodeId&&(p=lM(p.points[p.points.length-1].x+(B.globalX||0),p.points[p.points.length-1].y+(B.globalY||0),c),h.push({x:p.x-(w.globalX||0),y:p.y-(w.globalY||0)}));g.push({id:`overlay_${w.id}___${c.id}`,fromNodeId:w.id,toNodeId:c.id,points:h,type:"incoming"})}return{ms:d,wC:e,Ie:f,Ql:g}}
function oM(a,b){const c=a.ga.nodesById[a.selectedNodeId],d=[],e=[],f={},g=[];for(const u of c.outgoingEdges||[]){if(b[u.sourceNodeOutputId])continue;var h=a.ga.nodesById[u.targetNodeId];if(!h)continue;var m=FI(h.namespace,c.namespace),p=iM(a,m,c.id,h.id);if(!p)continue;d.push(p);m=kM(a,h,m);e.push(m);null==f[m.id]&&(f[m.id]=[]);f[m.id].push(h);h=[];const w=a.ga.nodesById[p.fromNodeId];if(p.fromNodeId!==a.selectedNodeId){const B=lM(p.points[0].x+(w.globalX||0),p.points[0].y+(w.globalY||0),c);h.push({x:B.x-
(c.globalX||0),y:B.y-(c.globalY||0)})}h.push(...p.points.map(B=>({x:B.x-(c.globalX||0)+(w.globalX||0),y:B.y-(c.globalY||0)+(w.globalY||0)})));p.toNodeId!==m.id&&(p=lM(p.points[p.points.length-1].x+(w.globalX||0),p.points[p.points.length-1].y+(w.globalY||0),m),h.push({x:p.x-(c.globalX||0),y:p.y-(c.globalY||0)}));g.push({id:`overlay_${c.id}___${m.id}`,fromNodeId:c.id,toNodeId:m.id,points:h,type:"outgoing"})}return{ms:d,wC:e,mf:f,Ql:g}}
function pM(a){if(a.selectedNodeId&&nI(a.ga.nodesById[a.selectedNodeId])){for(var b=new Set,c=new Set,d=[a.selectedNodeId];0<d.length;){var e=d.shift();if(!c.has(e)){c.add(e);var f=a.ga.nodesById[e];f.hideInLayout||b.add(e);for(const g of f.incomingEdges||[])d.push(g.sourceNodeId)}}c=new Set;for(d=[a.selectedNodeId];0<d.length;)if(e=d.shift(),!c.has(e)){c.add(e);f=a.ga.nodesById[e];f.hideInLayout||b.add(e);for(const g of f.outgoingEdges||[])d.push(g.targetNodeId)}for(const g of[...b])for(d=g;;){d=
a.ga.nodesById[d];if(!d.nsParentId||b.has(d.nsParentId))break;d=d.nsParentId;b.add(d)}a.ua={OG:a.selectedNodeId,Xs:b}}}function qM(a){const b=a.jh()||{},c={},d=new Set;for(const {node:e}of a.J)if(oI(e)&&!e.expanded){const f={};for(const g of e.descendantsOpNodeIds||[]){let h;const m=(null==(h=b[a.ga.nodesById[g].id])?void 0:h.bgColor)||"";m&&(null==f[m]&&(f[m]=0),f[m]++,d.add(m))}c[e.id]=f}return{mC:c,BE:[...d].sort((e,f)=>e.localeCompare(f))}}
function rM(a,b){b=new Bu({vc:Mu(Ru(Lu(Sv(a.Ox.position(),b),[{bb:"start",cb:"bottom",Na:"start",Ua:"top"},{bb:"start",cb:"top",Na:"start",Ua:"bottom"}]),b.clientWidth),20),Vb:!0,fc:"cdk-overlay-transparent-backdrop",maxHeight:"400px",hb:"io-tree-popup-container"});const c=a.Ox.create(b);a.Od=new Vv($K,a.yb);c.bd().subscribe(()=>{c.dispose()});return c}
function sM(a,b,c,d){const e=rM(a,b);b=e.attach(a.Od);c=aL(c,d);b.instance.Lo=!0;b.instance.rendererId=a.rendererId;ZK(b.instance,c);b.instance.ne.subscribe(()=>{e.dispose()})}function tM(a){return nI(a)?"Placeholder"===a.label?a.id:a.label:oI(a)?a.label:"-"}function jM(a,b){return nI(a)&&a.id===b||oI(a)&&(a.descendantsOpNodeIds||[]).includes(b)}function mM(a,b,c,d,e){return"horizontal"===e?Math.atan(Math.abs(d-b)/Math.abs(c-a)):Math.atan(Math.abs(c-a)/Math.abs(d-b))}
var uM=class{constructor(){this.Rb=!1;this.Ol=new Jk;this.Cg="";this.Yg=!1;this.Va=[];this.J=[];this.oa={};this.Lb=[];this.zoom=SL();this.N=1;this.lh=this.kh=0;this.Vr=this.selectedNodeId="";this.lb={};this.Od=null;this.Ud=-1;this.lt=void 0;this.C=of(VJ);this.vi=of(bJ);this.Ex=of(LL);this.G=of(Ci);this.Re=of(aJ);this.Ox=of(cw);this.yb=of(np);this.ta=of(Tr);this.yy=of(vE);this.nk="undefined"!==typeof navigator&&/Macintosh/.test(navigator.userAgent);this.F={};this.ea={};this.it=a=>{UL(this);a=a.data;
switch(a.eventType){case 3:this.rendererId===a.rendererId&&this.ll(a.modelGraph,a.rendererId,a.groupNodeId,a.expanded,a.deepestExpandedGroupNodeIds);break;case 5:this.rendererId===a.rendererId&&this.rl(a.rendererId,a.modelGraph,a.selectedNodeId,a.forRestoringUiState,a.rectToZoomFit,a.forRestoringSnapshotAfterTogglingFlattenLayers,a.targetDeepestGroupNodeIdsToExpand);break;case 7:this.rendererId===a.rendererId&&this.Ai(a.rendererId,a.modelGraph,a.nodeId,a.deepestExpandedGroupNodeIds,!0===a.noNodeShake,
!0===a.select);break;case 10:this.paneId===a.paneId&&this.Ol.emit({id:a.rendererId,Kn:a.modelGraph.nodesById[a.groupNodeId],initialPosition:a.initialPosition,ga:a.modelGraph})}};this.Sg=Cs(()=>{const a=nJ(this.C,this.paneId);if(a)return a.Sg});this.jh=Cs(()=>{let a;return((null==(a=HL(this.Ex,this.paneId,this.ga))?void 0:a.results)||{})[this.ga.id]});this.vi.C.addEventListener("message",this.it);Ks(()=>{this.Dz=this.C.G()});C(this.C.ja,tL(this.G)).subscribe(()=>{let a;this.rendererId===(null==(a=
this.Dz)?void 0:a.id)&&this.ae()});Ks(()=>{const a=this.C.F();(null==a?void 0:a.rendererId)===this.rendererId&&(a&&VL(this,a.nodeId,a.rendererId,a.noNodeShake,a.select),this.C.F.set(void 0))},{fq:!0});Ks(()=>{var a=nJ(this.C,this.paneId);if(a&&a.modelGraph&&(a=a.sD)){{const b=this.ga.nodesById[a];b?(XL(this,a,b.nsParentId?[b.nsParentId]:[]),a=!0):a=!1}a&&EJ(this.C,this.paneId)}},{fq:!0});Ks(()=>{const a=wL(this.Ex,this.ga);this.lb={};for(const b of a)this.lb[b.runId]=b});Ks(()=>{const a=this.jh();
a!==this.lt&&(this.gt(),this.lt=a)});Ks(()=>{if(nJ(this.C,this.paneId)){var a=kJ(this.C,this.paneId,this.rendererId);JSON.stringify(a)!==JSON.stringify(this.F)&&(this.F=a,XL(this,this.selectedNodeId))}});C(this.C.ea,tL(this.G)).subscribe(a=>{a.rendererId===this.rendererId&&YL(this,void 0,!0,a.cw)})}va(){this.vi.C.removeEventListener("message",this.it)}il(){}fl(){}xl(){this.N=ML.transform.k;this.kh=ML.transform.x;this.lh=ML.transform.y}ht(){}ll(a){bM(this,a);cM(this);this.selectedNodeId&&!this.O(this.selectedNodeId)&&
wJ(this.C,this.paneId,{nodeId:"",rendererId:this.rendererId,Cd:!1})}rl(a,b){bM(this,b);cM(this)}Ai(a,b,c,d,e,f){bM(this,b);cM(this);f&&wJ(this.C,this.paneId,{nodeId:c,rendererId:a,Cd:oI(this.ga.nodesById[c])})}Qc(){}ae(){}gt(){}O(){return!1}no(){(this.Yg=!this.Yg)?pM(this):this.ua=void 0}};uM.K=function(a){return new (a||uM)};
uM.Ca=Jf({type:uM,ha:[["renderer"]],inputs:{modelGraph:"modelGraph",rendererId:"rendererId",paneId:"paneId",am:"rootNodeId",Rb:"inPopup"},outputs:{Ol:"openInPopupClicked"},ka:!0,features:[Nr],ya:0,za:0,sa:function(){},pb:2});function vM(a,b){a.C.update(c=>{!0!==c[b]?c[b]=!0:delete c[b];return Object.assign({},c)})}function wM(a,b,c){var d=!0!==a.C()[b];for(const e of c)e!==b&&(a.C()[e]||(d=!1));if(d)a.C.set({});else{d={};for(const e of c)e!==b&&(d[e]=!0);a.C.set(d)}}function xM(a,b){a.D.update(c=>{!0!==c[b]?c[b]=!0:delete c[b];return Object.assign({},c)})}function yM(a,b,c){var d=!0!==a.D()[b];for(const e of c)e!==b&&(a.D()[e]||(d=!1));if(d)a.D.set({});else{d={};for(const e of c)e!==b&&(d[e]=!0);a.D.set(d)}}
function zM(a,b){return!a.C()[b]}function AM(a,b){return!a.D()[b]}var BM=class{constructor(){this.C=yr({});this.D=yr({})}};BM.K=function(a){return new (a||BM)};BM.na=Pd({ma:BM,ba:BM.K});function CM(a,b){a.C.update(c=>{if(!a.modelGraph)return c;var d=a.modelGraph.nodesById[b];if(oI(d)){d=d.descendantsOpNodeIds||[];const e=d.some(f=>c[f]);for(const f of d)nI(a.modelGraph.nodesById[f])&&(e?delete c[f]:c[f]=!0)}else c[b]?delete c[b]:c[b]=!0;return Object.assign({},c)})}
function DM(a){if(a.modelGraph){var b=JJ(a.D,a.modelGraph.id);if(b){b=JSON.parse(JSON.stringify(b));var c=a.C();a=b.nodes.filter(d=>!0===c[d.id]);for(const d of a){const e=(d.incomingEdges||[]).filter(g=>!c[g.sourceNodeId]),f=new Set(e.map(g=>g.targetNodeInputId));d.incomingEdges=(d.incomingEdges||[]).filter(g=>c[g.sourceNodeId]);d.inputsMetadata=(d.inputsMetadata||[]).filter(g=>!f.has(g.id))}return{id:`${b.id}_subgraph`,collectionLabel:b.collectionLabel,nodes:a}}}}
var EM=class{constructor(a){this.D=a;this.C=yr({});this.br=Cs(()=>0<Object.keys(this.C()).length);this.F=Cs(()=>Object.keys(this.C()).length);this.paneId=""}get modelGraph(){return this.D.P[this.paneId]}};EM.K=function(a){return new (a||EM)(r(VJ))};EM.na=Pd({ma:EM,ba:EM.K});PH`https://www.gstatic.com/external_hosted/threejs-r134/three.min.js`;const HI=PH`/static_files/main_deps.js`;
function FM(a){return ta(function*(){"undefined"===typeof THREE&&(yield GI());const b=yield Promise.all([GM("/static_files/GoogleSansTextRegular.png"),GM("/static_files/GoogleSansTextMedium.png"),GM("/static_files/GoogleSansTextBold.png"),GM("/static_files/icons_20240521.png"),HM("/static_files/GoogleSansTextRegular.json"),HM("/static_files/GoogleSansTextMedium.json"),HM("/static_files/GoogleSansTextBold.json"),HM("/static_files/icons_20240521.json")]);a.ja=b[0];a.ea=b[1];a.aa=b[2];a.ca=b[3];a.D=
b[4].fn;a.J=b[5].fn;a.F=b[6].fn;a.G=b[7].fn;a.C=b[4].En;a.T=b[5].En;a.O=b[6].En;a.P=b[7].En})}function GM(a){return ta(function*(){return new Promise(b=>{(new THREE.TextureLoader).load(a,c=>{b(c)})})})}function HM(a){return ta(function*(){const b=yield(yield fetch(a)).json(),c={};for(const d of b.chars)c[d.char]=d;return{En:b,fn:c}})}function IM(a,b){switch(b){case 0:return a.D;case 1:return a.J;case 2:return a.F;case 4:return a.G;default:return a.D}}
function JM(a,b){switch(b){case 0:return a.C;case 1:return a.T;case 2:return a.O;case 4:return a.P;default:return a.C}}var KM=class{constructor(){this.D={};this.J={};this.F={};this.G={};this.N=new Promise(a=>{const b=this;return ta(function*(){yield FM(b);a()})})}};KM.K=function(a){return new (a||KM)};KM.na=Pd({ma:KM,ba:KM.K,fa:"root"});function LM(a,b,c){if(0===b.length)a.C=void 0,a.D=void 0;else{var d=[],e=[],f=[],g=[],h=[],m=[],p=[],u=[],w={},B={};a.G={};var G=0,L=0;for(const {edge:na,index:K}of b){b=na.curvePoints||[];const W=c.nodesById[na.fromNodeId],S=c.nodesById[na.toNodeId],ja=W.globalX||0,F=W.globalY||0;for(let I=0;I<b.length-1;I++){var O=b[I],ba=b[I+1],ca=`${W.id}__${S.id}___${I}`;O=[O.x+ja,O.y+F,ba.x+ja,ba.y+F];(ba=a.ca[ca])?d.push(...ba.YB):d.push(...O);e.push(...O);m.push(.001*K);h.push(a.color.r,a.color.g,a.color.b);
w[ca]={YB:O,index:G};null==a.G[na.id]&&(a.G[na.id]=[]);a.G[na.id].push(w[ca]);I===b.length-2&&(ca=na.id,(ba=a.N[ca])?f.push(...ba.WC):f.push(...O),g.push(...O),p.push(.001*K+5E-4),u.push(a.color.r,a.color.g,a.color.b),B[ca]={index:L,WC:O});G++}L++}a.ca=w;a.N=B;a.F=h;a.ta=m;a.oa=p;c=(new THREE.InstancedBufferGeometry).copy(a.ea);c.instanceCount=m.length;c.setAttribute("endPoints",new THREE.InstancedBufferAttribute(new Float32Array(d),4));c.setAttribute("targetEndPoints",new THREE.InstancedBufferAttribute(new Float32Array(e),
4));c.setAttribute("color",new THREE.InstancedBufferAttribute(new Float32Array(h),3));c.setAttribute("yOffset",new THREE.InstancedBufferAttribute(new Float32Array(m),1));a.C=new THREE.Mesh(c,a.material);a.C.frustumCulled=!1;d=(new THREE.InstancedBufferGeometry).copy(a.ja);d.instanceCount=p.length;d.setAttribute("endPoints",new THREE.InstancedBufferAttribute(new Float32Array(f),4));d.setAttribute("targetEndPoints",new THREE.InstancedBufferAttribute(new Float32Array(g),4));d.setAttribute("color",new THREE.InstancedBufferAttribute(new Float32Array(u),
3));d.setAttribute("yOffset",new THREE.InstancedBufferAttribute(new Float32Array(p),1));a.D=new THREE.Mesh(d,a.ua);a.D.frustumCulled=!1}}function MM(a,b){if(a.C){var c=a.C.geometry.getAttribute("color");if(0<b.length)for(const d of b)for(const e of a.G[d]||[])c.setXYZ(e.index,.92,.92,.92),a.P.push(e);c.needsUpdate=!0}if(a.D){c=a.D.geometry.getAttribute("color");if(0<b.length)for(const d of b)b=a.N[d],c.setXYZ(b.index,.92,.92,.92),a.O.push(b);c.needsUpdate=!0}}
function NM(a,b){if(a.C){var c=a.C.geometry.getAttribute("yOffset");if(0<b.length)for(const d of b)for(const e of a.G[d]||[])c.setX(e.index,1E3),a.aa.push(e);c.needsUpdate=!0}if(a.D){c=a.D.geometry.getAttribute("yOffset");if(0<b.length)for(const d of b)b=a.N[d],c.setX(b.index,1E3),a.T.push(b);c.needsUpdate=!0}}
function OM(a){if(a.C){var b=a.C.geometry.getAttribute("color");if(0<a.P.length){for(var c of a.P){const d=c.index;b.setXYZ(d,a.F[3*d],a.F[3*d+1],a.F[3*d+2])}b.needsUpdate=!0}a.P=[]}if(a.D){b=a.D.geometry.getAttribute("color");if(0<a.O.length){for(const d of a.O)c=d.index,b.setXYZ(c,a.F[3*c],a.F[3*c+1],a.F[3*c+2]);b.needsUpdate=!0}a.O=[]}}
function PM(a){if(a.C){var b=a.C.geometry.getAttribute("yOffset");if(0<a.aa.length){for(var c of a.aa){const d=c.index;b.setX(d,a.ta[d])}b.needsUpdate=!0}a.aa=[]}if(a.D){b=a.D.geometry.getAttribute("yOffset");if(0<a.T.length){for(const d of a.T)c=d.index,b.setX(c,a.oa[c]);b.needsUpdate=!0}a.T=[]}}function QM(a){a.ca={};a.G={};a.N={}}
var RM=class{constructor(a,b){this.color=a;this.wa=b;this.ca={};this.G={};this.N={};this.J={value:-1};this.F=[];this.ta=[];this.oa=[];this.P=[];this.O=[];this.aa=[];this.T=[];this.ea=new THREE.PlaneGeometry(1,1);this.ea.rotateX(-Math.PI/2);this.material=new THREE.ShaderMaterial({uniforms:{edgeWidth:{value:this.wa},animationProgress:this.J},vertexShader:"\nprecision highp float;\n\n#define PI 3.1415926535897932384626433832795\n\nuniform float edgeWidth;\nuniform float animationProgress;\n\nattribute vec4 endPoints;\nattribute vec4 targetEndPoints;\nattribute vec3 color;\nattribute float yOffset;\n\nvarying vec3 vColor;\n\nfloat atan2(in float y, in float x) {\n  bool s = (abs(x) > abs(y));\n  return mix(PI/2.0 - atan(x,y), atan(y,x), s);\n}\n\nvoid main() {\n  vColor = color;\n\n  vec3 pos = position;\n\n  float curStartX = endPoints.x;\n  float curStartY = endPoints.y;\n  float curEndX = endPoints.z;\n  float curEndY = endPoints.w;\n  float targetStartX = targetEndPoints.x;\n  float targetStartY = targetEndPoints.y;\n  float targetEndX = targetEndPoints.z;\n  float targetEndY = targetEndPoints.w;\n\n  float progress = animationProgress * step(0.0, animationProgress); \n  float startX = curStartX + (targetStartX - curStartX) * progress;\n  float startY = curStartY + (targetStartY - curStartY) * progress;\n  float endX = curEndX + (targetEndX - curEndX) * progress;\n  float endY = curEndY + (targetEndY - curEndY) * progress;\n\n  float length = distance(vec2(startX, startY), vec2(endX, endY));\n  pos.x = (step(0.0, pos.x) * 2.0 - 1.0) * (edgeWidth / 2.0);\n  pos.z = (step(0.0, pos.z) * 2.0 - 1.0) * (length / 2.0);\n\n  float angle = PI / 2.0 - atan2(endY - startY, endX - startX);\n  float c = cos(angle);\n  float s = sin(angle);\n\n  float posX = pos.x;\n  float posZ = pos.z;\n  pos.x = posX * c + posZ * s;\n  pos.z = posZ * c - posX * s;\n\n  float centerX = (startX + endX) / 2.0;\n  float centerZ = (startY + endY) / 2.0;\n\n  gl_Position = projectionMatrix * modelViewMatrix *\n      vec4(pos.x + centerX, yOffset, pos.z + centerZ, 1.0);\n}\n",
fragmentShader:"\nprecision highp float;\n\nvarying vec3 vColor;\n\nvoid main() {\n  gl_FragColor = vec4(vColor, 1.0);\n}\n",transparent:!0});a=new THREE.Shape;a.moveTo(-3,-6).lineTo(0,-4).lineTo(3,-6).lineTo(0,0).lineTo(-3,-6);this.ja=new THREE.ShapeGeometry(a);this.ja.rotateX(-Math.PI/2);this.ua=new THREE.ShaderMaterial({uniforms:{animationProgress:this.J},vertexShader:"\nprecision highp float;\n\n#define PI 3.1415926535897932384626433832795\n\nuniform float animationProgress;\n\n// End points of the last segment of the edge.\nattribute vec4 endPoints;\nattribute vec4 targetEndPoints;\nattribute vec3 color;\nattribute float yOffset;\n\nvarying vec3 vColor;\n\nfloat atan2(in float y, in float x) {\n  bool s = (abs(x) > abs(y));\n  return mix(PI/2.0 - atan(x,y), atan(y,x), s);\n}\n\nvoid main() {\n  vColor = color;\n\n  vec3 pos = position;\n\n  float curStartX = endPoints.x;\n  float curStartY = endPoints.y;\n  float curEndX = endPoints.z;\n  float curEndY = endPoints.w;\n  float targetStartX = targetEndPoints.x;\n  float targetStartY = targetEndPoints.y;\n  float targetEndX = targetEndPoints.z;\n  float targetEndY = targetEndPoints.w;\n\n  float progress = animationProgress * step(0.0, animationProgress); \n  float startX = curStartX + (targetStartX - curStartX) * progress;\n  float startY = curStartY + (targetStartY - curStartY) * progress;\n  float endX = curEndX + (targetEndX - curEndX) * progress;\n  float endY = curEndY + (targetEndY - curEndY) * progress;\n\n  float angle = PI - atan2(endY - startY, endX - startX) + PI / 2.0;\n  float c = cos(angle);\n  float s = sin(angle);\n\n  float posX = pos.x;\n  float posZ = pos.z;\n  pos.x = posX * c + posZ * s;\n  pos.z = posZ * c - posX * s;\n\n  gl_Position = projectionMatrix * modelViewMatrix *\n      vec4(pos.x + endX, yOffset, pos.z + endY, 1.0);\n}\n",
fragmentShader:"\nprecision highp float;\n\nvarying vec3 vColor;\n\nvoid main() {\n  gl_FragColor = vec4(vColor, 1.0);\n}\n",transparent:!0})}clear(){this.C&&this.C.removeFromParent();this.D&&this.D.removeFromParent()}};function SM(a,b,c=!1,d=!1,e=!1,f=!1){if(0===b.length)a.F={},a.C=void 0,a.D=void 0;else{a.ua=b;var g=[],h=[],m=[],p=[],u=[],w=[],B=[],G=[],L=[],O=[],ba=0<Object.keys(a.F).length,ca=Object.assign({},a.F);a.F={};var na=0;for(let K=0;K<b.length;K++){const W=b[K],S=W.bound;let ja;const F=null==(ja=ca[W.id])?void 0:ja.bound;if(ba&&!d||e){let I,aa,Aa,Za,vb,kb,jb,bb;g.push(null!=(vb=null==(I=F)?void 0:I.x)?vb:S.x,null!=(kb=null==(aa=F)?void 0:aa.y)?kb:S.y,null!=(jb=null==(Aa=F)?void 0:Aa.width)?jb:f?S.width:
0,null!=(bb=null==(Za=F)?void 0:Za.height)?bb:f?S.height:0)}else g.push(S.x,S.y,S.width,S.height);m.push(S.x,S.y,S.width,S.height);h.push(W.Sd);p.push(W.Hd?1:0);u.push(W.borderWidth);w.push(W.bgColor.r,W.bgColor.g,W.bgColor.b);B.push(W.borderColor.r,W.borderColor.g,W.borderColor.b);G.push(0);L.push(W.opacity);O.push(W.qB?1:0);a.F[W.id]=W;na++}a.J=B;a.G=w;a.ea=u;a.ja=L;d=(new THREE.InstancedBufferGeometry).copy(a.ta);d.instanceCount=b.length;d.setAttribute("bound",new THREE.InstancedBufferAttribute(new Float32Array(g),
4));d.setAttribute("targetBound",new THREE.InstancedBufferAttribute(new Float32Array(m),4));d.setAttribute("yOffset",new THREE.InstancedBufferAttribute(new Float32Array(h),1));d.setAttribute("isRounded",new THREE.InstancedBufferAttribute(new Float32Array(p),1));d.setAttribute("borderWidth",new THREE.InstancedBufferAttribute(new Float32Array(u),1));d.setAttribute("bgColor",new THREE.InstancedBufferAttribute(new Float32Array(w),3));d.setAttribute("borderColor",new THREE.InstancedBufferAttribute(new Float32Array(B),
3));d.setAttribute("angle",new THREE.InstancedBufferAttribute(new Float32Array(G),1));d.setAttribute("opacity",new THREE.InstancedBufferAttribute(new Float32Array(L),1));d.setAttribute("changeColorWhenFar",new THREE.InstancedBufferAttribute(new Float32Array(O),1));a.C=new THREE.Mesh(d,a.material);a.C.frustumCulled=!1;if(c){a.D=new THREE.InstancedMesh(new THREE.BoxGeometry(1,1,1),a.wa,b.length);for(u=0;u<b.length;u++)w=b[u].bound,c=a,g=a.D,h=u,m=w.width,p=w.height,c.O.position.set(w.x,-10,w.y),c.O.scale.set(m,
1,p),c.O.updateMatrix(),g.setMatrixAt(h,c.O.matrix),g.instanceMatrix.needsUpdate=!0;a.D.frustumCulled=!1}}}function TM(a,b,c,d=!0){if(a.D){var e=b.intersectObject(a.D);b="";0<e.length&&(e=e[e.length-1].instanceId,null!=e&&(b=a.ua[e].id));a.N!==b&&(a.N=b,d&&(document.body.style.cursor=""===a.N?"default":"pointer"),c(a.N,a.F[a.N]))}}
function UM(a,b,c){if(a.C){b=b.filter(e=>""!==e);var d=a.C.geometry.getAttribute("borderColor");if(0<b.length)for(const e of b)if(b=a.F[e])d.setXYZ(b.index,c.r,c.g,c.b),b.borderColor.r=c.r,b.borderColor.g=c.g,b.borderColor.b=c.b,a.T.push(b);d.needsUpdate=!0}}
function VM(a){if(a.C){var b=a.C.geometry.getAttribute("borderColor");if(0<=a.T.length){for(const c of a.T){const d=c.index;b.setXYZ(d,a.J[3*d],a.J[3*d+1],a.J[3*d+2]);c.borderColor.r=a.J[3*d];c.borderColor.g=a.J[3*d+1];c.borderColor.b=a.J[3*d+2]}b.needsUpdate=!0}a.T=[]}}
function WM(a,b,c,d=!1){if(a.C){b=b.filter(f=>""!==f);var e=a.C.geometry.getAttribute("bgColor");if(0<b.length)for(const f of b){b=a.F[f];if(!b)continue;const g=b.index;if(d){const h=a.G[3*g+1],m=a.G[3*g+2];if(1!==a.G[3*g]||1!==h||1!==m)continue}e.setXYZ(g,c.r,c.g,c.b);b.bgColor.r=c.r;b.bgColor.g=c.g;b.bgColor.b=c.b;a.P.push(b)}e.needsUpdate=!0}}
function XM(a){if(a.C){var b=a.C.geometry.getAttribute("bgColor");if(0<=a.P.length){for(const c of a.P){const d=c.index;b.setXYZ(d,a.G[3*d],a.G[3*d+1],a.G[3*d+2]);c.bgColor.r=a.G[3*d];c.bgColor.g=a.G[3*d+1];c.bgColor.b=a.G[3*d+2]}b.needsUpdate=!0}a.P=[]}}function YM(a,b,c){if(a.C){b=b.filter(e=>""!==e);var d=a.C.geometry.getAttribute("borderWidth");if(0<b.length)for(const e of b)if(b=a.F[e])d.setX(b.index,c),b.borderWidth=c,a.aa.push(b);d.needsUpdate=!0}}
function ZM(a){if(a.C){var b=a.C.geometry.getAttribute("borderWidth");if(0<=a.aa.length){for(const c of a.aa){const d=c.index;b.setX(d,a.ea[d]);c.borderWidth=a.ea[d]}b.needsUpdate=!0}a.aa=[]}}function $M(a,b,c){if(a.C){b=b.filter(e=>""!==e);var d=a.C.geometry.getAttribute("opacity");if(0<b.length)for(const e of b)if(b=a.F[e])d.setX(b.index,c),b.opacity=c,a.ca.push(b);d.needsUpdate=!0}}
function aN(a){if(a.C){var b=a.C.geometry.getAttribute("opacity");if(0<=a.ca.length){for(const c of a.ca){const d=c.index;b.setX(d,a.ja[d]);c.opacity=a.ja[d]}b.needsUpdate=!0}a.ca=[]}}function bN(a,b,c){if(a.C){var d=a.C.geometry.getAttribute("angle");if(a=a.F[b])d.setX(a.index,c/180*Math.PI),d.needsUpdate=!0}}function cN(a,b){a.C&&(a.oa.value=b)}function dN(a,b,c){a.C&&(a.material.uniforms.bgColorWhenFar.value=[b.r,b.g,b.b,c])}
var eN=class{constructor(a){this.radius=a;this.N="";this.ua=[];this.F={};this.oa={value:-1};this.O=new THREE.Object3D;this.J=[];this.G=[];this.ea=[];this.ja=[];this.T=[];this.P=[];this.aa=[];this.ca=[];this.ta=new THREE.PlaneGeometry(1,1);this.ta.rotateX(-Math.PI/2);this.material=new THREE.ShaderMaterial({extensions:{derivatives:!0},uniforms:{borderRadius:{value:this.radius},animationProgress:this.oa,bgColorWhenFar:{value:[0,0,0,0]}},vertexShader:'\nprecision highp float;\n\nuniform float borderRadius;\n// Set this to <0 to disable animation.\nuniform float animationProgress;\nuniform vec4 bgColorWhenFar;\n\nattribute vec4 bound;\nattribute vec4 targetBound;\nattribute float yOffset;\nattribute float isRounded;\nattribute float borderWidth;\nattribute vec3 bgColor;\nattribute vec3 borderColor;\nattribute float angle;\nattribute float opacity;\nattribute float changeColorWhenFar;\n\nvarying vec2 vUv;\nvarying vec2 vSize;\nvarying float vIsRounded;\nvarying float vBorderWidth;\nvarying vec3 vBgColor;\nvarying vec3 vBorderColor;\nvarying float vOpacity;\nvarying float vChangeColorWhenFar;\n\nvoid main() {\n  vUv = uv;\n  vIsRounded = isRounded;\n  vBorderWidth = borderWidth;\n  vBgColor = bgColor;\n  vBorderColor = borderColor;\n  vOpacity = opacity;\n  vChangeColorWhenFar = changeColorWhenFar;\n\n  vec3 pos = position;\n  float curX = bound.x;\n  float curY = bound.y;\n  float curW = bound.z;\n  float curH = bound.w;\n\n  float progress = animationProgress * step(0.0, animationProgress); \n  float x = curX + (targetBound.x - curX) * progress;\n  float y = curY + (targetBound.y - curY) * progress;\n  float w = curW + (targetBound.z - curW) * progress;\n  float h = curH + (targetBound.w - curH) * progress;\n  vSize = vec2(w, h);\n\n  // For each vertex, move it by delta calculated below so that the final\n  // rectangle\'s width and height match the width and height stored in "bound".\n  //\n  // pos.x < 0: the first () below returns -1.\n  // pos.x > 0: the first () below returns 1.\n  pos.x = (step(0.0, pos.x) * 2.0 - 1.0) * (w / 2.0);\n  pos.z = (step(0.0, pos.z) * 2.0 - 1.0) * (h / 2.0);\n\n  // Rotate.\n  //\n  float c = cos(angle);\n  float s = sin(angle);\n  float posX = pos.x * c + pos.z * s;\n  float posZ = pos.z * c - pos.x * s;\n\n  gl_Position = projectionMatrix * modelViewMatrix *\n      vec4(posX + x, yOffset, posZ + y, 1.0);\n}\n',
fragmentShader:"\nprecision highp float;\n\nuniform float borderRadius;\nuniform vec4 bgColorWhenFar;\n\nvarying vec2 vUv;\nvarying vec2 vSize;\nvarying float vIsRounded;\nvarying float vBorderWidth;\nvarying vec3 vBgColor;\nvarying vec3 vBorderColor;\nvarying float vOpacity;\nvarying float vChangeColorWhenFar;\n\n// See:\n// https://www.shadertoy.com/view/4tc3DX#\n\n// Clamp [0..1] range\n#define saturate(a) clamp(a, 0.0, 1.0)\n\n// This function will make a signed distance field that says how far you are\n// from the edge of the line at any point U,V.\n// Pass it UVs, line end points, line thickness (x is along the line and y is\n// perpendicular), How rounded the end points should be (0.0 is rectangular,\n// setting rounded to thick.y will be circular).\nfloat LineDistField(vec2 uv, vec2 pA, vec2 pB, vec2 thick, float rounded, float dashOn) {\n  // Don't let it get more round than circular.\n  rounded = min(thick.y, rounded);\n  // midpoint\n  vec2 mid = (pB + pA) * 0.5;\n  // vector from point A to B\n  vec2 delta = pB - pA;\n  // Distance between endpoints\n  float lenD = length(delta);\n  // unit vector pointing in the line's direction\n  vec2 unit = delta / lenD;\n  // Check for when line endpoints are the same\n  if (lenD < 0.0001) unit = vec2(1.0, 0.0);\t// if pA and pB are same\n  // Perpendicular vector to unit - also length 1.0\n  vec2 perp = unit.yx * vec2(-1.0, 1.0);\n  // position along line from midpoint\n  float dpx = dot(unit, uv - mid);\n  // distance away from line at a right angle\n  float dpy = dot(perp, uv - mid);\n  // Make a distance function that is 0 at the transition from black to white\n  float disty = abs(dpy) - thick.y + rounded;\n  float distx = abs(dpx) - lenD * 0.5 - thick.x + rounded;\n\n  // Too tired to remember what this does. Something like rounded endpoints for distance function.\n  float dist = length(vec2(max(0.0, distx), max(0.0,disty))) - rounded;\n  dist = min(dist, max(distx, disty));\n\n  return dist;\n}\n\n// This makes a line in UV units. A 1.0 thick line will span a whole 0..1 in\n// UV space.\nfloat FillLine(vec2 uv, vec2 pA, vec2 pB, vec2 thick, float rounded) {\n  float df = LineDistField(uv, pA, pB, vec2(thick), rounded, 0.0);\n  return saturate(df / abs(dFdy(uv).y));\n}\n\nvoid main() {\n  // Normalize uv.\n  vec2 uv = vUv;\n  uv -= 0.5;\n  float aspect = vSize.x / vSize.y;\n  uv.x *= aspect;\n\n  vec4 finalColor = vec4(1.0);\n\n  float radius = mix(1.0 / vSize.y, borderRadius / vSize.y, step(0.5, vIsRounded));\n  float borderWidth = vBorderWidth / vSize.y;\n\n  // Border.\n  float c = FillLine(uv,\n    vec2(-0.5 * aspect, 0.0), vec2(0.5 * aspect, 0.0), vec2(0.0, 0.5), radius);\n  finalColor = mix(vec4(vBorderColor.rgb, 1.0), vec4(1.0, 1.0, 1.0, 0.0), c);\n\n  // Body.\n  float c2 = FillLine(uv,\n    vec2(-0.5 * aspect + borderWidth, 0.0),\n    vec2(0.5 * aspect - borderWidth, 0.0), vec2(0.0, 0.5 - borderWidth),\n    radius - 1.0 / vSize.y);\n  finalColor = mix(vec4(vBgColor.rgb, 1.0), finalColor, c2);\n  finalColor = saturate(finalColor);\n  finalColor.rgb = mix(finalColor.rgb, bgColorWhenFar.rgb, bgColorWhenFar.a * vChangeColorWhenFar);\n\n  float alpha = finalColor.w * vOpacity;\n  if (alpha < 0.00001) {\n    discard;\n  }\n  gl_FragColor = vec4(finalColor.rgb, alpha);\n}\n",
transparent:!0});this.wa=new THREE.MeshBasicMaterial({opacity:0,transparent:!0})}};function fN(a,b,c,d,e,f,g,h){let m=`${b}__${c}__${g}`,p=a.ca[m];var u=void 0;if(null==p){p=gN(a,b,c,f,g,h);if(null!=e){u=d/a.G;for(d=0;d<p.Lj.length;d++)if(g=p.Lj[d],(g.x+g.width)*u>e){b=b.substring(0,d-1);b+="...";break}u=b;p=gN(a,b,c,f);m=`${b}__${c}__${void 0}`}a.ca[m]=p}return{sizes:a.ca[m],ZE:u}}
function hN(a,b,c=!1,d=!1,e=!1){let f=0;a.O={};a.N={};const g=[],h=[],m=[],p=[],u=[],w=[],B=[],G=[],L=[],O=[],ba=0<Object.keys(a.J).length,ca=Object.assign({},a.J);a.J={};for(const I of b){b=IM(a.F,I.weight);const aa=JM(a.F,I.weight).common.scaleW,Aa=fN(a,I.label,I.weight,I.height,I.maxWidth,I.bk,I.angle,I.Iq).sizes,Za=I.height/a.G;let vb=I.x,kb=I.z;var na=(I.bk?{Lj:[{x:0,y:0,width:20,height:I.height}]}:fN(a,"a",I.weight,I.height).sizes).Lj[0].height*Za;switch(I.vAlign){case "top":kb-=Aa.ho*Za;break;
case "bottom":kb-=Aa.fo*Za;break;case "center":kb-=(Aa.ho+Aa.fo)/2*Za+na/2}switch(I.he){case "left":vb-=Aa.Wh*Za;break;case "right":vb-=Aa.Sh*Za;break;case "center":vb-=(Aa.Wh+Aa.Sh)/2*Za}na=I.y;const jb=w.length;for(let bb=0;bb<Aa.Lj.length;bb++){var K=Aa.Lj[bb],W=I.bk?I.label:I.label[bb],S=b[W]||b["?"];W=`${I.id}_${W}_${bb}`;var ja=K.width*Za,F=K.height*Za;const ma=vb+K.x*Za;K=kb+K.y*Za;K={x:I.Iq?ma:ma+ja/2,y:I.Iq?K:K+F/2,width:ja,height:F};ja=ca[W];if(ba&&!c||d){let Mc,pc,bc,Nc,ve,Ed,Dc,Fd;m.push(null!=
(ve=null==(Mc=ja)?void 0:Mc.x)?ve:K.x,null!=(Ed=null==(pc=ja)?void 0:pc.y)?Ed:K.y,null!=(Dc=null==(bc=ja)?void 0:bc.width)?Dc:e?K.width:0,null!=(Fd=null==(Nc=ja)?void 0:Nc.height)?Fd:e?K.height:0)}else m.push(K.x,K.y,K.width,K.height);p.push(K.x,K.y,K.width,K.height);u.push(na);B.push(S.x/aa,1-S.y/aa-S.height/aa,S.width/aa,S.height/aa);let T,da;S=null!=(da=null==(T=I.color)?void 0:T.r)?da:0;let oa,Ia;ja=null!=(Ia=null==(oa=I.color)?void 0:oa.g)?Ia:0;let db,Oa;F=null!=(Oa=null==(db=I.color)?void 0:
db.b)?Oa:0;h.push(S,ja,F);g.push(I.weight);w.push(1);G.push(I.angle||0);let tb,wb,sa,Ha,Wa,cb;L.push(null!=(Ha=null==(tb=I.borderColor)?void 0:tb.r)?Ha:-1,null!=(Wa=null==(wb=I.borderColor)?void 0:wb.g)?Wa:-1,null!=(cb=null==(sa=I.borderColor)?void 0:sa.b)?cb:-1);let Zb;O.push(null!=(Zb=I.hF)?Zb:.5);a.J[W]=K;f++}I.nodeId&&(a.O[I.nodeId]||(a.O[I.nodeId]=[]),a.O[I.nodeId].push({Vh:jb,Rh:w.length-1}),a.N[I.nodeId]||(a.N[I.nodeId]=[]),a.N[I.nodeId].push({Vh:jb,Rh:w.length-1}))}a.P=h;c=(new THREE.InstancedBufferGeometry).copy(a.ea);
c.instanceCount=u.length;c.setAttribute("bound",new THREE.InstancedBufferAttribute(new Float32Array(m),4));c.setAttribute("targetBound",new THREE.InstancedBufferAttribute(new Float32Array(p),4));c.setAttribute("uvBound",new THREE.InstancedBufferAttribute(new Float32Array(B),4));c.setAttribute("yOffset",new THREE.InstancedBufferAttribute(new Float32Array(u),1));c.setAttribute("weight",new THREE.InstancedBufferAttribute(new Float32Array(g),1));c.setAttribute("color",new THREE.InstancedBufferAttribute(new Float32Array(h),
3));c.setAttribute("opacity",new THREE.InstancedBufferAttribute(new Float32Array(w),1));c.setAttribute("angle",new THREE.InstancedBufferAttribute(new Float32Array(G),1));c.setAttribute("borderColor",new THREE.InstancedBufferAttribute(new Float32Array(L),3));c.setAttribute("weightLevel",new THREE.InstancedBufferAttribute(new Float32Array(O),1));a.C=new THREE.Mesh(c,a.material);a.C.frustumCulled=!1}
function iN(a,b){if(a.C){var c=a.C.geometry.getAttribute("opacity");if(0<b.length)for(const d of b)if(a.O[d])for(const {Vh:e,Rh:f}of a.O[d]){for(b=e;b<=f;b++)c.setX(b,.3);a.aa.push({Vh:e,Rh:f})}c.needsUpdate=!0}}function jN(a,b,c){if(a.C){var d=a.C.geometry.getAttribute("color");if(0<b.length)for(const e of b)if(a.N[e])for(const {Vh:f,Rh:g}of a.N[e]){for(b=f;b<=g;b++)d.setXYZ(b,c.r,c.g,c.b);a.T.push({Vh:f,Rh:g})}d.needsUpdate=!0}}
function kN(a){if(a.C){var b=a.C.geometry.getAttribute("opacity");if(0<=a.aa.length){for(const {Vh:c,Rh:d}of a.aa)for(let e=c;e<=d;e++)b.setX(e,1);b.needsUpdate=!0}a.aa=[]}}function lN(a){if(a.C){var b=a.C.geometry.getAttribute("color");if(0<=a.T.length){for(const {Vh:c,Rh:d}of a.T)for(let e=c;e<=d;e++)b.setXYZ(e,a.P[3*e],a.P[3*e+1],a.P[3*e+2]);b.needsUpdate=!0}a.T=[]}}
function gN(a,b,c,d,e,f){c=IM(a.F,c);const g=[];let h=0,m=Number.MAX_VALUE,p=Number.NEGATIVE_INFINITY;for(const u of d?[b]:b)(b=c[u])||(b=c["?"]),d=b.yoffset,f?(h+=Math.sin(e||0)*(b.yoffset+b.height/2),d=Math.cos(e||0)*(b.yoffset+b.height/2)):h+=b.xoffset,d={x:h,y:d,width:b.width,height:b.height},g.push(d),m=Math.min(m,d.x),p=Math.max(p,d.x+d.width),h+=.98*b.xadvance;return{Lj:g,Wh:m,ho:0,Sh:p,fo:a.G}}
var mN=class{constructor(a){this.F=a;this.ca={};this.J={};this.O={};this.N={};this.aa=[];this.T=[];this.P=[];this.D={value:-1};this.ea=new THREE.PlaneGeometry(1,1);this.ea.rotateX(-Math.PI/2);this.G=this.F.C.info.size;this.distanceRange=this.F.C.distanceField.distanceRange;this.material=new THREE.ShaderMaterial({uniforms:{textureRegular:{value:this.F.ja},textureMedium:{value:this.F.ea},textureBold:{value:this.F.aa},textureIcons:{value:this.F.ca},distanceRange:{value:this.distanceRange},animationProgress:this.D},
extensions:{derivatives:!0},vertexShader:'\nprecision highp float;\n\n// Set this to <0 to disable animation.\nuniform float animationProgress;\n\n// 0: regular. 1: medium. 2: bold.\nattribute float weight;\nattribute vec3 color;\nattribute vec4 bound;\nattribute vec4 targetBound;\nattribute vec4 uvBound;\nattribute float yOffset;\nattribute float opacity;\nattribute float angle;\nattribute vec3 borderColor;\nattribute float weightLevel;\n\nvarying vec2 vUv;\nvarying vec3 vColor;\nvarying float vWeight;\nvarying float vOpacity;\nvarying vec3 vBorderColor;\nvarying float vWeightLevel;\n\nvoid main() {\n  vUv = vec2(0.0, 0.0);\n  vec3 pos = position;\n  if (pos.x < 0.0 && pos.z < 0.0) {\n    vUv = vec2(uvBound.x, uvBound.y + uvBound.w);\n  } else if (pos.x > 0.0 && pos.z < 0.0) {\n    vUv = vec2(uvBound.x + uvBound.z, uvBound.y + uvBound.w);\n  } else if (pos.x > 0.0 && pos.z > 0.0) {\n    vUv = vec2(uvBound.x + uvBound.z, uvBound.y);\n  } else {\n    vUv = vec2(uvBound.x, uvBound.y);\n  }\n\n  vColor = color;\n  vWeight = weight;\n  vOpacity = opacity;\n  vBorderColor = borderColor;\n  vWeightLevel = weightLevel;\n\n  float curX = bound.x;\n  float curY = bound.y;\n  float curW = bound.z;\n  float curH = bound.w;\n\n  float x = curX;\n  float y = curY;\n  float w = curW;\n  float h = curH;\n\n  if (animationProgress >= 0.0) {\n    x = curX + (targetBound.x - curX) * animationProgress;\n    y = curY + (targetBound.y - curY) * animationProgress;\n    w = curW + (targetBound.z - curW) * animationProgress;\n    h = curH + (targetBound.w - curH) * animationProgress;\n  }\n\n  // For each vertex, move it by delta calculated below so that the final\n  // rectangle\'s width and height match the width and height stored in "bound".\n  //\n  if (pos.x < 0.0) {\n    pos.x = - w / 2.0;\n  } else if (pos.x > 0.0) {\n    pos.x = w / 2.0;\n  }\n\n  if (pos.z < 0.0) {\n    pos.z = - h / 2.0;\n  } else if (pos.z > 0.0) {\n    pos.z = h / 2.0;\n  }\n\n  // Rotate.\n  //\n  float c = cos(angle);\n  float s = sin(angle);\n  float posX = pos.x * c + pos.z * s;\n  float posZ = pos.z * c - pos.x * s;\n\n  gl_Position = projectionMatrix * modelViewMatrix *\n      vec4(posX + x, yOffset, posZ + y, 1.0);\n}\n',
fragmentShader:"\nprecision highp float;\n\nuniform sampler2D textureRegular;\nuniform sampler2D textureMedium;\nuniform sampler2D textureBold;\nuniform sampler2D textureIcons;\nuniform float distanceRange;\nvarying vec2 vUv;\nvarying float vWeight;\nvarying vec3 vColor;\nvarying float vOpacity;\nvarying vec3 vBorderColor;\nvarying float vWeightLevel;\n\nvec3 getSampleColor() {\n  if (vWeight < 0.5) {\n    return texture2D(textureRegular, vUv).rgb;\n  }\n  if (vWeight < 1.5) {\n    return texture2D(textureMedium, vUv).rgb;\n  }\n  if (vWeight < 2.5) {\n    return texture2D(textureBold, vUv).rgb;\n  }\n  return texture2D(textureIcons, vUv).rgb;\n}\n\nfloat median(float r, float g, float b) {\n  return max(min(r, g), min(max(r, g), b));\n}\n\nfloat screenPxRange() {\n  vec2 unitRange = vec2(distanceRange) / vec2(textureSize(textureRegular, 0));\n  vec2 screenTexSize = vec2(1.0) / fwidth(vUv);\n  return max(0.5 * dot(unitRange, screenTexSize), 1.0);\n}\n\nvoid main() {\n  vec3 sampleColor = getSampleColor();\n\n  float dist = median(sampleColor.r, sampleColor.g, sampleColor.b);\n  vec3 color = vColor;\n  float threshold = step(0.0, vBorderColor.r) * 0.35;\n  float pxDist = screenPxRange() * (dist - 0.5 + threshold) + vWeightLevel - threshold;\n\n  // Has border.\n  if (vBorderColor.r >= 0.0) {\n    float blur = fwidth(dist) / 2.0;\n    color = mix(vBorderColor, vColor, smoothstep(0.5 - blur, 0.5 + blur, dist));\n  }\n\n  float opacity = clamp(pxDist, 0.0, 1.0);\n  if (opacity > 0.001) {\n    opacity *= vOpacity;\n  }\n  gl_FragColor = vec4(color, opacity);\n}\n",
transparent:!0,alphaToCoverage:!0})}};const nN=["container"],oN=["canvas"],pN=["snapshotCanvas"],qN=["pngDownloaderCanvas"],rN=["hoverToolbar"],sN=["ioPicker"],tN=["subgraphMenuTrigger"],uN=["groupNodeIconMatTooltip"],vN=["ioPickerMatTooltip"],wN=["moreActionsMenuTrigger"],xN=["rangeZoomDragArea"],yN=["dragToSelectDragArea"];
function zN(a){if(a&1){const b=Zg();R(0,"div",33);V("click",function(){t(b);const c=Y();var d=c.wl;if(d=null!=d?d:c.zd)aM(c,d),gM(c,c.ga.nodesById[d],!0,!1);return x()});R(1,"mat-icon");Z(2,"unfold_less_double");U();Z(3," Collapse layer & all sub-layers ");U()}}
function AN(a){if(a&1){const b=Zg();R(0,"div",34);V("click",function(){t(b);const c=Y();var d=c.wl;d=c.ga.nodesById[null!=d?d:c.zd];var e=dM(d);const f=d.width||0;if(d.expanded){const g=fN(c.Ed,tM(d),2,11).sizes;e=BN(c,e+f/2+11/c.Ed.G*(g.Sh-g.Wh)/2+22,0).x}else e=BN(c,e+f+1,0).x;d={eventType:9,modelGraphId:c.ga.id,paneId:c.paneId,rendererId:tI(),groupNodeId:d.id,initialPosition:{x:e,y:BN(c,0,eM(d)).y}};c.vi.C.postMessage(d);return x()});R(1,"mat-icon");Z(2,"open_in_new");U();Z(3," Open layer in popup ");
U()}}function CN(a,b){if(a&1){const c=Zg();R(0,"div",35);V("click",function(){const d=t(c).da,e=Y();DN(e,d);return x()});R(1,"mat-icon");Z(2,"subdirectory_arrow_right");U();Z(3);U()}a&2&&(a=b.da,D(3),Cr(" ",a," "))}function EN(a){a&1&&(R(0,"div",32),Z(1),U());a&2&&(a=Y(),D(),Cr("FPS: ",a.hw,""))}const FN={r:1,g:1,b:1};
function GN(a){if(a.ga)if(HN(a),hM(a)){var b=nM(a,a.cz);if(0<b.Ql.length){var c=b.Ql.map(f=>({edge:Object.assign({},f,{curvePoints:vI(f.points)}),index:95E3}));LM(a.wa,c,a.ga);a.wa.C&&a.scene.add(a.wa.C);a.wa.D&&a.scene.add(a.wa.D);var d;if(null==(d=a.ea["Tensor shape"])?0:d.selected)c=IN(a,c,a.oz),hN(a.Pb,c,!1,!0,!0),a.Pb.C&&a.scene.add(a.Pb.C)}a.Ie=b.Ie;a.Bl=b.ms;b=oM(a,a.dz);if(0<b.Ql.length){c=b.Ql.map(f=>({edge:Object.assign({},f,{curvePoints:vI(f.points)}),index:95E3}));LM(a.Ea,c,a.ga);a.Ea.C&&
a.scene.add(a.Ea.C);a.Ea.D&&a.scene.add(a.Ea.D);var e;if(null==(e=a.ea["Tensor shape"])?0:e.selected)e=IN(a,c,a.pz),hN(a.Ec,e,!1,!0,!0),a.Ec.C&&a.scene.add(a.Ec.C)}a.mf=b.mf;a.El=b.ms;e=[];b=[];for(const f of Object.keys(Object.assign({},a.Ie,a.mf)))if(c=a.ga.nodesById[f],oI(c)){const g=(d=null!=a.Ie[f])?a.Ie[f].length:a.mf[f].length;e.push({id:`${f}${"||||"}${d?"input":"output"}`,index:e.length,bound:{x:dM(c)+20,y:eM(c)-3.5,width:40,height:14},Sd:95,Hd:!0,borderColor:{r:1,g:1,b:1},bgColor:d?a.kd:
a.ld,borderWidth:0,opacity:1});b.push({id:`${f}${"||||"}${d?"input":"output"}`,nodeId:f,label:`${g} ${d?"input":"output"}${1!==g?"s":""}`,height:8,he:"center",vAlign:"center",weight:1,color:{r:1,g:1,b:1},x:dM(c)+20,y:96,z:eM(c)-3.5+1})}hN(a.cc,b,!1,!0,!0);a.cc.C&&a.scene.add(a.cc.C);SM(a.T,e,!0,!1,!1,!0);a.T.C&&a.scene.add(a.T.C);a.T.D&&a.scene.add(a.T.D);JN(a,f=>{var g=a.wa;g.C&&(g.J.value=f);g=a.Ea;g.C&&(g.J.value=f);a.Pb.D.value=f;a.Ec.D.value=f;cN(a.T,f);a.cc.D.value=f})}else QM(a.wa),QM(a.Ea),
a.Pb.J={},a.Ec.J={},a.T.F={},a.cc.J={}}
function KN(a){if(a.ga){LN(a);var b=a.ga.nodesById[a.selectedNodeId],c=[],d=[];if(oI(b)&&null!=(null==b?void 0:b.identicalGroupIndex)){const f=b.identicalGroupIndex;var e=a.J.filter(({node:g})=>oI(g)&&g.identicalGroupIndex===f).map(({node:g})=>a.ga.nodesById[g.id]);for(const g of e){if(g.id===b.id)continue;e=dM(g)+34;const h=eM(g)-7+3.5;c.push({id:g.id,index:c.length,bound:{x:e,y:h,width:68,height:14},Sd:95,Hd:!0,borderColor:a.Cz,bgColor:a.Bz,borderWidth:1,opacity:1});d.push({id:g.id,label:"Identical layer",
height:8,he:"center",vAlign:"center",weight:1,color:{r:0,g:0,b:0},x:e,y:96,z:h})}}SM(a.ud,c,!1,!1,!0,!0);a.ud.C&&a.scene.add(a.ud.C);hN(a.Td,d,!1,!0,!0);a.Td.C&&a.scene.add(a.Td.C);JN(a,f=>{cN(a.ud,f);a.Td.D.value=f})}}
function MN(a){var b=!1;a.selectedNodeId!==a.Vr&&(a.Vr=a.selectedNodeId,b=!0);var c=!1;a.ua!==a.nt&&(a.nt=a.ua,c=!0);VM(a.D);XM(a.D);ZM(a.D);aN(a.D);aN(a.ja);kN(a.Ed);lN(a.Ed);kN(a.Ta);kN(a.Ob);kN(a.ca);(b||c)&&OM(a.edges);PM(a.edges);b=a.ga.nodesById[a.selectedNodeId];if(null!=b&&oI(b)){const m=b.identicalGroupIndex;null!=m&&(c=a.J.filter(({node:p})=>oI(p)&&p.identicalGroupIndex===m).map(({node:p})=>p.id),WM(a.D,c,a.Az))}c=a.ga.nodesById[a.zd];let d=oI(c)?a.wz:a.xz;var e;nI(c)&&(null==(e=c.style)?
0:e.hoveredBorderColor)&&(d=new THREE.Color(c.style.hoveredBorderColor));UM(a.D,[a.zd],d);a.selectedNodeId&&null!=b&&(UM(a.D,[a.selectedNodeId],a.Jz),YM(a.D,[a.selectedNodeId],2),WM(a.D,[a.selectedNodeId],a.Iz,nI(b)));$M(a.ja,[a.ul],.07);e=Object.keys(a.Ie);if(0<e.length){UM(a.D,e,new THREE.Color(a.kd.r,a.kd.g,a.kd.b));for(var f of e)YM(a.D,[f],1.5)}f=Object.keys(a.mf);if(0<f.length){UM(a.D,f,new THREE.Color(a.ld.r,a.ld.g,a.ld.b));for(var g of f)YM(a.D,[g],1.5)}NM(a.edges,[...a.Bl,...a.El].map(m=>
m.id));g=a.jh()||{};for(var h of Object.keys(g))a.O(h)&&nI(a.ga.nodesById[h])&&((f=g[h].bgColor)&&"transparent"!==f&&WM(a.D,[h],new THREE.Color(f)),(f=g[h].textColor)&&jN(a.Ed,[h],new THREE.Color(f)));null!=a.ua&&(h=Object.keys(a.ga.nodesById).filter(m=>!a.ua.Xs.has(m)&&a.O(m)),$M(a.D,h,.2),iN(a.Ed,h),iN(a.Ob,h),iN(a.ca,h),iN(a.Ta,h),h=a.Lb.filter(({edge:m})=>!a.ua.Xs.has(m.fromNodeId)||!a.ua.Xs.has(m.toNodeId)).map(({edge:m})=>m.id),MM(a.edges,h))}
function NN(a){ta(function*(){ON(a);const b=yield PN(a);HJ(a.C,b,a.ga.id,a.paneId)})}
function QN(a,b,c){ta(function*(){var d=a.ab.R.offsetWidth/RN();let e=a.ab.R.offsetHeight/RN();var f=a.camera;if(b){var g=a.md+20;f=a.sb-20;const h=a.sd+20,m=a.Kb-20;d=g-f;e=h-m;const p=5E3/RN();d>p&&(e=e*p/d,d=p);e>p&&(d=d*p/e,e=p);g=new THREE.OrthographicCamera(f,g,-m,-h,.001,1E3);g.position.y=200;g.lookAt(new THREE.Vector3(0,0,0));g.updateMatrixWorld();g.updateProjectionMatrix();f=g}g=a.KD.R;a.Ja||(a.Ja=new THREE.WebGLRenderer({canvas:g,powerPreference:"high-performance",precision:"highp",antialias:!0,
alpha:!0,preserveDrawingBuffer:!0}),a.Ja.setPixelRatio(RN()));c&&(a.scene.background=null,a.Ja.setClearColor(0,0));a.Ja.setSize(d,e,!1);dN(a.D,a.gb,0);a.Ja.render(a.scene,f);dN(a.D,a.gb,a.Ll/3);d=document.createElement("a");d.download="model_explorer_graph.png";wd(d,td(g.toDataURL()));d.click();a.scene.background=new THREE.Color(16777215)})}
function SN(a){if(a.fb){TN(a);var b=new Set;for(var c of Object.keys(a.fb.results)){var d=a.ga.nodesById[c];if(a.O(c))b.add(c);else{for(;d&&(d=a.ga.nodesById[d.nsParentId||""],d&&!a.O(d.id)););b.add(d.id)}}c=[];for(var e of b){b=a.ga.nodesById[e];d=(b.width||0)+4;var f=(b.height||0)+4;c.push({id:e,index:c.length,bound:{x:dM(b)-2+d/2,y:eM(b)-2+f/2,width:d,height:f},Sd:.001*a.oa[e].index+-3E-4,Hd:!0,borderColor:{r:1,g:1,b:1},bgColor:a.Oy,borderWidth:0,opacity:1})}SM(a.Qa,c);a.Qa.C&&a.scene.add(a.Qa.C);
e=[];b=11/a.Ed.G;for(const g of Object.keys(a.fb.results))if(a.O(g)){d=a.fb.results[g];for(const h of d)if("title"===h.type){d=a.ga.nodesById[g];f=a.oa[g].index;const m=fN(a.Ed,d.label,oI(d)?2:1,11).sizes;e.push({id:g,index:c.length,bound:{x:dM(d)+(d.width||0)/2,y:eM(d)+14-2*b,width:(m.Sh-m.Wh)*b+4,height:(m.fo-m.ho)*b+4},Sd:.001*f+3E-4,Hd:!0,borderColor:{r:1,g:1,b:1},bgColor:a.Oy,borderWidth:0,opacity:1})}}SM(a.Ic,e);a.Ic.C&&a.scene.add(a.Ic.C);JN(a,g=>{cN(a.Qa,g);cN(a.Ic,g)})}}
function UN(a){var b=[a.Qa.C,a.Ic.C];for(var c=a.scene.children.length-1;0<=c;c--){var d=a.scene.children[c];b.includes(d)||(d.geometry&&d.geometry.dispose(),a.scene.remove(d))}a.Vr="";a.nt=void 0;0<a.Lb.length&&(LM(a.edges,a.Lb,a.ga),a.edges.C&&a.scene.add(a.edges.C),a.edges.D&&a.scene.add(a.edges.D));b=[];for(const {node:W,index:S}of a.J){c=a.gb;for(var e of a.Sk)if(NI(W,e.queries,a.ga)){var f=void 0;d=(null==(f=e.styles.node_text_color)?void 0:f.value)||"";""!==d&&(c=new THREE.Color(d));break}b.push({id:`${W.id}_label`,
nodeId:W.id,label:tM(W),height:11,he:"center",vAlign:"center",weight:nI(W)?1:2,x:dM(W)+(W.width||0)/2,y:.001*S+4E-4,z:eM(W)+14,color:c})}hN(a.Ed,b);a.Ed.C&&a.scene.add(a.Ed.C);var g;if(null==(g=a.ea["Tensor shape"])?0:g.selected)g=IN(a,a.Lb,a.gb),hN(a.Ta,g),a.Ta.C&&a.scene.add(a.Ta.C);VN(a);a.sb=Number.POSITIVE_INFINITY;a.Kb=Number.POSITIVE_INFINITY;a.md=Number.NEGATIVE_INFINITY;a.sd=Number.NEGATIVE_INFINITY;g=a.J.length;e=[];f=[];b=[];c=[];d=[];const h=11/a.Ed.G;for(let W=0;W<g;W++){const S=a.J[W].node,
ja=a.J[W].index,F=dM(S),I=eM(S),aa=S.width||0,Aa=S.height||0;var m=oI(S),p=m?WN(a,S):{r:1,g:1,b:1},u=XN(m?a.qz:a.Hz);nI(S)&&S.style&&(S.style.backgroundColor&&(p=new THREE.Color(S.style.backgroundColor)),S.style.borderColor&&(u=new THREE.Color(S.style.borderColor)));m=a.rz;for(var w of a.Sk)if(NI(S,w.queries,a.ga)){var B=void 0,G=(null==(B=w.styles.node_bg_color)?void 0:B.value)||"";""!==G&&(p=new THREE.Color(G));var L=void 0;G=(null==(L=w.styles.node_border_color)?void 0:L.value)||"";""!==G&&(u=
new THREE.Color(G));var O=void 0;G=(null==(O=w.styles.node_text_color)?void 0:O.value)||"";""!==G&&(m=new THREE.Color(G));break}G=!0;oI(S)&&S.sectionContainer&&(G=!1);e.push({id:S.id,index:W,bound:{x:F+aa/2,y:I+Aa/2,width:aa,height:Aa},Sd:.001*ja,Hd:G,borderColor:u,bgColor:p,borderWidth:1.2,opacity:1,qB:(nI(S)||!S.expanded)&&1===p.r&&1===p.g&&1===p.b});nI(S)&&S.subgraphIds&&(c.push({id:`${S.id}`,index:c.length,bound:{x:dM(S)+(S.width||0)+9,y:eM(S)+7,width:14,height:14},Sd:.001*a.oa[S.id].index,Hd:!0,
borderColor:a.bz,bgColor:a.Kz,borderWidth:1,opacity:1}),d.push({id:S.id,nodeId:S.id,label:"0xe5da",height:28,he:"center",vAlign:"center",weight:4,color:a.bz,x:dM(S)+(S.width||0)+10,y:.001*a.oa[S.id].index+4E-4,z:eM(S)+23,bk:!0}));oI(S)&&(G=fN(a.Ed,tM(S),2,11).sizes,u=(G.Sh-G.Wh)*h,p=F+aa/2-u/2,u=F+aa/2+u/2,G=I+14+(G.fo-G.ho)*h+7.5,p=S.expanded?p-13:(F+p+1)/2+1,u=S.expanded?u+12:(F+aa+u-1)/2-1,f.push({id:S.id,nodeId:S.id,label:S.expanded?"0xe5d6":"0xe5d7",height:32,he:"center",vAlign:"center",weight:4,
color:m,x:p,y:.001*ja+4E-4,z:G,bk:!0}),f.push({id:S.id,nodeId:S.id,label:"0xe5d4",height:32,he:"center",vAlign:"center",weight:4,color:m,x:u,y:.001*ja+4E-4,z:G,bk:!0}),m=I+14-1,b.push({id:`${S.id}_${"left"}`,nodeId:S.id,index:b.length,bound:{x:p,y:m,width:16,height:16},Sd:.001*ja+3E-4,Hd:!0,borderColor:{r:1,g:1,b:1},bgColor:{r:0,g:0,b:0},borderWidth:0,opacity:0}),b.push({id:`${S.id}_${"right"}`,nodeId:S.id,index:b.length,bound:{x:u,y:m,width:16,height:16},Sd:.001*ja+3E-4,Hd:!0,borderColor:{r:1,g:1,
b:1},bgColor:{r:0,g:0,b:0},borderWidth:0,opacity:0}));a.sb=Math.min(a.sb,F);a.md=Math.max(a.md,F+aa);a.Kb=Math.min(a.Kb,I);a.sd=Math.max(a.sd,I+Aa)}SM(a.D,e,!0);a.D.C&&a.scene.add(a.D.C);a.D.D&&a.scene.add(a.D.D);hN(a.Ob,f);a.Ob.C&&a.scene.add(a.Ob.C);SM(a.ja,b,!0);a.ja.C&&a.scene.add(a.ja.C);a.ja.D&&a.scene.add(a.ja.D);SM(a.Vd,c,!0);a.Vd.C&&a.scene.add(a.Vd.C);hN(a.Ei,d);a.Ei.C&&a.scene.add(a.Ei.C);w=a.jh()||{};if(0!==Object.keys(w).length){var {mC:ba,BE:ca}=qM(a);w=[];for(const {node:W,index:S}of a.J)if(ba[W.id]){B=
W;L=B.width||0;O=ba[W.id];g=0;for(var na of Object.values(O))g+=na;f=e=0;for(var K of ca)null!=O[K]&&"transparent"!==K&&(b=O[K]/g*L,w.push({id:`${W.id}_${f}`,index:w.length,bound:{x:dM(B)+e+b/2,y:eM(B)+(B.height||0)-5+2.5,width:b,height:5},Sd:.001*S+5E-4,Hd:!1,borderColor:{r:1,g:1,b:1},bgColor:new THREE.Color(K),borderWidth:0,opacity:1}),e+=b,f++)}SM(a.Bi,w);a.Bi.C&&a.scene.add(a.Bi.C)}na=[];for(const W of a.ga.artificialGroupNodeIds||[])a.O(W)&&(K=a.ga.nodesById[W],w=(K.width||0)+2,B=(K.height||
0)+2,na.push({id:W,index:na.length,bound:{x:dM(K)-1+w/2,y:eM(K)-1+B/2,width:w,height:B},Sd:.001*a.oa[W].index+-5E-4,Hd:!1,borderColor:{r:1,g:1,b:1},bgColor:a.jz,borderWidth:0,opacity:1}));SM(a.hh,na,!1,!1,!0);a.hh.C&&a.scene.add(a.hh.C);SN(a);YN(a);ZN(a);JN(a)}
function YN(a){if(a.enableSubgraphSelection){var b=Object.keys(a.ez),c=new Set(b);$N(a);if(0===b.length)a.render();else{var d=[],e=new Set;for(var f of b)for(var g=a.ga.nodesById[f];;)if(g=a.ga.nodesById[g.nsParentId||""])e.add(g.id);else break;f=[];for(var h of e)if(a.O(h)){var m=a.ga.nodesById[h];e=dM(m)+(m.width||0);g=eM(m);m=(m.descendantsOpNodeIds||[]).filter(p=>c.has(a.ga.nodesById[p].id)).length;f.push({id:`${h}_subgraph_count_label`,nodeId:h,label:`${m}`,height:8,he:"center",vAlign:"center",
weight:1,color:a.gb,x:e,y:96,z:g+1});d.push({id:h,index:d.length,bound:{x:e,y:g,width:14*(1E3<=m?2:1.5),height:14},Sd:95.5,Hd:!0,borderColor:a.Lk,bgColor:FN,borderWidth:1.5,opacity:1})}for(const p of b)a.O(p)&&(b=a.ga.nodesById[p],h=dM(b)+(b.width||0),e=eM(b),d.push({id:p,index:d.length,bound:{x:h,y:e,width:14,height:14},Sd:95.5,Hd:!0,borderColor:a.Lk,bgColor:FN,borderWidth:1.5,opacity:1}),f.push({id:`${b.id}_checkmark`,nodeId:b.id,label:"0xe876",height:24,he:"center",vAlign:"center",weight:4,color:a.Lk,
x:h,y:96,z:e+14,bk:!0,hF:.9}));SM(a.Zd,d,!1,!1,!1,!0);a.Zd.C&&a.scene.add(a.Zd.C);hN(a.Wd,f,!1,!0,!0);a.Wd.C&&a.scene.add(a.Wd.C);JN(a,p=>{cN(a.Zd,p);a.Wd.D.value=p})}}}
function aO(a){const b=a.canvas.R;a.scene=new THREE.Scene;a.scene.background=new THREE.Color(16777215);a.camera=new THREE.OrthographicCamera(0,b.clientWidth/b.clientHeight*1E3,0,-1E3,.001,1E3);a.camera.position.y=200;a.camera.lookAt(new THREE.Vector3(0,0,0));a.camera.updateMatrixWorld();a.camera.updateProjectionMatrix();a.aa=new THREE.WebGLRenderer({canvas:b,powerPreference:"high-performance",precision:"highp",antialias:!0,alpha:!0});const c=RN();a.aa.setPixelRatio(c);a.aa.setSize(b.clientWidth,b.clientHeight);
a.Jc=new THREE.WebGLRenderer({canvas:a.ky.R,powerPreference:"high-performance",precision:"highp",antialias:!0,alpha:!0,preserveDrawingBuffer:!0});a.Jc.setPixelRatio(c);a.render();(new ResizeObserver(()=>{requestAnimationFrame(()=>{bO(a)})})).observe(a.ab.R);a.Hc=new THREE.Raycaster;a.Hc.params.Points.threshold=5.5}
function cO(a,b){if(!a.td){var c=a.canvas.R;a.Cl.x=b.offsetX/c.offsetWidth*2-1;a.Cl.y=2*-(b.offsetY/c.offsetHeight)+1;a.Hc.setFromCamera(a.Cl,a.camera);TM(a.D,a.Hc,d=>{a.zd=d;MN(a);a.render()});TM(a.ja,a.Hc,(d,e)=>{a.ul=d;a.lo=(null==e?void 0:e.nodeId)||"";MN(a);dO(a,e);a.render()},!1);TM(a.T,a.Hc,(d,e)=>{a.vl=d;a.Xn=-1E3;a.Wn=-1E3;a.mr="";d=a.vl.endsWith("input");if(""!==a.vl){const {x:f,y:g}=BN(a,e.bound.x-e.bound.width/2,e.bound.y-e.bound.height/2),{x:h,y:m}=BN(a,e.bound.x+e.bound.width/2,e.bound.y+
e.bound.height/2);a.Xn=g;a.Wn=f;a.nr=h-f;a.lr=m-g;a.mr=`Click to reveal ${d?"input":"output"} node(s)`}fn(a.ta)});TM(a.Vd,a.Hc,(d,e)=>{a.Tn=d;a.Oo=-1E3;a.No=-1E3;if(""!==a.Tn){const {x:f,y:g}=BN(a,e.bound.x-e.bound.width/2,e.bound.y-e.bound.height/2),{x:h,y:m}=BN(a,e.bound.x+e.bound.width/2,e.bound.y+e.bound.height/2);a.Oo=g;a.No=f;a.uy=h-f;a.Js=m-g}fn(a.ta)})}}function eO(a){const b=()=>{a.render(!0);requestAnimationFrame(b)};b()}
function fO(a,b,c){const d=new THREE.Vector3;d.set(b/a.canvas.R.offsetWidth*2-1,2*-(c/a.canvas.R.offsetHeight)+1,-1);a=d.unproject(a.camera);return{x:a.x,y:a.z}}
function gO(a,b){a.nk&&(b.metaKey||b.ctrlKey)||!a.nk&&b.ctrlKey?(a.td=!0,a.RD.start(b,(c,d,e,f,g)=>{c=fO(a,d,e);f=fO(a,f,g);g=Math.min(c.x,f.x);d=Math.min(c.y,f.y);a.Qc({x:g,y:d,width:Math.max(c.x,f.x)-g,height:Math.max(c.y,f.y)-d},.9,200,!1,!1);a.td=!1})):b.shiftKey&&a.enableSubgraphSelection&&(a.td=!0,a.UB.start(b,(c,d,e,f,g)=>{a.td=!1;if(c)a.zd?a.ga.nodesById[a.zd]&&a.enableSubgraphSelection&&CM(a.gh,a.zd):a.enableSubgraphSelection&&a.gh.C.set({});else{d=fO(a,d,e);e=fO(a,f,g);f=Math.min(d.x,e.x);
g=Math.max(d.x,e.x);c=Math.min(d.y,e.y);e=Math.max(d.y,e.y);d=[];for(var {node:h}of a.J){const p=dM(h);var m=eM(h);const u=f>p+(h.width||0),w=c>m+(h.height||0);m=e<m;g<p||u||w||m||d.push(h.id)}h=a.gh;if(0!==d.length&&h.modelGraph){f=Object.assign({},h.C());for(const p of d)if(g=h.modelGraph.nodesById[p],nI(g))f[p]?delete f[p]:f[p]=!0;else if(oI(g)&&!g.expanded)for(const u of g.descendantsOpNodeIds||[])f[u]?delete f[u]:f[u]=!0;h.C.set(f)}}}))}
function dO(a,b){a.Qb.top=-1E3;a.Qb.left=-1E3;a.Qb.width=0;a.Qb.height=0;a.Qb.Rd=void 0;if(""!==a.ul&&null!=b){const {x:c,y:d}=BN(a,b.bound.x-b.bound.width/2,b.bound.y-b.bound.height/2),{x:e,y:f}=BN(a,b.bound.x+b.bound.width/2,b.bound.y+b.bound.height/2);a.Qb.top=d;a.Qb.left=c;a.Qb.width=e-c;a.Qb.height=f-d;b.id.includes("_left")?a.Qb.Rd=a.ga.nodesById[b.nodeId].expanded?"Collapse layer":"Expand layer":b.id.includes("_right")&&(a.Qb.Rd="More actions")}fn(a.ta)}
function BN(a,b,c){var d=a.ab.R;const e=d.clientWidth;d=d.clientHeight;b=new THREE.Vector3(b,0,c);b.project(a.camera);return{x:b.x*e/2+e/2,y:-(b.y*d/2)+d/2}}function DN(a,b){ta(function*(){KJ(a.C,a.paneId,a.ga.id,b,yield PN(a));const c=JJ(a.C,b);c&&(wJ(a.C,a.paneId),sJ(a.C,!1),a.C.N.set(void 0),a.C.F.set(void 0),qJ(a.C,c))})}function hO(a,b){a=a.ga.nodesById[null!=b?b:a.zd];return null!=a&&oI(a)?a.expanded:!1}
function iO(a){var b=a.ab.R;const c=b.clientWidth;b=b.clientHeight;const d=c/b,e=a.kh-c/2,f=a.lh-b/2;a.camera.left=-500/a.N*d-e/c*1E3/a.N*d;a.camera.right=500/a.N*d-e/c*1E3/a.N*d;a.camera.top=500/a.N+500*f/a.N/b*2;a.camera.bottom=-500/a.N+500*f/a.N/b*2;a.camera.updateProjectionMatrix()}function ZN(a){const b=Math.max(0,Math.min(1,(7.5-(a.ab?30*a.ab.R.clientHeight/500/2:0)*a.N)/.5));1E-5>Math.abs(b-a.Ll)||(a.Ll=b,dN(a.D,a.gb,b/3))}
function jO(a,b){if(!a.C.C){var c=Date.now(),d=()=>{var e=Date.now()-c;e=Math.min(1,e/1100);e=-(Math.cos(Math.PI*e)-1)/2;const f=8*Math.sin(e*Math.PI*9);bN(a.D,b,f);bN(a.Qa,b,f);a.render();1<=e?(bN(a.D,b,0),bN(a.Qa,b,0),a.render()):requestAnimationFrame(d)};d()}}function kO(a,b,c=1){return a.ab?1E3*b/c/a.ab.R.clientHeight:0}
function lO(a,b,c,d,e=300){if(a.ab){var f=a.ab.R;b=f.clientWidth/f.clientHeight*-1E3/2/d+b;c=-c+500/d;d=TL.scale(d);var g=d.translate;if(a.ab){var h=a.ab.R,m=h.clientWidth;b=b/(m/h.clientHeight*500)/-2*m}else b=0;b=g.call(d,b,a.ab?c*a.ab.R.clientHeight/500/2:0);f=RL(f);0===e?f.call(a.zoom.transform,b):f.transition().duration(a.C.C?0:e).Of(PL).call(a.zoom.transform,b)}}function RN(){return 1===window.devicePixelRatio?1.5:window.devicePixelRatio}
function bO(a){var b=a.ab.R;const c=a.aa.domElement,d=b.clientWidth;b=b.clientHeight;if(0!==d&&0!==b&&(c.width!==d||c.height!==b)){if(null==a.Ci&&null==a.Di){const {x:e,y:f}=fO(a,d/2,b/2);a.Ci=e;a.Di=f}a.aa.setSize(d,b,!1);a.canvas.R.style.width="100%";a.canvas.R.style.height="100%";iO(a);a.render();a.Qc({x:a.Ci,y:a.Di,width:1E-7,height:1E-7},.9,0,!0);0<=a.Il&&window.clearTimeout(a.Il);a.Il=window.setTimeout(()=>{a.Ci=null;a.Di=null},500)}}
function VN(a){if(0!==Object.keys(a.F).filter(h=>a.F[h].selected).length){var b=[],c=9/a.ca.G,d=[];for(const {node:h,index:m}of a.J){const p=[];var e=38,f=0;let u=0;const w=[];if(nI(h)){var g=pI(a.F);for(const O of g)w.push({key:O,value:rI(h,O)});let B;if(null==(B=a.F["Op node attributes"])?0:B.selected){let O;w.push(...BI(h,(null==(O=a.F["Op node attributes"])?void 0:O.filterRegex)||""))}let G;(null==(G=a.F["Op node inputs"])?0:G.selected)&&w.push(...CI(h,a.ga));let L;(null==(L=a.F["Op node outputs"])?
0:L.selected)&&w.push(...DI(h));w.push(...EI(h,a.ga.id,a.F,a.lb))}else if(oI(h)){g=qI(a.F);for(const B of g)w.push({key:B,value:rI(h,B)})}for(const {key:B,value:G}of w){const {wj:L,UC:O,om:ba,dF:ca}=mO(a,h,m,B,G,e,c);b.push(L,ba);f=Math.max(O,f);u=Math.max(ca,u);p.push({wj:L,om:ba});e+=12}e=((h.width||0)-(f+u+4))/2;for(const B of p)B.wj.x=dM(h)+f,B.om.x=dM(h)+f+4,B.wj.x+=e,B.om.x+=e;0<p.length&&nI(h)&&(f=(h.width||0)-16,e=12*p.length,d.push({id:h.id,index:d.length,bound:{x:dM(h)+8+f/2,y:p[0].wj.z+
e/2-6,width:f,height:e},Sd:.001*m+2E-4,Hd:!0,borderColor:{r:1,g:1,b:1},bgColor:{r:1,g:1,b:1},borderWidth:1,opacity:1}))}0<b.length&&(hN(a.ca,b),a.ca.C&&a.scene.add(a.ca.C),SM(a.ih,d),a.ih.C&&a.scene.add(a.ih.C))}}
function JN(a,b=c=>{cN(a.D,c);a.Ob.D.value=c;cN(a.ja,c);cN(a.Vd,c);a.Ei.D.value=c;a.Ed.D.value=c;a.Ta.D.value=c;a.ca.D.value=c;cN(a.ih,c);cN(a.Bi,c);cN(a.hh,c);var d=a.edges;d.C&&(d.J.value=c)}){const c=Date.now(),d=()=>{var e=Date.now()-c;e=a.C.C?1:Math.min(1,e/200);e=Math.sin(e*Math.PI/2);b(e);a.render();1<=e?(b(e),a.render()):requestAnimationFrame(d)};d()}function WN(a,b){b=(b.namespace||"").split("/").filter(c=>""!==c).length;return XN(a.Cx[Math.min(a.Cx.length-1,b)])}
function XN(a){return{r:a.r,g:a.g,b:a.b}}
function IN(a,b,c){var d;const e=(null==(d=a.C.config())?void 0:d.edgeLabelFontSize)||7.5;var f;d=(null==(f=a.C.config())?void 0:f.disallowVerticalEdgeLabels)||!1;const g=[],h=IM(a.P,1);for(const {edge:na}of b){var m=a.ga.nodesById[na.fromNodeId];b=a.ga.nodesById[na.toNodeId];if(!nI(m)||!nI(b))continue;let K="?";b=m.outputsMetadata||{};for(const S of Object.keys(b))if(null!=(m.outgoingEdges||[]).find(ja=>ja.sourceNodeOutputId===S&&ja.targetNodeId===na.toNodeId)){K=b[S].shape||"?";K=K.split("").map(ja=>
{"x"===ja&&(ja="x");"\u2217"===ja&&(ja="*");""===ja&&(ja="");return null==h[ja]?"?":ja}).join("");break}b=na.curvePoints||[];f=m.globalX||0;m=m.globalY||0;const W=new THREE.CurvePath;for(var p=0;p<b.length-1;p++){var u=b[p],w=b[p+1];u=new THREE.LineCurve(new THREE.Vector2(u.x+f,u.y+m),new THREE.Vector2(w.x+f,w.y+m));W.add(u)}b=W.getLength();f=e/2/b;p=()=>{const S=W.getPointAt(.5);g.push({id:`${na.id}_${K}`,nodeId:na.toNodeId,label:K,height:e,he:"center",vAlign:"center",weight:1,x:S.x,y:95,z:S.y,color:c,
borderColor:{r:1,g:1,b:1}})};if(1<f*(K.length+3))p();else{m=[];u=Math.max(0,Math.min(.25,1-K.length*f-5/b));w=Math.max(.05,1-5/b-u-f*K.length);var B=!0;f=e/a.Ta.G;for(var G=0;G<w;G+=.05){var L=u+G;B=!0;var O=void 0;m=[];for(let S=0;S<K.length;S++){var ba=K[S];const ja=W.getPointAt(Math.min(L,1)),F=W.getTangentAt(Math.min(L,1));var ca=(2*Math.PI-Math.atan(F.y/F.x))%(2*Math.PI);0>ca&&(ca+=2*Math.PI);if(d&&ca>=Math.PI/4&&ca<=1.75*Math.PI){B=!1;break}m.push({zo:ja,position:Math.min(L,1),angle:ca,tan:F,
char:ba});if(null!=O&&(O=Math.abs(ca-O),.15<Math.min(O,Math.abs(O-Math.PI))&&(B=!1,G+.05<w)))break;O=ca;ba=h[ba];ca=0;S!==K.length-1&&(ca=h[K[S+1]].xadvance);L+=(ba.xadvance/2+ca/2)*f/b}if(B)break}if(B){if(p=m[0].zo.x>=m[m.length-1].zo.x){u=[];m=m[0].position;for(w=K.length-1;0<=w;w--)B=K[w],G=W.getPointAt(Math.min(1,m)),O=W.getTangentAt(Math.min(1,m)),L=(2*Math.PI-Math.atan(O.y/O.x))%(2*Math.PI),0>L&&(L+=2*Math.PI),u.push({zo:G,angle:L,tan:O,position:m,char:B}),B=h[B],G=0,1<=w&&(G=h[K[w-1]].xadvance),
m+=(B.xadvance/2+G/2)*f/b;m=u}for(b=0;b<m.length;b++)f=m[b],u=f.char,w=f.zo,B=f.angle,1E-7>Math.abs(f.tan.x)&&(B=(p?1:-1)*Math.PI/2),g.push({id:`${na.id}_${u}_${b}`,nodeId:na.toNodeId,label:u,height:e,he:"",vAlign:"",weight:1,x:w.x+1.5*Math.sin(B)*-e,y:95,z:w.y+1.5*Math.cos(B)*-e,color:c,angle:B,Iq:!0,borderColor:{r:1,g:1,b:1}})}else p()}}return g}
function mO(a,b,c,d,e,f,g){const h={id:`${b.id}_attrs_table_${d}_key`,nodeId:b.id,label:`${d}:`,height:9,he:"right",vAlign:"center",weight:1,x:dM(b),y:.001*c+4E-4,z:eM(b)+f,color:a.kz};var m=fN(a.ca,h.label,h.weight,h.height).sizes;m=(m.Sh-m.Wh)*g;b={id:`${b.id}_attrs_table_${d}_value`,nodeId:b.id,label:e,height:9,he:"left",vAlign:"center",weight:0,x:dM(b),y:.001*c+4E-4,z:eM(b)+f,color:a.lz,maxWidth:200};const {sizes:p,ZE:u}=fN(a.ca,b.label,b.weight,b.height,b.maxWidth);null!=u&&(b.label=u);return{wj:h,
UC:m,om:b,dF:(p.Sh-p.Wh)*g}}function HN(a){a.wa.clear();a.Ea.clear();a.Ie={};a.mf={};a.Bl=[];a.El=[];for(const b of[a.T.C,a.T.D,a.cc.C,a.Pb.C,a.Ec.C])b&&(b.geometry&&b.geometry.dispose(),a.scene.remove(b));a.T.D=void 0}function LN(a){for(const b of[a.ud.C,a.Td.C])b&&(b.geometry&&b.geometry.dispose(),a.scene.remove(b))}function ON(a){a.Oq=!0;fn(a.ta);setTimeout(()=>{a.Oq=!1;fn(a.ta)},300)}
function PN(a){return ta(function*(){var b=a.ab.R,c=fO(a,0,0);b=fO(a,b.clientWidth,b.clientHeight);c={x:c.x,y:c.y,width:b.x-c.x,height:b.y-c.y};b=[];uI(void 0,a.ga,b);var d=a.canvas.R,e=a.ky.R;d=320/d.width*d.height;a.Jc.setSize(320,d,!1);a.Jc.render(a.scene,a.camera);var f=window.devicePixelRatio;d=new OffscreenCanvas(320*f,d*f);f=d.getContext("2d");f.imageSmoothingQuality="high";f.drawImage(e,0,0,e.width,e.height,0,0,d.width,d.height);e=yield createImageBitmap(d);d=uJ(a.C,a.paneId);return{id:tI(),
rect:c,er:e,selectedNodeId:a.selectedNodeId,deepestExpandedGroupNodeIds:b,showOnNodeItemTypes:Object.assign({},a.F),Yb:Object.assign({},a.ea),flattenLayers:d}})}function TN(a){for(const b of[a.Qa.C,a.Ic.C])b&&(b.geometry&&b.geometry.dispose(),a.scene.remove(b))}function $N(a){for(const b of[a.Zd.C,a.Wd.C])b&&(b.geometry&&b.geometry.dispose(),a.scene.remove(b))}
var nO=class extends uM{constructor(){super();this.Ac=!1;this.Jz=new THREE.Color("#1A73E8");this.Iz=new THREE.Color("#C2E7FF");this.xz=new THREE.Color("#000");this.wz=new THREE.Color("#666");this.Az=new THREE.Color("#e2edff");this.Bz=new THREE.Color("#e3e3e3");this.Cz=new THREE.Color("#ccc");this.gb=new THREE.Color("#041E49");this.Hz=new THREE.Color("#777");this.qz=new THREE.Color("#aaa");new THREE.Color("#DADCE0");this.rz=new THREE.Color("#444746");let a;this.mz=new THREE.Color((null==(a=this.C.config())?
void 0:a.edgeColor)||"#aaa");this.kd=new THREE.Color("#009e73");this.oz=new THREE.Color("#125341");this.ld=new THREE.Color("#d55e00");this.pz=new THREE.Color("#994d11");this.kz=new THREE.Color("#808080");this.lz=new THREE.Color("#0d0d0d");this.jz=new THREE.Color("#800080");this.Oy=new THREE.Color("#f5d55a");this.bz=new THREE.Color("#135cbb");this.Kz=new THREE.Color("#d5e7ff");this.Cx=(()=>{const b=[];for(let c=0;6>c;c++)b.push(new THREE.Color(`hsl(212, 40%, ${Math.round(96+-2.4*c)}%)`));return b})();
new THREE.Color("#C2E7FF");new THREE.Color("#7facc9");this.Lk=new THREE.Color("#09B83E");this.Qb={top:-1E3,left:-1E3,width:0,height:0};this.wl="";this.Wn=this.Xn=-1E3;this.lr=this.nr=0;this.mr="";this.No=this.Oo=-1E3;this.Js=this.uy=0;this.Kv=[];this.Oq=!1;this.mh=-1;this.hg=0;this.hw="";this.Fz=of(Tk);this.P=of(KM);this.Gz=of(sL);this.gh=of(EM);this.Hy=of(BM);this.D=new eN(6);this.Ob=new mN(this.P);this.ja=new eN(99);this.Bi=new eN(0);this.hh=new eN(6);this.Qa=new eN(8);this.Ic=new eN(4);this.ud=
new eN(99);this.Td=new mN(this.P);this.T=new eN(99);this.cc=new mN(this.P);this.Vd=new eN(3);this.Ei=new mN(this.P);this.edges=new RM(this.mz,1);this.wa=new RM(this.kd,1.5);this.Ea=new RM(this.ld,1.5);this.Pb=new mN(this.P);this.Ec=new mN(this.P);this.Ed=new mN(this.P);this.Ta=new mN(this.P);this.ca=new mN(this.P);this.ih=new eN(4);this.Zd=new eN(99);this.Wd=new mN(this.P);this.Cl=new THREE.Vector2;this.td=!1;this.sd=this.Kb=this.md=this.sb=0;this.Ie={};this.mf={};this.Bl=[];this.El=[];this.Tn=this.vl=
this.lo=this.ul=this.zd="";this.Di=this.Ci=null;this.Ll=this.Il=-1;this.fb=void 0;this.Nk=[];this.Sk=[];this.ez={};this.cz={};this.dz={};Ks(()=>{const b=this.Sg();(null==b?void 0:b.rendererId)===this.rendererId&&(this.selectedNodeId=(null==b?void 0:b.nodeId)||"",this.Yg&&(this.selectedNodeId&&nI(this.ga.nodesById[this.selectedNodeId])?pM(this):this.ua=void 0),GN(this),KN(this),MN(this),this.render())});C(this.C.oa,tL(this.G)).subscribe(b=>{b.rendererId===this.rendererId&&NN(this)});C(this.C.ca,tL(this.G)).subscribe(b=>
{if(b.rendererId===this.rendererId){b=b.kb;b.showOnNodeItemTypes&&(this.F=Object.assign({},b.showOnNodeItemTypes),RJ(this.C,this.paneId,this.rendererId,this.F));b.Yb&&(this.ea=Object.assign({},b.Yb),SJ(this.C,this.paneId,this.rendererId,this.ea));const c=uJ(this.C,this.paneId),d=!0===b.flattenLayers;c!==d?(mJ(this.C,this.paneId,d,b),sJ(this.C,d)):XL(this,b.selectedNodeId||"",b.deepestExpandedGroupNodeIds||[],!1,b.rect,!0,b.showOnNodeItemTypes)}});C(this.C.ta,tL(this.G)).subscribe(b=>{b.rendererId===
this.rendererId&&QN(this,b.jC,b.VE)});Ks(()=>{const b=nJ(this.C,this.paneId);b&&b.modelGraph&&this.fb!==b.Rg&&(this.fb=b.Rg,SN(this),this.render())});Ks(()=>{var b=this.Gz.rules().filter(c=>LI(c.queries)&&0<Object.keys(c.styles).length);b=JSON.stringify(b);JSON.stringify(this.Nk)!==b&&(this.Nk=JSON.parse(b),this.Sk=MI(this.Nk),UN(this),GN(this),KN(this),MN(this),this.render())});Ks(()=>{if(nJ(this.C,this.paneId)){var b=UJ(this.C,this.paneId,this.rendererId);JSON.stringify(b)!==JSON.stringify(this.ea)&&
(this.ea=b,UN(this),GN(this),KN(this),MN(this),this.render())}});Ks(()=>{this.ez=this.gh.C();YN(this)});Ks(()=>{this.cz=this.Hy.C();this.dz=this.Hy.D();GN(this);MN(this);this.render()})}Fb(){this.Cg=this.modelGraph.id;this.ga=this.modelGraph;this.C.P[this.paneId]=this.ga;this.Rb||(this.F=eJ(this.C),this.ea=fJ(this.C));ZL(this,this.ab.R);aO(this);Qk(this.Fz,()=>{this.canvas.R.addEventListener("mousemove",d=>{cO(this,d)})});var a=this.C.N(),b=d=>{cM(this);UN(this);this.ae(.9,0);const e=nJ(this.C,this.paneId);
if(null!=(null==e?void 0:e.Ko)){const f=e.Ko;this.F=e.Ko.showOnNodeItemTypes||{};RJ(this.C,this.paneId,this.rendererId,this.F);XL(this,f.selectedNodeId||"",f.deepestExpandedGroupNodeIds||[],!1,f.rect,!0,f.showOnNodeItemTypes,!0);e.Ko=void 0}else null!=d&&""!==d&&setTimeout(()=>{this.C.F.set({nodeId:d,rendererId:this.rendererId,Cd:!1,noNodeShake:!0})});$L(this,this.am||"")};if(!a||0===a.paneStates.length||this.Rb){var c;let d;a=this.Rb?void 0:null==(c=nJ(this.C,this.paneId))?void 0:null==(d=c.Sg)?
void 0:d.nodeId;b(a)}else c=iJ(this.C,this.paneId),(a=a.paneStates[c])?(XL(this,a.selectedNodeId,a.deepestExpandedGroupNodeIds,!0),SI(this.Re,a.deepestExpandedGroupNodeIds,c)):b();b=window;null==b.me_test&&(b.me_test={SD:{}});a=this.Rb?-1:iJ(this.C,this.paneId);b.me_test.SD[a]=this;this.Ac&&eO(this)}va(){super.va();this.aa&&(this.aa.dispose(),this.aa.forceContextLoss());this.Jc&&(this.Jc.dispose(),this.Jc.forceContextLoss());this.Ja&&(this.Ja.dispose(),this.Ja.forceContextLoss());document.body.style.cursor=
"default"}no(){super.no();GN(this);MN(this);this.render()}get qE(){return!this.Rb}fl(a){super.fl(a);this.zd?this.ga.nodesById[this.zd]&&(a||aM(this,this.zd)):aM(this,"")}il(a,b){super.il(a,b);""===this.selectedNodeId||b||gM(this,this.ga.nodesById[this.selectedNodeId],a)}xl(){super.xl();requestAnimationFrame(()=>{this.camera&&(iO(this),ZN(this),this.render(),dO(this))})}ht(a,b){ML.Mo&&"mouseup"===ML.Mo.type&&(a=Math.abs(this.kh-a),b=Math.abs(this.lh-b),0<=a&&3>=a&&0<=b&&3>=b&&this.fl(ML.Mo.shiftKey))}ll(a,
b,c,d,e){super.ll(a,b,c,d,e);UN(this);null!=c?fM(this,c,a,400):this.ae();GN(this);KN(this);MN(this);this.render();this.Rb||SI(this.Re,e,iJ(this.C,this.paneId))}rl(a,b,c,d,e,f,g){super.rl(a,b,c,d,e,f,g);UN(this);GN(this);KN(this);MN(this);if(e){const h=()=>{this.Qc(e,1,200,!1,!1)};f?setTimeout(()=>{h()}):h()}else"______"!==c&&fM(this,c,b,d?0:400);this.selectedNodeId!==c&&aM(this,c||"");this.Rb||SI(this.Re,g||[],iJ(this.C,this.paneId))}gt(){UN(this);MN(this);this.render()}Ai(a,b,c,d,e,f,g=!1){super.Ai(a,
b,c,d,e,f);fM(this,c,b,400);g||(UN(this),GN(this),KN(this),MN(this),this.render(),this.Rb||SI(this.Re,d,iJ(this.C,this.paneId)));e||setTimeout(()=>{jO(this,c)},250)}Qc(a,b=.9,c=300,d=!1,e=!0,f=!1){if(this.ab){var g=this.ab.R,h=g.clientWidth*b;b*=g.clientHeight;g=a.width/a.height;if(d)h=this.N;else{if(g>h/b){if(this.ab){g=this.ab.R;var m=g.clientWidth;h=h/m*-2*(m/g.clientHeight)*500}else h=0;h/=a.width}else h=kO(this,b)/a.height;h=Math.abs(h)}g=a.x+a.width/2;m=a.y+a.height/2;!d&&e&&(h=Math.min(kO(this,
45)/30,h),f&&(d=kO(this,20)/30,h<d&&(m=a.y+kO(this,b/2-60,d),h=Math.max(d,h))));lO(this,g,m,h,c)}}ae(a=.9,b=200){this.Qc({x:this.sb,y:this.Kb,width:this.md-this.sb,height:this.sd-this.Kb},a,b)}O(a){return null!=this.oa[a]}render(a=!1){this.aa&&this.scene&&this.camera&&(this.aa.render(this.scene,this.camera),this.Ac&&a&&(0>this.mh&&(this.mh=performance.now()),this.hg+=1,a=performance.now()-this.mh,1E3<a&&(this.hw=(this.hg/a*1E3).toFixed(1),this.mh=-1,this.hg=0,en(this.ta))))}get enableSubgraphSelection(){let a;
return!0===(null==(a=this.C.config())?void 0:a.enableSubgraphSelection)}};nO.K=function(a){return new (a||nO)};
nO.Ca=Jf({type:nO,ha:[["webgl-renderer"]],eb:function(a,b){a&1&&(vr(nN,7),vr(oN,7),vr(pN,7),vr(qN,7),vr(rN,7),vr(sN,7),vr(tN,7),vr(uN,5),vr(vN,5),vr(wN,7),vr(xN,7),vr(yN,7));if(a&2){let c;wr(c=xr())&&(b.ab=c.first);wr(c=xr())&&(b.canvas=c.first);wr(c=xr())&&(b.ky=c.first);wr(c=xr())&&(b.KD=c.first);wr(c=xr())&&(b.fG=c.first);wr(c=xr())&&(b.Bw=c.first);wr(c=xr())&&(b.GE=c.first);wr(c=xr())&&(b.lw=c.first);wr(c=xr())&&(b.Cw=c.first);wr(c=xr())&&(b.gD=c.first);wr(c=xr())&&(b.RD=c.first);wr(c=xr())&&
(b.UB=c.first)}},inputs:{Ac:"benchmark"},ka:!0,features:[Ep,Nr],ya:39,za:59,Ba:[["container",""],["canvas",""],["snapshotCanvas",""],["pngDownloaderCanvas",""],["groupNodeIconMatTooltip","matTooltip"],["moreActionsMenuTrigger","matMenuTrigger"],["moreActionsMenu","matMenu"],["ioPicker",""],["ioPickerMatTooltip","matTooltip"],["subgraphIndicator",""],["subgraphMenuTrigger","matMenuTrigger"],["menu","matMenu"],["rangeZoomDragArea",""],["dragToSelectDragArea",""],[1,"container",3,"mouseleave"],[1,"main",
3,"mousedown","contextmenu"],[1,"snapshot"],[1,"download-png"],[1,"group-node-icon",3,"mouseenter","mouseleave","mousedown"],["matTooltipPosition","above",1,"group-node-icon-tooltip",3,"matTooltip","matTooltipShowDelay","matTooltipDisabled"],[1,"more-actions-menu-trigger",3,"matMenuTriggerFor"],[1,"model-explorer-menu-item-with-icon","expand-all",3,"click"],["class","model-explorer-menu-item-with-icon collapse-all",3,"click",4,"ngIf"],["class","model-explorer-menu-item-with-icon open-in-popup",3,
"click",4,"ngIf"],[1,"io-picker",3,"mousedown","mouseenter","mouseleave"],["matTooltipPosition","above",1,"io-picker-tooltip",3,"matTooltip"],["matTooltip","Jump to subgraph","matTooltipPosition","above",1,"subgraph-indicator",3,"mousedown","click"],[1,"subgraph-indicator-menu-trigger",3,"matMenuTriggerFor"],[1,"model-explorer-menu-item-with-icon"],["bgColor","rgba(97, 175, 252, 0.2)","borderColor","#1976d2"],["bgColor","rgba(9, 184, 62, 0.2)","borderColor","#09b83e"],[1,"flash"],[1,"fps"],[1,"model-explorer-menu-item-with-icon",
"collapse-all",3,"click"],[1,"model-explorer-menu-item-with-icon","open-in-popup",3,"click"],[1,"model-explorer-menu-item-with-icon",3,"click"]],sa:function(a,b){if(a&1){const c=Zg();R(0,"div",14,0);V("mouseleave",function(d){t(c);d=d.relatedTarget;null!=d&&d.classList.contains("cdk-overlay-backdrop")||(b.zd="",MN(b),dO(b),b.render());return x()});R(2,"canvas",15,1);V("mousedown",function(d){t(c);return x(gO(b,d))})("contextmenu",function(d){t(c);return x(d.preventDefault())});U();Oq(4,"canvas",16,
2)(6,"canvas",17,3);R(8,"div",18);V("mouseenter",function(){t(c);JE(b.lw);return x()})("mouseleave",function(){t(c);HE(b.lw);return x()})("mousedown",function(d){t(c);d.stopPropagation();b.ul.includes("_left")?(aM(b,b.lo),gM(b,b.ga.nodesById[b.lo])):b.ul.includes("_right")&&(b.wl=b.lo,WC(b.gD));return x()});U();Oq(9,"div",19,4)(11,"div",20,5);R(13,"mat-menu",null,6)(15,"div",21);V("click",function(){t(c);var d=b.wl;if(d=null!=d?d:b.zd)aM(b,d),gM(b,b.ga.nodesById[d],!0,!0);return x()});R(16,"mat-icon");
Z(17,"unfold_more_double");U();Z(18," Expand layer & all sub-layers ");U();H(19,zN,4,0,"div",22)(20,AN,4,0,"div",23);U();R(21,"div",24,7);V("mousedown",function(d){t(c);d.stopPropagation();d=b.vl.split("||||")[0];b.vl.endsWith("input")?1===b.Ie[d].length?VL(b,b.Ie[d][0].id,b.rendererId):sM(b,b.Bw.R,b.Ie[d],"incoming"):1===b.mf[d].length?VL(b,b.mf[d][0].id,b.rendererId):sM(b,b.Bw.R,b.mf[d],"outgoing");return x()})("mouseenter",function(){t(c);JE(b.Cw);return x()})("mouseleave",function(){t(c);HE(b.Cw);
return x()});U();Oq(23,"div",25,8);R(25,"div",26,9);V("mousedown",function(d){t(c);return x(d.stopPropagation())})("click",function(){t(c);if(b.Tn){var d=b.ga.nodesById[b.Tn];nI(d)&&(d=d.subgraphIds,1===d.length?DN(b,d[0]):1<d.length&&(b.Kv=d,WC(b.GE)))}return x()});U();Oq(27,"div",27,10);R(29,"mat-menu",null,11);Lq(31,CN,4,1,"div",28,Jq);U();Oq(33,"drag-area",29,12)(35,"drag-area",30,13)(37,"div",31);H(38,EN,2,1,"div",32);U()}if(a&2){a=Ar(14);const c=Ar(30);D(8);Up("top",b.Qb.top,"px")("left",b.Qb.left,
"px")("width",b.Qb.width,"px")("height",b.Qb.height,"px");D();Up("top",b.Qb.top,"px")("left",b.Qb.left,"px")("width",b.Qb.width,"px")("height",b.Qb.height,"px");Q("matTooltip",b.Qb.Rd||"")("matTooltipShowDelay",200)("matTooltipDisabled",null==b.Qb.Rd);D(2);Up("top",b.Qb.top+b.Qb.height,"px")("left",b.Qb.left,"px");Q("matMenuTriggerFor",a);D(8);Q("ngIf",hO(b,b.wl));D();Q("ngIf",b.qE);D();Up("top",b.Xn,"px")("left",b.Wn,"px")("width",b.nr,"px")("height",b.lr,"px");D(2);Up("top",b.Xn,"px")("left",b.Wn,
"px")("width",b.nr,"px")("height",b.lr,"px");Q("matTooltip",b.mr);D(2);Up("top",b.Oo,"px")("left",b.No,"px")("width",b.uy,"px")("height",b.Js,"px");D(2);Up("top",b.Oo+b.Js,"px")("left",b.No,"px");Q("matMenuTriggerFor",c);D(4);Nq(b.Kv);D(6);Wp("flashing",b.Oq);D();Gq(b.Ac?38:-1)}},Ha:[it,dt,YJ,$C,CC,QC,YC,iI,hI,OE,NE],styles:[".container[_ngcontent-%COMP%]{width:100%;height:100%;overflow:hidden;position:relative}.container[_ngcontent-%COMP%]   canvas.main[_ngcontent-%COMP%]{width:100%!important;height:100%!important;-moz-box-sizing:border-box;box-sizing:border-box}.container[_ngcontent-%COMP%]   canvas.download-png[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   canvas.snapshot[_ngcontent-%COMP%]{position:absolute;left:0;bottom:0;width:400px;height:300px;border:1px solid red;-moz-box-sizing:border-box;box-sizing:border-box;background-color:#fff;display:none}.container[_ngcontent-%COMP%]   .hover-toolbar[_ngcontent-%COMP%]{position:absolute;width:50px;height:40px;-moz-box-sizing:border-box;box-sizing:border-box;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-transition:opacity .15s;transition:opacity .15s;opacity:1;pointer-events:none}.container[_ngcontent-%COMP%]   .hover-toolbar.hide[_ngcontent-%COMP%]{-webkit-transition:none;transition:none;opacity:0}.container[_ngcontent-%COMP%]   .hover-toolbar[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]{width:100%;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center}.container[_ngcontent-%COMP%]   .hover-toolbar[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .main-content-wrapper[_ngcontent-%COMP%]{height:100%;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;pointer-events:all}.container[_ngcontent-%COMP%]   .hover-toolbar[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .main-content[_ngcontent-%COMP%]{font-size:12px;border:1px solid #ccc;background-color:hsla(0,0%,98%,.9);border-radius:99px;padding:0 8px;-moz-box-sizing:border-box;box-sizing:border-box;box-shadow:0 1px 3px 0 rgba(0,0,0,.1),0 1px 2px -1px rgba(0,0,0,.1);display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .hover-toolbar[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .main-content[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;cursor:pointer;opacity:.8;width:24px;padding:4px 0}.container[_ngcontent-%COMP%]   .hover-toolbar[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .main-content[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]:hover{opacity:1;background-color:#eee}.container[_ngcontent-%COMP%]   .hover-toolbar[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .main-content[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:16px;font-weight:700;width:16px;height:16px;line-height:16px;color:#ea8600}.container[_ngcontent-%COMP%]   .group-node-icon[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   .io-picker[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   .subgraph-indicator[_ngcontent-%COMP%]{position:absolute;-moz-box-sizing:border-box;box-sizing:border-box;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;background-color:transparent}.container[_ngcontent-%COMP%]   .group-node-icon[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   .io-picker[_ngcontent-%COMP%]{z-index:100}.container[_ngcontent-%COMP%]   .group-node-icon-tooltip[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   .io-picker-tooltip[_ngcontent-%COMP%]{position:absolute;z-index:50}.container[_ngcontent-%COMP%]   .more-actions-menu-trigger[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   .subgraph-indicator-menu-trigger[_ngcontent-%COMP%]{position:absolute;visibility:hidden}.container[_ngcontent-%COMP%]   .flash[_ngcontent-%COMP%]{position:absolute;top:0;left:0;width:100%;height:100%;z-index:5000;-moz-box-sizing:border-box;box-sizing:border-box;border:5px solid #1976d2;pointer-events:none;opacity:0}@-webkit-keyframes _ngcontent-%COMP%_flashing{5%{opacity:1}to{opacity:0}}@keyframes _ngcontent-%COMP%_flashing{5%{opacity:1}to{opacity:0}}.container[_ngcontent-%COMP%]   .flashing[_ngcontent-%COMP%]{-webkit-animation:_ngcontent-%COMP%_flashing .3s ease-out;animation:_ngcontent-%COMP%_flashing .3s ease-out}.container[_ngcontent-%COMP%]   .fps[_ngcontent-%COMP%]{position:absolute;top:12px;left:12px;z-index:100;font-size:32px;border:1px solid #777;padding:8px;border-radius:4px;background-color:#fff}  .multiline-tooltip{white-space:pre-line}  .multiline-tooltip-left{white-space:pre-line}  .multiline-tooltip-left>div{text-align:left}  .model-explorer-menu-item-with-icon{font-family:Google Sans Text,Arial,Helvetica,sans-serif;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;font-size:12px;padding:5px 8px;line-height:15px;letter-spacing:normal}  .model-explorer-menu-item-with-icon mat-icon{font-size:16px;width:16px;height:16px;margin-right:4px;color:#999}  .model-explorer-menu-item-with-icon:hover{background-color:rgba(0,0,0,.04)}"],
Pa:0});function oO(a){if(a&1){const b=Zg();R(0,"button",10);V("click",function(){t(b);const c=Y(2);return x(pO(c))});Z(1," Start ");U()}}function qO(a){a&1&&(R(0,"div",9),Z(1,"Generating graph. Please wait..."),U())}
function rO(a){a&1&&(R(0,"div",2)(1,"div",4)(2,"div",5),Z(3,"Node count"),U(),Oq(4,"input",6),U(),R(5,"div",4)(6,"div",5),Z(7,"Edge count"),U(),Oq(8,"input",6),U(),R(9,"div",4)(10,"div",5),Z(11,"Colorize"),U(),Oq(12,"input",7),U(),H(13,oO,2,0,"button",8)(14,qO,2,0,"div",9),U());a&2&&(a=Y(),D(4),Q("formControl",a.Ev),D(4),Q("formControl",a.Cv),D(4),Q("formControl",a.Bv),D(),Gq(a.iw?14:13))}function sO(a){a&1&&Oq(0,"webgl-renderer",3);a&2&&(a=Y(),Q("modelGraph",a.modelGraph)("benchmark",!0))}
function pO(a){a.iw=!0;fn(a.C);setTimeout(()=>{const b=a.Ev.value,c=a.Cv.value;var d=Math.max(500,Math.floor(300*Math.sqrt(b)));const e=[];for(var f=0;f<b;f++){var g=`n${f}`;g={nodeType:0,id:g,label:g,namespace:"",level:0,x:Math.random()*d,y:Math.random()*d,globalX:0,globalY:0,width:150+60*(Math.random()-.5),height:50+30*(Math.random()-.5)};a.Bv.value&&(g.style={backgroundColor:`rgb(${Math.floor(255*Math.random())}, ${Math.floor(255*Math.random())}, ${Math.floor(255*Math.random())})`});e.push(g)}d=
{};for(var h of e)d[h.id]=h;h={"":[]};for(f=0;f<c;f++){g=`n${Math.floor(Math.random()*b)}`;const w=`n${Math.floor(Math.random()*b)}`;var m=d[g],p=d[w];const B=`${g}_${w}`;var u=m.x+m.width/2;m=m.y+m.height/2;const G=p.x+p.width/2;p=p.y+p.height/2;u=[{x:u,y:m},{x:(u+G)/2+100*(Math.random()-.5),y:(m+p)/2+100*(Math.random()-.5)},{x:G,y:p}];p=vI(u);h[""].push({id:B,fromNodeId:g,toNodeId:w,points:u,curvePoints:p})}a.modelGraph={id:"benchmark_graph",collectionLabel:"benchmark_collection",nodes:e,nodesById:d,
rootNodes:e,edgesByGroupNodeIds:h,maxDescendantOpNodeCount:0,minDescendantOpNodeCount:0,layoutGraphEdges:{}};fn(a.C)},100)}var tO=class{constructor(a){this.C=a;this.Ev=new wx(1E4);this.Cv=new wx(5E3);this.Bv=new wx(!1);this.iw=!1}};tO.K=function(a){return new (a||tO)(y(Tr))};
tO.Ca=Jf({type:tO,ha:[["benchmark-runner"]],ka:!0,features:[Nr],ya:6,za:1,Ba:[[1,"container"],[1,"title"],[1,"form"],["rendererId","benchmark","paneId","benchmark",3,"modelGraph","benchmark"],[1,"row"],[1,"label"],["type","text",3,"formControl"],["type","checkbox",3,"formControl"],["mat-flat-button","","color","primary"],[1,"gen-msg"],["mat-flat-button","","color","primary",3,"click"]],sa:function(a,b){a&1&&(R(0,"div",0)(1,"div",1),Oq(2,"me-logo"),Z(3," Model Explorer Benchmark Test "),U(),H(4,rO,
15,4,"div",2)(5,sO,1,2,"webgl-renderer",3),U());a&2&&(D(4),Gq(b.modelGraph?5:4))},Ha:[it,BB,AB,vB,WJ,Dx,tw,rw,Nw,yx,nO],styles:[".container[_ngcontent-%COMP%]{width:100%;height:100%;overflow:hidden;-moz-box-sizing:border-box;box-sizing:border-box;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;position:relative}.container[_ngcontent-%COMP%]   .form[_ngcontent-%COMP%]{margin:12px}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]{font-weight:400;cursor:pointer;color:#444746;font-size:20px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;padding:8px;padding-left:12px;border-bottom:1px solid #ddd}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   me-logo[_ngcontent-%COMP%]{margin-right:6px}.container[_ngcontent-%COMP%]   .row[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;margin-top:12px}.container[_ngcontent-%COMP%]   .row[_ngcontent-%COMP%]   .label[_ngcontent-%COMP%]{font-size:12px;color:#777}.container[_ngcontent-%COMP%]   .row[_ngcontent-%COMP%]   input[type=text][_ngcontent-%COMP%]{width:200px;font-size:13px}.container[_ngcontent-%COMP%]   .row[_ngcontent-%COMP%]   input[type=checkbox][_ngcontent-%COMP%]{width:-webkit-fit-content;width:-moz-fit-content;width:fit-content}.container[_ngcontent-%COMP%]   .row[_ngcontent-%COMP%]:first-child{margin-top:0}.container[_ngcontent-%COMP%]   button[_ngcontent-%COMP%]{margin-top:12px}.container[_ngcontent-%COMP%]   .gen-msg[_ngcontent-%COMP%]{font-size:12px;margin-top:12px}.container[_ngcontent-%COMP%]   webgl-renderer[_ngcontent-%COMP%]{position:absolute;top:48px;bottom:0;width:100%}"],
Pa:0});var uO=class{constructor(){this.extensions={}}};uO.K=function(a){return new (a||uO)};uO.na=Pd({ma:uO,ba:uO.K});function vO(a){a&1&&(R(0,"tr")(1,"td"),Oq(2,"div",9),U(),R(3,"td")(4,"div",2),Z(5,"Artificial layer to reduce layout load"),U()()())}function wO(a){a&1&&(R(0,"tr")(1,"td"),Oq(2,"div",10),U(),R(3,"td")(4,"div",2),Z(5),U()()());a&2&&(a=Y(),D(2),Wp("group",a.Al),D(3),Cr(" Selected ",a.Al?"layer":"op"," "))}function xO(a){a&1&&(R(0,"tr")(1,"td"),Oq(2,"div",11),U(),R(3,"td")(4,"div",2),Z(5,"Identical layer (if any)"),U()()())}
function yO(a){a&1&&(Pq(0),R(1,"tr")(2,"td"),Oq(3,"div",12),U(),R(4,"td")(5,"div",2),Z(6,"Inputs (if any)"),U()()(),R(7,"tr")(8,"td"),Oq(9,"div",13),U(),R(10,"td")(11,"div",2),Z(12,"Outputs (if any)"),U()()(),Qq())}var zO=class{constructor(a,b){this.C=a;this.D=b;this.Mn=this.Al=this.fm=!1;Ks(()=>{var c=nJ(this.C,this.paneId);c&&(this.Mn=!0===c.Mn,(c=c.Sg)?(this.fm=""!==c.nodeId,this.Al=c.Cd):this.fm=!1,en(this.D))})}};zO.K=function(a){return new (a||zO)(y(VJ),y(Tr))};
zO.Ca=Jf({type:zO,ha:[["legends-panel"]],inputs:{paneId:"paneId"},ka:!0,features:[Nr],ya:35,za:4,Ba:[[1,"container"],[1,"key","op-node-key"],[1,"label"],[1,"key","group-node-key"],[4,"ngIf"],["colspan","2"],[1,"divider"],[1,"key","shortcut"],[1,"label","shortcut"],[1,"key","artificial-layer-key"],[1,"key","selected-node-key"],[1,"key","identical-group-key"],[1,"key","input-key"],[1,"key","output-key"]],sa:function(a,b){a&1&&(R(0,"div",0)(1,"table")(2,"tr")(3,"td"),Oq(4,"div",1),U(),R(5,"td")(6,"div",
2),Z(7,"Op"),U()()(),R(8,"tr")(9,"td"),Oq(10,"div",3),U(),R(11,"td")(12,"div",2),Z(13,"Layer"),U()()(),H(14,vO,6,0,"tr",4)(15,wO,6,3,"tr",4)(16,xO,6,0,"tr",4)(17,yO,13,0,"ng-container",4),R(18,"tr")(19,"td",5),Oq(20,"div",6),U()(),R(21,"tr")(22,"td")(23,"div",7),Z(24,"Zoom"),U()(),R(25,"td")(26,"div",8),Z(27,"Ctrl+Scroll"),U()()(),R(28,"tr")(29,"td")(30,"div",7),Z(31,"Pan"),U()(),R(32,"td")(33,"div",8),Z(34,"Drag or scroll"),U()()()()());a&2&&(D(14),Q("ngIf",b.Mn),D(),Q("ngIf",b.fm),D(),Q("ngIf",
b.fm&&b.Al),D(),Q("ngIf",b.fm&&!b.Al))},Ha:[it,dt,iI],styles:[".container[_ngcontent-%COMP%]{height:-webkit-fit-content;height:-moz-fit-content;height:fit-content;background-color:hsla(0,0%,100%,.9);padding:8px}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]{border-spacing:0;border-collapse:collapse;padding:0;font-size:11px}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .key[_ngcontent-%COMP%]{margin-right:4px;-moz-box-sizing:border-box;box-sizing:border-box}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .op-node-key[_ngcontent-%COMP%]{width:30px;height:14px;border-radius:5px;border:1px solid #666;background-color:#fff}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .group-node-key[_ngcontent-%COMP%]{width:30px;height:14px;border:1px solid #777;border-radius:5px;background-color:#eaeaf2}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .artificial-layer-key[_ngcontent-%COMP%]{border:1px solid purple;width:30px;height:14px;background-color:#f1f1f1}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .selected-node-key[_ngcontent-%COMP%]{width:30px;height:14px;border:2px solid #1a73e8;border-radius:5px;background-color:#c2e7ff}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .identical-group-key[_ngcontent-%COMP%]{width:30px;height:14px;border:1px solid #777;background-color:#e2edff}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .input-key[_ngcontent-%COMP%]{width:30px;height:14px;border-radius:5px;border:2px solid #009e73}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .output-key[_ngcontent-%COMP%]{width:30px;height:14px;border-radius:5px;border:2px solid #d55e00}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .shortcut[_ngcontent-%COMP%]{color:#777;font-size:10px;line-height:10px}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .divider[_ngcontent-%COMP%]{height:1px;border-top:1px solid #ccc;margin:8px 0 4px}"],
Pa:0});function AO(a,b){const c=b.clientX,d=b.clientY,e=b.offsetX,f=Number(a.C.style.top.replace("px","")),g=Number(a.C.style.left.replace("px","")),h=Number(a.C.style.width.replace("px","")),m=Number(a.C.style.height.replace("px","")),p=qk(document,"mousemove"),u=qk(window,"mouseup"),w=b.target.dataset.position,B=a.C.parentElement.offsetHeight,G=a.C.parentElement.offsetWidth;C(Uj([p]),Oo(u)).subscribe({next:([L])=>{L.preventDefault();var O=L.clientX-c;L=L.clientY-d;if(0!==O||0!==L)null==w?(a.C.style.top=
`${Math.min(B-28,Math.max(f+L,0))}px`,a.C.style.left=`${Math.min(G-e,Math.max(-e,g+O))}px`):(w.includes("right")&&(a.C.style.width=`${Math.max(48,h+O)}px`),w.includes("bottom")&&(a.C.style.height=`${Math.max(48,m+L)}px`),w.includes("left")&&(O=Math.max(48,h-O),a.C.style.width=`${O}px`,a.C.style.left=`${g+(h-O)}px`),w.includes("top")&&(O=Math.max(48,m-L),a.C.style.height=`${O}px`,a.C.style.top=`${f+(m-O)}px`))},complete:()=>{}})}var BO=class{constructor(a){this.C=a}};var CO=class{constructor(a){this.C=a;this.xb=new Jk;this.Fd=0}reset(){this.Fd=0;en(this.C)}get PB(){return 0===this.Fd}get OB(){return this.Fd===Math.ceil(this.Zn/this.Ke)-1}get GB(){const a=Math.min(this.Zn,this.Fd*this.Ke+1),b=Math.min((this.Fd+1)*this.Ke,this.Zn);return a===b?`${a}`:`${a} - ${b}`}};CO.K=function(a){return new (a||CO)(y(Tr))};
CO.Ca=Jf({type:CO,ha:[["paginator"]],inputs:{Ke:"pageSize",Zn:"itemsCount"},outputs:{xb:"change"},ka:!0,features:[Nr],ya:9,za:5,Ba:[[1,"container"],[1,"icon-container","prev-page",3,"click"],[1,"range-text"],[1,"icon-container","next-page",3,"click"]],sa:function(a,b){a&1&&(R(0,"div",0)(1,"div",1),V("click",function(){b.Fd--;b.Fd=Math.max(0,b.Fd);b.xb.emit(b.Fd)}),R(2,"mat-icon"),Z(3,"chevron_left"),U()(),R(4,"div",2),Z(5),U(),R(6,"div",3),V("click",function(){b.Fd++;b.Fd=Math.min(Math.ceil(b.Zn/
b.Ke)-1,b.Fd);b.xb.emit(b.Fd)}),R(7,"mat-icon"),Z(8,"chevron_right"),U()()());a&2&&(D(),Wp("disabled",b.PB),D(4),Cr(" ",b.GB," "),D(),Wp("disabled",b.OB))},Ha:[it,iI,hI,HG],styles:[".container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}.container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;width:24px;height:24px;border-radius:99px;cursor:pointer}.container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]:hover{background-color:#ddd}.container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]:hover   mat-icon[_ngcontent-%COMP%]{opacity:1}.container[_ngcontent-%COMP%]   .icon-container.disabled[_ngcontent-%COMP%]{pointer-events:none;opacity:.3}.container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{opacity:.7;color:#333;font-size:20px;width:20px;height:20px}.container[_ngcontent-%COMP%]   .range-text[_ngcontent-%COMP%]{color:#333;font-size:12px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center}"],
Pa:0});const DO=["searchInput"],EO=["content"];function FO(a){if(a&1){const b=Zg();R(0,"div",11);V("click",function(){t(b);const c=Y();c.Eh.setValue("");GO(c,"");return x()});R(1,"mat-icon",12);Z(2,"close");U()()}}function HO(a){if(a&1){const b=Zg();R(0,"paginator",25);V("change",function(c){t(b);const d=Y(2);d.eh=c;IO(d);return x()});U()}a&2&&(a=Y(2),Q("pageSize",a.Ke)("itemsCount",a.qs))}
function JO(a,b){if(a&1){const c=Zg();R(0,"div",26);V("click",function(){const d=t(c).da,e=Y(2);return x(KO(e,d))});R(1,"div",27)(2,"mat-icon");Z(3);U()();R(4,"div",28);Z(5);U()()}a&2&&(a=b.da,Wp("selected",a.selected),D(3),Br(a.Hl),D(2),Cr(" ",a.label," "))}function LO(a){a&1&&Oq(0,"io-tree",22);a&2&&(a=Y(2),Q("data",a.Iv)("rendererId",a.rendererId)("tooltipPosition","right")("showLocator",!1))}function MO(a){a&1&&(R(0,"div",23),Z(1," No matching results "),U())}
function NO(a){a&1&&(R(0,"div",13)(1,"div",14)(2,"div",15)(3,"div",16),Z(4),U(),H(5,HO,1,2,"paginator",17),U(),R(6,"div",18)(7,"div",19),Lq(8,JO,6,4,"div",20,Jq),U()()(),R(10,"div",21,2),H(12,LO,1,4,"io-tree",22)(13,MO,2,0,"div",23),U(),R(14,"div",24)(15,"mat-icon"),Z(16,"tips_and_updates"),U(),Z(17," Click to locate node. Alt+Click to locate and select node. "),U()());a&2&&(a=Y(),Up("max-height",a.bE,"px"),D(4),Cr(" ",a.cE," "),D(),Gq(a.sE?5:-1),D(3),Nq(a.ws),D(4),Gq(0<a.qs?12:13))}
function OO(a){a&1&&(R(0,"div",29),Z(1," No matching results "),U())}
function PO(a){a&1&&(R(0,"div",30),Z(1," Use regex to match nodes by label, attributes, input/output node label, and input/output metadata. "),Oq(2,"br")(3,"br"),Z(4," For attributes and I/O metadata, regex will try to match "),R(5,"span",31),Z(6),U(),Z(7," or "),R(8,"span",31),Z(9),U(),Z(10,". "),Oq(11,"br")(12,"br"),Z(13," Examples:"),Oq(14,"br"),R(15,"ul")(16,"li")(17,"span",32),Z(18,"axis:"),U(),Z(19," will match attributes whose key is "),R(20,"span",31),Z(21,"axis"),U(),Z(22,". "),U(),R(23,
"li")(24,"span",32),Z(25,"padding=[SAME|VALID]"),U(),Z(26," will match attributes whose key is "),R(27,"span",31),Z(28,"padding"),U(),Z(29," and value is either "),R(30,"span",31),Z(31,"SAME"),U(),Z(32," or "),R(33,"span",31),Z(34,"VALID"),U(),Z(35,". "),U()()());a&2&&(D(6),Er("","{","key","}",":","{","value","}",""),D(3),Er("","{","key","}","=","{","value","}",""))}function KO(a,b){b.selected=!b.selected;en(a.J);GO(a,a.Eh.value||"");setTimeout(()=>{a.content.R.scrollTop=0})}
function GO(a,b){if(b){var c=[],d=[],e={results:{}},f=new Set(a.ws.filter(g=>g.selected).map(g=>g.Hl));try{const g=new RegExp(b,"i");for(const h of a.ga.nodes){if(nI(h)&&h.hideInLayout)continue;const {matches:m,matchTypes:p}=JI(f,g,h,a.ga);0<m.length&&(c.push(h),d.push({matches:m,matchTypes:p}),e.results[h.id]=m)}MJ(a.D,a.rendererId,e)}catch(g){console.warn("Failed to search",g)}a.eh=0;a.C=[...c];a.N=d;IO(a);a.F=!1;en(a.J)}else a.eh=0,a.Iv=void 0,a.C=[],en(a.J),a.F=!1,NJ(a.D,a.rendererId)}
function IO(a){a.Iv=aL(a.C.slice(a.eh*a.Ke,(a.eh+1)*a.Ke),"incoming",void 0,a.N.slice(a.eh*a.Ke,(a.eh+1)*a.Ke))}
var QO=class{constructor(a,b,c){this.D=a;this.J=b;this.G=c;this.Eh=new wx("");this.ws=[{Hl:"title",label:"Label",selected:!0},{Hl:"list",label:"Attrs",selected:!0},{Hl:"input",label:"Inputs",selected:!0},{Hl:"output",label:"Outputs",selected:!0}];this.Ke=this.D.C?12:50;this.C=[];this.N=[];this.F=!1;this.eh=0;C(this.Eh.dh,So(()=>{this.F=!0;en(this.J)}),Rn(300),tL(this.G)).subscribe(d=>{GO(this,((null==d?void 0:d.toLowerCase())||"").trim())});C(this.D.Ea,tL(this.G)).subscribe(()=>{let d;this.rendererId===
(null==(d=this.D.G())?void 0:d.id)&&this.aE.R.focus()})}get cE(){let a;const b=(null==(a=this.C)?void 0:a.length)||0;return`${b} result${1===b?"":"s"}`}get iE(){return""!==(this.Eh.value||"").trim()}get tE(){return!this.F&&""!==(this.Eh.value||"").trim()&&!this.Yx}get Yx(){return!this.F&&""!==(this.Eh.value||"").trim()&&0===this.C.length&&this.ws.every(a=>a.selected)}get qs(){return this.C.length}get bE(){return document.body.offsetHeight-300}get sE(){return this.qs>this.Ke}};
QO.K=function(a){return new (a||QO)(y(VJ),y(Tr),y(Ci))};
QO.Ca=Jf({type:QO,ha:[["search-bar"]],eb:function(a,b){a&1&&(vr(DO,5),vr(EO,5));if(a&2){let c;wr(c=xr())&&(b.aE=c.first);wr(c=xr())&&(b.content=c.first)}},inputs:{ga:"curModelGraph",rendererId:"rendererId"},ka:!0,features:[Nr],ya:14,za:6,Ba:[["searchInput",""],["helpPopup",""],["content",""],[1,"container"],[1,"input-container"],[1,"search"],["placeholder","Search nodes/layers by regex",3,"formControl"],["class","btn-clear-container",3,"click",4,"ngIf"],["dialogLabel","help popup",1,"icon-container",
3,"xapInlineDialog","hoverDelayMs"],["class","search-results-container",3,"max-height",4,"ngIf"],["class","no-matches",4,"ngIf"],[1,"btn-clear-container",3,"click"],[1,"clear"],[1,"search-results-container"],[1,"title-container"],[1,"title"],[1,"title-label"],[3,"pageSize","itemsCount"],[1,"search-result-type-selector-container"],[1,"options-container"],[1,"option-container",3,"selected"],[1,"content"],[3,"data","rendererId","tooltipPosition","showLocator"],[1,"empty-results"],[1,"footer"],[3,"change",
"pageSize","itemsCount"],[1,"option-container",3,"click"],[1,"icon-container"],[1,"option-label"],[1,"no-matches"],[1,"model-explorer-search-help-popup"],[1,"code"],[1,"code","regex"]],sa:function(a,b){a&1&&(R(0,"div",3)(1,"div",4)(2,"mat-icon",5),Z(3,"search"),U(),Oq(4,"input",6,0),H(6,FO,3,0,"div",7),R(7,"div",8)(8,"mat-icon"),Z(9,"help_outline"),U()()(),H(10,NO,18,5,"div",9)(11,OO,2,0,"div",10),U(),H(12,PO,36,8,"ng-template",null,1,Rr));a&2&&(a=Ar(13),D(4),Q("formControl",b.Eh),D(2),Q("ngIf",b.iE),
D(),Q("xapInlineDialog",a)("hoverDelayMs",100),D(3),Q("ngIf",b.tE),D(),Q("ngIf",b.Yx))},Ha:[it,dt,$K,OB,RD,iI,hI,CO,Dx,tw,Nw,yx,HG,EG],styles:[".container[_ngcontent-%COMP%]{position:relative;overflow:visible;background-color:#fff;border-radius:99px;height:30px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;padding:0 8px}.container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;width:250px}.container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]   mat-icon.clear[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]   mat-icon.search[_ngcontent-%COMP%]{font-size:20px;width:20px;height:20px}.container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]   mat-icon.clear[_ngcontent-%COMP%]{opacity:.6;cursor:pointer}.container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]   mat-icon.clear[_ngcontent-%COMP%]:hover{opacity:.9}.container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]   input[_ngcontent-%COMP%]{height:24px;border:none;outline:none;-moz-box-sizing:border-box;box-sizing:border-box;padding:0 4px;font-size:12px;-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1}.container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]   .btn-clear-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center}.container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;cursor:pointer;color:#999;opacity:.8}.container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]:hover{opacity:1}.container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:20px;width:20px;height:20px}.container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]:has(input:focus)   mat-icon.search[_ngcontent-%COMP%]{color:#1a73e8}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]{overflow:hidden;position:absolute;top:calc(100% + 12px);left:-4px;border:1px solid #ccc;border-radius:4px;width:380px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;background-color:#fff;box-shadow:0 4px 6px -1px rgba(0,0,0,.1),0 2px 4px -2px rgba(0,0,0,.1)}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]   .title-container[_ngcontent-%COMP%]{background-color:#f6f6f6;border-bottom:1px solid #ddd;-moz-box-sizing:border-box;box-sizing:border-box;padding:0 10px 10px}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]{height:28px;min-height:28px;font-size:12px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   .title-label[_ngcontent-%COMP%]{white-space:nowrap}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;min-height:0;overflow-y:auto;padding-bottom:6px}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .empty-results[_ngcontent-%COMP%]{padding:12px 8px 6px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;font-size:12px;color:#ccc}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]   .search-result-type-selector-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;font-size:12px;margin-top:2px}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]   .search-result-type-selector-container[_ngcontent-%COMP%]   .options-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;gap:6px}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]   .search-result-type-selector-container[_ngcontent-%COMP%]   .option-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;cursor:pointer;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;border:1px solid #ccc;padding:4px 14px;border-radius:6px;background-color:#fff;color:#474747}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]   .search-result-type-selector-container[_ngcontent-%COMP%]   .option-container.selected[_ngcontent-%COMP%]{border-color:#a3c9ff;background-color:#a3c9ff;color:#001d35}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]   .search-result-type-selector-container[_ngcontent-%COMP%]   .option-container.selected[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{color:#001d35}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]   .search-result-type-selector-container[_ngcontent-%COMP%]   .option-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{width:18px;height:18px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;-moz-box-sizing:border-box;box-sizing:border-box;margin-right:4px}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]   .search-result-type-selector-container[_ngcontent-%COMP%]   .option-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:16px;width:16px;height:16px;color:#474747}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]   .search-result-type-selector-container[_ngcontent-%COMP%]   .option-container[_ngcontent-%COMP%]   .option-label[_ngcontent-%COMP%]{font-size:12px;font-family:Google Sans Text,Arial,Helvetica,sans-serif;letter-spacing:normal}.container[_ngcontent-%COMP%]   .footer[_ngcontent-%COMP%]{height:32px;min-height:32px;background-color:#f6f6f6;border-top:1px solid #ddd;-moz-box-sizing:border-box;box-sizing:border-box;padding:0 10px;font-size:12px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;color:#777}.container[_ngcontent-%COMP%]   .footer[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:16px;height:16px;width:16px;color:#777;margin-right:4px}.container[_ngcontent-%COMP%]   .no-matches[_ngcontent-%COMP%]{position:absolute;top:calc(100% + 12px);left:-4px;color:#999;padding:8px;border:1px solid #ccc;border-radius:4px;background-color:#fff;font-size:12px;box-shadow:0 4px 6px -1px rgba(0,0,0,.1),0 2px 4px -2px rgba(0,0,0,.1)}  .model-explorer-search-help-popup{padding:12px;font-size:12px;background-color:#fff}  .model-explorer-search-help-popup ul{-webkit-margin-after:0;margin-block-end:0}  .model-explorer-search-help-popup .code{display:inline-block;background-color:#fffdd0;font-family:monospace}  .model-explorer-search-help-popup .code.regex{background-color:#e6d0ff}"],
Pa:0});const RO=["dialog"],SO=["trigger"];function TO(a){a&1&&Z(0," Maximum state count reached ")}function UO(a){a&1&&Z(0," Save the current graph state ")}function VO(a){a&1&&(R(0,"div",7),H(1,TO,1,0)(2,UO,1,0),R(3,"div",8),Z(4," The saved state can be restored by clicking the corresponding state number. You can save up to 9 states. "),U()());a&2&&(a=Y(),D(),Gq(a.Bq?1:2))}
function WO(a){if(a&1){const c=Zg();R(0,"div",12)(1,"div",13)(2,"div");Z(3);U();R(4,"div",14);V("click",function(){t(c);const d=Y().index,e=Y(2);e.ga&&IJ(e.gk,d,e.ga.id,e.paneId);return x()});R(5,"mat-icon");Z(6,"delete");U();Z(7," Delete ");U()();Oq(8,"canvas");U()}if(a&2){var b=Y();a=b.da;b=b.index;Y(2);D(3);Cr("Shortcut key: ",b+1,"");D(5);Up("width",XO(a,!0),"px")("height",YO(a,!0),"px")}}
function ZO(a,b){if(a&1){const c=Zg();R(0,"div",11,1);V("opened",function(){const d=t(c).da;Y(2);const e=document.querySelector(".model-explorer-snapshot-popup canvas"),f=XO(d),g=YO(d);e.width=f;e.height=g;const h=e.getContext("2d");h.imageSmoothingQuality="high";h.drawImage(d.er,0,0,f,g);e.classList.add("show");return x()})("click",function(){const d=t(c).index,e=Y(2);return x($O(e,d))});Z(3);H(4,WO,9,5,"ng-template",null,2,Rr);U()}if(a&2){a=b.index;b=Ar(5);const c=Y(2);Q("xapInlineDialog",b)("overlaySize",
c.AE)("overlayPositions",c.zE)("hoverDelayMs",150);D(3);Cr(" ",a+1," ")}}function aP(a){a&1&&(R(0,"div",9),H(1,ZO,6,5,"div",10),U());a&2&&(a=Y(),D(),Q("ngForOf",a.gj)("ngForTrackBy",a.RE))}const bP=new Set("123456789".split(""));function XO(a,b=!1){return a.er.width/(b?window.devicePixelRatio:1)}function YO(a,b=!1){return a.er.height/(b?window.devicePixelRatio:1)}function $O(a,b){const c=a.gj[b];let d;null==(d=a.ly.get(b))||rG(d);a.gk.ca.next({kb:c,rendererId:a.rendererId})}
var cP=class{constructor(a,b){this.gk=a;this.C=b;this.ly=new cr;this.my=new cr;this.fd={minWidth:0,minHeight:0,maxWidth:340};this.AE={maxWidth:1E3,maxHeight:1E3};this.zE=[{bb:"start",cb:"bottom",Na:"start",Ua:"top",offsetY:12}];this.gj=[];Ks(()=>{const c=nJ(this.gk,this.paneId);this.ga=null==c?void 0:c.modelGraph;null!=(null==c?void 0:c.modelGraph)&&(this.gj=((null==c?void 0:c.Tg)||{})[c.modelGraph.id]||[],en(this.C))})}Xq(a){let b;if(bP.has(a.key)&&(null==(b=this.gk.G())?void 0:b.id)===this.rendererId&&
!yI()&&(a=Number(a.key)-1,a<=this.gj.length-1)){$O(this,a);let c;const d=null==(c=this.my.get(a))?void 0:c.R;d&&(d.classList.add("clicked"),setTimeout(()=>{d.classList.remove("clicked")},50))}}RE(a,b){return b.id}get Bq(){return 9<=this.gj.length}get sC(){return 0<this.gj.length}};cP.K=function(a){return new (a||cP)(y(VJ),y(Tr))};
cP.Ca=Jf({type:cP,ha:[["snapshot-manager"]],eb:function(a,b){a&1&&(vr(RO,5),vr(SO,5));if(a&2){let c;wr(c=xr())&&(b.ly=c);wr(c=xr())&&(b.my=c)}},Ia:function(a,b){a&1&&V("keypress",function(c){return b.Xq(c)},!1,Gi)},inputs:{paneId:"paneId",rendererId:"rendererId"},ka:!0,features:[Nr],ya:7,za:6,Ba:[["help",""],["dialog","xapInlineDialog","trigger",""],["snapshotPopup",""],[1,"container"],["dialogLabel","help",1,"mat-icon-container","take-snapshot",3,"click","xapInlineDialog","overlaySize","hoverDelayMs"],
[1,"toolbar-icon"],["class","snapshots-container",4,"ngIf"],[1,"model-explorer-help-popup"],[1,"description"],[1,"snapshots-container"],["class","snapshot","dialogLabel","snapshot popup",3,"xapInlineDialog","overlaySize","overlayPositions","hoverDelayMs","opened","click",4,"ngFor","ngForOf","ngForTrackBy"],["dialogLabel","snapshot popup",1,"snapshot",3,"opened","click","xapInlineDialog","overlaySize","overlayPositions","hoverDelayMs"],[1,"model-explorer-snapshot-popup"],[1,"info-row"],[1,"icon-container",
"delete",3,"click"]],sa:function(a,b){if(a&1){const c=Zg();R(0,"div",3)(1,"div",4);V("click",function(){t(c);b.Bq||b.gk.oa.next({rendererId:b.rendererId});return x()});R(2,"mat-icon",5);Z(3,"bookmark_add");U()();H(4,VO,5,1,"ng-template",null,0,Rr)(6,aP,2,2,"div",6);U()}a&2&&(a=Ar(5),D(),Wp("disabled",b.Bq),Q("xapInlineDialog",a)("overlaySize",b.fd)("hoverDelayMs",10),D(5),Q("ngIf",b.sC))},Ha:[it,$s,dt,PE,iI,hI,HG,EG],styles:[".container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;margin-right:3px}.container[_ngcontent-%COMP%]   .mat-icon-container[_ngcontent-%COMP%]{width:24px;height:24px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;cursor:pointer}.container[_ngcontent-%COMP%]   .mat-icon-container.disabled[_ngcontent-%COMP%]{cursor:default;opacity:.5}.container[_ngcontent-%COMP%]   .mat-icon-container.disabled[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]:hover{opacity:.6}.container[_ngcontent-%COMP%]   mat-icon.toolbar-icon[_ngcontent-%COMP%]{font-size:20px;width:20px;height:20px;opacity:.6}.container[_ngcontent-%COMP%]   mat-icon.toolbar-icon[_ngcontent-%COMP%]:hover{opacity:.9}.container[_ngcontent-%COMP%]   .snapshots-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;font-size:11px;margin-left:6px;margin-right:6px;gap:2px}.container[_ngcontent-%COMP%]   .snapshots-container[_ngcontent-%COMP%]   .snapshot[_ngcontent-%COMP%]{width:16px;height:16px;border-radius:8px;background-color:#7cacf8;color:#fff;-moz-box-sizing:border-box;box-sizing:border-box;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;font-weight:500;cursor:pointer}.container[_ngcontent-%COMP%]   .snapshots-container[_ngcontent-%COMP%]   .snapshot.clicked[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   .snapshots-container[_ngcontent-%COMP%]   .snapshot[_ngcontent-%COMP%]:hover{background-color:#0b57d0;color:#fff}  .model-explorer-snapshot-popup{font-size:11px;line-height:12px;background-color:#fff}  .model-explorer-snapshot-popup .info-row{color:#333;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;background-color:#f6f6f6;padding:4px 8px;border-bottom:1px solid #ddd}  .model-explorer-snapshot-popup .info-row .icon-container{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;cursor:pointer;opacity:.8}  .model-explorer-snapshot-popup .info-row .icon-container:hover{opacity:1}  .model-explorer-snapshot-popup .info-row .icon-container mat-icon{font-size:18px;width:18px;height:19px}  .model-explorer-snapshot-popup canvas{opacity:0;-webkit-transition:opacity .2s ease-out;transition:opacity .2s ease-out}  .model-explorer-snapshot-popup canvas.show{opacity:1}"],
Pa:0});function dP(a){a&1&&(R(0,"mat-icon",2),Z(1,"chevron_right"),U())}
function eP(a,b){if(a&1){const c=Zg();R(0,"div",1);V("click",function(){var d=t(c).Cb;const e=Y();if(d!==e.Nf.length-1){LJ(e.wi,e.paneId,d);var f=e.Nf[d];if(d=f.kb){var g;let h;f.Cg===(null==(g=nJ(e.wi,e.paneId))?void 0:null==(h=g.modelGraph)?void 0:h.id)?e.wi.ca.next({rendererId:e.paneId,kb:d}):(g=JJ(e.wi,f.Cg))&&qJ(e.wi,g,d.flattenLayers,d)}}return x()});Z(1);U();H(2,dP,2,0,"mat-icon",2)}if(a&2){a=b.da;b=b.Cb;const c=Y();Wp("current",b===c.Nf.length-1);D();Cr(" ",a.Cg," ");D();Gq(b!==c.Nf.length-
1?2:-1)}}var fP=class{constructor(a,b){this.wi=a;this.C=b;this.Nf=[];Ks(()=>{let c;const d=null==(c=nJ(this.wi,this.paneId))?void 0:c.bg;d!==this.D&&(this.Nf=(this.D=d)||[],en(this.C))})}};fP.K=function(a){return new (a||fP)(y(VJ),y(Tr))};
fP.Ca=Jf({type:fP,ha:[["subgraph-breadcrumbs"]],inputs:{paneId:"paneId"},ka:!0,features:[Nr],ya:3,za:0,Ba:[[1,"container"],[1,"item",3,"click"],[1,"separator"]],sa:function(a,b){a&1&&(R(0,"div",0),Lq(1,eP,3,4,null,null,Jq),U());a&2&&(D(),Nq(b.Nf))},Ha:[it,iI,hI,OE],styles:[".container[_ngcontent-%COMP%]{border:1px solid #3876c7;background-color:#d5e7ff;border-radius:4px;-moz-box-sizing:border-box;box-sizing:border-box;padding:4px 8px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;font-size:13px}.container[_ngcontent-%COMP%]   .item[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;color:#777}.container[_ngcontent-%COMP%]   .item[_ngcontent-%COMP%]:hover:not(.current){cursor:pointer;text-decoration:underline}.container[_ngcontent-%COMP%]   .item.current[_ngcontent-%COMP%]{color:#000}.container[_ngcontent-%COMP%]   .separator[_ngcontent-%COMP%]{color:#999;font-size:16px;width:16px;height:16px;margin:0 3px}"],
Pa:0});const gP=(a,b)=>b.type;function hP(a){a&1&&(R(0,"div",7),Z(1," Select data to view on nodes / edges "),U())}
function iP(a){if(a&1){const b=Zg();R(0,"div",14)(1,"input",15,3);V("keydown.enter",function(){t(b);const c=Ar(2);return x(c.blur())})("input",function(){t(b);const c=Ar(2),d=Y(3);return x(d.mn=c.value)})("change",function(){t(b);const c=Y().da,d=Y(2);QJ(d.Pe,d.paneId,d.rendererId,c.type,d.mn);jP(d);return x()});U();R(3,"div",16)(4,"mat-icon");Z(5,"help_outline");U()();R(6,"div",17);V("click",function(){t(b);const c=Ar(2);return x(c.blur())});R(7,"mat-icon");Z(8,"done");U()()()}if(a&2){a=Y().da;const b=
Y(2),c=Ar(9);D();Q("disabled",!a.selected)("value",b.mn);D(2);Q("xapInlineDialog",c)("hoverDelayMs",100)}}function kP(a,b){if(a&1){const c=Zg();R(0,"div",12);V("click",function(){const d=t(c).da,e=Y(2);OJ(e.Pe,e.paneId,e.rendererId,d.type);jP(e);return x()});Oq(1,"input",13);Z(2);U();H(3,iP,9,4,"div",14)}a&2&&(a=b.da,Y(2),D(),Q("checked",a.selected),D(),Cr(" ",a.type," "),D(),Gq("Op node attributes"===a.type?3:-1))}
function lP(a,b){if(a&1){const c=Zg();R(0,"div",12);V("click",function(){var d=t(c).da;const e=Y(2);PJ(e.Pe,e.paneId,e.rendererId,d.type);e.Rb||e.Pe.C||(d=UJ(e.Pe,e.paneId,e.rendererId),PI("model_explorer_show_on_edge_item_types_v2",JSON.stringify(d)));return x()});Oq(1,"input",13);Z(2);U()}a&2&&(a=b.da,D(),Q("checked",a.selected),D(),Cr(" ",a.type," "))}
function mP(a){a&1&&(R(0,"div",8)(1,"div",9)(2,"div"),Z(3,"View on nodes"),U(),R(4,"div",10)(5,"mat-icon"),Z(6,"close"),U()()(),Lq(7,kP,4,3,null,null,gP),R(9,"div",9)(10,"div"),Z(11,"View on edges"),U()(),Lq(12,lP,3,2,"div",11,gP),U());a&2&&(a=Y(),D(7),Nq(a.by),D(5),Nq(a.ay))}
function nP(a){a&1&&(R(0,"div",18),Z(1," Use regex to match "),R(2,"span",19),Z(3),U(),Z(4," or "),R(5,"span",19),Z(6),U(),Z(7,". "),Oq(8,"br")(9,"br"),Z(10," Examples:"),Oq(11,"br"),R(12,"ul")(13,"li")(14,"span",20),Z(15,"axis:"),U(),Z(16," will match attributes whose key is "),R(17,"span",19),Z(18,"axis"),U(),Z(19,". "),U(),R(20,"li")(21,"span",20),Z(22,"padding=[SAME|VALID]"),U(),Z(23," will match attributes whose key is "),R(24,"span",19),Z(25,"padding"),U(),Z(26," and value is either "),R(27,
"span",19),Z(28,"SAME"),U(),Z(29," or "),R(30,"span",19),Z(31,"VALID"),U(),Z(32,". "),U()()());a&2&&(D(3),Er("","{","key","}",":","{","value","}",""),D(3),Er("","{","key","}","=","{","value","}",""))}const oP="Op node id;Op node attributes;Op node inputs;Op node outputs;Layer node children count;Layer node descendants count".split(";"),pP=["Tensor shape"];
function jP(a){if(!a.Rb&&!a.Pe.C){a=kJ(a.Pe,a.paneId,a.rendererId);const b=Object.keys(a).filter(c=>c.startsWith("Node data provider: "));for(const c of b)delete a[c];PI("model_explorer_show_on_node_item_types_v2",JSON.stringify(a))}}
var qP=class{constructor(a,b,c,d){this.Pe=a;this.D=b;this.F=d;this.Rb=!1;this.G=Cs(()=>{const e=yJ(this.Pe,this.paneId);return(e?Object.values(wL(this.F,e)):[]).map(f=>f.runName)});this.C=[];this.fd={minWidth:0,minHeight:0};this.gF={minWidth:280,minHeight:0};this.by=[];this.ay=[];this.mn="";this.kf=!1;Ks(()=>{var e;const f=(null==(e=nJ(this.Pe,this.paneId))?void 0:e.showOnNodeItemTypes)||{};e=this.G();if(f!==this.N||JSON.stringify(e)!==JSON.stringify(this.C)){this.C=e;this.N=f;e=[];for(var g of oP){let h;
const m={type:g,selected:null==(h=(f[this.rendererId]||{})[g])?void 0:h.selected};e.push(m);if("Op node attributes"===g){let p;m.filterRegex=(null==(p=(f[this.rendererId]||{})[g])?void 0:p.filterRegex)||"";this.mn=m.filterRegex}}for(const h of this.C){g=`${"Node data provider: "}${h}`;let m;e.push({type:g,selected:null==(m=(f[this.rendererId]||{})[g])?void 0:m.selected})}this.by=e;en(this.D)}});Ks(()=>{var e;const f=(null==(e=nJ(this.Pe,this.paneId))?void 0:e.Yb)||{};if(f!==this.J){this.J=f;e=[];
for(const g of pP){let h;const m={type:g,selected:null==(h=(f[this.rendererId]||{})[g])?void 0:h.selected};e.push(m)}this.ay=e;en(this.D)}})}};qP.K=function(a){return new (a||qP)(y(VJ),y(Tr),y(QI),y(LL))};
qP.Ca=Jf({type:qP,ha:[["view-on-node"]],eb:function(a,b){a&1&&vr(GG,5);if(a&2){let c;wr(c=xr())&&(b.LD=c.first)}},inputs:{rendererId:"rendererId",paneId:"paneId",Rb:"inPopup"},ka:!0,features:[Nr],ya:10,za:5,Ba:[["help",""],["viewPopup",""],["helpPopup",""],["input",""],["dialogLabel","help",1,"container",3,"xapInlineDialog","overlaySize","hoverDelayMs"],["dialogLabel","view popup",1,"mat-icon-container","view",3,"opened","closed","click","xapInlineDialogClick","overlaySize"],[1,"toolbar-icon"],[1,
"model-explorer-help-popup"],[1,"model-explorer-view-popup"],[1,"label"],["xapInlineDialogClose","",1,"icon-container","close"],[1,"show-on-node-item"],[1,"show-on-node-item",3,"click"],["type","checkbox",3,"checked"],[1,"show-on-node-filter"],["placeholder","Filter by regex",1,"input-attrs-filter",3,"keydown.enter","input","change","disabled","value"],["dialogLabel","help popup",1,"icon-container",3,"xapInlineDialog","hoverDelayMs"],[1,"icon-container","ok",3,"click"],[1,"model-explorer-view-on-node-help-popup"],
[1,"code"],[1,"code","regex"]],sa:function(a,b){if(a&1){const c=Zg();R(0,"div",4)(1,"div",5);V("opened",function(){t(c);return x(b.kf=!0)})("closed",function(){t(c);return x(b.kf=!1)})("click",function(){t(c);b.kf&&rG(b.LD);return x()});R(2,"mat-icon",6);Z(3,"visibility");U()()();H(4,hP,2,0,"ng-template",null,0,Rr)(6,mP,14,0,"ng-template",null,1,Rr)(8,nP,33,8,"ng-template",null,2,Rr)}if(a&2){a=Ar(5);const c=Ar(7);Q("xapInlineDialog",a)("overlaySize",b.fd)("hoverDelayMs",10);D();Q("xapInlineDialogClick",
c)("overlaySize",b.gF)}},Ha:[it,PE,iI,hI,HG,EG,GG],styles:[".container[_ngcontent-%COMP%]   .mat-icon-container[_ngcontent-%COMP%]{width:20px;height:20px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;cursor:pointer;opacity:.6}.container[_ngcontent-%COMP%]   .mat-icon-container[_ngcontent-%COMP%]:hover{opacity:.9}.container[_ngcontent-%COMP%]   .mat-icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:20px;width:20px;height:20px}  .model-explorer-view-popup{padding:12px;padding-top:10px;font-size:12px;background-color:#fff;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column}  .model-explorer-view-popup .label{font-weight:500;font-size:11px;text-transform:uppercase;letter-spacing:.0727em;margin-bottom:6px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between}  .model-explorer-view-popup .label:not(:first-child){margin-top:12px}  .model-explorer-view-popup .label .icon-container{cursor:pointer;opacity:.7;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center}  .model-explorer-view-popup .label .icon-container:hover{opacity:1}  .model-explorer-view-popup .label mat-icon{font-size:20px;width:20px;height:20px;color:#999}  .model-explorer-view-popup .show-on-node-item{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;gap:4px;cursor:pointer;color:#555;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}  .model-explorer-view-popup .show-on-node-item input{cursor:pointer}  .model-explorer-view-popup .show-on-node-filter{margin:1px 0 4px 25px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;position:relative}  .model-explorer-view-popup .show-on-node-filter:focus-within .icon-container.ok{visibility:visible}  .model-explorer-view-popup .show-on-node-filter input{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;-moz-box-sizing:border-box;box-sizing:border-box;border:1px solid #aaa;border-radius:3px;padding:3px 26px 3px 4px;min-width:200px}  .model-explorer-view-popup .show-on-node-filter input::-webkit-input-placeholder{color:#aaa}  .model-explorer-view-popup .show-on-node-filter input::-moz-placeholder{color:#aaa}  .model-explorer-view-popup .show-on-node-filter input:-ms-input-placeholder{color:#aaa}  .model-explorer-view-popup .show-on-node-filter input::-ms-input-placeholder{color:#aaa}  .model-explorer-view-popup .show-on-node-filter input::placeholder{color:#aaa}  .model-explorer-view-popup .show-on-node-filter .icon-container{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;opacity:.8;cursor:pointer;margin-left:4px}  .model-explorer-view-popup .show-on-node-filter .icon-container:hover{opacity:1}  .model-explorer-view-popup .show-on-node-filter .icon-container.ok{position:absolute;right:26px;visibility:hidden}  .model-explorer-view-popup .show-on-node-filter .icon-container mat-icon{font-size:16px;width:16px;height:16px;color:#999}  .model-explorer-view-on-node-help-popup{padding:12px;font-size:12px;background-color:#fff}  .model-explorer-view-on-node-help-popup ul{-webkit-margin-after:0;margin-block-end:0}  .model-explorer-view-on-node-help-popup .code{display:inline-block;background-color:#fffdd0;font-family:monospace}  .model-explorer-view-on-node-help-popup .code.regex{background-color:#e6d0ff}  .cdk-overlay-container:has(.model-explorer-view-popup) .cdk-overlay-backdrop{pointer-events:none}"],
Pa:0});const rP=["webglRenderer"];function sP(a){a&1&&(Pq(0),Oq(1,"search-bar",21)(2,"div",15),Qq());a&2&&(a=Y(2),D(),Q("curModelGraph",a.modelGraph)("rendererId",a.rendererId))}function tP(a){a&1&&(R(0,"div",22),Z(1," Zoom to fit "),R(2,"div",23)(3,"mat-icon"),Z(4,"keyboard"),U(),Z(5," Shortcut: SPACE "),U()())}function uP(a){a&1&&(R(0,"div",22),Z(1," Expand all layers in the graph "),U())}function vP(a){a&1&&(R(0,"div",22),Z(1," Collapse all layers in the graph "),U())}
function wP(a){if(a&1){const b=Zg();R(0,"div",24);V("click",function(){t(b);const c=Y(2);c.Xc.ea.next({cw:!0,rendererId:c.rendererId});return x()});R(1,"mat-icon",25);Z(2,"unfold_more_double");U()();H(3,uP,2,0,"ng-template",null,3,Rr);R(5,"div",26);V("click",function(){t(b);const c=Y(2);c.Xc.ea.next({cw:!1,rendererId:c.rendererId});return x()});R(6,"mat-icon",27);Z(7,"unfold_less_double");U()();H(8,vP,2,0,"ng-template",null,4,Rr);Oq(10,"div",15)}if(a&2){a=Ar(4);const b=Ar(9),c=Y(2);Wp("disable",c.Vv);
Q("xapInlineDialog",a)("overlaySize",c.fd)("hoverDelayMs",10);D(5);Wp("disable",c.Vv);Q("xapInlineDialog",b)("overlaySize",c.fd)("hoverDelayMs",10)}}function xP(a){a&1&&(R(0,"div",22),Z(1," Flatten all layers in the graph "),R(2,"div",30),Z(3," Nodes will be laid out without any layers data "),U()())}
function yP(a){if(a&1){const b=Zg();R(0,"div",28);V("click",function(){t(b);const c=Y(2);let d,e;const f=null==(d=nJ(c.Xc,c.paneId))?void 0:null==(e=d.Sg)?void 0:e.nodeId;null!=f&&oI(c.modelGraph.nodesById[f])&&wJ(c.Xc,c.paneId);tJ(c.Xc,c.paneId);mJ(c.Xc,c.paneId,uJ(c.Xc,c.paneId));c.Xc.N.set(void 0);return x()});R(1,"mat-icon",29);Z(2,"layers_clear");U()();Oq(3,"div",15);H(4,xP,4,0,"ng-template",null,5,Rr)}if(a&2){a=Ar(5);const b=Y(2);Wp("enabled",b.gC());Q("xapInlineDialog",a)("overlaySize",b.fd)("hoverDelayMs",
10)}}function zP(a){a&1&&(R(0,"mat-icon",19),Z(1," multiple_stop "),U())}function AP(a){a&1&&(R(0,"mat-icon",20),Z(1," multiple_stop "),U())}function BP(a){a&1&&(R(0,"span"),Z(1,"Disable inputs/outputs tracing"),U())}function CP(a){a&1&&(R(0,"span"),Z(1,"Enable inputs/outputs tracing"),U())}function DP(a){a&1&&(R(0,"div",22),H(1,BP,2,0,"span")(2,CP,2,0,"span"),R(3,"div",30),Z(4," Highlight selected op node's ancestors and descendants and dim the rest "),U()());a&2&&(a=Y(2),D(),Gq(a.Yg?1:2))}
function EP(a){a&1&&(R(0,"div",22),Z(1," Download graph as PNG "),R(2,"div",30),Z(3," The maximum PNG size is limited to 5000 x 5000 "),U()())}
function FP(a){if(a&1){const b=Zg();Oq(0,"div",15);R(1,"div",31);V("menuOpened",function(){t(b);const c=Y(2);return x(c.Eq=!0)})("menuClosed",function(){t(b);const c=Y(2);return x(c.Eq=!1)});R(2,"mat-icon",32);Z(3," image ");U()();H(4,EP,4,0,"ng-template",null,6,Rr);R(6,"mat-menu",33,7)(8,"div",34);V("click",function(){t(b);const c=Y(2);return x(GP(c,!1))});Z(9," Graph in current viewport ");U();R(10,"div",34);V("click",function(){t(b);const c=Y(2);return x(GP(c,!0))});Z(11," Full graph ");U();R(12,
"div",35);V("click",function(c){t(b);const d=Y(2);c.stopPropagation();d.Ro.setValue(!d.Ro.value);return x()});Oq(13,"input",36);Z(14," Transparent background ");U()()}if(a&2){a=Ar(5);const b=Ar(7),c=Y(2);D();Q("xapInlineDialog",a)("overlaySize",c.fd)("hoverDelayMs",10)("xapInlineDialogDisabled",c.Eq)("matMenuTriggerFor",b);D(12);Q("formControl",c.Ro)}}function HP(a){a&1&&(Pq(0),Oq(1,"div",15)(2,"snapshot-manager",37),Qq());a&2&&(a=Y(2),D(2),Q("rendererId",a.rendererId)("paneId",a.paneId))}
function IP(a){if(a&1){const b=Zg();R(0,"div",12);H(1,sP,3,2,"ng-container",13);Oq(2,"view-on-node",14)(3,"div",15);R(4,"div",16);V("click",function(){t(b);Y().Xc.ja.next({});return x()});R(5,"mat-icon",17);Z(6,"fit_screen");U()();Oq(7,"div",15);H(8,tP,6,0,"ng-template",null,1,Rr)(10,wP,11,10)(11,yP,6,5);R(12,"div",18);V("click",function(){t(b);let c;null==(c=Y().bt)||c.no();return x()});H(13,zP,2,0,"mat-icon",19)(14,AP,2,0,"mat-icon",20);U();H(15,DP,5,1,"ng-template",null,2,Rr)(17,FP,15,6)(18,HP,
3,2,"ng-container",13);U()}if(a&2){a=Ar(9);const b=Ar(16),c=Y();Wp("in-popup",c.Rb)("pushed-down",c.gy);D();Q("ngIf",c.uE);D();Q("rendererId",c.rendererId)("paneId",c.paneId)("inPopup",c.Rb);D(2);Q("xapInlineDialog",a)("overlaySize",c.fd)("hoverDelayMs",10);D(6);Gq(c.kE?10:-1);D();Gq(c.lE?11:-1);D();Wp("enabled",c.Yg);Q("xapInlineDialog",b)("overlaySize",c.fd)("hoverDelayMs",10);D();Gq(c.RC||c.Bd?13:14);D(4);Gq(c.jE?17:-1);D();Q("ngIf",c.vE)}}
function JP(a){a&1&&Oq(0,"subgraph-breadcrumbs",38);a&2&&(a=Y(),Q("paneId",a.paneId))}function GP(a,b){a.Xc.ta.next({rendererId:a.rendererId,jC:b,VE:!0===a.Ro.value})}
var KP=class{constructor(a,b){this.Xc=a;this.D=b;this.Rb=!1;this.Ol=new Jk;this.fd={minWidth:0,minHeight:0,maxWidth:340};this.gC=Cs(()=>uJ(this.Xc,this.paneId));this.C=0;this.Eq=!1;this.Ro=new wx(!1);this.Nf=[];Ks(()=>{let c;this.Nf=(null==(c=nJ(this.Xc,this.paneId))?void 0:c.bg)||[];en(this.D)})}Fb(){let a,b;this.C=null!=(b=null==(a=this.Xc.config())?void 0:a.defaultRenderer)?b:0}Zq(a){this.Ol.emit(a)}get uE(){return!this.Rb}get kE(){return!this.Rb}get lE(){return!this.Rb}get jE(){return!this.Rb&&
0===this.C}get vE(){return!this.Rb&&0===this.C}get gy(){return!this.Rb&&1<this.Nf.length}get Vv(){return uJ(this.Xc,this.paneId)}get Yg(){let a;return!0===(null==(a=this.bt)?void 0:a.Yg)}get xE(){let a;return!(null==(a=this.Xc.config())?0:a.hideToolBar)}get RC(){return this.Xc.C}get Bd(){return!0}};KP.K=function(a){return new (a||KP)(y(VJ),y(Tr))};
KP.Ca=Jf({type:KP,ha:[["renderer-wrapper"]],eb:function(a,b){a&1&&vr(rP,5);if(a&2){let c;wr(c=xr())&&(b.bt=c.first)}},inputs:{modelGraph:"modelGraph",rendererId:"rendererId",paneId:"paneId",am:"rootNodeId",Rb:"inPopup"},outputs:{Ol:"openInPopupClicked"},ka:!0,features:[Nr],ya:5,za:7,Ba:[["webglRenderer",""],["helpZoomToFit",""],["helpTraceIO",""],["helpExpandAll",""],["helpCollapseAll",""],["helpFlattenLayers",""],["helpDownloadPng",""],["menu","matMenu"],[1,"container"],[3,"openInPopupClicked","modelGraph",
"rendererId","paneId","rootNodeId","inPopup"],[1,"toolbar",3,"in-popup","pushed-down"],[3,"paneId",4,"ngIf"],[1,"toolbar"],[4,"ngIf"],[3,"rendererId","paneId","inPopup"],[1,"vertical-divider"],["dialogLabel","help",1,"mat-icon-container","zoom-fit",3,"click","xapInlineDialog","overlaySize","hoverDelayMs"],[1,"toolbar-icon"],["dialogLabel","help",1,"mat-icon-container","trace-io",3,"click","xapInlineDialog","overlaySize","hoverDelayMs"],[1,"toolbar-icon","trace"],["fontSet","google-material-icons",
1,"toolbar-icon","trace","material-icons"],[3,"curModelGraph","rendererId"],[1,"model-explorer-help-popup"],[1,"shortcut"],["dialogLabel","help",1,"mat-icon-container","expand-all",3,"click","xapInlineDialog","overlaySize","hoverDelayMs"],[1,"toolbar-icon","expand-all"],["dialogLabel","help",1,"mat-icon-container","collapse-all",3,"click","xapInlineDialog","overlaySize","hoverDelayMs"],[1,"toolbar-icon","collapse-all"],["dialogLabel","help",1,"mat-icon-container","flatten-layers",3,"click","xapInlineDialog",
"overlaySize","hoverDelayMs"],[1,"toolbar-icon","flatten-layers"],[1,"description"],["dialogLabel","help",1,"mat-icon-container","download-png",3,"menuOpened","menuClosed","xapInlineDialog","overlaySize","hoverDelayMs","xapInlineDialogDisabled","matMenuTriggerFor"],[1,"toolbar-icon","download-png"],[1,"model-explorer-download-png-menu"],[1,"menu-item",3,"click"],[1,"transparent-background-toggle",3,"click"],["type","checkbox",3,"formControl"],[3,"rendererId","paneId"],[3,"paneId"]],sa:function(a,
b){if(a&1){const c=Zg();R(0,"div",8)(1,"webgl-renderer",9,0);V("openInPopupClicked",function(d){t(c);return x(b.Zq(d))});U();H(3,IP,19,21,"div",10)(4,JP,1,1,"subgraph-breadcrumbs",11);U()}a&2&&(D(),Q("modelGraph",b.modelGraph)("rendererId",b.rendererId)("paneId",b.paneId)("rootNodeId",b.am)("inPopup",b.Rb),D(2),Gq(b.xE?3:-1),D(),Q("ngIf",b.gy))},Ha:[it,dt,BB,$C,CC,QC,YC,PE,iI,hI,Dx,rw,Nw,yx,QO,cP,fP,qP,nO,HG,EG],styles:[".container[_ngcontent-%COMP%]{width:100%;height:100%;overflow:hidden;position:relative}.container[_ngcontent-%COMP%]   svg-rednerer[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   webgl-rednerer[_ngcontent-%COMP%]{width:100%;height:100%}.container[_ngcontent-%COMP%]   .toolbar[_ngcontent-%COMP%]{position:absolute;top:8px;left:12px;padding:0 8px 0 6px;-moz-box-sizing:border-box;box-sizing:border-box;height:40px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;background-color:#edf2fa;border-radius:99px;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;outline:2px solid #fff}.container[_ngcontent-%COMP%]   .toolbar.in-popup[_ngcontent-%COMP%]{padding-left:10px}.container[_ngcontent-%COMP%]   .toolbar[_ngcontent-%COMP%]:has(search-bar  input:focus){border-color:#1a73e8}.container[_ngcontent-%COMP%]   .toolbar[_ngcontent-%COMP%]   search-bar[_ngcontent-%COMP%]{margin-right:4px}.container[_ngcontent-%COMP%]   .toolbar.pushed-down[_ngcontent-%COMP%]{top:44px}.container[_ngcontent-%COMP%]   .toolbar[_ngcontent-%COMP%]   .mat-icon-container[_ngcontent-%COMP%]{width:24px;height:24px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center}.container[_ngcontent-%COMP%]   .toolbar[_ngcontent-%COMP%]   .mat-icon-container.enabled[_ngcontent-%COMP%]{background-color:#ffdeb1;outline:1px solid #ea8600;border-radius:4px}.container[_ngcontent-%COMP%]   .toolbar[_ngcontent-%COMP%]   .mat-icon-container.disable[_ngcontent-%COMP%]{opacity:.2;pointer-events:none}.container[_ngcontent-%COMP%]   .toolbar[_ngcontent-%COMP%]   mat-icon.toolbar-icon[_ngcontent-%COMP%]{font-size:20px;width:20px;height:20px;cursor:pointer;opacity:.6}.container[_ngcontent-%COMP%]   .toolbar[_ngcontent-%COMP%]   mat-icon.toolbar-icon[_ngcontent-%COMP%]:hover{opacity:.9}.container[_ngcontent-%COMP%]   .toolbar[_ngcontent-%COMP%]   mat-icon.toolbar-icon.collapse-all[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   .toolbar[_ngcontent-%COMP%]   mat-icon.toolbar-icon.expand-all[_ngcontent-%COMP%]{font-size:18px;width:18px;height:18px}.container[_ngcontent-%COMP%]   .toolbar[_ngcontent-%COMP%]   mat-icon.toolbar-icon.flatten-layers[_ngcontent-%COMP%]{font-size:20px;width:20px;height:20px;border:1px solid transparent}.container[_ngcontent-%COMP%]   .toolbar[_ngcontent-%COMP%]   mat-icon.toolbar-icon.trace[_ngcontent-%COMP%]{-webkit-transform:rotate(90deg);transform:rotate(90deg);border:1px solid transparent}.container[_ngcontent-%COMP%]   .toolbar[_ngcontent-%COMP%]   .vertical-divider[_ngcontent-%COMP%]{width:1px;border-left:1px solid #d0d0d0;margin:2px 5px;height:20px}.container[_ngcontent-%COMP%]   subgraph-breadcrumbs[_ngcontent-%COMP%]{position:absolute;top:8px;left:12px}  xap-inline-dialog-container:has(.model-explorer-view-popup){border-top-left-radius:0;border-top-right-radius:0}  .model-explorer-download-png-menu .menu-item{letter-spacing:normal!important;font-family:Google Sans Text,Arial,Helvetica,sans-serif!important;font-size:12px;padding:0 8px;cursor:pointer;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}  .model-explorer-download-png-menu .menu-item:hover{background-color:#eee}  .model-explorer-download-png-menu .transparent-background-toggle{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;font-size:12px;letter-spacing:normal;padding:6px 8px 0 4px;border-top:1px solid #ddd;margin-top:8px;cursor:pointer}  .model-explorer-download-png-menu .transparent-background-toggle input{cursor:inherit}"],
Pa:0});const LP=["rendererWrapper"];function MP(a){a.yj=!a.yj;a.yj?(a.F=a.root.R.style.height,a.root.R.style.height="26px"):a.root.R.style.height=a.F}function NP(a,b){b.stopPropagation();2!==b.button&&AO(new BO(a.root.R),b)}
var OP=class{constructor(a,b){this.C=a;this.root=b;this.mv=new Jk;this.selected=Cs(()=>{let c,d;return(null==(c=this.C.G())?void 0:c.id)===(null==(d=this.Yl)?void 0:d.rendererId)&&this.C.J()===this.paneId});this.yj=!1;this.F="";this.D=()=>{if(this.Yl){this.C.G.set({id:this.Yl.rendererId,Tr:1});var c;null==(c=this.Yl.bt)?c=void 0:c=c.selectedNodeId?{nodeId:c.selectedNodeId,rendererId:c.rendererId,Cd:oI(c.ga.nodesById[c.selectedNodeId])}:void 0;wJ(this.C,this.paneId,c);jJ(this.C,this.paneId)}}}Fb(){this.root.R.addEventListener("mousedown",
this.D,!0);const a=this.root.R;a.style.left=`${this.initialPosition.x}px`;a.style.top=`${this.initialPosition.y}px`;a.style.width="400px";a.style.height="400px"}va(){this.root.R.removeEventListener("mousedown",this.D,!0)}get title(){return this.Kn.label}};OP.K=function(a){return new (a||OP)(y(VJ),y(wi))};
OP.Ca=Jf({type:OP,ha:[["popup-panel"]],eb:function(a,b){a&1&&vr(LP,5);if(a&2){let c;wr(c=xr())&&(b.Yl=c.first)}},inputs:{id:"id",paneId:"paneId",Kn:"groupNode",initialPosition:"initialPosition",ga:"curModelGraph"},outputs:{mv:"closeClicked"},ka:!0,features:[Nr],ya:22,za:13,Ba:[["rendererWrapper",""],[1,"container",3,"mousedown","mouseup"],[1,"title",3,"dblclick"],[1,"label"],[1,"icons-container"],[1,"mat-icon-container","minimize",3,"click"],[1,"mat-icon-container","close",3,"click"],[3,"modelGraph",
"rendererId","paneId","rootNodeId","inPopup"],[1,"resize-box",3,"mousedown","mouseup"],["data-position","top"],["data-position","bottom"],["data-position","left"],["data-position","right"],["data-position","top-right"],["data-position","top-left"],["data-position","bottom-right"],["data-position","bottom-left"]],sa:function(a,b){if(a&1){const c=Zg();R(0,"div",1);V("mousedown",function(d){t(c);return x(NP(b,d))})("mouseup",function(){t(c);return x()});R(1,"div",2);V("dblclick",function(){t(c);return x(MP(b))});
R(2,"div",3);Z(3);U();R(4,"div",4)(5,"div",5);V("click",function(){t(c);return x(MP(b))});R(6,"mat-icon");Z(7);U()();R(8,"div",6);V("click",function(){t(c);return x(b.mv.emit(b.id))});R(9,"mat-icon");Z(10,"close");U()()()();Oq(11,"renderer-wrapper",7,0);U();R(13,"div",8);V("mousedown",function(d){t(c);return x(NP(b,d))})("mouseup",function(){t(c);return x()});Oq(14,"div",9)(15,"div",10)(16,"div",11)(17,"div",12)(18,"div",13)(19,"div",14)(20,"div",15)(21,"div",16);U()}a&2&&(Wp("selected",b.selected())("minimized",
b.yj),D(3),Br(b.title),D(4),Br(b.yj?"unfold_more":"unfold_less"),D(4),Q("modelGraph",b.ga)("rendererId",b.id)("paneId",b.paneId)("rootNodeId",b.Kn.id)("inPopup",!0),D(2),Wp("disabled",b.yj))},Ha:[iI,hI,KP],styles:["[_nghost-%COMP%]{position:absolute;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}.container[_ngcontent-%COMP%]{width:100%;height:100%;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;font-size:13px;outline:2px solid transparent;position:relative;background-color:#fff;border-radius:4px;overflow:hidden;border:2px solid #aaa;cursor:pointer}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]{height:27px;border-bottom:1px solid #ccc;-moz-box-sizing:border-box;box-sizing:border-box;background-color:#e7f3ff;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;-webkit-flex-shrink:0;-ms-flex-negative:0;flex-shrink:0;overflow:hidden;border-top-left-radius:2px;border-top-right-radius:2px;white-space:nowrap}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   .label[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;overflow:hidden;text-overflow:ellipsis;-moz-box-sizing:border-box;box-sizing:border-box;padding-left:8px;padding-right:4px}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   .icons-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   .mat-icon-container[_ngcontent-%COMP%]{width:24px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;opacity:.6;cursor:pointer}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   .mat-icon-container[_ngcontent-%COMP%]:hover{opacity:1}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   .mat-icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:16px;font-weight:700;width:16px;height:16px}.container[_ngcontent-%COMP%]   renderer-wrapper[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;min-height:0}.container.selected[_ngcontent-%COMP%]{box-shadow:0 4px 6px -1px rgba(0,0,0,.1),0 2px 4px -2px rgba(0,0,0,.1)}.container.selected[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]{background-color:#ea8600;color:#fff}.container.minimized[_ngcontent-%COMP%]   svg-renderer[_ngcontent-%COMP%]{display:none}.resize-box[_ngcontent-%COMP%]{position:absolute;inset:-4px;pointer-events:none;z-index:100}.resize-box.disabled[_ngcontent-%COMP%]   [data-position][_ngcontent-%COMP%]{pointer-events:none}.resize-box[_ngcontent-%COMP%]   div[_ngcontent-%COMP%]{position:absolute;pointer-events:all;background-color:transparent}.resize-box[_ngcontent-%COMP%]   [data-position=top][_ngcontent-%COMP%]{top:0;width:100%;height:8px;cursor:ns-resize}.resize-box[_ngcontent-%COMP%]   [data-position=bottom][_ngcontent-%COMP%]{bottom:0;width:100%;height:8px;cursor:ns-resize}.resize-box[_ngcontent-%COMP%]   [data-position=left][_ngcontent-%COMP%]{left:0;height:100%;width:8px;cursor:ew-resize}.resize-box[_ngcontent-%COMP%]   [data-position=right][_ngcontent-%COMP%]{right:0;height:100%;width:8px;cursor:ew-resize}.resize-box[_ngcontent-%COMP%]   [data-position=top-right][_ngcontent-%COMP%]{top:0;right:0;height:8px;width:8px;cursor:nesw-resize}.resize-box[_ngcontent-%COMP%]   [data-position=top-left][_ngcontent-%COMP%]{top:0;left:0;height:8px;width:8px;cursor:nwse-resize}.resize-box[_ngcontent-%COMP%]   [data-position=bottom-left][_ngcontent-%COMP%]{bottom:0;left:0;height:8px;width:8px;cursor:nesw-resize}.resize-box[_ngcontent-%COMP%]   [data-position=bottom-right][_ngcontent-%COMP%]{bottom:0;right:0;height:8px;width:8px;cursor:nwse-resize}"],
Pa:0});function PP(a){if(a&1){const b=Zg();R(0,"div",0)(1,"div",1);Z(2," Selection ");U();R(3,"div",2);Z(4);R(5,"div",3);V("click",function(){t(b);Y().rm.C.set({});return x()});Z(6," Clear ");U()();R(7,"div",4)(8,"button",5);V("click",function(){t(b);var c=Y(),d=DM(c.rm);d&&(c=document.createElement("a"),c.download=`${d.collectionLabel}_subgraph.json`,d=`data:text/json;charset=utf-8, ${encodeURIComponent(JSON.stringify([d],null,2))}`,wd(c,td(d)),c.click());return x()});Z(9," Download subgraph ");U()()()}a&
2&&(a=Y(),D(4),Cr(" ",a.dE()," "))}var QP=class{constructor(a){this.rm=a;this.br=this.rm.br;this.dE=Cs(()=>{const b=this.rm.F();return`${b} node${1===b?"":"s"}`})}};QP.K=function(a){return new (a||QP)(y(EM))};
QP.Ca=Jf({type:QP,ha:[["selection-panel"]],inputs:{paneId:"paneId"},ka:!0,features:[Nr],ya:1,za:1,Ba:[[1,"container"],[1,"title"],[1,"content"],[1,"link-clear",3,"click"],[1,"btns-container"],["mat-flat-button","","color","primary",3,"click"]],sa:function(a,b){a&1&&H(0,PP,10,1,"div",0);a&2&&Gq(b.br()?0:-1)},Ha:[it,BB,AB,vB],styles:[".container[_ngcontent-%COMP%]{font-size:12px;border:2px solid #09b83e;overflow:hidden;border-radius:6px;background-color:#fff}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]{padding:6px 8px;background-color:#09b83e;color:#fff;font-weight:500}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]{padding:16px 8px 12px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .link-clear[_ngcontent-%COMP%]{text-decoration:underline;color:#00639b;cursor:pointer;opacity:.8;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .link-clear[_ngcontent-%COMP%]:hover{opacity:1}.container[_ngcontent-%COMP%]   .btns-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;padding:8px}.container[_ngcontent-%COMP%]   .btns-container[_ngcontent-%COMP%]   button[_ngcontent-%COMP%]{letter-spacing:normal;width:100%}"],
Pa:0});const RP=["popupPanel"];function SP(a){if(a&1){const b=Zg();R(0,"renderer-wrapper",5);V("openInPopupClicked",function(c){t(b);const d=Y();return x(d.Zq(c))});U()}a&2&&(a=Y(),Q("modelGraph",a.modelGraph)("rendererId",a.rendererId)("paneId",a.paneId))}
function TP(a,b){if(a&1){const c=Zg();R(0,"popup-panel",6,0);V("closeClicked",function(d){t(c);const e=Y();return x(UP(e,d))});U()}a&2&&(a=b.da,b=Y(),Q("id",a.id)("paneId",b.paneId)("groupNode",a.Kn)("initialPosition",a.initialPosition)("curModelGraph",a.ga))}function VP(a){a&1&&Oq(0,"legends-panel",4);a&2&&(a=Y(),Q("paneId",a.paneId))}function UP(a,b){const c=a.yo.findIndex(d=>d.id===b);0<=c&&a.yo.splice(c,1)}
var WP=class{constructor(a,b,c,d){this.root=a;this.C=b;this.F=c;this.J=d;this.Xr=new cr;this.yo=[];this.Es=!0;this.G=e=>{null==e.target.closest("popup-panel")&&this.C.G.set({id:this.rendererId,Tr:2})};Ks(()=>{const e=this.C.G();var f=this.root.R.querySelector("popup-panel.top");f&&f.classList.remove("top");(f=this.Xr.find(g=>{let h;return(null==(h=g.Yl)?void 0:h.rendererId)===(null==e?void 0:e.id)}))&&f.root.R.classList.add("top")})}Fb(){this.root.R.addEventListener("mousedown",this.G,!0);this.J.paneId=
this.paneId}uc(a){a.modelGraph&&this.modelGraph&&(this.J.C.set({}),this.Es=!1,fn(this.F),this.Es=!0,fn(this.F))}pd(){const a=this.root.R;this.D=new ResizeObserver(()=>{this.handleResize()});this.D.observe(a)}va(){this.root.R.removeEventListener("mousedown",this.G,!0);this.D&&this.D.unobserve(this.root.R)}OE(a,b){return b.id}Zq(a){this.yo.push(a)}get pE(){let a;return!(null==(a=this.C.config())?0:a.hideLegends)}handleResize(){const a=this.root.R;for(const b of this.Xr){const c=b.root.R;if(!c)continue;
const d=Number(c.style.width.replace("px","")),e=Number(c.style.height.replace("px","")),f=Number(c.style.top.replace("px",""))+e;Number(c.style.left.replace("px",""))+d>a.offsetWidth&&(c.style.left=`${a.offsetWidth-d}px`);f>a.offsetHeight&&(c.style.top=`${a.offsetHeight-e}px`)}}};WP.K=function(a){return new (a||WP)(y(wi),y(VJ),y(Tr),y(EM))};
WP.Ca=Jf({type:WP,ha:[["graph-panel"]],eb:function(a,b){a&1&&vr(RP,5);if(a&2){let c;wr(c=xr())&&(b.Xr=c)}},inputs:{modelGraph:"modelGraph",rendererId:"rendererId",paneId:"paneId"},ka:!0,features:[Hg,Nr],ya:5,za:5,Ba:[["popupPanel",""],[1,"container"],[3,"modelGraph","rendererId","paneId","openInPopupClicked",4,"ngIf"],[3,"id","paneId","groupNode","initialPosition","curModelGraph","closeClicked",4,"ngFor","ngForOf","ngForTrackBy"],[3,"paneId"],[3,"openInPopupClicked","modelGraph","rendererId","paneId"],
[3,"closeClicked","id","paneId","groupNode","initialPosition","curModelGraph"]],sa:function(a,b){a&1&&(R(0,"div",1),H(1,SP,1,3,"renderer-wrapper",2)(2,TP,2,5,"popup-panel",3)(3,VP,1,1,"legends-panel",4),Oq(4,"selection-panel",4),U());a&2&&(D(),Q("ngIf",b.Es),D(),Q("ngForOf",b.yo)("ngForTrackBy",b.OE),D(),Gq(b.pE?3:-1),D(),Q("paneId",b.paneId))},Ha:[it,$s,dt,zO,OP,KP,QP],styles:[".container[_ngcontent-%COMP%]{width:100%;height:100%;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;position:relative;overflow:hidden}.container[_ngcontent-%COMP%]   renderer-wrapper[_ngcontent-%COMP%]{width:100%;height:100%}.container[_ngcontent-%COMP%]   popup-panel.top[_ngcontent-%COMP%]{z-index:100}.container[_ngcontent-%COMP%]   legends-panel[_ngcontent-%COMP%]{position:absolute;left:8px;bottom:8px;border-radius:6px;border:1px solid #ddd;overflow:hidden}.container[_ngcontent-%COMP%]   selection-panel[_ngcontent-%COMP%]{position:absolute;right:8px;top:8px;overflow:hidden}"],
Pa:0});const XP=["container"],YP=["oneLineText"];function ZP(a){a&1&&(R(0,"div",7),Z(1," Showing at most "),R(2,"span",8),Z(3),U()());a&2&&(a=Y(2),D(2),Q("matTooltipShowDelay",300),D(),Cr(" ",a.ml()," elements "))}function $P(a){a&1&&(H(0,ZP,4,2,"div",7),R(1,"div",4),Z(2),U());a&2&&(a=Y(),Gq(0!==a.ml()?0:-1),D(2),Br(a.text))}function aQ(a,b){a&1&&(R(0,"div",10)(1,"div",11),Z(2),U(),R(3,"div",12),Z(4),U()());a&2&&(a=b.da,b=b.Cb,D(2),Br(b),D(2),Br(a))}
function bQ(a){a&1&&(R(0,"div",7),Z(1," Namespace hierarchy "),U(),R(2,"div",9),Lq(3,aQ,5,2,"div",10,Iq),U());a&2&&(a=Y(),D(3),Nq(a.iD))}function cQ(a){a&1&&(R(0,"div",4),Z(1),U());a&2&&(a=Y(),D(),Br(a.hC))}function dQ(a){a&1&&Z(0);a&2&&(a=Y(),Cr(" ",a.text," "))}function eQ(a){a.Pr?(a.C=a.Pr.R.scrollWidth>a.Pr.R.offsetWidth,!a.expanded||"namespace"!==a.type&&"values"!==a.type||(a.C=!0)):a.C=!1}function fQ(a,b,c=!1){a.pw&&(b.stopPropagation(),c&&a.expanded||(a.expanded=!a.expanded))}
var gQ=class{constructor(a,b){this.G=a;this.D=b;this.type=this.text="";this.bgColor="transparent";this.textColor="inherit";this.C=this.expanded=!1}get xC(){return this.expanded}pd(){setTimeout(()=>{eQ(this);en(this.D)});this.ab&&(this.F=new ResizeObserver(()=>{eQ(this);en(this.D)}),this.F.observe(this.ab.R))}uc(){setTimeout(()=>{eQ(this);en(this.D)})}va(){this.F&&this.F.disconnect()}ml(){let a,b;return null!=(b=null==(a=this.G.config())?void 0:a.maxConstValueCount)?b:0}get pw(){eQ(this);return this.C}get yC(){return this.expanded?
"unfold_less":"unfold_more"}get oC(){return"transparent"!==this.bgColor}get iD(){const a=this.text.split("/");"<root>"!==this.text&&a.unshift("<root>");return a}get hC(){return this.text.replace("[","").replace("]","").split(",").map(a=>a.trim()).join("\n")}};gQ.K=function(a){return new (a||gQ)(y(VJ),y(Tr))};
gQ.Ca=Jf({type:gQ,ha:[["expandable-info-text"]],eb:function(a,b){a&1&&(vr(XP,5),vr(YP,5));if(a&2){let c;wr(c=xr())&&(b.ab=c.first);wr(c=xr())&&(b.Pr=c.first)}},Za:2,Ia:function(a,b){a&2&&Wp("expanded",b.xC)},inputs:{text:"text",type:"type",bgColor:"bgColor",textColor:"textColor"},ka:!0,features:[Hg,Nr],ya:13,za:13,Ba:[["container",""],["oneLineText",""],[1,"container",3,"click"],[1,"expanded-text"],[1,"monospace-content"],[1,"one-line-text"],[1,"icon-container","toggle",3,"click"],[1,"info-msg"],
["matTooltip","configurable in Settings in homepage","matTooltipPosition","above",1,"underline",3,"matTooltipShowDelay"],[1,"namespace-content"],[1,"namespace-row"],[1,"index"],[1,"namespace"]],sa:function(a,b){if(a&1){const c=Zg();R(0,"div",2,0);V("click",function(d){t(c);return x(fQ(b,d,!0))});R(2,"div",3);H(3,$P,3,2,"div",4)(4,bQ,5,0)(5,cQ,2,1,"div",4)(6,dQ,1,1);U();R(7,"div",5,1);Z(9);U();R(10,"div",6);V("click",function(d){t(c);return x(fQ(b,d))});R(11,"mat-icon");Z(12);U()()()}a&2&&(Up("background-color",
b.bgColor)("color",b.textColor),Wp("has-bg-color",b.oC)("has-overflow",b.pw)("expanded",b.expanded),D(3),Gq("values"===b.type?3:"namespace"===b.type?4:"quantization"===b.type?5:6),D(6),Cr(" ",b.text," "),D(3),Br(b.yC))},Ha:[it,PE,AE,NE,iI,hI],styles:["[_nghost-%COMP%]{overflow:hidden}.container[_ngcontent-%COMP%]{overflow:hidden;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:start;-webkit-align-items:flex-start;-moz-box-align:start;-ms-flex-align:start;align-items:flex-start;position:relative}.container.has-overflow[_ngcontent-%COMP%]{cursor:pointer}.container.has-overflow[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex}.container.has-overflow[_ngcontent-%COMP%]:hover   .icon-container[_ngcontent-%COMP%]{opacity:1}.container.has-bg-color[_ngcontent-%COMP%]{border-radius:4px;padding-left:4px}.container.expanded[_ngcontent-%COMP%]   .one-line-text[_ngcontent-%COMP%]{position:absolute;inset:0;visibility:hidden}.container.expanded[_ngcontent-%COMP%]   .expanded-text[_ngcontent-%COMP%]{display:block;-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;cursor:text}.container.expanded[_ngcontent-%COMP%]   .expanded-text[_ngcontent-%COMP%]   .info-msg[_ngcontent-%COMP%]{margin-bottom:2px;color:#999;font-size:12px}.container.expanded[_ngcontent-%COMP%]   .expanded-text[_ngcontent-%COMP%]   .info-msg[_ngcontent-%COMP%]   .underline[_ngcontent-%COMP%]{-webkit-text-decoration:underline dashed;-moz-text-decoration:underline dashed;text-decoration:underline dashed}.container.expanded[_ngcontent-%COMP%]   .expanded-text[_ngcontent-%COMP%]   .monospace-content[_ngcontent-%COMP%]{font-size:11px;white-space:pre-wrap;font-family:monospace;max-height:500px;overflow:auto}.container.expanded[_ngcontent-%COMP%]   .expanded-text[_ngcontent-%COMP%]   .namespace-content[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;gap:2px}.container.expanded[_ngcontent-%COMP%]   .expanded-text[_ngcontent-%COMP%]   .namespace-row[_ngcontent-%COMP%]{font-size:12px;line-height:14px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:start;-webkit-align-items:flex-start;-moz-box-align:start;-ms-flex-align:start;align-items:flex-start}.container.expanded[_ngcontent-%COMP%]   .expanded-text[_ngcontent-%COMP%]   .namespace-row[_ngcontent-%COMP%]   .index[_ngcontent-%COMP%]{width:16px;color:#999;-webkit-flex-shrink:0;-ms-flex-negative:0;flex-shrink:0;line-height:14px}.container.expanded[_ngcontent-%COMP%]   .expanded-text[_ngcontent-%COMP%]   .namespace-row[_ngcontent-%COMP%]   .namespace[_ngcontent-%COMP%]{line-height:14px}.container.expanded[_ngcontent-%COMP%]   .expanded-text[_ngcontent-%COMP%]   .namespace-row[_ngcontent-%COMP%]   .namespace.hidden[_ngcontent-%COMP%]{color:#999}.container.expanded[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{background-color:#eee;border:1px solid #999}.container.expanded[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{color:#999}.container[_ngcontent-%COMP%]   .one-line-text[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.container[_ngcontent-%COMP%]   .expanded-text[_ngcontent-%COMP%]{display:none}.container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{width:12px;height:12px;-webkit-flex-shrink:0;-ms-flex-negative:0;flex-shrink:0;display:none;background-color:#d7f1ff;border-radius:2px;margin-left:2px;cursor:pointer;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;border:1px solid #4e90b5;opacity:.75}.container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{color:#00639b;font-size:11px;width:11px;height:11px;font-weight:700}"],
Pa:0});const hQ=["container"];function iQ(a){a&1&&(R(0,"div",3),Z(1),U());a&2&&(a=Y(),D(),Cr(" ",a.label," "))}var jQ=class{constructor(a){this.C=a;this.label="";this.ey=!1;this.Yr={minWidth:0,minHeight:0};this.MD=[{bb:"end",cb:"top",Na:"end",Ua:"top",offsetY:-1}]}pd(){setTimeout(()=>{const a=this.ab.R;this.ey=a.scrollWidth>a.offsetWidth;en(this.C)})}};jQ.K=function(a){return new (a||jQ)(y(Tr))};
jQ.Ca=Jf({type:jQ,ha:[["hoverable-label"]],eb:function(a,b){a&1&&vr(hQ,5);if(a&2){let c;wr(c=xr())&&(b.ab=c.first)}},inputs:{label:"label"},ka:!0,features:[Nr],ya:5,za:6,Ba:[["container",""],["popup",""],["dialogLabel","popup",1,"container",3,"xapInlineDialog","overlaySize","overlayPositions","hoverDelayMs","xapInlineDialogDisabled"],[1,"model-explorer-hoverable-label-popup"]],sa:function(a,b){a&1&&(R(0,"div",2,0),Z(2),U(),H(3,iQ,2,1,"ng-template",null,1,Rr));a&2&&(a=Ar(4),Q("xapInlineDialog",a)("overlaySize",
b.Yr)("overlayPositions",b.MD)("hoverDelayMs",10)("xapInlineDialogDisabled",!b.ey),D(2),Cr(" ",b.label,"\n"))},Ha:[HG,EG],styles:["[_nghost-%COMP%] {\n  overflow: hidden;\n}\n\n.container[_ngcontent-%COMP%] {\n  overflow: hidden;\n  text-overflow: ellipsis;\n}\n\n  xap-inline-dialog-container:has(.model-explorer-hoverable-label-popup) {\n  width: 100%;\n  box-shadow: none;\n  border: 1px solid #ccc;\n  border-radius: 4px;\n}\n\n  .model-explorer-hoverable-label-popup {\n  padding: 2px;\n  font-size: 12px;\n  line-height: 12px;\n  background-color: white;\n  color: #999;\n  font-family: 'Google Sans Text', Arial, Helvetica, sans-serif;\n}"],
Pa:0});var kQ=class{constructor(){this.Be=-2;this.wg="asc";this.Ae=-2;this.ug="asc";this.Uf=this.Jf=this.Ug=!1;this.C=new Set}};kQ.K=function(a){return new (a||kQ)};kQ.na=Pd({ma:kQ,ba:kQ.K});const lQ=["paginator"],mQ=["childrenStatsPaginator"];function nQ(a){a&1&&(R(0,"div",26),Z(1),U());a&2&&(a=Y().index,D(),Br(a+1))}function oQ(a){a&1&&Oq(0,"mat-spinner",27)}function pQ(a){a&1&&(R(0,"div",28)(1,"mat-icon"),Z(2,"error"),U()());a&2&&(a=Y().da,Q("matTooltip",a.error)("matTooltipDisabled",!a.error))}
function qQ(a){if(a&1){const b=Zg();R(0,"div",29);V("click",function(c){t(b);const d=Y().da,e=Y();c.stopPropagation();d.runId===e.Mf||BJ(e.xi,e.paneId,d.runId);return x()});R(1,"mat-icon");Z(2);U()()}if(a&2){a=Y().da;const b=Y();Wp("selected",a.runId===b.Mf)("hide",!a.done);Q("matTooltip",rQ(b,a));D(2);Br(a.runId===b.Mf?"visibility":"visibility_off")}}
function sQ(a){if(a&1){const b=Zg();R(0,"div",30);V("click",function(){t(b);const c=Y().da,d=Y();d.ga&&(GL(d.hk,c.runId),TJ(d.xi,[`${"Node data provider: "}${c.runName}`]));return x()});R(1,"mat-icon");Z(2,"delete");U()()}a&2&&Q("matTooltip","Delete")}
function tQ(a,b){a&1&&(R(0,"div",17)(1,"div",18),H(2,nQ,2,1,"div",19)(3,oQ,1,0,"mat-spinner",20),U(),R(4,"div",21),Z(5),U(),R(6,"div",22),H(7,pQ,3,2,"div",23)(8,qQ,3,6,"div",24)(9,sQ,3,1,"div",25),U()());a&2&&(a=b.da,b=Y(),Wp("selected",a.runId===b.Mf),D(2),Q("ngIf",a.done),D(),Q("ngIf",!a.done),D(2),Br(a.runName),D(2),Q("ngIf",a.error),D(),Q("ngIf",!a.error),D(),Q("ngIf",a.done))}function uQ(a,b){a&1&&(R(0,"th",35)(1,"div",36)(2,"div",26),Z(3),U()()());a&2&&(a=b.index,D(3),Br(a+1))}
function vQ(a,b){a&1&&(R(0,"td",35),Z(1),U());a&2&&(a=b.da,Y(3),D(),Cr(" ",a===Number.POSITIVE_INFINITY||a===Number.NEGATIVE_INFINITY||isNaN(a)?"-":`${a}`," "))}function wQ(a,b){a&1&&(R(0,"tr")(1,"td"),Z(2),U(),H(3,vQ,2,1,"td",33),U());a&2&&(a=b.da,D(2),Br(a.stat),D(),Q("ngForOf",a.values))}
function xQ(a){a&1&&(R(0,"table",31)(1,"thead")(2,"tr")(3,"th",32),Z(4," Stat "),U(),H(5,uQ,4,1,"th",33),U()(),R(6,"tbody"),H(7,wQ,4,2,"tr",34),U()());a&2&&(a=Y(),D(5),Q("ngForOf",a.Tj),D(2),Q("ngForOf",a.hj)("ngForTrackBy",a.SE))}function yQ(a){if(a&1){const b=Zg();R(0,"paginator",37,3);V("change",function(c){t(b);const d=Y();return x(zQ(d,c))});U()}a&2&&(a=Y(),Q("pageSize",a.dg)("itemsCount",a.kv))}
function AQ(a){a&1&&(R(0,"mat-icon",43),Z(1),U());a&2&&(a=Y(2),D(),Cr(" ","asc"===a.ug?"arrow_upward":"arrow_downward"," "))}function BQ(a){a&1&&(R(0,"mat-icon",43),Z(1),U());a&2&&(a=Y(2),D(),Cr(" ","asc"===a.ug?"arrow_upward":"arrow_downward"," "))}function CQ(a){a&1&&(R(0,"mat-icon",43),Z(1),U());a&2&&(a=Y(3),D(),Cr(" ","asc"===a.ug?"arrow_upward":"arrow_downward"," "))}
function DQ(a,b){if(a&1){const c=Zg();R(0,"th",44);V("click",function(){const d=t(c).da,e=Y(2);return x(EQ(e,d.qv))});R(1,"div",36)(2,"div",26);Z(3);U();R(4,"div",45);Z(5);U();H(6,CQ,2,1,"mat-icon",40);U()()}a&2&&(a=b.da,b=Y(2),D(3),Br(a.YD+1),D(2),Br(a.label),D(),Q("ngIf",a.qv===b.Ae))}function FQ(a,b){a&1&&(R(0,"td",35),Z(1),U());a&2&&(a=b.da,D(),Cr(" ",a," "))}
function GQ(a,b){if(a&1){const c=Zg();R(0,"tr")(1,"td");Z(2);U();R(3,"td",46);V("click",function(){const d=t(c).da,e=Y(2);return x(HQ(e,d.id))});Z(4);U();H(5,FQ,2,1,"td",33);U()}a&2&&(a=b.da,D(2),Br(a.index),D(2),Cr(" ",a.label," "),D(),Q("ngForOf",a.yB))}
function IQ(a){if(a&1){const b=Zg();R(0,"table",38)(1,"thead")(2,"tr")(3,"th",39);V("click",function(){t(b);const c=Y();return x(EQ(c,-2))});R(4,"div",36);Z(5," # ");H(6,AQ,2,1,"mat-icon",40);U()();R(7,"th",41);V("click",function(){t(b);const c=Y();return x(EQ(c,-1))});R(8,"div",36);Z(9," Node ");H(10,BQ,2,1,"mat-icon",40);U()();H(11,DQ,7,3,"th",42);U()();R(12,"tbody");H(13,GQ,6,3,"tr",34);U()()}a&2&&(a=Y(),D(6),Q("ngIf",-2===a.Ae),D(4),Q("ngIf",-1===a.Ae),D(),Q("ngForOf",a.hn),D(2),Q("ngForOf",a.Gv)("ngForTrackBy",
a.Cy))}function JQ(a){if(a&1){const b=Zg();R(0,"paginator",37,4);V("change",function(c){t(b);const d=Y();return x(KQ(d,c))});U()}a&2&&(a=Y(),Q("pageSize",a.dg)("itemsCount",a.Lx))}function LQ(a){a&1&&(R(0,"mat-icon",43),Z(1),U());a&2&&(a=Y(2),D(),Cr(" ","asc"===a.wg?"arrow_upward":"arrow_downward"," "))}function MQ(a){a&1&&(R(0,"mat-icon",43),Z(1),U());a&2&&(a=Y(2),D(),Cr(" ","asc"===a.wg?"arrow_upward":"arrow_downward"," "))}
function NQ(a){if(a&1){const b=Zg();R(0,"div",49);V("click",function(c){t(b);const d=Y().da,e=Y(2);c.stopPropagation();d.runId===e.Mf||BJ(e.xi,e.paneId,d.runId);return x()});R(1,"mat-icon");Z(2);U()()}if(a&2){a=Y().da;const b=Y(2);Wp("selected",a.runId===b.Mf)("hide",!a.done);Q("matTooltip",rQ(b,a));D(2);Br(a.runId===b.Mf?"visibility":"visibility_off")}}function OQ(a){a&1&&(R(0,"mat-icon",43),Z(1),U());a&2&&(a=Y(3),D(),Cr(" ","asc"===a.wg?"arrow_upward":"arrow_downward"," "))}
function PQ(a,b){if(a&1){const c=Zg();R(0,"th",44);V("click",function(){const d=t(c).index,e=Y(2);return x(QQ(e,d))});R(1,"div",36)(2,"div",26);Z(3);U();H(4,NQ,3,6,"div",48)(5,OQ,2,1,"mat-icon",40);U()()}if(a&2){a=b.da;b=b.index;const c=Y(2);D(3);Br(b+1);D();Q("ngIf",!a.error);D();Q("ngIf",b===c.Be)}}function RQ(a,b){a&1&&(R(0,"td",35),Z(1),U());a&2&&(a=b.da,Up("background-color",a.bgColor)("color",a.textColor),D(),Cr(" ",a.strValue," "))}
function SQ(a,b){if(a&1){const c=Zg();R(0,"tr")(1,"td");Z(2);U();R(3,"td",46);V("click",function(){const d=t(c).da,e=Y(2);return x(HQ(e,d.id))});Z(4);U();H(5,RQ,2,5,"td",50);U()}a&2&&(a=b.da,D(2),Br(a.index),D(),Wp("input",a.MC)("output",a.OC),D(),Cr(" ",a.label," "),D(),Q("ngForOf",a.cols))}
function TQ(a){if(a&1){const b=Zg();R(0,"table",47)(1,"thead")(2,"tr")(3,"th",39);V("click",function(){t(b);const c=Y();return x(QQ(c,-2))});R(4,"div",36);Z(5," # ");H(6,LQ,2,1,"mat-icon",40);U()();R(7,"th",41);V("click",function(){t(b);const c=Y();return x(QQ(c,-1))});R(8,"div",36);Z(9," Node ");H(10,MQ,2,1,"mat-icon",40);U()();H(11,PQ,6,3,"th",42);U()();R(12,"tbody");H(13,SQ,6,7,"tr",34);U()()}a&2&&(a=Y(),D(6),Q("ngIf",-2===a.Be),D(4),Q("ngIf",-1===a.Be),D(),Q("ngForOf",a.Tj),D(2),Q("ngForOf",a.Hv)("ngForTrackBy",
a.Cy))}const UQ=["Sum %"];function rQ(a,b){return b.runId===a.Mf?"Visualizing in graph":"Click to visualize in graph"}function zQ(a,b){a.Gv=a.G.slice(b*a.dg,(b+1)*a.dg);en(a.D)}function EQ(a,b){a.C.ug=a.C.Ae===b?VQ(a.C.ug):0>b?"asc":"desc";a.C.Ae=b;WQ(a);let c;null==(c=a.mq)||c.reset();zQ(a,0)}function HQ(a,b){a.xi.F.set({nodeId:b,rendererId:a.paneId,Cd:!1})}function KQ(a,b){a.Hv=null==a.F?[]:a.F.slice(b*a.dg,(b+1)*a.dg);en(a.D)}
function QQ(a,b){a.C.wg=a.C.Be===b?VQ(a.wg):0>b?"asc":"desc";a.C.Be=b;XQ(a);let c;null==(c=a.Ur)||c.reset();KQ(a,0)}function YQ(a){return a.map(b=>{const c=[];c.push(b.runId);c.push(String(b.done));c.push(String(Object.keys(b.results||{}).length));return c.join("__")}).join(",")}
function ZQ(a){if(a.ga){var b=$Q(a),c=a.N[b];if(null!=c)a.J=c;else{const d=null==a.Qg?void 0:a.ga.nodesById[a.Qg];let e=new Set;null!=d&&(e=new Set(d.descendantsOpNodeIds||[]));a.J=a.ga.nodes.filter(f=>nI(f)&&!f.hideInLayout&&"GraphInputs"!==f.id&&"GraphOutputs"!==f.id&&(null==d||e.has(f.id)));a.N[b]=a.J}}}
function aR(a){if(a.ga&&0!==a.J.length){var b=wL(a.hk,a.ga);a.hj=[{stat:"Min",values:[]},{stat:"Max",values:[]},{stat:"Sum",values:[]},{stat:"Avg",values:[]}];var c=[];for(var d=0;d<b.length;d++)c.push({min:Number.POSITIVE_INFINITY,max:Number.NEGATIVE_INFINITY,oi:0,count:0});a.F=[];for(d=0;d<a.J.length;d++){var e=a.J[d],f=e.id,g=[];for(var h=0;h<b.length;h++){var m=((b[h].results||{})[a.ga.id]||{})[f],p=void 0,u=null==(p=m)?void 0:p.value,w=void 0,B=(null==(w=m)?void 0:w.strValue)||"-",G=void 0;const ba=
(null==(G=m)?void 0:G.bgColor)||"";var L=void 0;m=(null==(L=m)?void 0:L.textColor)||"black";g.push({value:u,strValue:B,bgColor:ba,textColor:m});null!=u&&"number"===typeof u&&(B=c[h],B.min=Math.min(u,B.min),B.max=Math.max(u,B.max),B.count++,B.oi+=u)}h=e.incomingEdges||[];h=0===h.length||h.some(ba=>"GraphInputs"===ba.sourceNodeId);e=e.outgoingEdges||[];e=0===e.length||e.some(ba=>"GraphOutputs"===ba.targetNodeId);a.F.push({id:f,index:d,MC:h,OC:e,label:a.ga.nodesById[f].label||"?",cols:g})}a.aa=[...a.F];
XQ(a);KQ(a,0);a.hj[0].values=c.map(ba=>ba.min);a.hj[1].values=c.map(ba=>ba.max);a.hj[2].values=c.map(ba=>ba.oi);a.hj[3].values=c.map(ba=>ba.oi/ba.count);a.hn=[];p=0;for(w=0;w<b.length;w++)for(var O of UQ)a.hn.push({qv:p,YD:w,label:O}),p++;a.G=[];O=a.Qg?a.ga.nodesById[a.Qg].nsChildrenIds||[]:a.ga.rootNodes.map(ba=>ba.id);for(p=0;p<O.length;p++){w=O[p];G=a.ga.nodesById[w];L=[];d=[];for(f=0;f<b.length;f++){e=b[f].results||{};h=0;g=!1;if(nI(G)){let ba;e=null==(ba=(e[a.ga.id]||{})[w])?void 0:ba.value;
null!=e&&"number"===typeof e&&(h=e/c[f].oi*100,g=!0)}else if(oI(G)){h=0;u=G.descendantsOpNodeIds||[];for(const ba of u){let ca;u=null==(ca=(e[a.ga.id]||{})[ba])?void 0:ca.value;null!=u&&"number"===typeof u&&(h+=u,g=!0)}h=h/c[f].oi*100}L.push(h);d.push(g?h.toFixed(1):"-")}a.G.push({id:w,label:G.label,index:p,zB:L,yB:d})}a.P=[...a.G];WQ(a);zQ(a,0);en(a.D)}}function VQ(a){switch(a){case "desc":return"asc";case "asc":return"desc";default:return a}}
function XQ(a){a.F=[...(a.aa||[])];a.F.sort((b,c)=>bR(cR(b,a.C.Be),cR(c,a.C.Be),a.C.wg))}function WQ(a){a.G=[...(a.P||[])];a.G.sort((b,c)=>bR(dR(b,a.C.Ae),dR(c,a.C.Ae),a.C.ug))}function eR(a,b){b.style.maxHeight=a.C.Ug?`${b.scrollHeight}px`:`${b.offsetHeight}px`;en(a.D);setTimeout(()=>{a.C.Ug=!a.C.Ug;en(a.D);a.C.Ug||setTimeout(()=>{b.style.maxHeight="fit-content"},150)})}
function fR(a,b){b.style.maxHeight=a.C.Jf?`${b.scrollHeight}px`:`${b.offsetHeight}px`;en(a.D);setTimeout(()=>{a.C.Jf=!a.C.Jf;en(a.D);a.C.Jf||setTimeout(()=>{b.style.maxHeight="fit-content"},150)})}function gR(a,b){b.style.maxHeight=a.C.Uf?`${b.scrollHeight}px`:`${b.offsetHeight}px`;en(a.D);setTimeout(()=>{a.C.Uf=!a.C.Uf;en(a.D);a.C.Uf||setTimeout(()=>{b.style.maxHeight="fit-content"},150)})}
function $Q(a){let b,c;return`${null==(b=a.ga)?void 0:b.collectionLabel}___${null==(c=a.ga)?void 0:c.id}___${a.Qg}`}function bR(a,b,c){if(null==a&&null==b)return 0;if(null==a&&null!=b)return"asc"===c?-1:1;if(null!=a&&null==b)return"asc"===c?1:-1;if("number"===typeof a&&"number"===typeof b)return"asc"===c?a-b:b-a;a=JSON.stringify(a);b=JSON.stringify(b);return"asc"===c?a.localeCompare(b):b.localeCompare(a)}
function cR(a,b){switch(b){case -2:return a.index;case -1:return a.label;default:return a.cols[b].value}}function dR(a,b){switch(b){case -2:return a.index;case -1:return a.label;default:return a.zB[b]}}
var hR=class{constructor(a,b,c,d){this.xi=a;this.C=b;this.hk=c;this.D=d;this.Hv=[];this.hj=[];this.G=[];this.Gv=[];this.P=[];this.Tj=[];this.Mf="";this.J=[];this.hn=[];this.dg=50;this.O="";this.N={};a=new URLSearchParams(document.location.search);a.has("nodeDataProviderDataSummaryTablePageSize")&&(this.dg=Number(a.get("nodeDataProviderDataSummaryTablePageSize")));Ks(()=>{let e;const f=null==(e=nJ(this.xi,this.paneId))?void 0:e.modelGraph;if(f){var g;this.Mf=(null==(g=HL(this.hk,this.paneId,f))?void 0:
g.runId)||"";en(this.D)}});Ks(()=>{var e,f=(this.ga=null==(e=nJ(this.xi,this.paneId))?void 0:e.modelGraph)?wL(this.hk,this.ga):[];let g=e=!1;this.T!==this.ga&&(this.T=this.ga,e=!0);f=YQ(f);this.O!==f&&(this.O=f,g=!0);if(this.ga&&(e||g)){this.Tj=[];e=wL(this.hk,this.ga);for(const p of e)this.Tj.push({runId:p.runId,runName:p.runName,done:p.done,error:p.error});en(this.D);this.C.Be=Math.min(this.C.Be,e.length-1);let h;null==(h=this.Ur)||h.reset();ZQ(this);aR(this);this.C.Ae=Math.min(this.C.Ae,this.hn.length-
1);let m;null==(m=this.mq)||m.reset()}})}uc(a){if(a.rootGroupNodeId){let b;null==(b=this.Ur)||b.reset();let c;null==(c=this.mq)||c.reset();ZQ(this);aR(this)}}PE(a,b){return b.runId}Cy(a,b){return b.id}SE(a,b){return b.stat}get Fs(){return this.Tj.some(a=>a.done)}get Lx(){return null==this.F?0:this.F.length}get kv(){return this.G.length}get DE(){return this.Ug?"arrow_right":"arrow_drop_down"}get CE(){return null==this.Qg?"Aggregated stats":"Aggregated stats in selected layer"}get Ug(){return this.C.Ug}get tB(){return this.Jf?
"arrow_right":"arrow_drop_down"}get sB(){return null==this.Qg?"Root-level nodes stats":"Child nodes stats in selected layer"}get Jf(){return this.C.Jf}get rD(){return this.Uf?"arrow_right":"arrow_drop_down"}get qD(){return null==this.Qg?"Node data":"Node data in selected layer"}get Uf(){return this.C.Uf}get wg(){return this.C.wg}get Be(){return this.C.Be}get ug(){return this.C.ug}get Ae(){return this.C.Ae}};hR.K=function(a){return new (a||hR)(y(VJ),y(kQ),y(LL),y(Tr))};
hR.Ca=Jf({type:hR,ha:[["node-data-provider-summary-panel"]],eb:function(a,b){a&1&&(vr(lQ,5),vr(mQ,5));if(a&2){let c;wr(c=xr())&&(b.Ur=c.first);wr(c=xr())&&(b.mq=c.first)}},inputs:{paneId:"paneId",Qg:"rootGroupNodeId"},ka:!0,features:[Hg,Nr],ya:29,za:19,Ba:[["statsTableContainer",""],["childrenStatsTableContainer",""],["nodeDataTableContainer",""],["childrenStatsPaginator",""],["paginator",""],[1,"container"],[1,"index-container"],["class","index-row",3,"selected",4,"ngFor","ngForOf","ngForTrackBy"],
[1,"table-container"],[1,"table-title-container"],[1,"title-label-container",3,"click"],["border","0","cellspacing","0","cellpadding","0",4,"ngIf"],[1,"table-container","children-stats"],[3,"pageSize","itemsCount"],["class","children-stats-results","border","0","cellspacing","0","cellpadding","0",4,"ngIf"],[1,"table-container","results"],["class","results","border","0","cellspacing","0","cellpadding","0",4,"ngIf"],[1,"index-row"],[1,"index-number-container"],["class","index-number",4,"ngIf"],["color",
"primary","diameter","16",4,"ngIf"],[1,"run-name"],[1,"action-icons-container"],["class","error",3,"matTooltip","matTooltipDisabled",4,"ngIf"],["class","icon-container visibility",3,"selected","hide","matTooltip","click",4,"ngIf"],["class","icon-container action",3,"matTooltip","click",4,"ngIf"],[1,"index-number"],["color","primary","diameter","16"],[1,"error",3,"matTooltip","matTooltipDisabled"],[1,"icon-container","visibility",3,"click","matTooltip"],[1,"icon-container","action",3,"click","matTooltip"],
"border 0 cellspacing 0 cellpadding 0".split(" "),[1,"stats"],["class","value-col",4,"ngFor","ngForOf"],[4,"ngFor","ngForOf","ngForTrackBy"],[1,"value-col"],[1,"header-content"],[3,"change","pageSize","itemsCount"],["border","0","cellspacing","0","cellpadding","0",1,"children-stats-results"],[1,"index",3,"click"],["class","sort",4,"ngIf"],[1,"header","node-label",3,"click"],["class","value-col",3,"click",4,"ngFor","ngForOf"],[1,"sort"],[1,"value-col",3,"click"],[1,"stat-label"],[1,"node-label",3,
"click"],["border","0","cellspacing","0","cellpadding","0",1,"results"],["class","icon-container visibility","matTooltipPosition","above",3,"selected","hide","matTooltip","click",4,"ngIf"],["matTooltipPosition","above",1,"icon-container","visibility",3,"click","matTooltip"],["class","value-col",3,"background-color","color",4,"ngFor","ngForOf"]],sa:function(a,b){if(a&1){const c=Zg();R(0,"div",5)(1,"div",6);H(2,tQ,10,8,"div",7);U();R(3,"div",8,0)(5,"div",9)(6,"div",10);V("click",function(){t(c);const d=
Ar(4);return x(eR(b,d))});R(7,"mat-icon");Z(8);U();Z(9);U()();H(10,xQ,8,3,"table",11);U();R(11,"div",12,1)(13,"div",9)(14,"div",10);V("click",function(){t(c);const d=Ar(12);return x(fR(b,d))});R(15,"mat-icon");Z(16);U();Z(17);U();H(18,yQ,2,2,"paginator",13);U();H(19,IQ,14,5,"table",14);U();R(20,"div",15,2)(22,"div",9)(23,"div",10);V("click",function(){t(c);const d=Ar(21);return x(gR(b,d))});R(24,"mat-icon");Z(25);U();Z(26);U();H(27,JQ,2,2,"paginator",13);U();H(28,TQ,14,5,"table",16);U()()}a&2&&(D(2),
Q("ngForOf",b.Tj)("ngForTrackBy",b.PE),D(),Wp("collapsed",b.Ug),D(5),Br(b.DE),D(),Cr(" ",b.CE," "),D(),Q("ngIf",b.Fs),D(),Wp("collapsed",b.Jf),D(5),Br(b.tB),D(),Cr(" ",b.sB," "),D(),Gq(b.kv>b.dg&&!b.Jf?18:-1),D(),Q("ngIf",b.Fs),D(),Wp("collapsed",b.Uf),D(5),Br(b.rD),D(),Cr(" ",b.qD," "),D(),Gq(b.Lx>b.dg&&!b.Uf?27:-1),D(),Q("ngIf",b.Fs))},Ha:[it,$s,dt,wD,vD,sD,iI,hI,OE,NE,CO,Dx],styles:[".container[_ngcontent-%COMP%]{padding-top:12px;-moz-box-sizing:border-box;box-sizing:border-box;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;height:100%}.container[_ngcontent-%COMP%]   .index-number[_ngcontent-%COMP%]{font-size:10px;width:16px;height:16px;border-radius:8px;-moz-box-sizing:border-box;box-sizing:border-box;border:1px solid #999;background-color:#eee;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center}.container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;width:16px;height:16px;opacity:.3;cursor:pointer}.container[_ngcontent-%COMP%]   .icon-container.selected[_ngcontent-%COMP%]{opacity:1;cursor:default}.container[_ngcontent-%COMP%]   .icon-container.selected[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{color:#e1a011}.container[_ngcontent-%COMP%]   .icon-container.hide[_ngcontent-%COMP%]{visibility:hidden}.container[_ngcontent-%COMP%]   .icon-container.action[_ngcontent-%COMP%]{opacity:.7;margin-left:4px}.container[_ngcontent-%COMP%]   .icon-container.action[_ngcontent-%COMP%]:hover{opacity:1}.container[_ngcontent-%COMP%]   .icon-container.visibility[_ngcontent-%COMP%]{margin-left:4px}.container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:16px;width:16px;height:16px;line-height:16px;color:#000}.container[_ngcontent-%COMP%]   .index-container[_ngcontent-%COMP%]   .index-row[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;overflow:hidden;padding:2px 8px}.container[_ngcontent-%COMP%]   .index-container[_ngcontent-%COMP%]   .index-row.selected[_ngcontent-%COMP%]{background-color:#fff2d5}.container[_ngcontent-%COMP%]   .index-container[_ngcontent-%COMP%]   .index-row[_ngcontent-%COMP%]   .index-number-container[_ngcontent-%COMP%]{width:16px;height:16px;-webkit-flex-shrink:0;-ms-flex-negative:0;flex-shrink:0;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;margin-right:6px}.container[_ngcontent-%COMP%]   .index-container[_ngcontent-%COMP%]   .index-row[_ngcontent-%COMP%]   .run-name[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;white-space:nowrap;overflow:hidden;text-overflow:ellipsis}.container[_ngcontent-%COMP%]   .index-container[_ngcontent-%COMP%]   .index-row[_ngcontent-%COMP%]   .action-icons-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-flex-shrink:0;-ms-flex-negative:0;flex-shrink:0}.container[_ngcontent-%COMP%]   select[_ngcontent-%COMP%]{width:100%}.container[_ngcontent-%COMP%]   .running[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;margin-top:16px}.container[_ngcontent-%COMP%]   .running[_ngcontent-%COMP%]   mat-spinner[_ngcontent-%COMP%]{margin-right:4px}.container[_ngcontent-%COMP%]   .running[_ngcontent-%COMP%]   .label[_ngcontent-%COMP%]{color:#777}.container[_ngcontent-%COMP%]   .error[_ngcontent-%COMP%]{width:16px;height:16px}.container[_ngcontent-%COMP%]   .error[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:16px;height:16px;width:16px;color:#a00}.container[_ngcontent-%COMP%]   .table-container[_ngcontent-%COMP%]{width:100%;-moz-box-sizing:border-box;box-sizing:border-box;padding:8px;margin-top:2px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;-webkit-transition:max-height .15s ease-out;transition:max-height .15s ease-out;overflow:hidden}.container[_ngcontent-%COMP%]   .table-container.collapsed[_ngcontent-%COMP%]{max-height:30px!important}.container[_ngcontent-%COMP%]   .table-container[_ngcontent-%COMP%]   .table-title-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;line-height:22px}.container[_ngcontent-%COMP%]   .table-container[_ngcontent-%COMP%]   .table-title-container[_ngcontent-%COMP%]   .title-label-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;cursor:pointer;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;line-height:15px}.container[_ngcontent-%COMP%]   .table-container[_ngcontent-%COMP%]   .table-title-container[_ngcontent-%COMP%]   .title-label-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{margin-left:-4px}.container[_ngcontent-%COMP%]   .table-container[_ngcontent-%COMP%]   paginator[_ngcontent-%COMP%]{-webkit-align-self:flex-end;-ms-flex-item-align:end;align-self:flex-end}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]{border:1px solid #ccc;border-radius:4px;width:100%;-moz-box-sizing:border-box;box-sizing:border-box;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;background-color:#fdfdfd;overflow:hidden}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]{height:24px;background-color:#f2f2f2}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   .header[_ngcontent-%COMP%]{font-weight:500}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]{font-size:11px;font-family:Arial,Helvetica,sans-serif}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   tr[_ngcontent-%COMP%]:hover{outline:1px solid #999}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   td[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   th[_ngcontent-%COMP%]{padding:1px 4px;text-align:left}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   td.node-label[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   th.node-label[_ngcontent-%COMP%]{border-left:1px solid #ddd}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   td.value-col[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   th.value-col[_ngcontent-%COMP%]{border-left:1px solid #ddd;max-width:80px;text-overflow:ellipsis;overflow:hidden}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .header-content[_ngcontent-%COMP%]{height:100%;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .header-content[_ngcontent-%COMP%]   mat-icon.sort[_ngcontent-%COMP%]{font-size:12px;width:12px;height:12px;color:#999;margin-left:4px}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .header-content[_ngcontent-%COMP%]   .stat-label[_ngcontent-%COMP%]{margin-left:6px;white-space:nowrap}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   th[_ngcontent-%COMP%]{cursor:pointer}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   td.node-label[_ngcontent-%COMP%]{cursor:pointer}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   td.node-label[_ngcontent-%COMP%]:hover{-webkit-text-decoration:underline dotted #0085f2;-moz-text-decoration:underline dotted #0085f2;text-decoration:underline dotted #0085f2}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   td.input[_ngcontent-%COMP%]{color:#009e73;font-weight:500}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   td.output[_ngcontent-%COMP%]{color:#d55e00;font-weight:500}"],
Pa:0});const iR=["inputValueContent"],jR=(a,b)=>b.id||b.label,kR=(a,b)=>b.Nd.id,lR=(a,b)=>b.key,mR=(a,b)=>b.Ej,nR=(a,b)=>b.id,oR=a=>({value:a});function pR(a,b){a&1&&(R(0,"tr")(1,"td",21),Oq(2,"hoverable-label",22),U(),R(3,"td",23),Oq(4,"expandable-info-text",24),U()());a&2&&(a=b.da,b=Y(2),Wp("search-match",qR(b,a.label)),D(2),Q("label",a.label),D(2),Q("text",a.value)("type",a.label)("bgColor",a.bgColor||"transparent")("textColor",a.textColor||"black"))}
function rR(a,b){if(a&1){const c=Zg();R(0,"div",15,1)(2,"div",16)(3,"button",17);V("click",function(){const d=t(c).da,e=Ar(1),f=Y();return x(sR(f,d.label,e))});R(4,"mat-icon");Z(5);U()();Z(6);U();R(7,"div",18)(8,"table",19);Lq(9,pR,5,7,"tr",20,jR);U()()()}a&2&&(a=b.da,b=Y(),Wp("collapsed",tR(b,a.label)),D(5),Br(uR(b,a.label)),D(),Cr(" ",a.label," "),D(3),Nq(a.items))}
function vR(a){if(a&1){const b=Zg();R(0,"div",15,2)(2,"div",16)(3,"button",17);V("click",function(){t(b);const c=Ar(1),d=Y();return x(sR(d,d.be.cp,c))});R(4,"mat-icon");Z(5);U()();Z(6," Node data providers ");U();Oq(7,"node-data-provider-summary-panel",25);U()}a&2&&(a=Y(),Wp("collapsed",tR(a,a.be.cp)),D(5),Br(uR(a,a.be.cp)),D(2),Q("paneId",a.paneId)("rootGroupNodeId",a.Jv))}
function wR(a){if(a&1){const b=Zg();R(0,"paginator",32);V("change",function(c){t(b);const d=Y(2);return x(xR(d,c))});U()}a&2&&(a=Y(2),Q("pageSize",a.Dc)("itemsCount",a.uq))}function yR(a){a&1&&(R(0,"div",35),Z(1),U());a&2&&(a=Y().da,Y(2),D(),Br(zR(a)))}
function AR(a){if(a&1){const b=Zg();R(0,"div",43);V("click",function(c){t(b);const d=Y(2).da,e=Y(2);return x(BR(e,d.Nd.id,e.Lh,c))});R(1,"mat-icon");Z(2);U()()}if(a&2){a=Y(2).da;const b=Y(2);Wp("visible",zM(b.ue,a.Nd.id));Q("matTooltip",zM(b.ue,a.Nd.id)?"Click to hide highlight":"Click to show highlight");D(2);Cr(" ",zM(b.ue,a.Nd.id)?"visibility":"visibility_off"," ")}}
function CR(a){if(a&1){const b=Zg();R(0,"div",38);V("click",function(c){t(b);const d=Y().da,e=Y(2);return x(e.nl(d.Nd.id,c))});Z(1);R(2,"div",39)(3,"mat-icon",40);Z(4,"my_location");U()();Oq(5,"div",41);H(6,AR,3,4,"div",42);U()}if(a&2){a=Y().da;const b=Y(2);Mp("data-id",a.Nd.id);D();Cr(" ",zR(a)," ");D();Q("matTooltip",b.Gl);D(4);Gq(a.Nd.hideInLayout?-1:6)}}
function DR(a){a&1&&(R(0,"tr",null,4)(2,"td",21)(3,"div",44)(4,"mat-icon",45),Z(5,"circle"),U(),Oq(6,"hoverable-label",22),U()(),R(7,"td",23),Oq(8,"expandable-info-text",46),U()());if(a&2){a=Y().da;const b=Y(3);Wp("search-match",ER(b,a.value));D(6);Q("label",a.key);D(2);Q("text",a.value)("type",a.key)}}function FR(a,b){a&1&&H(0,DR,9,5,"tr",20);a&2&&(a=b.da,Y(3),Gq(a.key.startsWith("__")?-1:0))}
function GR(a,b){a&1&&(R(0,"div",31)(1,"div",33)(2,"div",34),Z(3),U(),H(4,yR,2,1,"div",35)(5,CR,7,4,"div",36),U(),R(6,"table",37),Lq(7,FR,1,1,null,null,lR),U()());a&2&&(a=b.da,b=Y(2),D(),Wp("search-match",ER(b,HR(a))||ER(b,a.Nd.label)),D(2),Br(a.index),D(),Gq(a.Nd.hideInLayout?4:5),D(3),Nq(a.Uh))}
function IR(a){if(a&1){const b=Zg();R(0,"div",26,3)(2,"div",27)(3,"div",28)(4,"button",17);V("click",function(){t(b);const c=Ar(1),d=Y();return x(sR(d,d.be.ap,c))});R(5,"mat-icon");Z(6);U()();Z(7);U();H(8,wR,1,2,"paginator",29);U();R(9,"div",30);Lq(10,GR,9,4,"div",31,kR);U()()}a&2&&(a=Y(),Wp("collapsed",tR(a,a.be.ap)),D(6),Br(uR(a,a.be.ap)),D(),Cr(" inputs (",a.uq,") "),D(),Gq(a.oE?8:-1),D(2),Nq(a.Lh))}
function JR(a){if(a&1){const b=Zg();R(0,"paginator",32);V("change",function(c){t(b);const d=Y(2);d.to=d.Wf.slice(c*d.Dc,(c+1)*d.Dc);en(d.Xd);return x()});U()}a&2&&(a=Y(2),Q("pageSize",a.Dc)("itemsCount",a.Wf.length))}
function KR(a){if(a&1){const b=Zg();R(0,"div",50);V("click",function(c){t(b);const d=Y().da,e=Y(2);return x(LR(e,d.Ej,e.to,c))});R(1,"mat-icon");Z(2);U()()}if(a&2){a=Y().da;const b=Y(2);Wp("visible",AM(b.ue,a.Ej));Q("matTooltip",AM(b.ue,a.Ej)?"Click to hide highlight":"Click to show highlight");D(2);Cr(" ",AM(b.ue,a.Ej)?"visibility":"visibility_off"," ")}}function MR(a){a&1&&Rq(0)}
function NR(a){a&1&&(R(0,"div",53),H(1,MR,1,0,"ng-container",54),U());if(a&2){a=Y().da;const b=Y(6),c=Ar(11);D();Q("ngTemplateOutlet",c)("ngTemplateOutletContext",Qr(oR,b.kC(a)))}}
function OR(a,b){if(a&1){const c=Zg();R(0,"div",52);V("click",function(d){const e=t(c).da,f=Y(6);return x(f.nl(e.id,d))});Z(1);R(2,"div",39)(3,"mat-icon",40);Z(4,"my_location");U()()();H(5,NR,2,4,"ng-template",null,6,Rr)}if(a&2){a=b.da;b=Ar(6);const c=Y(6);Wp("search-match",PR(c,a.label));Q("xapInlineDialog",b)("overlaySize",c.oq)("overlayPositions",c.zD)("hoverDelayMs",50);Mp("data-id",a.id);D();Cr(" ",a.label," ");D();Q("matTooltip",c.Gl)}}
function QR(a){a&1&&Lq(0,OR,7,9,null,null,nR);a&2&&(a=Y().da,Nq(a.wv))}function RR(a){a&1&&Oq(0,"expandable-info-text",51);a&2&&(a=Y().da,Q("text",a.value))}function SR(a,b){a&1&&(R(0,"tr")(1,"td",21)(2,"div",44)(3,"mat-icon",45),Z(4,"circle"),U(),Oq(5,"hoverable-label",22),U()(),R(6,"td",23),H(7,QR,2,0)(8,RR,1,1,"expandable-info-text",51),U()());a&2&&(a=b.da,b=Y(4),Wp("search-match",PR(b,a.value)),D(5),Q("label",a.key),D(2),Gq(a.key===b.AD?7:8))}
function TR(a){a&1&&(R(0,"table",37),Lq(1,SR,9,4,"tr",20,lR),U());a&2&&(a=Y().da,D(),Nq(a.Uh))}function UR(a,b){a&1&&(R(0,"div",31)(1,"div",33)(2,"div",34),Z(3),U(),R(4,"div",35),Z(5),U(),Oq(6,"div",41),H(7,KR,3,4,"div",49),U(),H(8,TR,3,0,"table",37),U());a&2&&(a=b.da,b=Y(2),D(),Wp("search-match",PR(b,a.Os)),D(2),Br(a.index),D(2),Br(""===a.Os?"output":a.Os),D(2),Gq(VR(a)?7:-1),D(),Gq(0<a.Uh.length?8:-1))}
function WR(a){if(a&1){const b=Zg();R(0,"div",47,5)(2,"div",48)(3,"div",28)(4,"button",17);V("click",function(){t(b);const c=Ar(1),d=Y();return x(sR(d,d.be.fp,c))});R(5,"mat-icon");Z(6);U()();Z(7);U();H(8,JR,1,2,"paginator",29);U();R(9,"div",30);Lq(10,UR,9,6,"div",31,mR);U()()}a&2&&(a=Y(),Wp("collapsed",tR(a,a.be.fp)),D(6),Br(uR(a,a.be.fp)),D(),Cr(" outputs (",a.Fv,") "),D(),Gq(a.rE?8:-1),D(2),Nq(a.to))}
function YR(a){if(a&1){const b=Zg();R(0,"paginator",32);V("change",function(c){t(b);const d=Y(2);let e;d.Un=aL(d.Nh.slice(c*d.Dc,(c+1)*d.Dc),"incoming",(null==(e=d.Mc)?void 0:e.id)||"");return x()});U()}a&2&&(a=Y(2),Q("pageSize",a.Dc)("itemsCount",a.Nh.length))}
function ZR(a){if(a&1){const b=Zg();R(0,"div",15,7)(2,"div",55)(3,"div",28)(4,"button",17);V("click",function(){t(b);const c=Ar(1),d=Y();return x(sR(d,d.be.Yo,c))});R(5,"mat-icon");Z(6);U()();Z(7);U();H(8,YR,1,2,"paginator",29);U();Oq(9,"io-tree",56);U()}a&2&&(a=Y(),Wp("collapsed",tR(a,a.be.Yo)),D(6),Br(uR(a,a.be.Yo)),D(),Cr(" Identical layers (",a.Nh.length,") "),D(),Gq(a.mE?8:-1),D(),Q("data",a.Un)("rendererId",a.vq))}
function $R(a,b){a&1&&(R(0,"div",59),Z(1),U());a&2&&(a=b.da,D(),Cr(" ",a," "))}function aS(a,b){a&1&&(R(0,"div",57),Z(1," Namespace hierarchy "),U(),R(2,"div",58),Lq(3,$R,2,1,"div",59,Iq),U());a&2&&(a=b.value,D(3),Nq(a.split("/")))}const bS={oF:"Graph info",rF:"Node info",qF:"Layer info",kF:"Attributes",cp:"Node data providers",Yo:"Identical groups",ap:"inputs",fp:"outputs"};function qR(a,b){return null!=a.aa.find(c=>c.Rw===b)}
function sR(a,b,c){c&&(tR(a,b)?c.style.maxHeight=`${c.scrollHeight}px`:(c.style.maxHeight=`${c.offsetHeight}px`,c.style.overflow="hidden"),en(a.Xd),setTimeout(()=>{a.J.C.has(b)?a.J.C.delete(b):a.J.C.add(b);en(a.Xd);setTimeout(()=>{tR(a,b)?c.style.overflow="hidden":(c.style.overflow="visible",c.style.maxHeight="fit-content")},150)}))}function tR(a,b){return a.J.C.has(b)}function uR(a,b){return tR(a,b)?"expand_more":"expand_less"}
function xR(a,b){a.Lh=cS(b*a.Dc,a.C.slice(b*a.Dc,(b+1)*a.Dc),a.F.slice(b*a.Dc,(b+1)*a.Dc));en(a.Xd);setTimeout(()=>{dS(a)})}function zR(a){const b=a.Uh.find(c=>"__tensor_tag"===c.key);return b?`${b.value} (${a.Nd.label})`:a.Nd.label}function BR(a,b,c,d){d.stopPropagation();d.altKey?wM(a.ue,b,c.map(e=>e.Nd.id)):vM(a.ue,b)}function ER(a,b){return a.O?null!=a.O.find(c=>c.jf===b):!1}function HR(a){let b,c;return null!=(c=null==(b=a.Uh.find(d=>"__tensor_tag"===d.key))?void 0:b.value)?c:""}
function LR(a,b,c,d){d.stopPropagation();d.altKey?yM(a.ue,b,c.map(e=>e.Ej)):xM(a.ue,b)}function PR(a,b){return a.P?null!=a.P.find(c=>c.jf===b):!1}function VR(a){let b;return 0<((null==(b=a.Uh.find(c=>"connects to"===c.key))?void 0:b.wv)||[]).length}function eS(a,b){a.ga&&b?a.Mc=a.ga.nodesById[b]:a.Mc=void 0;fS(a);en(a.Xd);setTimeout(()=>{dS(a)})}
function gS(a){if(a.Mc&&a.G){var b=a.G.results[a.Mc.id]||[],c=[],d=[],e=[];for(const f of b)switch(f.type){case "input":c.push(f);break;case "output":d.push(f);break;case "list":e.push(f)}a.aa=e;a.P=d;a.O=c;en(a.Xd)}}
function fS(a){a.ii=[];a.Lh=[];a.C=[];a.F=[];a.Wf=[];a.Nh=[];a.Un=void 0;if(a.ea){if(a.ga){var b={label:"Graph info",Go:"graph",items:[]};a.ii.push(b);var c=0,d=0;for(var e of a.ga.nodes)nI(e)&&!e.hideInLayout?c++:oI(e)&&d++;b.items.push({rf:b,label:"op node count",value:String(c)},{rf:b,label:"layer count",value:String(d)})}}else if(a.Mc)if(nI(a.Mc)){if(a.ga&&a.Mc){e=a.Mc;var f={label:"Node info",Go:"op",items:[]};a.ii.push(f);f.items.push({rf:f,label:"op name",value:`${e.label}`});var g="id";f.items.push({rf:f,
label:g,value:e.id,aj:!0,Wj:a.ca.has(g)});g="namespace";f.items.push({rf:f,label:g,value:sI(e),aj:!0,Wj:a.ca.has(g)});if(0<Object.keys(e.attrs||{}).length){f={label:"Attributes",Go:"op",items:[]};g=e.attrs||{};for(var h of Object.keys(g))h.startsWith("__")||f.items.push({rf:f,label:h,value:g[h],aj:!0,Wj:a.oa.has(h)});0<f.items.length&&a.ii.push(f)}f=wL(a.N,a.ga);if(0<f.length){h={label:"Node data providers",Go:"op",items:[]};a.ii.push(h);for(b of f){var m=((b.results||{})[a.ga.id]||{})[e.id];f=(null==
(c=m)?void 0:c.strValue)||"-";g=(null==(d=m)?void 0:d.bgColor)||"transparent";let p;m=(null==(p=m)?void 0:p.textColor)||"black";h.items.push({id:b.runId,rf:h,label:b.runName,value:f,aj:b.done,Wj:null!=a.ja[b.runId],bgColor:g,textColor:m,hd:!b.done})}}}hS(a)}else oI(a.Mc)&&iS(a)}function dS(a){for(let b=0;b<a.jr.length;b++){let c;const d=null==(c=a.jr.get(b))?void 0:c.R;d&&d.scrollHeight>d.offsetHeight&&d.classList.add("expandable")}}
function cS(a,b,c){const d=[];for(let e=0;e<b.length;e++){const f=b[e],g=[];Object.entries(c[e]).forEach(([m,p])=>{g.push({key:m,value:p})});g.sort((m,p)=>m.key.localeCompare(p.key));g.push({key:"namespace",value:sI(b[e])});const h=f.attrs||{};h.__value&&g.push({key:"values",value:h.__value});d.push({index:e+a,Nd:f,Uh:g})}return d}
function jS(a,b){const c=Date.now(),d=a.width,e=()=>{var f=Date.now()-c;f=a.D.C?1:Math.min(1,f/150);f=Math.sin(f*Math.PI/2);const g=d+(b-d)*f;a.width=g;a.minWidth=g;en(a.Xd);1<=f?(a.width=b,a.minWidth=b,en(a.Xd)):requestAnimationFrame(e)};e()}
function hS(a){if(a.ga&&a.Mc){var b=a.Mc,c=b.incomingEdges||[];a.F=[];a.C=[];a.Lh=[];for(var d of c){var e=void 0,f=null==(e=a.ga)?void 0:e.nodesById[d.sourceNodeId];a.C.push(f);var g=(b.inputsMetadata||{})[d.targetNodeInputId]||{};f=Object.assign({},(f.outputsMetadata||{})[d.sourceNodeOutputId]||{});for(var h of Object.keys(f))null==g[h]&&"__tensor_tag"!==h&&(g[h]=f[h]);a.F.push(g)}a.uq=a.C.length;0<c.length&&(a.Lh=cS(0,a.C.slice(0,a.Dc),a.F.slice(0,a.Dc)));a.Wf=[];c=b.outputsMetadata||{};b=b.outgoingEdges||
[];d=0;for(const m of Object.keys(c)){e=[];g="";for(const p of Object.keys(c[m]))h=c[m][p],"__tensor_tag"===p&&(g=h),p.startsWith("__")||e.push({key:p,value:h});e.sort((p,u)=>p.key.localeCompare(u.key));h=b.filter(p=>p.sourceNodeOutputId===m).map(p=>a.ga.nodesById[p.targetNodeId]);e.push({key:"connects to",value:"",wv:h});a.Wf.push({index:d,Os:g,Ej:m,Uh:e});d++}a.Fv=a.Wf.length;a.to=a.Wf.slice(0,a.Dc)}}
function iS(a){if(a.ga&&a.Mc){var b=a.Mc,c={label:"Layer info",Go:"group",items:[]};a.ii.push(c);c.items.push({rf:c,label:"name",value:b.label});var d="namespace";c.items.push({rf:c,label:d,value:sI(b),aj:!0,Wj:a.T.has(d)});d="#children";c.items.push({rf:c,label:d,value:String((b.nsChildrenIds||[]).length),aj:!0,Wj:a.T.has(d)});d="#descendants";c.items.push({rf:c,label:d,value:String((b.descendantsNodeIds||[]).length),aj:!0,Wj:a.T.has(d)});null!=b.identicalGroupIndex&&(a.Nh=a.ga.nodes.filter(e=>oI(e)&&
e.identicalGroupIndex===b.identicalGroupIndex),a.Un=aL(a.Nh.slice(0,a.Dc),"incoming",b.id))}}
var kS=class{constructor(a,b,c,d,e){this.D=a;this.N=b;this.Xd=c;this.J=d;this.ue=e;this.jr=new cr;this.ca=new Set;this.oa=new Set;this.T=new Set;this.ja={};this.G=void 0;this.minWidth=this.width=370;this.ii=[];this.Lh=[];this.Wf=[];this.to=[];this.Nh=[];this.vq="";this.Fv=this.uq=0;this.ik=this.ps=!1;this.Dc=this.D.C?5:25;this.be=bS;this.AD="connects to";this.Gl="Click: locate\nAlt+click: select";this.kC=sI;this.oq={minWidth:100,minHeight:0,maxWidth:600};this.xv=[{bb:"start",cb:"top",Na:"end",Ua:"top",
offsetX:-32}];this.zD=[{bb:"start",cb:"top",Na:"end",Ua:"top",offsetX:-4}];this.aa=[];this.O=[];this.P=[];this.C=[];this.F=[];this.Nx=0;Ks(()=>{var f=nJ(this.D,this.paneId);if(f&&f.modelGraph){this.ga=f.modelGraph;var g;this.vq=(null==(g=f.Sg)?void 0:g.rendererId)||"";var h;f=(null==(h=f.Sg)?void 0:h.nodeId)||"";this.Jv!==f&&(eS(this,f),setTimeout(()=>{var m=this.ue;m.C.set({});m.D.set({});gS(this)}))}});Ks(()=>{const f=nJ(this.D,this.paneId);f&&f.modelGraph&&this.G!==f.Rg&&(this.G=f.Rg,gS(this))});
Ks(()=>{this.N.D();fS(this);en(this.Xd);setTimeout(()=>{dS(this)})})}Yq(a){a.preventDefault();document.body.style.cursor="ew-resize";const b=qk(document,"mousemove"),c=qk(window,"mouseup"),d=this.width,e=a.clientX;this.ps=!0;en(this.Xd);C(Uj([b]),Oo(c)).subscribe({next:([f])=>{this.minWidth=this.width=Math.max(64,d-(f.clientX-e));en(this.Xd)},complete:()=>{document.body.style.cursor="default";this.ps=!1;en(this.Xd)}})}nl(a,b){this.D.F.set({nodeId:a,rendererId:this.vq,Cd:!1,select:b.altKey})}QE(a,
b){return b.label}get ea(){return null!=this.ga&&null==this.Mc}get Zx(){return this.ga?(null==this.Mc||oI(this.Mc))&&0<wL(this.N,this.ga).length:!1}get Jv(){return this.Mc?this.Mc.id:void 0}get oE(){return this.C.length>this.Dc&&!tR(this,"inputs")}get rE(){return this.Wf.length>this.Dc&&!tR(this,"outputs")}get mE(){return this.Nh.length>this.Dc&&!tR(this,"Identical groups")}get vC(){return this.ik?"Show info panel":"Hide info panel"}get uC(){return this.ik?"chevron_left":"chevron_right"}};
kS.K=function(a){return new (a||kS)(y(VJ),y(LL),y(Tr),y(kQ),y(BM))};
kS.Ca=Jf({type:kS,ha:[["info-panel"]],eb:function(a,b){a&1&&vr(iR,5);if(a&2){let c;wr(c=xr())&&(b.jr=c)}},Za:4,Ia:function(a,b){a&2&&Up("width",b.width,"px")("min-width",b.minWidth,"px")},inputs:{paneId:"paneId"},ka:!0,features:[Lr([kQ]),Nr],ya:12,za:12,Ba:[["namespaceHierarchy",""],["sectionEle",""],["ndpSectionEle",""],["inputsSectionEle",""],["rowEle",""],["outputsSectionEle",""],["namespacePopup",""],["identicalGroupsSectionEle",""],[1,"container"],["class","section",3,"collapsed",4,"ngFor","ngForOf",
"ngForTrackBy"],["class","section",3,"collapsed",4,"ngIf"],["class","section inputs",3,"collapsed",4,"ngIf"],["class","section outputs",3,"collapsed",4,"ngIf"],[1,"resizer",3,"mousedown"],[1,"hide-toggle",3,"click","matTooltip"],[1,"section"],[1,"header"],["mat-icon-button","",1,"toggle",3,"click"],[1,"items-container"],[1,"metadata-table","info-attrs"],[3,"search-match"],[1,"key"],[3,"label"],[1,"value"],[3,"text","type","bgColor","textColor"],[3,"paneId","rootGroupNodeId"],[1,"section","inputs"],
[1,"header","input"],[1,"header-label-container"],[3,"pageSize","itemsCount"],[1,"flat-items-container"],[1,"flat-item"],[3,"change","pageSize","itemsCount"],[1,"name-row"],[1,"index"],[1,"name"],[1,"name","locator"],[1,"metadata-table"],[1,"name","locator",3,"click"],["matTooltipClass","multiline-tooltip-left","matTooltipPosition","right",1,"locator-icon-container",3,"matTooltip"],[1,"locator-icon"],[1,"filler"],["matTooltipPosition","left",1,"visibility-icon-container","input",3,"matTooltip","visible"],
["matTooltipPosition","left",1,"visibility-icon-container","input",3,"click","matTooltip"],[1,"key-container"],[1,"bullet"],[3,"text","type"],[1,"section","outputs"],[1,"header","output"],["matTooltipPosition","left",1,"visibility-icon-container","output",3,"visible","matTooltip"],["matTooltipPosition","left",1,"visibility-icon-container","output",3,"click","matTooltip"],[3,"text"],["dialogLabel","namespace popup",1,"target-node-container",3,"click","xapInlineDialog","overlaySize","overlayPositions",
"hoverDelayMs"],[1,"model-explorer-const-values-popup"],[4,"ngTemplateOutlet","ngTemplateOutletContext"],[1,"header","identical-groups"],[3,"data","rendererId"],[1,"values-count-msg"],[1,"namespace-content"],[1,"namespace-row"]],sa:function(a,b){if(a&1){const c=Zg();R(0,"div",8);H(1,rR,11,4,"div",9)(2,vR,8,5,"div",10)(3,IR,12,5,"div",11)(4,WR,12,5,"div",12)(5,ZR,10,7,"div",10);U();R(6,"div",13);V("mousedown",function(d){t(c);return x(b.Yq(d))});U();R(7,"div",14);V("click",function(){t(c);b.ik=!b.ik;
let d=0;b.ik?b.Nx=b.width:d=b.Nx;jS(b,d);return x()});R(8,"mat-icon");Z(9);U()();H(10,aS,5,0,"ng-template",null,0,Rr)}a&2&&(Wp("graph-info",b.Zx),D(),Q("ngForOf",b.ii)("ngForTrackBy",b.QE),D(),Q("ngIf",b.Zx),D(),Q("ngIf",0<b.Lh.length),D(),Q("ngIf",0<b.Wf.length),D(),Q("ngIf",b.Un),D(),Wp("resizing",b.ps),D(),Q("matTooltip",b.vC),D(2),Br(b.uC))},Ha:[it,$s,dt,ht,gQ,BB,AB,yB,wD,bE,PE,AE,NE,jQ,iI,hI,CO,$K,hR,HG,EG],styles:["[_nghost-%COMP%]{position:relative;width:370px;min-width:370px}.container[_ngcontent-%COMP%]{width:100%;height:100%;-moz-box-sizing:border-box;box-sizing:border-box;border-left:1px solid #e1e3e1;background-color:#fff;font-size:12px;overflow-y:auto;position:relative}.container[_ngcontent-%COMP%]   .section[_ngcontent-%COMP%]{-webkit-transition:max-height .15s ease-out;transition:max-height .15s ease-out;padding-bottom:8px;-moz-box-sizing:border-box;box-sizing:border-box}.container[_ngcontent-%COMP%]   .section.collapsed[_ngcontent-%COMP%]{max-height:32px!important;overflow:hidden}.container[_ngcontent-%COMP%]   .section.inputs[_ngcontent-%COMP%]{padding-bottom:16px}.container[_ngcontent-%COMP%]   .section[_ngcontent-%COMP%]   .flat-items-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;margin-top:10px;gap:8px}.container[_ngcontent-%COMP%]   .section[_ngcontent-%COMP%]   .flat-item[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;padding:0 2px 0 12px;font-size:12px}.container[_ngcontent-%COMP%]   .section[_ngcontent-%COMP%]   .flat-item[_ngcontent-%COMP%]   .name-row[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .section[_ngcontent-%COMP%]   .flat-item[_ngcontent-%COMP%]   .name-row.search-match[_ngcontent-%COMP%]{background-color:#f5e25a}.container[_ngcontent-%COMP%]   .section[_ngcontent-%COMP%]   .flat-item[_ngcontent-%COMP%]   .name-row[_ngcontent-%COMP%]   .index[_ngcontent-%COMP%]{color:#999;min-width:14px;margin-right:2px}.container[_ngcontent-%COMP%]   .section[_ngcontent-%COMP%]   .flat-item[_ngcontent-%COMP%]   .name-row[_ngcontent-%COMP%]   .name[_ngcontent-%COMP%]{color:#00639b;font-weight:500;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .section[_ngcontent-%COMP%]   .flat-item[_ngcontent-%COMP%]   .name-row[_ngcontent-%COMP%]   .name.locator[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;cursor:pointer}.container[_ngcontent-%COMP%]   .section[_ngcontent-%COMP%]   .flat-item[_ngcontent-%COMP%]   .name-row[_ngcontent-%COMP%]   .name.locator[_ngcontent-%COMP%]:hover{background-color:#f6f6f6}.container[_ngcontent-%COMP%]   .section[_ngcontent-%COMP%]   .flat-item[_ngcontent-%COMP%]   .name-row[_ngcontent-%COMP%]   .name.locator[_ngcontent-%COMP%]:hover   .locator-icon-container[_ngcontent-%COMP%]{opacity:.8}.container[_ngcontent-%COMP%]   .section[_ngcontent-%COMP%]   .flat-item[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]{margin-top:3px;margin-left:18px;margin-right:0}.container[_ngcontent-%COMP%]   .section[_ngcontent-%COMP%]:not(:first-child){border-top:1px solid #dadce0}.container[_ngcontent-%COMP%]   .filler[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1}.container[_ngcontent-%COMP%]   .header[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;font-size:11px;font-weight:700;padding:4px 12px 0;text-transform:uppercase;letter-spacing:.0327em;position:-webkit-sticky;position:sticky;top:0;z-index:100;-webkit-flex-shrink:0;-ms-flex-negative:0;flex-shrink:0;-moz-box-sizing:border-box;box-sizing:border-box;background-color:#fff;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;color:rgba(0,0,0,.87)}.container[_ngcontent-%COMP%]   .header.identical-groups[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   .header.input[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   .header.output[_ngcontent-%COMP%]{-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between}.container[_ngcontent-%COMP%]   .header[_ngcontent-%COMP%]   .header-label-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .header[_ngcontent-%COMP%]   button.toggle[_ngcontent-%COMP%]{padding:0;width:24px;height:24px;margin-left:-8px;margin-right:2px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center}.container[_ngcontent-%COMP%]   .header[_ngcontent-%COMP%]   button.toggle[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{color:inherit;font-size:20px;width:20px;height:20px}.container[_ngcontent-%COMP%]   .header[_ngcontent-%COMP%]   button.toggle[_ngcontent-%COMP%]     .mat-mdc-button-touch-target{display:none}.container[_ngcontent-%COMP%]   .output-item-container[_ngcontent-%COMP%]{font-family:Roboto,sans-serif}.container[_ngcontent-%COMP%]   .output-item-container[_ngcontent-%COMP%]:not(.last){margin-top:8px}.container[_ngcontent-%COMP%]   .output-item-label[_ngcontent-%COMP%]{font-size:13px;padding:4px 12px;color:#00639b;font-weight:500}.container[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]{border-spacing:0;border-collapse:collapse;font-size:12px;margin:0 12px;word-break:break-all;max-width:calc(100% - 28px)}.container[_ngcontent-%COMP%]   .metadata-table.info-attrs[_ngcontent-%COMP%]{margin-left:30px}.container[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   tr[_ngcontent-%COMP%]{vertical-align:text-top}.container[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   tr.search-match[_ngcontent-%COMP%]{background-color:#f5e25a}.container[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   tr[_ngcontent-%COMP%]:hover.has-hover-values{cursor:pointer;background-color:#f6f6f6}.container[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   tr[_ngcontent-%COMP%]   mat-icon.bullet[_ngcontent-%COMP%]{-webkit-flex-shrink:0;-ms-flex-negative:0;flex-shrink:0;font-size:4px;width:5px;height:5px;color:#bbb;margin-right:4px;margin-top:6px}.container[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   td.key[_ngcontent-%COMP%]{color:#999;padding-right:6px;white-space:nowrap;min-width:100px;width:100px;max-width:100px;overflow:hidden;text-overflow:ellipsis;position:relative}.container[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   td.key[_ngcontent-%COMP%]   .key-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:start;-webkit-align-items:flex-start;-moz-box-align:start;-ms-flex-align:start;align-items:flex-start;overflow:hidden;position:absolute;inset:0}.container[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   td.value[_ngcontent-%COMP%]{max-width:10px;overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.container[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   td.value[_ngcontent-%COMP%]:has(expandable-info-text.expanded){white-space:normal;max-width:inherit}.container[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   .target-node-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;cursor:pointer}.container[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   .target-node-container[_ngcontent-%COMP%]:hover   .locator-icon-container[_ngcontent-%COMP%]{opacity:.8}.container[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   .target-node-container.search-match[_ngcontent-%COMP%]{background-color:#f5e25a}.container[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   .values-hover-trigger[_ngcontent-%COMP%]{-moz-box-sizing:border-box;box-sizing:border-box;border-radius:99px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;background-color:#d7f1ff;color:#00639b;width:-webkit-fit-content;width:-moz-fit-content;width:fit-content;height:16px;padding:0 6px;font-size:10px}.container[_ngcontent-%COMP%]   .items-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;gap:4px;margin-top:10px;margin-bottom:6px}.container[_ngcontent-%COMP%]   .item-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;padding:0 12px}.container[_ngcontent-%COMP%]   .item-container.search-match[_ngcontent-%COMP%]{background-color:#f5e25a}.container[_ngcontent-%COMP%]   .item-container[_ngcontent-%COMP%]   .label-row[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between}.container[_ngcontent-%COMP%]   .item-container[_ngcontent-%COMP%]   .label[_ngcontent-%COMP%]{font-weight:500;color:#aaa;font-size:12px;margin-bottom:1px}.container[_ngcontent-%COMP%]   .item-container[_ngcontent-%COMP%]   .show-on-node-toggle[_ngcontent-%COMP%]{height:16px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;font-size:11px;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;-moz-box-sizing:border-box;box-sizing:border-box;cursor:pointer;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;-webkit-transform:scale(.7);transform:scale(.7);margin-right:-6px;margin-top:-1px}.container[_ngcontent-%COMP%]   .item-container[_ngcontent-%COMP%]   .value[_ngcontent-%COMP%]{font-size:13px;word-break:break-word}.container[_ngcontent-%COMP%]   .item-container[_ngcontent-%COMP%]   .value.big-text[_ngcontent-%COMP%]{font-size:14px;font-weight:500}.container[_ngcontent-%COMP%]   .item-container[_ngcontent-%COMP%]   .value.has-bg-color[_ngcontent-%COMP%]{padding:2px 4px;border-radius:4px;margin-top:2px}.container[_ngcontent-%COMP%]   .locator-icon-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;opacity:.5;margin-left:4px}.container[_ngcontent-%COMP%]   .locator-icon-container.left[_ngcontent-%COMP%]{margin-right:4px;margin-left:0;display:inline-block}.container[_ngcontent-%COMP%]   .locator-icon-container.left[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{margin-bottom:-2px}.container[_ngcontent-%COMP%]   .locator-icon-container[_ngcontent-%COMP%]:hover{opacity:.8}.container[_ngcontent-%COMP%]   .locator-icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{color:#333;font-size:12px;width:12px;height:12px}.container[_ngcontent-%COMP%]   .visibility-icon-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;opacity:.5;padding:0 4px 0 20px;cursor:pointer}.container[_ngcontent-%COMP%]   .visibility-icon-container[_ngcontent-%COMP%]:hover{opacity:.8}.container[_ngcontent-%COMP%]   .visibility-icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{color:#333;font-size:14px;width:14px;height:14px}.container[_ngcontent-%COMP%]   .visibility-icon-container.input.visible[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{color:#009e73}.container[_ngcontent-%COMP%]   .visibility-icon-container.output.visible[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{color:#d55e00}.resizer[_ngcontent-%COMP%]{position:absolute;top:0;bottom:0;left:0;width:6px;cursor:ew-resize;background-color:transparent;z-index:200}.resizer.resizing[_ngcontent-%COMP%], .resizer[_ngcontent-%COMP%]:hover{background-color:rgba(0,0,0,.04)}.hide-toggle[_ngcontent-%COMP%]{position:absolute;right:100%;bottom:2px;width:16px;height:20px;border:1px solid #ccc;border-right:none;-moz-box-sizing:border-box;box-sizing:border-box;border-radius:99px;border-top-right-radius:0;border-bottom-right-radius:0;cursor:pointer;background-color:#fff}.hide-toggle[_ngcontent-%COMP%]:hover{background-color:#eee}.hide-toggle[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{width:18px;height:18px;font-size:18px;color:#999}  xap-inline-dialog-container:has(.model-explorer-const-values-popup){width:100%}  .model-explorer-const-values-popup{padding:8px;line-height:12px;background-color:#fff}  .model-explorer-const-values-popup .values-count-msg{margin-bottom:8px;color:#999;font-size:12px}  .model-explorer-const-values-popup .content{font-size:11px;white-space:pre-wrap;font-family:monospace}  .model-explorer-const-values-popup .namespace-row{font-size:12px;padding:3px 0}"],
Pa:0});function lS(a){a&1&&Oq(0,"graph-panel",3);a&2&&(a=Y(),Q("@showModelGraph",void 0)("@.disabled",a.Cq)("modelGraph",a.Rl.modelGraph)("paneId",a.Rl.id)("rendererId",a.Rl.id))}function mS(a){a&1&&Oq(0,"info-panel",2);a&2&&(a=Y(),Q("paneId",a.Rl.id))}var nS=class{constructor(a,b){this.C=a;this.D=b}get Cq(){return this.C.C}get nE(){let a;return!(null==(a=this.C.config())?0:a.hideInfoPanel)}};nS.K=function(a){return new (a||nS)(y(VJ),y(Tr))};
nS.Ca=Jf({type:nS,ha:[["split-pane"]],inputs:{Rl:"pane"},ka:!0,features:[Lr([EM,BM]),Nr],ya:3,za:2,Ba:[[1,"container"],[3,"modelGraph","paneId","rendererId",4,"ngIf"],[3,"paneId"],[3,"modelGraph","paneId","rendererId"]],sa:function(a,b){a&1&&(R(0,"div",0),H(1,lS,1,5,"graph-panel",1)(2,mS,1,1,"info-panel",2),U());a&2&&(D(),Q("ngIf",b.Rl.modelGraph),D(),Gq(b.nE?2:-1))},Ha:[it,dt,WP,kS],styles:[".container[_ngcontent-%COMP%]{width:100%;height:100%;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;overflow:hidden}.container[_ngcontent-%COMP%]   graph-panel[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1}"],
data:{animation:[Yy("showModelGraph",[az("void",$y({opacity:0,transform:"scale(1.03, 1.03)"})),bz("void => *",Zy("150ms 100ms ease-out",$y({opacity:1,transform:"scale(1, 1)"})))])]},Pa:0});const oS=["panesContainer"],pS=["splitPane"],qS=(a,b)=>b.label;function rS(a,b){a&1&&(R(0,"div",10),Z(1),U());a&2&&(D(),Cr(" ",b," "))}function sS(a){a&1&&(R(0,"mat-icon"),Z(1,"error"),U())}function tS(a){a&1&&Oq(0,"mat-spinner",12)}function uS(a){a&1&&Oq(0,"mat-spinner",20)}function vS(a){a&1&&(R(0,"mat-icon"),Z(1,"check_circle"),U())}
function wS(a,b){a&1&&(R(0,"div",19),H(1,uS,1,0,"mat-spinner",20)(2,vS,2,0,"mat-icon"),R(3,"div",21),Z(4),U()());a&2&&(a=b.da,D(),Gq(a.ds?1:2),D(3),Cr(" ",a.label," "))}function xS(a){a&1&&(R(0,"div",18),Lq(1,wS,5,2,"div",19,qS),U());if(a&2){a=Y(3).da;const b=Y();D();Nq(b.Kj[a.id]||[])}}
function yS(a){a&1&&(R(0,"div",11),H(1,sS,2,0,"mat-icon")(2,tS,1,0,"mat-spinner",12),Z(3," Processing graph... "),U(),R(4,"div",13),Z(5," Please remain on this page while the model is being processed "),U(),R(6,"div",14)(7,"div",15),Oq(8,"div",16),U(),R(9,"div",17)(10,"mat-icon"),Z(11,"info"),U()(),H(12,xS,3,0,"ng-template",null,2,Rr),U());if(a&2){a=Ar(13);const b=Y(2).da,c=Y();D();Gq(zS(c,b)?1:2);D(7);Up("width",AS(c,b.id),"%");D();Q("xapInlineDialog",a)("hoverDelayMs",100)}}
function BS(a){a&1&&(R(0,"div",7),H(1,rS,2,1,"div",10)(2,yS,14,5),U());if(a&2){let b;a=Y().da;const c=Y();Q("@hideProcessingPanel",void 0)("@.disabled",c.Cq);D();Gq((b=zS(c,a))?1:2,b)}}
function CS(a){if(a&1){const b=Zg();R(0,"div",22)(1,"div",23);Z(2);U();R(3,"div",24)(4,"div",25);V("click",function(){t(b);const c=Y(2);GJ(c.Qe);return x()})("mousedown",function(c){t(b);return x(c.stopPropagation())});R(5,"mat-icon");Z(6,"swap_horiz");U()();R(7,"div",26);V("click",function(){t(b);const c=Y().da,d=Y();FJ(d.Qe,c.id);return x()})("mousedown",function(c){t(b);return x(c.stopPropagation())});R(8,"mat-icon");Z(9,"close");U()()()()}a&2&&(a=Y().da,Y(),D(2),Cr(" ",DS(a)," "))}
function ES(a,b){a&1&&(R(0,"div",6),H(1,BS,3,3,"div",7)(2,CS,10,1,"div",8),Oq(3,"split-pane",9,1),U());if(a&2){a=b.da;b=b.index;const c=Y();Up("width",100*(0===b?c.fj:1-c.fj),"%");Wp("selected",a.id===c.Qe.J());Mp("data-pane-id",a.id);D();Gq(FS(c,a)?1:-1);D();Q("ngIf",c.rw);D();Q("pane",a)}}function GS(a){if(a&1){const b=Zg();R(0,"div",27);V("mousedown",function(c){t(b);const d=Y(),e=Ar(1);return x(d.Yq(c,e))});Oq(1,"div",28);U()}a&2&&(a=Y(),Up("left",a.TD))}
function zS(a,b){let c,d;return null!=(d=null==(c=(a.Kj[b.id]||[]).find(e=>null!=e.error&&""!==e.error))?void 0:c.error)?d:""}function AS(a,b){return(a=a.Kj[b])?a.filter(c=>!c.ds).length/a.length*100:0}function DS(a){return(a=a.modelGraph)?`${a.collectionLabel} | ${a.id}`:"-"}function FS(a,b){return null==b.modelGraph||(a.Kj[b.id]||[]).some(c=>null!=c.error&&""!==c.error)}
function HS(a,b){var c=b.paneId;nJ(a.Qe,c)&&(c=a.Kj[c].find(d=>d.label===b.label),null!=c&&(c.ds=!1,c.error=b.error,fn(a.C)))}
var IS=class{constructor(a,b,c){this.C=a;this.Qe=b;this.D=c;this.Hs=new cr;this.Kj={};this.fj=1;this.La=this.Qe.La;Ks(()=>{var d=this.La();1<=d.length&&(this.fj=d[0].widthFraction);for(const e of d)e.modelGraph||(this.Kj[e.id]=OI.map(f=>({label:f,ds:!0})));fn(this.C);for(d=0;d<this.Hs.length;d++){let e;null!=(e=this.Hs.get(d))&&en(e.D)}});this.D.C.addEventListener("message",d=>{switch(d.data.eventType){case 8:HS(this,d.data)}})}pd(){this.ED.R.addEventListener("mousedown",a=>{(a=a.target.closest(".pane-container"))&&
jJ(this.Qe,a.dataset.paneId||"")},!0)}Yq(a,b){a.preventDefault();document.body.style.cursor="ew-resize";const c=qk(document,"mousemove"),d=qk(window,"mouseup"),e=this.Qe.La()[0].widthFraction,f=b.offsetWidth,g=f*e,h=a.clientX;en(this.C);C(Uj([c]),Oo(d)).subscribe({next:([m])=>{this.fj=Math.min(f-200,Math.max(200,g+(m.clientX-h)))/f;en(this.C)},complete:()=>{document.body.style.cursor="default";AJ(this.Qe,this.fj);en(this.C)}})}NE(a,b){return b.id}get rw(){return 1<this.Qe.La().length}get TD(){return`calc(${100*
this.fj}% - 5px)`}get Cq(){return this.Qe.C}};IS.K=function(a){return new (a||IS)(y(Tr),y(VJ),y(bJ))};
IS.Ca=Jf({type:IS,ha:[["split-panes-container"]],eb:function(a,b){a&1&&(vr(oS,5),vr(pS,5));if(a&2){let c;wr(c=xr())&&(b.ED=c.first);wr(c=xr())&&(b.Hs=c)}},ka:!0,features:[Nr],ya:4,za:3,Ba:[["panesContainer",""],["splitPane",""],["progressPopup",""],[1,"container"],["class","pane-container",3,"selected","width",4,"ngFor","ngForOf","ngForTrackBy"],["class","resizer split-pane",3,"left","mousedown",4,"ngIf"],[1,"pane-container"],[1,"loading"],["class","pane-title-container",4,"ngIf"],[3,"pane"],[1,"error-msg"],
[1,"processing-msg-container"],["color","primary","diameter","18"],[1,"stay-on-page-info"],[1,"progress-bar-wrapper"],[1,"progress-bar-container"],[1,"progress-bar-body"],["dialogLabel","progress details",1,"icon-container",3,"xapInlineDialog","hoverDelayMs"],[1,"model-explorer-processing-tasks-container"],[1,"processing-task-container"],["color","primary","diameter","16"],[1,"task-label"],[1,"pane-title-container"],[1,"pane-title"],[1,"buttons-container"],["matTooltip","Swap pane",1,"icon-container",
"swap",3,"click","mousedown"],["matTooltip","Close pane",1,"icon-container","close",3,"click","mousedown"],[1,"resizer","split-pane",3,"mousedown"],[1,"resizer-line"]],sa:function(a,b){a&1&&(R(0,"div",3,0),H(2,ES,5,8,"div",4)(3,GS,2,2,"div",5),U());a&2&&(D(2),Q("ngForOf",b.La())("ngForTrackBy",b.NE),D(),Q("ngIf",b.rw))},Ha:[it,$s,dt,wD,vD,sD,PE,AE,NE,iI,hI,nS,HG,EG],styles:[".container[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;position:relative;width:100%;height:100%}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]{position:relative;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .pane-title-container[_ngcontent-%COMP%]{height:24px;-moz-box-sizing:border-box;box-sizing:border-box;background-color:#eee;padding:0 12px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;font-size:12px;color:#999;cursor:pointer;-webkit-flex-shrink:0;-ms-flex-negative:0;flex-shrink:0}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .pane-title-container[_ngcontent-%COMP%]   .buttons-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .pane-title-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;cursor:pointer;color:#999;opacity:.8;margin-left:6px}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .pane-title-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]:hover{opacity:1}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .pane-title-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:18px;height:18px;width:18px}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   split-pane[_ngcontent-%COMP%]{width:100%;-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;overflow:hidden}.container[_ngcontent-%COMP%]   .pane-container.selected[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{color:#fff}.container[_ngcontent-%COMP%]   .pane-container.selected[_ngcontent-%COMP%]   .pane-title-container[_ngcontent-%COMP%]{background-color:#ea8600;color:#fff}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .loading[_ngcontent-%COMP%]{position:absolute;top:0;left:0;width:100%;height:100%;z-index:150;color:#999;background-color:#fff;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .loading[_ngcontent-%COMP%]   .processing-msg-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .loading[_ngcontent-%COMP%]   .processing-msg-container[_ngcontent-%COMP%]   mat-spinner[_ngcontent-%COMP%]{margin-right:8px}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .loading[_ngcontent-%COMP%]   .progress-bar-wrapper[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;margin-top:18px;margin-right:-30px}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .loading[_ngcontent-%COMP%]   .progress-bar-wrapper[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{cursor:pointer;opacity:.8;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .loading[_ngcontent-%COMP%]   .progress-bar-wrapper[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]:hover{opacity:1}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .loading[_ngcontent-%COMP%]   .progress-bar-wrapper[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{color:#ccc;font-size:18px;width:18px;height:18px;margin-left:6px}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .loading[_ngcontent-%COMP%]   .progress-bar-container[_ngcontent-%COMP%]{height:7px;width:280px;border:1px solid #ccc;border-radius:99px;-moz-box-sizing:border-box;box-sizing:border-box;overflow:hidden}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .loading[_ngcontent-%COMP%]   .progress-bar-container[_ngcontent-%COMP%]   .progress-bar-body[_ngcontent-%COMP%]{height:100%;-webkit-transition:width .1s;transition:width .1s;background-color:#ccc}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .loading[_ngcontent-%COMP%]   .stay-on-page-info[_ngcontent-%COMP%]{font-size:12px;width:100%;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;margin-top:4px}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .loading[_ngcontent-%COMP%]   .error-msg[_ngcontent-%COMP%]{font-size:13px;padding:16px;color:#a00}.container[_ngcontent-%COMP%]   .resizer[_ngcontent-%COMP%]{height:100%;width:10px;top:0;left:50%;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;position:absolute;padding-left:1px;cursor:ew-resize;z-index:200}.container[_ngcontent-%COMP%]   .resizer[_ngcontent-%COMP%]:hover{background-color:rgba(0,0,0,.04)}.container[_ngcontent-%COMP%]   .resizer[_ngcontent-%COMP%]:hover   .resizer-line[_ngcontent-%COMP%]{border-color:#fcc986}.container[_ngcontent-%COMP%]   .resizer[_ngcontent-%COMP%]   .resizer-line[_ngcontent-%COMP%]{height:100%;width:1px;border-left:1px solid #999}  .model-explorer-processing-tasks-container{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;padding:12px}  .model-explorer-processing-tasks-container .processing-task-container{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;font-size:13px;margin:2px 0}  .model-explorer-processing-tasks-container .processing-task-container mat-spinner{margin-right:8px;width:16px}  .model-explorer-processing-tasks-container .processing-task-container mat-icon{color:#0a0;font-size:16px;width:16px;height:16px;margin-right:8px}"],
data:{animation:[Yy("hideProcessingPanel",[bz("* => void",Zy("150ms 100ms ease-out",$y({opacity:0,transform:"scale(0.95, 0.95)"})))])]},Pa:0});function JS(a){a&1&&(R(0,"mat-icon",15),Z(1," turn_left "),U())}function KS(a){if(a&1){const b=Zg();R(0,"div",16);V("click",function(c){t(b);const d=Y().da,e=Y(2);return x(e.Sq(c,d))});R(1,"div",17);Oq(2,"div",18);U()()}}
function LS(a,b){if(a&1){const c=Zg();R(0,"div",7)(1,"div",8)(2,"div",9);V("click",function(){const d=t(c).da,e=Y(2);e.ne.next({});e.jk="";e.uj=!1;qJ(e.wf,d.graph);e.wf.N.set(void 0);wJ(e.wf,e.wf.J());e.wf.F.set(void 0);sJ(e.wf,!1);return x()});R(3,"div",10);H(4,JS,2,0,"mat-icon",11);R(5,"div",12);Z(6);U()();R(7,"div",13);Z(8);U()();H(9,KS,3,0,"div",14);U()()}a&2&&(a=b.da,b=Y(2),Mp("data-id",a.id),D(),Up("padding-left",MS(b,a),"px"),Wp("selected",a.id===b.selectedGraphId()),D(3),Q("ngIf",NS(b,a)),
D(2),Cr(" ",a.id," "),D(2),Cr(" ",OS(a.Ir)," "),D(),Q("ngIf",b.Ds))}function PS(a,b){a&1&&(R(0,"div",4)(1,"div",5),Z(2),U(),H(3,LS,10,9,"div",6),U());a&2&&(a=b.da,b=Y(),D(2),Cr(" ",a.label," "),D(),Q("ngForOf",a.graphs)("ngForTrackBy",b.LE))}function MS(a,b){if(a.uj)return 24;let c;return 24+12*(null!=(c=b.graph.level)?c:0)}function NS(a,b){let c;return!a.uj&&0<(null!=(c=b.graph.level)?c:0)}function OS(a){return`${a} node${1===a?"":"s"}`}
var QS=class{constructor(a){this.wf=a;this.Mh=[];this.ne=new Jk;this.uj=!1;this.selectedGraphId=Cs(()=>{const b=rJ(this.wf);return b&&b.modelGraph?b.modelGraph.id:""});this.jk=""}Wq(a){this.jk=a.toLowerCase()}Sq(a,b){a.stopPropagation();this.ne.next({});this.jk="";this.uj=!1;hJ(this.wf,b.graph)}KE(a){return`${a}`}LE(a,b){return`${b.graph.collectionLabel}___${b.graph.id}`}get FB(){const a=[];this.uj=!1;for(const {label:b,collection:c,graphs:d}of this.Mh){const e={label:b,collection:c,graphs:[]};for(const f of d)""===
this.jk||f.id.toLowerCase().includes(this.jk)?e.graphs.push(f):this.uj=!0;0<e.graphs.length&&a.push(e)}return a}get Ds(){return 1===this.wf.La().length}};QS.K=function(a){return new (a||QS)(y(VJ))};
QS.Ca=Jf({type:QS,ha:[["graph-selector-panel"]],inputs:{Mh:"graphCollectionItems"},outputs:{ne:"onClose"},ka:!0,features:[Nr],ya:4,za:3,Ba:[["input",""],[1,"container"],["placeholder","Filter graphs",3,"input"],["class","collection-container",4,"ngFor","ngForOf","ngForTrackBy"],[1,"collection-container"],[1,"collection-label"],["class","graph-item-container",4,"ngFor","ngForOf","ngForTrackBy"],[1,"graph-item-container"],[1,"graph-item"],[1,"graph-info-wrapper",3,"click"],[1,"graph-label-wrapper"],
["class","indent-symbol",4,"ngIf"],[1,"graph-label"],[1,"node-count-label"],["class","action-button","matTooltip","Open in split pane","matTooltipPosition","right",3,"click",4,"ngIf"],[1,"indent-symbol"],["matTooltip","Open in split pane","matTooltipPosition","right",1,"action-button",3,"click"],[1,"block-container"],[1,"right-block"]],sa:function(a,b){if(a&1){const c=Zg();R(0,"div",1)(1,"input",2,0);V("input",function(){t(c);const d=Ar(2);return x(b.Wq(d.value))});U();H(3,PS,4,3,"div",3);U()}a&2&&
(Q("@transformPanel","showing"),D(3),Q("ngForOf",b.FB)("ngForTrackBy",b.KE))},Ha:[it,$s,dt,PE,AE,NE,cB,iI,hI,PD,Dx],styles:[".container[_ngcontent-%COMP%]{box-shadow:0 3px 5px -1px rgba(0,0,0,.2),0 6px 10px 0 rgba(0,0,0,.14),0 1px 18px 0 rgba(0,0,0,.12);-webkit-transform-origin:top center;transform-origin:top center;font-size:12px;padding:8px 0;border-radius:4px;border-top-left-radius:0;border-top-right-radius:0;max-height:100%;overflow-y:auto}.container[_ngcontent-%COMP%]   input[_ngcontent-%COMP%]{font-size:12px;margin:4px 10px;width:calc(100% - 20px);-moz-box-sizing:border-box;box-sizing:border-box;pointer-events:all;border:1px solid #ccc;height:20px;border-radius:4px;padding:0 2px}.container[_ngcontent-%COMP%]   .collection-container[_ngcontent-%COMP%]   .collection-label[_ngcontent-%COMP%]{font-weight:700;padding:4px 12px}.container[_ngcontent-%COMP%]   .graph-item-container[_ngcontent-%COMP%]   .graph-item[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;padding:4px 12px 4px 24px;cursor:pointer}.container[_ngcontent-%COMP%]   .graph-item-container[_ngcontent-%COMP%]   .graph-item[_ngcontent-%COMP%]:hover{background-color:rgba(0,0,0,.04)}.container[_ngcontent-%COMP%]   .graph-item-container[_ngcontent-%COMP%]   .graph-item.selected[_ngcontent-%COMP%]{background-color:#c2e7ff}.container[_ngcontent-%COMP%]   .graph-item-container[_ngcontent-%COMP%]   .graph-item[_ngcontent-%COMP%]   .graph-info-wrapper[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1}.container[_ngcontent-%COMP%]   .graph-item-container[_ngcontent-%COMP%]   .graph-item[_ngcontent-%COMP%]   .graph-label-wrapper[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .graph-item-container[_ngcontent-%COMP%]   .graph-item[_ngcontent-%COMP%]   .indent-symbol[_ngcontent-%COMP%]{font-size:16px;width:16px;height:16px;color:#999;margin-right:4px;-webkit-transform:rotate(-180deg);transform:rotate(-180deg)}.container[_ngcontent-%COMP%]   .graph-item-container[_ngcontent-%COMP%]   .graph-item[_ngcontent-%COMP%]   .graph-label[_ngcontent-%COMP%]{overflow:hidden;white-space:nowrap;text-overflow:ellipsis}.container[_ngcontent-%COMP%]   .graph-item-container[_ngcontent-%COMP%]   .graph-item[_ngcontent-%COMP%]   .node-count-label[_ngcontent-%COMP%]{color:#999;margin-left:12px}.container[_ngcontent-%COMP%]   .graph-item-container[_ngcontent-%COMP%]   .graph-item[_ngcontent-%COMP%]   .action-button[_ngcontent-%COMP%]{-webkit-flex-shrink:0;-ms-flex-negative:0;flex-shrink:0;width:16px;height:16px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;margin-left:6px;opacity:.8;cursor:pointer}.container[_ngcontent-%COMP%]   .graph-item-container[_ngcontent-%COMP%]   .graph-item[_ngcontent-%COMP%]   .action-button[_ngcontent-%COMP%]:hover{opacity:1}.container[_ngcontent-%COMP%]   .graph-item-container[_ngcontent-%COMP%]   .graph-item[_ngcontent-%COMP%]   .action-button[_ngcontent-%COMP%]   .block-container[_ngcontent-%COMP%]{border-radius:3px;border:1px solid #999;height:100%;width:100%;-moz-box-sizing:border-box;box-sizing:border-box;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:end;-webkit-justify-content:flex-end;-moz-box-pack:end;-ms-flex-pack:end;justify-content:flex-end;padding:1px}.container[_ngcontent-%COMP%]   .graph-item-container[_ngcontent-%COMP%]   .graph-item[_ngcontent-%COMP%]   .action-button[_ngcontent-%COMP%]   .block-container[_ngcontent-%COMP%]   .right-block[_ngcontent-%COMP%]{height:100%;width:50%;background-color:#999;border-top-right-radius:2px;border-bottom-right-radius:2px}"],
data:{animation:ZJ},Pa:0});const RS=["input"];function SS(a){a&1&&(R(0,"div",15),Z(1),U());a&2&&(a=Y(),D(),Cr(" ",a.Sx," nodes "))}function TS(a){if(a&1){const b=Zg();R(0,"mat-optgroup",16)(1,"mat-option")(2,"input",17,1);V("click",function(c){t(b);return x(c.stopPropagation())})("keydown.space",function(c){t(b);return x(c.stopPropagation())})("input",function(){t(b);const c=Ar(3),d=Y();return x(d.Wq(c.value))});U()()()}}
function US(a){if(a&1){const b=Zg();R(0,"div",24);V("click",function(c){t(b);const d=Y().da,e=Y(2);return x(e.Sq(c,d))});R(1,"div",25);Oq(2,"div",26);U()()}}function VS(a,b){a&1&&(R(0,"mat-option",20)(1,"div",21)(2,"div",22)(3,"div",7),Z(4),U(),R(5,"div",15),Z(6),U()(),H(7,US,3,0,"div",23),U()());a&2&&(a=b.da,b=Y(2),Wp("selected",a.graph===b.Yf.value),Q("value",a.graph),Mp("data-id",a.id),D(4),Br(a.id),D(2),Cr(" ",a.Ir," nodes "),D(),Q("ngIf",b.Ds))}
function WS(a,b){a&1&&(R(0,"mat-optgroup",18),H(1,VS,8,7,"mat-option",19),U());a&2&&(a=b.da,Q("label",a.label),D(),Q("ngForOf",a.graphs))}const XS=new OffscreenCanvas(500,300),YS={};function ZS(a,b=12,c=!1){var d=YS[a];null==d&&(d=XS.getContext("2d"),d.font=`${b}px "Google Sans Text", Arial, Helvetica, sans-serif`,c&&(d.font=`bold ${d.font}`),b=d.measureText(a).width,d=YS[a]=b);return d}
function $S(a,b){let c=!1;for(const d of a.Mh()){for(const e of d.graphs)if(e.graph.id===b){a.Sx=e.Ir;a.ys=d.label;a.Rx=d.collection;a.Yf.setValue(e.graph);c=!0;break}if(c)break}}function aT(a,b){b=new Bu({vc:Lu(Sv(a.C.position(),b),[{bb:"end",cb:"bottom",Na:"end",Ua:"top"}]),maxHeight:"calc(100% - 70px)",Vb:!0,fc:"cdk-overlay-transparent-backdrop",wc:a.C.C.C(),hb:"graph-selector-panel"});const c=a.C.create(b);a.Od=new Vv(QS,a.yb);c.bd().subscribe(()=>{c.dispose()});return c}
function bT(a,b){const c=aT(a,b);b=c.attach(a.Od);b.instance.Mh=a.Mh();b.instance.ne.subscribe(()=>{c.dispose()})}
var cT=class{constructor(a,b,c){this.Yd=a;this.C=b;this.yb=c;this.Yf=new wx;this.Sx=0;this.ys="";this.Mh=Cs(()=>{var d=this.Yd.config();if(!d)return[];var e=this.Yd.O();this.nodeLabelsToHide=new Set((d.nodeLabelsToHide||[]).map(g=>g.toLowerCase()));d=[];const f=this.bp().toLowerCase();for(const g of e){e={label:g.label,collection:g,graphs:[]};for(const h of g.graphs){if(""!==f&&!h.id.toLowerCase().includes(f))continue;const m=h.nodes.filter(u=>!this.nodeLabelsToHide.has(u.label.toLowerCase())).length,
p=ZS(` ${h.id}    ${m} nodes`)+30;e.graphs.push({id:h.id,graph:h,Ir:m,width:p})}0<e.graphs.length&&(d.push(e),ZS(g.label,12,!0))}return d});this.lC=Cs(()=>{let d=0;const e=this.Yd.O();for(const f of e)d+=f.graphs.length;return d});this.nodeLabelsToHide=new Set;this.bp=yr("");this.Od=null;this.selectedGraphId=Cs(()=>{const d=rJ(this.Yd);return d&&d.modelGraph?d.modelGraph.id:""});Ks(()=>{const d=this.selectedGraphId();d&&$S(this,d)})}Wq(a){this.bp.set(a)}Sq(a,b){a.stopPropagation();this.hD.close();
hJ(this.Yd,b.graph)}get Ds(){return 1===this.Yd.La().length}};cT.K=function(a){return new (a||cT)(y(VJ),y(cw),y(np))};
cT.Ca=Jf({type:cT,ha:[["graph-selector"]],eb:function(a,b){a&1&&(vr(OD,5),vr(RS,5));if(a&2){let c;wr(c=xr())&&(b.hD=c.first);wr(c=xr())&&(b.dC=c.first)}},ka:!0,features:[Nr],ya:21,za:6,Ba:[["selector",""],["input",""],[1,"container"],[1,"collection-label"],[1,"graph-selector-label",3,"click"],[1,"selector-label-container"],[1,"selector-label-content"],[1,"option-label"],["class","node-count-label",4,"ngIf"],[1,"triangle"],["matTooltip","Download processed graphs json",1,"mat-icon-container",3,"click"],
[1,"select-form"],["color","primary","panelClass","graph-selector-panel",3,"selectionChange","openedChange","formControl"],["class","graph-search-optgroup",4,"ngIf"],[3,"label",4,"ngFor","ngForOf"],[1,"node-count-label"],[1,"graph-search-optgroup"],["placeholder","Filter graphs",3,"click","keydown.space","input"],[3,"label"],[3,"selected","value",4,"ngFor","ngForOf"],[3,"value"],[1,"option-wrapper"],[1,"option-container"],["class","action-button","matTooltip","Open in split pane","matTooltipPosition",
"right",3,"click",4,"ngIf"],["matTooltip","Open in split pane","matTooltipPosition","right",1,"action-button",3,"click"],[1,"block-container"],[1,"right-block"]],sa:function(a,b){if(a&1){const c=Zg();R(0,"div",2)(1,"div",3);Z(2);U();R(3,"div",4,0);V("click",function(){t(c);const d=Ar(4);return x(bT(b,d))});R(5,"div",5)(6,"div",6)(7,"div",7);Z(8);U();H(9,SS,2,1,"div",8);U();R(10,"div",9);Z(11,"\u25bc");U()()();R(12,"div",10);V("click",function(){t(c);if(null!=b.Rx){var d=document.createElement("a");
d.download=`${b.ys}.json`;var e=`data:text/json;charset=utf-8, ${encodeURIComponent(JSON.stringify(b.Rx,null,2))}`;wd(d,td(e));d.click()}return x()});R(13,"mat-icon");Z(14,"download");U()();R(15,"mat-form-field",11)(16,"mat-label");Z(17,"Select an option");U();R(18,"mat-select",12);V("selectionChange",function(){t(c);b.Yf.value&&($S(b,b.Yf.value.id),qJ(b.Yd,b.Yf.value),b.Yd.N.set(void 0),wJ(b.Yd,b.Yd.J()),b.Yd.F.set(void 0),sJ(b.Yd,!1));return x()})("openedChange",function(d){t(c);d||(b.dC.R.value=
"",b.bp.set(""));return x()});H(19,TS,4,0,"mat-optgroup",13)(20,WS,2,2,"mat-optgroup",14);U()()()}a&2&&(D(2),Cr(" ",b.ys," "),D(6),Br((null==b.Yf.value?null:b.Yf.value.id)||"-"),D(),Q("ngIf",b.Yf.value),D(9),Q("formControl",b.Yf),D(),Q("ngIf",1<b.lC()),D(),Q("ngForOf",b.Mh()))},Ha:[it,$s,dt,PE,AE,NE,cB,bB,qA,iI,hI,PD,OD,Vz,vz,Dx,Nw,yx],styles:[".container[_ngcontent-%COMP%]{font-size:12px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;line-height:14px;position:relative}.container[_ngcontent-%COMP%]   .collection-label[_ngcontent-%COMP%]{font-weight:700;margin-right:6px}.container[_ngcontent-%COMP%]   .select-form[_ngcontent-%COMP%]{height:0;max-height:0;position:absolute;top:26px;right:28px}.container[_ngcontent-%COMP%]   .select-form[_ngcontent-%COMP%]    >*{height:0}.container[_ngcontent-%COMP%]   .selector-label-content[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1}.container[_ngcontent-%COMP%]   .option-label[_ngcontent-%COMP%]{overflow:hidden;white-space:nowrap;text-overflow:ellipsis;margin-right:2px}.container[_ngcontent-%COMP%]   .node-count-label[_ngcontent-%COMP%]{color:#999;margin-left:12px}.container[_ngcontent-%COMP%]   .triangle[_ngcontent-%COMP%]{font-size:10px;margin-left:8px}.container[_ngcontent-%COMP%]   .graph-selector-label[_ngcontent-%COMP%]{font-size:12px;border:1px solid #aaa;border-radius:3px;padding:2px 4px;cursor:pointer;-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;height:24px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-moz-box-sizing:border-box;box-sizing:border-box}.container[_ngcontent-%COMP%]   .graph-selector-label[_ngcontent-%COMP%]   .selector-label-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1}.container[_ngcontent-%COMP%]   .graph-selector-label[_ngcontent-%COMP%]:hover{border-color:#ea8600}.container[_ngcontent-%COMP%]   .mat-icon-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;margin-left:4px;cursor:pointer;opacity:.6}.container[_ngcontent-%COMP%]   .mat-icon-container[_ngcontent-%COMP%]:hover{opacity:1}  .graph-selector-panel{background-color:#fff}  .graph-selector-panel mat-option{height:24px;font-size:12px;min-height:unset;padding:1px 0;padding-left:20px!important;background-color:none!important}  .graph-selector-panel mat-option .option-wrapper{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}  .graph-selector-panel mat-option .option-container{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;letter-spacing:normal;font-family:Google Sans Text,Arial,Helvetica,sans-serif;-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1}  .graph-selector-panel mat-option .option-container .option-label{overflow:hidden;white-space:nowrap;text-overflow:ellipsis;color:#000!important}  .graph-selector-panel mat-option .option-container .node-count-label{color:#999}  .graph-selector-panel mat-option .action-button{-webkit-flex-shrink:0;-ms-flex-negative:0;flex-shrink:0;width:16px;height:16px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;margin-left:4px;opacity:.8;cursor:pointer}  .graph-selector-panel mat-option .action-button:hover{opacity:1}  .graph-selector-panel mat-option .action-button .block-container{border-radius:3px;border:1px solid #999;height:100%;width:100%;-moz-box-sizing:border-box;box-sizing:border-box;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:end;-webkit-justify-content:flex-end;-moz-box-pack:end;-ms-flex-pack:end;justify-content:flex-end;padding:1px}  .graph-selector-panel mat-option .action-button .block-container .right-block{height:100%;width:50%;background-color:#999;border-top-right-radius:2px;border-bottom-right-radius:2px}  .graph-selector-panel mat-option span{white-space:break-spaces!important;word-break:break-all;line-height:14px!important;width:100%;-moz-box-sizing:border-box;box-sizing:border-box;padding-right:10px}  .graph-selector-panel mat-option mat-pseudo-checkbox{display:none}  .graph-selector-panel mat-option.selected{background-color:#fff1de!important}  .graph-selector-panel mat-optgroup span{min-height:unset;font-size:12px;font-weight:700;padding-left:8px}  .graph-selector-panel mat-optgroup.graph-search-optgroup{pointer-events:none}  .graph-selector-panel mat-optgroup.graph-search-optgroup span{padding:0}  .graph-selector-panel mat-optgroup.graph-search-optgroup mat-option{padding-left:16px!important;padding-right:12px!important;height:28px}  .graph-selector-panel mat-optgroup.graph-search-optgroup input{font-size:12px;width:100%;-moz-box-sizing:border-box;box-sizing:border-box;pointer-events:all;border:1px solid #ccc;height:20px;border-radius:4px;padding:0 2px}"],
Pa:0});function dT(a){a&1&&(R(0,"div",2),Z(1," Model Explorer "),R(2,"span",3),Z(3),U(),Z(4," is available. You are running "),R(5,"span",3),Z(6),U(),Z(7,". "),Oq(8,"br")(9,"br"),Z(10," Run the following command in your console to upgrade: "),R(11,"div",4),Z(12," pip install -U ai-edge-model-explorer "),U()());a&2&&(a=Y(2),D(3),Cr("v",a.info().version,""),D(3),Cr("v",a.info().runningVersion,""))}
function eT(a){a&1&&(R(0,"div",1)(1,"mat-icon"),Z(2,"upgrade"),U(),Z(3," New version available "),U(),H(4,dT,13,2,"ng-template",null,0,Rr));a&2&&(a=Ar(5),Q("xapInlineDialog",a)("hoverDelayMs",100))}function fT(a){ta(function*(){try{const b=yield fetch("/api/v1/check_new_version");if(b.ok){const c=yield b.json();a.info.set(c)}}catch(b){}})}var gT=class{constructor(){this.info=yr({version:"",runningVersion:""});fT(this)}};gT.K=function(a){return new (a||gT)};gT.na=Pd({ma:gT,ba:gT.K,fa:"root"});
var hT=class{constructor(a){this.C=a;this.info=this.C.info}};hT.K=function(a){return new (a||hT)(y(gT))};hT.Ca=Jf({type:hT,ha:[["new-version-chip"]],ka:!0,features:[Nr],ya:1,za:1,Ba:[["upgrade",""],["dialogLabel","upgrade",1,"container",3,"xapInlineDialog","hoverDelayMs"],[1,"model-explorer-upgrade-popup"],[1,"bold"],[1,"code"]],sa:function(a,b){a&1&&H(0,eT,6,2);a&2&&Gq(""!==b.info().version?0:-1)},Ha:[it,iI,hI,HG,EG],styles:[".container[_ngcontent-%COMP%] {\n  font-size: 12px;\n  display: flex;\n  align-items: center;\n  color: #ab6c17;\n  background-color: #ffefd9;\n  line-height: 18px;\n  border-radius: 99px;\n  font-weight: 500;\n  padding: 2px 10px 2px 4px;\n  cursor: pointer;\n\n  mat-icon {\n    font-size: 16px;\n    width: 16px;\n    height: 16px;\n  }\n}\n\n  .model-explorer-upgrade-popup {\n  padding: 8px;\n  font-size: 12px;\n  background-color: white;\n  line-height: normal;\n\n  .bold {\n    font-weight: 500;\n  }\n\n  .code {\n    background-color: #f1f1f1;\n    font-family: monospace;\n    margin-top: 8px;\n    padding: 4px;\n    font-size: 11px;\n  }\n}"]});function iT(a){a&1&&Oq(0,"mat-spinner",6)}function jT(a){a&1&&(R(0,"mat-icon"),Z(1,"add_circle_outline"),U())}function kT(a){a&1&&(R(0,"div",8),Z(1," Add custom data and styles to the nodes in the current model "),U())}
function lT(a){if(a&1){const b=Zg();R(0,"div",16)(1,"div",17);Z(2,"OR");U()();R(3,"div",18)(4,"div",13);Z(5,"From CNS");U();R(6,"textarea",19,3);V("focus",function(){t(b);const c=Ar(7);return x(c.select())})("input",function(){t(b);const c=Y(2);en(c.nz);return x()})("change",function(){t(b);const c=Ar(7),d=Y(2);PI("extension_dropdown_json_cns_path",c.value);d.gr=c.value;return x()});U();R(8,"button",20);V("click",function(){t(b);const c=Ar(7),d=Y(2);return x(mT(d,c))});Z(9);U()()}if(a&2){a=Ar(7);
const b=Y(2);D(6);Q("value",b.gr);D(2);Q("disabled",nT(b,a));D();Cr(" ",b.gi()?"Loading...":"Load"," ")}}
function oT(a){if(a&1){const b=Zg();R(0,"div",9)(1,"div",10)(2,"div");Z(3,"Load json data file");U();R(4,"a",11);Z(5," Guide ");U()();R(6,"div",12)(7,"div",13);Z(8,"From computer");U();R(9,"button",14);V("click",function(){t(b);const c=Ar(12);return x(c.click())});Z(10," Upload ");U()();R(11,"input",15,2);V("change",function(){t(b);const c=Ar(12),d=Y();return x(d.Tq(c))});U();H(13,lT,10,3);U()}a&2&&(a=Y(),D(13),Gq(a.ke?13:-1))}
function mT(a,b){return ta(function*(){const c=xJ(a.D);if(c){var d=b.value.trim();yield FL(a.C,d,c);var e;null==(e=a.Xv)||rG(e)}})}function nT(a,b){b=b.value.trim();return a.gi()||""===b||!b.startsWith("/cns")||!b.endsWith(".json")}
var pT=class{constructor(a,b,c,d){this.D=a;this.nz=b;this.C=d;this.extensions=[];this.Pw=!0;this.gr="";this.fd={minWidth:0,minHeight:0};this.VB={minWidth:0,minHeight:0,maxHeight:500};this.gi=this.C.gi;this.gr=window.localStorage.getItem("extension_dropdown_json_cns_path")||""}Tq(a){var b=a.files;if(b){var c=xJ(this.D);if(c){for(const d of b)b=new FileReader,b.onload=e=>{const f=tI();try{let h;var g=null==(h=e.target)?void 0:h.result;const m=JSON.parse(g);e={};null!=m.results&&null==m.results.results?
c&&(e[c.id]=m):e=m;CL(this.C,f,d.name,c,e)}catch(h){CL(this.C,f,d.name,c),DL(this.C,f,{[c.id]:{results:{}}},c,`Failed to process JSON file. ${h}`)}finally{let h;null==(h=this.Xv)||rG(h)}},b.readAsText(d);a.value=""}}}get ke(){return!1}};pT.K=function(a){return new (a||pT)(y(VJ),y(Tr),y(QI),y(LL))};
pT.Ca=Jf({type:pT,ha:[["node-data-provider-dropdown"]],eb:function(a,b){a&1&&vr(GG,5);if(a&2){let c;wr(c=xr())&&(b.Xv=c.first)}},ka:!0,features:[Nr],ya:10,za:6,Ba:[["help",""],["dropdown",""],["input",""],["textarea",""],["dialogLabel","help",1,"container",3,"xapInlineDialog","overlaySize","hoverDelayMs"],["dialogLabel","dropdown",1,"icon-container",3,"xapInlineDialogClick","overlaySize"],["color","primary","diameter","18"],[1,"label"],[1,"model-explorer-help-popup"],[1,"model-explorer-node-data-provider-dropdown"],
[1,"section-label","upload"],["href","https://github.com/google-ai-edge/model-explorer/wiki/2.-User-Guide#custom-node-data","target","_blank"],[1,"upload-container"],[1,"description"],["mat-flat-button","","color","primary",1,"upload-json-file-button","upload",3,"click"],["type","file","multiple","","accept",".json",1,"upload-per-node-data-input",3,"change"],[1,"or-divider"],[1,"or-label"],[1,"load-json-from-cns-container"],[3,"focus","input","change","value"],["mat-flat-button","","color","primary",
1,"upload-json-file-button",3,"click","disabled"]],sa:function(a,b){a&1&&(R(0,"div",4)(1,"div",5),H(2,iT,1,0,"mat-spinner",6)(3,jT,2,0,"mat-icon"),R(4,"span",7),Z(5,"Add per-node data"),U()()(),H(6,kT,2,0,"ng-template",null,0,Rr)(8,oT,14,1,"ng-template",null,1,Rr));if(a&2){a=Ar(7);const c=Ar(9);Q("xapInlineDialog",a)("overlaySize",b.fd)("hoverDelayMs",10);D();Q("xapInlineDialogClick",c)("overlaySize",b.VB);D();Gq(b.gi()?2:3)}},Ha:[it,BB,AB,vB,wD,vD,sD,iI,hI,OE,HG,EG,GG],styles:[".container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;cursor:pointer;color:#666;padding-bottom:12px;margin-bottom:-12px}.container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{margin-right:4px}.container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   mat-spinner[_ngcontent-%COMP%]{margin-right:6px}.container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   .label[_ngcontent-%COMP%]{font-size:13px}.container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]:hover{color:#444}  .model-explorer-node-data-provider-dropdown{font-size:12px;background-color:#fff;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;padding-bottom:8px}  .model-explorer-node-data-provider-dropdown .section-label{padding:8px 12px;font-size:11px;background:#f1f1f1;font-weight:500;text-transform:uppercase}  .model-explorer-node-data-provider-dropdown .section-label .address-container{text-transform:none;font-weight:400;letter-spacing:normal;color:#777;line-height:12px;padding-bottom:5px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:start;-webkit-align-items:flex-start;-moz-box-align:start;-ms-flex-align:start;align-items:flex-start;margin-top:-4px;margin-left:-2px}  .model-explorer-node-data-provider-dropdown .section-label .address-container .address-label{overflow:hidden;line-break:anywhere;line-height:12px;-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;padding-top:3px}  .model-explorer-node-data-provider-dropdown .section-label .address-container .settings-icon-container{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;margin-right:4px;pointer-events:all;cursor:pointer;opacity:.7}  .model-explorer-node-data-provider-dropdown .section-label .address-container .settings-icon-container:hover{opacity:1}  .model-explorer-node-data-provider-dropdown .section-label .address-container .settings-icon-container mat-icon{font-size:16px;width:16px;height:16px}  .model-explorer-node-data-provider-dropdown .section-label.upload{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;min-width:180px}  .model-explorer-node-data-provider-dropdown .section-label.upload a{font-size:11px;letter-spacing:normal;text-transform:none;pointer-events:all}  .model-explorer-node-data-provider-dropdown .description{font-size:12px;color:#777;line-height:18px;letter-spacing:normal}  .model-explorer-node-data-provider-dropdown .upload-container{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;-webkit-box-align:start;-webkit-align-items:flex-start;-moz-box-align:start;-ms-flex-align:start;align-items:flex-start;padding:0 16px;margin-top:12px}  .model-explorer-node-data-provider-dropdown .load-json-from-cns-container{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;-webkit-box-align:start;-webkit-align-items:flex-start;-moz-box-align:start;-ms-flex-align:start;align-items:flex-start;padding:0 16px;margin-top:10px;width:200px;-moz-box-sizing:border-box;box-sizing:border-box}  .model-explorer-node-data-provider-dropdown .load-json-from-cns-container textarea{width:100%;height:48px;-moz-box-sizing:border-box;box-sizing:border-box;margin:2px 0;resize:none;border-radius:3px;font-family:Google Sans Text,Google Sans,Arial,Helvetica,sans-serif;font-size:11px;padding:2px}  .model-explorer-node-data-provider-dropdown .upload-json-file-button{margin:4px 0;width:90px;height:30px;font-size:12px!important;letter-spacing:normal!important}  .model-explorer-node-data-provider-dropdown .upload-json-file-button.upload{margin-top:2px}  .model-explorer-node-data-provider-dropdown .upload-json-file-button   .mat-mdc-button-touch-target{display:none}  .model-explorer-node-data-provider-dropdown .or-divider{height:1px;border-top:1px solid #eee;position:relative;margin-top:12px}  .model-explorer-node-data-provider-dropdown .or-divider .or-label{font-size:10px;top:-9px;color:#aaa;position:absolute;padding:2px;background-color:#fff;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;width:16px;left:calc(50% - 8px)}  .model-explorer-node-data-provider-dropdown .upload-per-node-data-input{display:none}  .model-explorer-node-data-provider-dropdown .loading-extension,   .model-explorer-node-data-provider-dropdown .no-extension{padding:12px 0 4px 16px;color:#999;font-size:13px;letter-spacing:normal}"],
Pa:0});const qT=(a,b)=>b.type;
function rT(a,b){if(a&1){const c=Zg();R(0,"div",18);V("click",function(){var d=t(c).da,e=Y(2).Cb,f=Y();d=d.type;var g=f.Rc[e];if(1!==g.matchTypes.length||g.matchTypes[0]!==d)e=f.Rc[e],g=e.matchTypes.indexOf(d),0<=g?e.matchTypes.splice(g,1):e.matchTypes.push(d),f.fi.emit([...f.Rc]);return x()});R(1,"mat-icon");Z(2);U()()}a&2&&(a=b.da,b=Y(2).da,Y(),Wp("selected",b.matchTypes.includes(a.type))("disabled",1===b.matchTypes.length&&b.matchTypes[0]===a.type),Q("matTooltip",a.Rd),D(2),Br(a.type))}
function sT(a){if(a&1){const b=Zg();R(0,"div",12)(1,"div",13);Z(2,"R");U();R(3,"div",14);Z(4,"E");U()();R(5,"input",15,1);V("change",function(){t(b);const c=Ar(6),d=Y().Cb,e=Y();e.Rc[d].queryRegex=c.value.trim();e.fi.emit([...e.Rc]);return x()});U();R(7,"div",16);Lq(8,rT,3,6,"div",17,qT);U()}if(a&2){a=Y().da;const b=Y();D(5);Q("value",a.queryRegex);D(3);Nq(b.Ru)}}
function tT(a){if(a&1){const b=Zg();R(0,"div",19);Z(1," [\u00b7\u00b7] ");U();R(2,"div",20)(3,"input",21,2);V("change",function(){t(b);const c=Ar(4),d=Ar(8),e=Ar(12),f=Y().Cb,g=Y();return x(uT(g,f,c.value.trim(),d.value.trim(),e.value.trim()))});U();R(5,"div",22);Z(6,":");U();R(7,"input",23,3);V("change",function(){t(b);const c=Ar(4),d=Ar(8),e=Ar(12),f=Y().Cb,g=Y();return x(uT(g,f,c.value.trim(),d.value.trim(),e.value.trim()))});U();R(9,"div",24);Z(10,",");U();R(11,"input",25,4);V("change",function(){t(b);
const c=Ar(4),d=Ar(8),e=Ar(12),f=Y().Cb,g=Y();return x(uT(g,f,c.value.trim(),d.value.trim(),e.value.trim()))});U()()}a&2&&(a=Y().da,Y(),D(3),Q("value",a.attrName),D(4),Q("value",vT(a.min)),D(4),Q("value",vT(a.max)))}function wT(a,b){a&1&&(R(0,"option",28),Z(1),U());a&2&&(a=b.da,b=Y(2).da,Q("value",a.type)("selected",a.type===b.nodeType),D(),Cr(" ",a.label," "))}
function xT(a){if(a&1){const b=Zg();R(0,"div",26)(1,"mat-icon");Z(2,"device_hub");U()();R(3,"select",27,5);V("change",function(){t(b);const c=Ar(4),d=Y().Cb,e=Y();return x(e.ow(d,c.value))});Lq(5,wT,2,3,"option",28,qT);U()}a&2&&(a=Y(2),D(5),Nq(a.Su))}function yT(a){if(a&1){const b=Zg();R(0,"div",29);V("click",function(){t(b);const c=Y().Cb,d=Y();d.Rc.splice(c,1);d.fi.emit([...d.Rc]);return x()});R(1,"mat-icon");Z(2,"delete");U()()}}
function zT(a,b){a&1&&(R(0,"div",7),H(1,sT,10,1)(2,tT,13,3)(3,xT,7,0)(4,yT,3,0,"div",11),U());if(a&2){let c;a=b.da;b=Y();D();Gq((c=a.type)===b.lk.st?1:c===b.lk.ft?2:c===b.lk.vz?3:-1);D(3);Gq("node_type"!==a.type?4:-1)}}function AT(a){a&1&&(R(0,"div",31)(1,"div",13),Z(2,"R"),U(),R(3,"div",14),Z(4,"E"),U()())}function BT(a){a&1&&(R(0,"div",32),Z(1," [\u00b7\u00b7] "),U())}
function CT(a,b){if(a&1){const c=Zg();R(0,"div",30);V("click",function(){var d=t(c).da,e=Y();d=d.type;switch(d){case "regex":e.Rc.push({type:d,queryRegex:"",matchTypes:["title"]});break;case "attr_value_range":e.Rc.push({type:d,attrName:"",min:Number.NEGATIVE_INFINITY,max:Number.POSITIVE_INFINITY});break;case "node_type":e.Rc.push({type:d,nodeType:"op_nodes"})}e.fi.emit([...e.Rc]);return x()});H(1,AT,5,0,"div",31)(2,BT,2,0,"div",32);R(3,"div",33);Z(4);U()()}if(a&2){let c;a=b.da;b=Y();D();Gq((c=a.type)===
b.lk.st?1:c===b.lk.ft?2:-1);D(3);Br(a.label)}}function uT(a,b,c,d,e){let f=Number.NEGATIVE_INFINITY,g=Number.POSITIVE_INFINITY;""===d||isNaN(Number(d))||(f=Number(d));""===e||isNaN(Number(e))||(g=Number(e));b=a.Rc[b];b.attrName=c;b.min=f;b.max=g;a.fi.emit([...a.Rc])}function vT(a){return null==a||a===Number.NEGATIVE_INFINITY||a===Number.POSITIVE_INFINITY?"":`${a}`}
var DT=class{constructor(){this.queries=[];this.fi=new Jk;this.lk=mI;this.Ru=[{type:"title",Rd:"Match label"},{type:"list",Rd:"Match attributes"},{type:"input",Rd:"Match inputs (op node only)"},{type:"output",Rd:"Match outputs (op node only)"}];this.Su=[{type:"op_nodes",label:"Op nodes only"},{type:"layer_nodes",label:"Layer nodes only"},{type:"op_and_layer_nodes",label:"Op and layer nodes"}];this.Pu=[{type:"regex",label:"Regex"},{type:"attr_value_range",label:"Attribute value range"}];this.Rc=[]}Fb(){this.Rc=
JSON.parse(JSON.stringify(this.queries))}ow(a,b){this.Rc[a].nodeType=b;this.fi.emit([...this.Rc])}};DT.K=function(a){return new (a||DT)(y(Tr))};
DT.Ca=Jf({type:DT,ha:[["complex-queries"]],inputs:{queries:"queries"},outputs:{fi:"queriesUpdated"},ka:!0,features:[Nr],ya:11,za:1,Ba:[["menu","matMenu"],["queryInput",""],["inputAttrName",""],["inputMin",""],["inputMax",""],["select",""],[1,"container"],[1,"query-container"],[1,"btn-add-query",3,"matMenuTriggerFor"],[1,"model-explorer-add-query-menu"],[1,"menu-item"],["matTooltip","Delete query","matTooltipPosition","right",1,"icon-container","delete"],["matTooltip","Regex","n","","matTooltipPosition",
"left",1,"model-explorer-regex-icon"],[1,"r"],[1,"e"],["placeholder","Regex",1,"regex","common-input",3,"change","value"],[1,"targets-container"],[1,"icon-container","match-type",3,"matTooltip","selected","disabled"],[1,"icon-container","match-type",3,"click","matTooltip"],["matTooltip","Attribute value range","matTooltipPosition","left",1,"model-explorer-attr-value-range-icon"],[1,"attrs-value-range-container"],["placeholder","Attr name",1,"common-input","attr-name",3,"change","value"],[1,"colon"],
["placeholder","min",1,"common-input","min",3,"change","value"],[1,"to"],["placeholder","max",1,"common-input","max",3,"change","value"],["matTooltip","Node type","matTooltipPosition","left",1,"model-explorer-node-type-icon"],[1,"node-type-selector",3,"change"],[3,"value","selected"],["matTooltip","Delete query","matTooltipPosition","right",1,"icon-container","delete",3,"click"],[1,"menu-item",3,"click"],[1,"model-explorer-regex-icon"],[1,"model-explorer-attr-value-range-icon"],[1,"label"]],sa:function(a,
b){a&1&&(R(0,"div",6),Lq(1,zT,5,2,"div",7,Jq),R(3,"div",8)(4,"mat-icon"),Z(5,"add"),U(),Z(6," Add query "),U(),R(7,"mat-menu",9,0),Lq(9,CT,5,2,"div",10,qT),U()());a&2&&(a=Ar(8),D(),Nq(b.Rc),D(2),Q("matMenuTriggerFor",a),D(6),Nq(b.Pu))},Ha:[it,$C,CC,QC,YC,PE,AE,NE,iI,hI,HG],styles:[".query-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;gap:6px;margin-bottom:4px;position:relative;padding-right:36px}.query-container[_ngcontent-%COMP%]   input[_ngcontent-%COMP%]{height:22px;border:1px solid #ccc;border-radius:4px;-moz-box-sizing:border-box;box-sizing:border-box;padding:0 4px;outline:none}.query-container[_ngcontent-%COMP%]   input.regex[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1}.query-container[_ngcontent-%COMP%]   .attrs-value-range-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;width:240px}.query-container[_ngcontent-%COMP%]   .attrs-value-range-container[_ngcontent-%COMP%]   .colon[_ngcontent-%COMP%], .query-container[_ngcontent-%COMP%]   .attrs-value-range-container[_ngcontent-%COMP%]   .to[_ngcontent-%COMP%]{font-weight:700;margin:0 2px;height:22px}.query-container[_ngcontent-%COMP%]   .attrs-value-range-container[_ngcontent-%COMP%]   input[_ngcontent-%COMP%]{min-width:0}.query-container[_ngcontent-%COMP%]   .attrs-value-range-container[_ngcontent-%COMP%]   input.attr-name[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1}.query-container[_ngcontent-%COMP%]   .attrs-value-range-container[_ngcontent-%COMP%]   input.max[_ngcontent-%COMP%], .query-container[_ngcontent-%COMP%]   .attrs-value-range-container[_ngcontent-%COMP%]   input.min[_ngcontent-%COMP%]{width:50px}.query-container[_ngcontent-%COMP%]   .node-type-selector[_ngcontent-%COMP%]{border:1px solid #ccc;border-radius:4px;outline:none;height:23px;cursor:pointer;-moz-box-sizing:border-box;box-sizing:border-box;width:160px}.query-container[_ngcontent-%COMP%]   .icon-container.delete[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;position:absolute;right:0;top:4px;cursor:pointer;opacity:.6}.query-container[_ngcontent-%COMP%]   .icon-container.delete[_ngcontent-%COMP%]:hover{opacity:1}.query-container[_ngcontent-%COMP%]   .icon-container.delete[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:16px;width:16px;height:16px}.targets-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;gap:2px}.targets-container[_ngcontent-%COMP%]   .icon-container.match-type[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;background-color:#ddd;border-radius:4px;cursor:pointer;padding:4px}.targets-container[_ngcontent-%COMP%]   .icon-container.match-type.selected[_ngcontent-%COMP%]{background-color:#a3c9ff}.targets-container[_ngcontent-%COMP%]   .icon-container.match-type.disabled[_ngcontent-%COMP%]{cursor:default}.targets-container[_ngcontent-%COMP%]   .icon-container.match-type[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:14px;width:14px;height:14px;color:#333}.btn-add-query[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;cursor:pointer;width:-webkit-fit-content;width:-moz-fit-content;width:fit-content;line-height:16px;opacity:.6;white-space:nowrap;margin-top:2px;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;margin-left:19px}.btn-add-query[_ngcontent-%COMP%]:hover{opacity:1}.btn-add-query[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:16px;width:16px;height:16px;margin-right:2px}  .model-explorer-add-query-menu .menu-item{letter-spacing:normal!important;font-family:Google Sans Text,Arial,Helvetica,sans-serif!important;font-size:12px;padding:0 8px;cursor:pointer;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}  .model-explorer-add-query-menu .menu-item .label{margin-left:6px}  .model-explorer-add-query-menu .menu-item:hover{background-color:#eee}  .model-explorer-regex-icon{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;width:16px;height:16px;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;font-weight:700;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;color:#aaa}  .model-explorer-regex-icon .r{font-size:12px}  .model-explorer-regex-icon .e{font-size:8px;margin-top:2px}  .model-explorer-attr-value-range-icon{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;width:16px;height:16px;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;font-size:10px;font-weight:700;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;color:#aaa}  .model-explorer-node-type-icon{width:16px;height:16px}  .model-explorer-node-type-icon mat-icon{font-size:16px;width:16px;height:16px;color:#aaa}"],
Pa:0});function ET(a){a&1&&(R(0,"div",3),Oq(1,"io-tree",4),U());a&2&&(a=Y(),D(),Q("data",a.Dv)("rendererId",a.rendererId)("tooltipPosition","right"))}var FT=class{constructor(){this.nodes=[];this.rendererId="";this.Lw="node";this.Yr={minWidth:320,maxWidth:640,minHeight:0};this.Dv=[]}uc(a){a.nodes&&(this.Dv=aL(this.nodes,"incoming"))}get label(){const a=this.nodes.length;return`${a} ${this.Lw}${1===a?"":"s"}`}};FT.K=function(a){return new (a||FT)};
FT.Ca=Jf({type:FT,ha:[["node-list-viewer"]],inputs:{nodes:"nodes",rendererId:"rendererId",Lw:"labelSuffix"},ka:!0,features:[Hg,Nr],ya:5,za:5,Ba:[["nodeList",""],[1,"container"],["panelClass","model-explorer-node-list-panel","dialogLabel","nodeList",1,"label-container",3,"xapInlineDialogClick","overlaySize"],[1,"model-explorer-node-list"],[3,"data","rendererId","tooltipPosition"]],sa:function(a,b){a&1&&(R(0,"div",1)(1,"div",2),Z(2),U(),H(3,ET,2,3,"ng-template",null,0,Rr),U());a&2&&(a=Ar(4),D(),Wp("disabled",
0===b.nodes.length),Q("xapInlineDialogClick",a)("overlaySize",b.Yr),D(),Cr(" ",b.label," "))},Ha:[it,$K,iI,HG,GG],styles:[".container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;position:relative}.label-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;padding:4px 6px;border-radius:4px;background-color:#a3c9ff;line-height:14px;cursor:pointer;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;opacity:.8}.label-container[_ngcontent-%COMP%]:hover{opacity:1}.label-container.disabled[_ngcontent-%COMP%]{pointer-events:none;background-color:#ddd}  .model-explorer-node-list{font-size:12px;background-color:#fff;line-height:normal}  .model-explorer-node-list-panel xap-inline-dialog-container{width:100%}"],
Pa:0});var GT=new gf("CDK_DRAG_PARENT");var HT=new gf("CdkDragHandle"),IT=class{get disabled(){return this.Ib}set disabled(a){this.Ib=a;this.C.next(this)}constructor(a,b){this.element=a;this.D=b;this.C=new hj;this.Ib=!1;null==b||b.T(this)}va(){let a;null==(a=this.D)||a.ca(this);this.C.complete()}};IT.K=function(a){return new (a||IT)(y(wi),y(GT,12))};IT.Fa=Uf({type:IT,ha:[["","cdkDragHandle",""]],Ka:[1,"cdk-drag-handle"],inputs:{disabled:[2,"cdkDragHandleDisabled","disabled",As]},ka:!0,features:[Lr([{pa:HT,Ab:IT}]),Lp]});var JT=new gf("CDK_DRAG_CONFIG");const KT=Lt({passive:!1,capture:!0}),LT=new Set;var Os=class{};Os.K=function(a){return new (a||Os)};Os.Ca=Jf({type:Os,ha:[["ng-component"]],Ka:["cdk-drag-resets-container",""],ka:!0,features:[Nr],ya:0,za:0,sa:function(){},styles:["@layer cdk-resets{.cdk-drag-preview{background:none;border:none;padding:0;color:inherit}}\n"],pb:2,Pa:0});function MT(a,b){a.D.add(b);1===a.D.size&&Qk(a.qa,()=>{a.ia.addEventListener("touchmove",a.T,KT)})}
function NT(a,b){a.C.update(c=>{const d=c.indexOf(b);return-1<d?(c.splice(d,1),[...c]):c});0===a.C().length&&a.O()}function OT(a,b){a.D.delete(b);NT(a,b);0===a.D.size&&a.ia.removeEventListener("touchmove",a.T,KT)}
function PT(a,b,c){-1<a.C().indexOf(b)||(a.ea(),a.C.update(d=>[...d,b]),1===a.C().length&&(c=c.type.startsWith("touch"),a.F.set(c?"touchend":"mouseup",{handler:d=>a.N.next(d),options:!0}).set("scroll",{handler:d=>a.scroll.next(d),options:!0}).set("selectstart",{handler:a.ja,options:KT}),c||a.F.set("mousemove",{handler:d=>a.J.next(d),options:KT}),Qk(a.qa,()=>{a.F.forEach((d,e)=>{a.ia.addEventListener(e,d.handler,d.options)})})))}
function QT(a,b){const c=[a.scroll];b&&b!==a.ia&&c.push(new bj(d=>Qk(a.qa,()=>{const e=f=>{a.C().length&&d.next(f)};b.addEventListener("scroll",e,!0);return()=>{b.removeEventListener("scroll",e,!0)}})));return wk(...c)}
var RT=class{constructor(a,b){this.qa=a;this.G=of(ip);this.ca=of(og);this.P=new Set;this.D=new Set;this.C=yr([]);this.F=new Map;this.aa=c=>c.je();this.J=new hj;this.N=new hj;this.scroll=new hj;this.ja=c=>{0<this.C().length&&c.preventDefault()};this.T=c=>{0<this.C().length&&(this.C().some(this.aa)&&c.preventDefault(),this.J.next(c))};this.ia=b}je(a){return-1<this.C().indexOf(a)}va(){this.D.forEach(a=>OT(this,a));this.P.forEach(a=>{this.P.delete(a)});this.O();this.J.complete();this.N.complete()}O(){this.F.forEach((a,
b)=>{this.ia.removeEventListener(b,a.handler,a.options)});this.F.clear()}ea(){if(!LT.has(this.G)){LT.add(this.G);const a=Ns({Pf:this.ca});this.G.ob(()=>{LT.delete(this.G);0===LT.size&&a.destroy()})}}};RT.K=function(a){return new (a||RT)(r(Tk),r(kt))};RT.na=Pd({ma:RT,ba:RT.K,fa:"root"});function ST(a){const b=a.cloneNode(!0),c=b.querySelectorAll("[id]"),d=a.nodeName.toLowerCase();b.removeAttribute("id");for(let e=0;e<c.length;e++)c[e].removeAttribute("id");"canvas"===d?TT(a,b):("input"===d||"select"===d||"textarea"===d)&&UT(a,b);VT("canvas",a,b,TT);VT("input, textarea, select",a,b,UT);return b}function VT(a,b,c,d){b=b.querySelectorAll(a);if(b.length)for(a=c.querySelectorAll(a),c=0;c<b.length;c++)d(b[c],a[c])}let WT=0;
function UT(a,b){"file"!==b.type&&(b.value=a.value);"radio"===b.type&&b.name&&(b.name=`mat-clone-${b.name}-${WT++}`)}function TT(a,b){if(b=b.getContext("2d"))try{b.drawImage(a,0,0)}catch(c){}};function XT(a){a=a.getBoundingClientRect();return{top:a.top,right:a.right,bottom:a.bottom,left:a.left,width:a.width,height:a.height,x:a.x,y:a.y}}function YT(a,b,c){a.top+=b;a.bottom=a.top+a.height;a.left+=c;a.right=a.left+a.width};function ZT(a,b){const c=Tt(b);b=a.oe.get(c);if(!b)return null;b=b.vs;let d;if(c===a.ia){var e=window.scrollX;d=window.scrollY}else d=c.scrollTop,e=c.scrollLeft;const f=b.top-d,g=b.left-e;a.oe.forEach((h,m)=>{h.lv&&c!==m&&c.contains(m)&&YT(h.lv,f,g)});b.top=d;b.left=e;return{top:f,left:g}}
var $T=class{constructor(a){this.ia=a;this.oe=new Map}clear(){this.oe.clear()}cache(a){this.clear();this.oe.set(this.ia,{vs:{top:window.scrollY,left:window.scrollX}});a.forEach(b=>{this.oe.set(b,{vs:{top:b.scrollTop,left:b.scrollLeft},lv:XT(b)})})}};function aU(a,b){a=a.rootNodes;if(1===a.length&&a[0].nodeType===b.ELEMENT_NODE)return a[0];const c=b.createElement("div");a.forEach(d=>c.appendChild(d));return c};function bU(a,b,c){for(let d in b)if(b.hasOwnProperty(d)){const e=b[d];if(e){let f;a.setProperty(d,e,(null==(f=c)?0:f.has(d))?"important":"")}else a.removeProperty(d)}}function cU(a,b){const c=b?"":"none";bU(a.style,{"touch-action":b?"":"none","-webkit-user-drag":b?"":"none","-webkit-tap-highlight-color":b?"":"transparent","user-select":c,"-ms-user-select":c,"-webkit-user-select":c,"-moz-user-select":c})}
function dU(a,b){bU(a.style,{position:b?"":"fixed",top:b?"":"0",opacity:b?"":"0",left:b?"":"-999em"},eU)}function fU(a,b){return b&&"none"!=b?a+" "+b:a}function gU(a,b){a.style.width=`${b.width}px`;a.style.height=`${b.height}px`;a.style.transform=hU(b.left,b.top)}function hU(a,b){return`translate3d(${Math.round(a)}px, ${Math.round(b)}px, 0)`};function iU(a){return parseFloat(a)*(-1<a.toLowerCase().indexOf("ms")?1:1E3)}function jU(a){a=getComputedStyle(a);var b=kU(a,"transition-property"),c=b.find(d=>"transform"===d||"all"===d);if(!c)return 0;b=b.indexOf(c);c=kU(a,"transition-duration");a=kU(a,"transition-delay");return iU(c[b])+iU(a[b])}function kU(a,b){return a.getPropertyValue(b).split(",").map(c=>c.trim())};const lU=new Set(["position"]);
var mU=class{constructor(a,b,c,d,e,f,g,h,m){this.ia=a;this.P=b;this.N=c;this.D=d;this.O=e;this.aa=f;this.G=g;this.F=h;this.ca=m}attach(a){this.C=this.T();a.appendChild(this.C);"showPopover"in this.C&&this.C.showPopover()}destroy(){this.C.remove();let a;null==(a=this.J)||a.destroy();this.C=this.J=null}getBoundingClientRect(){return this.C.getBoundingClientRect()}Wi(a){this.C.classList.add(a)}addEventListener(a,b){this.C.addEventListener(a,b)}removeEventListener(a,b){this.C.removeEventListener(a,b)}T(){const a=
this.O,b=this.aa;var c=a?a.sa:null;let d;if(c&&a){const e=a.eo?this.D:null;c=a.gg.xd(c,a.context);fn(c);d=aU(c,this.ia);this.J=c;a.eo?gU(d,e):d.style.transform=hU(this.G.x,this.G.y)}else d=ST(this.P),gU(d,this.D),this.F&&(d.style.transform=this.F);bU(d.style,{"pointer-events":"none",margin:"0",position:"fixed",top:"0",left:"0","z-index":this.ca+""},lU);cU(d,!1);d.classList.add("cdk-drag-preview");d.setAttribute("popover","manual");d.setAttribute("dir",this.N);b&&(Array.isArray(b)?b.forEach(e=>d.classList.add(e)):
d.classList.add(b));return d}};const nU=Lt({passive:!0}),oU=Lt({passive:!1}),pU=Lt({passive:!1,capture:!0}),eU=new Set(["position"]);function qU(a,b){const c=Ts(b);c!==a.C&&(a.C&&a.lh(a.C),Qk(a.qa,()=>{c.addEventListener("mousedown",a.Kb,oU);c.addEventListener("touchstart",a.Kb,nU);c.addEventListener("dragstart",a.hh,oU)}),a.ca=void 0,a.C=c);"undefined"!==typeof SVGElement&&a.C instanceof SVGElement&&(a.lb=a.C.ownerSVGElement);return a}function rU(a,b){a.Jc=b}
function sU(a,b){a.F=b.map(d=>Ts(d));a.F.forEach(d=>cU(d,a.disabled));a.Lb();const c=new Set;a.T.forEach(d=>{-1<a.F.indexOf(d)&&c.add(d)});a.T=c}function CX(a,b){a.ea=b?Ts(b):null;a.Pb.unsubscribe();b&&(a.Pb=a.Ai.xb(10).subscribe(()=>a.Di()));return a}function DX(a,b){a.N={x:0,y:0};a.P.x=b.x;a.P.y=b.y;a.D||a.Ec(b.x,b.y)}function EX(a){void 0===a.ld&&(a.ld=Rt(a.C));return a.ld}
var HX=class{get disabled(){return this.Ib||!(!this.D||!this.D.disabled)}set disabled(a){a!==this.Ib&&(this.Ib=a,this.Lb(),this.F.forEach(b=>cU(b,a)))}constructor(a,b,c,d,e,f){this.Aa=b;this.ia=c;this.qa=d;this.Ai=e;this.O=f;this.P={x:0,y:0};this.N={x:0,y:0};this.Ja=yr(!1);this.gb=new hj;this.Pb=this.td=this.kh=this.jh=Si.EMPTY;this.ea=null;this.ih=!0;this.F=[];this.T=new Set;this.md="ltr";this.mj=0;this.Ib=!1;this.Ta=new hj;this.mi=new hj;this.released=new hj;this.ended=new hj;this.pj=new hj;this.rj=
new hj;this.oj=new hj;this.Aj=this.gb;this.Kb=g=>{this.Ta.next();if(this.F.length){const h=this.Zd(g);!h||this.T.has(h)||this.disabled||this.Re(h,g)}else this.disabled||this.Re(this.C,g)};this.il=g=>{var h=this.Va(g);if(this.Ja()){g.cancelable&&g.preventDefault();var m=this.Wd(h);this.Ic=!0;this.hg=h;this.Il(m);if(this.D)this.mh(m,h);else{h=this.ze?this.ja:this.G;var p=this.N;p.x=m.x-h.x+this.P.x;p.y=m.y-h.y+this.P.y;this.Ec(p.x,p.y)}this.gb.nc.length&&this.qa.run(()=>{this.gb.next({source:this,sx:m,
event:g,distance:this.Hc(m),delta:this.Qc})})}else Math.abs(h.x-this.G.x)+Math.abs(h.y-this.G.y)>=this.Aa.un&&((h=Date.now()>=this.nk+this.Lk(g),p=this.D,h)?p&&(p.je()||p.T())||(g.cancelable&&g.preventDefault(),this.Ja.set(!0),this.qa.run(()=>this.Cl(g))):this.Vd(g))};this.rl=g=>{this.Vd(g)};this.hh=g=>{if(this.F.length){const h=this.Zd(g);!h||this.T.has(h)||this.disabled||g.preventDefault()}else this.disabled||g.preventDefault()};rU(qU(this,a),b.uG||null);this.wa=new $T(c);MT(f,this)}dispose(){this.lh(this.C);
if(this.je()){let b;null==(b=this.C)||b.remove()}let a;null==(a=this.Ea)||a.remove();this.Ud();this.Td();OT(this.O,this);this.kd();this.Ta.complete();this.mi.complete();this.released.complete();this.ended.complete();this.pj.complete();this.rj.complete();this.oj.complete();this.gb.complete();this.F=[];this.T.clear();this.D=void 0;this.Pb.unsubscribe();this.wa.clear();this.ea=this.C=this.lb=this.sd=this.Qa=this.Ea=this.Jc=null}je(){return this.Ja()&&this.O.je(this)}reset(){this.C.style.transform=this.ca||
"";this.N={x:0,y:0};this.P={x:0,y:0}}Ll(a){this.D=a}no(){const a=this.hg;a&&this.D&&this.mh(this.Wd(a),a)}kd(){this.jh.unsubscribe();this.kh.unsubscribe();this.td.unsubscribe();let a;null==(a=EX(this))||a.removeEventListener("selectstart",FX,pU)}Ud(){let a;null==(a=this.J)||a.destroy();this.J=null}Td(){let a;null==(a=this.Ob)||a.remove();let b;null==(b=this.sb)||b.destroy();this.Ob=this.sb=null}Vd(a){if(this.O.je(this)&&(this.kd(),NT(this.O,this),this.Lb(),this.F&&(this.C.style.webkitTapHighlightColor=
this.Bl),this.Ja()))if(this.released.next({source:this,event:a}),this.D)this.D.N(),this.Bi().then(()=>{this.Ci(a);this.ud();NT(this.O,this)});else{this.P.x=this.N.x;const b=this.Va(a);this.P.y=this.N.y;this.qa.run(()=>{this.ended.next({source:this,distance:this.Hc(b),nj:b,event:a})});this.ud();NT(this.O,this)}}Cl(a){GX(a)&&(this.gh=Date.now());this.Lb();const b=EX(this),c=this.D;b&&Qk(this.qa,()=>{b.addEventListener("selectstart",FX,pU)});if(c){const d=this.C,e=d.parentNode,f=this.Ob=this.Ei(),g=
this.Ea=this.Ea||this.ia.createComment("");e.insertBefore(g,d);this.ca=d.style.transform||"";this.J=new mU(this.ia,this.C,this.md,this.ja,this.Qa||null,this.Ul||null,this.G,this.ca,this.Aa.zIndex||1E3);this.J.attach(this.Sk(e,b));dU(d,!1);this.ia.body.appendChild(e.replaceChild(f,d));this.mi.next({source:this,event:a});c.start();this.aa=c;this.fb=c.C(this)}else this.mi.next({source:this,event:a}),this.aa=this.fb=void 0;this.wa.cache(c?c.P():[])}Re(a,b){this.Jc&&b.stopPropagation();var c=this.je(),
d=GX(b);const e=!d&&0!==b.button,f=this.C,g=Tt(b),h=!d&&this.gh&&this.gh+800>Date.now();d=d?Fx(b):Ex(b);g&&g.draggable&&"mousedown"===b.type&&b.preventDefault();c||e||h||d||(this.F.length&&(c=f.style,this.Bl=c.webkitTapHighlightColor||"",c.webkitTapHighlightColor="transparent"),this.Ic=!1,this.Ja.set(this.Ic),this.kd(),this.ja=this.C.getBoundingClientRect(),this.jh=this.O.J.subscribe(this.il),this.kh=this.O.N.subscribe(this.rl),this.td=QT(this.O,EX(this)).subscribe(m=>this.El(m)),this.ea&&(this.oa=
XT(this.ea)),this.ta=(c=this.Qa)&&c.sa&&!c.eo?{x:0,y:0}:this.Nk(this.ja,a,b),a=this.G=this.hg=this.Va(b),this.Qc={x:0,y:0},this.ll={x:a.x,y:a.y},this.nk=Date.now(),PT(this.O,this,b))}Ci(a){dU(this.C,!0);this.Ea.parentNode.replaceChild(this.C,this.Ea);this.Ud();this.Td();this.ja=this.oa=this.ua=this.ca=void 0;this.qa.run(()=>{const b=this.D,c=b.C(this),d=this.Va(a),e=this.Hc(d),f=b.D(d.x,d.y);this.ended.next({source:this,distance:e,nj:d,event:a});this.oj.next({item:this,dd:c,Le:this.fb,ab:b,wx:this.aa,
Jw:f,distance:e,nj:d,event:a});b.O(this,c,this.fb,this.aa,f,e,d,a);this.D=this.aa})}mh({x:a,y:b},{x:c,y:d}){let e=this.aa.F(this,a,b);!e&&this.D!==this.aa&&this.aa.D(a,b)&&(e=this.aa);e&&e!==this.D&&this.qa.run(()=>{this.rj.next({item:this,ab:this.D});this.D.Mq(this);this.D=e;this.D.xn(this,a,b,e===this.aa&&e.aa?this.fb:void 0);this.pj.next({item:this,ab:e,dd:e.C(this)})});this.je()&&(this.D.J(c,d),this.D.G(this,a,b,this.Qc),this.ze?this.cc(a,b):this.cc(a-this.ta.x,b-this.ta.y))}Bi(){if(!this.Ic)return Promise.resolve();
const a=this.Ob.getBoundingClientRect();this.J.Wi("cdk-drag-animating");this.cc(a.left,a.top);const b=jU(this.J.C);return 0===b?Promise.resolve():Qk(this.qa,()=>new Promise(c=>{const d=f=>{if(!f||Tt(f)===this.J&&"transform"===f.propertyName){let g;null==(g=this.J)||g.removeEventListener("transitionend",d);c();clearTimeout(e)}},e=setTimeout(d,1.5*b);this.J.addEventListener("transitionend",d)}))}Ei(){var a=this.sd;const b=a?a.sa:null;b?(this.sb=a.gg.xd(b,a.context),fn(this.sb),a=aU(this.sb,this.ia)):
a=ST(this.C);a.style.pointerEvents="none";a.classList.add("cdk-drag-placeholder");return a}Nk(a,b,c){b=(b=b===this.C?null:b)?b.getBoundingClientRect():a;c=GX(c)?c.targetTouches[0]:c;const d=this.ae();return{x:b.left-a.left+(c.pageX-b.left-d.left),y:b.top-a.top+(c.pageY-b.top-d.top)}}Va(a){var b=this.ae(),c=GX(a)?a.touches[0]||a.changedTouches[0]||{pageX:0,pageY:0}:a;a=c.pageX-b.left;b=c.pageY-b.top;if(this.lb&&(c=this.lb.getScreenCTM())){const d=this.lb.createSVGPoint();d.x=a;d.y=b;return d.matrixTransform(c.inverse())}return{x:a,
y:b}}Wd(a){var b=this.D?this.D.Ph:null;let {x:c,y:d}=this.ze?this.ze(a,this,this.ja,this.ta):a;if("x"===this.Ph||"x"===b)d=this.G.y-(this.ze?this.ta.y:0);else if("y"===this.Ph||"y"===b)c=this.G.x-(this.ze?this.ta.x:0);if(this.oa){const {x:e,y:f}=this.ze?{x:0,y:0}:this.ta;a=this.oa;const {width:g,height:h}=this.fl();b=a.top+f;const m=a.bottom-(h-f);c=Math.max(a.left+e,Math.min(a.right-(g-e),c));d=Math.max(b,Math.min(m,d))}return{x:c,y:d}}Il(a){const b=a.x;a=a.y;const c=this.Qc,d=this.ll,e=Math.abs(a-
d.y);Math.abs(b-d.x)>this.Aa.Sl&&(c.x=b>d.x?1:-1,d.x=b);e>this.Aa.Sl&&(c.y=a>d.y?1:-1,d.y=a);return c}Lb(){if(this.C&&this.F){var a=0<this.F.length||!this.je();a!==this.ih&&(this.ih=a,cU(this.C,a))}}lh(a){a.removeEventListener("mousedown",this.Kb,oU);a.removeEventListener("touchstart",this.Kb,nU);a.removeEventListener("dragstart",this.hh,oU)}Ec(a,b){a=hU(a,b);b=this.C.style;null==this.ca&&(this.ca=b.transform&&"none"!=b.transform?b.transform:"");b.transform=fU(a,this.ca)}cc(a,b){let c;const d=(null==
(c=this.Qa)?0:c.sa)?void 0:this.ca;this.J.C.style.transform=fU(hU(a,b),d)}Hc(a){const b=this.G;return b?{x:a.x-b.x,y:a.y-b.y}:{x:0,y:0}}ud(){this.oa=this.ua=void 0;this.wa.clear()}Di(){let {x:a,y:b}=this.P;if(!(0===a&&0===b||this.je())&&this.ea){var c=this.C.getBoundingClientRect(),d=this.ea.getBoundingClientRect();if(!(0===d.width&&0===d.height||0===c.width&&0===c.height)){var e=d.left-c.left,f=c.right-d.right,g=d.top-c.top,h=c.bottom-d.bottom;d.width>c.width?(0<e&&(a+=e),0<f&&(a-=f)):a=0;d.height>
c.height?(0<g&&(b+=g),0<h&&(b-=h)):b=0;a===this.P.x&&b===this.P.y||DX(this,{y:b,x:a})}}}Lk(a){const b=this.mj;return"number"===typeof b?b:GX(a)?b.MG:b?b.Kd:0}El(a){const b=ZT(this.wa,a);b&&(a=Tt(a),this.oa&&a!==this.ea&&a.contains(this.ea)&&YT(this.oa,b.top,b.left),this.G.x+=b.left,this.G.y+=b.top,this.D||(this.N.x-=b.left,this.N.y-=b.top,this.Ec(this.N.x,this.N.y)))}ae(){let a;return(null==(a=this.wa.oe.get(this.ia))?void 0:a.vs)||{top:window.scrollY,left:window.scrollX}}Sk(a,b){const c=this.xl||
"global";return"parent"===c?a:"global"===c?(a=this.ia,b||a.fullscreenElement||a.webkitFullscreenElement||a.mozFullScreenElement||a.msFullscreenElement||a.body):Ts(c)}fl(){this.ua&&(this.ua.width||this.ua.height)||(this.ua=this.J?this.J.getBoundingClientRect():this.ja);return this.ua}Zd(a){return this.F.find(b=>a.target&&(a.target===b||b.contains(a.target)))}};function GX(a){return"t"===a.type[0]}function FX(a){a.preventDefault()};const IX={un:5,Sl:5};function JX(a,b,c=IX){return new HX(b,c,a.ia,a.qa,a.D,a.C)}var KX=class{constructor(a,b,c,d){this.ia=a;this.qa=b;this.D=c;this.C=d}};KX.K=function(a){return new (a||KX)(r(kt),r(Tk),r(xu),r(RT))};KX.na=Pd({ma:KX,ba:KX.K,fa:"root"});var LX=new gf("CdkDropList");function MX(a,b){const c=b.Ph,d=b.mj,e=b.ze,f=b.Ul,g=b.kq,h=b.UF,m=b.Rj;b=b.Zr;a.disabled=null==h?!1:h;a.mj=d||0;c&&(a.Ph=c);e&&(a.ze=e);f&&(a.Ul=f);g&&(a.kq=g);m&&(a.Rj=m);b&&(a.Zr=b)}
function NX(a,b){b.mi.subscribe(c=>{a.mi.emit({source:a,event:c.event});en(a.Da)});b.released.subscribe(c=>{a.released.emit({source:a,event:c.event})});b.ended.subscribe(c=>{a.ended.emit({source:a,distance:c.distance,nj:c.nj,event:c.event});en(a.Da)});b.pj.subscribe(c=>{a.pj.emit({ab:c.ab.data,item:a,dd:c.dd})});b.rj.subscribe(c=>{a.rj.emit({ab:c.ab.data,item:a})});b.oj.subscribe(c=>{a.oj.emit({Le:c.Le,dd:c.dd,wx:c.wx.data,ab:c.ab.data,Jw:c.Jw,item:a,distance:c.distance,nj:c.nj,event:c.event})})}
var PX=class{get disabled(){return this.Ib||this.G&&this.G.disabled}set disabled(a){this.Ib=a;this.Zc.disabled=this.Ib}constructor(a,b,c,d,e,f,g,h,m,p,u){this.element=a;this.G=b;this.qa=d;this.N=e;this.P=g;this.Da=m;this.ea=p;this.J=u;this.Ga=new hj;this.C=new jj([]);this.mi=new Jk;this.released=new Jk;this.ended=new Jk;this.pj=new Jk;this.rj=new Jk;this.oj=new Jk;this.Aj=new bj(w=>{const B=C(this.Zc.Aj,Cj(G=>({source:this,sx:G.sx,event:G.event,delta:G.delta,distance:G.distance}))).subscribe(w);return()=>
{B.unsubscribe()}});this.Ra=of(ai);this.Zc=JX(h,a,{un:f&&null!=f.un?f.un:5,Sl:f&&null!=f.Sl?f.Sl:5,zIndex:null==f?void 0:f.zIndex});this.Zc.data=this;OX.push(this);f&&MX(this,f);b&&this.Zc.Ll(b.C);this.oa(this.Zc);NX(this,this.Zc)}reset(){this.Zc.reset()}pd(){el(()=>{this.O();this.ja();this.Fn&&DX(this.Zc,this.Fn)},{xa:this.Ra})}uc(a){const b=a.rootElementSelector;a=a.freeDragPosition;b&&!b.Bn&&this.O();a&&!a.Bn&&this.Fn&&DX(this.Zc,this.Fn)}va(){this.G&&this.G.removeItem(this);const a=OX.indexOf(this);
-1<a&&OX.splice(a,1);Qk(this.qa,()=>{this.C.complete();this.Ga.next();this.Ga.complete();this.Zc.dispose()})}T(a){const b=ij(this.C);b.push(a);this.C.next(b)}ca(a){const b=ij(this.C);a=b.indexOf(a);-1<a&&(b.splice(a,1),this.C.next(b))}Ea(a){this.D=a}ua(a){a===this.D&&(this.D=null)}wa(a){this.F=a}ta(a){a===this.F&&(this.F=null)}O(){const a=this.element.R;let b=a;if(this.Rj){let c;b=void 0!==a.closest?a.closest(this.Rj):null==(c=a.parentElement)?void 0:c.closest(this.Rj)}qU(this.Zc,b||a)}aa(){const a=
this.kq;return a?"string"===typeof a?this.element.R.closest(a):Ts(a):null}oa(a){a.Ta.subscribe(()=>{if(!a.je()){const c=this.P;var b=this.mj;const d=this.F?{sa:this.F.eg,context:this.F.data,gg:this.N}:null,e=this.D?{sa:this.D.eg,context:this.D.data,eo:this.D.eo,gg:this.N}:null;a.disabled=this.disabled;a.Ph=this.Ph;a.mj="object"===typeof b&&b?b:Us(b);a.ze=this.ze;a.Ul=this.Ul;b=CX(a,this.aa());b.sd=d;b.Qa=e;b.xl=this.Zr||"global";c&&(a.md=c.value)}});C(a.Ta,io()).subscribe(()=>{if(this.J)a.Jc=this.J.Zc;
else for(var b=this.element.R.parentElement;b;){if(b.classList.contains("cdk-drag")){let c;rU(a,(null==(c=OX.find(d=>d.element.R===b))?void 0:c.Zc)||null);break}b=b.parentElement}})}ja(){C(this.C,So(a=>{a=a.map(b=>b.element);this.ea&&this.Rj&&a.push(this.element);sU(this.Zc,a)}),Lo(a=>wk(...a.map(b=>C(b.C,Ko(b))))),Oo(this.Ga)).subscribe(a=>{const b=this.Zc,c=a.element.R;a.disabled?!b.T.has(c)&&-1<b.F.indexOf(c)&&(b.T.add(c),cU(c,!0)):b.T.has(c)&&(b.T.delete(c),cU(c,b.disabled))})}},OX=[];
PX.K=function(a){return new (a||PX)(y(wi),y(LX,12),y(kt),y(Tk),y(np),y(JT,8),y(pu,8),y(KX),y(Tr),y(HT,10),y(GT,12))};
PX.Fa=Uf({type:PX,ha:[["","cdkDrag",""]],Ka:[1,"cdk-drag"],Za:4,Ia:function(a,b){a&2&&Wp("cdk-drag-disabled",b.disabled)("cdk-drag-dragging",b.Zc.je())},inputs:{data:[0,"cdkDragData","data"],Ph:[0,"cdkDragLockAxis","lockAxis"],Rj:[0,"cdkDragRootElement","rootElementSelector"],kq:[0,"cdkDragBoundary","boundaryElement"],mj:[0,"cdkDragStartDelay","dragStartDelay"],Fn:[0,"cdkDragFreeDragPosition","freeDragPosition"],disabled:[2,"cdkDragDisabled","disabled",As],ze:[0,"cdkDragConstrainPosition","constrainPosition"],
Ul:[0,"cdkDragPreviewClass","previewClass"],Zr:[0,"cdkDragPreviewContainer","previewContainer"]},outputs:{mi:"cdkDragStarted",released:"cdkDragReleased",ended:"cdkDragEnded",pj:"cdkDragEntered",rj:"cdkDragExited",oj:"cdkDragDropped",Aj:"cdkDragMoved"},ib:["cdkDrag"],ka:!0,features:[Lr([{pa:GT,Ab:PX}]),Lp,Hg]});new gf("CdkDragPlaceholder");new gf("CdkDragPreview");new gf("CdkDropListGroup");var QX=class{};QX.K=function(a){return new (a||QX)};QX.Ya=Sf({type:QX});QX.Xa=Qd({Wa:[KX],imports:[yu]});const RX=(a,b)=>b.type;function SX(a){a&1&&(R(0,"div",24),Z(1," Rules use your defined queries to match nodes for styling. All queries within a rule are connected with "),R(2,"span",25),Z(3,"AND"),U(),Z(4," logic. "),U())}function TX(a){a&1&&(R(0,"div",28),Oq(1,"node-list-viewer",36),U());if(a&2){a=Y().Cb;const b=Y(2);D();Q("nodes",(b.sm[a]||{})[0]||[])("rendererId",b.Nw)}}
function UX(a){a&1&&(R(0,"div",28)(1,"div",37),Z(2,"Left pane"),U(),Oq(3,"node-list-viewer",36),U(),R(4,"div",28)(5,"div",37),Z(6,"Right pane"),U(),Oq(7,"node-list-viewer",36),U());if(a&2){a=Y().Cb;const b=Y(2);D(3);Q("nodes",(b.sm[a]||{})[0]||[])("rendererId",b.Nw);D(4);Q("nodes",(b.sm[a]||{})[1]||[])("rendererId",b.UD)}}
function VX(a){if(a&1){const b=Zg();R(0,"div",41)(1,"label")(2,"input",42,3);V("change",function(){t(b);const c=Ar(3),d=Y(2).da,e=Y().Cb,f=Y(2);return x(WX(f,e,d,c.value))});U()();R(4,"input",43,4);V("change",function(){t(b);const c=Ar(5),d=Y(2).da,e=Y().Cb,f=Y(2);return x(WX(f,e,d,c.value))});U()()}if(a&2){a=Y(2).da;const b=Y().da;Y(2);D();Up("background-color",XX(b,a));D();Q("value",XX(b,a));D(2);Q("value",XX(b,a))}}
function YX(a){a&1&&H(0,VX,6,4,"div",41);if(a&2){a=Y().da;const b=Y().da;Y(2);Gq(null!=b.styles[a.id]?0:-1)}}function ZX(a){if(a&1){const b=Zg();R(0,"div",44)(1,"input",45,5);V("change",function(){t(b);const c=Ar(2),d=Y(2).da,e=Y().Cb,f=Y(2);return x(WX(f,e,d,c.value))});U()()}if(a&2){a=Y(2).da;const b=Y().da;Y(2);D();Q("value",XX(b,a))}}function $X(a){a&1&&H(0,ZX,3,1,"div",44);if(a&2){a=Y().da;const b=Y().da;Y(2);Gq(null!=b.styles[a.id]?0:-1)}}
function aY(a,b){if(a&1){const c=Zg();R(0,"div",30)(1,"div",38)(2,"input",39,2);V("change",function(){const d=t(c).da,e=Ar(3),f=Y().Cb,g=Y(2);oL(g.vd,f,d,e.checked);return x()});U();R(4,"div",40);V("click",function(){const d=t(c).da,e=Y().Cb,f=Y(2);oL(f.vd,e,d);return x()});Z(5);U()();H(6,YX,1,1)(7,$X,1,1);U()}if(a&2){let c;a=b.da;b=Y().da;const d=Y(2);D(2);Q("checked",null!=b.styles[a.id]);D(3);Cr(" ",a.label," ");D();Gq((c=a.type)===d.tt.COLOR?6:c===d.tt.ep?7:-1)}}
function bY(a){if(a&1){const b=Zg();R(0,"div",46);V("click",function(){t(b);const c=Y().Cb,d=Y(2);kL(d.vd,c);return x()});R(1,"mat-icon");Z(2,"arrow_upward_alt");U()()}}function cY(a){if(a&1){const b=Zg();R(0,"div",47);V("click",function(){t(b);const c=Y().Cb,d=Y(2);lL(d.vd,c);return x()});R(1,"mat-icon");Z(2,"arrow_downward_alt");U()()}}
function dY(a,b){if(a&1){const c=Zg();R(0,"tr")(1,"td",19)(2,"complex-queries",26);V("queriesUpdated",function(d){const e=t(c).Cb,f=Y(2);nL(f.vd,e,d);return x()});U();R(3,"div",27);H(4,TX,2,2,"div",28)(5,UX,8,4);U()();R(6,"td",22)(7,"div",29);Lq(8,aY,8,3,"div",30,RX);U()();R(10,"td",23)(11,"div",31);H(12,bY,3,0,"div",32)(13,cY,3,0,"div",33);R(14,"div",34);V("click",function(){const d=t(c).Cb,e=Y(2);mL(e.vd,d);return x()});R(15,"mat-icon");Z(16,"content_copy");U()();R(17,"div",35);V("click",function(){const d=
t(c).Cb,e=Y(2);rL(e.vd,d);return x()});R(18,"mat-icon");Z(19,"delete");U()()()()()}if(a&2){a=b.da;const c=b.Cb;b=b.hz;const d=Y(2);D(2);Q("queries",a.queries);D(2);Gq(1===d.ox?4:2===d.ox?5:-1);D(4);Nq(d.lB);D(4);Gq(0!==c?12:-1);D();Gq(c!==b-1?13:-1)}}
function eY(a){a&1&&(R(0,"table",10)(1,"thead")(2,"th",19)(3,"div",20),Z(4," Queries to match nodes "),R(5,"div",21)(6,"mat-icon"),Z(7,"help_outline"),U()(),H(8,SX,5,0,"ng-template",null,1,Rr),U()(),R(10,"th",22),Z(11,"Styles to apply"),U(),Oq(12,"th",23),U(),R(13,"tbody"),Lq(14,dY,20,4,"tr",null,Jq),U()());if(a&2){a=Ar(9);const b=Y();D(5);Q("xapInlineDialog",a)("overlaySize",b.fd)("hoverDelayMs",10);D(9);Nq(b.rules())}}
function fY(a){a&1&&(R(0,"div",11),Z(1,' No rules configured. Click the "Add rule" button below to get started. '),U())}function WX(a,b,c,d){let e=d;d.startsWith("rgb")?e=gY(d):d.startsWith("#")||(e=lI[d]||d);qL(a.vd,b,c,e)}function XX(a,b){let c;return(null==(c=a.styles[b.id])?void 0:c.value)||""}function hY(a,b){if(b.files&&0!==b.files.length){var c=new FileReader;c.onload=d=>{let e;d=JSON.parse(null==(e=d.target)?void 0:e.result);hL(a.vd,d)};c.readAsText(b.files[0])}}
function gY(a){return(a=a.match(/^rgba?\((\d+),\s*(\d+),\s*(\d+)(?:,\s*(\d+\.{0,1}\d*))?\)$/))?(a=a.slice(1,4).map(b=>Number(b).toString(16).padStart(2,"0")).join(""),a[0]===a[1]&&a[2]===a[3]&&a[4]===a[5]?`#${a[0]}${a[2]}${a[4]}`:`#${a}`):"unknown"}
var iY=class{constructor(a,b,c){this.C=a;this.D=b;this.vd=c;this.rules=this.vd.rules;this.ol=this.vd.ol;this.rC=Cs(()=>0<this.rules().length);this.tt=eL;this.lB=fL;this.Ru=[{type:"title",Rd:"Match label"},{type:"list",Rd:"Match attributes"},{type:"input",Rd:"Match inputs (op node only)"},{type:"output",Rd:"Match outputs (op node only)"}];this.Su=[{type:"op_nodes",label:"Ops only"},{type:"layer_nodes",label:"Layers only"},{type:"op_and_layer_nodes",label:"Ops & layers"}];this.Pu=[{type:"regex",label:"Regex"},
{type:"attr_value_range",label:"Attribute value range"}];this.fd={minWidth:0,minHeight:0,maxWidth:340};this.sm={};Ks(()=>{this.sm=this.vd.D();en(this.D)})}ow(a,b){pL(this.vd,a,b)}get ox(){return this.C.La().length}get Nw(){return this.C.La()[0].id}get UD(){return this.C.La()[1].id}};iY.K=function(a){return new (a||iY)(y(VJ),y(Tr),y(sL))};
iY.Ca=Jf({type:iY,ha:[["node-styler-dialog"]],ka:!0,features:[Nr],ya:26,za:1,Ba:[["input",""],["helpQuery",""],["cb",""],["colorPicker",""],["colorInput",""],["numberInput",""],["mat-dialog-title","","cdkDrag","","cdkDragRootElement",".cdk-overlay-pane","cdkDragHandle","",1,"title-container"],[1,"title"],[1,"description"],["mat-dialog-content","",1,"dialog-content"],["border","0","cellspacing","0","cellpadding","0",1,"rules-table"],[1,"no-rules-message"],["mat-dialog-actions","",1,"btns-container"],
[1,"left-btns"],["mat-flat-button","","color","primary",1,"action-button","add-rule",3,"click"],["mat-flat-button","","color","primary",1,"action-button","load-rules",3,"click"],["mat-flat-button","","color","primary",1,"action-button","save-rules",3,"click"],["type","file","accept","application/JSON",1,"import-rules-input",3,"change"],["mat-button","","mat-dialog-close","",1,"close"],[1,"query"],[1,"query-header-container"],["dialogLabel","help",1,"icon-container",3,"xapInlineDialog","overlaySize",
"hoverDelayMs"],[1,"styles"],[1,"action-btns"],[1,"model-explorer-help-popup"],[1,"bold"],[3,"queriesUpdated","queries"],[1,"node-list-viewers-container"],[1,"node-list-viewer-container"],[1,"styles-container"],[1,"color-style-container"],[1,"action-buttons-container"],["matTooltip","Move up",1,"icon-container","move-up"],["matTooltip","Move down",1,"icon-container","move-down"],["matTooltip","Duplicate rule",1,"icon-container","duplicate-rule",3,"click"],["matTooltip","Delete rule",1,"icon-container",
"delete-rule",3,"click"],["labelSuffix","matched node",3,"nodes","rendererId"],[1,"label"],[1,"style-name-container"],["type","checkbox",3,"change","checked"],[3,"click"],[1,"color-picker-container"],["type","color",3,"change","value"],["type","text",3,"change","value"],[1,"number-editor-container"],[3,"change","value"],["matTooltip","Move up",1,"icon-container","move-up",3,"click"],["matTooltip","Move down",1,"icon-container","move-down",3,"click"]],sa:function(a,b){if(a&1){const c=Zg();R(0,"div",
6)(1,"div",7);Z(2,"Node styler");U();R(3,"div",8);Z(4," Use the buttons below to apply styling rules to this graph ");U()();R(5,"div",9);H(6,eY,16,3,"table",10)(7,fY,2,0,"div",11);U();R(8,"div",12)(9,"div",13)(10,"button",14);V("click",function(){t(c);jL(b.vd);return x()});R(11,"mat-icon");Z(12,"add");U();Z(13," Add rule ");U();R(14,"button",15);V("click",function(){t(c);const d=Ar(23);return x(d.click())});R(15,"mat-icon");Z(16,"file_upload");U();Z(17," Import rules ");U();R(18,"button",16);V("click",
function(){t(c);const d=document.createElement("a");d.download="node_styler_rules.json";const e=`data:text/json;charset=utf-8, ${encodeURIComponent(JSON.stringify(b.rules(),null,2))}`;wd(d,td(e));d.click();return x()});R(19,"mat-icon");Z(20,"file_download");U();Z(21," Export rules ");U();R(22,"input",17,0);V("change",function(){t(c);const d=Ar(23);return x(hY(b,d))});U()();R(24,"button",18);Z(25,"Done");U()()}a&2&&(D(6),Gq(b.rC()?6:7))},Ha:[it,DT,QX,PX,IT,BB,AB,vB,zC,sC,uC,wC,vC,$C,PE,AE,NE,iI,hI,
FT,HG,EG],styles:["[_ngcontent-%COMP%]:not(mat-icon){font-family:Google Sans Text,Arial,Helvetica,sans-serif!important;letter-spacing:normal!important}.title-container[_ngcontent-%COMP%]{position:relative}.title[_ngcontent-%COMP%]{font-size:24px;font-weight:500;margin-top:-30px}.description[_ngcontent-%COMP%]{line-height:14px;font-size:12px;color:#777;margin-top:6px;margin-bottom:4px;font-weight:400}.btns-container[_ngcontent-%COMP%]{width:100%;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;padding:0 24px}.action-button[_ngcontent-%COMP%]{height:30px;padding:0 10px;font-size:13px}.action-button[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{margin-right:2px}.import-rules-input[_ngcontent-%COMP%]{display:none}.dialog-content[_ngcontent-%COMP%]{padding:0;-moz-box-sizing:border-box;box-sizing:border-box;border-top:1px solid #e6e6e6;border-bottom:1px solid #e6e6e6;background-color:#f9f9f9;overflow:hidden auto}.dialog-content[_ngcontent-%COMP%]   .no-rules-message[_ngcontent-%COMP%]{padding:12px 24px;font-size:12px;color:#777}.common-input[_ngcontent-%COMP%]{padding:0 4px;border:1px solid #ccc;border-radius:4px;height:22px;-moz-box-sizing:border-box;box-sizing:border-box;outline:none}.common-input[_ngcontent-%COMP%]:focus{border-color:#1a73e8;outline:1px solid #1a73e8}.rules-table[_ngcontent-%COMP%]{width:100%;font-size:12px}.rules-table[_ngcontent-%COMP%]   td[_ngcontent-%COMP%]:not(.action-btns), .rules-table[_ngcontent-%COMP%]   th[_ngcontent-%COMP%]:not(.action-btns){border-right:1px solid #e6e6e6}.rules-table[_ngcontent-%COMP%]   tr[_ngcontent-%COMP%]   td[_ngcontent-%COMP%]{border-bottom:1px solid #e6e6e6}.rules-table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]{text-align:left;height:32px;position:-webkit-sticky;position:sticky;top:0;background-color:#f1f1f1;z-index:100}.rules-table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   th[_ngcontent-%COMP%]{font-weight:500;border-bottom:1px solid #e6e6e6}.rules-table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   th.query[_ngcontent-%COMP%]{padding-left:24px;width:410px}.rules-table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   th.query[_ngcontent-%COMP%]   .query-header-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.rules-table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   th.query[_ngcontent-%COMP%]   .query-header-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{width:16px;height:16px;opacity:.6;cursor:pointer;margin-left:4px}.rules-table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   th.query[_ngcontent-%COMP%]   .query-header-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]:hover{opacity:1}.rules-table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   th.query[_ngcontent-%COMP%]   .query-header-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:16px;width:16px;height:16px}.rules-table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   th.target[_ngcontent-%COMP%]{padding-left:12px}.rules-table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   th.styles[_ngcontent-%COMP%]{padding-left:12px;min-width:254px}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   td[_ngcontent-%COMP%]{padding-top:8px;padding-bottom:8px}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   td.query[_ngcontent-%COMP%]{padding-left:24px;padding-right:12px;vertical-align:top}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   td.query[_ngcontent-%COMP%]   .node-list-viewers-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;gap:12px}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   td.query[_ngcontent-%COMP%]   .node-list-viewer-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;width:-webkit-fit-content;width:-moz-fit-content;width:fit-content;margin-top:12px}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   td.query[_ngcontent-%COMP%]   .node-list-viewer-container[_ngcontent-%COMP%]   .label[_ngcontent-%COMP%]{line-height:18px}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   td.target[_ngcontent-%COMP%]{vertical-align:top;padding-left:12px;padding-right:12px}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   td.styles[_ngcontent-%COMP%]{padding-left:12px;padding-right:18px;vertical-align:top}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   td.styles[_ngcontent-%COMP%]   .styles-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;gap:2px}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   td.action-btns[_ngcontent-%COMP%]{vertical-align:top;padding-right:12px;width:80px}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   td.action-btns[_ngcontent-%COMP%]   .action-buttons-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-pack:end;-webkit-justify-content:flex-end;-moz-box-pack:end;-ms-flex-pack:end;justify-content:flex-end;gap:2px}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   td.action-btns[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;opacity:.6;cursor:pointer;margin-top:2px}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   td.action-btns[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]:hover{opacity:1}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   td.action-btns[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:16px;width:16px;height:16px}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   .color-style-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   .color-style-container[_ngcontent-%COMP%]   .style-name-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;cursor:pointer;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;width:100px}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   .color-style-container[_ngcontent-%COMP%]   .style-name-container[_ngcontent-%COMP%]   input[_ngcontent-%COMP%]{cursor:pointer}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   .color-style-container[_ngcontent-%COMP%]   .color-picker-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;overflow:hidden;border-radius:4px;border:1px solid #ccc;margin-left:4px;width:-webkit-fit-content;width:-moz-fit-content;width:fit-content;background-color:#fff;-moz-box-sizing:border-box;box-sizing:border-box;height:24px;position:relative}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   .color-style-container[_ngcontent-%COMP%]   .color-picker-container[_ngcontent-%COMP%]:focus-within{border-color:#1a73e8;outline:1px solid #1a73e8}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   .color-style-container[_ngcontent-%COMP%]   .color-picker-container[_ngcontent-%COMP%]   label[_ngcontent-%COMP%]{display:inline-block;width:16px;height:16px;margin:2px;background-color:red;border-radius:4px;cursor:pointer;border:1px solid #ddd}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   .color-style-container[_ngcontent-%COMP%]   .color-picker-container[_ngcontent-%COMP%]   label[_ngcontent-%COMP%]   input[_ngcontent-%COMP%]{visibility:hidden}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   .color-style-container[_ngcontent-%COMP%]   .color-picker-container[_ngcontent-%COMP%]   input[type=text][_ngcontent-%COMP%]{width:100px;border:none;height:100%;background-color:transparent;outline:none}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   .color-style-container[_ngcontent-%COMP%]   .number-editor-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;overflow:hidden;margin-left:4px;width:-webkit-fit-content;width:-moz-fit-content;width:fit-content}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   .color-style-container[_ngcontent-%COMP%]   .number-editor-container[_ngcontent-%COMP%]   input[_ngcontent-%COMP%]{width:80px;height:28px;border:1px solid #ccc;border-radius:4px;background-color:transparent;-moz-box-sizing:border-box;box-sizing:border-box;padding:0 4px;outline:none}"],
Pa:0});var jY=class{constructor(a,b,c){this.Ih=a;this.C=b;this.yb=c;this.ol=this.C.ol}};jY.K=function(a){return new (a||jY)(y(pC),y(sL),y(np))};
jY.Ca=Jf({type:jY,ha:[["node-styler"]],ka:!0,features:[Nr],ya:3,za:2,Ba:[["mat-icon-button","","aria-label","node-styler","matTooltip","Style nodes with custom rules",3,"click"]],sa:function(a,b){a&1&&(R(0,"button",0),V("click",function(){b.Ih.open(iY,{width:"800px",height:"600px",yb:b.yb,Vb:!1,Ym:!1})}),R(1,"mat-icon"),Z(2,"palette"),U()());a&2&&(D(),Wp("highlight",b.ol()))},Ha:[it,BB,AB,yB,PE,AE,NE,xC,iI,hI,HG],styles:["button[_ngcontent-%COMP%]   mat-icon.highlight[_ngcontent-%COMP%]{background:-webkit-linear-gradient(45deg,#0089ff 0 30%,#f1af00 70% 100%);background:linear-gradient(45deg,#0089ff 0 30%,#f1af00 70% 100%);background-clip:text;-webkit-background-clip:text;-webkit-text-fill-color:transparent}"],
Pa:0});function kY(a){a&1&&(R(0,"div",5)(1,"a",6)(2,"button",7)(3,"mat-icon"),Z(4,"bug_report"),U()()(),R(5,"a",8)(6,"button",9)(7,"mat-icon"),Z(8,"article"),U()()()())}function lY(a){a&1&&(R(0,"div",5)(1,"a",10)(2,"button",11)(3,"mat-icon"),Z(4,"article"),U()()(),R(5,"a",12)(6,"button",13)(7,"mat-icon"),Z(8,"bug_report"),U()()()(),Oq(9,"open-in-new-tab-button"))}var mY=class{constructor(a){this.C=a;this.km=new Jk}get ke(){return!1}get Bd(){return!0}get QB(){return this.C.C}};
mY.K=function(a){return new (a||mY)(y(VJ))};
mY.Ca=Jf({type:mY,ha:[["title-bar"]],outputs:{km:"titleClicked"},ka:!0,features:[Nr],bc:["*"],ya:14,za:3,Ba:[[1,"container"],["matTooltip","Back home",1,"title",3,"click","matTooltipDisabled"],[1,"right-container"],[1,"divider"],[1,"divider","tighter-right"],[1,"icons-container"],["href","https://b.corp.google.com/issues/new?component=1338731&template=1850894","target","_blank",1,"bug-report"],"mat-icon-button;;aria-label;bug;matTooltip;File bug or feature request".split(";"),["href","http://go/model-explorer-doc",
"target","_blank"],"mat-icon-button  aria-label doc matTooltip g3doc".split(" "),["href","https://github.com/google-ai-edge/model-explorer","target","_blank"],"mat-icon-button  aria-label doc matTooltip GitHub".split(" "),["href","https://github.com/google-ai-edge/model-explorer/issues","target","_blank"],"mat-icon-button;;aria-label;bug;matTooltip;File issue or feature request".split(";")],sa:function(a,b){a&1&&(Zq(),R(0,"div",0)(1,"div",1),V("click",function(){return b.km.emit()}),Oq(2,"me-logo"),
Z(3," Model Explorer "),Oq(4,"new-version-chip"),U(),R(5,"div",2),Oq(6,"node-data-provider-dropdown")(7,"div",3)(8,"graph-selector")(9,"div",4)(10,"node-styler"),H(11,kY,9,0,"div",5)(12,lY,10,0),$q(13),U()());a&2&&(D(),Q("matTooltipDisabled",b.QB),D(10),Gq(b.ke?11:-1),D(),Gq(b.Bd?12:-1))},Ha:[it,BB,AB,yB,PE,AE,NE,cT,WJ,iI,hI,OE,hT,pT,jY,kI],styles:[".container[_ngcontent-%COMP%]{width:100%;height:100%;padding:8px;padding-left:12px;-moz-box-sizing:border-box;box-sizing:border-box;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;border-bottom:1px solid #ddd}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]{font-weight:400;cursor:pointer;color:#444746;font-size:20px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   me-logo[_ngcontent-%COMP%]{margin-right:6px}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   new-version-chip[_ngcontent-%COMP%]{margin-left:16px}.container[_ngcontent-%COMP%]   .right-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .divider[_ngcontent-%COMP%]{width:1px;height:28px;border-left:1px solid #ccc;margin:0 16px}.container[_ngcontent-%COMP%]   .divider.tighter-right[_ngcontent-%COMP%]{margin-right:4px}.container[_ngcontent-%COMP%]   .icons-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .icons-container[_ngcontent-%COMP%]   a[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;color:#000}.container[_ngcontent-%COMP%]   open-in-new-tab-button[_ngcontent-%COMP%]{margin-left:8px}"],
Pa:0});function nY(a){a&1&&Oq(0,"benchmark-runner")}function oY(a){if(a&1){const b=Zg();R(0,"title-bar",4);V("titleClicked",function(){t(b);const c=Y(3);return x(c.km.emit())});$q(1);U()}}function pY(a){a&1&&(R(0,"div",3),Z(1,"No graphs"),U())}function qY(a){a&1&&Oq(0,"split-panes-container")}function rY(a){a&1&&(H(0,oY,2,0,"title-bar"),R(1,"div",2),H(2,pY,2,0,"div",3)(3,qY,1,0,"split-panes-container"),U());a&2&&(a=Y(2),Gq(a.wE?0:-1),D(2),Gq(a.qC?2:3))}
function sY(a){if(a&1){const b=Zg();R(0,"div",1);V("dragover",function(c){t(b);const d=Y();return x(d.Uq(c))})("drop",function(c){t(b);const d=Y();return x(d.Vq(c))});H(1,nY,1,0,"benchmark-runner")(2,rY,4,2,"div",2);U()}a&2&&(a=Y(),D(),Gq(a.Ac?1:2))}PH`https://fonts.googleapis.com/icon?family=Material+Icons`;function tY(a){ta(function*(){yield a.N.N;a.ready=!0;en(a.F)})}
function uY(a,b,c){for(const d of a.graphCollections)for(const e of d.graphs)if(e.id===c&&(""===b||e.collectionLabel===b))return e}function vY(a,b,c){const d=zJ(a.C);d?CL(a.D,tI(),b,d,{[d.id]:c},!1):console.warn("Model graph in pane with index 0 doesn't exist")}function wY(a,b,c){const d=zJ(a.C);d?CL(a.D,tI(),b,d,c,!1):console.warn("Model graph in pane with index 0 doesn't exist")}
function xY(a,b,c){var d=a.graphCollections,e=void 0;for(const f of d)if(d=f.graphs.find(g=>g.id===c)){e=d;break}e?(pJ(a.C,e,0),e=a.C.La()[0].id,a.C.N.set(void 0),wJ(a.C,e),a.C.F.set(void 0),EJ(a.C,e,b)):console.warn(`Failed to locate graph with id "${c}"`)}function yY(a,b,c){ta(function*(){yield Promise.all(b.map(d=>FL(a.D,d,c)))})}
var zY=class{constructor(a,b,c,d,e,f,g){this.C=a;this.F=b;this.G=c;this.J=d;this.N=e;this.O=f;this.D=g;this.Ac=!1;this.pD=[];this.km=new Jk;this.Iy=new Jk;this.Uw=new Jk;this.Hx=new Jk;this.ready=!1;Ks(()=>{const h=this.O.C();h&&this.Iy.emit(h)});Ks(()=>{this.Hx.emit(this.C.Og())});window.addEventListener("message",h=>{h=h.data;switch(h.cmd){case "model-explorer-load-node-data-file":var m=h.path;if(m){var p=zJ(this.C);p?yY(this,[m],p):console.warn("Model graph in pane with index 0 doesn't exist")}break;
case "model-explorer-select-node-by-output-tensor-name":const u=h.tensorName;if(u){const w=xJ(this.C);if(w){h=void 0;for(m of w.nodes)if(nI(m)){const B=m.outputsMetadata||{};for(p of Object.keys(B))if(u===B[p].tensor_name){h=m;break}if(h)break}h&&xY(this,h.id,w.id)}}break;case "model-explorer-select-node-by-node-id":(p=h.nodeId)&&(m=xJ(this.C))&&(p=m.nodesById[p])&&xY(this,p.id,m.id)}});C(this.C.wa,tL(this.G)).subscribe(h=>{this.Uw.next(h)});tY(this)}Fb(){this.C.config.set(this.config||{});gJ(this.C,
this.graphCollections);this.C.N.set(this.ie);if(!this.ie||0===this.ie.paneStates.length){if(0<this.graphCollections.length&&0<this.graphCollections[0].graphs.length){var a=[...this.graphCollections[0].graphs].sort((d,e)=>e.nodes.length-d.nodes.length)[0];qJ(this.C,a)}}else if(1===this.ie.paneStates.length){var b=this.ie.paneStates[0];a=uY(this,b.selectedCollectionLabel,b.selectedGraphId);b=!0===b.flattenLayers;a?qJ(this.C,a,b):qJ(this.C,this.graphCollections[0].graphs[0],b);sJ(this.C,b)}else if(2===
this.ie.paneStates.length){a=this.ie.paneStates[0];b=uY(this,a.selectedCollectionLabel,a.selectedGraphId);var c=!0===a.flattenLayers;b?qJ(this.C,b,c):qJ(this.C,this.graphCollections[0].graphs[0],c);sJ(this.C,c);b=this.ie.paneStates[1];c=!0===b.flattenLayers;const d=uY(this,b.selectedCollectionLabel,b.selectedGraphId);d?hJ(this.C,d,c):hJ(this.C,this.graphCollections[0].graphs[0],c);a.selected?vJ(this.C,0):b.selected&&vJ(this.C,1);AJ(this.C,a.widthFraction)}}Xq(a){if(" "===a.key)yI()||this.C.ja.next({});
else if("f"===a.key&&(a.ctrlKey||a.metaKey)){let b;(null==(b=this.config)?0:b.hideTitleBar)||a.preventDefault();this.C.Ea.next({})}}Uq(a){a.preventDefault()}Vq(a){const b=this;return ta(function*(){if(a.dataTransfer){a.stopPropagation();a.preventDefault();var c=[],d;if(null==(d=a.dataTransfer)?0:d.items)[...a.dataTransfer.items].forEach(f=>{"file"===f.kind&&(f=f.getAsFile())&&c.push(f)});else{var e;c.push(...((null==(e=a.dataTransfer)?void 0:e.files)||[]))}d=!1;if(0!==c.length)for(const f of c){if(!f.name.endsWith(".json"))continue;
const g=new FileReader;e=yield new Promise(h=>{g.onload=m=>{let p;m=JSON.parse(null==(p=m.target)?void 0:p.result);h(m)};g.readAsText(f)});null!=e.results&&null==e.results.results?(vY(b,f.name,e),d=!0):Object.values(e).some(h=>null!=h.results)&&(wY(b,f.name,e),d=!0)}d||b.J.open("File(s) not supported","Dismiss")}})}get qC(){return 0===this.graphCollections.reduce((a,b)=>a+b.graphs.length,0)}get wE(){let a;return!(null==(a=this.config)?0:a.hideTitleBar)}};
zY.K=function(a){return new (a||zY)(y(VJ),y(Tr),y(Ci),y(vE),y(KM),y(aJ),y(LL))};
zY.Ca=Jf({type:zY,ha:[["model-graph-visualizer"]],Ia:function(a,b){a&1&&V("keydown",function(c){return b.Xq(c)},!1,Gi)},inputs:{graphCollections:"graphCollections",config:"config",ie:"initialUiState",Ac:"benchmark",pD:"nodeDataSources"},outputs:{km:"titleClicked",Iy:"uiStateChanged",Uw:"modelGraphProcessed",Hx:"remoteNodeDataPathsChanged"},ka:!0,features:[Lr([VJ,uO,LL,sL,aJ,bJ]),Nr],bc:["*"],ya:1,za:1,Ba:[["class","container",3,"dragover","drop",4,"ngIf"],[1,"container",3,"dragover","drop"],[1,"main"],
[1,"no-graphs-message"],[3,"titleClicked"]],sa:function(a,b){a&1&&(Zq(),H(0,sY,3,1,"div",0));a&2&&Q("ngIf",b.ready)},Ha:[tO,it,dt,mY,IS],styles:[".container[_ngcontent-%COMP%]{width:100%;height:100%;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;position:relative}.container[_ngcontent-%COMP%]   benchmark-runner[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1}.container[_ngcontent-%COMP%]   title-bar[_ngcontent-%COMP%]{height:48px;min-height:48px}.container[_ngcontent-%COMP%]   .main[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;position:relative;overflow:hidden}.container[_ngcontent-%COMP%]   .main[_ngcontent-%COMP%]   .no-graphs-message[_ngcontent-%COMP%]{position:absolute;top:0;left:0;width:100%;height:100%;z-index:200;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;color:#999}.container[_ngcontent-%COMP%]   .main[_ngcontent-%COMP%]   split-panes-container[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1}.container[_ngcontent-%COMP%]   .main[_ngcontent-%COMP%]   graph-selector[_ngcontent-%COMP%]{position:absolute;top:12px;left:12px;z-index:100}  .model-explorer-help-popup{font-size:12px;padding:8px;background-color:#fff;line-height:normal}  .model-explorer-help-popup .bold{font-weight:700}  .model-explorer-help-popup .shortcut{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;color:#777;margin-top:8px}  .model-explorer-help-popup .shortcut mat-icon{font-size:20px;width:20px;height:20px;margin-right:2px;color:#777}  .model-explorer-help-popup .description{color:#777;margin-top:8px}"],
Pa:0});const AY=new Map([[1,"float"],[2,"double"],[3,"int32"],[4,"uint8"],[5,"int16"],[6,"int8"],[7,"string"],[8,"complex64"],[9,"int64"],[10,"bool"],[11,"qint8"],[12,"qint8"],[13,"qint32"],[14,"bfloat16"],[15,"qint16"],[16,"qint16"],[17,"uint16"],[18,"complex128"],[19,"half"],[20,"resource"],[21,"variant"],[22,"uint32"],[23,"uint64"]]);
function BY(a,b){const c={};if(null!=b.weightsManifest)for(var d of b.weightsManifest)for(var e of d.weights)c[e.name]=e;const f={};d=b.modelTopology.node.map(p=>{p={id:p.name,label:p.op,namespace:p.name,incomingEdges:CY(p),attrs:DY(p)};return f[p.id]=p});for(var g of b.modelTopology.node)if(f[g.name])for(b=g.input||[],e=0;e<b.length;e++){var h=b[e],m=h.split(":");let p=h;h="0";2===m.length&&(p=m[0],h=m[1]);if(m=f[p])null==m.outputsMetadata&&(m.outputsMetadata=[]),m.outputsMetadata.push({id:h,attrs:[]})}for(const p of d)if(g=
c[p.id])null==p.outputsMetadata&&(p.outputsMetadata=[]),p.outputsMetadata.push({id:"0",attrs:[{key:"tensor_name",value:g.name},{key:"shape",value:g.shape.join("x")},{key:"dtype",value:g.dtype}]});return{label:a,graphs:[{id:"default",nodes:d}]}}function CY(a){return(a.input||[]).map((b,c)=>{const d=b.split(":");return{sourceNodeId:2===d.length?d[0]:b,sourceNodeOutputId:2===d.length?d[1]:"0",targetNodeInputId:`${c}`}})}
function DY(a){a=a.attr||{};const b=[];for(const c of Object.keys(a)){const d=a[c];let e="";null!=d.list?0<(d.list.s||[]).length?e=(d.list.s||[]).map(f=>EY({s:f})).join(", "):0<(d.list.b||[]).length?e=(d.list.b||[]).map(f=>EY({b:f})).join(", "):0<(d.list.f||[]).length?e=(d.list.f||[]).map(f=>EY({f})).join(", "):0<(d.list.i||[]).length?e=(d.list.i||[]).map(f=>EY({i:f})).join(", "):e="[]":e=EY(d);b.push({key:c,value:e})}return b}
function EY(a){let b="";null!=a.s?b=Array.isArray(a.s)?`${a.s.map(c=>"number"===typeof c?String.fromCharCode(c):`${c}`).join("")}`:atob(a.s):null!=a.i?b=`${a.i}`:null!=a.f?b=`${a.f}`:null!=a.b?b=a?"true":"false":null!=a.type&&(b=AY.get(a.type)||"unknown");return b};const FY=/^\/cns\/(.*)|^\/placer\/(.*)|^\/tfhub\/(.*)|^\/google_src\/(.*)|^\/google3\/(.*)|^google3\/(.*)/;function GY(a){return ta(function*(){return new Promise((b,c)=>{const d=new FileReader;d.onload=()=>{const e=JSON.parse(d.result),{graphCollections:f,error:g}=HY(a.name,e);g?c(g):f&&b(f)};d.readAsText(a)})})}
function HY(a,b){var c;null==b.modelTopology?c=null!=b.label&&null!=b.graphs?{graphCollections:[b]}:Array.isArray(b)&&null!=b[0].subgraphs?{graphCollections:IY(b,a)}:Array.isArray(b)&&(0===b.length||0<b.length&&null!=b[0].id&&null!=b[0].nodes)?{graphCollections:[{label:a,graphs:b}]}:{error:"Unsupported JSON format"}:c={graphCollections:[BY(a,b)]};return c}function IY(a,b){return a.map(c=>({label:""===c.label?b:`${b} (${c.label})`,graphs:c.subgraphs}))};var JY={sF:"Not started",tF:"Converting",yF:"Uploading",DONE:"Done",nF:"Error"};const KY=(a,b)=>b.id;function LY(a){a&1&&(R(0,"span",4),Z(1,"Default"),U())}function MY(a,b){if(a&1){const c=Zg();R(0,"div",2);V("click",function(){const d=t(c).da;Y().ne.next(d);return x()});R(1,"div",3);Z(2);H(3,LY,2,0,"span",4);U();R(4,"div",5);Z(5);U()()}if(a&2){a=b.da;b=b.Cb;const c=Y();Wp("selected",(null==c.Sc?null:c.Sc.id)===a.id);D(2);Cr(" ",a.name||a.id," ");D();Gq(0===b?3:-1);D(2);Cr(" ",a.description||"-"," ")}}var NY=class{constructor(){this.lq=[];this.ne=new Jk}};
NY.K=function(a){return new (a||NY)};
NY.Ca=Jf({type:NY,ha:[["adapter-selector-panel"]],inputs:{lq:"candidates",Sc:"selectedAdapter"},outputs:{ne:"onClose"},ka:!0,features:[Nr],ya:3,za:1,Ba:[[1,"container"],[1,"candidate-container",3,"selected"],[1,"candidate-container",3,"click"],[1,"name"],[1,"default-label"],[1,"description"]],sa:function(a,b){a&1&&(R(0,"div",0),Lq(1,MY,6,5,"div",1,KY),U());a&2&&(Q("@transformPanel","showing"),D(),Nq(b.lq))},Ha:[it,PE,cB,iI,PD,Dx],styles:[".container[_ngcontent-%COMP%]{box-shadow:0 2px 6px 2px rgba(0,0,0,.15),0 1px 2px 0 rgba(0,0,0,.3);-webkit-transform-origin:top center;transform-origin:top center;font-size:13px;padding:8px 0;border-radius:8px;background-color:#fff}.container[_ngcontent-%COMP%]   .candidate-container[_ngcontent-%COMP%]{padding:8px 16px}.container[_ngcontent-%COMP%]   .candidate-container.selected[_ngcontent-%COMP%]{background-color:#c2e7ff}.container[_ngcontent-%COMP%]   .candidate-container[_ngcontent-%COMP%]:hover:not(.selected){background-color:#f2f2f2;cursor:pointer}.container[_ngcontent-%COMP%]   .candidate-container[_ngcontent-%COMP%]   .name[_ngcontent-%COMP%]{font-size:14px;color:#1f1f1f;line-height:20px;margin-bottom:2px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .candidate-container[_ngcontent-%COMP%]   .name[_ngcontent-%COMP%]   .default-label[_ngcontent-%COMP%]{display:inline-block;padding:0 10px;-moz-box-sizing:border-box;box-sizing:border-box;background-color:#f3e0b4;color:#70571e;border:1px solid #dbac41;font-size:11px;line-height:14px;border-radius:99px;margin-left:8px}.container[_ngcontent-%COMP%]   .candidate-container[_ngcontent-%COMP%]   .description[_ngcontent-%COMP%]{font-size:12px;color:#444746}"],data:{animation:[Yy("transformPanel",
[az("void",$y({opacity:0,transform:"scale(1, 0.8)"})),bz("void => showing",Zy("120ms cubic-bezier(0, 0, 0.2, 1)",$y({opacity:1,transform:"scale(1, 1)"}))),bz("* => void",Zy("100ms linear",$y({opacity:0})))])]},Pa:0});function OY(a,b){a=a.split("/").filter(c=>""!==c);a=a[a.length-1].split(".");return 1<a.length?b.C[a[a.length-1]]||[]:[]};function PY(a){ta(function*(){let b;b=yield QY();a.extensions=b;a.hd.set(!1)})}function RY(a){return ta(function*(){const b={method:"POST",headers:{"Content-Type":"application/json"}};b.body=JSON.stringify(a);try{const c=yield fetch("/apipost/v1/send_command",b);return c.ok?{xB:yield c.json()}:{jx:`Failed to convert model. ${c.status}`}}catch(c){return{jx:c}}})}
function QY(){return ta(function*(){try{const a=yield fetch("/api/v1/get_extensions",{credentials:"include"});return a.ok?yield a.json():(console.error(`Failed to get extensions: ${a.status}`),[])}catch(a){return console.error("Failed to get extensions.",a),[]}})}var SY=class{constructor(){this.hd=yr(!0);this.extensions=[];PY(this)}};SY.K=function(a){return new (a||SY)};SY.na=Pd({ma:SY,ba:SY.K,fa:"root"});function TY(a,b){for(const c of Object.values(a.C))for(const d of c)if(d.id===b)return d}var UY=class{constructor(){this.C={}}register(a){for(const b of a.fileExts){null==this.C[b]&&(this.C[b]=[]);const c=this.C[b];null!=c.find(d=>d.id===a.id)?console.warn("Adapter extension exists",a):c.push(a)}}},VY=class extends UY{constructor(a){super();this.D=a;Ks(()=>{if(!this.D.hd())for(const b of this.D.extensions)"adapter"===b.type&&this.register(b)})}};VY.K=function(a){return new (a||VY)(r(SY))};
VY.na=Pd({ma:VY,ba:VY.K,fa:"root"});var WY=Symbol("RouteTitle");function XY(a,b,c){const d=c.path.split("/");if(d.length>a.length||"full"===c.px&&(0<b.C||d.length<a.length))return null;b={};for(c=0;c<d.length;c++){const e=d[c],f=a[c];if(":"===e[0])b[e.substring(1)]=f;else if(e!==f.path)return null}return{Vk:a.slice(0,d.length),ux:b}};function YY(a,b){const c=a?ZY(a):void 0;var d=b?ZY(b):void 0;if(!c||!d||c.length!=d.length)return!1;for(let e=0;e<c.length;e++)if(d=c[e],!$Y(a[d],b[d]))return!1;return!0}function ZY(a){return[...Object.keys(a),...Object.getOwnPropertySymbols(a)]}function $Y(a,b){if(Array.isArray(a)&&Array.isArray(b)){if(a.length!==b.length)return!1;a=[...a].sort();const c=[...b].sort();return a.every((d,e)=>c[e]===d)}return a===b}function aZ(a){return Ik(a)?a:vn(a)?Xj(Promise.resolve(a)):oj(a)};const dZ={exact:bZ,subset:cZ},gZ={exact:eZ,subset:fZ,ignored:()=>!0};function hZ(a,b,c){return dZ[c.paths](a.root,b.root,c.Tw)&&gZ[c.tb](a.tb,b.tb)&&!("exact"===c.ac&&a.ac!==b.ac)}function eZ(a,b){return YY(a,b)}function bZ(a,b,c){if(!iZ(a.segments,b.segments)||!jZ(a.segments,b.segments,c)||a.C!==b.C)return!1;for(const d in b.children)if(!a.children[d]||!bZ(a.children[d],b.children[d],c))return!1;return!0}
function fZ(a,b){return Object.keys(b).length<=Object.keys(a).length&&Object.keys(b).every(c=>$Y(a[c],b[c]))}function cZ(a,b,c){return kZ(a,b,b.segments,c)}
function kZ(a,b,c,d){if(a.segments.length>c.length)return a=a.segments.slice(0,c.length),!iZ(a,c)||0<b.C||!jZ(a,c,d)?!1:!0;if(a.segments.length===c.length){if(!iZ(a.segments,c)||!jZ(a.segments,c,d))return!1;for(var e in b.children)if(!a.children[e]||!cZ(a.children[e],b.children[e],d))return!1;return!0}e=c.slice(0,a.segments.length);c=c.slice(a.segments.length);return iZ(a.segments,e)&&jZ(a.segments,e,d)&&a.children.primary?kZ(a.children.primary,b,c,d):!1}
function jZ(a,b,c){return b.every((d,e)=>gZ[c](a[e].parameters,d.parameters))}var nZ=class{constructor(a=new lZ([],{}),b={},c=null){this.root=a;this.tb=b;this.ac=c}toString(){return mZ(this)}},lZ=class{constructor(a,b){this.segments=a;this.children=b;this.parent=null;Object.values(b).forEach(c=>c.parent=this)}get C(){return Object.keys(this.children).length}toString(){return oZ(this)}},qZ=class{constructor(a,b){this.path=a;this.parameters=b}toString(){return pZ(this)}};
function rZ(a,b){return iZ(a,b)&&a.every((c,d)=>YY(c.parameters,b[d].parameters))}function iZ(a,b){return a.length!==b.length?!1:a.every((c,d)=>c.path===b[d].path)}function sZ(a,b){let c=[];Object.entries(a.children).forEach(([d,e])=>{"primary"===d&&(c=c.concat(b(e,d)))});Object.entries(a.children).forEach(([d,e])=>{"primary"!==d&&(c=c.concat(b(e,d)))});return c}var tZ=class{};tZ.K=function(a){return new (a||tZ)};tZ.na=Pd({ma:tZ,ba:()=>new uZ,fa:"root"});
function mZ(a){const b=`/${vZ(a.root,!0)}`,c=wZ(a.tb);a="string"===typeof a.ac?`#${encodeURI(a.ac)}`:"";return`${b}${c}${a}`}
var uZ=class{parse(a){a=new xZ(a);yZ(a,"/");var b=""===a.C||a.C.startsWith("?")||a.C.startsWith("#")?new lZ([],{}):new lZ([],zZ(a));const c={};if(yZ(a,"?")){do{var d,e,f=a.C.match(AZ);if(e=f?f[0]:"")a.capture(e),f="",yZ(a,"=")&&(d=(d=a.C.match(BZ))?d[0]:"")&&(f=d,a.capture(f)),e=decodeURIComponent(e.replace(/\+/g,"%20")),f=decodeURIComponent(f.replace(/\+/g,"%20")),c.hasOwnProperty(e)?(d=c[e],Array.isArray(d)||(d=[d],c[e]=d),d.push(f)):c[e]=f}while(yZ(a,"&"))}return new nZ(b,c,yZ(a,"#")?decodeURIComponent(a.C):
null)}};function oZ(a){return a.segments.map(b=>pZ(b)).join("/")}
function vZ(a,b){if(!(0<a.C))return oZ(a);if(b){b=a.children.primary?vZ(a.children.primary,!1):"";const c=[];Object.entries(a.children).forEach(([d,e])=>{"primary"!==d&&c.push(`${d}:${vZ(e,!1)}`)});return 0<c.length?`${b}(${c.join("//")})`:b}b=sZ(a,(c,d)=>"primary"===d?[vZ(a.children.primary,!1)]:[`${d}:${vZ(c,!1)}`]);return 1===Object.keys(a.children).length&&null!=a.children.primary?`${oZ(a)}/${b[0]}`:`${oZ(a)}/(${b.join("//")})`}
function CZ(a){return encodeURIComponent(a).replace(/%40/g,"@").replace(/%3A/gi,":").replace(/%24/g,"$").replace(/%2C/gi,",")}function DZ(a){return CZ(a).replace(/%3B/gi,";")}function EZ(a){return CZ(a).replace(/\(/g,"%28").replace(/\)/g,"%29").replace(/%26/gi,"&")}function pZ(a){return`${EZ(a.path)}${FZ(a.parameters)}`}function FZ(a){return Object.entries(a).map(([b,c])=>`;${EZ(b)}=${EZ(c)}`).join("")}
function wZ(a){a=Object.entries(a).map(([b,c])=>Array.isArray(c)?c.map(d=>`${DZ(b)}=${DZ(d)}`).join("&"):`${DZ(b)}=${DZ(c)}`).filter(b=>b);return a.length?`?${a.join("&")}`:""}const GZ=/^[^\/()?;#]+/;function HZ(a){return(a=a.match(GZ))?a[0]:""}const IZ=/^[^\/()?;=#]+/,AZ=/^[^=?&#]+/,BZ=/^[^&#]+/;function yZ(a,b){return a.C.startsWith(b)?(a.C=a.C.substring(b.length),!0):!1}
function zZ(a){if(""===a.C)return{};yZ(a,"/");const b=[];for(a.C.startsWith("(")||b.push(JZ(a));a.C.startsWith("/")&&!a.C.startsWith("//")&&!a.C.startsWith("/(");)a.capture("/"),b.push(JZ(a));let c={};a.C.startsWith("/(")&&(a.capture("/"),c=KZ(a,!0));let d={};a.C.startsWith("(")&&(d=KZ(a,!1));if(0<b.length||0<Object.keys(c).length)d.primary=new lZ(b,c);return d}
function JZ(a){var b=HZ(a.C);if(""===b&&a.C.startsWith(";"))throw new ae(4009,!1);a.capture(b);b=decodeURIComponent(b);const c={};for(;yZ(a,";");){var d,e=a;if(d=(d=e.C.match(IZ))?d[0]:""){e.capture(d);var f="";if(yZ(e,"=")){const g=HZ(e.C);g&&(f=g,e.capture(f))}c[decodeURIComponent(d)]=decodeURIComponent(f)}}return new qZ(b,c)}
function KZ(a,b){const c={};for(a.capture("(");!yZ(a,")")&&0<a.C.length;){var d=HZ(a.C),e=a.C[d.length];if("/"!==e&&")"!==e&&";"!==e)throw new ae(4010,!1);e=void 0;-1<d.indexOf(":")?(e=d.slice(0,d.indexOf(":")),a.capture(e),a.capture(":")):b&&(e="primary");d=zZ(a);c[e]=1===Object.keys(d).length?d.primary:new lZ([],d);yZ(a,"//")}return c}class xZ{constructor(a){this.C=this.url=a}capture(a){if(!yZ(this,a))throw new ae(4011,!1);}}function LZ(a){return 0<a.segments.length?new lZ([],{primary:a}):a}
function MZ(a){var b={};for(const [c,d]of Object.entries(a.children)){const e=c,f=MZ(d);if("primary"===e&&0===f.segments.length&&0<f.C)for(const [g,h]of Object.entries(f.children))b[g]=h;else if(0<f.segments.length||0<f.C)b[e]=f}a=new lZ(a.segments,b);1===a.C&&a.children.primary&&(b=a.children.primary,a=new lZ(a.segments.concat(b.segments),b.children));return a}function NZ(a){return a instanceof nZ};function OZ(a,b=null,c=null){a=PZ(a);return QZ(a,[],b,c)}function PZ(a){function b(f){var g={};for(const h of f.children){const m=b(h);g[h.mb]=m}g=new lZ(f.url,g);f===a&&(c=g);return g}let c;var d=b(a.root);d=LZ(d);let e;return null!=(e=c)?e:d}
function QZ(a,b,c,d){let e=a;for(;e.parent;)e=e.parent;if(0===b.length)return RZ(e,e,e,c,d);b=SZ(b);if(b.D&&1===b.C.length&&"/"==b.C[0])return RZ(e,e,new lZ([],{}),c,d);var f=e;if(b.D)a=new TZ(f,!0,0);else if(a)if(null===a.parent)a=new TZ(a,!0,0);else{f=a;a=a.segments.length-1+(UZ(b.C[0])?0:1);let g=b.F;for(;g>a;){g-=a;f=f.parent;if(!f)throw new ae(4005,!1);a=f.segments.length}a=new TZ(f,!1,a-g)}else a=new TZ(f,!1,NaN);b=a.C?VZ(a.sf,a.index,b.C):WZ(a.sf,a.index,b.C);return RZ(e,a.sf,b,c,d)}
function UZ(a){return"object"===typeof a&&null!=a&&!a.Hg&&!a.Qx}function XZ(a){return"object"===typeof a&&null!=a&&a.Hg}function RZ(a,b,c,d,e){let f={};d&&Object.entries(d).forEach(([g,h])=>{f[g]=Array.isArray(h)?h.map(m=>`${m}`):`${h}`});a=a===b?c:YZ(a,b,c);a=LZ(MZ(a));return new nZ(a,f,e)}function YZ(a,b,c){const d={};Object.entries(a.children).forEach(([e,f])=>{d[e]=f===b?c:YZ(f,b,c)});return new lZ(a.segments,d)}
class ZZ{constructor(a,b,c){this.D=a;this.F=b;this.C=c;if(a&&0<c.length&&UZ(c[0]))throw new ae(4003,!1);if((a=c.find(XZ))&&a!==(0<c.length?c[c.length-1]:null))throw new ae(4004,!1);}}
function SZ(a){if("string"===typeof a[0]&&1===a.length&&"/"===a[0])return new ZZ(!0,0,a);let b=0,c=!1;a=a.reduce((d,e,f)=>{if("object"===typeof e&&null!=e){if(e.Hg){const g={};Object.entries(e.Hg).forEach(([h,m])=>{g[h]="string"===typeof m?m.split("/"):m});return[...d,{Hg:g}]}if(e.Qx)return[...d,e.Qx]}return"string"!==typeof e?[...d,e]:0===f?(e.split("/").forEach((g,h)=>{if(0!=h||"."!==g)0==h&&""===g?c=!0:".."===g?b++:""!=g&&d.push(g)}),d):[...d,e]},[]);return new ZZ(c,b,a)}
class TZ{constructor(a,b,c){this.sf=a;this.C=b;this.index=c}}
function WZ(a,b,c){null!=a||(a=new lZ([],{}));if(0===a.segments.length&&0<a.C)return VZ(a,b,c);a:{var d=a;var e=0;let h=b;const m={match:!1,wo:0,tv:0};for(;h<d.segments.length;){if(e>=c.length){d=m;break a}const p=d.segments[h];var f=c[e];if(XZ(f))break;f=`${f}`;var g=e<c.length-1?c[e+1]:null;if(0<h&&void 0===f)break;if(f&&g&&"object"===typeof g&&void 0===g.Hg){if(f!=p.path||!YY(g,p.parameters)){d=m;break a}e+=2}else{g={};if(f!=p.path||!YY(g,p.parameters)){d=m;break a}e++}h++}d={match:!0,wo:h,tv:e}}e=
c.slice(d.tv);return d.match&&d.wo<a.segments.length?(b=new lZ(a.segments.slice(0,d.wo),{}),b.children.primary=new lZ(a.segments.slice(d.wo),a.children),VZ(b,0,e)):d.match&&0===e.length?new lZ(a.segments,{}):!d.match||0<a.C?d.match?VZ(a,0,e):$Z(a,b,c):$Z(a,b,c)}
function VZ(a,b,c){if(0===c.length)return new lZ(a.segments,{});const d=XZ(c[0])?c[0].Hg:{primary:c},e={};if(Object.keys(d).some(f=>"primary"!==f)&&a.children.primary&&1===a.C&&0===a.children.primary.segments.length)return c=VZ(a.children.primary,b,c),new lZ(a.segments,c.children);Object.entries(d).forEach(([f,g])=>{"string"===typeof g&&(g=[g]);null!==g&&(e[f]=WZ(a.children[f],b,g))});Object.entries(a.children).forEach(([f,g])=>{void 0===d[f]&&(e[f]=g)});return new lZ(a.segments,e)}
function $Z(a,b,c){const d=a.segments.slice(0,b);let e=0;for(;e<c.length;){var f=c[e];if(XZ(f))return a=a_(f.Hg),new lZ(d,a);if(0===e&&UZ(c[0])){d.push(new qZ(a.segments[b].path,b_(c[0])));e++;continue}f=XZ(f)?f.Hg.primary:`${f}`;const g=e<c.length-1?c[e+1]:null;f&&g&&UZ(g)?(d.push(new qZ(f,b_(g))),e+=2):(d.push(new qZ(f,{})),e++)}return new lZ(d,{})}function a_(a){const b={};Object.entries(a).forEach(([c,d])=>{"string"===typeof d&&(d=[d]);null!==d&&(b[c]=$Z(new lZ([],{}),0,d))});return b}
function b_(a){const b={};Object.entries(a).forEach(([c,d])=>b[c]=`${d}`);return b};var c_=class{constructor(a,b){this.id=a;this.url=b}},d_=class extends c_{constructor(a,b,c=null){super(a,b);this.type=0;this.Qj=c}toString(){return`NavigationStart(id: ${this.id}, url: '${this.url}')`}},e_=class extends c_{constructor(a,b,c){super(a,b);this.dc=c;this.type=1}toString(){return`NavigationEnd(id: ${this.id}, url: '${this.url}', urlAfterRedirects: '${this.dc}')`}},f_=class extends c_{constructor(a,b,c,d){super(a,b);this.reason=c;this.code=d;this.type=2}toString(){return`NavigationCancel(id: ${this.id}, url: '${this.url}')`}},
g_=class extends c_{constructor(a,b){super(a,b);this.reason="";this.code=0;this.type=16}},h_=class extends c_{constructor(a,b,c,d){super(a,b);this.error=c;this.target=d;this.type=3}toString(){return`NavigationError(id: ${this.id}, url: '${this.url}', error: ${this.error})`}},i_=class extends c_{constructor(a,b,c,d){super(a,b);this.dc=c;this.state=d;this.type=4}toString(){return`RoutesRecognized(id: ${this.id}, url: '${this.url}', urlAfterRedirects: '${this.dc}', state: ${this.state})`}},j_=class extends c_{constructor(a,
b,c,d){super(a,b);this.dc=c;this.state=d;this.type=7}toString(){return`GuardsCheckStart(id: ${this.id}, url: '${this.url}', urlAfterRedirects: '${this.dc}', state: ${this.state})`}},k_=class extends c_{constructor(a,b,c,d,e){super(a,b);this.dc=c;this.state=d;this.C=e;this.type=8}toString(){return`GuardsCheckEnd(id: ${this.id}, url: '${this.url}', urlAfterRedirects: '${this.dc}', state: ${this.state}, shouldActivate: ${this.C})`}},l_=class extends c_{constructor(a,b,c,d){super(a,b);this.dc=c;this.state=
d;this.type=5}toString(){return`ResolveStart(id: ${this.id}, url: '${this.url}', urlAfterRedirects: '${this.dc}', state: ${this.state})`}},m_=class extends c_{constructor(a,b,c,d){super(a,b);this.dc=c;this.state=d;this.type=6}toString(){return`ResolveEnd(id: ${this.id}, url: '${this.url}', urlAfterRedirects: '${this.dc}', state: ${this.state})`}},n_=class{constructor(a){this.C=a;this.type=9}toString(){return`RouteConfigLoadStart(path: ${this.C.path})`}},o_=class{constructor(a){this.C=a;this.type=
10}toString(){return`RouteConfigLoadEnd(path: ${this.C.path})`}},p_=class{constructor(a){this.kb=a;this.type=11}toString(){return`ChildActivationStart(path: '${this.kb.jb&&this.kb.jb.path||""}')`}},q_=class{constructor(a){this.kb=a;this.type=12}toString(){return`ChildActivationEnd(path: '${this.kb.jb&&this.kb.jb.path||""}')`}},r_=class{constructor(a){this.kb=a;this.type=13}toString(){return`ActivationStart(path: '${this.kb.jb&&this.kb.jb.path||""}')`}},s_=class{constructor(a){this.kb=a;this.type=
14}toString(){return`ActivationEnd(path: '${this.kb.jb&&this.kb.jb.path||""}')`}},t_=class{},u_=class{constructor(a,b){this.url=a;this.Kl=b}};var w_=class{constructor(a){this.xa=a;this.C=this.mb=null;this.children=new v_(this.xa);this.D=null}};function x_(a,b){let c=a.getContext(b);c||(c=new w_(a.Gj),a.C.set(b,c));return c}function y_(a,b){if(a=a.getContext(b))a.mb=null,a.D=null}var v_=class{constructor(a){this.Gj=a;this.C=new Map}getContext(a){return this.C.get(a)||null}};v_.K=function(a){return new (a||v_)(r(og))};v_.na=Pd({ma:v_,ba:v_.K,fa:"root"});function z_(a,b){return A_(b,a.vb).map(c=>c.value)}var C_=class{constructor(a){this.vb=a}get root(){return this.vb.value}parent(a){a=z_(this,a);return 1<a.length?a[a.length-2]:null}children(a){return(a=B_(a,this.vb))?a.children.map(b=>b.value):[]}firstChild(a){return(a=B_(a,this.vb))&&0<a.children.length?a.children[0].value:null}};function B_(a,b){if(a===b.value)return b;for(const c of b.children)if(b=B_(a,c))return b;return null}
function A_(a,b){if(a===b.value)return[b];for(const c of b.children){const d=A_(a,c);if(d.length)return d.unshift(b),d}return[]}var D_=class{constructor(a,b){this.value=a;this.children=b}toString(){return`TreeNode(${this.value})`}};function E_(a){const b={};a&&a.children.forEach(c=>b[c.value.mb]=c);return b};var G_=class extends C_{constructor(a,b){super(a);this.kb=b;F_(this,a)}toString(){return this.kb.toString()}},H_=class{constructor(a,b,c,d,e,f,g,h){this.N=a;this.G=b;this.J=c;this.F=d;this.D=e;this.mb=f;this.component=g;this.C=h;let m,p;this.title=null!=(p=null==(m=this.D)?void 0:C(m,Cj(u=>u[WY])))?p:oj(void 0);this.url=a;this.params=b;this.tb=c;this.ac=d;this.data=e}get jb(){return this.C.jb}get root(){return this.mg.root}get parent(){return this.mg.parent(this)}get firstChild(){return this.mg.firstChild(this)}get children(){return this.mg.children(this)}toString(){return this.kb?
this.kb.toString():`Future(${this.C})`}};
function I_(a,b,c="emptyOnly"){const d=a.jb;let e;if(null===b||"always"!==c&&""!==(null==d?void 0:d.path)&&(b.component||(null==(e=b.jb)?0:e.Fl))){let f;a={params:Object.assign({},a.params),data:Object.assign({},a.data),resolve:Object.assign({},a.data,null!=(f=a.C)?f:{})}}else a={params:Object.assign({},b.params,a.params),data:Object.assign({},b.data,a.data),resolve:Object.assign({},a.data,b.data,null==d?void 0:d.data,a.C)};!d||"string"!==typeof d.title&&null!==d.title||(a.resolve[WY]=d.title);return a}
var J_=class{get title(){let a;return null==(a=this.data)?void 0:a[WY]}constructor(a,b,c,d,e,f,g,h,m){this.url=a;this.params=b;this.tb=c;this.ac=d;this.data=e;this.mb=f;this.component=g;this.jb=h;this.D=m}get root(){return this.mg.root}get parent(){return this.mg.parent(this)}get firstChild(){return this.mg.firstChild(this)}get children(){return this.mg.children(this)}toString(){return`Route(url:'${this.url.map(a=>a.toString()).join("/")}', path:'${this.jb?this.jb.path:""}')`}},L_=class extends C_{constructor(a){super(a);
this.url="";F_(this,a)}toString(){return K_(this.vb)}};function F_(a,b){b.value.mg=a;b.children.forEach(c=>F_(a,c))}function K_(a){const b=0<a.children.length?` { ${a.children.map(K_).join(", ")} } `:"";return`${a.value}${b}`}
function M_(a){if(a.kb){const d=a.kb,e=a.C;a.kb=e;YY(d.tb,e.tb)||a.J.next(e.tb);d.ac!==e.ac&&a.F.next(e.ac);YY(d.params,e.params)||a.G.next(e.params);a:{var b=d.url;var c=e.url;if(b.length!==c.length)b=!1;else{for(let f=0;f<b.length;++f)if(!YY(b[f],c[f])){b=!1;break a}b=!0}}b||a.N.next(e.url);YY(d.data,e.data)||a.D.next(e.data)}else a.kb=a.C,a.D.next(a.C.data)}function N_(a,b){const c=YY(a.params,b.params)&&rZ(a.url,b.url),d=!a.parent!==!b.parent;return c&&!d&&(!a.parent||N_(a.parent,b.parent))};function O_(a,b){let c;return(null==(c=a.D.getContext(b))?void 0:c.mb)===a}function P_(a){var b=a.D,c=a.name;const d=x_(b,c);d.mb=a;b.C.set(c,d);!a.C&&(b=a.D.getContext(a.name),null==b?0:b.C)&&(b.D?a.attach(b.D,b.C):Q_(a,b.C,b.xa))}function Q_(a,b,c){if(a.pr)throw new ae(4013,!1);a.F=b;const d=a.location,e=b.kb.component,f=x_(a.D,a.name).children;a.C=tp(d,e,{index:d.length,xa:new R_(b,f,d.xa),Pf:c});en(a.J);let g;null==(g=a.G)||g.C(a);a.Ou.emit(a.C.instance)}
var T_=class{constructor(){this.F=this.C=null;this.name="primary";this.Ou=new Jk;this.Mv=new Jk;this.Xu=new Jk;this.Qv=new Jk;this.D=of(v_);this.location=of(np);this.J=of(Tr);this.G=of(S_,{optional:!0})}uc(a){if(a.name){const {Bn:b,xx:c}=a.name;b||(O_(this,c)&&(this.pn(),y_(this.D,c)),P_(this))}}va(){O_(this,this.name)&&y_(this.D,this.name);let a;null==(a=this.G)||a.D(this)}Fb(){P_(this)}get pr(){return!!this.C}get component(){if(!this.C)throw new ae(4012,!1);return this.C.instance}detach(){if(!this.C)throw new ae(4012,
!1);this.location.detach();const a=this.C;this.F=this.C=null;this.Qv.emit(a.instance);return a}attach(a,b){this.C=a;this.F=b;this.location.kr(a.C);let c;null==(c=this.G)||c.C(this);this.Xu.emit(a.instance)}pn(){if(this.C){const a=this.component;this.C.destroy();this.F=this.C=null;this.Mv.emit(a)}}};T_.K=function(a){return new (a||T_)};T_.Fa=Uf({type:T_,ha:[["router-outlet"]],inputs:{name:"name"},outputs:{Ou:"activate",Mv:"deactivate",Xu:"attach",Qv:"detach"},ib:["outlet"],ka:!0,features:[Hg]});
class R_{F(a){return new R_(this.C,this.D,a)}constructor(a,b,c){this.C=a;this.D=b;this.parent=c}get(a,b){return a===H_?this.C:a===v_?this.D:this.parent.get(a,b)}}var S_=new gf("");function U_(a,b,c){if(c&&b.value.jb===c.value.kb.jb){var d=c.value;d.C=b.value;b=V_(a,b,c);return new D_(d,b)}d=W_(b.value);b=b.children.map(e=>U_(a,e));return new D_(d,b)}function V_(a,b,c){return b.children.map(d=>{for(const e of c.children)if(d.value.jb===e.value.kb.jb)return U_(a,d,e);return U_(a,d)})}function W_(a){return new H_(new jj(a.url),new jj(a.params),new jj(a.tb),new jj(a.ac),new jj(a.data),a.mb,a.component,a)};function X_(a){const {Vl:b,Kl:c}=NZ(a)?{Vl:a,Kl:void 0}:a;a=Y_(0);a.url=b;a.Kl=c;return a}function Y_(a){const b=Error("NavigationCancelingError: ");b.ngNavigationCancelingError=!0;b.pB=a;return b}function Z_(a){return!!a&&a.ngNavigationCancelingError};var $_=class{};$_.K=function(a){return new (a||$_)};$_.Ca=Jf({type:$_,ha:[["ng-component"]],ka:!0,features:[Nr],ya:1,za:0,sa:function(a){a&1&&Oq(0,"router-outlet")},Ha:[T_],pb:2});function a0(a,b){a.Wa&&!a.Ra&&(a.Ra=un(a.Wa,b,`Route: ${a.path}`));let c;return null!=(c=a.Ra)?c:b}function b0(a){const b=a.children&&a.children.map(b0);a=b?Object.assign({},a,{children:b}):Object.assign({},a);a.component||a.Fl||!b&&!a.co||!a.mb||"primary"===a.mb||(a.component=$_);return a}function c0(a){return a.mb||"primary"}function d0(a,b){const c=a.filter(d=>c0(d)===b);c.push(...a.filter(d=>c0(d)!==b));return c}
function e0(a){if(!a)return null;var b;if(null==(b=a.jb)?0:b.Ra)return a.jb.Ra;for(a=a.parent;a;a=a.parent){b=a.jb;let c;if(null==(c=b)?0:c.Bk)return b.Bk;let d;if(null==(d=b)?0:d.Ra)return b.Ra}return null};var i0=(a,b,c)=>Cj(d=>{var e=new f0(b,d.jm,d.wq,c);const f=e.F.vb,g=e.C?e.C.vb:null;g0(e,f,g,a);M_(e.F.root);h0(e,f,g,a);return d});function g0(a,b,c,d){const e=E_(c);b.children.forEach(f=>{const g=f.value.mb;var h=e[g],m=f.value;const p=h?h.value:null;m===p?m.component?(m=d.getContext(m.mb))&&g0(a,f,h,m.children):g0(a,f,h,d):p&&j0(a,h,d);delete e[g]});Object.values(e).forEach(f=>{j0(a,f,d)})}
function h0(a,b,c,d){const e=E_(c);b.children.forEach(f=>{var g=e[f.value.mb],h=f.value,m=g?g.value:null;M_(h);h===m?h.component?(h=x_(d,h.mb),h0(a,f,g,h.children)):h0(a,f,g,d):h.component?(g=x_(d,h.mb),m=e0(h.kb),g.D=null,g.C=h,g.xa=null!=m?m:g.xa,g.mb&&Q_(g.mb,h,g.xa),h0(a,f,null,g.children)):h0(a,f,null,d);a.D(new s_(f.value.kb))});b.children.length&&a.D(new q_(b.value.kb))}
function j0(a,b,c){const d=c.getContext(b.value.mb);c=d&&b.value.component?d.children:c;b=E_(b);for(const e of Object.values(b))j0(a,e,c);d&&(d.mb&&(d.mb.pn(),d.children.C=new Map),d.D=null,d.C=null)}var f0=class{constructor(a,b,c,d){this.rs=a;this.F=b;this.C=c;this.D=d}};const k0=Symbol("INITIAL_VALUE");function l0(){return Lo(a=>C(Uj(a.map(b=>C(b,io(),Ko(k0)))),Cj(b=>{for(const c of b)if(!0!==c){if(c===k0)return k0;if(!1===c||NZ(c))return c}return!0}),xk(b=>b!==k0),io()))};var m0=class{constructor(a){this.path=a;this.C=this.path[this.path.length-1]}},n0=class{constructor(a,b){this.component=a;this.C=b}};function o0(a,b){const c=Symbol(),d=b.get(a,c);return d===c?"function"!==typeof a||null!==Rd(a)?b.get(a):a:d}function p0(a,b,c,d,e={canDeactivateChecks:[],canActivateChecks:[]}){const f=E_(b);a.children.forEach(g=>{q0(g,f[g.value.mb],c,d.concat([g.value]),e);delete f[g.value.mb]});Object.entries(f).forEach(([g,h])=>r0(h,c.getContext(g),e));return e}
function q0(a,b,c,d,e={canDeactivateChecks:[],canActivateChecks:[]}){const f=a.value,g=b?b.value:null,h=c?c.getContext(a.value.mb):null;if(g&&f.jb===g.jb){const m=s0(g,f,f.jb.GG);m?e.canActivateChecks.push(new m0(d)):(f.data=g.data,f.C=g.C);f.component?p0(a,b,h?h.children:null,d,e):p0(a,b,c,d,e);m&&h&&h.mb&&h.mb.pr&&e.canDeactivateChecks.push(new n0(h.mb.component,g))}else g&&r0(b,h,e),e.canActivateChecks.push(new m0(d)),f.component?p0(a,null,h?h.children:null,d,e):p0(a,null,c,d,e)}
function s0(a,b,c){if("function"===typeof c)return c(a,b);switch(c){case "pathParamsChange":return!iZ(a.url,b.url);case "pathParamsOrQueryParamsChange":return!iZ(a.url,b.url)||!YY(a.tb,b.tb);case "always":return!0;case "paramsOrQueryParamsChange":return!N_(a,b)||!YY(a.tb,b.tb);default:return!N_(a,b)}}
function r0(a,b,c){const d=E_(a),e=a.value;Object.entries(d).forEach(([f,g])=>{e.component?b?r0(g,b.children.getContext(f),c):r0(g,null,c):r0(g,b,c)});e.component?b&&b.mb&&b.mb.pr?c.canDeactivateChecks.push(new n0(b.mb.component,e)):c.canDeactivateChecks.push(new n0(null,e)):c.canDeactivateChecks.push(new n0(null,e))};function t0(a){return"function"===typeof a}function u0(a){return a instanceof uj||"EmptyError"===(null==a?void 0:a.name)};function v0(a,b){return Yj(c=>{const d=c.qd,e=c.Fh;var f=c.tj;const g=f.canActivateChecks;f=f.canDeactivateChecks;return 0===f.length&&0===g.length?oj(Object.assign({},c,{Qf:!0})):C(w0(f,d,e,a),Yj(h=>h&&"boolean"===typeof h?x0(d,g,a,b):oj(h)),Cj(h=>Object.assign({},c,{Qf:h})))})}function w0(a,b,c,d){return C(Xj(a),Yj(e=>y0(e.component,e.C,c,b,d)),ro(e=>!0!==e,!0))}
function x0(a,b,c,d){return C(Xj(b),Yj(e=>{var f=e.C.parent;null!==f&&d&&d(new p_(f));f=oj(!0);var g=e.C;null!==g&&d&&d(new r_(g));g=oj(!0);return mk(f,g,z0(a,e.path,c),A0(a,e.C,c))},1),ro(e=>!0!==e,!0))}function A0(a,b,c){var d=b.jb?b.jb.dv:null;if(!d||0===d.length)return oj(!0);d=d.map(e=>nk(()=>{var f;const g=null!=(f=e0(b))?f:c,h=o0(e,g);f=h&&t0(h.dv)?h.dv(b,a):Cg(g,()=>h(b,a));return C(aZ(f),ro())}));return C(oj(d),l0())}
function z0(a,b,c){const d=b[b.length-1];b=b.slice(0,b.length-1).reverse().map(e=>{const f=e.jb?e.jb.ev:null;return f&&0!==f.length?{node:e,tj:f}:null}).filter(e=>null!==e).map(e=>nk(()=>{const f=e.tj.map(g=>{let h;const m=null!=(h=e0(e.node))?h:c,p=o0(g,m);g=p&&t0(p.ev)?p.ev(d,a):Cg(m,()=>p(d,a));return C(aZ(g),ro())});return C(oj(f),l0())}));return C(oj(b),l0())}
function y0(a,b,c,d,e){var f=b&&b.jb?b.jb.fv:null;if(!f||0===f.length)return oj(!0);f=f.map(g=>{let h;const m=null!=(h=e0(b))?h:e,p=o0(g,m);g=p&&t0(p.fv)?p.fv(a,b,c,d):Cg(m,()=>p(a,b,c,d));return C(aZ(g),ro())});return C(oj(f),l0())}function B0(a,b,c){var d=b.gv;if(void 0===d||0===d.length)return oj(!0);d=d.map(e=>{const f=o0(e,a);e=f&&t0(f.gv)?f.gv(b,c):Cg(a,()=>f(b,c));return aZ(e)});return C(oj(d),l0(),C0())}
function C0(){return Zi([So(a=>{if("boolean"!==typeof a)throw X_(a);}),Cj(a=>!0===a)])}function D0(a,b,c){var d=b.hv;if(!d||0===d.length)return oj(!0);d=d.map(e=>{const f=o0(e,a);e=f&&t0(f.hv)?f.hv(b,c):Cg(a,()=>f(b,c));return aZ(e)});return C(oj(d),l0(),C0())};var E0=class{constructor(a){this.sf=a||null}},F0=class extends Error{constructor(a){super();this.C=a}};function G0(a){return pj(new E0(a))}function H0(a){let b=[];for(a=a.root;;){b=b.concat(a.segments);if(0===a.C)return oj(b);if(1<a.C||!a.children.primary)return pj(new ae(4E3,!1));a=a.children.primary}}function I0(a,b,c,d,e){b=J0(a,b,c.root,d,e);return new nZ(b,K0(c.tb,a.D.tb),c.ac)}
function L0(a,b,c,d,e,f){if("string"!==typeof c){const g=c,h=e.tb,m=e.ac,p=e.jb,u=e.url,w=e.mb,B=e.params,G=e.data,L=e.title;c=Cg(f,()=>g({params:B,data:G,tb:h,ac:m,jb:p,url:u,mb:w,title:L}));if(c instanceof nZ)throw new F0(c);}a=I0(a,c,a.C.parse(c),b,d);if("/"===c[0])throw new F0(a);return a}function J0(a,b,c,d,e){const f=M0(c.segments,d,e);let g={};Object.entries(c.children).forEach(([h,m])=>{g[h]=J0(a,b,m,d,e)});return new lZ(f,g)}
function K0(a,b){const c={};Object.entries(a).forEach(([d,e])=>{c[d]="string"===typeof e&&":"===e[0]?b[e.substring(1)]:e});return c}function M0(a,b,c){return a.map(d=>{if(":"===d.path[0]){if(d=c[d.path.substring(1)],!d)throw new ae(4001,!1);}else a:{let e=0;for(const f of b){if(f.path===d.path){b.splice(e);d=f;break a}e++}}return d})}var N0=class{constructor(a,b){this.C=a;this.D=b}};const O0={Qh:!1,Wk:[],Xl:[],parameters:{},Ao:{}};function P0(a,b,c,d){const e=Q0(a,b,c);if(!e.Qh)return oj(e);d=a0(b,d);return C(D0(d,b,c),Cj(f=>!0===f?e:Object.assign({},O0)))}
function Q0(a,b,c){if("**"===b.path)return{Qh:!0,parameters:0<c.length?(0<c.length?c[c.length-1]:null).parameters:{},Wk:c,Xl:[],Ao:{}};if(""===b.path)return"full"===b.px&&(0<a.C||0<c.length)?Object.assign({},O0):{Qh:!0,Wk:[],Xl:c,parameters:{},Ao:{}};a=(b.yr||XY)(c,a,b);if(!a)return Object.assign({},O0);const d={};var e;Object.entries(null!=(e=a.ux)?e:{}).forEach(([g,h])=>{d[g]=h.path});e=0<a.Vk.length?Object.assign({},d,a.Vk[a.Vk.length-1].parameters):d;let f;return{Qh:!0,Wk:a.Vk,Xl:c.slice(a.Vk.length),
parameters:e,Ao:null!=(f=a.ux)?f:{}}}function R0(a,b,c,d){if(0<c.length&&S0(a,c,d)){var e=lZ;a=new lZ(c,a.children);c={};c.primary=a;for(var f of d)""===f.path&&"primary"!==c0(f)&&(d=new lZ([],{}),c[c0(f)]=d);c={sf:new e(b,c),Gs:[]}}else if(0===c.length&&T0(a,c,d)){b=lZ;f=a.segments;var g=a.children;const h={};for(e of d)U0(a,c,e)&&!g[c0(e)]&&(d=new lZ([],{}),h[c0(e)]=d);d=Object.assign({},g,h);c={sf:new b(f,d),Gs:c}}else c={sf:new lZ(a.segments,a.children),Gs:c};return c}
function S0(a,b,c){return c.some(d=>U0(a,b,d)&&"primary"!==c0(d))}function T0(a,b,c){return c.some(d=>U0(a,b,d))}function U0(a,b,c){return(0<a.C||0<b.length)&&"full"===c.px?!1:""===c.path};class V0{}function W0(a,b,c,d,e,f,g="emptyOnly"){return(new X0(a,b,c,d,e,g,f)).recognize()}function Y0(a,b,c,d,e,f){return 0===d.segments.length&&0<d.C?Z0(a,b,c,d,f):C($0(a,b,c,d,d.segments,e,!0,f),Cj(g=>g instanceof D_?[g]:[]))}
function Z0(a,b,c,d,e){const f=[];for(const g of Object.keys(d.children))"primary"===g?f.unshift(g):f.push(g);return C(Xj(f),Yj(g=>{const h=d.children[g],m=d0(c,g);return Y0(a,b,m,h,g,e)},1),zo((g,h)=>{g.push(...h);return g}),Xn(null),vo(),Yj(g=>{if(null===g)return G0(d);g=a1(g);b1(g);return oj(g)}))}
function $0(a,b,c,d,e,f,g,h){return C(Xj(c),Yj(m=>{let p;return C(c1(a,null!=(p=m.Ra)?p:b,c,m,d,e,f,g,h),Pn(u=>{if(u instanceof E0)return oj(null);throw u;}))},1),ro(m=>!!m),Pn(m=>{if(u0(m))return 0!==e.length||d.children[f]?G0(d):oj(new V0);throw m;}))}function c1(a,b,c,d,e,f,g,h,m){return(c0(d)===g||"primary"!==g&&U0(e,f,d))&&Q0(e,d,f).Qh?void 0===d.Vl?d1(a,b,e,d,f,g,m):a.F&&h?e1(a,b,e,c,d,f,g,m):G0(e):G0(e)}
function d1(a,b,c,d,e,f,g){const h=P0(c,d,e,b);"**"===d.path&&(c.children={});return C(h,Lo(m=>{if(!m.Qh)return G0(c);let p;b=null!=(p=d.Ra)?p:b;return C(f1(a,b,d,e),Lo(({Sj:u})=>{var w;const B=null!=(w=d.Bk)?w:b;w=m.Wk;const G=m.Xl;var L;let O;const ba=new J_(w,m.parameters,Object.freeze(Object.assign({},a.C.tb)),a.C.ac,d.data||{},c0(d),null!=(O=null!=(L=d.component)?L:d.Ak)?O:null,d,d.resolve||{});L=I_(ba,g,a.Fj);ba.params=Object.freeze(L.params);ba.data=Object.freeze(L.data);const {sf:ca,Gs:na}=
R0(c,w,G,u);return 0===na.length&&0<ca.C?C(Z0(a,B,u,ca,ba),Cj(K=>new D_(ba,K))):0===u.length&&0===na.length?oj(new D_(ba,[])):C($0(a,B,u,ca,na,c0(d)===f?"primary":f,!0,ba),Cj(K=>new D_(ba,K instanceof D_?[K]:[])))}))}))}
function e1(a,b,c,d,e,f,g,h){const {Qh:m,parameters:p,Wk:u,Ao:w,Xl:B}=Q0(c,e,f);if(!m)return G0(c);"string"===typeof e.Vl&&"/"===e.Vl[0]&&(a.D++,31<a.D&&(a.F=!1));var G;let L;f=new J_(f,p,Object.freeze(Object.assign({},a.C.tb)),a.C.ac,e.data||{},c0(e),null!=(L=null!=(G=e.component)?G:e.Ak)?L:null,e,e.resolve||{});G=I_(f,h,a.Fj);f.params=Object.freeze(G.params);f.data=Object.freeze(G.data);e=L0(a.O,u,e.Vl,w,f,b);return C(H0(e),Yj(O=>$0(a,b,d,c,O.concat(B),g,!1,h)))}
function f1(a,b,c,d){return c.children?oj({Sj:c.children,xa:b}):c.co?void 0!==c.Im?oj({Sj:c.Im,xa:c.Bk}):C(B0(b,c,d),Yj(e=>e?C(a.G.co(b,c),So(f=>{c.Im=f.Sj;c.Bk=f.xa})):pj(Y_(3)))):oj({Sj:[],xa:b})}
var X0=class{constructor(a,b,c,d,e,f,g){this.xa=a;this.G=b;this.J=c;this.config=d;this.C=e;this.Fj=f;this.N=g;this.O=new N0(this.N,this.C);this.D=0;this.F=!0}recognize(){const a=R0(this.C.root,[],[],this.config).sf;return C(this.match(a),Cj(({children:b,WD:c})=>{b=new L_(new D_(c,b));c=OZ(c,this.C.tb,this.C.ac);c.tb=this.C.tb;b.url=mZ(c);return{state:b,Ey:c}}))}match(a){const b=new J_([],Object.freeze({}),Object.freeze(Object.assign({},this.C.tb)),this.C.ac,Object.freeze({}),"primary",this.J,null,
{});return C(Y0(this,this.xa,this.config,a,"primary",b),Cj(c=>({children:c,WD:b})),Pn(c=>{if(c instanceof F0)return this.C=c.C,this.match(c.C.root);if(c instanceof E0)throw new ae(4002,`'${c.sf}'`);throw c;}))}};function b1(a){a.sort((b,c)=>"primary"===b.value.mb?-1:"primary"===c.value.mb?1:b.value.mb.localeCompare(c.value.mb))}function g1(a){return(a=a.value.jb)&&""===a.path}
function a1(a){const b=[],c=new Set;for(const d of a)g1(d)?(a=b.find(e=>d.value.jb===e.value.jb),void 0!==a?(a.children.push(...d.children),c.add(a)):b.push(d)):b.push(d);for(const d of c)a=a1(d.children),b.push(new D_(d.value,a));return b.filter(d=>!c.has(d))};function h1(a,b,c,d,e,f){return Yj(g=>C(W0(a,b,c,d,g.ed,e,f),Cj(({state:h,Ey:m})=>Object.assign({},g,{qd:h,dc:m}))))};function i1(a,b){return Yj(c=>{const d=c.qd,e=c.tj.canActivateChecks;if(!e.length)return oj(c);const f=new Set(e.map(m=>m.C)),g=new Set;for(const m of f)if(!g.has(m))for(const p of j1(m))g.add(p);let h=0;return C(Xj(g),Yj(m=>{if(f.has(m))return k1(m,d,a,b);m.data=I_(m,m.parent,a).resolve;return oj(void 0)},1),So(()=>h++),so(),Yj(()=>h===g.size?oj(c):kj))})}function j1(a){const b=a.children.map(c=>j1(c)).flat();return[a,...b]}
function k1(a,b,c,d){const e=a.jb,f=a.D;void 0!==(null==e?void 0:e.title)&&"string"!==typeof e.title&&null!==e.title&&(f[WY]=e.title);return C(l1(f,a,b,d),Cj(g=>{a.C=g;a.data=I_(a,a.parent,c).resolve;return null}))}function l1(a,b,c,d){const e=ZY(a);if(0===e.length)return oj({});const f={};return C(Xj(e),Yj(g=>C(m1(a[g],b,c,d),ro(),So(h=>{f[g]=h}))),so(),wo(f),Pn(g=>u0(g)?kj:pj(g)))}
function m1(a,b,c,d){let e;const f=null!=(e=e0(b))?e:d,g=o0(a,f);a=g.resolve?g.resolve(b,c):Cg(f,()=>g(b,c));return aZ(a)};function n1(a){return Lo(b=>{const c=a(b);return c?C(Xj(c),Cj(()=>b)):oj(b)})};function o1(a){let b;for(a=a.root;void 0!==a;){let c;b=null!=(c=a.data[WY])?c:b;a=a.children.find(d=>"primary"===d.mb)}return b}var p1=class{};p1.K=function(a){return new (a||p1)};p1.na=Pd({ma:p1,ba:()=>of(q1),fa:"root"});var q1=class extends p1{constructor(a){super();this.title=a}};q1.K=function(a){return new (a||q1)(r(vH))};q1.na=Pd({ma:q1,ba:q1.K,fa:"root"});var r1=new gf("",{fa:"root",ba:()=>({})});var s1=new gf(""),v1=class{constructor(){this.D=new WeakMap;this.C=new WeakMap;of(qs)}Fl(a){if(this.D.get(a))return this.D.get(a);if(a.Ak)return oj(a.Ak);this.F&&this.F(a);var b=C(aZ(a.Fl()),Cj(t1),So(c=>{this.G&&this.G(a);a.Ak=c}),po(()=>{this.D.delete(a)}));b=C(new zj(b,()=>new hj),xj());this.D.set(a,b);return b}co(a,b){if(this.C.get(b))return this.C.get(b);if(b.Im)return oj({Sj:b.Im,xa:b.Bk});this.F&&this.F(b);a=C(u1(b,a,this.G),po(()=>{this.C.delete(b)}));a=C(new zj(a,()=>new hj),xj());this.C.set(b,
a);return a}};v1.K=function(a){return new (a||v1)};v1.na=Pd({ma:v1,ba:v1.K,fa:"root"});function u1(a,b,c){return C(aZ(a.co()),Cj(t1),Yj(d=>d instanceof ei||Array.isArray(d)?oj(d):Xj(Promise.resolve(new sn(d)))),Cj(d=>{c&&c(a);let e;Array.isArray(d)||(e=d.create(b).xa,d=e.get(s1,[],{optional:!0,self:!0}).flat());return{Sj:d.map(b0),xa:e}}))}function t1(a){return a&&"object"===typeof a&&"default"in a?a["default"]:a};var w1=class{};w1.K=function(a){return new (a||w1)};w1.na=Pd({ma:w1,ba:()=>of(x1),fa:"root"});var x1=class{merge(a){return a}};x1.K=function(a){return new (a||x1)};x1.na=Pd({ma:x1,ba:x1.K,fa:"root"});var y1=new gf("");new gf("");var z1=new gf("");function A1(a,b){const c=++a.Yh;let d;null==(d=a.ah)||d.next(Object.assign({},a.ah.value,b,{id:c}))}function B1(a,b,c){c=new f_(b.id,mZ(b.ed),"",c);a.ic.next(c);b.resolve(!1)}function C1(a){let b,c;return(null==(b=a.C)?void 0:b.ed.toString())!==(null==(c=a.C)?void 0:c.Tb.toString())}function D1(a){let b,c;return a.N.parse(a.location.path(!0)).toString()!==(null==(b=a.C)?void 0:b.ed.toString())&&!(null==(c=a.C)?0:c.extras.Xj)}
function E1(a,b,c,d){a.ah=new jj({id:0,Tb:c,Lv:c,ed:c,dc:c,Co:c,extras:{},resolve:()=>{},reject:()=>{},promise:Promise.resolve(!0),source:"imperative",Qj:null,Fh:d.kb,qd:null,wq:d,jm:null,tj:{canActivateChecks:[],canDeactivateChecks:[]},Qf:null});return C(a.ah,xk(e=>0!==e.id),Cj(e=>Object.assign({},e,{ed:e.Co})),Lo(e=>{let f=!1,g=!1;return C(oj(e),Lo(h=>{if(a.Yh>e.id)return B1(a,e,1),kj;a.C=e;a.D={id:h.id,hr:h.Co,ed:h.ed,Oe:h.source,extras:h.extras,OD:a.G?Object.assign({},a.G,{OD:null}):null};const m=
!b.jo||C1(a)||D1(a);let p;const u=null!=(p=h.extras.Mr)?p:b.Mr;return m||"reload"===u?C(oj(h),Lo(w=>{let B;const G=null==(B=a.ah)?void 0:ij(B);a.ic.next(new d_(w.id,mZ(w.ed),w.Qj));let L;return G!==(null==(L=a.ah)?void 0:ij(L))?kj:Promise.resolve(w)}),h1(a.Pf,a.F,a.J,b.config,a.N,a.Fj),So(w=>{e.qd=w.qd;e.dc=w.dc;a.D=Object.assign({},a.D,{jl:w.dc});w=new i_(w.id,mZ(w.ed),mZ(w.dc),w.qd);a.ic.next(w)})):(a.ic.next(new g_(h.id,mZ(h.Co))),h.resolve(!1),kj)}),So(h=>{h=new j_(h.id,mZ(h.ed),mZ(h.dc),h.qd);
a.ic.next(h)}),Cj(h=>{var m=Object,p=m.assign;var u=h.Fh;const w=h.qd.vb;u=p0(w,u?u.vb:null,a.O,[w.value]);return a.C=e=p.call(m,{},h,{tj:u})}),v0(a.Pf,h=>a.ic.next(h)),So(h=>{if((e.Qf=h.Qf)&&"boolean"!==typeof h.Qf)throw X_(h.Qf);h=new k_(h.id,mZ(h.ed),mZ(h.dc),h.qd,!!h.Qf);a.ic.next(h)}),xk(h=>h.Qf?!0:(B1(a,h,3),!1)),n1(h=>{if(h.tj.canActivateChecks.length)return C(oj(h),So(m=>{m=new l_(m.id,mZ(m.ed),mZ(m.dc),m.qd);a.ic.next(m)}),Lo(m=>{let p=!1;return C(oj(m),i1(a.Fj,a.Pf),So({next:()=>p=!0,complete:()=>
{p||B1(a,m,2)}}))}),So(m=>{m=new m_(m.id,mZ(m.ed),mZ(m.dc),m.qd);a.ic.next(m)}))}),n1(h=>{const m=p=>{const u=[];let w;(null==(w=p.jb)?0:w.Fl)&&!p.jb.Ak&&u.push(C(a.F.Fl(p.jb),So(B=>{p.component=B}),Cj(()=>{})));for(const B of p.children)u.push(...m(B));return u};return C(Uj(m(h.qd.root)),Xn(null),io())}),n1(()=>a.T()),Lo(()=>{const h=e.Fh,m=e.qd;let p;const u=null==(p=a.aa)?void 0:p.call(a,a.Pf,h.root,m.root);return u?C(Xj(u),Cj(()=>e)):oj(e)}),Cj(h=>{var m=h.qd;var p=h.wq;p=U_(b.rs,m.vb,p?p.vb:
void 0);m=new G_(p,m);a.C=e=Object.assign({},h,{jm:m});a.D.jm=m;return e}),So(()=>{a.ic.next(new t_)}),i0(a.O,b.rs,h=>a.ic.next(h)),io(),So({next:h=>{f=!0;a.G=a.D;a.ic.next(new e_(h.id,mZ(h.ed),mZ(h.dc)));let m;if(null!=(m=a.ja)){const p=o1(h.jm.kb);void 0!==p&&(m.title.C.title=p||"")}h.resolve(!0)},complete:()=>{f=!0}}),Oo(C(a.P,So(h=>{throw h;}))),po(()=>{f||g||B1(a,e,1);let h;(null==(h=a.C)?void 0:h.id)===e.id&&(a.D=null,a.C=null)}),Pn(h=>{g=!0;if(Z_(h))a.ic.next(new f_(e.id,mZ(e.ed),h.message,
h.pB)),Z_(h)&&NZ(h.url)?a.ic.next(new u_(h.url,h.Kl)):e.resolve(!1);else{let m;const p=new h_(e.id,mZ(e.ed),h,null!=(m=e.qd)?m:void 0);try{Cg(a.Pf,()=>{let w;return null==(w=a.ea)?void 0:w.call(a,p)});a.ic.next(p);const u=b.yn(h);e.resolve(!!u)}catch(u){a.options.DG?e.resolve(!1):e.reject(u)}}return kj}))}))}
var F1=class{get ca(){return 0!==this.Yh}constructor(){this.G=this.C=this.D=null;this.ic=new hj;this.P=new hj;this.F=of(v1);this.Pf=of(og);this.N=of(tZ);this.O=of(v_);this.location=of(Ft);of(S_,{optional:!0});this.ja=of(p1);this.options=of(r1,{optional:!0})||{};this.Fj=this.options.Fj||"emptyOnly";of(w1);this.aa=of(y1,{optional:!0});this.ea=of(z1,{optional:!0});this.Yh=0;this.T=()=>oj(void 0);this.J=null;this.F.G=a=>this.ic.next(new o_(a));this.F.F=a=>this.ic.next(new n_(a))}complete(){let a;null==
(a=this.ah)||a.complete()}};F1.K=function(a){return new (a||F1)};F1.na=Pd({ma:F1,ba:F1.K,fa:"root"});var G1=class{};G1.K=function(a){return new (a||G1)};G1.na=Pd({ma:G1,ba:()=>of(H1),fa:"root"});var I1=class{store(){}},H1=class extends I1{};H1.K=(()=>{let a;return function(b){return(a||(a=Lh(H1)))(b||H1)}})();H1.na=Pd({ma:H1,ba:H1.K,fa:"root"});var J1=class{};J1.K=function(a){return new (a||J1)};J1.na=Pd({ma:J1,ba:()=>of(K1),fa:"root"});function L1(a){return{Ne:a.Ne,Tb:a.Tb,Qd:a.Qd}}function M1(a,b){return a.location.subscribe(c=>{"popstate"===c.type&&b(c.url,c.state)})}function N1(a,b,c){b=mZ(b);var d=a.location;d.path()==d.normalize(b+ut(""))||c.extras.Zl?(c=Object.assign({},c.extras.state,O1(a,c.id,a.D)),Ct(a.location,b,c)):(c=Object.assign({},c.extras.state,O1(a,c.id,a.D+1)),Bt(a.location,b,c))}
function P1(a,b,c=!1){"computed"===a.Rk?(c=a.C-a.D,0!==c?Dt(a.location,c):a.Tb===b.jl&&0===c&&(Q1(a,b),R1(a))):"replace"===a.Rk&&(c&&Q1(a,b),R1(a))}function O1(a,b,c){return"computed"===a.Rk?{Yh:b,gz:c}:{Yh:b}}function Q1(a,b){a.Qd=a.G.Qd;a.Tb=a.G.Tb;let c;a.Ne=a.F.merge(a.Tb,null!=(c=b.jl)?c:a.Ne)}function R1(a){Ct(a.location,mZ(a.Ne),O1(a,a.J,a.C))}
var K1=class extends J1{constructor(){super(...arguments);this.location=of(Ft);of(tZ);this.options=of(r1,{optional:!0})||{};this.Rk=this.options.Rk||"replace";this.F=of(w1);this.ck=this.options.ck||"deferred";this.Ne=this.Tb=new nZ;this.C=0;this.J=-1;const a=new L_(new D_(new J_([],{},{},"",{},"primary",null,null,{}),[])),b=new H_(new jj([new qZ("",{})]),new jj({}),new jj({}),new jj(""),new jj({}),"primary",null,a.root);b.kb=a.root;this.Qd=new G_(new D_(b,[]),a);this.G=L1(this)}Qj(){return this.location.getState()}get D(){if("computed"!==
this.Rk)return this.C;let a,b;return null!=(b=null==(a=this.Qj())?void 0:a.gz)?b:this.C}};K1.K=(()=>{let a;return function(b){return(a||(a=Lh(K1)))(b||K1)}})();K1.na=Pd({ma:K1,ba:K1.K,fa:"root"});function S1(a,b){C(a.ic,xk(c=>c instanceof e_||c instanceof f_||c instanceof h_||c instanceof g_),Cj(c=>c instanceof e_||c instanceof g_?0:c instanceof f_&&(0===c.code||1===c.code)?2:1),xk(c=>2!==c),io()).subscribe(()=>{b()})};function T1(a){throw a;}var U1={paths:"exact",ac:"ignored",Tw:"ignored",tb:"exact"},V1={paths:"subset",ac:"ignored",Tw:"ignored",tb:"subset"};
function W1(a){const b=a.C.ic.subscribe(c=>{try{const e=a.C.C,f=a.C.D;if(null!==e&&null!==f){var d=a.D;if(c instanceof d_)d.G=L1(d);else if(c instanceof g_)d.Ne=f.hr;else if(c instanceof i_){if("eager"===d.ck&&!f.extras.Xj){const g=d.F.merge(f.jl,f.hr);N1(d,g,f)}}else c instanceof t_?(d.Tb=f.jl,d.Ne=d.F.merge(f.jl,f.hr),d.Qd=f.jm,"deferred"===d.ck&&(f.extras.Xj||N1(d,d.Ne,f))):c instanceof f_&&(3===c.code||2===c.code)?P1(d,f):c instanceof h_?P1(d,f,!0):c instanceof e_&&(d.J=c.id,d.C=d.D);if(c instanceof
f_&&0!==c.code&&1!==c.code)a.jo=!0;else if(c instanceof e_)a.jo=!0;else if(c instanceof u_){const g=c.Kl,h=a.J.merge(c.url,e.Lv),m=Object.assign({},{info:e.extras.info,Xj:e.extras.Xj,Zl:e.extras.Zl||"eager"===a.ck||"imperative"!==e.source},g);X1(a,h,"imperative",null,m,{resolve:e.resolve,reject:e.reject,promise:e.promise})}}c instanceof t_||c instanceof u_||a.Bb.next(c)}catch(e){a.C.P.next(e)}});a.P.add(b)}
function X1(a,b,c,d,e,f){if(a.O)return Promise.resolve(!1);let g,h;f?(g=f.resolve,h=f.reject,f=f.promise):f=new Promise((p,u)=>{g=p;h=u});const m=a.G.add();S1(a,()=>{queueMicrotask(()=>a.G.remove(m))});A1(a.C,{source:c,Qj:d,Tb:a.Tb,Lv:a.Tb,Co:b,extras:e,resolve:g,reject:h,promise:f,Fh:a.Qd.kb,wq:a.Qd});return f.catch(p=>Promise.reject(p))}function Y1(a){null!=a.F||(a.F=M1(a.D,(b,c)=>{setTimeout(()=>{Z1(a,b,"popstate",c)},0)}))}
function Z1(a,b,c,d){const e={Zl:!0},f=(null==d?0:d.Yh)?d:null;d&&(d=Object.assign({},d),delete d.Yh,delete d.gz,0!==Object.keys(d).length&&(e.state=d));b=$1(a,b);X1(a,b,c,f,e)}function $1(a,b){try{return a.N.parse(b)}catch(c){return a.N.parse("/")}}function a2(a){return Object.entries(a).reduce((b,[c,d])=>{null!==d&&void 0!==d&&(b[c]=d);return b},{})}
function b2(a,b,c={}){const d=c.BG,e=c.tb;var f=c.ac;f=c.yG?a.Tb.ac:f;switch(c.Dx){case "merge":c=Object.assign({},a.Tb.tb,e);break;case "preserve":c=a.Tb.tb;break;default:c=e||null}null!==c&&(c=a2(c));let g;try{g=PZ(d?d.kb:a.Qd.kb.root)}catch(h){if("string"!==typeof b[0]||"/"!==b[0][0])b=[];g=a.Tb.root}return QZ(g,b,c,null!=f?f:null)}function c2(a,b,c={Xj:!1}){b=NZ(b)?b:$1(a,b);b=a.J.merge(b,a.Ne);return X1(a,b,"imperative",null,c)}
var d2=class{get Tb(){return this.D.Tb}get Ne(){return this.D.Ne}get ic(){return this.Bb}get Qd(){return this.D.Qd}constructor(){this.O=!1;this.console=of(zn);this.D=of(J1);this.options=of(r1,{optional:!0})||{};this.G=of(An);this.ck=this.options.ck||"deferred";this.C=of(F1);this.N=of(tZ);this.location=of(Ft);this.J=of(w1);this.Bb=new hj;this.yn=this.options.yn||T1;this.jo=!1;this.rs=of(G1);this.Mr=this.options.Mr||"ignore";let a,b;this.config=null!=(b=null==(a=of(s1,{optional:!0}))?void 0:a.flat())?
b:[];of(S_,{optional:!0});this.P=new Si;this.config=this.config.map(b0);this.jo=!1;E1(this.C,this,this.Tb,this.Qd).subscribe({error:c=>{this.console.warn(c)}});W1(this)}get url(){return mZ(this.Tb)}va(){this.dispose()}dispose(){this.C.complete();this.F&&(this.F.unsubscribe(),this.F=void 0);this.O=!0;this.P.unsubscribe()}navigate(a,b={Xj:!1}){for(let c=0;c<a.length;c++)if(null==a[c])throw new ae(4008,!1);return c2(this,b2(this,a,b),b)}isActive(a,b){let c;!0===b?c=Object.assign({},U1):!1===b?c=Object.assign({},
V1):c=b;if(NZ(a))return hZ(this.Tb,a,c);a=$1(this,a);return hZ(this.Tb,a,c)}};d2.K=function(a){return new (a||d2)};d2.na=Pd({ma:d2,ba:d2.K,fa:"root"});var e2=new gf("");function f2(a){return a.Qd.root}new gf("",{fa:"root",ba:()=>!1});function g2(){const a=of(ai);return b=>{var c=a.get(ip);if(b===c.Lf[0]){b=a.get(d2);var d=a.get(h2);1===a.get(i2)&&(Y1(b),b.C.ca||Z1(b,b.location.path(!0),"imperative",b.D.Qj()));var e;null==(e=a.get(j2,null,8))||e.IG();var f;null==(f=a.get(e2,null,8))||f.init();c=c.T[0];b.Qd.root.component=c;b.C.J=c;d.closed||(d.next(),d.complete(),d.unsubscribe())}}}const h2=new gf("",{ba:()=>new hj}),i2=new gf("",{fa:"root",ba:()=>1}),j2=new gf("");var k2=new gf("ROUTER_FORROOT_GUARD"),l2=class{};l2.K=function(a){return new (a||l2)(r(k2,8))};l2.Ya=Sf({type:l2});l2.Xa=Qd({});new gf("");function m2(a){const b={};a.Ac?b.benchmark="1":(b.data=JSON.stringify({models:a.models,nodeData:a.nodeData,uiState:a.uiState}),b.renderer=a.C,b.show_open_in_new_tab=a.Jo?"1":"0",b.enable_subgraph_selection=a.enableSubgraphSelection?"1":"0");const c=JSON.stringify(b);c!==a.D&&(a.D=c,a.F.navigate([],{tb:b,Dx:"",Zl:!1}))}function n2(a,b){a.models=b;m2(a)}function o2(a,b){a.uiState=b;m2(a)}
var p2=class{constructor(a){this.F=a;this.models=[];this.nodeData=[];this.D="";this.C="webgl";this.enableSubgraphSelection=this.Ac=this.Jo=!1;a=new URLSearchParams(document.location.search);var b=a.get("data");if(b){let c=b=JSON.parse(b);null!=b.modelUrl&&(c={models:[{url:b.modelUrl}],uiState:{paneStates:[{deepestExpandedGroupNodeIds:b.deepestExpandedGroupNodeIds||[],selectedNodeId:b.selectedNodeId||"",selectedGraphId:b.selectedGraphId||"",selectedCollectionLabel:"",widthFraction:1}]}});this.models=
c.models;this.uiState=c.uiState;this.nodeData=c.nodeData}this.C=a.get("renderer")||"webgl";this.Jo="1"===a.get("show_open_in_new_tab");this.enableSubgraphSelection="1"===a.get("enable_subgraph_selection");this.Ac="1"===a.get("benchmark")}};p2.K=function(a){return new (a||p2)(r(d2))};p2.na=Pd({ma:p2,ba:p2.K,fa:"root"});const q2=["modelPathInput"];
function r2(a){a&1&&(R(0,"div",19),Z(1," Select models from your comupter or enter the file paths directly (up to 10 in total)."),Oq(2,"br"),Z(3," Supported model formats: "),R(4,"a",32),Z(5,"TF"),U(),Z(6," (.pb, .pbtxt, .graphdef), "),R(7,"a",33),Z(8,"TFLite"),U(),Z(9," (.tflite), "),R(10,"a",34),Z(11,"TFJS"),U(),Z(12," (.json), "),R(13,"a",35),Z(14,"JAX"),U(),Z(15," (.pb), "),R(16,"a",36),Z(17,"PyTorch ExportedProgram"),U(),Z(18," (.pt2), "),R(19,"span",37),Z(20," MLIR "),U(),Z(21," (.mlir, .mlirbc). "),
U());if(a&2){a=Y();const b=Ar(37);D(19);Q("xapInlineDialog",b)("overlaySize",a.Xh)("hoverDelayMs",50)}}
function s2(a){a&1&&(R(0,"div",19),Z(1," Enter model CNS/"),R(2,"span",38),Z(3,"TFHub"),U(),Z(4," paths/urls or select models from your computer (up to 10). Supported model format: "),R(5,"span",39),Z(6," TF "),U(),R(7,"span",40),Z(8," TFLite "),U(),R(9,"span",41),Z(10," TFJS "),U(),R(11,"span",42),Z(12," JAX "),U(),R(13,"span",43),Z(14," MLIR "),U(),R(15,"span",44),Z(16," JSON graphs "),U()());if(a&2){a=Y();const b=Ar(27),c=Ar(29),d=Ar(31),e=Ar(33),f=Ar(35),g=Ar(39);D(5);Q("xapInlineDialog",b)("overlaySize",
a.Xh)("hoverDelayMs",50);D(2);Q("xapInlineDialog",c)("overlaySize",a.Xh)("hoverDelayMs",50);D(2);Q("xapInlineDialog",d)("overlaySize",a.Xh)("hoverDelayMs",50);D(2);Q("xapInlineDialog",e)("overlaySize",a.Xh)("hoverDelayMs",50);D(2);Q("xapInlineDialog",f)("overlaySize",a.Xh)("hoverDelayMs",50);D(2);Q("xapInlineDialog",g)("overlaySize",a.Xh)("hoverDelayMs",50)}}
function t2(a,b){if(a&1){const c=Zg();R(0,"mat-option",23)(1,"div",45);Z(2);R(3,"div",46)(4,"div",47);V("click",function(d){const e=t(c).Cb,f=Y();d.stopPropagation();f.vg.setValue(f.zj[e].path);f.Vw.R.focus();return x()});R(5,"mat-icon");Z(6,"edit");U()();R(7,"div",47);V("click",function(d){const e=t(c).Cb,f=Y();d.stopPropagation();f.zj.splice(e,1);u2(f);PI("model_explorer_model_paths",JSON.stringify(f.zj));return x()});R(8,"mat-icon");Z(9,"delete");U()()()()()}a&2&&(a=b.da,Q("value",a),D(2),Cr(" ",
a," "))}function v2(a){a&1&&(R(0,"div",29),Z(1," You have selected maximum number of models "),U())}function w2(a){a&1&&Oq(0,"mat-spinner",57)}function x2(a){a&1&&(R(0,"div",63),Z(1),U());a&2&&(a=Y(2).da,D(),Cr(" ",a.errorMessage," "))}function y2(a){a&1&&(R(0,"div",62)(1,"mat-icon"),Z(2,"info"),U()(),H(3,x2,2,1,"ng-template",null,13,Rr));if(a&2){a=Ar(4);const b=Y(3);Q("xapInlineDialog",a)("overlaySize",b.aC)("hoverDelayMs",50)}}
function z2(a){if(a&1){const b=Zg();R(0,"div",64,14);V("click",function(){t(b);const c=Ar(1),d=Y().da,e=Y(2);return x(A2(e,d,c))});R(2,"div",65);Z(3);U();R(4,"mat-icon");Z(5,"arrow_drop_down");U()()}if(a&2){a=Y().da;const b=Y(2);Wp("disabled",b.hd());D(3);Cr(" ",B2(a)," ")}}function C2(a){a&1&&(R(0,"div",60),Z(1," No supported adapter "),U())}
function D2(a,b){if(a&1){const c=Zg();R(0,"tr")(1,"td")(2,"div",53)(3,"mat-checkbox",54,12);V("change",function(d){const e=t(c).da;Y(2);e.selected=d.checked;return x()});U();R(5,"div",55);V("click",function(){const d=t(c).da,e=Ar(4);Y(2);d.selected=!e.checked;return x()});Z(6);U()()();R(7,"td")(8,"div",56);H(9,w2,1,0,"mat-spinner",57);R(10,"div",58);Z(11);U();H(12,y2,5,3);U()();R(13,"td");H(14,z2,6,3,"div",59)(15,C2,2,0,"div",60);U();R(16,"td")(17,"button",61);V("click",function(){const d=t(c).index,
e=Y(2);e.od.splice(d,1);en(e.kk);return x()});R(18,"mat-icon");Z(19,"delete");U()()()()}a&2&&(a=b.da,b=Y(2),D(2),Wp("disabled",!(0<(a.Hf||[]).length)),D(),Q("checked",a.selected)("disabled",!(0<(a.Hf||[]).length)||b.hd()),D(2),Wp("disabled",!(0<(a.Hf||[]).length)||b.hd()),D(),Cr(" ",a.path," "),D(3),Gq("Converting"===a.status()||"Uploading"===a.status()?9:-1),D(),Wp("done",a.status()===b.sz.DONE)("error",b.Fe(a)),D(),Cr(" ",E2(a)," "),D(),Gq(b.Fe(a)?12:-1),D(2),Gq(0<(a.Hf||[]).length?14:15),D(3),
Q("disabled",b.hd()))}function F2(a){a&1&&(R(0,"div",30)(1,"table")(2,"thead")(3,"tr")(4,"th",48),Z(5,"Model name"),U(),R(6,"th"),Z(7,"Status"),U(),R(8,"th")(9,"div",49),Z(10," Adapter "),R(11,"div",50)(12,"mat-icon",51),Z(13,"help_outline"),U()()()(),R(14,"th"),Z(15,"Delete"),U()()(),R(16,"tbody"),H(17,D2,20,16,"tr",52),U()()());if(a&2){a=Y();const b=Ar(43);D(11);Q("xapInlineDialog",b)("overlayPositions",a.iB)("hoverDelayMs",100);D(6);Q("ngForOf",a.od)("ngForTrackBy",a.ME)}}
function G2(a){if(a&1){const b=Zg();R(0,"div",31)(1,"div",66)(2,"button",67);V("click",function(){t(b);const c=Y();for(const d of c.od)d.selected=!1;return x()});R(3,"mat-icon");Z(4,"close");U()();Z(5);U();R(6,"button",68);V("click",function(){t(b);const c=Y();return x(H2(c))});Z(7," View selected models ");U()()}a&2&&(a=Y(),D(),Wp("hide",0===a.zs),D(),Q("disabled",a.hd()),D(3),Cr(" ",a.zs," selected "),D(),Q("disabled",0===a.zs||a.hd()))}
function I2(a){a&1&&(R(0,"div",69)(1,"div"),Z(2," Enter the "),R(3,"b"),Z(4,"CNS path"),U(),Z(5,", "),R(6,"b"),Z(7,"Placer path"),U(),Z(8,", "),R(9,"b")(10,"a",70),Z(11,"Piper path"),U()(),Z(12,", "),R(13,"b"),Z(14,"TFHub path"),U(),Z(15,", or "),R(16,"b"),Z(17,"CNS Viewer url"),U(),Z(18," to a TF model "),R(19,"b"),Z(20,"directory"),U(),Z(21,", or upload the "),R(22,"b"),Z(23,".pb"),U(),Z(24," file from your computer. "),Oq(25,"br")(26,"br"),R(27,"span",71),Z(28,"When using CNS/Placer/TFHub, the model file inside is required to be named "),
R(29,"b"),Z(30,"saved_model.pb"),U(),Z(31,"."),U(),Oq(32,"br")(33,"br"),U(),R(34,"div",72),Z(35," Examples: "),R(36,"ul")(37,"li"),Z(38,"/cns/ok-d/home/odml-tooling-team/mgv/models/coco-ssd-tf/"),U(),R(39,"li"),Z(40,"/tfhub/prod/tfhub.dev/tensorflow/tutorials/spam-detection/1/"),U(),R(41,"li"),Z(42,"google3/path/to/tf/model/on/piper/"),U()()()())}
function J2(a){a&1&&(R(0,"div",69)(1,"div"),Z(2," Enter the "),R(3,"b"),Z(4,"CNS path"),U(),Z(5,", "),R(6,"b"),Z(7,"Placer path"),U(),Z(8,", "),R(9,"b")(10,"a",70),Z(11,"Piper path"),U()(),Z(12,", "),R(13,"b"),Z(14,"TFHub path"),U(),Z(15,", or "),R(16,"b"),Z(17,"CNS Viewer url"),U(),Z(18," to a TFLite model "),R(19,"b"),Z(20,"file"),U(),Z(21,", or upload the "),R(22,"b"),Z(23,".tflite"),U(),Z(24," file from your computer. "),U(),R(25,"div",72),Z(26," Examples: "),R(27,"ul")(28,"li"),Z(29,"/cns/ok-d/home/odml-tooling-team/mgv/models/coco-ssd.tflite"),
U(),R(30,"li"),Z(31,"/tfhub/prod/tfhub.dev/tensorflow/lite-model/tutorials/spam-detection/tflite/1/1.tflite"),U(),R(32,"li"),Z(33,"google3/path/to/tflite/model/on/piper.tflite"),U()()()())}function K2(a){a&1&&(R(0,"div",69)(1,"div"),Z(2," Enter the "),R(3,"b"),Z(4,"url"),U(),Z(5," for the model.json file, or upload it from your computer. "),U(),R(6,"div",72),Z(7," Example: https://tfhub.dev/tensorflow/tfjs-model/ssd_mobilenet_v2/1/default/1/model.json?tfjs-format=file "),U()())}
function L2(a){a&1&&(R(0,"div",69)(1,"div"),Z(2," JAX models must be convereted to TF, TFLite, or TFJS prior to loading in Model Explorer. Check out the "),R(3,"a",73),Z(4,"JAX to TF"),U(),Z(5," or "),R(6,"a",74),Z(7,"JAX to TFLite/TFJS"),U(),Z(8," documentation for conversion instructions. "),U()())}
function M2(a){a&1&&(R(0,"div",69)(1,"div"),Z(2," Enter the "),R(3,"b"),Z(4,"CNS path"),U(),Z(5,", "),R(6,"b"),Z(7,"Placer path"),U(),Z(8,", "),R(9,"b")(10,"a",70),Z(11,"Piper path"),U()(),Z(12,", "),R(13,"b"),Z(14,"TFHub path"),U(),Z(15,", or "),R(16,"b"),Z(17,"CNS Viewer url"),U(),Z(18," to a MLIR model "),R(19,"b"),Z(20,"file"),U(),Z(21,", or upload the "),R(22,"b"),Z(23,".mlir"),U(),Z(24," or "),R(25,"b"),Z(26,".mlirbc"),U(),Z(27," file from your computer. Only support TF, TFL and StableHLO dialect. "),
U(),R(28,"div",72),Z(29," Examples: "),R(30,"ul")(31,"li"),Z(32,"/cns/sandbox/home/yijieyang/mlir/tfl_ssd_debug_info.mlir"),U()()()())}function N2(a){a&1&&(R(0,"div",69)(1,"div"),Z(2," Support TF, TFL and StableHLO dialect. "),U()())}
function O2(a){a&1&&(R(0,"div",69)(1,"div"),Z(2," Enter the "),R(3,"b"),Z(4,"url"),U(),Z(5," for the JSON graphs file (downloadable from "),R(6,"a",75),Z(7,"here"),U(),Z(8,", or convert your graphs into "),R(9,"a",76),Z(10,"this format"),U(),Z(11,"), or upload it from your computer. Using JSON graphs file has a much faster processing time. "),U(),R(12,"div",72),Z(13," Example: https://storage.googleapis.com/tfweb/model-explorer-models/mobilebert_fp32.tflite.json "),U()())}
function P2(a){a&1&&(R(0,"div",77),Z(1," Max upload size: 256 MB. "),Oq(2,"br")(3,"br"),Z(4," Model too large? Try "),R(5,"a",78),Z(6,"converting"),U(),Z(7," it to JSON graphs and upload the JSON file instead. "),U())}
function Q2(a){a&1&&(R(0,"div",79)(1,"span",80),Z(2,"Adapters"),U(),Z(3,' transform model files into an intermediate format that Model Explorer can understand and visualize. For certain model types, multiple adapters may exist, offering different transformation approaches. The "default" adapter should cover the majority of common scenarios. '),Oq(4,"br")(5,"br"),Z(6," You can also create your own "),R(7,"span",80),Z(8,"adapter extensions"),U(),Z(9," to add support for more model formats. See "),R(10,
"a",81),Z(11,"this guide"),U(),Z(12," for instructions. "),U())}function A2(a,b,c){let d;const e=null==(d=b.Sc)?void 0:d.id,f=R2(a,c);c=f.attach(a.Od);c.instance.Sc=b.Sc;c.instance.lq=b.Hf||[];c.instance.ne.subscribe(g=>{f.dispose();(null==g?void 0:g.id)!==e&&null!=b.errorMessage&&(b.status.set("Not started"),b.errorMessage=void 0,b.selected=!0);b.Sc=g;en(a.kk)})}function B2(a){if(null==a.Sc)return"?";let b;return 0===(a.Hf||[]).indexOf(a.Sc)?"Default":(null==(b=a.Sc)?void 0:b.name)||"?"}
function E2(a){a=a.status();return"Uploading"===a?"Processing":a}function H2(a){a.hd.set(!0);const b=a.od.filter(c=>c.selected);S2(a.J,b).then(()=>{a.hd.set(!1)});o2(a.F);n2(a.F,b.filter(c=>"remote"===c.type||"graphs_json_from_server"===c.type||"file_path"===c.type).map(c=>{let d;return{url:c.path,adapterId:null==(d=c.Sc)?void 0:d.id}}));a.vj.set(b.some(c=>"local"===c.type))}
function u2(a){const b=(a.vg.value||"").toLowerCase();a.ew=a.zj.filter(c=>c.path.toLowerCase().includes(b)).map(c=>c.path);en(a.kk)}function T2(){return JSON.parse(window.localStorage.getItem("model_explorer_model_paths")||"[]")}
function U2(a,b){a.od=b.map(c=>{if(c.url.startsWith("graphs://")){var d=TY(a.C,"builtin_json"),e=null==d?[]:[d];return{path:c.url,type:"graphs_json_from_server",status:yr("Not started"),selected:0<e.length,Hf:e,Sc:d}}d=OY(c.url,a.C);(e=TY(a.C,c.converterId||c.adapterId||""))||(e=0<d.length?d[0]:void 0);return{path:c.url,type:"file_path",status:yr("Not started"),selected:0<d.length,Hf:d,Sc:e}});fn(a.kk);a.od.some(c=>c.selected)&&H2(a)}
function V2(a,b){b=b.filter(c=>null==a.od.find(d=>{let e,f,g,h;return d.path===c.path&&(null==(e=d.file)?void 0:e.size)===(null==(f=c.file)?void 0:f.size)&&(null==(g=d.file)?void 0:g.lastModified)===(null==(h=c.file)?void 0:h.lastModified)}));a.od.push(...b);10<a.od.length&&a.od.splice(10);en(a.kk)}
function W2(a,b){const c=T2();for(const d of b)b=c.findIndex(e=>e.path===d),0<=b?(b=c.splice(b,1))&&0<b.length&&(b[0].ts=Date.now(),c.unshift(b[0])):c.unshift({path:d,ts:Date.now()});50<c.length&&c.splice(50);PI("model_explorer_model_paths",JSON.stringify(c));a.zj=c}
function X2(a){return ta(function*(){yield new Promise(c=>{setTimeout(c)});var b=a.vg.value;null!=b&&(b=b.trim().split(",").filter(c=>""!==c.trim()).map(c=>{const d=OY(c,a.C);return{path:c,type:a.ke?"remote":"file_path",status:yr("Not started"),selected:0<d.length,Hf:d,Sc:0<d.length?d[0]:void 0}}),V2(a,b),W2(a,b.map(c=>c.path)),a.vg.setValue(""),setTimeout(()=>{let c;null==(c=a.aD)||iB(c);a.Vw.R.blur()}))})}
function Y2(a,b){const c=[];for(let d=0;d<b.length;d++){const e=b[d],f=OY(e.name,a.C);c.push({path:e.name,type:"local",status:yr("Not started"),selected:0<f.length,file:e,Hf:f,Sc:0<f.length?f[0]:void 0})}V2(a,c)}
function R2(a,b){b=new Bu({vc:Lu(Sv(a.D.position(),b),[{bb:"start",cb:"bottom",Na:"start",Ua:"top"}]),Vb:!0,fc:"cdk-overlay-transparent-backdrop",wc:a.D.C.C(),maxWidth:"380px",hb:"graph-selector-panel"});const c=a.D.create(b);a.Od=new Vv(NY,a.yb);c.bd().subscribe(()=>{c.dispose()});return c}
var Z2=class{constructor(a,b,c,d,e,f,g,h){this.kk=a;this.C=b;this.G=c;this.J=e;this.D=f;this.F=g;this.yb=h;this.vg=new wx("");this.od=[];this.zj=[];this.ew=[];this.sz=JY;this.Xh={maxWidth:400,minHeight:0};this.aC={minHeight:0};this.iB=[{bb:"start",cb:"top",Na:"start",Ua:"bottom",offsetY:-4}];this.hd=yr(!1);this.vj=yr(!1);this.Od=null;C(this.vg.dh,tL(this.G)).subscribe(()=>{u2(this)});this.zj=T2();u2(this)}Tq(a){const b=a.files;if(b){var c=[];for(let d=0;d<b.length;d++){const e=b.item(d);e&&c.push(e)}Y2(this,
c);a.value=""}}ME(a,b){let c,d;return`${b.path}_${null==(c=b.file)?void 0:c.size}_${null==(d=b.file)?void 0:d.lastModified}`}Fe(a){return"Error"===a.status()}get Aq(){if(this.ar)return!0;if(this.ke){var a=(this.vg.value||"").toLowerCase().trim();return""!==a&&!FY.test(a)&&!a.startsWith("http")||""===a}a=(this.vg.value||"").trim();return""===a||""!==a&&!a.startsWith("/")&&!a.startsWith("~")}get Bd(){return!0}get ke(){return!1}get zs(){return this.od.filter(a=>a.selected).length}get ar(){return 10===
this.od.length}get eD(){return this.Bd?"Absolute file paths (recommended for large models)":"Model CNS/TFHub paths or urls"}};Z2.K=function(a){return new (a||Z2)(y(Tr),y(VY),y(Ci),y(QI),y("ModelLoaderService"),y(cw),y(p2),y(np))};
Z2.Ca=Jf({type:Z2,ha:[["model-source-input"]],eb:function(a,b){a&1&&(vr(q2,5),vr(dB,5));if(a&2){let c;wr(c=xr())&&(b.Vw=c.first);wr(c=xr())&&(b.aD=c.first)}},ka:!0,features:[Nr],ya:44,za:17,Ba:[["auto","matAutocomplete"],["modelPathInput",""],["uploadInput",""],["tfHelp",""],["tfliteHelp",""],["tfjsHelp",""],["jaxHelp",""],["mlirHelp",""],["externalMlirHelp",""],["graphsJsonHelp",""],["uploadHelp",""],["adapterHelp",""],["checkbox",""],["errorInfo",""],["selector",""],[1,"container"],[1,"content"],
[1,"title"],[1,"description-container"],[1,"description"],[1,"source-input-container"],[1,"input-container"],[1,"model-path-autocomplete",3,"optionSelected"],[3,"value"],[3,"keydown.enter","placeholder","formControl","matAutocomplete"],["mat-raised-button","",1,"btn-add-model-path",3,"click","disabled"],[1,"or-label"],["mat-flat-button","","dialogLabel","uploadHelp",1,"btn-upload",3,"click","disabled","xapInlineDialog","xapInlineDialogDisabled","hoverDelayMs"],["type","file","multiple","",1,"upload-input",
3,"change"],[1,"model-count-warning"],[1,"models-table"],[1,"table-action-bar"],["href","https://www.tensorflow.org/","target","_blank"],["href","https://www.tensorflow.org/lite","target","_blank"],["href","https://www.tensorflow.org/js","target","_blank"],["href","https://www.tensorflow.org/guide/jax2tf","target","_blank"],["href","https://github.com/google-ai-edge/model-explorer/wiki/2.-User-Guide#how-to-select-pytorch-models","target","_blank"],["dialogLabel","externalMlirHelp",1,"help-trigger",
"external",3,"xapInlineDialog","overlaySize","hoverDelayMs"],["matTooltip","In the form of /tfhub/...",1,"tfhub"],["dialogLabel","tfHelp",1,"help-trigger",3,"xapInlineDialog","overlaySize","hoverDelayMs"],["dialogLabel","tfliteHelp",1,"help-trigger",3,"xapInlineDialog","overlaySize","hoverDelayMs"],["dialogLabel","tfjsHelp",1,"help-trigger",3,"xapInlineDialog","overlaySize","hoverDelayMs"],["dialogLabel","jaxHelp",1,"help-trigger",3,"xapInlineDialog","overlaySize","hoverDelayMs"],["dialogLabel","mlirHelp",
1,"help-trigger",3,"xapInlineDialog","overlaySize","hoverDelayMs"],["dialogLabel","graphsJsonHelp",1,"help-trigger",3,"xapInlineDialog","overlaySize","hoverDelayMs"],[1,"option-container"],[1,"icons-container"],[1,"mat-icon-container",3,"click"],[1,"model-name-col"],[1,"adapter-header"],["dialogLabel","adapterHelp",1,"icon-container",3,"xapInlineDialog","overlayPositions","hoverDelayMs"],["fontSet","google-material-icons"],[4,"ngFor","ngForOf","ngForTrackBy"],[1,"model-name-container"],[3,"change",
"checked","disabled"],[1,"model-path",3,"click"],[1,"model-status-container"],["color","primary","diameter","16"],[1,"model-status-label-container"],[1,"selected-adapter-container",3,"disabled"],[1,"no-supported-adapter-msg"],["mat-icon-button","",1,"btn-delete-model-item",3,"click","disabled"],["dialogLabel","errorInfo",1,"error-info-container",3,"xapInlineDialog","overlaySize","hoverDelayMs"],[1,"model-explorer-error-info-popup"],[1,"selected-adapter-container",3,"click"],[1,"selected-adapter-label"],
[1,"selection-status"],["mat-icon-button","",1,"btn-deselect-all",3,"click","disabled"],["mat-flat-button","","color","primary",1,"btn-view-models",3,"click","disabled"],[1,"model-explorer-model-format-help-popup"],["href","http://go/google-src","target","_blank"],[1,"info"],[1,"example"],["href","https://g3doc.corp.google.com/third_party/py/jax/experimental/jax2tf/README.md","target","_blank"],["href","https://g3doc.corp.google.com/third_party/tensorflow/google/g3doc/jax_production/g3doc/guides/mobile.md",
"target","_blank"],["href","https://screenshot.googleplex.com/8u9evbbgNDiNmJX","target","_blank"],["href","https://source.corp.google.com/piper///depot/google3/learning/brain/mobile/lite/tooling/model_graph_visualizer/module/common/input_graph.ts?q=symbol:%5CbGraphCollection%5Cb","target","_blank"],[1,"model-explorer-upload-help-popup"],["href","https://source.corp.google.com/piper///depot/google3/third_party/tensorflow/compiler/mlir/lite/experimental/google/tooling/","target","_blank"],[1,"model-explorer-adapter-help-popup"],
[1,"bold"],["href","https://github.com/google-ai-edge/model-explorer/wiki/6.-Develop-Adapter-Extension","target","_blank"]],sa:function(a,b){if(a&1){const c=Zg();R(0,"div",15)(1,"div",16)(2,"div",17);Z(3," Select Models ");U();R(4,"div",18);H(5,r2,22,3,"div",19)(6,s2,17,18,"div",19);U();R(7,"div",20)(8,"div",21)(9,"mat-autocomplete",22,0);V("optionSelected",function(d){t(c);b.Aq||(Uz(d.gx),X2(b));return x()});Lq(11,t2,10,2,"mat-option",23,Jq);U();R(13,"input",24,1);V("keydown.enter",function(){t(c);
return x(!b.Aq&&X2(b))});U();R(15,"button",25);V("click",function(){t(c);return x(X2(b))});Z(16," Add ");U()();R(17,"div",26);Z(18,"or");U();R(19,"button",27);V("click",function(){t(c);const d=Ar(22);return x(d.click())});Z(20," Select from your computer ");U();R(21,"input",28,2);V("change",function(){t(c);const d=Ar(22);return x(b.Tq(d))});U()();H(23,v2,2,0,"div",29)(24,F2,18,5,"div",30);U();H(25,G2,8,5,"div",31);U();H(26,I2,43,0,"ng-template",null,3,Rr)(28,J2,34,0,"ng-template",null,4,Rr)(30,K2,
8,0,"ng-template",null,5,Rr)(32,L2,9,0,"ng-template",null,6,Rr)(34,M2,33,0,"ng-template",null,7,Rr)(36,N2,3,0,"ng-template",null,8,Rr)(38,O2,14,0,"ng-template",null,9,Rr)(40,P2,8,0,"ng-template",null,10,Rr)(42,Q2,13,0,"ng-template",null,11,Rr)}if(a&2){a=Ar(10);const c=Ar(41);D();Wp("no-models",0===b.od.length);D(4);Gq(b.Bd?5:-1);D();Gq(b.ke?6:-1);D();Wp("reverse",b.Bd);D(4);Nq(b.ew);D(2);Q("placeholder",b.eD)("formControl",b.vg)("matAutocomplete",a);D(2);Q("disabled",b.Aq||b.hd());D(4);Q("disabled",
b.ar||b.hd())("xapInlineDialog",c)("xapInlineDialogDisabled",b.Bd)("hoverDelayMs",50);D(4);Gq(b.ar?23:-1);D();Gq(0<b.od.length?24:-1);D();Gq(0<b.od.length?25:-1)}},Ha:[it,$s,pB,nw,mA,Vz,dB,BB,AB,vB,yB,OB,DB,IB,AC,wD,vD,sD,RD,PE,AE,NE,iI,hI,Dx,tw,Nw,yx,HG,EG],styles:[".container[_ngcontent-%COMP%]{width:100%;height:100%;-moz-box-sizing:border-box;box-sizing:border-box;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:start;-webkit-align-items:flex-start;-moz-box-align:start;-ms-flex-align:start;align-items:flex-start;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;border:1px solid #dadce0;border-radius:8px;overflow:hidden}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]{width:100%;-moz-box-sizing:border-box;box-sizing:border-box;padding:24px;padding-bottom:0;overflow:hidden;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column}.container[_ngcontent-%COMP%]   .content.no-models[_ngcontent-%COMP%]{padding-bottom:24px}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]{color:#1f1f1f;font-size:24px}.container[_ngcontent-%COMP%]   .description-container[_ngcontent-%COMP%]{color:#444746;font-size:14px;margin-top:8px}.container[_ngcontent-%COMP%]   .description-container[_ngcontent-%COMP%]   .description[_ngcontent-%COMP%]{line-height:20px}.container[_ngcontent-%COMP%]   .description-container[_ngcontent-%COMP%]   .description[_ngcontent-%COMP%]   .tfhub[_ngcontent-%COMP%]{display:inline-block;text-decoration:underline;-webkit-text-decoration-color:#aaa;-moz-text-decoration-color:#aaa;text-decoration-color:#aaa;cursor:help}.container[_ngcontent-%COMP%]   .description-container[_ngcontent-%COMP%]   .description[_ngcontent-%COMP%]   .help-trigger[_ngcontent-%COMP%]{display:inline-block;margin-right:6px;text-decoration:underline;-webkit-text-decoration-color:#aaa;-moz-text-decoration-color:#aaa;text-decoration-color:#aaa;cursor:help}.container[_ngcontent-%COMP%]   .description-container[_ngcontent-%COMP%]   .description[_ngcontent-%COMP%]   .help-trigger.external[_ngcontent-%COMP%]{margin-right:0}.container[_ngcontent-%COMP%]   .description-container[_ngcontent-%COMP%]   .description[_ngcontent-%COMP%]   a[_ngcontent-%COMP%]{color:inherit}.container[_ngcontent-%COMP%]   .source-input-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;height:56px;-moz-box-sizing:border-box;box-sizing:border-box;width:100%;margin-top:8px;padding:8px 0;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-flex-shrink:0;-ms-flex-negative:0;flex-shrink:0}.container[_ngcontent-%COMP%]   .source-input-container.reverse[_ngcontent-%COMP%]{-webkit-box-orient:horizontal;-webkit-box-direction:reverse;-webkit-flex-direction:row-reverse;-moz-box-orient:horizontal;-moz-box-direction:reverse;-ms-flex-direction:row-reverse;flex-direction:row-reverse}.container[_ngcontent-%COMP%]   .source-input-container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]{height:100%;-moz-box-sizing:border-box;box-sizing:border-box;background-color:#f0f4f9;border-radius:99px;-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;padding-left:18px}.container[_ngcontent-%COMP%]   .source-input-container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]   input[_ngcontent-%COMP%]{height:100%;-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;border:none;outline:none;background-color:transparent;color:#444746;font-size:14px;margin-right:16px}.container[_ngcontent-%COMP%]   .source-input-container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]   .btn-add-model-path[_ngcontent-%COMP%]{background-color:#fff;border-radius:99px;color:#0b57d0;height:40px}.container[_ngcontent-%COMP%]   .source-input-container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]   .btn-add-model-path[disabled][_ngcontent-%COMP%]{background-color:rgba(31,31,31,.12);color:rgba(31,31,31,.38)}.container[_ngcontent-%COMP%]   .source-input-container[_ngcontent-%COMP%]   .or-label[_ngcontent-%COMP%]{font-size:14px;margin:0 20px;color:#5f6368}.container[_ngcontent-%COMP%]   .source-input-container[_ngcontent-%COMP%]   .btn-upload[_ngcontent-%COMP%]{height:100%;background-color:#c2e7ff;border-radius:99px;color:#001d35}.container[_ngcontent-%COMP%]   .source-input-container[_ngcontent-%COMP%]   .btn-upload[disabled][_ngcontent-%COMP%]{background-color:rgba(31,31,31,.12);color:rgba(31,31,31,.38)}.container[_ngcontent-%COMP%]   .source-input-container[_ngcontent-%COMP%]   .upload-input[_ngcontent-%COMP%]{display:none}.container[_ngcontent-%COMP%]   .model-count-warning[_ngcontent-%COMP%]{color:#ab6c17;background-color:#ffefd9;margin:4px 0;width:-webkit-fit-content;width:-moz-fit-content;width:fit-content;padding:2px 9px;border-radius:4px;font-size:13px;font-weight:500}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]{width:100%;color:#1f1f1f;font-size:14px;margin-top:18px;overflow:hidden auto}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]{width:100%;border-spacing:0;border-collapse:collapse;padding:0}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]{position:-webkit-sticky;position:sticky;top:0;z-index:100;box-shadow:inset 0 -1px 0 #c4c7c5;background-color:#fff}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   .model-name-col[_ngcontent-%COMP%]{width:100%}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   .adapter-header[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;gap:4px}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   .adapter-header[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;cursor:pointer;opacity:.8}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   .adapter-header[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]:hover{opacity:1}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   .adapter-header[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:20px;width:20px;height:20px;color:#777}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   th[_ngcontent-%COMP%]{height:49px;text-align:left;padding:0;font-weight:500}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   tr[_ngcontent-%COMP%]{height:49px;text-align:left;padding:0}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   tr[_ngcontent-%COMP%]:not(:last-child){border-bottom:1px solid #c4c7c5}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   td[_ngcontent-%COMP%]{padding:0}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .model-name-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;padding-left:12px;margin-right:12px;word-break:break-all}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .model-name-container.disabled[_ngcontent-%COMP%]{pointer-events:none}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .model-name-container[_ngcontent-%COMP%]   mat-checkbox[_ngcontent-%COMP%]{margin-right:2px}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .model-name-container[_ngcontent-%COMP%]   .model-path[_ngcontent-%COMP%]{cursor:pointer;line-height:16px;padding:8px 0}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .model-name-container[_ngcontent-%COMP%]   .model-path.disabled[_ngcontent-%COMP%]{pointer-events:none}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .selected-adapter-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;cursor:pointer;height:48px;margin-right:24px;white-space:nowrap}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .selected-adapter-container.disabled[_ngcontent-%COMP%]{opacity:.5;pointer-events:none}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .selected-adapter-container[_ngcontent-%COMP%]:hover   mat-icon[_ngcontent-%COMP%]{opacity:1}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .selected-adapter-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{opacity:.7}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .no-supported-adapter-msg[_ngcontent-%COMP%]{color:#ab6c17;background-color:#ffefd9;padding:2px 8px;width:-webkit-fit-content;width:-moz-fit-content;width:fit-content;border-radius:4px;font-size:12px;margin-right:24px;white-space:nowrap;font-weight:500}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .model-status-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;margin-right:32px}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .model-status-container[_ngcontent-%COMP%]   mat-spinner[_ngcontent-%COMP%]{margin-right:4px}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .model-status-container[_ngcontent-%COMP%]   .model-status-label-container[_ngcontent-%COMP%]{font-size:12px;padding:2px 8px;background-color:#f1f3f4;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;color:#3c4043;width:-webkit-fit-content;width:-moz-fit-content;width:fit-content;border-radius:4px;white-space:nowrap;font-weight:500}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .model-status-container[_ngcontent-%COMP%]   .model-status-label-container.done[_ngcontent-%COMP%]{color:#0d652d;background-color:#e6f4ea}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .model-status-container[_ngcontent-%COMP%]   .model-status-label-container.error[_ngcontent-%COMP%]{color:#a50e0e;background-color:#fce8e6}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .model-status-container[_ngcontent-%COMP%]   .error-info-container[_ngcontent-%COMP%]{width:20px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;margin-left:4px;cursor:pointer}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .model-status-container[_ngcontent-%COMP%]   .error-info-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{width:20px;height:20px;font-size:20px}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .btn-delete-model-item[_ngcontent-%COMP%]{width:36px;height:36px;padding:0;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .btn-delete-model-item[disabled][_ngcontent-%COMP%]{opacity:.5}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .btn-delete-model-item[_ngcontent-%COMP%]     mat-icon{font-size:20px;width:20px;height:20px;color:#444746}.container[_ngcontent-%COMP%]   .table-action-bar[_ngcontent-%COMP%]{-webkit-flex-shrink:0;-ms-flex-negative:0;flex-shrink:0;height:60px;width:100%;-moz-box-sizing:border-box;box-sizing:border-box;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;background-color:#f0f4f9;padding:0 24px 0 38px}.container[_ngcontent-%COMP%]   .table-action-bar[_ngcontent-%COMP%]   .selection-status[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;font-size:14px;color:#444746}.container[_ngcontent-%COMP%]   .table-action-bar[_ngcontent-%COMP%]   .selection-status.hide[_ngcontent-%COMP%]{visibility:hidden}.container[_ngcontent-%COMP%]   .table-action-bar[_ngcontent-%COMP%]   .btn-deselect-all[_ngcontent-%COMP%]{width:36px;height:36px;padding:0;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center}.container[_ngcontent-%COMP%]   .table-action-bar[_ngcontent-%COMP%]   .btn-deselect-all[_ngcontent-%COMP%]     mat-icon{font-size:20px;width:20px;height:20px;color:#444746}.container[_ngcontent-%COMP%]   .table-action-bar[_ngcontent-%COMP%]   .btn-view-models[_ngcontent-%COMP%]{height:40px;border-radius:99px}.container[_ngcontent-%COMP%]   .instructions-container[_ngcontent-%COMP%]{font-size:13px;color:#555;margin-top:24px;display:none}.container[_ngcontent-%COMP%]   .instructions-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:18px;width:18px;height:18px;margin-right:4px}.container[_ngcontent-%COMP%]   .instructions-container[_ngcontent-%COMP%]   .instruction-title[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .instructions-container[_ngcontent-%COMP%]   .instruction-title[_ngcontent-%COMP%]   .model-format[_ngcontent-%COMP%]{color:#333;margin-right:4px;margin-left:4px;border:1px solid #ccc;border-radius:4px;padding:2px 4px;cursor:pointer}.container[_ngcontent-%COMP%]   .instructions-container[_ngcontent-%COMP%]   .instruction-title[_ngcontent-%COMP%]   .model-format[_ngcontent-%COMP%]:hover{background-color:#eee}  .model-explorer-upload-help-popup{padding:12px;font-weight:400;color:#555;line-height:14px;font-size:12px;font-family:Google Sans Text,Arial,Helvetica,sans-serif}  .model-explorer-adapter-help-popup{padding:12px;font-weight:400;color:#555;line-height:15px;font-size:12px;font-family:Google Sans Text,Arial,Helvetica,sans-serif}  .model-explorer-adapter-help-popup .bold{font-weight:700}  .model-explorer-model-format-help-popup{padding:12px;font-weight:400;color:#555;line-height:15px;font-size:12px;font-family:Google Sans Text,Arial,Helvetica,sans-serif;letter-spacing:normal}  .model-explorer-model-format-help-popup a{color:inherit}  .model-explorer-model-format-help-popup .example{margin-top:8px;color:#aaa}  .model-explorer-model-format-help-popup .example ul{margin:4px 0;-webkit-padding-start:28px;-moz-padding-start:28px;padding-inline-start:28px}  .model-explorer-model-format-help-popup .example li{margin-top:4px}  .model-explorer-model-format-help-popup .info{display:inline-block;background-color:#ffd1a8}  .model-explorer-error-info-popup{padding:12px;font-weight:400;background-color:#f0f4f9;line-height:14px;font-size:12px;font-family:Google Sans Text,Arial,Helvetica,sans-serif}  .model-path-autocomplete mat-option{min-height:13px;font-size:13px;line-height:13px;padding-top:5px;padding-bottom:5px}  .model-path-autocomplete mat-option:hover .option-container .mat-icon-container{visibility:visible}  .model-path-autocomplete mat-option>span{width:100%}  .model-path-autocomplete mat-option .option-container{width:100%;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;word-break:break-all}  .model-path-autocomplete mat-option .option-container .icons-container{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;gap:4px;min-width:50px;-webkit-box-pack:end;-webkit-justify-content:flex-end;-moz-box-pack:end;-ms-flex-pack:end;justify-content:flex-end}  .model-path-autocomplete mat-option .option-container .mat-icon-container{visibility:hidden;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;opacity:.6;width:18px}  .model-path-autocomplete mat-option .option-container .mat-icon-container:hover{opacity:1}  .model-path-autocomplete mat-option .option-container .mat-icon-container mat-icon{font-size:18px;width:18px;height:18px}"],
Pa:0});const $2=(a,b)=>b.name;function a3(a){if(a&1){const b=Zg();R(0,"div",11);V("click",function(){t(b);const c=Y().da;return x(c.expanded=!c.expanded)});R(1,"mat-icon");Z(2,"unfold_more");U()()}}function b3(a,b){a&1&&(R(0,"div",3)(1,"div",6),Z(2),U(),R(3,"div",7),Z(4),U(),R(5,"div",8)(6,"div",9),Z(7),U(),H(8,a3,3,0,"div",10),U()());a&2&&(a=b.da,D(2),Br(a.name),D(2),Br(a.copyright),D(),Wp("expanded",a.expanded),D(2),Br(a.ao),D(),Gq(a.expanded?-1:8))}
var c3=class{constructor(){this.XC=[{name:"Angular",copyright:"Copyright (c) 2010-2024 Google LLC.",ao:'The MIT License\n\nCopyright (c) 2010-2024 Google LLC. https://angular.io/license\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the "Software"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in\nall copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN\nTHE SOFTWARE.'},
{name:"d3 (v5.7.0)",copyright:"Copyright 2010-2017 Mike Bostock",ao:'Copyright 2010-2017 Mike Bostock\nAll rights reserved.\n\nRedistribution and use in source and binary forms, with or without modification,\nare permitted provided that the following conditions are met:\n\n* Redistributions of source code must retain the above copyright notice, this\n  list of conditions and the following disclaimer.\n\n* Redistributions in binary form must reproduce the above copyright notice,\n  this list of conditions and the following disclaimer in the documentation\n  and/or other materials provided with the distribution.\n\n* Neither the name of the author nor the names of contributors may be used to\n  endorse or promote products derived from this software without specific prior\n  written permission.\n\nTHIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND\nANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED\nWARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE\nDISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR\nANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES\n(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;\nLOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON\nANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT\n(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS\nSOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.'},
{name:"dagre (v0.8.5)",copyright:"Copyright (c) 2012-2014 Chris Pettitt",ao:'Copyright (c) 2012-2014 Chris Pettitt\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the "Software"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in\nall copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN\nTHE SOFTWARE.'},
{name:"three.js (r134)",copyright:"Copyright \u00a9 2010-2021 three.js authors",ao:'The MIT License\n\nCopyright \u00a9 2010-2021 three.js authors\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the "Software"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in\nall copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN\nTHE SOFTWARE.'}]}};
c3.K=function(a){return new (a||c3)};
c3.Ca=Jf({type:c3,ha:[["open-source-libs-dialog"]],ka:!0,features:[Nr],ya:9,za:0,Ba:[["mat-dialog-title",""],[1,"title"],["mat-dialog-content","",1,"dialog-content"],[1,"lib-container"],["mat-dialog-actions","","align","end"],["mat-button","","mat-dialog-close",""],[1,"name"],[1,"copyright"],[1,"license-container"],[1,"license-content"],[1,"icon-container"],[1,"icon-container",3,"click"]],sa:function(a,b){a&1&&(R(0,"div",0)(1,"div",1),Z(2,"Open source libraries"),U()(),R(3,"div",2),Lq(4,b3,9,6,"div",
3,$2),U(),R(6,"div",4)(7,"button",5),Z(8,"Close"),U()());a&2&&(D(4),Nq(b.XC))},Ha:[it,BB,AB,vB,zC,sC,uC,wC,vC,iI,hI],styles:["[_ngcontent-%COMP%]:not(mat-icon){font-family:Google Sans Text,Arial,Helvetica,sans-serif!important;letter-spacing:normal!important}.title[_ngcontent-%COMP%]{font-size:24px;font-weight:500;margin-top:-16px}.dialog-content[_ngcontent-%COMP%]{padding-bottom:0}.lib-container[_ngcontent-%COMP%]{margin:18px 0}.lib-container[_ngcontent-%COMP%]   .name[_ngcontent-%COMP%]{font-size:16px;font-weight:700;color:#333}.lib-container[_ngcontent-%COMP%]   .copyright[_ngcontent-%COMP%]{font-size:12px;color:#777}.lib-container[_ngcontent-%COMP%]   .license-container[_ngcontent-%COMP%]{width:520px;font-size:11px;border:1px solid #ccc;margin-top:8px;background-color:#f9f9f9;height:100px;overflow:hidden;position:relative}.lib-container[_ngcontent-%COMP%]   .license-container.expanded[_ngcontent-%COMP%]{height:-webkit-fit-content;height:-moz-fit-content;height:fit-content}.lib-container[_ngcontent-%COMP%]   .license-container[_ngcontent-%COMP%]   .license-content[_ngcontent-%COMP%]{white-space:pre-wrap;line-height:14px;padding:8px;overflow:hidden;height:100%;-moz-box-sizing:border-box;box-sizing:border-box}.lib-container[_ngcontent-%COMP%]   .license-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{position:absolute;right:8px;top:8px;cursor:pointer;opacity:.8}.lib-container[_ngcontent-%COMP%]   .license-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]:hover{opacity:1}"]});var d3={iz:0,ep:1,Ez:2,COLOR:3,0:"BOOLEAN",1:"NUMBER",2:"TEXT_MULTILINE",3:"COLOR"},e3={label:"Maximum element count for constant tensor values",key:"const_element_count_limit",type:1,defaultValue:16,help:"Controls the number of values extracted from the constant tensors during model processing. Increasing this number may impact performance due to larger payload sizes."},f3={label:"Hide op nodes with labels below (comma separated)",key:"hide_op_nodes_with_labels",type:2,defaultValue:"Const,pseudo_const,pseudo_qconst,ReadVariableOp",
help:"Removes op nodes from model graphs if their label matches any of the labels entered below."},g3={label:"Maximum number of nodes in an artificial layer",key:"artificial_layer_node_count_threshold",type:1,defaultValue:1E3,help:"Controls the maximum number of immediate child nodes displayed under a layer. When the number of child nodes exceeds this limit, Model Explorer automatically groups them into smaller, more manageable artificial layers to improve layout performance and readability."},h3=
{label:"Edge label font size",key:"edge_label_font_size",type:1,defaultValue:7.5},i3={label:"Edge color",key:"edge_color",type:3,defaultValue:"#aaaaaa"},j3={label:"Disallow vertical edge labels",key:"disallow_vertical_edge_labels",type:0,defaultValue:!1,help:"-"},k3={label:"Keep layers with a single op node child",key:"keep_layers_with_a_single_child",type:0,defaultValue:!1,help:"By default, layers with a single op node as its child are automatically removed to improve graph readability. Turn this toggle on to keep those layers."},
l3=[e3,f3,g3,h3,i3,k3,{label:"Show welcome card",key:"show_welcome_card",type:0,defaultValue:!0},j3];function m3(a,b){return null==a.C[b.key]?!0===b.defaultValue:!0===a.C[b.key]}function n3(a,b){a=a.C[b.key];return null!=a?Number(a):b.defaultValue||0}function o3(a,b,c){a.C[c]=b;window.localStorage.setItem("model_explorer_settings",JSON.stringify(a.C))}function p3(a,b,c){isNaN(b)||(a.C[c]=b,window.localStorage.setItem("model_explorer_settings",JSON.stringify(a.C)))}
function q3(a,b,c){a.C[c]=b;window.localStorage.setItem("model_explorer_settings",JSON.stringify(a.C))}function r3(){return l3.find(a=>"show_welcome_card"===a.key)}var s3=class{constructor(){const a=window.localStorage.getItem("model_explorer_settings")||"";this.C=""===a?{}:JSON.parse(a)}getStringValue(a){const b=this.C[a.key];return null!=b?b:a.defaultValue||""}};s3.K=function(a){return new (a||s3)};s3.na=Pd({ma:s3,ba:s3.K,fa:"root"});function t3(a){a&1&&(Z(0," When "),R(1,"a",19),Z(2,"View tensor shape on edge"),U(),Z(3," is turned on, Model Explorer will try to position the shape along the edge curve. When the edge is close to vertical, the shape may be hard to read. Enabling this option will force the shapes to be rendered horizontally in such situations to improve readbility. "))}function u3(a){a&1&&Z(0);a&2&&(a=Y(3).da,Cr(" ",a.help," "))}
function v3(a){a&1&&(R(0,"div",18),H(1,t3,4,0)(2,u3,1,1),U());a&2&&(a=Y(2).da,D(),Gq("disallow_vertical_edge_labels"===a.key?1:2))}function w3(a){a&1&&(R(0,"div",16)(1,"mat-icon",17),Z(2,"help_outline"),U()(),H(3,v3,3,1,"ng-template",null,0,Rr));if(a&2){a=Ar(4);const b=Y(2);Q("xapInlineDialog",a)("overlaySize",b.fd)("overlayPositions",b.tC)("hoverDelayMs",100)}}
function x3(a){if(a&1){const b=Zg();Pq(0);R(1,"mat-slide-toggle",20);V("change",function(c){t(b);const d=Y().da,e=Y();return x(o3(e.Mb,c.checked,d.key))});U();Qq()}if(a&2){a=Y().da;const b=Y();D();Q("checked",m3(b.Mb,a))}}
function y3(a){if(a&1){const b=Zg();Pq(0);R(1,"input",21,1);V("focus",function(){t(b);const c=Ar(2);return x(c.select())})("change",function(){t(b);const c=Ar(2),d=Y().da,e=Y();return x(p3(e.Mb,c.valueAsNumber,d.key))});U();R(3,"div",22);V("click",function(){t(b);const c=Y().da,d=Y();p3(d.Mb,c.defaultValue,c.key);return x()});Z(4,"Reset to default");U();Qq()}if(a&2){a=Y().da;const b=Y();D();Q("value",n3(b.Mb,a))}}
function z3(a){if(a&1){const b=Zg();Pq(0);R(1,"textarea",21,1);V("focus",function(){t(b);const c=Ar(2);return x(c.select())})("change",function(){t(b);const c=Ar(2),d=Y().da,e=Y();return x(q3(e.Mb,c.value,d.key))});U();R(3,"div",22);V("click",function(){t(b);const c=Y().da,d=Y();q3(d.Mb,c.defaultValue,c.key);return x()});Z(4,"Reset to default");U();Qq()}if(a&2){a=Y().da;const b=Y();D();Q("value",b.Mb.getStringValue(a))}}
function A3(a){if(a&1){const b=Zg();Pq(0);R(1,"div",23)(2,"label")(3,"input",24,2);V("change",function(){t(b);const c=Ar(4),d=Y().da,e=Y();return x(q3(e.Mb,c.value,d.key))});U()();R(5,"input",25,3);V("change",function(){t(b);const c=Ar(4),d=Y().da,e=Y();return x(q3(e.Mb,c.value,d.key))});U()();R(7,"div",22);V("click",function(){t(b);const c=Y().da,d=Y();q3(d.Mb,c.defaultValue,c.key);return x()});Z(8,"Reset to default");U();Qq()}if(a&2){a=Y().da;const b=Y();D(2);Up("background-color",b.Mb.getStringValue(a));
D();Q("value",b.Mb.getStringValue(a));D(2);Q("value",b.Mb.getStringValue(a))}}function B3(a,b){a&1&&(R(0,"div",12),Pq(1,13),R(2,"div",14),Z(3),H(4,w3,5,4),U(),H(5,x3,2,1,"ng-container",15)(6,y3,5,1,"ng-container",15)(7,z3,5,1,"ng-container",15)(8,A3,9,4,"ng-container",15),Qq(),U());a&2&&(a=b.da,b=Y(),D(),Q("ngSwitch",a.type),D(2),Cr(" ",a.label," "),D(),Gq(a.help?4:-1),D(),Q("ngSwitchCase",b.tm.iz),D(),Q("ngSwitchCase",b.tm.ep),D(),Q("ngSwitchCase",b.tm.Ez),D(),Q("ngSwitchCase",b.tm.COLOR))}
var C3=class{constructor(a){this.Mb=a;this.tm=d3;this.fd={minWidth:0,minHeight:0,maxWidth:340};this.tC=[{bb:"end",cb:"top",Na:"start",Ua:"top",offsetX:4}];this.kB=l3}};C3.K=function(a){return new (a||C3)(y(s3))};
C3.Ca=Jf({type:C3,ha:[["settings-dialog"]],ka:!0,features:[Nr],ya:11,za:1,Ba:[["helpPopup",""],["input",""],["colorPicker",""],["colorInput",""],["mat-dialog-title","",1,"title-container"],[1,"title"],[1,"local-storage-info"],["mat-dialog-content","",1,"dialog-content"],[1,"settings-container"],["class","setting-item",4,"ngFor","ngForOf"],["mat-dialog-actions","","align","end",1,"buttons-container"],["mat-button","","mat-dialog-close",""],[1,"setting-item"],[3,"ngSwitch"],[1,"setting-label"],[4,"ngSwitchCase"],
["dialogLabel","help popup",1,"icon-container",3,"xapInlineDialog","overlaySize","overlayPositions","hoverDelayMs"],["fontSet","google-material-icons"],[1,"model-explorer-settings-help-popup"],["href","https://github.com/google-ai-edge/model-explorer/wiki/2.-User-Guide#overlay-data-on-nodesedges","target","_blank"],[3,"change","checked"],["type","number",3,"focus","change","value"],[1,"reset",3,"click"],[1,"color-picker-container"],["type","color",3,"change","value"],["type","text",3,"change","value"]],
sa:function(a,b){a&1&&(R(0,"div",4)(1,"div",5),Z(2,"Advanced settings"),U(),R(3,"div",6),Z(4," Settings are stored in current browser's local storage "),U()(),R(5,"div",7)(6,"div",8),H(7,B3,9,7,"div",9),U()(),R(8,"div",10)(9,"button",11),Z(10,"Close"),U()());a&2&&(D(7),Q("ngForOf",b.kB))},Ha:[it,$s,ft,gt,BB,AB,vB,zC,sC,uC,wC,vC,bE,TD,XD,iI,hI,HG,EG],styles:["[_ngcontent-%COMP%]:not(mat-icon){font-family:Google Sans Text,Arial,Helvetica,sans-serif}.title-container[_ngcontent-%COMP%]{border-bottom:1px solid #ddd}.title[_ngcontent-%COMP%]{font-size:24px;font-weight:500;margin-top:-16px}.local-storage-info[_ngcontent-%COMP%]{font-size:12px;color:#777;margin-top:6px}.dialog-content[_ngcontent-%COMP%]{padding-bottom:0}.settings-container[_ngcontent-%COMP%]{font-size:13px;color:#000;margin-top:20px;margin-bottom:10px}.setting-item[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;-webkit-box-align:start;-webkit-align-items:flex-start;-moz-box-align:start;-ms-flex-align:start;align-items:flex-start;margin-bottom:24px}.setting-item[_ngcontent-%COMP%]   .setting-label[_ngcontent-%COMP%]{line-height:16px;margin-bottom:4px;width:100%;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between}.setting-item[_ngcontent-%COMP%]   .setting-label[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{width:16px;height:16px;margin-left:12px;opacity:.4;cursor:pointer}.setting-item[_ngcontent-%COMP%]   .setting-label[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]:hover{opacity:.6}.setting-item[_ngcontent-%COMP%]   .setting-label[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:16px;width:16px;height:16px}.setting-item[_ngcontent-%COMP%]   input[_ngcontent-%COMP%], .setting-item[_ngcontent-%COMP%]   textarea[_ngcontent-%COMP%]{-moz-box-sizing:border-box;box-sizing:border-box;width:100%}.setting-item[_ngcontent-%COMP%]   input[_ngcontent-%COMP%]::-webkit-inner-spin-button, .setting-item[_ngcontent-%COMP%]   input[_ngcontent-%COMP%]::-webkit-outer-spin-button{-webkit-appearance:none;margin:0}.setting-item[_ngcontent-%COMP%]   .reset[_ngcontent-%COMP%]{color:#999;cursor:pointer;margin-top:2px;text-decoration:underline;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}.setting-item[_ngcontent-%COMP%]   .reset[_ngcontent-%COMP%]:hover{color:#777}.color-picker-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;overflow:hidden;border-radius:2px;border:1px solid #767676;width:100%;background-color:#fff;-moz-box-sizing:border-box;box-sizing:border-box;height:24px;position:relative}.color-picker-container[_ngcontent-%COMP%]:focus-within{border-color:#1a73e8;outline:1px solid #1a73e8}.color-picker-container[_ngcontent-%COMP%]   label[_ngcontent-%COMP%]{display:inline-block;width:16px;height:16px;margin:2px;background-color:red;border-radius:4px;cursor:pointer;border:1px solid #ddd}.color-picker-container[_ngcontent-%COMP%]   label[_ngcontent-%COMP%]   input[_ngcontent-%COMP%]{visibility:hidden}.color-picker-container[_ngcontent-%COMP%]   input[type=text][_ngcontent-%COMP%]{border:none;height:100%;background-color:transparent;outline:none}.buttons-container[_ngcontent-%COMP%]{border-top:1px solid #ddd}  .model-explorer-settings-help-popup{font-size:12px;padding:8px;background-color:#fff;line-height:normal}  .model-explorer-settings-help-popup .bold{font-weight:700}"]});function D3(a){a&1&&(Pq(0),R(1,"div",6)(2,"a",7)(3,"mat-icon"),Z(4,"open_in_new"),U(),R(5,"span",8),Z(6,"Learn more from GitHub"),U()()(),Qq())}function E3(a){a&1&&(Pq(0),R(1,"div",6)(2,"a",9)(3,"mat-icon"),Z(4,"open_in_new"),U(),R(5,"span",8),Z(6,"Learn more from g3doc"),U()()(),R(7,"div",6)(8,"a",10)(9,"mat-icon"),Z(10,"group"),U(),R(11,"span",8),Z(12,"Join chat space"),U()()(),Qq())}var F3=class{get Bd(){return!0}};F3.K=function(a){return new (a||F3)};
F3.Ca=Jf({type:F3,ha:[["welcome-card"]],ka:!0,features:[Nr],ya:9,za:2,Ba:[[1,"container"],[1,"left"],[1,"title"],[1,"description"],[1,"doc-links-container"],[4,"ngIf"],[1,"doc-link"],["href","https://github.com/google-ai-edge/model-explorer","target","_blank"],[1,"doc-text"],["href","http://go/model-explorer-doc","target","_blank"],["href","https://chat.google.com/room/AAAAxS-rXf0?cls=4","target","_blank"]],sa:function(a,b){a&1&&(R(0,"div",0)(1,"div",1)(2,"div",2),Z(3," Welcome to Model Explorer "),
U(),R(4,"div",3),Z(5," A modern model graph visualizer and debugger "),U(),R(6,"div",4),H(7,D3,7,0,"ng-container",5)(8,E3,13,0,"ng-container",5),U()()());a&2&&(D(7),Q("ngIf",b.Bd),D(),Q("ngIf",!b.Bd))},Ha:[it,dt,iI,hI],styles:[".container[_ngcontent-%COMP%]{width:100%;height:100%;-moz-box-sizing:border-box;box-sizing:border-box;border:1px solid #dadce0;border-radius:8px;padding:24px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]{color:#1f1f1f;font-size:24px}.container[_ngcontent-%COMP%]   .description[_ngcontent-%COMP%]{color:#3c4043;margin-top:8px;font-size:14px}.container[_ngcontent-%COMP%]   .doc-links-container[_ngcontent-%COMP%]{margin-top:24px}.container[_ngcontent-%COMP%]   .doc-link[_ngcontent-%COMP%]{font-size:14px;color:#0b57d0;cursor:pointer;opacity:.8;margin-bottom:8px}.container[_ngcontent-%COMP%]   .doc-link[_ngcontent-%COMP%]   .doc-text[_ngcontent-%COMP%]{color:#0b57d0}.container[_ngcontent-%COMP%]   .doc-link[_ngcontent-%COMP%]   a[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;text-decoration:none}.container[_ngcontent-%COMP%]   .doc-link[_ngcontent-%COMP%]   [_ngcontent-%COMP%]:hover{opacity:1}.container[_ngcontent-%COMP%]   .doc-link[_ngcontent-%COMP%]   [_ngcontent-%COMP%]:hover   .doc-text[_ngcontent-%COMP%]{text-decoration:underline}.container[_ngcontent-%COMP%]   .doc-link[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:18px;width:18px;height:18px;margin-right:8px;color:#0b57d0}.container[_ngcontent-%COMP%]   .video-container[_ngcontent-%COMP%]{width:430px;height:100%;border-radius:8px;background-color:#f1f1f1;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;color:#999;font-size:13px}"]});var G3=class{};G3.K=function(a){return new (a||G3)};G3.na=Pd({ma:G3,ba:G3.K,fa:"root"});const H3=["modelSourceInput"],I3=["modelGraphVisualizer"],Pr=()=>[];function J3(a){a&1&&(R(0,"a",8)(1,"button",20)(2,"mat-icon"),Z(3,"article"),U()()())}function K3(a){a&1&&(R(0,"a",21)(1,"button",22)(2,"mat-icon"),Z(3,"article"),U()()(),R(4,"a",23)(5,"button",24)(6,"mat-icon"),Z(7,"bug_report"),U()()())}function L3(a){a&1&&(R(0,"a",9)(1,"button",25)(2,"mat-icon"),Z(3,"alternate_email"),U()()())}
function M3(a){if(a&1){const b=Zg();R(0,"button",26)(1,"mat-icon");Z(2,"info_outline");U()();R(3,"mat-menu",null,0)(5,"button",27);V("click",function(){t(b);Y().Ih.open(c3,{});return x()});Z(6," Open source libraries ");U();R(7,"div",28);Z(8);U()();Oq(9,"open-in-new-tab-button")}if(a&2){a=Ar(4);const b=Y();Q("matMenuTriggerFor",a);D(8);Cr("v",b.runningVersion(),"")}}
function N3(a){if(a&1){const b=Zg();R(0,"div",11);Oq(1,"welcome-card");R(2,"div",29);V("click",function(){t(b);const c=Y();o3(c.Mb,!1,"show_welcome_card");return x()});R(3,"mat-icon");Z(4,"close");U()()()}}function O3(a){a&1&&(R(0,"div",12),Oq(1,"mat-spinner",30),U())}function P3(a){a&1&&(R(0,"div",13),Oq(1,"model-source-input",null,1),U())}function Q3(a){a&1&&(R(0,"a",15)(1,"mat-icon"),Z(2,"bug_report"),U()())}function R3(a){a&1&&H(0,Q3,3,0,"a",31);a&2&&(a=Y(),Q("ngIf",null==a.le()&&!a.Ac))}
function S3(a){a&1&&(R(0,"div",16),Z(1,"The "),R(2,"a",32),Z(3," Google Terms of Service "),U(),Z(4," apply to your use of this offering "),U())}function T3(a){if(a&1){const b=Zg();R(0,"div",34);Oq(1,"div",35);R(2,"div",36)(3,"button",37);V("click",function(){t(b);const c=Y(2);return x(U3(c))});R(4,"mat-icon");Z(5,"share");U()()()()}a&2&&(a=Y(2),D(2),Q("matTooltip",a.Xx()),D(),Q("disabled",a.vj()))}
function V3(a){if(a&1){const b=Zg();R(0,"model-graph-visualizer",33,2);V("titleClicked",function(){t(b);const c=Y();return x(W3(c))})("modelGraphProcessed",function(c){t(b);const d=Y();if(d.Og&&0<d.Og.length&&!d.Gx){let e;null==(e=d.dD)||yY(e,d.Og,c.modelGraph);d.Gx=!0}return x()})("uiStateChanged",function(c){t(b);const d=Y();o2(d.yi,c);return x()})("remoteNodeDataPathsChanged",function(c){t(b);var d=Y().yi;d.nodeData=c;m2(d);return x()});H(2,T3,6,2,"div",34);U()}a&2&&(a=Y(),Q("graphCollections",
a.Ac?Or():a.le())("benchmark",a.Ac)("config",a.EB)("initialUiState",a.ie)("nodeDataSources",a.Og),D(2),Gq(a.ke?2:-1))}
function U3(a){null!=window.gtag&&window.gtag("event","share_clicked",Object.assign({},void 0));const b=window.location.href,c=new URLSearchParams;c.append("url",b);c.append("contains_no_pii","true");fetch("https://shortn.corp.google.com/",{mode:"cors",credentials:"include",method:"POST",headers:{Accept:"application/json","Content-Type":"application/x-www-form-urlencoded"},body:c.toString()}).then(d=>{d.json().then(e=>{navigator.clipboard.writeText(e.url);a.G.open("Sharable URL copied to clipboard",
"Close",{duration:5E3})})}).catch(d=>{console.error(d);navigator.clipboard.writeText(b);a.G.open("Sharable URL copied to clipboard","Close",{duration:5E3})})}function W3(a,b=!1){b?a.F.navigate(["/"]).then(()=>{window.location.reload()}):window.history.back()}function X3(a){a.le.set(void 0);a.ie=void 0;setTimeout(()=>{a.F.navigate([],{tb:{},Dx:"",Zl:!0})})}
var Y3=class{constructor(a,b,c,d,e,f,g,h,m,p,u){this.Ih=a;this.D=b;this.J=d;this.N=e;this.C=f;this.F=g;this.Mb=h;this.G=m;this.yi=u;this.Pw=this.D.hd;this.le=this.J.le;this.runningVersion=Cs(()=>this.N.info().runningVersion);this.Ac=this.el=!1;this.Og=[];this.vj=yr(!1);this.Xx=yr("");this.Gx=!1;this.ie=this.yi.uiState;Ks(()=>{this.D.hd()||setTimeout(()=>{const w=this.yi.models;null!=w&&0<w.length?U2(this.io,w):this.ie=void 0})});Ks(()=>{null!=this.le()&&window.history.pushState({ts:Date.now()},"")});
this.Ac="1"===(new URLSearchParams(document.location.search)).get("benchmark");this.Og=this.yi.nodeData||[]}pd(){this.io&&(this.vj=this.io.vj,this.Xx=Cs(()=>this.vj()?"Share is not available for uploaded models":"Share"))}Uq(a){null==this.le()&&(this.el=!0);a.preventDefault()}Vq(a){a.preventDefault();this.el=!1;const b=[];let c;if(null==(c=a.dataTransfer)?0:c.items)[...a.dataTransfer.items].forEach(d=>{"file"===d.kind&&(d=d.getAsFile())&&b.push(d)});else{let d;b.push(...((null==(d=a.dataTransfer)?
void 0:d.files)||[]))}Y2(this.io,b)}get Bd(){return!0}get ke(){return!1}get yE(){const a=r3();return a?m3(this.Mb,a):!0}get EB(){const a="1"===this.C.kb.tb.show_tflite_consts;return{nodeLabelsToHide:this.Mb.getStringValue(f3).split(",").map(b=>b.trim()).filter(b=>""!==b&&(!a||a&&"pseudo_const"!==b&&"pseudo_qconst"!==b)),artificialLayerNodeCountThreshold:n3(this.Mb,g3),edgeLabelFontSize:n3(this.Mb,h3),edgeColor:this.Mb.getStringValue(i3),maxConstValueCount:n3(this.Mb,e3),disallowVerticalEdgeLabels:m3(this.Mb,
j3),enableSubgraphSelection:this.yi.enableSubgraphSelection,keepLayersWithASingleChild:m3(this.Mb,k3)}}};Y3.K=function(a){return new (a||Y3)(y(pC),y(SY),y(G3),y("ModelLoaderService"),y(gT),y(H_),y(d2),y(s3),y(vE),y(KM),y(p2))};
Y3.Ca=Jf({type:Y3,ha:[["home-page"]],eb:function(a,b){a&1&&(vr(H3,5),vr(I3,5));if(a&2){let c;wr(c=xr())&&(b.io=c.first);wr(c=xr())&&(b.dD=c.first)}},Ia:function(a,b){a&1&&V("popstate",function(){return X3(b)},!1,Fi)},ka:!0,features:[Nr],ya:25,za:14,Ba:[["menu","matMenu"],["modelSourceInput",""],["modelGraphVisualizer",""],[1,"container",3,"dragover","dragleave","dragend","drop"],[1,"title"],[1,"name",3,"click"],[1,"icons-container"],["mat-icon-button","","aria-label","settings","matTooltip","Advanced settings",
3,"click"],["href","http://go/model-explorer-doc","target","_blank"],["href","https://moma.corp.google.com/team/1963115384013","target","_blank"],[1,"content"],[1,"welcome-card-container"],[1,"loading-adapter-extension-container"],[1,"model-source-input-container"],[1,"padding"],["href","https://b.corp.google.com/issues/new?component=1338731&template=1850894","target","_blank","matTooltip","File bug or feature request",1,"bug-report"],[1,"utos"],[3,"graphCollections","benchmark","config","initialUiState",
"nodeDataSources","titleClicked","modelGraphProcessed","uiStateChanged","remoteNodeDataPathsChanged",4,"ngIf"],[1,"dragover-overlay"],[1,"msg"],"mat-icon-button  aria-label doc matTooltip g3doc".split(" "),["href","https://github.com/google-ai-edge/model-explorer","target","_blank"],"mat-icon-button  aria-label doc matTooltip GitHub".split(" "),["href","https://github.com/google-ai-edge/model-explorer/issues","target","_blank"],"mat-icon-button;;aria-label;bug;matTooltip;File issue or feature request".split(";"),
"mat-icon-button;;aria-label;contact;matTooltip;Contact us".split(";"),["mat-icon-button","","aria-label","about","matTooltip","About",3,"matMenuTriggerFor"],["mat-menu-item","",3,"click"],[1,"model-explorer-version"],["matTooltip","Dismiss",1,"icon-container",3,"click"],["color","primary","diameter","24"],["href","https://b.corp.google.com/issues/new?component=1338731&template=1850894","class","bug-report","target","_blank","matTooltip","File bug or feature request",4,"ngIf"],["href","https://policies.google.com/terms?hl=en-US#toc-intro",
"target","_blank"],[3,"titleClicked","modelGraphProcessed","uiStateChanged","remoteNodeDataPathsChanged","graphCollections","benchmark","config","initialUiState","nodeDataSources"],[1,"btns-container"],[1,"divider"],[1,"share-button-container",3,"matTooltip"],["mat-icon-button","","aria-label","share",3,"click","disabled"]],sa:function(a,b){a&1&&(R(0,"div",3),V("dragover",function(c){return b.Uq(c)})("dragleave",function(){b.el=!1})("dragend",function(){b.el=!1})("drop",function(c){return b.Vq(c)}),
R(1,"div",4)(2,"div",5),V("click",function(){return W3(b,!0)}),Oq(3,"me-logo"),Z(4," Model Explorer "),Oq(5,"new-version-chip"),U(),R(6,"div",6)(7,"button",7),V("click",function(){b.Ih.open(C3,{})}),R(8,"mat-icon"),Z(9,"settings"),U()(),H(10,J3,4,0,"a",8)(11,K3,8,0)(12,L3,4,0,"a",9)(13,M3,10,2),U()(),R(14,"div",10),H(15,N3,5,0,"div",11)(16,O3,2,0,"div",12)(17,P3,3,0,"div",13),Oq(18,"div",14),U(),H(19,R3,1,1,"a",15)(20,S3,5,0,"div",16)(21,V3,3,7,"model-graph-visualizer",17),R(22,"div",18)(23,"div",
19),Z(24,"Drop to add model files"),U()()());a&2&&(Wp("dragover",b.el),D(),Wp("hide",null!=b.le()||b.Ac),D(9),Gq(b.ke?10:11),D(2),Gq(b.ke?12:-1),D(),Gq(b.Bd?13:-1),D(),Wp("hide",null!=b.le()||b.Ac),D(),Gq(b.yE?15:-1),D(),Gq(b.Pw()?16:17),D(3),Gq(b.ke?19:-1),D(),Gq(null==b.le()&&b.Bd&&!b.Ac?20:-1),D(),Q("ngIf",null!=b.le()||b.Ac))},Ha:[it,dt,PE,AE,NE,BB,AB,yB,$C,CC,QC,JC,YC,wD,vD,sD,yE,WJ,xC,iI,hI,zY,Z2,hT,kI,F3],styles:[".container[_ngcontent-%COMP%]{width:100%;height:100%;overflow:hidden;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;position:relative}.container.dragover[_ngcontent-%COMP%]   .dragover-overlay[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex}.container[_ngcontent-%COMP%]   .dragover-overlay[_ngcontent-%COMP%]{display:none;position:absolute;left:0;top:0;width:100%;height:100%;background-color:rgba(0,0,0,.1);pointer-events:none;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:end;-webkit-justify-content:flex-end;-moz-box-pack:end;-ms-flex-pack:end;justify-content:flex-end;z-index:500}.container[_ngcontent-%COMP%]   .dragover-overlay[_ngcontent-%COMP%]   .msg[_ngcontent-%COMP%]{margin-bottom:16px;color:#fff;padding:4px 12px;border-radius:99px;background-color:#4285f4}@-webkit-keyframes _ngcontent-%COMP%_rotate{0%{-webkit-transform:rotate(0deg);transform:rotate(0deg)}30%{-webkit-transform:rotate(180deg);transform:rotate(180deg)}60%{-webkit-transform:rotate(1turn);transform:rotate(1turn)}to{-webkit-transform:rotate(1turn);transform:rotate(1turn)}}@keyframes _ngcontent-%COMP%_rotate{0%{-webkit-transform:rotate(0deg);transform:rotate(0deg)}30%{-webkit-transform:rotate(180deg);transform:rotate(180deg)}60%{-webkit-transform:rotate(1turn);transform:rotate(1turn)}to{-webkit-transform:rotate(1turn);transform:rotate(1turn)}}@-webkit-keyframes _ngcontent-%COMP%_goUp{0%{-webkit-transform:translateY(10%);transform:translateY(10%);opacity:0}30%{-webkit-transform:translate(0);transform:translate(0);opacity:1}to{-webkit-transform:translateY(-25%);transform:translateY(-25%);opacity:0}}@keyframes _ngcontent-%COMP%_goUp{0%{-webkit-transform:translateY(10%);transform:translateY(10%);opacity:0}30%{-webkit-transform:translate(0);transform:translate(0);opacity:1}to{-webkit-transform:translateY(-25%);transform:translateY(-25%);opacity:0}}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]{padding:8px 12px;-moz-box-sizing:border-box;box-sizing:border-box;height:48px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;cursor:pointer;color:#444746;border-bottom:1px solid transparent}.container[_ngcontent-%COMP%]   .title.hide[_ngcontent-%COMP%]{display:none}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   .name[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;font-size:20px}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   .name[_ngcontent-%COMP%]   me-logo[_ngcontent-%COMP%]{margin-right:6px}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   .name[_ngcontent-%COMP%]   new-version-chip[_ngcontent-%COMP%]{margin-left:16px}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   .icons-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   .icons-container[_ngcontent-%COMP%]   a[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;color:#000}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   .icons-container[_ngcontent-%COMP%]   open-in-new-tab-button[_ngcontent-%COMP%]{margin-left:8px}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:start;-webkit-justify-content:flex-start;-moz-box-pack:start;-ms-flex-pack:start;justify-content:flex-start;position:relative;margin-top:16px;overflow:hidden}.container[_ngcontent-%COMP%]   .content.hide[_ngcontent-%COMP%]{display:none}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .welcome-card-container[_ngcontent-%COMP%]{width:1016px;-webkit-flex-shrink:0;-ms-flex-negative:0;flex-shrink:0;position:relative}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .welcome-card-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{position:absolute;top:6px;right:6px;cursor:pointer;opacity:.5}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .welcome-card-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]:hover{opacity:.8}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .welcome-card-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:16px;width:16px;height:16px}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .loading-adapter-extension-container[_ngcontent-%COMP%]{margin-top:20px}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .model-source-input-container[_ngcontent-%COMP%]{width:1016px;margin-top:20px;overflow:hidden;z-index:100;background-color:#fff}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .padding[_ngcontent-%COMP%]{width:100%;-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;min-height:12px}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .cover[_ngcontent-%COMP%]{position:absolute;top:0;left:0;width:100%;height:100%;z-index:5000;color:#333;background-color:hsla(0,0%,100%,.9);-webkit-backdrop-filter:blur(10px);backdrop-filter:blur(10px);display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .cover[_ngcontent-%COMP%]   .loading-subtitle[_ngcontent-%COMP%]{font-size:14px;color:#999;margin-top:20px}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .loading[_ngcontent-%COMP%]   .wait-icon[_ngcontent-%COMP%]{display:inline-block;margin:0 8px;-webkit-animation:_ngcontent-%COMP%_rotate 2s ease-in-out 0s infinite;animation:_ngcontent-%COMP%_rotate 2s ease-in-out 0s infinite}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .uploading[_ngcontent-%COMP%]   .upload-icon[_ngcontent-%COMP%]{display:inline-block;margin:0 8px;-webkit-animation:_ngcontent-%COMP%_goUp 1s ease-in-out 0s infinite;animation:_ngcontent-%COMP%_goUp 1s ease-in-out 0s infinite}.container[_ngcontent-%COMP%]   .utos[_ngcontent-%COMP%]{position:absolute;right:12px;bottom:12px;font-size:12px;color:#999}.container[_ngcontent-%COMP%]   .utos[_ngcontent-%COMP%]   a[_ngcontent-%COMP%]{color:#999}.container[_ngcontent-%COMP%]   model-graph-visualizer[_ngcontent-%COMP%]{width:100%;height:100%}.container[_ngcontent-%COMP%]   .bug-report[_ngcontent-%COMP%]{position:absolute;bottom:8px;left:8px;display:inline-block;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;color:#000;opacity:.5}.container[_ngcontent-%COMP%]   .bug-report[_ngcontent-%COMP%]:hover{opacity:.8}.container[_ngcontent-%COMP%]   .btns-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .btns-container[_ngcontent-%COMP%]   .share-button-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center}.container[_ngcontent-%COMP%]   .btns-container[_ngcontent-%COMP%]   .divider[_ngcontent-%COMP%]{border-left:1px solid #ccc;height:28px;-moz-box-sizing:border-box;box-sizing:border-box;margin-left:8px;margin-right:8px}  .model-explorer-version{font-size:12px;text-align:right;padding:8px 16px 0;color:#999}"]});var Z3=class{};Z3.K=function(a){return new (a||Z3)};Z3.Ca=Jf({type:Z3,ha:[["app-root"]],ka:!0,features:[Nr],ya:1,za:0,sa:function(a){a&1&&Oq(0,"home-page")},Ha:[Y3,l2]});function S2(a,b){return ta(function*(){var c=[];for(const d of b)"local"!==d.type&&"graphs_json_from_server"!==d.type&&"file_path"!==d.type||c.push($3(a,d));c=yield Promise.all(c);for(const d of b)null!=d.errorMessage&&(d.errorMessage=II(d.errorMessage));null==c.find(d=>0===d.length)&&a.le.set(c.flat())})}
function $3(a,b){return ta(function*(){b.status.set("Converting");var c=[];if("file_path"===b.type){var d;switch(null==(d=b.Sc)?void 0:d.id){case "builtin_json":try{const f=yield a4(b.path),g=b.path.split("/").pop()||"untitled",h=JSON.parse(f),m=HY(g,h);if(m.error)throw Error(`Failed to process file: ${m.error})`);m.graphCollections&&(c=m.graphCollections);b.status.set("Done")}catch(f){b.selected=!1,b.status.set("Error"),b.errorMessage=f}break;default:c=b.path;var e=c.split("/").pop()||"untitled";
c=yield b4(a,b,c,e,!1)}}else switch(d=b.file,null==(e=b.Sc)?void 0:e.id){case "builtin_json":try{c="graphs_json_from_server"===b.type?yield c4(b.path):yield GY(d),b.status.set("Done")}catch(h){b.selected=!1,b.status.set("Error"),b.errorMessage=h}break;default:b.status.set("Uploading");const {path:f,error:g}=yield d4(d);if(g)return b.selected=!1,b.status.set("Error"),b.errorMessage=g,[];c=yield b4(a,b,f,d.name,!0)}return c})}
function a4(a){return ta(function*(){const b=yield(yield fetch(`${"/api/v1/read_text_file"}?path=${a}`)).json();if(b.error)throw Error(`Failed to read file: ${b.error}`);return b.content})}
function b4(a,b,c,d,e){return ta(function*(){let f=[];b.status.set("Converting");var g,h=(null==(g=b.Sc)?void 0:g.id)||"";g=a.Mb;const m={};for(const w of l3)switch(w.type){case 0:m[w.key]=m3(g,w);break;case 1:m[w.key]=n3(g,w)}const {xB:p,jx:u}=yield RY({cmdId:"convert",extensionId:h,modelPath:c,settings:m,deleteAfterConversion:e});if(h=(null==p?void 0:p.error)||u)return b.selected=!1,b.status.set("Error"),b.errorMessage=h,[];p&&(f=e4(p,d));b.status.set("Done");return f})}
function c4(a){return ta(function*(){var b=a.replace("graphs://","");const c=b.lastIndexOf("/"),d=b.substring(0,c);b=yield(yield fetch(`${"/api/v1/load_graphs_json"}?graph_index=${Number(b.substring(c+1))}`)).json();return e4(b,d)})}
function d4(a){return ta(function*(){var b=new FormData;b.append("file",a,a.name);b=yield fetch("/apipost/v1/upload",{method:"POST",body:b});if(b.ok)return{path:JSON.parse(yield b.text()).path};console.error(yield b.text());return{path:"",error:"Failed to upload model. Check console for details"}})}function e4(a,b){return a.graphs?[{label:b,graphs:a.graphs}]:a.graphCollections?a.graphCollections.map(c=>({label:""===c.label?b:`${b} (${c.label})`,graphs:c.graphs})):[]}
var f4=class{constructor(a){this.Mb=a;this.le=yr()}};f4.K=function(a){return new (a||f4)(r(s3),r(SY))};f4.na=Pd({ma:f4,ba:f4.K,fa:"root"});var g4=class{};function h4(){return new ae(3008,!1)};function i4(a){if("number"==typeof a)return a;a=a.match(/^(-?[\.\d]+)(m?s)/);return!a||2>a.length?0:j4(parseFloat(a[1]),a[2])}function j4(a,b){switch(b){case "s":return 1E3*a;default:return a}}
function k4(a,b,c){if(a.hasOwnProperty("duration"))b=a;else a:{var d=/^(-?[\.\d]+)(m?s)(?:\s+(-?[\.\d]+)(m?s))?(?:\s+([-a-z]+(?:\(.+?\))?))?$/i;let e=0,f="";if("string"===typeof a){d=a.match(d);if(null===d){b.push(new ae(3E3,!1));b={duration:0,delay:0,easing:""};break a}a=j4(parseFloat(d[1]),d[2]);const g=d[3];null!=g&&(e=j4(parseFloat(g),d[4]));(d=d[5])&&(f=d)}c||(c=!1,d=b.length,0>a&&(b.push(new ae(3100,!1)),c=!0),0>e&&(b.push(new ae(3101,!1)),c=!0),c&&b.splice(d,0,new ae(3E3,!1)));b={duration:a,
delay:e,easing:f}}return b}function l4(a){return a.length?a[0]instanceof Map?a:a.map(b=>new Map(Object.entries(b))):[]}function m4(a,b,c){b.forEach((d,e)=>{const f=n4(e);c&&!c.has(e)&&c.set(e,a.style[f]);a.style[f]=d})}function o4(a,b){b.forEach((c,d)=>{d=n4(d);a.style[d]=""})}function p4(a){if(Array.isArray(a)){var b;1==a.length?b=a[0]:b={type:2,steps:a,options:null};a=b}return a}
function q4(a,b,c){const d=b.params||{};a=r4(a);a.length&&a.forEach(e=>{d.hasOwnProperty(e)||c.push(new ae(3001,!1))})}const s4=RegExp("{{\\s*(.+?)\\s*}}","g");function r4(a){let b=[];if("string"===typeof a){let c;for(;c=s4.exec(a);)b.push(c[1]);s4.lastIndex=0}return b}function t4(a,b,c){const d=`${a}`,e=d.replace(s4,(f,g)=>{g=b[g];null==g&&(c.push(new ae(3003,!1)),g="");return g.toString()});return e==d?a:e}const u4=/-+([a-z0-9])/g;function n4(a){return a.replace(u4,(...b)=>b[1].toUpperCase())}
function v4(a,b,c){if(c.size&&b.length){let d=b[0],e=[];c.forEach((f,g)=>{d.has(g)||e.push(g);d.set(g,f)});if(e.length)for(c=1;c<b.length;c++){let f=b[c];e.forEach(g=>f.set(g,w4(a,g)))}}return b}
function x4(a,b,c){switch(b.type){case 7:return a.az(b,c);case 0:return a.Ys(b,c);case 1:return a.Zs(b,c);case 2:return a.Yy(b,c);case 3:return a.Wy(b,c);case 4:return a.Ty(b,c);case 5:return a.So(b,c);case 6:return a.qm(b,c);case 8:return a.To(b,c);case 9:return a.Uy(b,c);case 10:return a.Vy(b,c);case 11:return a.Xy(b,c);case 12:return a.Zy(b,c);default:throw new ae(3004,!1);}}function w4(a,b){return window.getComputedStyle(a)[b]};const y4=new Set("width height minWidth minHeight maxWidth maxHeight left top bottom right fontSize outlineWidth outlineOffset paddingTop paddingLeft paddingBottom paddingRight marginTop marginLeft marginBottom marginRight borderRadius borderWidth borderTopWidth borderLeftWidth borderRightWidth borderBottomWidth textIndent perspective".split(" "));
function z4(a,b,c){let d="";const e=b.toString().trim();y4.has(a)&&0!==b&&"0"!==b&&("number"===typeof b?d="px":(a=b.match(/^[+-]?[\d\.]+([a-z]*)$/))&&0==a[1].length&&c.push(new ae(3005,!1)));return e+d}var A4=class extends g4{};function B4(a){switch(a.length){case 0:return new fz;case 1:return a[0];default:return new jz(a)}}function C4(a,b=new Map,c=new Map){const d=[],e=[];let f=-1,g=null;a.forEach(h=>{const m=h.get("offset"),p=m==f,u=p&&g||new Map;h.forEach((w,B)=>{let G=B;if("offset"!==B)switch(G=n4(G),w){case "!":w=b.get(B);break;case "*":w=c.get(B);break;default:w=z4(G,w,d)}u.set(G,w)});p||e.push(u);g=u;f=m});if(d.length)throw new ae(3502,!1);return e}
function D4(a,b,c,d){switch(b){case "start":a.onStart(()=>d(c&&E4(c,"start",a)));break;case "done":a.Md(()=>d(c&&E4(c,"done",a)));break;case "destroy":a.ob(()=>d(c&&E4(c,"destroy",a)))}}function E4(a,b,c){const d=c.totalTime;b=F4(a.element,a.Pc,a.Bg,a.se,b||a.HD,void 0==d?a.totalTime:d,c.disabled?!0:!1);a=a._data;null!=a&&(b._data=a);return b}function F4(a,b,c,d,e="",f=0,g){return{element:a,Pc:b,Bg:c,se:d,HD:e,totalTime:f,disabled:!!g}}function G4(a,b,c){let d=a.get(b);d||a.set(b,d=c);return d}
function H4(a){const b=a.indexOf(":");return[a.substring(1,b),a.slice(b+1)]}const I4="undefined"===typeof document?null:document.documentElement;function J4(a){a=a.parentNode||a.host||null;return a===I4?null:a}function K4(a,b){for(;b;){if(b===a)return!0;b=J4(b)}return!1};var L4=class{};function M4(a,b){const c=[];"string"==typeof a?a.split(/\s*,\s*/).forEach(d=>{a:{if(":"==d[0]&&(d=N4(d,b),"function"==typeof d)){c.push(d);d=void 0;break a}var e=d.match(/^(\*|[-\w]+)\s*(<?[=-]>)\s*(\*|[-\w]+)$/);if(null==e||4>e.length)b.push(new ae(3015,!1)),d=c;else{d=e[1];var f=e[2];e=e[3];c.push(O4(d,e));"<"!=f[0]||"*"==d&&"*"==e||c.push(O4(e,d));d=void 0}}return d}):c.push(a);return c}
function N4(a,b){switch(a){case ":enter":return"void => *";case ":leave":return"* => void";case ":increment":return(c,d)=>parseFloat(d)>parseFloat(c);case ":decrement":return(c,d)=>parseFloat(d)<parseFloat(c);default:return b.push(new ae(3016,!1)),"* => *"}}const P4=new Set(["true","1"]),Q4=new Set(["false","0"]);
function O4(a,b){const c=P4.has(a)||Q4.has(a),d=P4.has(b)||Q4.has(b);return(e,f)=>{let g="*"==a||a==e,h="*"==b||b==f;!g&&c&&"boolean"===typeof e&&(g=e?P4.has(a):Q4.has(a));!h&&d&&"boolean"===typeof f&&(h=f?P4.has(b):Q4.has(b));return g&&h}};const R4=RegExp("s*:selfs*,?","g");
var X4=class{D(a,b){b=new S4(b);this.F(b);return x4(this,p4(a),b)}F(a){a.D="";a.G=new Map;a.G.set("",new Map);a.C=0}az(a,b){let c=b.Mg=0,d=b.ef=0;const e=[],f=[];"@"==a.name.charAt(0)&&b.errors.push(new ae(3006,!1));a.MB.forEach(g=>{this.F(b);if(0==g.type){var h=g.name;h.toString().split(/\s*,\s*/).forEach(m=>{g.name=m;e.push(this.Ys(g,b))});g.name=h}else 1==g.type?(h=this.Zs(g,b),c+=h.Mg,d+=h.ef,f.push(h)):b.errors.push(new ae(3007,!1))});return{type:7,name:a.name,ni:e,ah:f,Mg:c,ef:d,options:null}}Ys(a,
b){const c=this.qm(a.styles,b),d=a.options&&a.options.params||null;if(c.BB){const e=new Set,f=d||{};c.styles.forEach(g=>{g instanceof Map&&g.forEach(h=>{r4(h).forEach(m=>{f.hasOwnProperty(m)||e.add(m)})})});e.size&&b.errors.push(h4([...e.values()]))}return{type:0,name:a.name,style:c,options:d?{params:d}:null}}Zs(a,b){b.Mg=0;b.ef=0;const c=x4(this,p4(a.animation),b);return{type:1,Sw:M4(a.bC,b.errors),animation:c,Mg:b.Mg,ef:b.ef,options:T4(a.options)}}Yy(a,b){return{type:2,steps:a.steps.map(c=>x4(this,
c,b)),options:T4(a.options)}}Wy(a,b){const c=b.C;let d=0;const e=a.steps.map(f=>{b.C=c;f=x4(this,f,b);d=Math.max(d,b.C);return f});b.C=d;return{type:3,steps:e,options:T4(a.options)}}Ty(a,b){const c=U4(a.fg,b.errors);b.F=c;var d=a.styles?a.styles:$y({});if(5==d.type)a=this.So(d,b);else{d=a.styles;a=!1;d||(a=!0,d={},c.easing&&(d.easing=c.easing),d=$y(d));b.C+=c.duration+c.delay;const e=this.qm(d,b);e.KC=a;a=e}b.F=null;return{type:4,fg:c,style:a,options:null}}qm(a,b){a=this.C(a,b);this.G(a,b);return a}C(a,
b){const c=[],d=Array.isArray(a.styles)?a.styles:[a.styles];for(let g of d)"string"===typeof g?"*"===g?c.push(g):b.errors.push(new ae(3002,!1)):c.push(new Map(Object.entries(g)));let e=!1,f=null;c.forEach(g=>{if(g instanceof Map&&(g.has("easing")&&(f=g.get("easing"),g.delete("easing")),!e))for(let h of g.values())if(0<=h.toString().indexOf("{{")){e=!0;break}});return{type:6,styles:c,easing:f,offset:a.offset,BB:e,options:null}}G(a,b){const c=b.F;let d=b.C,e=b.C;c&&0<e&&(e-=c.duration+c.delay);a.styles.forEach(f=>
{"string"!==typeof f&&f.forEach((g,h)=>{const m=b.G.get(b.D),p=m.get(h);let u=!0;p&&(e!=d&&e>=p.startTime&&d<=p.endTime&&(b.errors.push(new ae(3010,!1)),u=!1),e=p.startTime);u&&m.set(h,{startTime:e,endTime:d});b.options&&q4(g,b.options,b.errors)})})}So(a,b){const c={type:5,styles:[],options:null};if(!b.F)return b.errors.push(new ae(3011,!1)),c;let d=0;const e=[];let f=!1,g=!1,h=0;const m=a.steps.map(L=>{L=this.C(L,b);let O=null!=L.offset?L.offset:V4(L.styles),ba=0;null!=O&&(d++,ba=L.offset=O);g=g||
0>ba||1<ba;f=f||ba<h;h=ba;e.push(ba);return L});g&&b.errors.push(new ae(3012,!1));f&&b.errors.push(new ae(3200,!1));a=a.steps.length;let p=0;0<d&&d<a?b.errors.push(new ae(3202,!1)):0==d&&(p=1/(a-1));const u=a-1,w=b.C,B=b.F,G=B.duration;m.forEach((L,O)=>{O=0<p?O==u?1:p*O:e[O];const ba=O*G;b.C=w+B.delay+ba;B.duration=ba;this.G(L,b);L.offset=O;c.styles.push(L)});return c}To(a,b){return{type:8,animation:x4(this,p4(a.animation),b),options:T4(a.options)}}Uy(a,b){b.ef++;return{type:9,options:T4(a.options)}}Vy(a,
b){return{type:10,animation:this.To(a.animation,b),options:T4(a.options)}}Xy(a,b){const c=b.D,d=a.options||{};b.Mg++;b.J=a;const [e,f]=W4(a.bm);b.D=c.length?c+" "+e:e;G4(b.G,b.D,new Map);const g=x4(this,p4(a.animation),b);b.J=null;b.D=c;return{type:11,bm:e,limit:d.limit||0,optional:!!d.optional,BC:f,animation:g,rG:a.bm,options:T4(a.options)}}Zy(a,b){b.J||b.errors.push(new ae(3013,!1));const c="full"===a.fg?{duration:0,delay:0,easing:"full"}:k4(a.fg,b.errors,!0);return{type:12,animation:x4(this,p4(a.animation),
b),fg:c,options:null}}};function W4(a){const b=a.split(/\s*,\s*/).find(c=>":self"==c)?!0:!1;b&&(a=a.replace(R4,""));a=a.replace(/@\*/g,".ng-trigger").replace(/@\w+/g,c=>".ng-trigger-"+c.slice(1)).replace(/:animating/g,".ng-animating");return[a,b]}var S4=class{constructor(a){this.errors=a;this.ef=this.Mg=0;this.F=this.D=this.J=null;this.C=0;this.G=new Map;this.options=null}};
function V4(a){if("string"==typeof a)return null;let b=null;Array.isArray(a)?a.forEach(c=>{c instanceof Map&&c.has("offset")&&(b=parseFloat(c.get("offset")),c.delete("offset"))}):a instanceof Map&&a.has("offset")&&(b=parseFloat(a.get("offset")),a.delete("offset"));return b}
function U4(a,b){if(a.hasOwnProperty("duration"))return a;if("number"==typeof a)return{duration:k4(a,b).duration,delay:0,easing:""};if(a.split(/\s+/).some(c=>"{"==c.charAt(0)&&"{"==c.charAt(1)))return b={duration:0,delay:0,easing:"",dynamic:!0},b.strValue=a,b;a=k4(a,b);return{duration:a.duration,delay:a.delay,easing:a.easing}}function T4(a){if(a){if(a=Object.assign({},a),a.params){var b=a;var c=(c=a.params)?Object.assign({},c):null;b.params=c}}else a={};return a};function Y4(a,b,c,d,e,f,g=null,h=!1){return{type:1,element:a,Dg:b,Jj:c,di:d,duration:e,delay:f,totalTime:e+f,easing:g,FE:h}};var Z4=class{constructor(){this.C=new Map}get(a){return this.C.get(a)||[]}append(a,b){let c=this.C.get(a);c||this.C.set(a,c=[]);c.push(...b)}has(a){return this.C.has(a)}clear(){this.C.clear()}};const $4=RegExp(":enter","g"),a5=RegExp(":leave","g");function b5(a,b,c,d,e,f=new Map,g=new Map,h,m,p=[]){return c5(a,b,c,d,e,f,g,h,m,p)}
function c5(a,b,c,d,e,f,g,h,m,p=[]){var u=new d5;m=m||new Z4;a=new e5(a,b,m,d,e,p,[]);a.options=h;d=h.delay?i4(h.delay):0;f5(a.C,d);g5(a.C,[f],null,a.errors,h);x4(u,c,a);c=a.tf.filter(w=>w.ta());if(c.length&&g.size){let w;for(f=c.length-1;0<=f;f--)if(u=c[f],u.element===b){w=u;break}w&&w.oa===w.C&&g5(w,[g],null,a.errors,h)}return c.length?c.map(w=>w.ja()):[Y4(b,[],[],[],0,d,"",!1)]}
var d5=class{az(){}Ys(){}Zs(){}Uy(a,b){var c=b.ca.get(b.element);if(c){const d=h5(b,a.options),e=b.C.D;c=this.D(c,d,d.options);e!=c&&i5(b,c)}b.D=a}Vy(a,b){const c=h5(b,a.options);i5(c);this.C([a.options,a.animation.options],b,c);this.To(a.animation,c);i5(b,c.C.D);b.D=a}C(a,b,c){for(const d of a){let e;if(a=null==(e=d)?void 0:e.delay){let f,g;a="number"===typeof a?a:i4(t4(a,null!=(g=null==(f=d)?void 0:f.params)?g:{},b.errors));j5(c,a)}}}D(a,b,c){let d=b.C.D;const e=null!=c.duration?i4(c.duration):
null,f=null!=c.delay?i4(c.delay):null;0!==e&&a.forEach(g=>{const h={duration:null!=e?e:g.duration,delay:b.C.D+(null!=f?f:0)+g.delay,easing:""};g=new k5(b.J,g.element,g.Dg,g.Jj,g.di,h,g.EE);b.tf.push(g);d=Math.max(d,h.duration+h.delay)});return d}To(a,b){l5(b,a.options,!0);x4(this,a.animation,b);b.D=a}Yy(a,b){const c=b.N;let d=b;const e=a.options;e&&(e.params||e.delay)&&(d=h5(b,e),i5(d),null!=e.delay&&(6==d.D.type&&(m5(d.C),d.D=n5),j5(d,i4(e.delay))));a.steps.length&&(a.steps.forEach(f=>x4(this,f,
d)),o5(d.C),d.N>c&&i5(d));b.D=a}Wy(a,b){const c=[];let d=b.C.D;const e=a.options&&a.options.delay?i4(a.options.delay):0;a.steps.forEach(f=>{const g=h5(b,a.options);e&&j5(g,e);x4(this,f,g);d=Math.max(d,g.C.D);c.push(g.C)});c.forEach(f=>p5(b.C,f));i5(b,d);b.D=a}F(a,b){return a.dynamic?(a=a.strValue,a=b.params?t4(a,b.params,b.errors):a,k4(a,b.errors)):{duration:a.duration,delay:a.delay,easing:a.easing}}Ty(a,b){const c=b.F=this.F(a.fg,b),d=b.C;c.delay&&(q5(b.C,b.C.duration+c.delay),m5(d));const e=a.style;
5==e.type?this.So(e,b):(q5(b.C,b.C.duration+c.duration),this.qm(e,b),o5(d));b.F=null;b.D=a}qm(a,b){const c=b.C;var d=b.F;!d&&0<c.C.size&&(c.duration+=1,c.T());d=d&&d.easing||a.easing;if(a.KC){d&&c.aa.set("easing",d);for(let [e,f]of c.F)d=e,c.P.set(d,f||"*"),c.C.set(d,"*");c.oa=c.C}else g5(c,a.styles,d,b.errors,b.options);b.D=a}So(a,b){const c=b.F,d=b.C.duration,e=c.duration,f=h5(b).C;f.easing=c.easing;a.styles.forEach(g=>{q5(f,(g.offset||0)*e);g5(f,g.styles,g.easing,b.errors,b.options);o5(f)});p5(b.C,
f);i5(b,d+e);b.D=a}Xy(a,b){var c=b.C.D;const d=a.options||{},e=d.delay?i4(d.delay):0;e&&(6===b.D.type||0==c&&0<b.C.C.size)&&(m5(b.C),b.D=n5);let f=c;c=r5(b,a.bm,a.limit,a.BC,d.optional?!0:!1,b.errors);b.G=c.length;let g=null;c.forEach((h,m)=>{b.ge=m;m=h5(b,a.options,h);e&&j5(m,e);h===b.element&&(g=m.C);x4(this,a.animation,m);o5(m.C);f=Math.max(f,m.C.D)});b.ge=0;b.G=0;i5(b,f);g&&(p5(b.C,g),m5(b.C));b.D=a}Zy(a,b){const c=b.aa,d=b.C;var e=a.fg,f=Math.abs(e.duration);const g=f*(b.G-1);f*=b.ge;switch(0>
e.duration?"reverse":e.easing){case "reverse":f=g-f;break;case "full":f=c.T}e=b.C;f&&f5(e,f);e=e.D;x4(this,a.animation,b);b.D=a;c.T=d.D-e+(d.startTime-c.C.startTime)}};const n5={};function h5(a,b=null,c){c=c||a.element;c=new e5(a.J,c,a.ca,a.O,a.P,a.errors,a.tf,a.C.fork(c,0));c.D=a.D;c.F=a.F;c.options=a.ea();l5(c,b);c.ge=a.ge;c.G=a.G;c.aa=a;a.N++;return c}function i5(a,b){a.D=n5;a.C=a.C.fork(a.element,b);a.tf.push(a.C)}function j5(a,b){0<b&&f5(a.C,b)}
function l5(a,b,c){if(b){var d=a.options;null!=b.duration&&(d.duration=i4(b.duration));null!=b.delay&&(d.delay=i4(b.delay));var e=b.params;if(e){let f=d.params;f||(f=a.options.params={});Object.keys(e).forEach(g=>{c&&f.hasOwnProperty(g)||(f[g]=t4(e[g],f,a.errors))})}}}
function r5(a,b,c,d,e,f){let g=[];d&&g.push(a.element);0<b.length&&(b=b.replace($4,"."+a.O),b=b.replace(a5,"."+a.P),a=a.J.query(a.element,b,1!=c),0!==c&&(a=0>c?a.slice(a.length+c,a.length):a.slice(0,c)),g.push(...a));e||0!=g.length||f.push(new ae(3014,!1));return g}
var e5=class{constructor(a,b,c,d,e,f,g,h){this.J=a;this.element=b;this.ca=c;this.O=d;this.P=e;this.errors=f;this.tf=g;this.F=this.aa=null;this.D=n5;this.N=0;this.options={};this.T=this.G=this.ge=0;this.C=h||new s5(this.J,b,0);g.push(this.C)}get params(){return this.options.params}ea(){const a={};if(this.options){const b=this.options.params;if(b){const c=a.params={};Object.keys(b).forEach(d=>{c[d]=b[d]})}}return a}};
function f5(a,b){const c=1===a.G.size&&a.N.size;a.duration||c?(q5(a,a.D+b),c&&m5(a)):a.startTime+=b}function g5(a,b,c,d,e){c&&a.aa.set("easing",c);c=e&&e.params||{};b=t5(b,a.F);for(let [f,g]of b){b=f;e=t4(g,c,d);a.N.set(b,e);if(!a.J.has(b)){let h;a.P.set(b,null!=(h=a.F.get(b))?h:"*")}a.ea(b,e)}}function m5(a){for(let [b,c]of a.J){let d=b,e=c;a.N.set(d,e);a.ea(d,e)}}function o5(a){0!=a.N.size&&(a.N.forEach((b,c)=>{a.C.set(c,b)}),a.N.clear(),a.J.forEach((b,c)=>{a.C.has(c)||a.C.set(c,b)}))}
function p5(a,b){b.ca.forEach((c,d)=>{const e=a.ca.get(d);(!e||c.time>e.time)&&a.ea(d,c.value)})}function q5(a,b){o5(a);a.duration=b;a.T()}
var s5=class{constructor(a,b,c,d){this.ua=a;this.element=b;this.startTime=c;this.O=d;this.duration=0;this.easing=null;this.aa=new Map;this.C=new Map;this.G=new Map;this.ca=new Map;this.J=new Map;this.N=new Map;this.P=new Map;this.oa=null;this.O||(this.O=new Map);this.F=this.O.get(b);this.F||(this.F=this.J,this.O.set(b,this.J));this.T()}ta(){switch(this.G.size){case 0:return!1;case 1:return 0<this.C.size;default:return!0}}get D(){return this.startTime+this.duration}fork(a,b){o5(this);return new s5(this.ua,
a,b||this.D,this.O)}T(){this.C&&(this.aa=this.C);this.C=this.G.get(this.duration);this.C||(this.C=new Map,this.G.set(this.duration,this.C))}ea(a,b){this.J.set(a,b);this.F.set(a,b);this.ca.set(a,{time:this.D,value:b})}get properties(){const a=[];for(let b in this.C)a.push(b);return a}ja(){o5(this);const a=new Set,b=new Set,c=1===this.G.size&&0===this.duration;let d=[];this.G.forEach((g,h)=>{g=new Map([...this.P,...g]);g.forEach((m,p)=>{"!"===m?a.add(p):"*"===m&&b.add(p)});c||g.set("offset",h/this.duration);
d.push(g)});const e=[...a.values()],f=[...b.values()];if(c){const g=d[0],h=new Map(g);g.set("offset",0);h.set("offset",1);d=[g,h]}return Y4(this.element,d,e,f,this.duration,this.startTime,this.easing,!1)}};
class k5 extends s5{constructor(a,b,c,d,e,f,g=!1){super(a,b,f.delay);this.Dg=c;this.Jj=d;this.di=e;this.wa=g;this.fg={duration:f.duration,delay:f.delay,easing:f.easing}}ta(){return 1<this.Dg.length}ja(){let a=this.Dg,{delay:b,duration:c,easing:d}=this.fg;if(this.wa&&b){const g=[],h=c+b;var e=b/h,f=new Map(a[0]);f.set("offset",0);g.push(f);f=new Map(a[0]);f.set("offset",u5(e));g.push(f);e=a.length-1;for(f=1;f<=e;f++){let m=new Map(a[f]);const p=m.get("offset");m.set("offset",u5((b+p*c)/h));g.push(m)}c=
h;b=0;d="";a=g}return Y4(this.element,a,this.Jj,this.di,c,b,d,!0)}}function u5(a){const b=Math.pow(10,2);return Math.round(a*b)/b}function t5(a,b){const c=new Map;let d;a.forEach(e=>{if("*"===e){null!=d||(d=b.keys());for(let f of d)c.set(f,"*")}else for(let [f,g]of e)c.set(f,g)});return c};function v5(a,b,c,d,e,f,g,h,m,p,u,w,B){return{type:0,element:a,Pc:b,Kw:e,Bg:c,Hn:f,se:d,lm:g,tf:h,PD:m,Jj:p,di:u,totalTime:w,errors:B}};const w5={};function x5(a,b,c,d){let e=a.F.get("*");void 0!==b&&(e=a.F.get(null==b?void 0:b.toString())||e);return e?y5(e,c,d):new Map}
var B5=class{constructor(a,b,c){this.G=a;this.C=b;this.F=c}match(a,b,c,d){return z5(this.C.Sw,a,b,c,d)}D(a,b,c,d,e,f,g,h,m,p){const u=[];var w=this.C.options&&this.C.options.params||w5;g=x5(this,c,g&&g.params||w5,u);const B=h&&h.params||w5;h=x5(this,d,B,u);const G=new Set,L=new Map,O=new Map,ba="void"===d;let ca;w={params:A5(B,w),delay:null==(ca=this.C.options)?void 0:ca.delay};a=p?[]:b5(a,b,this.C.animation,e,f,g,h,w,m,u);let na=0;a.forEach(K=>{na=Math.max(K.duration+K.delay,na)});if(u.length)return v5(b,
this.G,c,d,ba,g,h,[],[],L,O,na,u);a.forEach(K=>{const W=K.element,S=G4(L,W,new Set);K.Jj.forEach(F=>S.add(F));const ja=G4(O,W,new Set);K.di.forEach(F=>ja.add(F));W!==b&&G.add(W)});return v5(b,this.G,c,d,ba,g,h,a,[...G.values()],L,O,na)}};function z5(a,b,c,d,e){return a.some(f=>f(b,c,d,e))}function A5(a,b){const c=Object.assign({},b);Object.entries(a).forEach(([d,e])=>{null!=e&&(c[d]=e)});return c}
function y5(a,b,c){const d=new Map,e=A5(b,a.C);a.styles.styles.forEach(f=>{"string"!==typeof f&&f.forEach((g,h)=>{g&&(g=t4(g,e,c));const m=n4(h);g=z4(m,g,c);d.set(h,g)})});return d}var C5=class{constructor(a,b){this.styles=a;this.C=b}};function D5(a,b,c,d,e){return a.F.find(f=>f.match(b,c,d,e))||null}var G5=class{constructor(a,b){this.name=a;this.C=b;this.F=[];this.ni=new Map;b.ni.forEach(c=>{this.ni.set(c.name,new C5(c.style,c.options&&c.options.params||{}))});E5(this.ni,"true","1");E5(this.ni,"false","0");b.ah.forEach(c=>{this.F.push(new B5(a,c,this.ni))});this.D=F5(a,this.ni)}};function F5(a,b){return new B5(a,{type:1,animation:{type:2,steps:[],options:null},Sw:[()=>!0],options:null,Mg:0,ef:0},b)}
function E5(a,b,c){a.has(b)?a.has(c)||a.set(c,a.get(b)):a.has(c)&&a.set(b,a.get(c))};const H5=new Z4;
var I5=class{constructor(a){this.G=a;this.J=new Map;this.F=new Map;this.C=[]}register(a,b){const c=[];b=(new X4).D(b,c,[]);if(c.length)throw new ae(3503,!1);this.J.set(a,b)}create(a,b,c={}){var d=[];const e=this.J.get(a),f=new Map;e?(b=b5(this.G,b,e,"ng-enter","ng-leave",new Map,new Map,c,H5,d),b.forEach(g=>{const h=G4(f,g.element,new Map);g.di.forEach(m=>h.set(m,null))})):(d.push(new ae(3300,!1)),b=[]);if(d.length)throw new ae(3504,!1);f.forEach((g,h)=>{g.forEach((m,p)=>{g.set(p,w4(h,p))})});d=b.map(g=>
{var h=f.get(g.element);const m=g.element;h=C4(g.Dg,new Map,h);return this.G.animate(m,h,g.duration,g.delay,g.easing,[],!0)});d=B4(d);this.F.set(a,d);d.ob(()=>this.destroy(a));this.C.push(d);return d}destroy(a){const b=this.D(a);b.destroy();this.F.delete(a);a=this.C.indexOf(b);0<=a&&this.C.splice(a,1)}D(a){a=this.F.get(a);if(!a)throw new ae(3301,!1);return a}Id(a,b,c,d){b=F4(b,"","","");D4(this.D(a),c,b,d);return()=>{}}};const J5=[],K5={mc:"",ki:!1,Cs:!1,Ln:!1,ls:!1},L5={mc:"",Cs:!1,ki:!1,Ln:!1,ls:!0};function M5(a,b){const c=b.params;if(c){const d=a.options.params;Object.keys(c).forEach(e=>{null==d[e]&&(d[e]=c[e])})}}class N5{get params(){return this.options.params}constructor(a,b){this.mc=void 0===b?"":b;var c=(b=a&&a.hasOwnProperty("value"))?a.value:a;this.value=null!=c?c:null;b?(a=Object.assign({},a),this.options=(delete a.value,a)):this.options={};this.options.params||(this.options.params={})}}const O5=new N5("void");
function P5(a,b){a.C.C.delete(b);a.G.delete(b);const c=a.C.N.get(b);c&&(c.forEach(d=>d.destroy()),a.C.N.delete(b))}function Q5(a,b,c,d,e){const f=a.C.C.get(b),g=new Map;if(f){const h=[];f.forEach((m,p)=>{g.set(p,m.value);a.D.has(p)&&(m=a.Oe(b,p,"void",e))&&h.push(m)});if(h.length)return R5(a.C,a.id,b,!0,c,g),d&&B4(h).Md(()=>S5(a.C,b)),!0}return!1}
function T5(a,b){const c=a.G.get(b),d=a.C.C.get(b);if(c&&d){const e=new Set;c.forEach(f=>{f=f.name;if(!e.has(f)){e.add(f);var g=a.D.get(f).D,h=d.get(f)||O5,m=new N5("void"),p=new U5(a.id,f,b);a.C.oa++;a.J.push({element:b,Pc:f,transition:g,Bg:h,se:m,bi:p,Ew:!0})}})}}
function V5(a,b){const c=[];a.J.forEach(d=>{const e=d.bi;if(!e.C){var f=d.element,g=a.G.get(f);g&&g.forEach(h=>{if(h.name==d.Pc){const m=F4(f,d.Pc,d.Bg.value,d.se.value);m._data=b;D4(d.bi,h.ai,m,h.callback)}});e.O?W5(a.C,()=>{e.destroy()}):c.push(d)}});a.J=[];return c.sort((d,e)=>{const f=d.transition.C.ef,g=e.transition.C.ef;return 0==f||0==g?f-g:K4(d.element,e.element)?1:-1})}
class X5{constructor(a,b,c){this.id=a;this.ff=b;this.C=c;this.F=[];this.D=new Map;this.J=[];this.G=new Map;this.N="ng-tns-"+a;Y5(b,this.N)}Id(a,b,c,d){if(!this.D.has(b))throw new ae(3302,!1);if(null==c||0==c.length)throw new ae(3303,!1);if("start"!=c&&"done"!=c)throw new ae(3400,!1);const e=G4(this.G,a,[]),f={name:b,ai:c,callback:d};e.push(f);const g=G4(this.C.C,a,new Map);g.has(b)||(Y5(a,"ng-trigger"),Y5(a,"ng-trigger-"+b),g.set(b,O5));return()=>{W5(this.C,()=>{const h=e.indexOf(f);0<=h&&e.splice(h,
1);this.D.has(b)||g.delete(b)})}}register(a,b){if(this.D.has(a))return!1;this.D.set(a,b);return!0}P(a){a=this.D.get(a);if(!a)throw new ae(3401,!1);return a}Oe(a,b,c,d){d=void 0===d?!0:d;const e=this.P(b),f=new U5(this.id,b,a);var g=this.C.C.get(a);g||(Y5(a,"ng-trigger"),Y5(a,"ng-trigger-"+b),this.C.C.set(a,g=new Map));let h=g.get(b);const m=new N5(c,this.id);c&&c.hasOwnProperty("value")||!h||M5(m,h.options);g.set(b,m);h||(h=O5);if("void"!==m.value&&h.value===m.value){if(!Z5(h.params,m.params)){d=
[];const u=x5(e.D,h.value,h.params,d),w=x5(e.D,m.value,m.params,d);if(d.length)throw new ae(3402,!1);W5(this.C,()=>{o4(a,u);m4(a,w)})}}else{c=G4(this.C.N,a,[]);c.forEach(u=>{u.mc==this.id&&u.Pc==b&&u.Pd&&u.destroy()});g=D5(e,h.value,m.value,a,m.params);var p=!1;if(!g){if(!d)return;g=e.D;p=!0}this.C.oa++;this.J.push({element:a,Pc:b,transition:g,Bg:h,se:m,bi:f,Ew:p});p||(Y5(a,"ng-animate-queued"),f.onStart(()=>{$5(a,"ng-animate-queued")}));f.Md(()=>{var u=this.F.indexOf(f);0<=u&&this.F.splice(u,1);
if(u=this.C.N.get(a)){let w=u.indexOf(f);0<=w&&u.splice(w,1)}});this.F.push(f);c.push(f);return f}}O(a,b){const c=this.C.F.query(a,".ng-trigger",!0);c.forEach(d=>{if(!d.__ng_removed){var e=a6(this.C,d);e.size?e.forEach(f=>Q5(f,d,b,!1,!0)):P5(this,d)}});b6(this.C,()=>c.forEach(d=>P5(this,d)))}removeNode(a,b){const c=this.C;a.childElementCount&&this.O(a,b);if(!Q5(this,a,b,!0)){var d=!1;if(c.Ja){var e=c.ca.length?c.T.get(a):[];if(e&&e.length)d=!0;else for(e=a;e=e.parentNode;)if(c.C.get(e)){d=!0;break}}T5(this,
a);d?R5(c,this.id,a,!1,b):(d=a.__ng_removed,d&&d!==K5||(W5(c,()=>P5(this,a)),c6(c,a),c.wa(a,b)))}}destroy(a){this.F.forEach(b=>b.destroy());this.O(this.ff,a)}}function W5(a,b){a.ua.push(b)}function a6(a,b){const c=new Set;if(b=a.C.get(b))for(let d of b.values())d.mc&&(b=a.J(d.mc))&&c.add(b);return c}function b6(a,b){a.ea.push(b)}function R5(a,b,c,d,e,f){a.D.push(c);c.__ng_removed={mc:b,ki:e,Ln:d,ls:!1,cs:f}}
function S5(a,b){const c=b.__ng_removed;if(c&&c.ki){b.__ng_removed=K5;if(c.mc){c6(a,b);const e=a.J(c.mc);e&&P5(e,b)}a.wa(b,c.ki)}let d;(null==(d=b.classList)?0:d.contains("ng-animate-disabled"))&&d6(a,b,!1);a.F.query(b,".ng-animate-disabled",!0).forEach(e=>{d6(a,e,!1)})}function c6(a,b){let c=a.F.query(b,".ng-trigger",!0);c.forEach(d=>e6(a,d));0!=a.T.size&&(c=a.F.query(b,".ng-animating",!0),c.forEach(d=>f6(a,d)))}
function g6(a,b,c,d){if(h6(c)){var e=c.__ng_removed;e&&e.ki&&(e.ki=!1,e.Cs=!0,e=a.D.indexOf(c),0<=e&&a.D.splice(e,1));b&&(b=a.J(b))&&Y5(c,b.N);d&&a.G.push(c)}}function d6(a,b,c){c?a.P.has(b)||(a.P.add(b),Y5(b,"ng-animate-disabled")):a.P.has(b)&&(a.P.delete(b),$5(b,"ng-animate-disabled"))}function e6(a,b){(a=a.N.get(b))&&a.forEach(c=>{c.Pd?c.O=!0:c.destroy()})}function f6(a,b){(a=a.T.get(b))&&a.forEach(c=>c.finish())}
function i6(a,b,c,d){return 0<c.length?a.F.animate(b.element,c,b.duration,b.delay,b.easing,d):new fz(b.duration,b.delay)}
var q6=class{wa(a,b){this.Qa(a,b)}constructor(a,b){this.ta=a;this.F=b;this.ca=[];this.ja=new Map;this.N=new Map;this.T=new Map;this.C=new Map;this.P=new Set;this.oa=this.Ja=0;this.aa={};this.O=[];this.ua=[];this.ea=[];this.Ea=new Map;this.G=[];this.D=[];this.Qa=()=>{}}Ta(a,b){const c=this.O,d=this.Ea;if(0<=c.length-1){let e=!1,f=J4(b);for(;f;){const g=d.get(f);if(g){c.splice(c.indexOf(g)+1,0,a);e=!0;break}f=J4(f)}e||c.unshift(a)}else c.push(a);d.set(b,a);return a}register(a,b){var c=this.aa[a];c||
(c=new X5(a,b,this),this.ta&&K4(this.ta,b)?this.Ta(c,b):(this.ja.set(b,c),this.G.push(b)),c=this.aa[a]=c);return c}destroy(a,b){a&&(W5(this,()=>{}),b6(this,()=>{const c=this.J(a);this.Ea.delete(c.ff);const d=this.O.indexOf(c);0<=d&&this.O.splice(d,1);c.destroy(b);delete this.aa[a]}))}J(a){return this.aa[a]}Oe(a,b,c,d){return h6(b)&&(a=this.J(a))?(a.Oe(b,c,d),!0):!1}removeNode(a,b,c){if(h6(b)){var d=a?this.J(a):null;d?d.removeNode(b,c):R5(this,a,b,!1,c);(d=this.Ea.get(b))&&d.id!==a&&d.removeNode(b,
c)}else this.wa(b,c)}Id(a,b,c,d,e){return h6(b)?this.J(a).Id(b,c,d,e):()=>{}}lb(a,b,c,d,e){return a.transition.D(this.F,a.element,a.Bg.value,a.se.value,c,d,a.Bg.options,a.se.options,b,e)}flush(a){a=void 0===a?-1:a;let b=[];this.ja.size&&(this.ja.forEach((d,e)=>this.Ta(d,e)),this.ja.clear());if(this.Ja&&this.G.length)for(var c=0;c<this.G.length;c++)Y5(this.G[c],"ng-star-inserted");if(this.O.length&&(this.oa||this.D.length)){c=[];try{b=this.sb(c,a)}finally{for(a=0;a<c.length;a++)c[a]()}}else for(a=
0;a<this.D.length;a++)S5(this,this.D[a]);this.oa=0;this.G.length=0;this.D.length=0;this.ua.forEach(d=>d());this.ua=[];if(this.ea.length){const d=this.ea;this.ea=[];b.length?B4(b).Md(()=>{d.forEach(e=>e())}):d.forEach(e=>e())}}sb(a,b){const c=new Z4,d=[],e=new Map,f=[],g=new Map,h=new Map,m=new Map,p=new Set;this.P.forEach(ma=>{p.add(ma);ma=this.F.query(ma,".ng-animate-queued",!0);for(let T=0;T<ma.length;T++)p.add(ma[T])});const u=this.ta;var w=Array.from(this.C.keys());const B=j6(w,this.G),G=new Map;
let L=0;B.forEach((ma,T)=>{const da="ng-enter"+L++;G.set(T,da);ma.forEach(oa=>Y5(oa,da))});const O=[],ba=new Set;var ca=new Set;for(var na=0;na<this.D.length;na++){var K=this.D[na];const ma=K.__ng_removed;ma&&ma.ki&&(O.push(K),ba.add(K),ma.Ln?this.F.query(K,".ng-star-inserted",!0).forEach(T=>ba.add(T)):ca.add(K))}const W=new Map,S=j6(w,Array.from(ba));S.forEach((ma,T)=>{const da="ng-leave"+L++;W.set(T,da);ma.forEach(oa=>Y5(oa,da))});a.push(()=>{B.forEach((ma,T)=>{const da=G.get(T);ma.forEach(oa=>
$5(oa,da))});S.forEach((ma,T)=>{const da=W.get(T);ma.forEach(oa=>$5(oa,da))});O.forEach(ma=>{S5(this,ma)})});const ja=[],F=[];for(a=this.O.length-1;0<=a;a--)V5(this.O[a],b).forEach(ma=>{const T=ma.bi,da=ma.element;ja.push(T);if(this.G.length){var oa=da.__ng_removed;if(oa&&oa.Cs){if(oa.cs&&oa.cs.has(ma.Pc)){oa=oa.cs.get(ma.Pc);var Ia=this.C.get(ma.element);if(Ia&&Ia.has(ma.Pc)){var db=Ia.get(ma.Pc);db.value=oa;Ia.set(ma.Pc,db)}}T.destroy();return}}oa=!u||!K4(u,da);Ia=W.get(da);db=G.get(da);const Oa=
this.lb(ma,c,db,Ia,oa);if(Oa.errors&&Oa.errors.length)F.push(Oa);else if(oa)T.onStart(()=>o4(da,Oa.Hn)),T.ob(()=>m4(da,Oa.lm)),d.push(T);else if(ma.Ew)T.onStart(()=>o4(da,Oa.Hn)),T.ob(()=>m4(da,Oa.lm)),d.push(T);else{var tb=[];Oa.tf.forEach(wb=>{wb.EE=!0;this.P.has(wb.element)||tb.push(wb)});Oa.tf=tb;c.append(da,Oa.tf);f.push({yw:Oa,bi:T,element:da});Oa.PD.forEach(wb=>G4(g,wb,[]).push(T));Oa.Jj.forEach((wb,sa)=>{if(wb.size){let Ha=h.get(sa);Ha||h.set(sa,Ha=new Set);wb.forEach((Wa,cb)=>Ha.add(cb))}});
Oa.di.forEach((wb,sa)=>{let Ha=m.get(sa);Ha||m.set(sa,Ha=new Set);wb.forEach((Wa,cb)=>Ha.add(cb))})}});if(F.length){const ma=[];F.forEach(()=>{ma.push(new ae(3505,!1))});ja.forEach(T=>T.destroy());throw new ae(3402,!1);}const I=new Map,aa=new Map;f.forEach(ma=>{const T=ma.element;c.has(T)&&(aa.set(T,T),this.fb(ma.bi.mc,ma.yw,I))});d.forEach(ma=>{const T=ma.element;this.Va(T,!1,ma.mc,ma.Pc,null).forEach(da=>{G4(I,T,[]).push(da);da.destroy()})});const Aa=O.filter(ma=>k6(ma,h,m)),Za=new Map;l6(Za,ca,
m).forEach(ma=>{k6(ma,h,m)&&Aa.push(ma)});const vb=new Map;B.forEach(ma=>{l6(vb,new Set(ma),h)});Aa.forEach(ma=>{const T=Za.get(ma),da=vb.get(ma);let oa,Ia;Za.set(ma,new Map([...(null!=(oa=null==T?void 0:T.entries())?oa:[]),...(null!=(Ia=null==da?void 0:da.entries())?Ia:[])]))});const kb=[],jb=[],bb={};f.forEach(ma=>{const T=ma.element,da=ma.bi,oa=ma.yw;if(c.has(T))if(p.has(T))da.ob(()=>m4(T,oa.lm)),da.disabled=!0,da.totalTime=oa.totalTime,d.push(da);else{var Ia=bb;if(1<aa.size){ma=T;const db=[];
for(;ma=ma.parentNode;){const Oa=aa.get(ma);if(Oa){Ia=Oa;break}db.push(ma)}db.forEach(Oa=>aa.set(Oa,Ia))}ma=this.gb(da.mc,oa,I,e,vb,Za);m6(da,ma);Ia===bb?kb.push(da):((ma=this.N.get(Ia))&&ma.length&&(da.F=B4(ma)),d.push(da))}else o4(T,oa.Hn),da.ob(()=>m4(T,oa.lm)),jb.push(da),p.has(T)&&d.push(da)});jb.forEach(ma=>{var T=e.get(ma.element);T&&T.length&&(T=B4(T),m6(ma,T))});d.forEach(ma=>{ma.F?n6(ma,ma.F):ma.destroy()});for(b=0;b<O.length;b++)if(ca=O[b],a=ca.__ng_removed,$5(ca,"ng-leave"),!a||!a.Ln){a=
[];if(g.size)for((w=g.get(ca))&&w.length&&a.push(...w),w=this.F.query(ca,".ng-animating",!0),na=0;na<w.length;na++)(K=g.get(w[na]))&&K.length&&a.push(...K);a=a.filter(ma=>!ma.C);a.length?o6(this,ca,a):S5(this,ca)}O.length=0;kb.forEach(ma=>{this.ca.push(ma);ma.Md(()=>{ma.destroy();this.ca.splice(this.ca.indexOf(ma),1)});ma.play()});return kb}Va(a,b,c,d,e){let f=[];if(b)(e=this.T.get(a))&&(f=e);else if(a=this.N.get(a)){const g=!e||"void"==e;a.forEach(h=>{h.Pd||(g||h.Pc==d)&&f.push(h)})}if(c||d)f=f.filter(g=>
c&&c!=g.mc||d&&d!=g.Pc?!1:!0);return f}fb(a,b,c){var d=b.Pc;const e=b.element;a=b.Kw?void 0:a;d=b.Kw?void 0:d;for(const f of b.tf){const g=f.element,h=g!==e,m=G4(c,g,[]);this.Va(g,h,a,d,b.se).forEach(p=>{const u=p.D;u.Ok&&u.Ok();p.destroy();m.push(p)})}o4(e,b.Hn)}gb(a,b,c,d,e,f){const g=b.Pc,h=b.element,m=[],p=new Set,u=new Set,w=b.tf.map(G=>{const L=G.element;p.add(L);var O=L.__ng_removed;if(O&&O.ls)return new fz(G.duration,G.delay);O=L!==h;var ba=p6((c.get(L)||J5).map(K=>K.D)).filter(K=>K.element?
K.element===L:!1),ca=e.get(L);const na=f.get(L);ca=C4(G.Dg,ca,na);ba=i6(this,G,ca,ba);G.FE&&d&&u.add(L);O&&(G=new U5(a,g,L),m6(G,ba),m.push(G));return ba});m.forEach(G=>{G4(this.T,G.element,[]).push(G);G.Md(()=>{var L=this.T,O=G.element;let ba=L.get(O);ba&&(ba.length&&ba.splice(ba.indexOf(G),1),0==ba.length&&L.delete(O));return ba})});p.forEach(G=>Y5(G,"ng-animating"));const B=B4(w);B.ob(()=>{p.forEach(G=>$5(G,"ng-animating"));m4(h,b.lm)});u.forEach(G=>{G4(d,G,[]).push(B)});return B}};
function m6(a,b){a.N||(a.D=b,a.J.forEach((c,d)=>{c.forEach(e=>D4(b,d,void 0,e))}),a.J.clear(),a.N=!0,a.totalTime=b.totalTime,a.Pd=!1)}function n6(a,b){const c=a.D;if(c.si)b.onStart(()=>c.si("start"));b.Md(()=>a.finish());b.ob(()=>a.destroy())}
var U5=class{constructor(a,b,c){this.mc=a;this.Pc=b;this.element=c;this.D=new fz;this.N=!1;this.J=new Map;this.C=!1;this.F=null;this.disabled=this.O=!1;this.Pd=!0;this.totalTime=0}G(a,b){G4(this.J,a,[]).push(b)}Md(a){this.Pd&&this.G("done",a);this.D.Md(a)}onStart(a){this.Pd&&this.G("start",a);this.D.onStart(a)}ob(a){this.Pd&&this.G("destroy",a);this.D.ob(a)}init(){this.D.init()}Rf(){return this.Pd?!1:this.D.Rf()}play(){!this.Pd&&this.D.play()}pause(){!this.Pd&&this.D.pause()}re(){!this.Pd&&this.D.re()}finish(){this.D.finish()}destroy(){this.C=
!0;this.D.destroy()}reset(){!this.Pd&&this.D.reset()}setPosition(a){this.Pd||this.D.setPosition(a)}si(a){const b=this.D;b.si&&b.si(a)}};function h6(a){return a&&1===a.nodeType}function r6(a,b){const c=a.style.display;a.style.display=null!=b?b:"none";return c}
function l6(a,b,c){const d=[];b.forEach(g=>d.push(r6(g)));const e=[];c.forEach((g,h)=>{const m=new Map;g.forEach(p=>{const u=w4(h,p);m.set(p,u);u&&0!=u.length||(h.__ng_removed=L5,e.push(h))});a.set(h,m)});let f=0;b.forEach(g=>r6(g,d[f++]));return e}
function j6(a,b){function c(g){if(!g)return 1;let h=f.get(g);if(h)return h;const m=g.parentNode;d.has(m)?h=m:e.has(m)?h=1:h=c(m);f.set(g,h);return h}const d=new Map;a.forEach(g=>d.set(g,[]));if(0==b.length)return d;const e=new Set(b),f=new Map;b.forEach(g=>{const h=c(g);1!==h&&d.get(h).push(g)});return d}function Y5(a,b){let c;null==(c=a.classList)||c.add(b)}function $5(a,b){let c;null==(c=a.classList)||c.remove(b)}function o6(a,b,c){B4(c).Md(()=>S5(a,b))}
function p6(a){const b=[];s6(a,b);return b}function s6(a,b){for(let c=0;c<a.length;c++){const d=a[c];d instanceof jz?s6(d.C,b):b.push(d)}}function Z5(a,b){const c=Object.keys(a);var d=Object.keys(b);if(c.length!=d.length)return!1;for(d=0;d<c.length;d++){const e=c[d];if(!b.hasOwnProperty(e)||a[e]!==b[e])return!1}return!0}function k6(a,b,c){const d=c.get(a);if(!d)return!1;let e=b.get(a);e?d.forEach(f=>e.add(f)):b.set(a,d);c.delete(a);return!0};function t6(a,b){b6(a.C,b)}
var u6=class{constructor(a,b){this.G={};this.D=()=>{};this.C=new q6(a.body,b);this.F=new I5(b);this.C.Qa=(c,d)=>{this.D(c,d)}}register(a,b){this.C.register(a,b)}destroy(a,b){this.C.destroy(a,b)}process(a,b,c,d){if("@"==c.charAt(0)){const [f,g]=H4(c);a=this.F;c=f;var e=g;if("register"==e)a.register(c,d[0]);else if("create"==e)a.create(c,b,d[0]||{});else switch(b=a.D(c),e){case "play":b.play();break;case "pause":b.pause();break;case "reset":b.reset();break;case "restart":b.re();break;case "finish":b.finish();
break;case "init":b.init();break;case "setPosition":b.setPosition(parseFloat(d[0]));break;case "destroy":a.destroy(c)}}else this.C.Oe(a,b,c,d)}Id(a,b,c,d,e){if("@"==c.charAt(0)){const [f,g]=H4(c);return this.F.Id(f,b,g,e)}return this.C.Id(a,b,c,d,e)}flush(a=-1){this.C.flush(a)}};function v6(a,b){let c=null,d=null;Array.isArray(b)&&b.length?(c=w6(b[0]),1<b.length&&(d=w6(b[b.length-1]))):b instanceof Map&&(c=w6(b));return c||d?new x6(a,c,d):null}
var x6=class{constructor(a,b,c){this.C=a;this.F=b;this.D=c;this.qc=0;let d=y6.get(a);d||y6.set(a,d=new Map);this.G=d}start(){1>this.qc&&(this.F&&m4(this.C,this.F,this.G),this.qc=1)}finish(){this.start();2>this.qc&&(m4(this.C,this.G),this.D&&(m4(this.C,this.D),this.D=null),this.qc=1)}destroy(){this.finish();3>this.qc&&(y6.delete(this.C),this.F&&(o4(this.C,this.F),this.D=null),this.D&&(o4(this.C,this.D),this.D=null),m4(this.C,this.G),this.qc=3)}},y6=new WeakMap;
function w6(a){let b=null;a.forEach((c,d)=>{if("display"===d||"position"===d)b=b||new Map,b.set(d,c)});return b};function z6(a){a.N||(a.N=!0,a.G.forEach(b=>b()),a.G=[])}function A6(a){if(!a.ca){a.ca=!0;var b=a.Dg;a.C=a.Ea(a.element,b,a.options);a.ua=b.length?b[b.length-1]:new Map;var c=()=>z6(a);a.C.addEventListener("finish",c);a.ob(()=>{a.C.removeEventListener("finish",c)})}}
var B6=class{constructor(a,b,c,d){this.element=a;this.Dg=b;this.options=c;this.D=d;this.G=[];this.J=[];this.O=[];this.Ga=this.P=this.N=this.ca=!1;this.ea=[];this.ja=[];this.time=0;this.F=null;this.Fh=new Map;this.oa=c.duration;this.T=c.delay||0;this.time=this.oa+this.T}init(){A6(this);this.wa()}wa(){this.T?this.aa():this.C.pause()}ta(a){const b=[];a.forEach(c=>{b.push(Object.fromEntries(c))});return b}Ea(a,b,c){return a.animate(this.ta(b),c)}onStart(a){this.ja.push(a);this.J.push(a)}Md(a){this.ea.push(a);
this.G.push(a)}ob(a){this.O.push(a)}play(){A6(this);this.Rf()||(this.J.forEach(a=>a()),this.J=[],this.P=!0,this.D&&this.D.start());this.C.play()}pause(){this.init();this.C.pause()}finish(){this.init();this.D&&this.D.finish();z6(this);this.C.finish()}reset(){this.aa();this.P=this.N=this.Ga=!1;this.J=this.ja;this.G=this.ea}aa(){this.C&&this.C.cancel()}re(){this.reset();this.play()}Rf(){return this.P}destroy(){this.Ga||(this.Ga=!0,this.aa(),z6(this),this.D&&this.D.destroy(),this.O.forEach(a=>a()),this.O=
[])}setPosition(a){void 0===this.C&&this.init();this.C.currentTime=a*this.time}get totalTime(){return this.T+this.oa}Ok(){const a=new Map;this.Rf()&&this.ua.forEach((b,c)=>{"offset"!==c&&a.set(c,this.N?b:w4(this.element,c))});this.Fh=a}si(a){a="start"===a?this.J:this.G;a.forEach(b=>b());a.length=0}};var C6=class{query(a,b,c){a=c?Array.from(a.querySelectorAll(b)):(a=a.querySelector(b))?[a]:[];return a}animate(a,b,c,d,e,f=[]){const g={duration:c,delay:d,fill:0==d?"both":"forwards"};e&&(g.easing=e);const h=new Map;e=f.filter(p=>p instanceof B6);0!==c&&0!==d||e.forEach(p=>{p.Fh.forEach((u,w)=>h.set(w,u))});let m=l4(b).map(p=>new Map(p));m=v4(a,m,h);b=v6(a,m);return new B6(a,m,g,b)}};var D6=class{constructor(a,b,c,d){this.mc=a;this.rb=b;this.C=c;this.D=d}get data(){return this.rb.data}cl(a){let b,c;null==(c=(b=this.rb).cl)||c.call(b,a)}destroy(){this.C.destroy(this.mc,this.rb);t6(this.C,()=>{queueMicrotask(()=>{this.rb.destroy()})});let a;null==(a=this.D)||a.call(this)}createElement(a,b){return this.rb.createElement(a,b)}createComment(a){return this.rb.createComment(a)}tq(a){return this.rb.tq(a)}appendChild(a,b){this.rb.appendChild(a,b);g6(this.C.C,this.mc,b,!1)}insertBefore(a,
b,c,d=!0){this.rb.insertBefore(a,b,c);g6(this.C.C,this.mc,b,d)}removeChild(a,b){this.C.C.removeNode(this.mc,b,this.rb)}xs(a,b){return this.rb.xs(a,b)}parentNode(a){return this.rb.parentNode(a)}nextSibling(a){return this.rb.nextSibling(a)}setAttribute(a,b,c,d){this.rb.setAttribute(a,b,c,d)}removeAttribute(a,b,c){this.rb.removeAttribute(a,b,c)}Wi(a,b){this.rb.Wi(a,b)}Do(a,b){this.rb.Do(a,b)}setStyle(a,b,c,d){this.rb.setStyle(a,b,c,d)}ks(a,b,c){this.rb.ks(a,b,c)}setProperty(a,b,c){"@"==b.charAt(0)&&
"@.disabled"==b?d6(this.C.C,a,!!c):this.rb.setProperty(a,b,c)}setValue(a,b){this.rb.setValue(a,b)}Id(a,b,c){return this.rb.Id(a,b,c)}},H6=class extends D6{constructor(a,b,c,d){super(b,c,d);this.ba=a;this.mc=b}setProperty(a,b,c){"@"==b.charAt(0)?"."==b.charAt(1)&&"@.disabled"==b?d6(this.C.C,a,void 0===c?!0:!!c):this.C.process(this.mc,a,b.slice(1),c):this.rb.setProperty(a,b,c)}Id(a,b,c){if("@"==b.charAt(0)){a=E6(a);b=b.slice(1);let d="";"@"!=b.charAt(0)&&([b,d]=F6(b));return this.C.Id(this.mc,a,b,d,
e=>{G6(this.ba,e._data||-1,c,e)})}return this.rb.Id(a,b,c)}};function E6(a){switch(a){case "body":return document.body;case "document":return document;case "window":return window;default:return a}}function F6(a){const b=a.indexOf(".");return[a.substring(0,b),a.slice(b+1)]};function G6(a,b,c,d){if(0<=b&&b<a.F)a.zb.run(()=>c(d));else{var e=a.G;0==e.length&&queueMicrotask(()=>{a.zb.run(()=>{e.forEach(f=>{const [g,h]=f;g(h)});a.G=[]})});e.push([c,d])}}
var I6=class{constructor(a,b,c){this.rb=a;this.C=b;this.zb=c;this.J=0;this.F=1;this.G=[];this.N=new Map;this.D=0;b.D=(d,e)=>{const f=null==e?void 0:e.parentNode(d);f&&e.removeChild(f,d)}}Zk(a,b){const c=this.rb.Zk(a,b);let d;if(!a||!(null==b?0:null==(d=b.data)?0:d.animation)){const h=this.N;a=h.get(c);a||(a=new D6("",c,this.C,()=>h.delete(c)),h.set(c,a));return a}const e=b.id,f=b.id+"-"+this.J;this.J++;this.C.register(f,a);const g=h=>{if(Array.isArray(h))h.forEach(g);else{var m=this.C,p=h.name,u=
e+"-"+p,w=m.G[u];if(!w){w=[];h=(new X4).D(h,w,[]);if(w.length)throw new ae(3404,!1);w=new G5(p,h);m.G[u]=w}m=m.C;(u=m.aa[f])&&u.register(p,w)&&m.Ja++}};b.data.animation.forEach(g);return new H6(this,f,c,this.C)}an(){this.D++;this.rb.an&&this.rb.an()}O(){queueMicrotask(()=>{this.F++})}end(){this.D--;0==this.D&&Qk(this.zb,()=>{this.O();this.C.flush(this.F)});this.rb.end&&this.rb.end()}};var J6=class extends u6{constructor(a,b,c){super(a,b,c)}va(){this.flush()}};J6.K=function(a){return new (a||J6)(r(kt),r(L4),r(g4))};J6.na=Pd({ma:J6,ba:J6.K});const K6=[{pa:g4,Gc:function(){return new A4}},{pa:u6,Fc:J6},{pa:yi,Gc:function(a,b,c){return new I6(a,b,c)},Bc:[eH,u6,Tk]}];var L6=[{pa:L4,Gc:()=>new C6},{pa:qi,nb:"BrowserAnimations"},...K6];[...K6];var M6=class{};M6.K=function(a){return new (a||M6)};M6.Ya=Sf({type:M6});M6.Xa=Qd({Wa:L6,imports:[TH]});var N6={Wa:[bg([M6]),function(a,...b){return{fk:[{pa:s1,Eb:!0,nb:a},[],{pa:H_,Gc:f2,Bc:[d2]},{pa:bp,Eb:!0,Gc:g2},b.map(c=>c.fk)]}}([]),{pa:"ModelLoaderService",Fc:f4}]},O6=Object,P6=O6.assign,Q6,R6={Wa:[(null==N6?void 0:N6.Wa)||[],SH]};let S6;Q6={mB:[...tH,...(null!=(S6=null==R6?void 0:R6.Wa)?S6:[])],JD:qH};
(function(a){try{const b=a.VD,c=a.mB,d=zs(a.JD),e=[ls(),{pa:fi,Ab:is},...(c||[])],f=(new tn({Wa:e,parent:d,Nv:"",Mx:!1})).xa,g=f.get(Tk);return g.run(()=>{tg(f);const h=f.get(Ai,null);let m;Qk(g,()=>{m=g.onError.subscribe({next:w=>{h.handleError(w)}})});const p=()=>f.destroy(),u=d.get(rs);u.add(p);f.ob(()=>{m.unsubscribe();u.delete(p)});return dp(h,g,()=>{const w=f.get(yn);xn(w);return w.F.then(()=>{f.get(ps,"en-US");const B=f.get(ip);void 0!==b&&B.Pk(b);return B})})})}catch(b){return Promise.reject(b)}})(P6.call(O6,
{},{VD:Z3},Q6)).catch(a=>{console.error(a)});
